@@ -26,6 +26,8 @@
 #include "CiftiBrainModelsMap.h"
 #include "CiftiSeriesMap.h"
 
+#include <map>
+
 namespace caret {
     
     class AlgorithmCiftiCreateDenseTimeseries : public AbstractAlgorithm
@@ -35,15 +37,28 @@ namespace caret {
         static float getSubAlgorithmWeight();
         static float getAlgorithmInternalWeight();
     public:
-        AlgorithmCiftiCreateDenseTimeseries(ProgressObject* myProgObj, CiftiFile* myCiftiOut, const VolumeFile* myVol = NULL, const VolumeFile* myVolLabel = NULL,
-                                            const MetricFile* leftData = NULL, const MetricFile* leftRoi = NULL,
+        struct SurfParam
+        {
+            const MetricFile* data;
+            const MetricFile* roi;
+            SurfParam() { data = NULL; roi = NULL; }
+            SurfParam(const MetricFile* dataIn, const MetricFile* roiIn = NULL) { data = dataIn; roi = roiIn; }
+        };
+        AlgorithmCiftiCreateDenseTimeseries(ProgressObject* myProgObj, CiftiFile* myCiftiOut, const VolumeFile* myVol, const VolumeFile* myVolLabel,
+                                            const MetricFile* leftData, const MetricFile* leftRoi = NULL,
                                             const MetricFile* rightData = NULL, const MetricFile* rightRoi = NULL,
                                             const MetricFile* cerebData = NULL, const MetricFile* cerebRoi = NULL,
                                             const float& timestep = 1.0f, const float& timestart = 0.0f, const CiftiSeriesMap::Unit& myUnit = CiftiSeriesMap::SECOND);
-        static CiftiBrainModelsMap makeDenseMapping(const VolumeFile* myVol = NULL,
-                                     const VolumeFile* myVolLabel = NULL, const MetricFile* leftData = NULL, const MetricFile* leftRoi = NULL,
+        AlgorithmCiftiCreateDenseTimeseries(ProgressObject* myProgObj, CiftiFile* myCiftiOut, const VolumeFile* myVol = NULL, const VolumeFile* myVolLabel = NULL,
+                                            const std::map<StructureEnum::Enum, SurfParam> surfParams = std::map<StructureEnum::Enum, SurfParam>(),
+                                            const float& timestep = 1.0f, const float& timestart = 0.0f, const CiftiSeriesMap::Unit& myUnit = CiftiSeriesMap::SECOND);
+        static CiftiBrainModelsMap makeDenseMapping(const VolumeFile* myVol,
+                                     const VolumeFile* myVolLabel, const MetricFile* leftData, const MetricFile* leftRoi = NULL,
                                      const MetricFile* rightData = NULL, const MetricFile* rightRoi = NULL, const MetricFile* cerebData = NULL,
                                      const MetricFile* cerebRoi = NULL);//where should this go?  should also have version that accepts LabelFile
+        static CiftiBrainModelsMap makeDenseMapping(const VolumeFile* myVol = NULL,
+                                     const VolumeFile* myVolLabel = NULL,
+                                     const std::map<StructureEnum::Enum, SurfParam> surfParams = std::map<StructureEnum::Enum, SurfParam>());
         static OperationParameters* getParameters();
         static void useParameters(OperationParameters* myParams, ProgressObject* myProgObj);
         static AString getCommandSwitch();
