@@ -109,7 +109,7 @@ void AlgorithmCiftiSeparate::useParameters(OperationParameters* myParams, Progre
         throw AlgorithmException("incorrect string for direction, use ROW or COLUMN");
     }
     bool outputRequested = false;
-    const vector<ParameterComponent*>& labelInstances = *(myParams->getRepeatableParameterInstances(3));
+    const vector<ParameterComponent*>& labelInstances = myParams->getRepeatableParameterInstances(3);
     for (int i = 0; i < (int)labelInstances.size(); ++i)
     {
         outputRequested = true;
@@ -118,7 +118,7 @@ void AlgorithmCiftiSeparate::useParameters(OperationParameters* myParams, Progre
         StructureEnum::Enum myStruct = StructureEnum::fromName(structName, &ok);
         if (!ok)
         {
-            throw AlgorithmException("unrecognized structure type");
+            throw AlgorithmException("unrecognized structure type: '" + structName + "'");
         }
         LabelFile* labelOut = labelInstances[i]->getOutputLabel(2);
         MetricFile* roiOut = NULL;
@@ -129,7 +129,7 @@ void AlgorithmCiftiSeparate::useParameters(OperationParameters* myParams, Progre
         }
         AlgorithmCiftiSeparate(NULL, ciftiIn, myDir, myStruct, labelOut, roiOut);
     }
-    const vector<ParameterComponent*>& metricInstances = *(myParams->getRepeatableParameterInstances(4));
+    const vector<ParameterComponent*>& metricInstances = myParams->getRepeatableParameterInstances(4);
     for (int i = 0; i < (int)metricInstances.size(); ++i)
     {
         outputRequested = true;
@@ -138,7 +138,7 @@ void AlgorithmCiftiSeparate::useParameters(OperationParameters* myParams, Progre
         StructureEnum::Enum myStruct = StructureEnum::fromName(structName, &ok);
         if (!ok)
         {
-            throw AlgorithmException("unrecognized structure type");
+            throw AlgorithmException("unrecognized structure type: '" + structName + "'");
         }
         MetricFile* metricOut = metricInstances[i]->getOutputMetric(2);
         MetricFile* roiOut = NULL;
@@ -149,7 +149,7 @@ void AlgorithmCiftiSeparate::useParameters(OperationParameters* myParams, Progre
         }
         AlgorithmCiftiSeparate(NULL, ciftiIn, myDir, myStruct, metricOut, roiOut);
     }
-    const vector<ParameterComponent*>& volumeInstances = *(myParams->getRepeatableParameterInstances(5));
+    const vector<ParameterComponent*>& volumeInstances = myParams->getRepeatableParameterInstances(5);
     for (int i = 0; i < (int)volumeInstances.size(); ++i)
     {
         outputRequested = true;
@@ -158,7 +158,7 @@ void AlgorithmCiftiSeparate::useParameters(OperationParameters* myParams, Progre
         StructureEnum::Enum myStruct = StructureEnum::fromName(structName, &ok);
         if (!ok)
         {
-            throw AlgorithmException("unrecognized structure type");
+            throw AlgorithmException("unrecognized structure type: '" + structName + "'");
         }
         VolumeFile* volOut = volumeInstances[i]->getOutputVolume(2);
         VolumeFile* roiOut = NULL;
@@ -257,7 +257,7 @@ AlgorithmCiftiSeparate::AlgorithmCiftiSeparate(ProgressObject* myProgObj, const
         metricOut->setNumberOfNodesAndColumns(numNodes, colSize);
         metricOut->setStructure(myStruct);
         const CiftiMappingType& myNamesMap = *(myXML.getMap(1 - myDir));
-        for (int j = 0; j < rowSize; ++j)
+        for (int j = 0; j < colSize; ++j)
         {
             metricOut->setMapName(j, myNamesMap.getIndexName(j));
         }
@@ -344,6 +344,7 @@ AlgorithmCiftiSeparate::AlgorithmCiftiSeparate(ProgressObject* myProgObj, const
                 }
             }
         }
+        myTable.setHierarchy(myLabelsMap.getMapLabelTable(0)->getHierarchy()); //grab the first one for now, maybe figure out how to try to merge hierarchies later, if it matters
         *(labelOut->getLabelTable()) = myTable;
         int32_t unusedLabel = myTable.getUnassignedLabelKey();
         if (roiOut != NULL)
@@ -365,7 +366,7 @@ AlgorithmCiftiSeparate::AlgorithmCiftiSeparate(ProgressObject* myProgObj, const
         labelOut->setNumberOfNodesAndColumns(numNodes, colSize);
         labelOut->setStructure(myStruct);
         const CiftiMappingType& myNamesMap = *(myXML.getMap(1 - myDir));
-        for (int j = 0; j < rowSize; ++j)
+        for (int j = 0; j < colSize; ++j)
         {
             labelOut->setMapName(j, myNamesMap.getIndexName(j));
         }
@@ -397,6 +398,7 @@ AlgorithmCiftiSeparate::AlgorithmCiftiSeparate(ProgressObject* myProgObj, const
             map<int32_t, int32_t> thisRemap = myTable.append(*(myLabelsMap.getMapLabelTable(i)));
             cumulativeRemap.insert(thisRemap.begin(), thisRemap.end());
         }
+        myTable.setHierarchy(myLabelsMap.getMapLabelTable(0)->getHierarchy()); //ditto
         *(labelOut->getLabelTable()) = myTable;
         int32_t unusedLabel = myTable.getUnassignedLabelKey();
         for (int64_t i = 0; i < colSize; ++i)
@@ -496,7 +498,7 @@ AlgorithmCiftiSeparate::AlgorithmCiftiSeparate(ProgressObject* myProgObj, const
         volOut->reinitialize(newdims, mySform);
         volOut->setValueAllVoxels(0.0f);
         const CiftiMappingType& myNamesMap = *(myXML.getMap(1 - myDir));
-        for (int j = 0; j < rowSize; ++j)
+        for (int j = 0; j < colSize; ++j)
         {
             volOut->setMapName(j, myNamesMap.getIndexName(j));
         }
@@ -614,7 +616,7 @@ AlgorithmCiftiSeparate::AlgorithmCiftiSeparate(ProgressObject* myProgObj, const
         volOut->reinitialize(newdims, mySform);
         volOut->setValueAllVoxels(0.0f);
         const CiftiMappingType& myNamesMap = *(myXML.getMap(1 - myDir));
-        for (int j = 0; j < rowSize; ++j)
+        for (int j = 0; j < colSize; ++j)
         {
             volOut->setMapName(j, myNamesMap.getIndexName(j));
         }
@@ -654,7 +656,7 @@ void AlgorithmCiftiSeparate::getCroppedVolSpace(const CiftiFile* ciftiIn, const
     int64_t numVoxels = (int64_t)myMap.size();
     if (numVoxels > 0)
     {//make a voxel bounding box to minimize memory usage
-        int extrema[6] = { myMap[0].m_ijk[0],
+        int64_t extrema[6] = { myMap[0].m_ijk[0],
             myMap[0].m_ijk[0],
             myMap[0].m_ijk[1],
             myMap[0].m_ijk[1],
@@ -699,7 +701,7 @@ void AlgorithmCiftiSeparate::getCroppedVolSpaceAll(const CiftiFile* ciftiIn, con
     int64_t numVoxels = (int64_t)myMap.size();
     if (numVoxels > 0)
     {//make a voxel bounding box to minimize memory usage
-        int extrema[6] = { myMap[0].m_ijk[0],
+        int64_t extrema[6] = { myMap[0].m_ijk[0],
             myMap[0].m_ijk[0],
             myMap[0].m_ijk[1],
             myMap[0].m_ijk[1],
