@@ -29,6 +29,8 @@
 #include "AlgorithmCiftiSeparate.h"
 #include "AlgorithmCiftiReplaceStructure.h"
 
+#include <cmath>
+
 using namespace caret;
 using namespace std;
 
@@ -46,17 +48,19 @@ OperationParameters* AlgorithmCiftiSmoothing::getParameters()
 {
     OperationParameters* ret = new OperationParameters();
     ret->addCiftiParameter(1, "cifti", "the input cifti");
-    ret->addDoubleParameter(2, "surface-kernel", "the sigma for the gaussian surface smoothing kernel, in mm");
-    ret->addDoubleParameter(3, "volume-kernel", "the sigma for the gaussian volume smoothing kernel, in mm");
+    ret->addDoubleParameter(2, "surface-kernel", "the size of the gaussian surface smoothing kernel in mm, as sigma by default");
+    ret->addDoubleParameter(3, "volume-kernel", "the size of the gaussian volume smoothing kernel in mm, as sigma by default");
     ret->addStringParameter(4, "direction", "which dimension to smooth along, ROW or COLUMN");
     ret->addCiftiOutputParameter(5, "cifti-out", "the output cifti");
     
-    OptionalParameter* leftSurfOpt = ret->createOptionalParameter(6, "-left-surface", "specify the left surface to use");
+    ret->createOptionalParameter(13, "-fwhm", "kernel sizes are FWHM, not sigma");
+    
+    OptionalParameter* leftSurfOpt = ret->createOptionalParameter(6, "-left-surface", "specify the left cortical surface to use");
     leftSurfOpt->addSurfaceParameter(1, "surface", "the left surface file");
     OptionalParameter* leftCorrAreasOpt = leftSurfOpt->createOptionalParameter(2, "-left-corrected-areas", "vertex areas to use instead of computing them from the left surface");
     leftCorrAreasOpt->addMetricParameter(1, "area-metric", "the corrected vertex areas, as a metric");
     
-    OptionalParameter* rightSurfOpt = ret->createOptionalParameter(7, "-right-surface", "specify the right surface to use");
+    OptionalParameter* rightSurfOpt = ret->createOptionalParameter(7, "-right-surface", "specify the right cortical surface to use");
     rightSurfOpt->addSurfaceParameter(1, "surface", "the right surface file");
     OptionalParameter* rightCorrAreasOpt = rightSurfOpt->createOptionalParameter(2, "-right-corrected-areas", "vertex areas to use instead of computing them from the right surface");
     rightCorrAreasOpt->addMetricParameter(1, "area-metric", "the corrected vertex areas, as a metric");
@@ -66,6 +70,12 @@ OperationParameters* AlgorithmCiftiSmoothing::getParameters()
     OptionalParameter* cerebCorrAreasOpt = cerebSurfOpt->createOptionalParameter(2, "-cerebellum-corrected-areas", "vertex areas to use instead of computing them from the cerebellum surface");
     cerebCorrAreasOpt->addMetricParameter(1, "area-metric", "the corrected vertex areas, as a metric");
     
+    ParameterComponent* genSurfOpt = ret->createRepeatableParameter(13, "-surface", "specify a surface by structure name");
+    genSurfOpt->addStringParameter(1, "structure", "the surface structure name");
+    genSurfOpt->addSurfaceParameter(2, "surface", "the surface file");
+    OptionalParameter* genCorrAreasOpt = genSurfOpt->createOptionalParameter(3, "-corrected-areas", "vertex areas to use instead of computing them from the surface");
+    genCorrAreasOpt->addMetricParameter(1, "area-metric", "the corrected vertex areas, as a metric");
+    
     OptionalParameter* roiOpt = ret->createOptionalParameter(9, "-cifti-roi", "smooth only within regions of interest");
     roiOpt->addCiftiParameter(1, "roi-cifti", "the regions to smooth within, as a cifti file");
     
@@ -75,7 +85,7 @@ OperationParameters* AlgorithmCiftiSmoothing::getParameters()
     
     ret->createOptionalParameter(12, "-merged-volume", "smooth across subcortical structure boundaries");
     
-    ret->setHelpText(
+    AString helpText =
         AString("The input cifti file must have a brain models mapping on the chosen dimension, columns for .dtseries, and either for .dconn.  ") +
         "By default, data in different structures is smoothed independently (i.e., \"parcel constrained\" smoothing), so volume structures that touch do not smooth across this boundary.  " +
         "Specify -merged-volume to ignore these boundaries.  " +
@@ -84,8 +94,15 @@ OperationParameters* AlgorithmCiftiSmoothing::getParameters()
         "for the reduction of structure in a group average surface.  It is better to smooth the data on individuals before averaging, when feasible.\n\n" +
         "The -fix-zeros-* options will treat values of zero as lack of data, and not use that value when generating the smoothed values, but will fill zeros with extrapolated values.  " +
         "The ROI should have a brain models mapping along columns, exactly matching the mapping of the chosen direction in the input file.  " +
-        "Data outside the ROI is ignored."
-    );
+        "Data outside the ROI is ignored.\n\n" +
+        "The <structure> argument of -surface must be one of the following strings:\n";
+    vector<StructureEnum::Enum> myStructureEnums;
+    StructureEnum::getAllEnums(myStructureEnums);
+    for (int i = 0; i < (int)myStructureEnums.size(); ++i)
+    {
+        helpText += "\n" + StructureEnum::toName(myStructureEnums[i]);
+    }
+    ret->setHelpText(helpText);
     return ret;
 }
 
@@ -94,6 +111,11 @@ void AlgorithmCiftiSmoothing::useParameters(OperationParameters* myParams, Progr
     CiftiFile* myCifti = myParams->getCifti(1);
     float surfKern = (float)myParams->getDouble(2);
     float volKern = (float)myParams->getDouble(3);
+    if (myParams->getOptionalParameter(13)->m_present)
+    {
+        surfKern = surfKern / (2.0f * sqrt(2.0f * log(2.0f)));
+        volKern = volKern / (2.0f * sqrt(2.0f * log(2.0f)));
+    }
     AString directionName = myParams->getString(4);
     int myDir;
     if (directionName == "ROW")
@@ -107,6 +129,7 @@ void AlgorithmCiftiSmoothing::useParameters(OperationParameters* myParams, Progr
     CiftiFile* myCiftiOut = myParams->getOutputCifti(5);
     SurfaceFile* myLeftSurf = NULL, *myRightSurf = NULL, *myCerebSurf = NULL;
     MetricFile* myLeftAreas = NULL, *myRightAreas = NULL, *myCerebAreas = NULL;
+    map<StructureEnum::Enum, SurfParam> surfParams;
     OptionalParameter* leftSurfOpt = myParams->getOptionalParameter(6);
     if (leftSurfOpt->m_present)
     {
@@ -116,6 +139,7 @@ void AlgorithmCiftiSmoothing::useParameters(OperationParameters* myParams, Progr
         {
             myLeftAreas = leftCorrAreasOpt->getMetric(1);
         }
+        surfParams[StructureEnum::CORTEX_LEFT] = SurfParam(myLeftSurf, myLeftAreas);
     }
     OptionalParameter* rightSurfOpt = myParams->getOptionalParameter(7);
     if (rightSurfOpt->m_present)
@@ -126,6 +150,7 @@ void AlgorithmCiftiSmoothing::useParameters(OperationParameters* myParams, Progr
         {
             myRightAreas = rightCorrAreasOpt->getMetric(1);
         }
+        surfParams[StructureEnum::CORTEX_RIGHT] = SurfParam(myRightSurf, myRightAreas);
     }
     OptionalParameter* cerebSurfOpt = myParams->getOptionalParameter(8);
     if (cerebSurfOpt->m_present)
@@ -136,6 +161,22 @@ void AlgorithmCiftiSmoothing::useParameters(OperationParameters* myParams, Progr
         {
             myCerebAreas = cerebCorrAreasOpt->getMetric(1);
         }
+        surfParams[StructureEnum::CEREBELLUM] = SurfParam(myCerebSurf, myCerebAreas);
+    }
+    auto genSurfArgs = myParams->getRepeatableParameterInstances(13);
+    for (auto instance : genSurfArgs)
+    {
+        bool ok = false;
+        StructureEnum::Enum structure = StructureEnum::fromName(instance->getString(1), &ok);
+        if (!ok) throw AlgorithmException("unrecognized structure identifier: " + instance->getString(1));
+        if (surfParams.find(structure) != surfParams.end()) throw AlgorithmException("more than one surface argument specified for structure '" + instance->getString(1) + "'");
+        auto areasOpt = instance->getOptionalParameter(3);
+        if (areasOpt->m_present)
+        {
+            surfParams[structure] = SurfParam(instance->getSurface(2), areasOpt->getMetric(1));
+        } else {
+            surfParams[structure] = SurfParam(instance->getSurface(2));
+        }
     }
     CiftiFile* roiCifti = NULL;
     OptionalParameter* roiOpt = myParams->getOptionalParameter(9);
@@ -147,17 +188,33 @@ void AlgorithmCiftiSmoothing::useParameters(OperationParameters* myParams, Progr
     bool fixZerosSurf = myParams->getOptionalParameter(11)->m_present;
     bool mergedVolume = myParams->getOptionalParameter(12)->m_present;
     AlgorithmCiftiSmoothing(myProgObj, myCifti, surfKern, volKern, myDir, myCiftiOut,
-                            myLeftSurf, myRightSurf, myCerebSurf,
+                            surfParams,
                             roiCifti, fixZerosVol, fixZerosSurf,
-                            myLeftAreas, myRightAreas, myCerebAreas, mergedVolume);
+                            mergedVolume);
 }
 
 AlgorithmCiftiSmoothing::AlgorithmCiftiSmoothing(ProgressObject* myProgObj, const CiftiFile* myCifti, const float& surfKern, const float& volKern, const int& myDir, CiftiFile* myCiftiOut,
                                                  const SurfaceFile* myLeftSurf, const SurfaceFile* myRightSurf, const SurfaceFile* myCerebSurf,
                                                  const CiftiFile* roiCifti, bool fixZerosVol, bool fixZerosSurf,
-                                                 const MetricFile* myLeftAreas, const MetricFile* myRightAreas, const MetricFile* myCerebAreas, const bool& mergedVolume) : AbstractAlgorithm(myProgObj)
+                                                 const MetricFile* myLeftAreas, const MetricFile* myRightAreas, const MetricFile* myCerebAreas, const bool& mergedVolume) : AbstractAlgorithm(NULL)
+{
+    map<StructureEnum::Enum, SurfParam> surfParams;
+    if (myLeftSurf != NULL) surfParams[StructureEnum::CORTEX_LEFT] = SurfParam(myLeftSurf, myLeftAreas);
+    if (myRightSurf != NULL) surfParams[StructureEnum::CORTEX_RIGHT] = SurfParam(myRightSurf, myRightAreas);
+    if (myCerebSurf != NULL) surfParams[StructureEnum::CEREBELLUM] = SurfParam(myCerebSurf, myCerebAreas);
+    AlgorithmCiftiSmoothing(myProgObj, myCifti, surfKern, volKern, myDir, myCiftiOut,
+                            surfParams,
+                            roiCifti, fixZerosVol, fixZerosSurf,
+                            mergedVolume);
+}
+
+AlgorithmCiftiSmoothing::AlgorithmCiftiSmoothing(ProgressObject* myProgObj, const CiftiFile* myCifti, const float& surfKern, const float& volKern, const int& myDir, CiftiFile* myCiftiOut,
+                                                 const map<StructureEnum::Enum, AlgorithmCiftiSmoothing::SurfParam> surfParams,
+                                                 const CiftiFile* roiCifti, bool fixZerosVol, bool fixZerosSurf,
+                                                 const bool& mergedVolume) : AbstractAlgorithm(myProgObj)
 {
     LevelProgress myProgress(myProgObj);
+    if (!(surfKern > 0.0f) && !(volKern > 0.0f)) throw AlgorithmException("zero smoothing kernels requested for both volume and surface");
     const CiftiXMLOld& myXML = myCifti->getCiftiXMLOld();
     vector<StructureEnum::Enum> surfaceList, volumeList;
     if (myDir == CiftiXMLOld::ALONG_COLUMN)
@@ -179,30 +236,11 @@ AlgorithmCiftiSmoothing::AlgorithmCiftiSmoothing(ProgressObject* myProgObj, cons
     }
     for (int whichStruct = 0; whichStruct < (int)surfaceList.size(); ++whichStruct)
     {//sanity check surfaces
-        const SurfaceFile* mySurf = NULL;
-        const MetricFile* myAreas = NULL;
-        AString surfType;
-        switch (surfaceList[whichStruct])
-        {
-            case StructureEnum::CORTEX_LEFT:
-                mySurf = myLeftSurf;
-                myAreas = myLeftAreas;
-                surfType = "left";
-                break;
-            case StructureEnum::CORTEX_RIGHT:
-                mySurf = myRightSurf;
-                myAreas = myRightAreas;
-                surfType = "right";
-                break;
-            case StructureEnum::CEREBELLUM:
-                mySurf = myCerebSurf;
-                myAreas = myCerebAreas;
-                surfType = "cerebellum";
-                break;
-            default:
-                throw AlgorithmException("found surface model with incorrect type: " + StructureEnum::toName(surfaceList[whichStruct]));
-                break;
-        }
+        AString surfType = StructureEnum::toName(surfaceList[whichStruct]);
+        auto result = surfParams.find(surfaceList[whichStruct]);
+        if (result == surfParams.end()) throw AlgorithmException(surfType + " surface required but not provided");
+        const SurfaceFile* mySurf = result->second.surface;
+        const MetricFile* myAreas = result->second.correctedAreas;
         if (mySurf == NULL)
         {
             throw AlgorithmException(surfType + " surface required but not provided");
@@ -227,57 +265,57 @@ AlgorithmCiftiSmoothing::AlgorithmCiftiSmoothing(ProgressObject* myProgObj, cons
     myCiftiOut->setCiftiXML(myXML);
     for (int whichStruct = 0; whichStruct < (int)surfaceList.size(); ++whichStruct)
     {
-        const SurfaceFile* mySurf = NULL;
-        const MetricFile* myAreas = NULL;
-        switch (surfaceList[whichStruct])
-        {
-            case StructureEnum::CORTEX_LEFT:
-                mySurf = myLeftSurf;
-                myAreas = myLeftAreas;
-                break;
-            case StructureEnum::CORTEX_RIGHT:
-                mySurf = myRightSurf;
-                myAreas = myRightAreas;
-                break;
-            case StructureEnum::CEREBELLUM:
-                mySurf = myCerebSurf;
-                myAreas = myCerebAreas;
-                break;
-            default:
-                break;
-        }
+        auto result = surfParams.find(surfaceList[whichStruct]);
+        CaretAssert(result != surfParams.end()); //we already checked that these are all present, but assert anyway
+        const SurfaceFile* mySurf = result->second.surface;
+        const MetricFile* myAreas = result->second.correctedAreas;
         MetricFile myMetric, myRoi, myMetricOut;
         AlgorithmCiftiSeparate(NULL, myCifti, myDir, surfaceList[whichStruct], &myMetric, &myRoi);
-        if (roiCifti != NULL)
-        {//due to above testing, we know the structure mask is the same, so just overwrite the ROI from the mask
-            AlgorithmCiftiSeparate(NULL, roiCifti, CiftiXMLOld::ALONG_COLUMN, surfaceList[whichStruct], &myRoi);
+        if (surfKern > 0.0f)
+        {
+            if (roiCifti != NULL)
+            {//due to above testing, we know the structure mask is the same, so just overwrite the ROI from the mask
+                AlgorithmCiftiSeparate(NULL, roiCifti, CiftiXMLOld::ALONG_COLUMN, surfaceList[whichStruct], &myRoi);
+            }
+            AlgorithmMetricSmoothing(NULL, mySurf, &myMetric, surfKern, &myMetricOut, &myRoi, false, fixZerosSurf, -1, myAreas);
+            AlgorithmCiftiReplaceStructure(NULL, myCiftiOut, myDir, surfaceList[whichStruct], &myMetricOut);
+        } else {
+            AlgorithmCiftiReplaceStructure(NULL, myCiftiOut, myDir, surfaceList[whichStruct], &myMetric);
         }
-        AlgorithmMetricSmoothing(NULL, mySurf, &myMetric, surfKern, &myMetricOut, &myRoi, false, fixZerosSurf, -1, myAreas);
-        AlgorithmCiftiReplaceStructure(NULL, myCiftiOut, myDir, surfaceList[whichStruct], &myMetricOut);
     }
     if (mergedVolume)
     {
         VolumeFile myVol, myRoi, myVolOut;
         int64_t offset[3];
         AlgorithmCiftiSeparate(NULL, myCifti, myDir, &myVol, offset, &myRoi, true);
-        if (roiCifti != NULL)
-        {//due to above testing, we know the structure mask is the same, so just overwrite the ROI from the mask
-            AlgorithmCiftiSeparate(NULL, roiCifti, CiftiXMLOld::ALONG_COLUMN, &myRoi, offset, NULL, true);
+        if (volKern > 0.0f)
+        {
+            if (roiCifti != NULL)
+            {//due to above testing, we know the structure mask is the same, so just overwrite the ROI from the mask
+                AlgorithmCiftiSeparate(NULL, roiCifti, CiftiXMLOld::ALONG_COLUMN, &myRoi, offset, NULL, true);
+            }
+            AlgorithmVolumeSmoothing(NULL, &myVol, volKern, &myVolOut, &myRoi, fixZerosVol);
+            AlgorithmCiftiReplaceStructure(NULL, myCiftiOut, myDir, &myVolOut, true);
+        } else {
+            AlgorithmCiftiReplaceStructure(NULL, myCiftiOut, myDir, &myVol, true);
         }
-        AlgorithmVolumeSmoothing(NULL, &myVol, volKern, &myVolOut, &myRoi, fixZerosVol);
-        AlgorithmCiftiReplaceStructure(NULL, myCiftiOut, myDir, &myVolOut, true);
     } else {
         for (int whichStruct = 0; whichStruct < (int)volumeList.size(); ++whichStruct)
         {
             VolumeFile myVol, myRoi, myVolOut;
             int64_t offset[3];
             AlgorithmCiftiSeparate(NULL, myCifti, myDir, volumeList[whichStruct], &myVol, offset, &myRoi, true);
-            if (roiCifti != NULL)
-            {//due to above testing, we know the structure mask is the same, so just overwrite the ROI from the mask
-                AlgorithmCiftiSeparate(NULL, roiCifti, CiftiXMLOld::ALONG_COLUMN, volumeList[whichStruct], &myRoi, offset, NULL, true);
+            if (volKern > 0.0f)
+            {
+                if (roiCifti != NULL)
+                {//due to above testing, we know the structure mask is the same, so just overwrite the ROI from the mask
+                    AlgorithmCiftiSeparate(NULL, roiCifti, CiftiXMLOld::ALONG_COLUMN, volumeList[whichStruct], &myRoi, offset, NULL, true);
+                }
+                AlgorithmVolumeSmoothing(NULL, &myVol, volKern, &myVolOut, &myRoi, fixZerosVol);
+                AlgorithmCiftiReplaceStructure(NULL, myCiftiOut, myDir, volumeList[whichStruct], &myVolOut, true);
+            } else {
+                AlgorithmCiftiReplaceStructure(NULL, myCiftiOut, myDir, volumeList[whichStruct], &myVol, true);
             }
-            AlgorithmVolumeSmoothing(NULL, &myVol, volKern, &myVolOut, &myRoi, fixZerosVol);
-            AlgorithmCiftiReplaceStructure(NULL, myCiftiOut, myDir, volumeList[whichStruct], &myVolOut, true);
         }
     }
 }
