@@ -57,7 +57,8 @@ OperationParameters* AlgorithmMetricTFCE::getParameters()
     ret->addMetricOutputParameter(3, "metric-out", "the output metric");
     
     OptionalParameter* presmoothOpt = ret->createOptionalParameter(4, "-presmooth", "smooth the metric before running TFCE");
-    presmoothOpt->addDoubleParameter(1, "kernel", "the sigma for the gaussian smoothing kernel, in mm");
+    presmoothOpt->addDoubleParameter(1, "kernel", "the size of the gaussian smoothing kernel in mm, as sigma by default");
+    presmoothOpt->createOptionalParameter(2, "-fwhm", "kernel size is FWHM, not sigma");
     
     OptionalParameter* roiOpt = ret->createOptionalParameter(5, "-roi", "select a region of interest to run TFCE on");
     roiOpt->addMetricParameter(1, "roi-metric", "the area to run TFCE on, as a metric");
@@ -73,7 +74,8 @@ OperationParameters* AlgorithmMetricTFCE::getParameters()
     corrAreaOpt->addMetricParameter(1, "area-metric", "the corrected vertex areas, as a metric");
     
     ret->setHelpText(
-        AString("Threshold-free cluster enhancement is a method to increase the relative value of regions that would form clusters in a standard thresholding test.  ") +
+        AString("This command does not do any statistical analysis.  Please use something like PALM if you are just trying to do statistics on your data.\n\n") +
+        "Threshold-free cluster enhancement is a method to increase the relative value of regions that would form clusters in a standard thresholding test.  " +
         "This is accomplished by evaluating the integral of:\n\n" +
         "e(h, p)^E * h^H * dh\n\n" +
         "at each vertex p, where h ranges from 0 to the maximum value in the data, and e(h, p) is the extent of the cluster containing vertex p at threshold h.  " +
@@ -96,6 +98,10 @@ void AlgorithmMetricTFCE::useParameters(OperationParameters* myParams, ProgressO
     {
         presmooth = (float)presmoothOpt->getDouble(1);
         if (presmooth <= 0.0f) throw AlgorithmException("presmooth kernel size must be positive");
+        if (presmoothOpt->getOptionalParameter(2)->m_present)
+        {
+            presmooth = presmooth / (2.0f * sqrt(2.0f * log(2.0f)));
+        }
     }
     MetricFile* myRoi = NULL;
     OptionalParameter* roiOpt = myParams->getOptionalParameter(5);
@@ -217,7 +223,7 @@ void AlgorithmMetricTFCE::processColumn(const SurfaceFile* mySurf, const float*
     }
 }
 
-namespace _algorithm_metric_tfce
+namespace
 {//hidden namespace just to make sure things don't collide
     struct Cluster
     {
@@ -230,6 +236,7 @@ namespace _algorithm_metric_tfce
             first = true;
             accumVal = 0.0;
             totalArea = 0.0;
+            lastVal = 0.0f;
         }
         void addMember(const int& node, const float& val, const float& area, const float& param_e, const float& param_h)
         {
@@ -272,8 +279,6 @@ namespace _algorithm_metric_tfce
     }
 }
 
-using namespace _algorithm_metric_tfce;
-
 void AlgorithmMetricTFCE::tfce_pos(TopologyHelper* myHelper, const float* colData, double* accumData, const float* roiData, const float& param_e, const float& param_h, const float* areaData)
 {
     int numNodes = myHelper->getNumberOfNodes();
