@@ -37,7 +37,7 @@ namespace caret {
         AlgorithmVolumeLabelToSurfaceMapping(ProgressObject* myProgObj, const VolumeFile* myVolume, const SurfaceFile* mySurface, LabelFile* myLabelOut,
                                              const SurfaceFile* innerSurf, const SurfaceFile* outerSurf,
                                              const VolumeFile* myRoiVol = NULL, const int32_t& subdivisions = 3, const bool& thinColumns = false,
-                                             const int64_t& mySubVol = -1);
+                                             const int64_t& mySubVol = -1, const float dilateDist = -1.0f);
         static OperationParameters* getParameters();
         static void useParameters(OperationParameters* myParams, ProgressObject* myProgObj);
         static AString getCommandSwitch();
