@@ -26,6 +26,7 @@
 #undef __ANNOTATION_GROUP_DECLARE__
 
 #include "Annotation.h"
+#include "AnnotationCoordinate.h"
 #include "AnnotationPointSizeText.h"
 #include "BrainConstants.h"
 #include "CaretAssert.h"
@@ -57,12 +58,25 @@ using namespace caret;
  *     Annotation coordinate space for the group.
  * @param tabOrWindowIndex
  *     Index of tab or window for tab or window space.
+ * @param spacerTabIndex
+ *     Index of a spacer tab.
+ * @param mediaFileName
+ *     Name of media file
+ * @param histologySpaceKey
+ *     Key for histology space
+ * @param histologySlicesFileName
+ *     Name of histology slices file
+ * @param histologySliceIndex
+ *     Index of histology slice
  */
 AnnotationGroup::AnnotationGroup(AnnotationFile* annotationFile,
                                  const AnnotationGroupTypeEnum::Enum groupType,
                                  const int32_t uniqueKey,
                                  const AnnotationCoordinateSpaceEnum::Enum coordinateSpace,
-                                 const int32_t tabOrWindowIndex)
+                                 const int32_t tabOrWindowIndex,
+                                 const SpacerTabIndex& spacerTabIndex,
+                                 const AString& mediaFileName,
+                                 const HistologySpaceKey& histologySpaceKey)
 : CaretObjectTracksModification(),
 DisplayGroupAndTabItemInterface(),
 SceneableInterface()
@@ -70,7 +84,7 @@ SceneableInterface()
     CaretAssert(annotationFile);
     CaretAssert(groupType != AnnotationGroupTypeEnum::INVALID);
     CaretAssert(uniqueKey > 0);
-    CaretAssert(coordinateSpace != AnnotationCoordinateSpaceEnum::PIXELS);
+    CaretAssert(coordinateSpace != AnnotationCoordinateSpaceEnum::VIEWPORT);
     
     initializeInstance();
     
@@ -81,6 +95,12 @@ SceneableInterface()
         case AnnotationGroupTypeEnum::INVALID:
             CaretAssertMessage(0, "Should never get here");
             break;
+        case AnnotationGroupTypeEnum::SAMPLES_RETROSPECTIVE:
+            m_groupKey.setSamplesRetrospectiveUniqueKey(uniqueKey);
+            break;
+        case AnnotationGroupTypeEnum::SAMPLES_PROSPECTIVE:
+            m_groupKey.setSamplesProspectiveUniqueKey(uniqueKey);
+            break;
         case AnnotationGroupTypeEnum::SPACE:
             m_groupKey.setSpaceGroupUniqueKey(uniqueKey);
             break;
@@ -91,10 +111,21 @@ SceneableInterface()
     
     m_coordinateSpace  = coordinateSpace;
     m_tabOrWindowIndex = tabOrWindowIndex;
+    m_spacerTabIndex   = spacerTabIndex;
+    m_mediaFileName    = mediaFileName;
+    m_histologySpaceKey = histologySpaceKey;
     
     switch (m_coordinateSpace) {
-        case AnnotationCoordinateSpaceEnum::PIXELS:
-            CaretAssert(0);
+        case AnnotationCoordinateSpaceEnum::CHART:
+            break;
+        case AnnotationCoordinateSpaceEnum::HISTOLOGY:
+            CaretAssert(m_histologySpaceKey.isValid());
+            break;
+        case AnnotationCoordinateSpaceEnum::MEDIA_FILE_NAME_AND_PIXEL:
+            CaretAssert( ! m_mediaFileName.isEmpty());
+            break;
+        case AnnotationCoordinateSpaceEnum::SPACER:
+            CaretAssert(m_spacerTabIndex.isValid());
             break;
         case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
             break;
@@ -104,6 +135,9 @@ SceneableInterface()
             CaretAssert((tabOrWindowIndex >= 0)
                         && (tabOrWindowIndex < BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS));
             break;
+        case AnnotationCoordinateSpaceEnum::VIEWPORT:
+            CaretAssert(0);
+            break;
         case AnnotationCoordinateSpaceEnum::WINDOW:
             CaretAssert((tabOrWindowIndex >= 0)
                         && (tabOrWindowIndex < BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_WINDOWS));
@@ -167,7 +201,10 @@ AnnotationGroup::copyHelperAnnotationGroup(const AnnotationGroup& obj)
     m_coordinateSpace  = obj.m_coordinateSpace;
     m_name             = obj.m_name;
     m_tabOrWindowIndex = obj.m_tabOrWindowIndex;
+    m_spacerTabIndex   = obj.m_spacerTabIndex;
     *m_displayGroupAndTabItemHelper = *obj.m_displayGroupAndTabItemHelper;
+    m_mediaFileName    = obj.m_mediaFileName;
+    m_histologySpaceKey = obj.m_histologySpaceKey;
     
     CaretAssertMessage(0, "What to do with annotations remove copy constructor/operator=");
 }
@@ -179,9 +216,12 @@ void
 AnnotationGroup::initializeInstance()
 {
     m_groupKey.reset();
-    m_coordinateSpace  = AnnotationCoordinateSpaceEnum::PIXELS;
+    m_coordinateSpace  = AnnotationCoordinateSpaceEnum::VIEWPORT;
     m_name             = "";
     m_tabOrWindowIndex = -1;
+    m_spacerTabIndex = SpacerTabIndex();
+    m_mediaFileName = "";
+    m_histologySpaceKey = HistologySpaceKey();
     
     m_displayGroupAndTabItemHelper = new DisplayGroupAndTabItemHelper();
     
@@ -259,8 +299,21 @@ AnnotationGroup::getName() const
         
         AString spaceName = AnnotationCoordinateSpaceEnum::toGuiName(m_coordinateSpace);
         switch (m_coordinateSpace) {
-            case AnnotationCoordinateSpaceEnum::PIXELS:
-                CaretAssertMessage(0, "Should never be pixels");
+            case AnnotationCoordinateSpaceEnum::CHART:
+                break;
+            case AnnotationCoordinateSpaceEnum::HISTOLOGY:
+                spaceName.append(" "
+                                 + m_histologySpaceKey.getHistologySlicesFileName()
+                                 + " "
+                                 + m_histologySpaceKey.getSliceName());
+                break;
+            case AnnotationCoordinateSpaceEnum::MEDIA_FILE_NAME_AND_PIXEL:
+                spaceName.append(" "
+                                 + m_mediaFileName);
+                break;
+            case AnnotationCoordinateSpaceEnum::SPACER:
+                spaceName.append(" "
+                                 + m_spacerTabIndex.getWindowRowColumnGuiText());
                 break;
             case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
                 break;
@@ -270,6 +323,9 @@ AnnotationGroup::getName() const
                 spaceName.append(" "
                                  + AString::number(getTabOrWindowIndex() + 1));
                 break;
+            case AnnotationCoordinateSpaceEnum::VIEWPORT:
+                CaretAssertMessage(0, "Should never be viewport");
+                break;
             case AnnotationCoordinateSpaceEnum::WINDOW:
                 spaceName.append(" "
                                  + AString::number(getTabOrWindowIndex() + 1));
@@ -280,16 +336,16 @@ AnnotationGroup::getName() const
             case AnnotationGroupTypeEnum::INVALID:
                 CaretAssertMessage(0, "Should never get here");
                 break;
+            case AnnotationGroupTypeEnum::SAMPLES_RETROSPECTIVE:
+                m_name.append("Retrospective");
+                break;
+            case AnnotationGroupTypeEnum::SAMPLES_PROSPECTIVE:
+                m_name.append("Prospective");
+                break;
             case AnnotationGroupTypeEnum::SPACE:
-//                m_name.append("Space Group: "
-//                              + spaceName);
                 m_name.append(spaceName);
                 break;
             case AnnotationGroupTypeEnum::USER:
-//                m_name.append("User Group "
-//                              + AString::number(m_groupKey.getUserGroupUniqueKey())
-//                              + ": "
-//                              + spaceName);
                 m_name.append("Group "
                               + AString::number(m_groupKey.getUserGroupUniqueKey())
                               + ": "
@@ -310,6 +366,33 @@ AnnotationGroup::getTabOrWindowIndex() const
     return m_tabOrWindowIndex;
 }
 
+/**
+ * Index of a spacer tab.
+ */
+SpacerTabIndex
+AnnotationGroup::getSpacerTabIndex() const
+{
+    return m_spacerTabIndex;
+}
+
+/*
+ * @return Name of media file
+ */
+AString
+AnnotationGroup::getMediaFileName() const
+{
+    return m_mediaFileName;
+}
+
+/**
+ * @return The histology space key
+ */
+const HistologySpaceKey&
+AnnotationGroup::getHistologySpaceKey() const
+{
+    return m_histologySpaceKey;
+}
+
 /**
  * @return Unique key displayed in annotation group name.
  */
@@ -322,6 +405,12 @@ AnnotationGroup::getUniqueKey() const
         case AnnotationGroupTypeEnum::INVALID:
             CaretAssertMessage(0, "Should never get here");
             break;
+        case AnnotationGroupTypeEnum::SAMPLES_RETROSPECTIVE:
+            uniqueKey = m_groupKey.getSamplesRetrospectiveUniqueKey();
+            break;
+        case AnnotationGroupTypeEnum::SAMPLES_PROSPECTIVE:
+            uniqueKey = m_groupKey.getSamplesProspectiveUniqueKey();
+            break;
         case AnnotationGroupTypeEnum::SPACE:
             uniqueKey = m_groupKey.getSpaceGroupUniqueKey();
             break;
@@ -419,7 +508,28 @@ AnnotationGroup::validateAddedAnnotation(const Annotation* annotation)
     }
     
     switch (space) {
-        case AnnotationCoordinateSpaceEnum::PIXELS:
+        case AnnotationCoordinateSpaceEnum::CHART:
+            break;
+        case AnnotationCoordinateSpaceEnum::HISTOLOGY:
+            if (m_histologySpaceKey != annotation->getCoordinate(0)->getHistologySpaceKey()) {
+                CaretLogSevere("Attempting to add annotation with non-matching histology slices file name");
+                CaretAssert(0);
+                return false;
+            }
+            break;
+        case AnnotationCoordinateSpaceEnum::MEDIA_FILE_NAME_AND_PIXEL:
+            if (m_mediaFileName != annotation->getCoordinate(0)->getMediaFileName()) {
+                CaretLogSevere("Attempting to add annotation with non-matching media file name");
+                CaretAssert(0);
+                return false;
+            }
+            break;
+        case AnnotationCoordinateSpaceEnum::SPACER:
+            if (m_spacerTabIndex != annotation->getSpacerTabIndex()) {
+                CaretLogSevere("Attempting to add anntation with non-matching spacer tab index");
+                CaretAssert(0);
+                return false;
+            }
             break;
         case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
             break;
@@ -432,6 +542,8 @@ AnnotationGroup::validateAddedAnnotation(const Annotation* annotation)
                 return false;
             }
             break;
+        case AnnotationCoordinateSpaceEnum::VIEWPORT:
+            break;
         case AnnotationCoordinateSpaceEnum::WINDOW:
             if (m_tabOrWindowIndex != annotation->getWindowIndex()) {
                 CaretLogSevere("Attempting to add anntation with non-matching tab index: ");
@@ -892,17 +1004,19 @@ AnnotationGroup::isItemExpandable() const
  *
  * @param displayGroup
  *     The display group.
- * @param tabIndex
+ * @param tabIndexIn
  *     Index of the tab.
  */
 bool
 AnnotationGroup::isItemExpanded(const DisplayGroupEnum::Enum displayGroup,
-                                const int32_t tabIndex) const
+                                const int32_t tabIndexIn) const
 {
     if (m_coordinateSpace == AnnotationCoordinateSpaceEnum::WINDOW) {
        return  m_displayGroupAndTabItemHelper->isExpandedInWindow(m_tabOrWindowIndex);
     }
     
+    const int32_t tabIndex = updateDisplayGroupTabIndex(displayGroup,
+                                                        tabIndexIn);
     return m_displayGroupAndTabItemHelper->isExpanded(displayGroup,
                                                       tabIndex);
 }
@@ -912,14 +1026,14 @@ AnnotationGroup::isItemExpanded(const DisplayGroupEnum::Enum displayGroup,
  *
  * @param displayGroup
  *     The display group.
- * @param tabIndex
+ * @param tabIndexIn
  *     Index of the tab.
  * @param status
  *     New expanded status.
  */
 void
 AnnotationGroup::setItemExpanded(const DisplayGroupEnum::Enum displayGroup,
-                                 const int32_t tabIndex,
+                                 const int32_t tabIndexIn,
                                  const bool status)
 {
     if (m_coordinateSpace == AnnotationCoordinateSpaceEnum::WINDOW) {
@@ -927,6 +1041,8 @@ AnnotationGroup::setItemExpanded(const DisplayGroupEnum::Enum displayGroup,
                                                             status);
     }
     else {
+        const int32_t tabIndex = updateDisplayGroupTabIndex(displayGroup,
+                                                            tabIndexIn);
         m_displayGroupAndTabItemHelper->setExpanded(displayGroup,
                                                     tabIndex,
                                                     status);
@@ -938,13 +1054,16 @@ AnnotationGroup::setItemExpanded(const DisplayGroupEnum::Enum displayGroup,
  *
  * @param displayGroup
  *     The display group.
- * @param tabIndex
+ * @param tabIndexIn
  *     Index of the tab.
  */
 TriStateSelectionStatusEnum::Enum
 AnnotationGroup::getItemDisplaySelected(const DisplayGroupEnum::Enum displayGroup,
-                                 const int32_t tabIndex) const
+                                 const int32_t tabIndexIn) const
 {
+    const int32_t tabIndex = updateDisplayGroupTabIndex(displayGroup,
+                                                        tabIndexIn);
+    
     TriStateSelectionStatusEnum::Enum status = TriStateSelectionStatusEnum::UNSELECTED;
     
     const int numChildren = getNumberOfAnnotations();
@@ -981,14 +1100,14 @@ AnnotationGroup::getItemDisplaySelected(const DisplayGroupEnum::Enum displayGrou
  *
  * @param displayGroup
  *     The display group.
- * @param tabIndex
+ * @param tabIndexIn
  *     Index of the tab.
  * @param status
  *     New selection status.
  */
 void
 AnnotationGroup::setItemDisplaySelected(const DisplayGroupEnum::Enum displayGroup,
-                                 const int32_t tabIndex,
+                                 const int32_t tabIndexIn,
                                  const TriStateSelectionStatusEnum::Enum status)
 {
     switch (status) {
@@ -1002,18 +1121,108 @@ AnnotationGroup::setItemDisplaySelected(const DisplayGroupEnum::Enum displayGrou
             break;
     }
     
+    const int32_t tabIndex = updateDisplayGroupTabIndex(displayGroup,
+                                                        tabIndexIn);
+    
     /*
      * Note: An annotation group's selection status is based
      * of the the group's annotations so we do not need to set
      * an explicit selection status for the group.
      */
-    
     DisplayGroupAndTabItemInterface::setChildrenDisplaySelectedHelper(this,
                                                                 displayGroup,
                                                                 tabIndex,
                                                                 status);
 }
 
+/**
+ * Copy the selections for annotations that are displayed in more than one
+ * tab.  This includes chart, stereotaxic, and surface annotations.
+ *
+ * @param sourceTabIndex
+ *     Index of source tab (copy "from")
+ * @param targetTabIndex
+ *     Index of target tab (copy "to")
+ */
+void
+AnnotationGroup::copySelections(const int32_t sourceTabIndex,
+                                const int32_t targetTabIndex)
+{
+    bool supportedSpaceFlag(false);
+    switch (m_coordinateSpace) {
+        case AnnotationCoordinateSpaceEnum::CHART:
+            supportedSpaceFlag = true;
+            break;
+        case AnnotationCoordinateSpaceEnum::HISTOLOGY:
+            break;
+        case AnnotationCoordinateSpaceEnum::MEDIA_FILE_NAME_AND_PIXEL:
+            break;
+        case AnnotationCoordinateSpaceEnum::SPACER:
+            break;
+        case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
+            supportedSpaceFlag = true;
+            break;
+        case AnnotationCoordinateSpaceEnum::SURFACE:
+            supportedSpaceFlag = true;
+            break;
+        case AnnotationCoordinateSpaceEnum::TAB:
+            break;
+        case AnnotationCoordinateSpaceEnum::VIEWPORT:
+            break;
+        case AnnotationCoordinateSpaceEnum::WINDOW:
+            break;
+    }
+    
+    if ( ! supportedSpaceFlag) {
+        return;
+    }
+    
+    /*
+     * Note: We DO NOT copy the selection status for the group.
+     * When getItemDisplaySelected() is called, it will determine
+     * the selection status based (on, partial, off) based upon
+     * the selection status of the annotations in the group.
+     */
+    setItemExpanded(DisplayGroupEnum::DISPLAY_GROUP_TAB,
+                    targetTabIndex,
+                    isItemExpanded(DisplayGroupEnum::DISPLAY_GROUP_TAB,
+                                   sourceTabIndex));
+    
+    for (auto ann : m_annotations) {
+        ann->setItemDisplaySelected(DisplayGroupEnum::DISPLAY_GROUP_TAB,
+                                    targetTabIndex,
+                                    ann->getItemDisplaySelected(DisplayGroupEnum::DISPLAY_GROUP_TAB,
+                                                                sourceTabIndex));
+        ann->setItemExpanded(DisplayGroupEnum::DISPLAY_GROUP_TAB,
+                             targetTabIndex,
+                             ann->isItemExpanded(DisplayGroupEnum::DISPLAY_GROUP_TAB,
+                                                 sourceTabIndex));
+    }
+}
+
+/**
+ * Update the tab index to correspond to the tab index used for this
+ * annotation group if it is in tab annotation space.  This functionality 
+ * was added to resolve WB-831.
+ *
+ * @param displayGroup
+ *     The display group.
+ * @param tabIndex
+ *     Index of the tab.
+ */
+int32_t
+AnnotationGroup::updateDisplayGroupTabIndex(const DisplayGroupEnum::Enum displayGroup,
+                                       const int32_t tabIndex) const
+{
+    int32_t tabIndexOut(tabIndex);
+    if (getCoordinateSpace() == AnnotationCoordinateSpaceEnum::TAB) {
+        if (displayGroup == DisplayGroupEnum::DISPLAY_GROUP_TAB) {
+            tabIndexOut = getTabOrWindowIndex();
+        }
+    }
+    return tabIndexOut;
+}
+
 /**
  * Is this item selected for editing in the given window?
  *
@@ -1023,7 +1232,7 @@ AnnotationGroup::setItemDisplaySelected(const DisplayGroupEnum::Enum displayGrou
  *     Selection status.
  */
 bool
-AnnotationGroup::isItemSelectedForEditingInWindow(const int32_t windowIndex)
+AnnotationGroup::isItemSelectedForEditingInWindow(const int32_t /*windowIndex*/)
 {
     return false;
 }
