@@ -31,13 +31,12 @@
 #include "AnnotationArrangerInputs.h"
 #include "AnnotationCoordinate.h"
 #include "AnnotationManager.h"
-#include "AnnotationOneDimensionalShape.h"
+#include "AnnotationTwoCoordinateShape.h"
 #include "AnnotationRedoUndoCommand.h"
 #include "AnnotationText.h"
 #include "BrainOpenGLTextRenderInterface.h"
 #include "CaretAssert.h"
 #include "CaretException.h"
-#include "DisplayPropertiesAnnotationTextSubstitution.h"
 #include "EventGetViewportSize.h"
 #include "EventManager.h"
 
@@ -54,8 +53,9 @@ using namespace caret;
 /**
  * Constructor.
  */
-AnnotationArrangerExecutor::AnnotationArrangerExecutor()
+AnnotationArrangerExecutor::AnnotationArrangerExecutor(const UserInputModeEnum::Enum userInputMode)
 : CaretObject(),
+m_userInputMode(userInputMode),
 m_mode(MODE_NONE),
 m_annotationManager(NULL),
 m_alignment(AnnotationAlignmentEnum::ALIGN_BOTTOM),
@@ -75,6 +75,8 @@ AnnotationArrangerExecutor::~AnnotationArrangerExecutor()
 /**
  * Apply alignment modification to selected annotations
  *
+ * @param userInputMode
+ *     The current user input mode which MUST be ANNOTATIONS or TILE_TABS_LAYOUT_EDITING
  * @param annotationManager
  *     The annotation manager.
  * @param arrangerInputs
@@ -115,6 +117,8 @@ AnnotationArrangerExecutor::alignAnnotations(AnnotationManager* annotationManage
 /**
  * Apply distribute modification to selected annotations
  *
+ * @param userInputMode
+ *     The current user input mode which MUST be ANNOTATIONS or TILE_TABS_LAYOUT_EDITING
  * @param annotationManager
  *     The annotation manager.
  * @param arrangerInputs
@@ -339,7 +343,8 @@ AnnotationArrangerExecutor::distributeAnnotationsPrivate(const AnnotationArrange
         CaretAssert(beforeMoving.size() == afterMoving.size());
         AnnotationRedoUndoCommand* undoCommand = new AnnotationRedoUndoCommand();
         undoCommand->setModeLocationAndSize(beforeMoving,
-                                            afterMoving);
+                                            afterMoving,
+                                            AnnotationDistributeEnum::toGuiName(m_distribute));
         undoCommand->setDescription(AnnotationDistributeEnum::toGuiName(m_distribute));
         
         validFlag = m_annotationManager->applyCommand(undoCommand,
@@ -449,7 +454,8 @@ AnnotationArrangerExecutor::alignAnnotationsPrivate(const AnnotationArrangerInpu
         CaretAssert(beforeMoving.size() == afterMoving.size());
         AnnotationRedoUndoCommand* undoCommand = new AnnotationRedoUndoCommand();
         undoCommand->setModeLocationAndSize(beforeMoving,
-                                            afterMoving);
+                                            afterMoving,
+                                            AnnotationAlignmentEnum::toGuiName(m_alignment));
         undoCommand->setDescription(AnnotationAlignmentEnum::toGuiName(m_alignment));
         
         validFlag = m_annotationManager->applyCommand(undoCommand,
@@ -520,8 +526,8 @@ AnnotationArrangerExecutor::initializeForArranging(const AnnotationArrangerInput
          annInfoIter++) {
         const AnnotationInfo& annInfo = *annInfoIter;
         
-        m_allAnnotationsBoundingBox.update(annInfo.m_windowBoundingBox.getMinXYZ());
-        m_allAnnotationsBoundingBox.update(annInfo.m_windowBoundingBox.getMaxXYZ());
+        m_allAnnotationsBoundingBox.update(annInfo.m_windowBoundingBox.getMinXYZ().data());
+        m_allAnnotationsBoundingBox.update(annInfo.m_windowBoundingBox.getMaxXYZ().data());
     }
 }
 
@@ -542,6 +548,7 @@ AnnotationArrangerExecutor::getAnnotationsForArranging(const AnnotationArrangerI
     annotationsOut.clear();
     
     std::vector<AnnotationCoordinateSpaceEnum::Enum> spaces;
+    spaces.push_back(AnnotationCoordinateSpaceEnum::SPACER);
     spaces.push_back(AnnotationCoordinateSpaceEnum::TAB);
     spaces.push_back(AnnotationCoordinateSpaceEnum::WINDOW);
     
@@ -616,6 +623,16 @@ AnnotationArrangerExecutor::setupAnnotationInfo(const AnnotationArrangerInputs&
             case AnnotationCoordinateSpaceEnum::CHART:
                 CaretAssert(0);
                 break;
+            case AnnotationCoordinateSpaceEnum::HISTOLOGY:
+                CaretAssert(0);
+                break;
+            case AnnotationCoordinateSpaceEnum::MEDIA_FILE_NAME_AND_PIXEL:
+                CaretAssert(0);
+                break;
+            case AnnotationCoordinateSpaceEnum::SPACER:
+                getSpacerTabViewport(annotation->getSpacerTabIndex(),
+                                     annViewport);
+                break;
             case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
                 CaretAssert(0);
                 break;
@@ -637,8 +654,8 @@ AnnotationArrangerExecutor::setupAnnotationInfo(const AnnotationArrangerInputs&
                 break;
         }
         
-        const AnnotationOneDimensionalShape* oneDimAnn = dynamic_cast<const AnnotationOneDimensionalShape*>(annotation);
-        const AnnotationTwoDimensionalShape* twoDimAnn = dynamic_cast<const AnnotationTwoDimensionalShape*>(annotation);
+        const AnnotationTwoCoordinateShape* oneDimAnn = dynamic_cast<const AnnotationTwoCoordinateShape*>(annotation);
+        const AnnotationOneCoordinateShape* twoDimAnn = dynamic_cast<const AnnotationOneCoordinateShape*>(annotation);
         
         float viewportPixelOneXYZ[3] = { 0.0, 0.0, 0.0 };
         float viewportPixelTwoXYZ[3] = { 0.0, 0.0, 0.0 };
@@ -864,7 +881,9 @@ void
 AnnotationArrangerExecutor::getTabViewport(const int32_t tabIndex,
                                            int32_t tabViewportOut[4])
 {
-    
+    /*
+     * Check cached viewports to avoid retrieving it a second time
+     */
     std::map<int32_t, ViewportArray>::iterator iter = m_tabViewports.find(tabIndex);
     if (iter != m_tabViewports.end()) {
         tabViewportOut[0] = iter->second.m_viewport[0];
@@ -893,6 +912,31 @@ AnnotationArrangerExecutor::getTabViewport(const int32_t tabIndex,
     }
 }
 
+/**
+ * Get the viewport for the given spacer tab.  Viewports are cached for efficiency.
+ *
+ * @param spacerTabIndex
+ *     Index of the spacer tab.
+ * @param tabViewportOut
+ *     Viewport of the tab.
+ * @throw  CaretException
+ *     If there is an error.
+ */
+void
+AnnotationArrangerExecutor::getSpacerTabViewport(const SpacerTabIndex& spacerTabIndex,
+                                                 int32_t tabViewportOut[4])
+{
+    EventGetViewportSize vpEvent(spacerTabIndex);
+    EventManager::get()->sendEvent(vpEvent.getPointer());
+    if (vpEvent.isViewportSizeValid()) {
+        vpEvent.getViewportSize(tabViewportOut);
+    }
+    else {
+        throw CaretException("Failed to get viewport size for spacer tab index "
+                             + spacerTabIndex.toString());
+    }
+}
+
 /**
  * Get a description of this object's content.
  * @return String describing this object's content.
@@ -1005,8 +1049,8 @@ AnnotationArrangerExecutor::AnnotationInfo::moveAnnotationByXY(Annotation* annot
                                                                const float dy) const
 {
     CaretAssert(m_annotation);
-    AnnotationOneDimensionalShape* oneDimAnn = dynamic_cast<AnnotationOneDimensionalShape*>(annotation);
-    AnnotationTwoDimensionalShape* twoDimAnn = dynamic_cast<AnnotationTwoDimensionalShape*>(annotation);
+    AnnotationTwoCoordinateShape* oneDimAnn = dynamic_cast<AnnotationTwoCoordinateShape*>(annotation);
+    AnnotationOneCoordinateShape* twoDimAnn = dynamic_cast<AnnotationOneCoordinateShape*>(annotation);
     
     const float newPixelOneX = m_viewportPixelOneXY[0] + dx;
     const float relativeOneX = (newPixelOneX / m_viewport[2]) * 100.0;
