@@ -22,6 +22,7 @@
 
 #include <cstdlib>
 
+#include <QRegularExpression>
 #include <QStringList>
 
 #define __BRAIN_OPENGL_DEFINE_H
@@ -32,6 +33,7 @@
 #include "CaretLogger.h"
 #include "CaretPreferences.h"
 #include "DummyFontTextRenderer.h"
+#include "EventAnnotationTextGetBounds.h"
 #include "EventGetBrainOpenGLTextRenderer.h"
 #include "EventGraphicsOpenGLCreateBufferObject.h"
 #include "EventGraphicsOpenGLCreateTextureName.h"
@@ -64,6 +66,7 @@ BrainOpenGL::BrainOpenGL(BrainOpenGLTextRenderInterface* textRenderer)
     this->borderBeingDrawn = NULL;
     m_drawHighlightedEndPoints = false;
     
+    EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_ANNOTATION_TEXT_GET_BOUNDS);
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_GET_TEXT_RENDERER_FOR_WINDOW);
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_GRAPHICS_OPENGL_CREATE_BUFFER_OBJECT);
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_GRAPHICS_OPENGL_CREATE_TEXTURE_NAME);
@@ -94,7 +97,30 @@ BrainOpenGL::~BrainOpenGL()
 void
 BrainOpenGL::receiveEvent(Event* event)
 {
-    if (event->getEventType() == EventTypeEnum::EVENT_GRAPHICS_OPENGL_CREATE_BUFFER_OBJECT) {
+    if (event->getEventType() == EventTypeEnum::EVENT_ANNOTATION_TEXT_GET_BOUNDS) {
+        EventAnnotationTextGetBounds* textBoundsEvent = dynamic_cast<EventAnnotationTextGetBounds*>(event);
+        CaretAssert(textBoundsEvent);
+        
+        if (m_textRenderer != NULL) {
+            double textWidth(0.0);
+            double textHeight(0.0);
+            BrainOpenGLTextRenderInterface::DrawingFlags textDrawingFlags;
+            textDrawingFlags.setDrawSubstitutedText(false);
+
+            m_textRenderer->getTextWidthHeightInPixels(textBoundsEvent->getAnnotationText(),
+                                                       textDrawingFlags,
+                                                       textBoundsEvent->getViewportWidth(),
+                                                       textBoundsEvent->getViewportHeight(),
+                                                       textWidth,
+                                                       textHeight);
+            
+            textBoundsEvent->setTextWidthHeight(textWidth,
+                                                textHeight);
+        }
+        
+        textBoundsEvent->setEventProcessed();
+    }
+    else if (event->getEventType() == EventTypeEnum::EVENT_GRAPHICS_OPENGL_CREATE_BUFFER_OBJECT) {
         EventGraphicsOpenGLCreateBufferObject* createBufferEvent
         = dynamic_cast<EventGraphicsOpenGLCreateBufferObject*>(event);
         CaretAssert(createBufferEvent);
@@ -200,17 +226,23 @@ BrainOpenGL::receiveEvent(Event* event)
  *
  * @param windowIndex
  *    Index of window for drawing.
+ * @param windowsUserInputMode
+ *    User input mode for window
  * @param brain
  *    The brain (must be valid!)
  * @param contextSharingGroupPointer
  *    Pointer to the active OpenGL context.
  * @param viewportContents
  *    Viewport info for drawing.
+ * @param graphicsFramesPerSecond
+ *    Frames for second for display in window
  */
 void BrainOpenGL::drawModels(const int32_t windowIndex,
+                             const UserInputModeEnum::Enum windowsUserInputMode,
                              Brain* brain,
                              void* contextSharingGroupPointer,
-                             const std::vector<const BrainOpenGLViewportContent*>& viewportContents)
+                             const std::vector<const BrainOpenGLViewportContent*>& viewportContents,
+                             const GraphicsFramesPerSecond* graphicsFramesPerSecond)
 {
     m_contextSharingGroupPointer = contextSharingGroupPointer;
     
@@ -222,8 +254,10 @@ void BrainOpenGL::drawModels(const int32_t windowIndex,
     
     
     drawModelsImplementation(windowIndex,
+                             windowsUserInputMode,
                              brain,
-                             vpContents);
+                             vpContents,
+                             graphicsFramesPerSecond);
     
     deleteUnusedOpenGLNames();
     
@@ -235,6 +269,8 @@ void BrainOpenGL::drawModels(const int32_t windowIndex,
  *
  * @param windowIndex
  *    Index of window for selection.
+ * @param windowsUserInputMode
+ *    User input mode for window
  * @param brain
  *    The brain (must be valid!)
  * @param contextSharingGroupPointer
@@ -254,6 +290,7 @@ void BrainOpenGL::drawModels(const int32_t windowIndex,
  *    selected.
  */
 void BrainOpenGL::selectModel(const int32_t windowIndex,
+                              const UserInputModeEnum::Enum windowsUserInputMode,
                               Brain* brain,
                               void* contextSharingGroupPointer,
                               const BrainOpenGLViewportContent* viewportContent,
@@ -264,6 +301,7 @@ void BrainOpenGL::selectModel(const int32_t windowIndex,
     m_contextSharingGroupPointer = contextSharingGroupPointer;
 
     selectModelImplementation(windowIndex,
+                              windowsUserInputMode,
                               brain,
                               viewportContent,
                               mouseX,
@@ -283,6 +321,8 @@ void BrainOpenGL::selectModel(const int32_t windowIndex,
  *
  * @param windowIndex
  *    Index of window for projection
+ * @param windowsUserInputMode
+ *    User input mode for window
  * @param brain
  *    The brain (must be valid!)
  * @param contextSharingGroupPointer
@@ -297,16 +337,18 @@ void BrainOpenGL::selectModel(const int32_t windowIndex,
  *    Output with projection result.
  */
 void BrainOpenGL::projectToModel(const int32_t windowIndex,
+                                 const UserInputModeEnum::Enum windowUserInputMode,
                                  Brain* brain,
-                    void* contextSharingGroupPointer,
-                    const BrainOpenGLViewportContent* viewportContent,
-                    const int32_t mouseX,
-                    const int32_t mouseY,
-                    SurfaceProjectedItem& projectionOut)
+                                 void* contextSharingGroupPointer,
+                                 const BrainOpenGLViewportContent* viewportContent,
+                                 const int32_t mouseX,
+                                 const int32_t mouseY,
+                                 SurfaceProjectedItem& projectionOut)
 {
     m_contextSharingGroupPointer = contextSharingGroupPointer;
 
     projectToModelImplementation(windowIndex,
+                                 windowUserInputMode,
                                  brain,
                                  viewportContent,
                                  mouseX,
@@ -568,22 +610,34 @@ BrainOpenGL::testForVersionOfOpenGLSupported(const AString& versionOfOpenGL)
 
 /**
  * Test for the required version of OpenGL Workbench needs.
- *
+ * @param guiFlag
+ *    True if in GUI
  * @param errorMessageOut
  *     Output with error message if required version is not available.
  * @return 
  *     True if required version available, else false.
  */
 bool
-BrainOpenGL::testForRequiredOpenGLVersion(AString& errorMessageOut)
+BrainOpenGL::testForRequiredOpenGLVersion(const bool guiFlag,
+                                          AString& errorMessageOut)
 {
-    const AString minimumOpenGLVersion("1.5");
+    const AString minimumOpenGLVersion("2.1");
     if ( ! BrainOpenGL::testForVersionOfOpenGLSupported(minimumOpenGLVersion)) {
-        const AString msg("OpenGL Version "
+        AString msg("OpenGL Version "
                           + minimumOpenGLVersion
                           + " or later is required.  This computer has version "
-                          + BrainOpenGL::getOpenGLVersion()
-                          + "\nYou may continue but the software may crash.");
+                          + BrainOpenGL::getOpenGLVersion());
+        if (guiFlag) {
+            msg += ("\nYou may continue but the software may crash.");
+        }
+        else {
+            msg += ("\nIf Virtual GL is running, it will intercept OpenGL function "
+                    "\ncalls and prevent the correct OpenGL libraries from being "
+                    "\nused resulting in an invalid OpenGL version number.  "
+                    "\nYou will need to disable Virtual GL.  If the environment "
+                    "\nvariable LD_PRELOAD contains 'faker' libraries, unsetting "
+                    "\nthis environment variable may disable Virtual GL.");
+        }
         errorMessageOut = msg;
         return false;
     }
@@ -598,7 +652,22 @@ BrainOpenGL::testForRequiredOpenGLVersion(AString& errorMessageOut)
 AString
 BrainOpenGL::getOpenGLVersion()
 {
-    return QLatin1String(reinterpret_cast<const char*>(glGetString(GL_VERSION)));
+    QString versionStr;
+    
+    const GLubyte* versionPtr(glGetString(GL_VERSION));
+    if (versionPtr != NULL) {
+        versionStr = QLatin1String(reinterpret_cast<const char*>(versionPtr));
+        if (versionStr.isEmpty()) {
+            /*
+             * Note: version may be empty in 1.1
+             */
+            versionStr = "1.1  (Unknown/Empty)";
+        }
+    }
+    else {
+        versionStr = "1.1  Error (pointer == 0)";
+    }
+    return versionStr;
 }
 
 
@@ -621,7 +690,7 @@ BrainOpenGL::getOpenGLMajorMinorVersions(const AString& versionString,
      * Major and minor version are separated by a period.
      * Vendor information may follow and begin with whitespace.
      */
-    const QStringList sl = versionString.split(QRegExp("[\\s|\\.]"));
+    const QStringList sl = versionString.split(QRegularExpression("[\\s|\\.]"));
     
     if (sl.count() >= 2) {
         minorVersionOut = sl.at(1);
@@ -638,226 +707,6 @@ BrainOpenGL::getOpenGLMajorMinorVersions(const AString& versionString,
     }
 }
 
-///**
-// * Initialize the drawing mode using the most optimal drawing given
-// * the compile time and run time constraints.
-// */
-//void
-//BrainOpenGL::initializeOpenGL()
-//{
-//    AString compileVersions = "OpenGL Header File Versions Supported: ";
-//#ifdef GL_VERSION_1_1
-//    compileVersions += " 1.1";
-//#endif
-//#ifdef GL_VERSION_1_2
-//    compileVersions += " 1.2";
-//#endif
-//#ifdef GL_VERSION_1_3
-//    compileVersions += " 1.3";
-//#endif
-//#ifdef GL_VERSION_1_4
-//    compileVersions += " 1.4";
-//#endif
-//#ifdef GL_VERSION_1_5
-//    compileVersions += " 1.5";
-//#endif
-//#ifdef GL_VERSION_2_0
-//    compileVersions += " 2.0";
-//#endif
-//#ifdef GL_VERSION_2_1
-//    compileVersions += " 2.1";
-//#endif
-//#ifdef GL_VERSION_3_0
-//    compileVersions += " 3.0";
-//#endif
-//#ifdef GL_VERSION_3_1
-//    compileVersions += " 3.1";
-//#endif
-//#ifdef GL_VERSION_3_2
-//    compileVersions += " 3.2";
-//#endif
-//#ifdef GL_VERSION_3_3
-//    compileVersions += " 3.3";
-//#endif
-//#ifdef GL_VERSION_4_0
-//    compileVersions += " 4.0";
-//#endif
-//#ifdef GL_VERSION_4_1
-//    compileVersions += " 4.1";
-//#endif
-//#ifdef GL_VERSION_4_2
-//    compileVersions += " 4.2";
-//#endif
-//#ifdef GL_VERSION_4_3
-//    compileVersions += " 4.3";
-//#endif
-//#ifdef GL_VERSION_4_4
-//    compileVersions += " 4.4";
-//#endif
-//#ifdef GL_VERSION_4_4
-//    compileVersions += " 4.4";
-//#endif
-//#ifdef GL_VERSION_4_5
-//    compileVersions += " 4.5";
-//#endif
-//#ifdef GL_VERSION_5_0
-//    compileVersions += " 5.0";
-//#endif
-//    
-//#ifdef GL_OES_VERSION_1_0
-//    compileVersions += " ES_1.0";
-//#endif
-//#ifdef GL_ES_VERSION_2_0
-//    compileVersions += " ES_2.0";
-//#endif
-//#ifdef GL_ES_VERSION_3_0
-//    compileVersions += " ES_3.0";
-//#endif
-//    
-//    s_runtimeLibraryVersionOfOpenGL = QLatin1String(reinterpret_cast<const char*>(glGetString(GL_VERSION)));
-//    if (s_runtimeLibraryVersionOfOpenGL.isEmpty()) {
-//        s_runtimeLibraryVersionOfOpenGL = "1.1";
-//    }
-//    getOpenGLMajorMinorVersions(s_runtimeLibraryVersionOfOpenGL,
-//                                s_runtimeLibraryMajorVersionOfOpenGL,
-//                                s_runtimeLibraryMinorVersionOfOpenGL);
-//    
-//    //
-//    // Note: The version string might be something like 1.2.4.  std::atof()
-//    // will get just the 1.2 which is okay.
-//    //
-//    const char* vendorStr = (char*)(glGetString(GL_VENDOR));
-//    const char* renderStr = (char*)(glGetString(GL_RENDERER));
-//    AString lineInfo = (compileVersions
-//                        + "\nOpenGL Runtime Version: " + s_runtimeLibraryVersionOfOpenGL
-//                        + "\nMajor Runtime Version: " + BrainOpenGL::s_runtimeLibraryMajorVersionOfOpenGL
-//                        + "\nMinor Runtime Version: " + BrainOpenGL::s_runtimeLibraryMinorVersionOfOpenGL
-//                        + "\nOpenGL Vendor: " + AString(vendorStr)
-//                        + "\nOpenGL Renderer: " + AString(renderStr));
-//    
-//    lineInfo += "\n";
-//    lineInfo += ("\nFont Renderer: " + m_textRenderer->getName());
-//    lineInfo += "\n";
-//    
-//#ifdef GL_VERSION_2_0
-//    if (testForVersionOfOpenGLSupported("2.0")) {
-//        GLfloat values[2];
-//        glGetFloatv (GL_ALIASED_LINE_WIDTH_RANGE, values);
-//        const AString aliasedLineWidthRange = ("GL_ALIASED_LINE_WIDTH_RANGE value is "
-//                                               + AString::fromNumbers(values, 2, ", "));
-//        
-//        glGetFloatv (GL_SMOOTH_LINE_WIDTH_RANGE, values);
-//        const AString smoothLineWidthRange = ("GL_SMOOTH_LINE_WIDTH_RANGE value is "
-//                                              + AString::fromNumbers(values, 2, ", "));
-//        
-//        glGetFloatv (GL_SMOOTH_LINE_WIDTH_GRANULARITY, values);
-//        const AString smoothLineWidthGranularity = ("GL_SMOOTH_LINE_WIDTH_GRANULARITY value is "
-//                                                    + AString::number(values[0]));
-//        
-//        lineInfo += ("\n" + aliasedLineWidthRange
-//                     + "\n" + smoothLineWidthRange
-//                     + "\n" + smoothLineWidthGranularity);
-//    }
-//#endif // GL_VERSION_2_0
-////#else  // GL_VERSION_2_0
-//    GLfloat values[2];
-//    glGetFloatv (GL_LINE_WIDTH_RANGE, values);
-//    const AString lineWidthRange = ("GL_LINE_WIDTH_RANGE value is "
-//                                    + AString::fromNumbers(values, 2, ", "));
-//    
-//    glGetFloatv (GL_LINE_WIDTH_GRANULARITY, values);
-//    const AString lineWidthGranularity = ("GL_LINE_WIDTH_GRANULARITY value is "
-//                                          + AString::number(values[0]));
-//    lineInfo += ("\n" + lineWidthRange
-//                + "\n" + lineWidthGranularity);
-////#endif // GL_VERSION_2_0
-//    
-//    float sizes[2];
-//    glGetFloatv(GL_POINT_SIZE_RANGE, sizes);
-//    s_minPointSize = sizes[0];
-//    s_maxPointSize = sizes[1];
-//    glGetFloatv(GL_LINE_WIDTH_RANGE, sizes);
-//    s_minLineWidth = sizes[0];
-//    s_maxLineWidth = sizes[1];
-//    
-//    s_supportsDisplayLists = false;
-//    s_supportsImmediateMode = false;
-//    s_supportsVertexBuffers = false;
-//    
-//    GLint maximumNumberOfClipPlanes;
-//    glGetIntegerv(GL_MAX_CLIP_PLANES,
-//                  & maximumNumberOfClipPlanes);
-//    lineInfo += ("\n\nMaximum number of clipping planes is "
-//                 + AString::number(maximumNumberOfClipPlanes));
-//    
-//    GLint redBits, greenBits, blueBits, alphaBits;
-//    glGetIntegerv(GL_RED_BITS,   &redBits);
-//    glGetIntegerv(GL_GREEN_BITS, &greenBits);
-//    glGetIntegerv(GL_BLUE_BITS,  &blueBits);
-//    glGetIntegerv(GL_ALPHA_BITS, &alphaBits);
-//    lineInfo += ("\n\nBuffer bits red/green/blue/apha: ("
-//                 + AString::number(redBits) + ", "
-//                 + AString::number(greenBits) + ", "
-//                 + AString::number(blueBits) + ", "
-//                 + AString::number(alphaBits) + ")");
-//    
-//    /*
-//     * Get the OpenGL Extensions.
-//     */
-//    bool haveARBCompatibility = false;
-//    const QString extensionsString((char*)glGetString(GL_EXTENSIONS));
-//    const QStringList extensionsList = extensionsString.split(QChar(' '));
-//    QStringListIterator extensionsIterator(extensionsList);
-//    AString extInfo = ("\n\nOpenGL Extensions:");
-//    while (extensionsIterator.hasNext()) {
-//        const QString ext = extensionsIterator.next();
-//        extInfo += ("\n   " + ext);
-//        
-//        if (ext == "GL_ARB_compatibility") {
-//            haveARBCompatibility = true;
-//        }
-//    }
-//    
-//    if (testForVersionOfOpenGLSupported("3.1")) {
-//        if (haveARBCompatibility == false) {
-//            CaretLogSevere("OpenGL 3.1 or later and ARB compatibilty extensions not found.\n"
-//                           "OpenGL may fail.");
-//        }
-//    }
-//
-//#if BRAIN_OPENGL_INFO_SUPPORTS_IMMEDIATE
-//    s_supportsImmediateMode = true;
-//#endif // BRAIN_OPENGL_INFO_SUPPORTS_IMMEDIATE
-//    
-//#if BRAIN_OPENGL_INFO_SUPPORTS_DISPLAY_LISTS
-//    s_supportsDisplayLists = true;
-//#endif // BRAIN_OPENGL_INFO_SUPPORTS_DISPLAY_LISTS
-//    
-//#ifdef BRAIN_OPENGL_INFO_SUPPORTS_VERTEX_BUFFERS
-//    s_supportsVertexBuffers = true;
-//#endif // BRAIN_OPENGL_INFO_SUPPORTS_VERTEX_BUFFERS
-//    
-//    lineInfo += ("\n\nBest Drawing Mode: "
-//            + BrainOpenGL::getBestDrawingModeName());
-//    lineInfo += ("\nDisplay Lists Supported: "
-//            + AString::fromBool(s_supportsDisplayLists));
-//    lineInfo += ("\nImmediate Mode Supported: "
-//            + AString::fromBool(s_supportsImmediateMode));
-//    lineInfo += ("\nVertex Buffers Supported: "
-//            + AString::fromBool(s_supportsVertexBuffers));
-//    
-//    lineInfo += extInfo;
-//    
-//    CaretLogConfig(lineInfo);
-//    
-//    m_openGLInformation = lineInfo;
-//    
-//    /*
-//     * Call to validate the draw mode selection logic.
-//     */
-//    getBestDrawingMode();
-//}
-
 /**
  * Initialize the drawing mode using the most optimal drawing given
  * the compile time and run time constraints.
@@ -866,10 +715,7 @@ void
 BrainOpenGL::initializeOpenGL()
 {
 
-    s_runtimeLibraryVersionOfOpenGL = QLatin1String(reinterpret_cast<const char*>(glGetString(GL_VERSION)));
-    if (s_runtimeLibraryVersionOfOpenGL.isEmpty()) {
-        s_runtimeLibraryVersionOfOpenGL = "1.1";
-    }
+    s_runtimeLibraryVersionOfOpenGL = getOpenGLVersion();
     getOpenGLMajorMinorVersions(s_runtimeLibraryVersionOfOpenGL,
                                 s_runtimeLibraryMajorVersionOfOpenGL,
                                 s_runtimeLibraryMinorVersionOfOpenGL);
@@ -928,6 +774,17 @@ BrainOpenGL::initializeOpenGL()
      * Call to validate the draw mode selection logic.
      */
     getBestDrawingMode();
+    
+    GraphicsUtilitiesOpenGL::setMajorMinorVersion(s_runtimeLibraryMajorVersionOfOpenGL.toInt(),
+                                                  s_runtimeLibraryMinorVersionOfOpenGL.toInt());
+
+    GLint textureMax, texture3DMax;
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE,
+                  &textureMax);
+    glGetIntegerv(GL_MAX_3D_TEXTURE_SIZE,
+                  &texture3DMax);
+    GraphicsUtilitiesOpenGL::setMaximumTextureDimension(textureMax,
+                                                        texture3DMax);
 }
 
 /**
@@ -1031,6 +888,9 @@ BrainOpenGL::getOpenGLInformation()
     
 #ifdef GL_VERSION_2_0
     if (testForVersionOfOpenGLSupported("2.0")) {
+        const char* glslVersionString = (char*)glGetString(GL_SHADING_LANGUAGE_VERSION);
+        lineInfo += ("\nOpenGL Shading Language (GLSL) Version: " + AString(glslVersionString) + "\n");
+        
         GLfloat values[2];
         glGetFloatv (GL_ALIASED_LINE_WIDTH_RANGE, values);
         const AString aliasedLineWidthRange = ("GL_ALIASED_LINE_WIDTH_RANGE value is "
@@ -1074,20 +934,25 @@ BrainOpenGL::getOpenGLInformation()
     lineInfo += ("\n\nMaximum number of clipping planes is "
                  + AString::number(maximumNumberOfClipPlanes));
     
-    GLint maxNameStackDepth, maxModelStackDepth, maxProjStackDepth;
+    GLint maxNameStackDepth, maxModelStackDepth, maxProjStackDepth, maxAttributeStackDepth;
     glGetIntegerv(GL_MAX_PROJECTION_STACK_DEPTH,
                   &maxProjStackDepth);
     glGetIntegerv(GL_MAX_MODELVIEW_STACK_DEPTH,
                   &maxModelStackDepth);
     glGetIntegerv(GL_MAX_NAME_STACK_DEPTH,
                   &maxNameStackDepth);
+    glGetIntegerv(GL_MAX_ATTRIB_STACK_DEPTH,
+                  &maxAttributeStackDepth);
+    
     lineInfo += ("\n\nMaximum Modelview Matrix Stack Depth "
                  + QString::number(maxModelStackDepth));
     lineInfo += ("\nMaximum Name Matrix Stack Depth "
                  + QString::number(maxNameStackDepth));
     lineInfo += ("\nMaximum Projection Matrix Stack Depth "
                  + QString::number(maxProjStackDepth));
-    
+    lineInfo += ("\nMaximum Attribute Stack Depth "
+                 + QString::number(maxAttributeStackDepth));
+
     GLint redBits, greenBits, blueBits, alphaBits;
     glGetIntegerv(GL_RED_BITS,   &redBits);
     glGetIntegerv(GL_GREEN_BITS, &greenBits);
@@ -1099,6 +964,15 @@ BrainOpenGL::getOpenGLInformation()
                  + AString::number(blueBits) + ", "
                  + AString::number(alphaBits) + ")");
     
+    GLint depthBits;
+    glGetIntegerv(GL_DEPTH_BITS, &depthBits);
+    lineInfo += ("\nDepth Bits: "
+                 + AString::number(depthBits));
+    GLint stencilBits;
+    glGetIntegerv(GL_STENCIL_BITS, &stencilBits);
+    lineInfo += ("\nStencil Bits: "
+                 + AString::number(stencilBits));
+    
     lineInfo += ("\n\nBest Drawing Mode: "
                  + BrainOpenGL::getBestDrawingModeName());
     lineInfo += ("\nDisplay Lists Supported: "
@@ -1118,6 +992,79 @@ BrainOpenGL::getOpenGLInformation()
     lineInfo += ("\nSamples Count: " + AString::number(sampleCount));
     lineInfo += ("\nMultisampling Enabled: " + AString::fromBool(glIsEnabled(GL_MULTISAMPLE)));
     
+    GLint textureMax, texture3DMax;
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE,
+                  &textureMax);
+    glGetIntegerv(GL_MAX_3D_TEXTURE_SIZE,
+                  &texture3DMax);
+    lineInfo += "\n";
+    lineInfo += ("\nTexture Max: " + AString::number(textureMax));
+    lineInfo += ("\nTexture 3D Max: " + AString::number(texture3DMax));
+    lineInfo += "\n";
+
+    GLint numCompTexFormats(0);
+    glGetIntegerv(GL_NUM_COMPRESSED_TEXTURE_FORMATS, &numCompTexFormats);
+    lineInfo += ("\nNum Compressed Texture Formats: " + AString::number(numCompTexFormats));
+    
+    if (numCompTexFormats > 0) {
+        std::vector<GLint> compTextureFormats;
+        compTextureFormats.resize(numCompTexFormats);
+        glGetIntegerv(GL_COMPRESSED_TEXTURE_FORMATS, &compTextureFormats[0]);
+        for (GLint fmt : compTextureFormats) {
+            QString name;
+            QString decimalValue;
+            QString hexValue;
+            GraphicsUtilitiesOpenGL::getTextCompressionEnumInfo(fmt, name, decimalValue, hexValue);
+            lineInfo += ("\n   Comp Texture Format: "
+                         + decimalValue
+                         + " "
+                         + hexValue
+                         + " "
+                         + name);
+        }
+    }
+
+    GLint numCompTexFormatsARB(0);
+    glGetIntegerv(GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB, &numCompTexFormatsARB);
+    lineInfo += ("\nNum ARB Compressed Texture Formats: " + AString::number(numCompTexFormatsARB));
+    
+    if (numCompTexFormatsARB > 0) {
+        std::vector<GLint> compTextureFormats;
+        compTextureFormats.resize(numCompTexFormats);
+        glGetIntegerv(GL_COMPRESSED_TEXTURE_FORMATS_ARB, &compTextureFormats[0]);
+        for (GLint fmt : compTextureFormats) {
+            QString name;
+            QString decimalValue;
+            QString hexValue;
+            GraphicsUtilitiesOpenGL::getTextCompressionEnumInfo(fmt, name, decimalValue, hexValue);
+            lineInfo += ("\n   ARB Comp Texture Format: "
+                         + decimalValue
+                         + " "
+                         + hexValue
+                         + " "
+                         + name);
+        }
+    }
+
+    std::vector<GLenum> compRgbEnums;
+    compRgbEnums.push_back(GL_COMPRESSED_RGB);
+    compRgbEnums.push_back(GL_COMPRESSED_RGBA);
+    if ( ! compRgbEnums.empty()) {
+        lineInfo += ("\nCompressed RGB(A) Formats:");
+        for (GLint fmt : compRgbEnums) {
+            QString name;
+            QString decimalValue;
+            QString hexValue;
+            GraphicsUtilitiesOpenGL::getTextCompressionEnumInfo(fmt, name, decimalValue, hexValue);
+            lineInfo += ("\n   ARB Comp Texture Format: "
+                         + decimalValue
+                         + " "
+                         + hexValue
+                         + " "
+                         + name);
+        }
+    }
+    
     lineInfo += "\n";
     lineInfo += "\n";
     lineInfo += "Note that State of OpenGL may be different when drawing objects.\n";
@@ -1462,4 +1409,16 @@ BrainOpenGL::testForOpenGLError(const AString& message,
     }
 }
 
+/**
+ * Allow tab highlighting (needs disabling when replacing scenes)
+ *
+ * @param flag
+ *     New status.
+ */
+void
+BrainOpenGL::setAllowTabHighlighting(const bool flag)
+{
+    s_allowTabHighlightingFlag = flag;
+}
+
 
