@@ -33,6 +33,8 @@
 #include "FiberOrientationColoringTypeEnum.h"
 #include "FiberOrientationSymbolTypeEnum.h"
 #include "FiberTrajectoryColorModel.h"
+#include "GraphicsRegionSelectionBox.h"
+#include "HistologySpaceKey.h"
 #include "ProjectionViewTypeEnum.h"
 #include "SelectionItemDataTypeEnum.h"
 #include "StructureEnum.h"
@@ -40,12 +42,11 @@
 #include "VolumeSliceViewPlaneEnum.h"
 #include "WholeBrainVoxelDrawingMode.h"
 
-
-class QGLWidget;
-class QImage;
-
 namespace caret {
     
+    class Annotation;
+    class AnnotationBrowserTab;
+    class AnnotationScaleBar;
     class AnnotationText;
     class BoundingBox;
     class Brain;
@@ -54,9 +55,7 @@ namespace caret {
     class BrainOpenGLShapeCube;
     class BrainOpenGLShapeCylinder;
     class BrainOpenGLShapeRing;
-    class BrainOpenGLShapeRingOutline;
     class BrainOpenGLShapeSphere;
-    class BrainOpenGLTextureManager;
     class BrainOpenGLViewportContent;
     class BrowserTabContent;
     class CaretMappableDataFile;
@@ -66,19 +65,23 @@ namespace caret {
     class FiberOrientation;
     class SelectionItem;
     class SelectionManager;
+    class GraphicsOrthographicProjection;
+    class HistologySlice;
     class IdentificationWithColor;
     class ImageFile;
-    class Plane;
-    class Surface;
     class Model;
     class ModelChart;
+    class ModelChartTwo;
+    class ModelHistology;
+    class ModelMedia;
     class ModelSurface;
     class ModelSurfaceMontage;
     class ModelVolume;
     class ModelWholeBrain;
     class Palette;
     class PaletteColorMapping;
-    class PaletteFile;
+    class Plane;
+    class Surface;
     class SurfaceFile;
     class SurfaceMontageConfigurationCerebellar;
     class SurfaceMontageConfigurationCerebral;
@@ -102,25 +105,10 @@ namespace caret {
         BrainOpenGLFixedPipeline& operator=(const BrainOpenGLFixedPipeline&);
         
     public:
-        BrainOpenGLFixedPipeline(const int32_t windowIndex,
-                                 BrainOpenGLTextRenderInterface* textRenderer);
+        BrainOpenGLFixedPipeline(BrainOpenGLTextRenderInterface* textRenderer);
 
         ~BrainOpenGLFixedPipeline();
         
-        void drawModels(Brain* brain,
-                        std::vector<BrainOpenGLViewportContent*>& viewportContents);
-        
-        void selectModel(Brain* brain,
-                         BrainOpenGLViewportContent* viewportContent,
-                         const int32_t mouseX,
-                         const int32_t mouseY,
-                         const bool applySelectionBackgroundFiltering);
-        
-        void projectToModel(Brain* brain,
-                            BrainOpenGLViewportContent* viewportContent,
-                            const int32_t mouseX,
-                            const int32_t mouseY,
-                            SurfaceProjectedItem& projectionOut);
         
         void initializeOpenGL();
         
@@ -133,8 +121,46 @@ namespace caret {
                                                  int32_t& subViewportSizeOut,
                                                  int32_t& gapOut);
         
-        virtual BrainOpenGLTextureManager* getTextureManager();
+        static void drawGraphicsRegionSelectionBox(const GraphicsRegionSelectionBox* graphicsRegionSelectionBox,
+                                                   const GraphicsRegionSelectionBox::DrawMode drawMode,
+                                                   const float rgba[4]);
         
+    protected:
+        void drawModelsImplementation(const int32_t windowIndex,
+                                      const UserInputModeEnum::Enum windowUserInputMode,
+                                      Brain* brain,
+                                      const std::vector<const BrainOpenGLViewportContent*>& viewportContents,
+                                      const GraphicsFramesPerSecond* graphicsFramesPerSecond) override;
+        
+        void selectModelImplementation(const int32_t windowIndex,
+                                       const UserInputModeEnum::Enum windowUserInputMode,
+                                       Brain* brain,
+                                       const BrainOpenGLViewportContent* viewportContent,
+                                       const int32_t mouseX,
+                                       const int32_t mouseY,
+                                       const bool applySelectionBackgroundFiltering) override;
+        
+        void projectToModelImplementation(const int32_t windowIndex,
+                                          const UserInputModeEnum::Enum windowUserInputMode,
+                                          Brain* brain,
+                                          const BrainOpenGLViewportContent* viewportContent,
+                                          const int32_t mouseX,
+                                          const int32_t mouseY,
+                                          SurfaceProjectedItem& projectionOut) override;
+        
+    protected:
+        virtual void loadObjectToWindowTransform(EventOpenGLObjectToWindowTransform* transformEvent) override;
+        
+        virtual void loadObjectToWindowTransform(GraphicsObjectToWindowTransform* transform,
+                                                 const GraphicsOrthographicProjection& orthographicProjection,
+                                                 const double centerToEyeDistance,
+                                                 const bool centerToEyeDistanceValidFlag) override;
+
+        virtual void loadObjectToWindowTransform(GraphicsObjectToWindowTransform* transform,
+                                                 const std::array<float, 4>& orthoLeftRightBottomTop,
+                                                 const double centerToEyeDistance,
+                                                 const bool centerToEyeDistanceValidFlag) override;
+
     private:
         class VolumeDrawInfo {
         public:
@@ -145,6 +171,7 @@ namespace caret {
                            const FastStatistics* statistics,
                            const WholeBrainVoxelDrawingMode::Enum wholeBrainVoxelDrawingMode,
                            const int32_t mapIndex,
+                           const int32_t tabIndex,
                            const float opacity);
             Brain* brain;
             CaretMappableDataFile* mapFile;
@@ -154,6 +181,7 @@ namespace caret {
             WholeBrainVoxelDrawingMode::Enum wholeBrainVoxelDrawingMode;
             const FastStatistics* statistics;
             int32_t mapIndex;
+            int32_t tabIndex;
             float opacity;
         };
 
@@ -165,12 +193,19 @@ namespace caret {
             float fanMultiplier;
             bool isDrawWithMagnitude;
             float minimumMagnitude;
+            float maximumUncertainty;
             float magnitudeMultiplier;
             Plane* plane;
             FiberOrientationSymbolTypeEnum::Enum symbolType;
             StructureEnum::Enum structure;
         };
         
+        enum class SurfaceTabType {
+            SINGLE_SURFACE,
+            SURFACE_MONTAGE,
+            WHOLE_BRAIN
+        };
+        
         void setFiberOrientationDisplayInfo(const DisplayPropertiesFiberOrientation* dpfo,
                                             const DisplayGroupEnum::Enum displayGroup,
                                             const int32_t tabIndex,
@@ -180,31 +215,44 @@ namespace caret {
                                             FiberOrientationDisplayInfo& dispInfo);
 
         void drawModelInternal(Mode mode,
-                               BrainOpenGLViewportContent* viewportContent);
+                               const BrainOpenGLViewportContent* viewportContent);
         
         void initializeMembersBrainOpenGL();
         
-        void drawChartData(BrowserTabContent* browserTabContent,
-                            ModelChart* chartData,
-                            const int32_t viewport[4]);
+        void drawChartOneData(BrowserTabContent* browserTabContent,
+                              ModelChart* chartData,
+                              const int32_t viewport[4]);
         
-        void drawSurfaceModel(ModelSurface* surfaceModel,
-                                   const int32_t viewport[4]);
+        void drawChartTwoData(const BrainOpenGLViewportContent* viewportContent,
+                              ModelChartTwo* chartData,
+                              const int32_t viewport[4]);
+        
+        void drawSurfaceModel(BrowserTabContent* browserTabContent,
+                              ModelSurface* surfaceModel,
+                              const int32_t viewport[4]);
         
         void drawSurface(Surface* surface,
+                         const SurfaceTabType surfaceTabType,
+                         const float surfaceScaling,
+                         const int32_t viewportHeight,
                          const float* nodeColoringRGBA,
-                         const bool drawAnnotationsInModelSpaceFlag);
+                         const bool drawAnnotationsInModelSpaceFlag,
+                         const bool depthTestingEnabled);
         
         void drawSurfaceNodes(Surface* surface,
                               const float* nodeColoringRGBA);
         
+        void drawSurfaceCutEdges(Surface* surface,
+                                 const float* nodeColoringRGBA);
+        
         void drawSurfaceTrianglesWithVertexArrays(const Surface* surface,
                                                   const float* nodeColoringRGBA);
         
         void drawSurfaceTriangles(Surface* surface,
                                   const float* nodeColoringRGBA);
         
-        void drawSurfaceNodeAttributes(Surface* surface);
+        void drawSurfaceNodeAttributes(Surface* surface,
+                                       const int32_t viewportHeight);
         
         void drawSurfaceBorderBeingDrawn(const Surface* surface);
         
@@ -213,6 +261,7 @@ namespace caret {
         struct BorderDrawInfo {
             Surface* anatomicalSurface;
             Surface* surface;
+            TopologyHelper* topologyHelper;
             Border* border;
             int32_t borderFileIndex;
             int32_t borderIndex;
@@ -253,7 +302,18 @@ namespace caret {
         void drawFiberTrajectories(const Plane* plane,
                                    const StructureEnum::Enum structure);
         
-        void drawVolumeModel(BrowserTabContent* browserTabContent,
+        void drawHistologyModel(const BrainOpenGLViewportContent* viewportContent,
+                                BrowserTabContent* browserTabContent,
+                                ModelHistology* mediaHistology,
+                                const int32_t viewport[4]);
+
+        void drawMediaModel(const BrainOpenGLViewportContent* viewportContent,
+                            BrowserTabContent* browserTabContent,
+                            ModelMedia* mediaModel,
+                            const int32_t viewport[4]);
+        
+        void drawVolumeModel(const BrainOpenGLViewportContent* viewportContent,
+                             BrowserTabContent* browserTabContent,
                                   ModelVolume* volumeModel,
                                   const int32_t viewport[4]);
         
@@ -266,6 +326,8 @@ namespace caret {
 
         void drawVolumeVoxelsAsCubesWholeBrain(std::vector<VolumeDrawInfo>& volumeDrawInfoIn);
         
+        void drawVolumeVoxelsAsCubesWholeBrainOutsideFaces(std::vector<VolumeDrawInfo>& volumeDrawInfoIn);
+        
         void drawVolumeOrthogonalSliceWholeBrain(const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                        const int64_t sliceIndex,
                                        std::vector<VolumeDrawInfo>& volumeDrawInfoIn);
@@ -299,9 +361,10 @@ namespace caret {
                                  Brain* brain,
                                  std::vector<VolumeDrawInfo>& volumeDrawInfoOut);
         
-        void drawWholeBrainModel(BrowserTabContent* browserTabContent,
-                                      ModelWholeBrain* wholeBrainModel,
-                                      const int32_t viewport[4]);
+        void drawWholeBrainModel(const BrainOpenGLViewportContent* viewportContent,
+                                 BrowserTabContent* browserTabContent,
+                                 ModelWholeBrain* wholeBrainModel,
+                                 const int32_t viewport[4]);
         
         void drawSurfaceMontageModel(BrowserTabContent* browserTabContent,
                                      ModelSurfaceMontage* surfaceMontageModel,
@@ -311,9 +374,20 @@ namespace caret {
         void setOrthographicProjection(const int32_t viewport[4],
                                        const ProjectionViewTypeEnum::Enum projectionType);
         
+        /**
+         * Fitting model into window with orthographic projection
+         */
+        enum class OrthoFitMode {
+            /** Fit to window with height (used by hippocampus flat maps) */
+            SET_FROM_WIDTH,
+            /** Fit to window with width */
+            SET_FROM_HEIGHT
+        };
+        
         void setOrthographicProjectionForWithBoundingBox(const int32_t viewport[4],
                                                          const ProjectionViewTypeEnum::Enum projectionType,
-                                                          const BoundingBox* boundingBox);
+                                                         const OrthoFitMode orthoFitMode,
+                                                         const BoundingBox* boundingBox);
         
         void setOrthographicProjectionWithHeight(const int32_t viewport[4],
                                                  const ProjectionViewTypeEnum::Enum projectionType,
@@ -334,6 +408,19 @@ namespace caret {
         
         void disableLineAntiAliasing();
         
+        void applyHistologyOrientationYoking();
+        
+        bool getPixelDepthAndRGBA(const int32_t windowX,
+                                  const int32_t windowY,
+                                  float& depthOut,
+                                  float rgbaOut[4]);
+        
+        void getIndexFromColorSelection(const int32_t x,
+                                        const int32_t y,
+                                        SelectionItemDataTypeEnum::Enum& dataTypeOut,
+                                        int32_t& indexOut,
+                                        float& depthOut);
+        
         void getIndexFromColorSelection(const SelectionItemDataTypeEnum::Enum dataType,
                                            const int32_t x,
                                            const int32_t y,
@@ -379,21 +466,6 @@ namespace caret {
         
         void drawSurfaceAxes();
         
-        void drawCircleOutline(const uint8_t rgba[4],
-                               const double diameter);
-        
-        void drawCircleFilled(const uint8_t rgba[4],
-                              const double diameter);
-        
-        void drawEllipseOutline(const uint8_t rgba[4],
-                                const double majorAxis,
-                                const double minorAxis,
-                                const double lineThickness);
-        
-        void drawEllipseFilled(const uint8_t rgba[4],
-                               const double majorAxis,
-                               const double minorAxis);
-        
         void drawSphereWithDiameter(const uint8_t rgba[4],
                                     const double diameter);
         
@@ -451,23 +523,35 @@ namespace caret {
         void drawTextAtModelCoords(const float modelXYZ[3],
                                    const AnnotationText& annotationText);
         
-        void drawWindowAnnotations(const int windowViewport[4]);
+        void drawWindowAnnotations(const int windowViewport[4],
+                                   const GraphicsFramesPerSecond* graphicsFramesPerSecond);
         
-//        void drawTabAnnotations(BrainOpenGLViewportContent* tabContent,
-//                                const int32_t tabViewport[4]);
-        void drawTabAnnotations(BrainOpenGLViewportContent* tabContent);
+        void drawSpacerAnnotations(const BrainOpenGLViewportContent* tabContent);
         
-        void drawAllPalettes(Brain* brain);
+        void drawTabAnnotations(const BrainOpenGLViewportContent* tabContent);
         
-        void drawPalette(const Palette* palette,
-                         const PaletteColorMapping* paletteColorMapping,
-                         const FastStatistics* statistics,
-                         const int paletteDrawingIndex);
+        void drawChartCoordinateSpaceAnnotations(const BrainOpenGLViewportContent* viewportContent);
         
-        void drawBackgroundImage(BrainOpenGLViewportContent* vpContent);
+        void drawHistologySpaceAnnotations(const BrainOpenGLViewportContent* viewportContent,
+                                           const HistologySpaceKey& histologySpaceKey,
+                                           const HistologySlice* histologySlice,
+                                           const float sliceSpacing);
         
-        void drawImage(const int viewport[4],
-                       const ImageFile* imageFile);
+        void drawMediaSpaceAnnotations(const BrainOpenGLViewportContent* viewportContent);
+        
+        void drawGraphicsTiming(const int windowViewport[4],
+                                const GraphicsFramesPerSecond* graphicsFramesPerSecond);
+
+        void drawBackgroundImage(const BrainOpenGLViewportContent* vpContent);
+        
+        void drawImage(const BrainOpenGLViewportContent* vpContent,
+                       ImageFile* imageFile,
+                       const float windowZ,
+                       const float frontZ,
+                       const float minimumThreshold,
+                       const float maximumThreshold,
+                       const float opacity,
+                       const bool drawControlPointsFlag);
         
         void setProjectionModeData(const float screenDepth,
                                           const float xyz[3],
@@ -476,10 +560,6 @@ namespace caret {
                                           const int barycentricNodes[3],
                                           const int numberOfNodes);
         
-        void setLineWidth(const float lineWidth);
-        
-        void setPointSize(const float pointSize);
-        
         enum ClippingDataType {
             CLIPPING_DATA_TYPE_FEATURES,
             CLIPPING_DATA_TYPE_SURFACE,
@@ -506,14 +586,50 @@ namespace caret {
                                           const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                           const int64_t sliceIndex) const;
         
-        void updateForegroundAndBackgroundColors(BrainOpenGLViewportContent* vpContent);
+        void updateForegroundAndBackgroundColors(const BrainOpenGLViewportContent* vpContent);
         
         void setTabViewport(const BrainOpenGLViewportContent* vpContent);
         
-        void setAnnotationColorBarsForDrawing(std::vector<BrainOpenGLViewportContent*>& viewportContents);
+        void setAnnotationColorBarsAndBrowserTabsForDrawing(const std::vector<const BrainOpenGLViewportContent*>& viewportContents,
+                                                            const bool selectionModeFlag);
+
+        void drawTabHighlighting(const float width,
+                                 const float height,
+                                 const float rgb[3]);
+        
+        void applyVolumePropertiesOpacity();
+        
+        void drawSolidBackgroundInAreasOutsideWindowAspectLocking(const int32_t windowBeforeAspectLockingViewport[4],
+                                                                  const int32_t windowAfterAspectLockingViewport[4]);
+        
+        void drawStippledBackgroundInAreasOutsideWindowAspectLocking(const int32_t windowBeforeAspectLockingViewport[4],
+                                                                     const int32_t windowAfterAspectLockingViewport[4]);
+        
+        void setupScaleBarDrawingInformation(BrowserTabContent* browserTabContent);
+        
+        void setupScaleBarDrawingInformation(BrowserTabContent* browserTabContent,
+                                             const float orthographicProjectionLeft,
+                                             const float orthographicProjectionRight);
+        
+        enum class BlendDataType {
+            CHART_TWO_MATRIX,
+            FEATURE_IMAGE, /* older image display selected in Features ToolBox*/
+            FIBER_TRAJECTORIES,
+            SEPARATE_BLENDING,
+            SURFACE_PROPERTIES_OPACITY,
+            VOLUME_ALL_VIEW_CUBES,
+            VOLUME_ALL_VIEW_SLICES,
+            VOLUME_MPR_SLICES,
+            VOLUME_ORTHOGONAL_SLICES
+        };
+        
+        static void setupBlending(const BlendDataType blendDataType);
         
         /** Index of window */
-        const int32_t m_windowIndex;
+        int32_t m_windowIndex = -1;
+        
+        /** User input mode for window */
+        UserInputModeEnum::Enum m_windowUserInputMode;
         
         /** Indicates OpenGL has been initialized */
         bool initializedOpenGLFlag;
@@ -580,15 +696,6 @@ namespace caret {
         /** Cube symbol */
         BrainOpenGLShapeCube* m_shapeCube;
         
-        /** Outline circle symbol */
-        BrainOpenGLShapeRing* m_shapeCircleOutline;
-        
-        /** Outline ellipse symbol.   KEY is line thickness */
-        std::map<float, BrainOpenGLShapeRingOutline*> m_shapeEllipseOutlines;
-        
-        /** Filled circle symbol */
-        BrainOpenGLShapeRing* m_shapeCircleFilled;
-        
         /** Rounded Cube symbol */
         BrainOpenGLShapeCube* m_shapeCubeRounded;
         
@@ -609,11 +716,15 @@ namespace caret {
         
         CaretPointer<BrainOpenGLAnnotationDrawingFixedPipeline> m_annotationDrawing;
         
+        std::vector<AnnotationBrowserTab*> m_annotationBrowserTabsForDrawing;
+        
         std::vector<AnnotationColorBar*> m_annotationColorBarsForDrawing;
         
-        /** The texture manager. */
-        CaretPointer<BrainOpenGLTextureManager> m_textureManager;
+        std::vector<AnnotationScaleBar*> m_annotationScaleBarsForDrawing;
         
+        /** Some graphics using annotations for some elements so user can select and edit them */
+        std::vector<Annotation*> m_specialCaseGraphicsAnnotations;
+
         static bool s_staticInitialized;
 
         static const float s_gluLookAtCenterFromEyeOffsetDistance;
@@ -624,9 +735,20 @@ namespace caret {
         
         friend class BrainOpenGLAnnotationDrawingFixedPipeline;
         friend class BrainOpenGLChartDrawingFixedPipeline;
+        friend class BrainOpenGLChartTwoDrawingFixedPipeline;
+        friend class BrainOpenGLFociDrawing;
+        friend class BrainOpenGLHistologySliceDrawing;
+        friend class BrainOpenGLHistologyStereotaxicSliceDrawing;
+        friend class BrainOpenGLIdentificationDrawing;
+        friend class BrainOpenGLMediaCoordinateDrawing;
+        friend class BrainOpenGLMediaDrawing;
+        friend class BrainOpenGLVolumeSurfaceClippedOutlineDrawing;
+        friend class BrainOpenGLVolumeMprThreeDrawing;
+        friend class BrainOpenGLVolumeMprTwoDrawing;
         friend class BrainOpenGLVolumeObliqueSliceDrawing;
         friend class BrainOpenGLVolumeSliceDrawing;
-        friend class OldBrainOpenGLVolumeSliceDrawing;
+        friend class BrainOpenGLVolumeSurfaceOutlineDrawing;
+        friend class BrainOpenGLVolumeTextureSliceDrawing;
     };
 
 #ifdef __BRAIN_OPENGL_FIXED_PIPELINE_DEFINE_H
