@@ -24,39 +24,79 @@
 #include "BrainOpenGLFixedPipeline.h"
 #include "CaretObject.h"
 #include "DisplayGroupEnum.h"
+#include "ModelTypeEnum.h"
 #include "VolumeSliceProjectionTypeEnum.h"
 #include "VolumeSliceDrawingTypeEnum.h"
+#include "VolumeSliceViewAllPlanesLayoutEnum.h"
 #include "VolumeSliceViewPlaneEnum.h"
 
 
 namespace caret {
-
+    
     class Brain;
+    class BrainOpenGLViewportContent;
     class BrowserTabContent;
     class CiftiMappableDataFile;
     class Matrix4x4;
     class ModelVolume;
     class ModelWholeBrain;
-    class PaletteFile;
     class Plane;
     class VolumeMappableInterface;
     
     class BrainOpenGLVolumeSliceDrawing : public CaretObject {
         
     public:
+        /**
+         * Indicates drawing volume sclice "ALL that shows
+         * axial, coronal, and parasagittal at same time
+         */
+        enum class AllSliceViewMode {
+            ALL_YES,
+            ALL_NO
+        };
+        
         BrainOpenGLVolumeSliceDrawing();
         
         virtual ~BrainOpenGLVolumeSliceDrawing();
         
         void draw(BrainOpenGLFixedPipeline* fixedPipelineDrawing,
+                  BrainOpenGLViewportContent* viewportContent,
                   BrowserTabContent* browserTabContent,
                   std::vector<BrainOpenGLFixedPipeline::VolumeDrawInfo>& volumeDrawInfo,
                   const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
                   const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
                   const int32_t viewport[4]);
-
+        
+        static void getOrthographicProjection(const AllSliceViewMode allSliceViewMode,
+                                              const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                              const BoundingBox& boundingBox,
+                                              const float zoomFactor,
+                                              const int viewport[4],
+                                              double orthographicBoundsOut[6]);
+        
+        static void setOrthographicProjection(const AllSliceViewMode allSliceViewMode,
+                                              const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                              const BoundingBox& boundingBox,
+                                              const float zoomFactor,
+                                              const int viewport[4],
+                                              double orthographicBoundsOut[6]);
+        
+        static void drawIdentificationSymbols(BrainOpenGLFixedPipeline* fixedPipelineDrawing,
+                                              BrowserTabContent* browserTabContent,
+                                              const VolumeMappableInterface* volume,
+                                              const int32_t mapIndex,
+                                              const Plane& plane,
+                                              const float sliceThickness);
+        
+        static void drawMontageSliceCoordinates(BrainOpenGLFixedPipeline* fixedPipelineDrawing,
+                                                const BrowserTabContent* browserTabContent,
+                                                const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                                const int32_t viewport[4],
+                                                const Vector3D sliceXYZ,
+                                                const float sliceOffset);
+        
         // ADD_NEW_METHODS_HERE
-
+        
     private:
         /**
          * Holds values in the slice for a volume so that they
@@ -168,8 +208,27 @@ namespace caret {
             std::vector<int64_t> m_sliceOffsets;
         };
         
-        BrainOpenGLVolumeSliceDrawing(const BrainOpenGLVolumeSliceDrawing&);
+        class GridInfo {
+        public:
+            GridInfo() : GridInfo(-1, -1, -1, -1) { }
+            
+            GridInfo(const int32_t numberOfRows,
+                     const int32_t numberOfColumns,
+                     const int32_t rowIndex,
+                     const int32_t columnIndex)
+            : m_numberOfRows(numberOfRows),
+            m_numberOfColumns(numberOfColumns),
+            m_rowIndex(rowIndex),
+            m_columnIndex(columnIndex) { }
+            
+            const int32_t m_numberOfRows;
+            const int32_t m_numberOfColumns;
+            const int32_t m_rowIndex;
+            const int32_t m_columnIndex;
+        };
 
+        BrainOpenGLVolumeSliceDrawing(const BrainOpenGLVolumeSliceDrawing&);
+        
         BrainOpenGLVolumeSliceDrawing& operator=(const BrainOpenGLVolumeSliceDrawing&);
         
         void drawVolumeSlicesForAllStructuresView(const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
@@ -178,32 +237,37 @@ namespace caret {
         void drawVolumeSliceViewPlane(const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
                                       const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
                                       const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                      const VolumeSliceViewAllPlanesLayoutEnum::Enum allPlanesLayout,
                                       const int32_t viewport[4]);
         
-        void drawVolumeSliceViewType(const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
-                           const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
-                           const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                           const int32_t viewport[4]);
+        void drawVolumeSliceViewType(const AllSliceViewMode allSliceViewMode,
+                                     const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
+                                     const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
+                                     const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                     const int32_t viewport[4]);
         
-        void drawVolumeSliceViewTypeMontage(const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
+        void drawVolumeSliceViewTypeMontage(const AllSliceViewMode allSliceViewMode,
+                                            const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
                                             const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
-                                  const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                                  const int32_t viewport[4]);
+                                            const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                            const int32_t viewport[4]);
         
-        void drawVolumeSliceViewProjection(const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
+        void drawVolumeSliceViewProjection(const AllSliceViewMode allSliceViewMode,
+                                           const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
                                            const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
-                                 const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                                 const float sliceCoordinates[3],
-                                 const int32_t viewport[4]);
-        
-        void drawObliqueSlice(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                              Matrix4x4& transformationMatrix,
-                              const Plane& plane);
+                                           const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                           const float sliceCoordinates[3],
+                                           const int32_t viewport[4],
+                                           const GridInfo& gridInfo);
         
         void drawOrthogonalSlice(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
                                  const float sliceCoordinates[3],
                                  const Plane& plane);
         
+        void drawOrthogonalSliceAllView(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                        const float sliceCoordinates[3],
+                                        const Plane& plane);
+        
         void drawOrthogonalSliceWithCulling(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
                                             const float sliceCoordinates[3],
                                             const Plane& plane);
@@ -213,12 +277,11 @@ namespace caret {
                                       const float sliceCoordinates[3],
                                       Plane& planeOut);
         
-        void drawAxesCrosshairsOrthoAndOblique(const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
-                                               const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
-                                               const float sliceCoordinates[3],
-                                               const bool drawCrosshairsFlag,
-                                               const bool drawCrosshairLabelsFlag);
-        
+        void drawAxesCrosshairsOrtho(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                     const float sliceCoordinates[3],
+                                     const bool drawCrosshairsFlag,
+                                     const bool drawCrosshairLabelsFlag);
+
         void setVolumeSliceViewingAndModelingTransformations(const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
                                                              const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
                                                              const Plane& plane);
@@ -232,9 +295,8 @@ namespace caret {
                         const Plane& slicePlane,
                         const float sliceCoordinates[3]);
         
-        void drawSurfaceOutline(const Plane& plane);
-        
-        void drawVolumeSliceFoci(const Plane& plane);
+        void drawVolumeSliceFoci(const Plane& plane,
+                                 const VolumeSliceViewPlaneEnum::Enum sliceViewPlane);
         
         void drawAxesCrosshairs(const VolumeSliceProjectionTypeEnum::Enum sliceProjectionType,
                                 const VolumeSliceDrawingTypeEnum::Enum sliceDrawingType,
@@ -247,9 +309,8 @@ namespace caret {
         
         void drawSquare(const float size);
         
-        void drawOrientationAxes(const int viewport[4]);
-        
-        void setOrthographicProjection(const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+        void setOrthographicProjection(const AllSliceViewMode allSliceViewMode,
+                                       const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
                                        const int viewport[4]);
         
         void drawOrthogonalSliceVoxels(const float sliceNormalVector[3],
@@ -263,19 +324,21 @@ namespace caret {
                                        const VolumeMappableInterface* volumeInterface,
                                        const int32_t volumeIndex,
                                        const int32_t mapIndex,
-                                       const uint8_t sliceOpacity);
-        
+                                       const uint8_t sliceOpacity,
+                                       const VolumeSliceViewPlaneEnum::Enum sliceViewPlane,
+                                       const int32_t sliceIndex);
+       
         void drawOrthogonalSliceVoxelsSingleQuads(const float sliceNormalVector[3],
-                                       const float coordinate[3],
-                                       const float rowStep[3],
-                                       const float columnStep[3],
-                                       const int64_t numberOfColumns,
-                                       const int64_t numberOfRows,
-                                       const std::vector<uint8_t>& sliceRGBA,
-                                       const VolumeMappableInterface* volumeInterface,
-                                       const int32_t volumeIndex,
-                                       const int32_t mapIndex,
-                                       const uint8_t sliceOpacity);
+                                                  const float coordinate[3],
+                                                  const float rowStep[3],
+                                                  const float columnStep[3],
+                                                  const int64_t numberOfColumns,
+                                                  const int64_t numberOfRows,
+                                                  const std::vector<uint8_t>& sliceRGBA,
+                                                  const VolumeMappableInterface* volumeInterface,
+                                                  const int32_t volumeIndex,
+                                                  const int32_t mapIndex,
+                                                  const uint8_t sliceOpacity);
         
         void drawOrthogonalSliceVoxelsQuadIndicesAndStrips(const float sliceNormalVector[3],
                                                            const float coordinate[3],
@@ -289,13 +352,10 @@ namespace caret {
                                                            const int32_t mapIndex,
                                                            const uint8_t sliceOpacity);
         
-        bool getVoxelCoordinateBoundsAndSpacing(float boundsOut[6],
-                                                float spacingOut[3]);
-        
-        void createObliqueTransformationMatrix(const float sliceCoordinates[3],
-                                               Matrix4x4& obliqueTransformationMatrixOut);
-        
-        void drawIdentificationSymbols(const Plane& plane);
+        void drawIdentificationSymbols(const VolumeMappableInterface* volume,
+                                       const int32_t mapIndex,
+                                       const Plane& plane,
+                                       const float sliceThickness);
         
         void addVoxelToIdentification(const int32_t volumeIndex,
                                       const int32_t mapIndex,
@@ -316,10 +376,13 @@ namespace caret {
                                                         const float sliceCoordinates[3],
                                                         const float sliceNormalVector[3]);
         
-        void processIdentification();
+        void processIdentification(const bool doNotReplaceUnderlayFlag,
+                                   const Plane& plane);
         
         void resetIdentification();
         
+        ModelTypeEnum::Enum m_modelType;
+        
         ModelVolume* m_modelVolume;
         
         ModelWholeBrain* m_modelWholeBrain;
@@ -334,9 +397,9 @@ namespace caret {
         
         std::vector<BrainOpenGLFixedPipeline::VolumeDrawInfo> m_volumeDrawInfo;
         
-        BrowserTabContent* m_browserTabContent;
+        BrainOpenGLViewportContent* m_viewportContent = NULL;
         
-        PaletteFile* m_paletteFile;
+        BrowserTabContent* m_browserTabContent;
         
         DisplayGroupEnum::Enum m_displayGroup;
         
@@ -360,6 +423,6 @@ namespace caret {
 #ifdef __BRAIN_OPEN_GL_VOLUME_SLICE_DRAWING_DECLARE__
     const int32_t BrainOpenGLVolumeSliceDrawing::IDENTIFICATION_INDICES_PER_VOXEL = 8;
 #endif // __BRAIN_OPEN_GL_VOLUME_SLICE_DRAWING_DECLARE__
-
+    
 } // namespace
 #endif  //__BRAIN_OPEN_GL_VOLUME_SLICE_DRAWING_H__
