@@ -45,6 +45,7 @@
 #include "SelectionManager.h"
 #include "LabelFile.h"
 #include "MathFunctions.h"
+#include "MetricDynamicConnectivityFile.h"
 #include "MetricFile.h"
 #include "ModelSurface.h"
 #include "OverlaySet.h"
@@ -178,6 +179,7 @@ BrainStructure::addLabelFile(LabelFile* labelFile,
             DataFileException e(labelFile->getFileName(),
                                 message);
             CaretLogThrowing(e);
+
             throw e;
         }
     }
@@ -191,7 +193,8 @@ BrainStructure::addLabelFile(LabelFile* labelFile,
         DataFileException e(labelFile->getFileName(),
                             message);
         CaretLogThrowing(e);
-        throw e;        
+
+        throw e;
     }
     
     if (addFileToBrainStructure) {
@@ -231,6 +234,7 @@ BrainStructure::addMetricFile(MetricFile* metricFile,
             DataFileException e(metricFile->getFileName(),
                                 message);
             CaretLogThrowing(e);
+            
             throw e;
         }
     }
@@ -244,11 +248,21 @@ BrainStructure::addMetricFile(MetricFile* metricFile,
         DataFileException e(metricFile->getFileName(),
                             message);
         CaretLogThrowing(e);
-        throw e;        
+                
+        throw e;
     }
     
     if (addFileToBrainStructure) {
         m_metricFiles.push_back(metricFile);
+
+        /*
+         * Enable dynamic connectivity using preferences
+         */
+        CaretPreferences* prefs = SessionManager::get()->getCaretPreferences();
+        MetricDynamicConnectivityFile* metricDynConn = metricFile->getMetricDynamicConnectivityFile();
+        if (metricDynConn != NULL) {
+            metricDynConn->setEnabledAsLayer(prefs->isDynamicConnectivityDefaultedOn());
+        }
     }
 }
 
@@ -284,6 +298,7 @@ BrainStructure::addRgbaFile(RgbaFile* rgbaFile,
             DataFileException e(rgbaFile->getFileName(),
                                 message);
             CaretLogThrowing(e);
+            
             throw e;
         }
     }
@@ -298,7 +313,8 @@ BrainStructure::addRgbaFile(RgbaFile* rgbaFile,
         DataFileException e(rgbaFile->getFileName(),
                             message);
         CaretLogThrowing(e);
-        throw e;        
+
+        throw e;
     }
     
     if (addFileToBrainStructure) {
@@ -339,12 +355,13 @@ BrainStructure::addSurface(Surface* surface,
             DataFileException e(surface->getFileName(),
                                 message);
             CaretLogThrowing(e);
+            
             throw e;
         }
     }
     
     if (surface->getStructure() != getStructure()) {
-        AString message = ("Trying to add metric file with structure \""
+        AString message = ("Trying to add surface file with structure \""
                            + StructureEnum::toGuiName(surface->getStructure())
                            + " to BrainStructure for \""
                            + StructureEnum::toGuiName(getStructure())
@@ -352,7 +369,8 @@ BrainStructure::addSurface(Surface* surface,
         DataFileException e(surface->getFileName(),
                             message);
         CaretLogThrowing(e);
-        throw e;        
+
+        throw e;
     }
     if (numNodes == 0) {
         const int32_t numSurfaceNodes = surface->getNumberOfNodes();
@@ -518,6 +536,33 @@ BrainStructure::getSurfacesOfType(const SurfaceTypeEnum::Enum surfaceType,
     }
 }
 
+/**
+ * @return Surface that matches both the surface type and secondary surface type.  If more than
+ * one surface matches, it will probably be whichever was loaded first.
+ * @param surfaceType
+ *    The surface type
+ * @param secondarySurfaceType
+ *    The secondary surface type
+ */
+Surface* 
+BrainStructure::getSurfaceOfTypeAndSecondaryType(const SurfaceTypeEnum::Enum surfaceType,
+                                                 const SecondarySurfaceTypeEnum::Enum secondarySurfaceType) const
+{
+    std::vector<Surface*> surfaces;
+    getSurfacesOfType(surfaceType,
+                      surfaces);
+    
+    Surface* surfaceOut(NULL);
+    for (Surface* s : surfaces) {
+        if (s->getSecondaryType() == secondarySurfaceType) {
+            surfaceOut = s;
+            break;
+        }
+    }
+    
+    return surfaceOut;
+}
+
 /**
  * Get all surfaces.
  *
@@ -578,6 +623,8 @@ BrainStructure::getPrimaryAnatomicalSurfacePrivate() const
         Surface* pialSurface         = NULL;
         Surface* anatomicalSurface   = NULL;
         Surface* fiducialSurface     = NULL;
+        Surface* innerSurface        = NULL;
+        Surface* outerSurface        = NULL;
         const int32_t numSurfaces = static_cast<int32_t>(primaryAnatomicalSurfaces.size());
         
         for (int32_t i = 0; i < numSurfaces; i++) {
@@ -588,20 +635,34 @@ BrainStructure::getPrimaryAnatomicalSurfacePrivate() const
             if (primaryAnatomicalSurfaces[i]->getSurfaceType() == SurfaceTypeEnum::ANATOMICAL) {
                 const SecondarySurfaceTypeEnum::Enum secondType = primaryAnatomicalSurfaces[i]->getSecondaryType();
                 const AString name = primaryAnatomicalSurfaces[i]->getFileNameNoPath().toLower();
-                if (secondType == SecondarySurfaceTypeEnum::MIDTHICKNESS) {
-                    if (midThicknessSurface == NULL) {
-                        midThicknessSurface = primaryAnatomicalSurfaces[i];
-                    }
-                }
-                if (secondType == SecondarySurfaceTypeEnum::GRAY_WHITE) {
-                    if (whiteMatterSurface == NULL) {
-                        whiteMatterSurface = primaryAnatomicalSurfaces[i];
-                    }
-                }
-                if (secondType == SecondarySurfaceTypeEnum::PIAL) {
-                    if (pialSurface == NULL) {
-                        pialSurface = primaryAnatomicalSurfaces[i];
-                    }
+                switch (secondType) {
+                    case SecondarySurfaceTypeEnum::INVALID:
+                        break;
+                    case SecondarySurfaceTypeEnum::MIDTHICKNESS:
+                        if (midThicknessSurface == NULL) {
+                            midThicknessSurface = primaryAnatomicalSurfaces[i];
+                        }
+                        break;
+                    case SecondarySurfaceTypeEnum::GRAY_WHITE:
+                        if (whiteMatterSurface == NULL) {
+                            whiteMatterSurface = primaryAnatomicalSurfaces[i];
+                        }
+                        break;
+                    case SecondarySurfaceTypeEnum::PIAL:
+                        if (pialSurface == NULL) {
+                            pialSurface = primaryAnatomicalSurfaces[i];
+                        }
+                        break;
+                    case SecondarySurfaceTypeEnum::INNER:
+                        if (innerSurface == NULL) {
+                            innerSurface = primaryAnatomicalSurfaces[i];
+                        }
+                        break;
+                    case SecondarySurfaceTypeEnum::OUTER:
+                        if (outerSurface == NULL) {
+                            outerSurface = primaryAnatomicalSurfaces[i];
+                        }
+                        break;
                 }
             }
         }
@@ -637,6 +698,16 @@ BrainStructure::getPrimaryAnatomicalSurfacePrivate() const
                     fiducialSurface = primaryAnatomicalSurfaces[i];
                 }
             }
+            if (name.indexOf("inner") >= 0) {
+                if (innerSurface == NULL) {
+                    innerSurface = primaryAnatomicalSurfaces[i];
+                }
+            }
+            if (name.indexOf("outer") >= 0) {
+                if (outerSurface == NULL) {
+                    outerSurface = primaryAnatomicalSurfaces[i];
+                }
+            }
         }
         
         if (midThicknessSurface != NULL) {
@@ -654,6 +725,12 @@ BrainStructure::getPrimaryAnatomicalSurfacePrivate() const
         else if (fiducialSurface != NULL) {
             m_primaryAnatomicalSurface = fiducialSurface;
         }
+        else if (innerSurface != NULL) {
+            m_primaryAnatomicalSurface = innerSurface;
+        }
+        else if (outerSurface != NULL) {
+            m_primaryAnatomicalSurface = outerSurface;
+        }
     }
     
     if (m_primaryAnatomicalSurface != NULL) {
@@ -1109,6 +1186,19 @@ BrainStructure::getNodeAttributes() const
     return m_nodeAttributes;
 }
 
+/**
+ * @return True if this brain structure contains any data files, else false.
+ */
+bool
+BrainStructure::hasDataFiles() const
+{
+    std::vector<CaretDataFile*> allDataFiles;
+    getAllDataFiles(allDataFiles);
+    
+    return ( ! allDataFiles.empty());
+}
+
+
 /**
  * Get all loaded data files.
  * @param allDataFilesOut
@@ -1123,9 +1213,15 @@ BrainStructure::getAllDataFiles(std::vector<CaretDataFile*>& allDataFilesOut) co
     allDataFilesOut.insert(allDataFilesOut.end(),
                            m_labelFiles.begin(),
                            m_labelFiles.end());
-    allDataFilesOut.insert(allDataFilesOut.end(),
-                           m_metricFiles.begin(),
-                           m_metricFiles.end());
+    for (auto mf : m_metricFiles) {
+        allDataFilesOut.push_back(mf);
+        MetricDynamicConnectivityFile* dynFile = mf->getMetricDynamicConnectivityFile();
+        if (dynFile != NULL) {
+            if (dynFile->isDataValid()) {
+                allDataFilesOut.push_back(dynFile);
+            }
+        }
+    }
     allDataFilesOut.insert(allDataFilesOut.end(),
                            m_rgbaFiles.begin(),
                            m_rgbaFiles.end());
@@ -1185,72 +1281,6 @@ BrainStructure::removeWithoutDeleteDataFile(const CaretDataFile* caretDataFile)
     return false;
 }
 
-
-///**
-// * Remove AND DELETE a data file from memory (does NOT delete file on disk.)
-// * Searches all of the loaded files for given file, and, when found
-// * deletes the file.
-// *
-// * @param caretDataFile
-// *    Data file to remove.  After calling this method and the file was
-// *    deleted (true was returned) this pointer is no longer valid.
-// * @return
-// *    true if file was removed, else false.
-// */
-//bool 
-//BrainStructure::removeAndDeleteDataFile(CaretDataFile* caretDataFile)
-//{
-//    if (removeWithoutDeleteDataFile(caretDataFile)) {
-//        delete caretDataFile;
-//        return true;
-//    }
-//    
-//    return false;
-
-//    std::vector<Surface*>::iterator surfaceIterator = 
-//        std::find(m_surfaces.begin(),
-//                  m_surfaces.end(),
-//                  caretDataFile);
-//    if (surfaceIterator != m_surfaces.end()) {
-//        Surface* s = *surfaceIterator;
-//        removeSurface(s);
-//        return true;
-//    }
-//    
-//    std::vector<LabelFile*>::iterator labelIterator = 
-//    std::find(m_labelFiles.begin(),
-//              m_labelFiles.end(),
-//              caretDataFile);
-//    if (labelIterator != m_labelFiles.end()) {
-//        delete caretDataFile;
-//        m_labelFiles.erase(labelIterator);
-//        return true;
-//    }
-//    
-//    std::vector<MetricFile*>::iterator metricIterator = 
-//    std::find(m_metricFiles.begin(),
-//              m_metricFiles.end(),
-//              caretDataFile);
-//    if (metricIterator != m_metricFiles.end()) {
-//        delete caretDataFile;
-//        m_metricFiles.erase(metricIterator);
-//        return true;
-//    }
-//    
-//    std::vector<RgbaFile*>::iterator rgbaIterator = 
-//    std::find(m_rgbaFiles.begin(),
-//              m_rgbaFiles.end(),
-//              caretDataFile);
-//    if (rgbaIterator != m_rgbaFiles.end()) {
-//        delete caretDataFile;
-//        m_rgbaFiles.erase(rgbaIterator);
-//        return true;
-//    }
-//    
-//    return false;
-
-//}
-
 /**
  * Find a map in a metric file that contains shape data.
  * It first looks for a shape NIFTI intent code.  If that
@@ -1513,3 +1543,24 @@ BrainStructure::restoreFromScene(const SceneAttributes* sceneAttributes,
     
 }
 
+/**
+ * Match surface sizes to the primary anatomical surface
+ *
+ * @param matchStatus
+ *     The match status
+ */
+void
+BrainStructure::matchSurfacesToPrimaryAnatomical(const bool matchStatus)
+{
+    const Surface* primaryAnatomical = getPrimaryAnatomicalSurface();
+    if (primaryAnatomical == NULL) {
+        return;
+    }
+    
+    for (auto s : m_surfaces) {
+        if (s != primaryAnatomical) {
+            s->matchToAnatomicalSurface(primaryAnatomical,
+                                        matchStatus);
+        }
+    }
+}
