@@ -23,7 +23,11 @@
 #include "DisplayPropertiesFiberOrientation.h"
 #undef __DISPLAY_PROPERTIES_FIBER_ORIENTATION_DECLARE__
 
+#include "Brain.h"
 #include "CaretAssert.h"
+#include "CiftiFiberOrientationFile.h"
+#include "DisplayPropertyDataBoolean.h"
+#include "DisplayPropertyDataFloat.h"
 #include "SceneAttributes.h"
 #include "SceneClass.h"
 #include "SceneClassAssistant.h"
@@ -47,10 +51,15 @@ using namespace caret;
 
 /**
  * Constructor.
+ * @param brain
+ *    The brain.
  */
-DisplayPropertiesFiberOrientation::DisplayPropertiesFiberOrientation()
-: DisplayProperties()
+DisplayPropertiesFiberOrientation::DisplayPropertiesFiberOrientation(const Brain* brain)
+: DisplayProperties(),
+m_brain(brain)
 {
+    CaretAssert(m_brain);
+    
     const float aboveLimit = 0.63;
     const float belowLimit = -0.63;
     const float minimumMagnitude = 0.05;
@@ -88,6 +97,16 @@ DisplayPropertiesFiberOrientation::DisplayPropertiesFiberOrientation()
         m_displaySphereOrientationsInDisplayGroup[i] = displaySphereOrientions;
     }
 
+    m_drawFiberTrajectoriesInFront.reset(new DisplayPropertyDataBoolean(false));
+    m_sceneAssistant->add("m_drawFiberTrajectoriesInFront",
+                          "DisplayPropertyDataBoolean",
+                          m_drawFiberTrajectoriesInFront.get());
+    
+    m_maximumUncertainty.reset(new DisplayPropertyDataFloat(s_defaultMaximumUncertainty));
+    m_sceneAssistant->add("m_maximumUncertainty",
+                          "DisplayPropertyDataFloat",
+                          m_maximumUncertainty.get());
+    
     m_sceneAssistant->addTabIndexedEnumeratedTypeArray<DisplayGroupEnum,DisplayGroupEnum::Enum>("m_displayGroup",
                                                                                                 m_displayGroup);
     m_sceneAssistant->addTabIndexedBooleanArray("m_displayStatusInTab",
@@ -145,10 +164,10 @@ DisplayPropertiesFiberOrientation::DisplayPropertiesFiberOrientation()
                                m_fanMultiplierInDisplayGroup,
                                DisplayGroupEnum::NUMBER_OF_GROUPS,
                                m_fanMultiplierInDisplayGroup[0]);
-    m_sceneAssistant->addArray("m_displaySphereOrientationsInTab",
-                               m_displaySphereOrientationsInTab,
+    m_sceneAssistant->addArray("m_displaySphereOrientationsInDisplayGroup",
+                               m_displaySphereOrientationsInDisplayGroup,
                                DisplayGroupEnum::NUMBER_OF_GROUPS,
-                               m_displaySphereOrientationsInTab[0]);
+                               m_displaySphereOrientationsInDisplayGroup[0]);
     
     m_sceneAssistant->addTabIndexedEnumeratedTypeArray<FiberOrientationColoringTypeEnum, FiberOrientationColoringTypeEnum::Enum>("m_fiberColoringTypeInTab",
                                                                                                            m_fiberColoringTypeInTab);
@@ -170,6 +189,8 @@ DisplayPropertiesFiberOrientation::~DisplayPropertiesFiberOrientation()
 void 
 DisplayPropertiesFiberOrientation::reset()
 {
+    m_maximumUncertainty->setAllValues(s_defaultMaximumUncertainty);
+    m_drawFiberTrajectoriesInFront->setAllValues(false);
 }
 
 /**
@@ -203,6 +224,8 @@ DisplayPropertiesFiberOrientation::copyDisplayProperties(const int32_t sourceTab
     m_lengthMultiplierInTab[targetTabIndex] = m_lengthMultiplierInTab[sourceTabIndex];
     m_fiberColoringTypeInTab[targetTabIndex] = m_fiberColoringTypeInTab[sourceTabIndex];
     m_fanMultiplierInTab[targetTabIndex] = m_fanMultiplierInTab[sourceTabIndex];
+    m_maximumUncertainty->copyValues(sourceTabIndex, targetTabIndex);
+    m_drawFiberTrajectoriesInFront->copyValues(sourceTabIndex, targetTabIndex);
 }
 
 /**
@@ -256,6 +279,40 @@ DisplayPropertiesFiberOrientation::setDisplayed(const DisplayGroupEnum::Enum  di
     }
 }
 
+/**
+ * @return  Draw fiber trajectories in front
+ * @param displayGroup
+ *    The display group.
+ * @param tabIndex
+ *    Index of browser tab.
+ */
+bool
+DisplayPropertiesFiberOrientation::isDrawFiberTrajectoriesInFront(const DisplayGroupEnum::Enum  displayGroup,
+                                               const int32_t tabIndex) const
+{
+    return m_drawFiberTrajectoriesInFront->getValue(displayGroup, 
+                                                    tabIndex);
+}
+
+/**
+ * Set draw fiber trajectories in front
+ * @param displayGroup
+ *    The display group.
+ * @param tabIndex
+ *    Index of browser tab.
+ * @param displayStatus
+ *    New status.
+ */
+void
+DisplayPropertiesFiberOrientation::setDrawFiberTrajectoriesInFront(const DisplayGroupEnum::Enum  displayGroup,
+                                                const int32_t tabIndex,
+                                                const bool displayStatus)
+{
+    m_drawFiberTrajectoriesInFront->setValue(displayGroup,
+                                             tabIndex,
+                                             displayStatus);
+}
+
 /**
  * @return  Display status of sphere orientations.
  * @param displayGroup
@@ -564,6 +621,40 @@ DisplayPropertiesFiberOrientation::setMinimumMagnitude(const DisplayGroupEnum::E
     }
 }
 
+/**
+ * @return The maximum uncertainty
+ * @param displayGroup
+ *    The display group.
+ * @param tabIndex
+ *    Index of browser tab.
+ */
+float
+DisplayPropertiesFiberOrientation::getMaximumUncertainty(const DisplayGroupEnum::Enum displayGroup,
+                                                         const int32_t tabIndex) const
+{
+    return m_maximumUncertainty->getValue(displayGroup,
+                                          tabIndex);
+}
+
+/**
+ * Set the maximum uncertainty to the given value.
+ * @param displayGroup
+ *    The display group.
+ * @param tabIndex
+ *    Index of browser tab.
+ * @param maximumUncertainty
+ *     New value for maximum uncertainty.
+ */
+void
+DisplayPropertiesFiberOrientation::setMaximumUncertainty(const DisplayGroupEnum::Enum displayGroup,
+                                                         const int32_t tabIndex,
+                                                         const float maximumUncertainty)
+{
+    m_maximumUncertainty->setValue(displayGroup,
+                                   tabIndex,
+                                   maximumUncertainty);
+}
+
 /**
  * @return The length multiplier.
  * @param displayGroup
@@ -773,7 +864,9 @@ DisplayPropertiesFiberOrientation::restoreFromScene(const SceneAttributes* scene
             break;
         case SceneTypeEnum::SCENE_TYPE_GENERIC:
             break;
-    }    
+    }
+    
+    setMaximumUncertaintyFromFiles();
 }
 
 /**
@@ -827,3 +920,27 @@ DisplayPropertiesFiberOrientation::setFanMultiplier(const DisplayGroupEnum::Enum
     }
 }
 
+/**
+ * Set the fiber's maximum uncertainty to maximum found in all fiber orientation files
+ */
+void
+DisplayPropertiesFiberOrientation::setMaximumUncertaintyFromFiles()
+{
+    float maxUncert(s_defaultMaximumUncertainty);
+    
+    std::vector<CiftiFiberOrientationFile*> fiberFiles;
+    m_brain->getConnectivityFiberOrientationFiles(fiberFiles);
+    
+    if ( ! fiberFiles.empty()) {
+        maxUncert = 0.0;
+        for (const auto& ff : fiberFiles) {
+            const float u = ff->getMaximumVariance();
+            if (u > maxUncert) {
+                maxUncert = u;
+            }
+        }
+    }
+    
+    m_maximumUncertainty->setAllValues(maxUncert);
+}
+
