@@ -40,6 +40,8 @@
 #include "SurfaceMontageConfigurationCerebellar.h"
 #include "SurfaceMontageConfigurationCerebral.h"
 #include "SurfaceMontageConfigurationFlatMaps.h"
+#include "SurfaceMontageConfigurationHippocampus.h"
+#include "SurfaceMontageConfigurationHippocampusFlatMaps.h"
 #include "SurfaceSelectionModel.h"
 
 using namespace caret;
@@ -65,6 +67,8 @@ ModelSurfaceMontage::ModelSurfaceMontage(Brain* brain)
         m_cerebellarConfiguration[i] = new SurfaceMontageConfigurationCerebellar(i);
         m_cerebralConfiguration[i] = new SurfaceMontageConfigurationCerebral(i);
         m_flatMapsConfiguration[i] = new SurfaceMontageConfigurationFlatMaps(i);
+        m_hippocampusConfiguration[i] = new SurfaceMontageConfigurationHippocampus(i);
+        m_hippocampusFlatMapsConfiguration[i] = new SurfaceMontageConfigurationHippocampusFlatMaps(i);
     }
     
     std::vector<StructureEnum::Enum> overlaySurfaceStructures;
@@ -83,6 +87,8 @@ ModelSurfaceMontage::~ModelSurfaceMontage()
         delete m_cerebellarConfiguration[i];
         delete m_cerebralConfiguration[i];
         delete m_flatMapsConfiguration[i];
+        delete m_hippocampusConfiguration[i];
+        delete m_hippocampusFlatMapsConfiguration[i];
     }
 }
 
@@ -107,6 +113,8 @@ ModelSurfaceMontage::initializeSelectedSurfaces()
         m_cerebellarConfiguration[i]->initializeSelectedSurfaces();
         m_cerebralConfiguration[i]->initializeSelectedSurfaces();
         m_flatMapsConfiguration[i]->initializeSelectedSurfaces();
+        m_hippocampusConfiguration[i]->initializeSelectedSurfaces();
+        m_hippocampusFlatMapsConfiguration[i]->initializeSelectedSurfaces();
     }
 }
 
@@ -174,6 +182,8 @@ ModelSurfaceMontage::initializeOverlays()
         m_cerebellarConfiguration[i]->getOverlaySet()->initializeOverlays();
         m_cerebralConfiguration[i]->getOverlaySet()->initializeOverlays();
         m_flatMapsConfiguration[i]->getOverlaySet()->initializeOverlays();
+        m_hippocampusConfiguration[i]->getOverlaySet()->initializeOverlays();
+        m_hippocampusFlatMapsConfiguration[i]->getOverlaySet()->initializeOverlays();
     }
 }
 
@@ -229,6 +239,7 @@ ModelSurfaceMontage::getSelectedSurface(const StructureEnum::Enum structure,
             switch (structure) {
                 case StructureEnum::CEREBELLUM:
                     selectionModels.push_back(smcfm->getCerebellumSurfaceSelectionModel());
+                    break;
                 case StructureEnum::CORTEX_LEFT:
                     selectionModels.push_back(smcfm->getLeftSurfaceSelectionModel());
                     break;
@@ -240,6 +251,40 @@ ModelSurfaceMontage::getSelectedSurface(const StructureEnum::Enum structure,
             }
         }
             break;
+        case SurfaceMontageConfigurationTypeEnum::HIPPOCAMPUS_CONFIGURATION:
+        {
+            SurfaceMontageConfigurationHippocampus* smcc = getHippocampusConfiguration(windowTabNumber);
+            switch (structure) {
+                case StructureEnum::CORTEX_LEFT:
+                    selectionModels.push_back(smcc->getLeftFirstSurfaceSelectionModel());
+                    selectionModels.push_back(smcc->getLeftSecondSurfaceSelectionModel());
+                    break;
+                case StructureEnum::CORTEX_RIGHT:
+                    selectionModels.push_back(smcc->getRightFirstSurfaceSelectionModel());
+                    selectionModels.push_back(smcc->getRightSecondSurfaceSelectionModel());
+                    break;
+                default:
+                    break;
+            }
+        }
+            break;
+        case SurfaceMontageConfigurationTypeEnum::HIPPOCAMPUS_FLAT_CONFIGURATION:
+        {
+            SurfaceMontageConfigurationHippocampusFlatMaps* smhfc = getHippocampusFlatMapsConfiguration(windowTabNumber);
+            switch (structure) {
+                case StructureEnum::CORTEX_LEFT:
+                    selectionModels.push_back(smhfc->getLeftFirstSurfaceSelectionModel());
+                    selectionModels.push_back(smhfc->getLeftSecondSurfaceSelectionModel());
+                    break;
+                case StructureEnum::CORTEX_RIGHT:
+                    selectionModels.push_back(smhfc->getRightFirstSurfaceSelectionModel());
+                    selectionModels.push_back(smhfc->getRightSecondSurfaceSelectionModel());
+                    break;
+                default:
+                    break;
+            }
+        }
+            break;
     }
     
     Surface* surfaceOut = NULL;
@@ -285,6 +330,18 @@ ModelSurfaceMontage::getSelectedConfiguration(const int32_t tabIndex)
                                   tabIndex);
             return m_flatMapsConfiguration[tabIndex];
             break;
+        case SurfaceMontageConfigurationTypeEnum::HIPPOCAMPUS_CONFIGURATION:
+            CaretAssertArrayIndex(m_hippocampusConfiguration,
+                                  BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                                  tabIndex);
+            return m_hippocampusConfiguration[tabIndex];
+            break;
+        case SurfaceMontageConfigurationTypeEnum::HIPPOCAMPUS_FLAT_CONFIGURATION:
+            CaretAssertArrayIndex(m_hippocampusFlatMapsConfiguration,
+                                  BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                                  tabIndex);
+            return m_hippocampusFlatMapsConfiguration[tabIndex];
+            break;
     }
 
     return NULL;
@@ -326,6 +383,12 @@ ModelSurfaceMontage::getSelectedConfigurationType(const int32_t tabIndex) const
     if (m_flatMapsConfiguration[tabIndex]->isValid()) {
         validTypes.push_back(SurfaceMontageConfigurationTypeEnum::FLAT_CONFIGURATION);
     }
+    if (m_hippocampusConfiguration[tabIndex]->isValid()) {
+        validTypes.push_back(SurfaceMontageConfigurationTypeEnum::HIPPOCAMPUS_CONFIGURATION);
+    }
+    if (m_hippocampusFlatMapsConfiguration[tabIndex]->isValid()) {
+        validTypes.push_back(SurfaceMontageConfigurationTypeEnum::HIPPOCAMPUS_FLAT_CONFIGURATION);
+    }
 
     /*
      * Verify selected type is valid
@@ -356,6 +419,22 @@ ModelSurfaceMontage::getSelectedConfigurationType(const int32_t tabIndex) const
                 validTypeSelected = true;
             }
             break;
+        case SurfaceMontageConfigurationTypeEnum::HIPPOCAMPUS_CONFIGURATION:
+            CaretAssertArrayIndex(m_hippocampusConfiguration,
+                                  BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                                  tabIndex);
+            if (m_hippocampusConfiguration[tabIndex]->isValid()) {
+                validTypeSelected = true;
+            }
+            break;
+        case SurfaceMontageConfigurationTypeEnum::HIPPOCAMPUS_FLAT_CONFIGURATION:
+            CaretAssertArrayIndex(m_hippocampusFlatMapsConfiguration,
+                                  BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                                  tabIndex);
+            if (m_hippocampusFlatMapsConfiguration[tabIndex]->isValid()) {
+                validTypeSelected = true;
+            }
+            break;
     }
     
     /*
@@ -488,6 +567,73 @@ ModelSurfaceMontage::getFlatMapsConfiguration(const int32_t tabIndex) const
     return m_flatMapsConfiguration[tabIndex];
 }
 
+/**
+ * Get the hippocampus configuration in the given tab.
+ *
+ * @param tabIndex
+ *    Index of tab.
+ * @return
+ *    hippocampus configuration.
+ */
+SurfaceMontageConfigurationHippocampus* 
+ModelSurfaceMontage::getHippocampusConfiguration(const int32_t tabIndex)
+{
+    CaretAssertArrayIndex(m_hippocampusConfiguration,
+                          BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                          tabIndex);
+    return m_hippocampusConfiguration[tabIndex];
+}
+
+/**
+ * Get the hippocampus configuration in the given tab.
+ *
+ * @param tabIndex
+ *    Index of tab.
+ * @return
+ *    hippocampus configuration.
+ */
+const SurfaceMontageConfigurationHippocampus*
+ModelSurfaceMontage::getHippocampusConfiguration(const int32_t tabIndex) const
+{
+    CaretAssertArrayIndex(m_hippocampusConfiguration,
+                          BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                          tabIndex);
+    return m_hippocampusConfiguration[tabIndex];
+}
+
+/**
+ * Get the hippocampus flat configuration in the given tab.
+ *
+ * @param tabIndex
+ *    Index of tab.
+ * @return
+ *    hippocampus flat configuration.
+ */
+SurfaceMontageConfigurationHippocampusFlatMaps*
+ModelSurfaceMontage::getHippocampusFlatMapsConfiguration(const int32_t tabIndex)
+{
+    CaretAssertArrayIndex(m_hippocampusFlatMapsConfiguration,
+                          BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                          tabIndex);
+    return m_hippocampusFlatMapsConfiguration[tabIndex];
+}
+
+/**
+ * Get the hippocampus flat configuration in the given tab.
+ *
+ * @param tabIndex
+ *    Index of tab.
+ * @return
+ *    hippocampus flat configuration.
+ */
+const SurfaceMontageConfigurationHippocampusFlatMaps*
+ModelSurfaceMontage::getHippocampusFlatMapsConfiguration(const int32_t tabIndex) const
+{
+    CaretAssertArrayIndex(m_hippocampusFlatMapsConfiguration,
+                          BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                          tabIndex);
+    return m_hippocampusFlatMapsConfiguration[tabIndex];
+}
 
 /**
  * Save information specific to this type of model to the scene.
@@ -512,7 +658,11 @@ ModelSurfaceMontage::saveModelSpecificInformationToScene(const SceneAttributes*
                                                                                   SceneObjectDataTypeEnum::SCENE_CLASS);
     SceneObjectMapIntegerKey* flatConfigurationMap = new SceneObjectMapIntegerKey("m_flatMapsConfiguration",
                                                                              SceneObjectDataTypeEnum::SCENE_CLASS);
-    
+    SceneObjectMapIntegerKey* hippoConfigurationMap = new SceneObjectMapIntegerKey("m_hippocampusConfiguration",
+                                                                                  SceneObjectDataTypeEnum::SCENE_CLASS);
+    SceneObjectMapIntegerKey* hippoFlatConfigurationMap = new SceneObjectMapIntegerKey("m_hippocampusFlatMapsConfiguration",
+                                                                                   SceneObjectDataTypeEnum::SCENE_CLASS);
+
 
     /*
      * Get indices of tabs that are to be saved to scene.
@@ -538,72 +688,19 @@ ModelSurfaceMontage::saveModelSpecificInformationToScene(const SceneAttributes*
         flatConfigurationMap->addClass(tabIndex,
                                            m_flatMapsConfiguration[tabIndex]->saveToScene(sceneAttributes,
                                                                                           "m_flatMapsConfiguration" + tabString));
+        hippoConfigurationMap->addClass(tabIndex,
+                                       m_hippocampusConfiguration[tabIndex]->saveToScene(sceneAttributes,
+                                                                                      "m_hippocampusConfiguration" + tabString));
+        hippoFlatConfigurationMap->addClass(tabIndex,
+                                        m_hippocampusFlatMapsConfiguration[tabIndex]->saveToScene(sceneAttributes,
+                                                                                          "m_hippocampusFlatMapsConfiguration" + tabString));
     }
     
     sceneClass->addChild(cerebellarConfigurationMap);
     sceneClass->addChild(cerebralConfigurationMap);
     sceneClass->addChild(flatConfigurationMap);
-    
-    
-//    /*
-//     * Surfaces
-//     */
-//    SceneObjectMapIntegerKey* leftSurfaceMap = new SceneObjectMapIntegerKey("m_leftSurfaceSelectionModel",
-//                                                                  SceneObjectDataTypeEnum::SCENE_CLASS);
-//    SceneObjectMapIntegerKey* leftSecondSurfaceMap = new SceneObjectMapIntegerKey("m_leftSecondSurfaceSelectionModel",
-//                                                                SceneObjectDataTypeEnum::SCENE_CLASS);
-//    SceneObjectMapIntegerKey* rightSurfaceMap = new SceneObjectMapIntegerKey("m_rightSurfaceSelectionModel",
-//                                                                SceneObjectDataTypeEnum::SCENE_CLASS);
-//    SceneObjectMapIntegerKey* rightSecondSurfaceMap = new SceneObjectMapIntegerKey("m_rightSecondSurfaceSelectionModel",
-//                                                                SceneObjectDataTypeEnum::SCENE_CLASS);
-//    for (int32_t iat = 0; iat < numActiveTabs; iat++) {
-//        const int32_t tabIndex = tabIndices[iat];
-//        const AString tabString = ("[" + AString::number(tabIndex) + "]");
-//
-//        leftSurfaceMap->addClass(tabIndex,
-//                                 m_leftSurfaceSelectionModel[tabIndex]->saveToScene(sceneAttributes, 
-//                                                                                    "m_leftSurfaceSelectionModel" + tabString));
-//        leftSecondSurfaceMap->addClass(tabIndex,
-//                                 m_leftSecondSurfaceSelectionModel[tabIndex]->saveToScene(sceneAttributes, 
-//                                                                                    "m_leftSecondSurfaceSelectionModel" + tabString));
-//        rightSurfaceMap->addClass(tabIndex,
-//                                 m_rightSurfaceSelectionModel[tabIndex]->saveToScene(sceneAttributes, 
-//                                                                                    "m_rightSurfaceSelectionModel" + tabString));
-//        rightSecondSurfaceMap->addClass(tabIndex,
-//                                 m_rightSecondSurfaceSelectionModel[tabIndex]->saveToScene(sceneAttributes, 
-//                                                                                    "m_rightSecondSurfaceSelectionModel" + tabString));
-//    }
-//    sceneClass->addChild(leftSurfaceMap);
-//    sceneClass->addChild(leftSecondSurfaceMap);
-//    sceneClass->addChild(rightSurfaceMap);
-//    sceneClass->addChild(rightSecondSurfaceMap);
-//    
-//    /*
-//     * Selections
-//     */
-//    SceneObjectMapIntegerKey* rightEnabledMap = new SceneObjectMapIntegerKey("m_rightEnabled",
-//                                                                       SceneObjectDataTypeEnum::SCENE_BOOLEAN);
-//    SceneObjectMapIntegerKey* leftEnabledMap = new SceneObjectMapIntegerKey("m_leftEnabled",
-//                                                                                  SceneObjectDataTypeEnum::SCENE_BOOLEAN);
-//    SceneObjectMapIntegerKey* firstSurfaceEnabledMap = new SceneObjectMapIntegerKey("m_firstSurfaceEnabled",
-//                                                                                  SceneObjectDataTypeEnum::SCENE_BOOLEAN);
-//    SceneObjectMapIntegerKey* secondSurfaceEnabledMap = new SceneObjectMapIntegerKey("m_secondSurfaceEnabled",
-//                                                                                  SceneObjectDataTypeEnum::SCENE_BOOLEAN);
-//    for (int32_t iat = 0; iat < numActiveTabs; iat++) {
-//        const int32_t tabIndex = tabIndices[iat];
-//        rightEnabledMap->addBoolean(tabIndex,
-//                                         m_rightEnabled[tabIndex]);
-//        leftEnabledMap->addBoolean(tabIndex,
-//                                         m_leftEnabled[tabIndex]);
-//        firstSurfaceEnabledMap->addBoolean(tabIndex,
-//                                         m_firstSurfaceEnabled[tabIndex]);
-//        secondSurfaceEnabledMap->addBoolean(tabIndex,
-//                                         m_secondSurfaceEnabled[tabIndex]);
-//    }
-//    sceneClass->addChild(rightEnabledMap);
-//    sceneClass->addChild(leftEnabledMap);
-//    sceneClass->addChild(firstSurfaceEnabledMap);
-//    sceneClass->addChild(secondSurfaceEnabledMap);
+    sceneClass->addChild(hippoConfigurationMap);
+    sceneClass->addChild(hippoFlatConfigurationMap);
 }
 
 /**
@@ -647,7 +744,7 @@ ModelSurfaceMontage::restoreModelSpecificInformationFromScene(const SceneAttribu
         }
 
         /*
-         * Restore Cortext
+         * Restore Cortex
          */
         const SceneObjectMapIntegerKey* cerebralMap = sceneClass->getMapIntegerKey("m_cerebralConfiguration");
         if (cerebellumMap != NULL) {
@@ -689,6 +786,48 @@ ModelSurfaceMontage::restoreModelSpecificInformationFromScene(const SceneAttribu
             }
         }
         
+        /*
+         * Restore Hippocampus
+         */
+        const SceneObjectMapIntegerKey* hippoMap = sceneClass->getMapIntegerKey("m_hippocampusConfiguration");
+        if (hippoMap != NULL) {
+            const std::map<int32_t, SceneObject*>& structureMap = hippoMap->getMap();
+            for (std::map<int32_t, SceneObject*>::const_iterator iter = structureMap.begin();
+                 iter != structureMap.end();
+                 iter++) {
+                const int32_t key = iter->first;
+                const SceneClass* hippoSceneClass = dynamic_cast<const SceneClass*>(iter->second);
+                hippoSceneClass->setRestored(true);// prevents "failed to restore"
+                CaretAssertArrayIndex(m_hippocampusConfiguration,
+                                      BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                                      key);
+                
+                m_hippocampusConfiguration[key]->restoreFromScene(sceneAttributes,
+                                                                  hippoSceneClass);
+            }
+        }
+
+        /*
+         * Restore Hippocampus Flat Maps
+         */
+        const SceneObjectMapIntegerKey* hippoFlatMap = sceneClass->getMapIntegerKey("m_hippocampusFlatMapsConfiguration");
+        if (hippoFlatMap != NULL) {
+            const std::map<int32_t, SceneObject*>& structureMap = hippoFlatMap->getMap();
+            for (std::map<int32_t, SceneObject*>::const_iterator iter = structureMap.begin();
+                 iter != structureMap.end();
+                 iter++) {
+                const int32_t key = iter->first;
+                const SceneClass* hippoFlatSceneClass = dynamic_cast<const SceneClass*>(iter->second);
+                hippoFlatSceneClass->setRestored(true);// prevents "failed to restore"
+                CaretAssertArrayIndex(m_hippocampusFlatMapsConfiguration,
+                                      BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
+                                      key);
+                
+                m_hippocampusFlatMapsConfiguration[key]->restoreFromScene(sceneAttributes,
+                                                                          hippoFlatSceneClass);
+            }
+        }
+        
         sceneClass->getEnumerateTypeArrayForTabIndices<SurfaceMontageConfigurationTypeEnum,SurfaceMontageConfigurationTypeEnum::Enum>("m_selectedConfigurationType",
                                                                                                                                       m_selectedConfigurationType);
     }
@@ -1051,6 +1190,8 @@ ModelSurfaceMontage::copyTabContent(const int32_t sourceTabIndex,
     m_cerebellarConfiguration[destinationTabIndex]->copyConfiguration(m_cerebellarConfiguration[sourceTabIndex]);
     m_cerebralConfiguration[destinationTabIndex]->copyConfiguration(m_cerebralConfiguration[sourceTabIndex]);
     m_flatMapsConfiguration[destinationTabIndex]->copyConfiguration(m_flatMapsConfiguration[sourceTabIndex]);
+    m_hippocampusConfiguration[destinationTabIndex]->copyConfiguration(m_hippocampusConfiguration[sourceTabIndex]);
+    m_hippocampusFlatMapsConfiguration[destinationTabIndex]->copyConfiguration(m_hippocampusFlatMapsConfiguration[sourceTabIndex]);
     m_selectedConfigurationType[destinationTabIndex] = m_selectedConfigurationType[sourceTabIndex];
 }
 
