@@ -22,8 +22,6 @@
 #include <algorithm>
 #include <deque>
 
-#include <QRegExp>
-
 #define __OVERLAY_SET_DECLARE__
 #include "OverlaySet.h"
 #undef __OVERLAY_SET_DECLARE__
@@ -44,6 +42,7 @@
 #include "ModelVolume.h"
 #include "ModelWholeBrain.h"
 #include "Overlay.h"
+#include "OverlaySetInitializer.h"
 #include "PlainTextStringBuilder.h"
 #include "Scene.h"
 #include "SceneClass.h"
@@ -173,31 +172,102 @@ OverlaySet::getUnderlay()
  * @return Returns the bottom-most overlay that is set a a volume file.
  * Will return NULL if no, enabled overlays are set to a volume file.
  */
-VolumeMappableInterface*
-OverlaySet::getUnderlayVolume()
+Overlay*
+OverlaySet::getUnderlayContainingVolume()
 {
-    VolumeMappableInterface* vf = NULL;
+    Overlay* underlayOut(NULL);
     
-    for (int32_t i = (getNumberOfDisplayedOverlays() - 1); i >= 0; i--) {
+    const int32_t numOverlays = getNumberOfDisplayedOverlays();
+    for (int32_t i = (numOverlays - 1); i >= 0; i--) {
         if (m_overlays[i]->isEnabled()) {
             CaretMappableDataFile* mapFile = NULL;
             int32_t mapIndex;
+            CaretAssertArrayIndex(m_overlays, BrainConstants::MAXIMUM_NUMBER_OF_OVERLAYS, i);
             m_overlays[i]->getSelectionData(mapFile,
                                             mapIndex);
             
             if (mapFile != NULL) {
                 if (mapFile->isVolumeMappable()) {
-                    vf = dynamic_cast<VolumeMappableInterface*>(mapFile);
+                    const VolumeMappableInterface* vf = dynamic_cast<VolumeMappableInterface*>(mapFile);
                     if (vf != NULL) {
+                        underlayOut = m_overlays[i];
                         break;
                     }
                 }
             }
         }
     }
+    
+    if (underlayOut == NULL) {
+        /*
+         * If we are here, either there are no volume files or
+         * no overlays are enabled containing a volume file.
+         * So, find the lowest layer than contains a volume file,
+         * even if the layer is disabled.
+         */
+        for (int32_t i = 0; i < numOverlays; i++) {
+            CaretMappableDataFile* mapFile = NULL;
+            int32_t mapIndex;
+            CaretAssertArrayIndex(m_overlays, BrainConstants::MAXIMUM_NUMBER_OF_OVERLAYS, i);
+            m_overlays[i]->getSelectionData(mapFile,
+                                            mapIndex);
+            
+            if (mapFile != NULL) {
+                if (mapFile->isVolumeMappable()) {
+                    const VolumeMappableInterface* vf = dynamic_cast<VolumeMappableInterface*>(mapFile);
+                    if (vf != NULL) {
+                        underlayOut = m_overlays[i];
+                        break;
+                    }
+                }
+            }
+        }
+    }
+    
+    return underlayOut;
+}
+
+/*
+ * Get the bottom-most overlay that is a volume file for the given
+ * browser tab and return its volume file.
+ * @param browserTabContent
+ *    Content of browser tab.
+ * @return Returns the bottom-most overlay that is set a a volume file.
+ * Will return NULL if no, enabled overlays are set to a volume file.
+ */
+VolumeMappableInterface*
+OverlaySet::getUnderlayVolume()
+{
+    VolumeMappableInterface* vf = NULL;
+ 
+    Overlay* underlay = getUnderlayContainingVolume();
+    if (underlay != NULL) {
+        CaretMappableDataFile* mapFile = NULL;
+        int32_t mapIndex;
+        underlay->getSelectionData(mapFile, mapIndex);
+        if (mapFile != NULL) {
+            vf = dynamic_cast<VolumeMappableInterface*>(mapFile);
+        }
+    }
+    
     return vf;
 }
 
+/*
+ * Get the bottom-most overlay that is a volume file for the given
+ * browser tab and return its volume file.
+ * @param browserTabContent
+ *    Content of browser tab.
+ * @return Returns the bottom-most overlay that is set a a volume file.
+ * Will return NULL if no, enabled overlays are set to a volume file.
+ */
+const VolumeMappableInterface*
+OverlaySet::getUnderlayVolume() const
+{
+    OverlaySet* nonConstOverlaySet(const_cast<OverlaySet*>(this));
+    return nonConstOverlaySet->getUnderlayVolume();
+}
+
 /**
  * If NO overlay (any overlay) is set to a volume, set the underlay to the first
  * volume that it finds.
@@ -214,11 +284,9 @@ OverlaySet::setUnderlayToVolume()
         if (overlayIndex >= 0) {
             std::vector<CaretMappableDataFile*> mapFiles;
             CaretMappableDataFile* mapFile;
-            //AString mapUniqueID;
             int32_t mapIndex;
             m_overlays[overlayIndex]->getSelectionData(mapFiles, 
                                                           mapFile, 
-                                                          //mapUniqueID,
                                                           mapIndex);
             
             const int32_t numMapFiles = static_cast<int32_t>(mapFiles.size());
@@ -456,500 +524,6 @@ OverlaySet::moveDisplayedOverlayDown(const int32_t overlayIndex)
     }
 }
 
-/**
- * Match the desired names to maps (or file if no maps match) and optionally
- * structure to files/maps and return the matches.
- *
- * Note: If there are NO match names, all files are matched using the first
- * map in each file.
- *
- * @param matchedFilesOut
- *     Output to which matched files are APPENDED.
- * @param matchedFileIndicesOut
- *     Output to which matched file indices are APPENDED.
- * @param matchToStructures
- *     If not empty, only include files that map to these structures.  If
- *     matchToVolumeData is true, this parameter is ignored.  If this is (empty
- *     OR All) AND matchToVolumeData is false, all structures match.
- * @param dataFileType
- *     Data file type of desired files.
- * @param matchToVolumeData
- *     Include only files that map to volume data.  If true, matchToStructures
- *     is ignored.
- * @param matchToNamesRegularExpressionText
- *     Text for regular expression used for name matching.
- * @param matchToNamesRegularExpressionResult
- *     Status of regular expression matching for inclusion of file.
- * @param matchOneFilePerStructure
- *     If true, limit matched files so there is no more than one file
- *     for each structure.
- * @param
- *     True if matching files were found, else false.
- */
-bool
-OverlaySet::findFilesWithMapNamed(std::vector<CaretMappableDataFile*>& matchedFilesOut,
-                                  std::vector<int32_t>& matchedFileIndicesOut,
-                                  const std::vector<StructureEnum::Enum>& matchToStructures,
-                                  const DataFileTypeEnum::Enum dataFileType,
-                                  const bool matchToVolumeData,
-                                  const AString& matchToNamesRegularExpressionText,
-                                  const bool matchToNamesRegularExpressionResult,
-                                  const bool matchOneFilePerStructure)
-{
-    std::vector<CaretMappableDataFile*> matchedFiles;
-    std::vector<int32_t> matchedFileIndices;
-    
-    /*
-     * Aggregate matching names and make them lower case
-     */
-    QRegExp regularExpression;
-    if (matchToNamesRegularExpressionText.isEmpty() == false) {
-        regularExpression = QRegExp(matchToNamesRegularExpressionText);
-    }
-    
-    /*
-     * Get files matching data type
-     */
-    EventCaretMappableDataFilesGet mapFileGetEvent(dataFileType);
-    EventManager::get()->sendEvent(mapFileGetEvent.getPointer());
-    std::vector<CaretMappableDataFile*> matchToMapFiles;
-    mapFileGetEvent.getAllFiles(matchToMapFiles);
-    const int32_t numberOfMatchFiles = static_cast<int32_t>(matchToMapFiles.size());
-    if (numberOfMatchFiles <= 0) {
-        return false;
-    }
-    
-    /*
-     * Determine which files should be tested by examing 
-     * structure or if volume
-     */
-    std::vector<CaretMappableDataFile*> testMapFiles;
-    for (int32_t iFile = 0; iFile < numberOfMatchFiles; iFile++) {
-        bool fileMatchFlag = false;
-        CaretMappableDataFile* mapFile = matchToMapFiles[iFile];
-        
-        /*
-         * Volume mappable files only?
-         */
-        if (matchToVolumeData) {
-            if (mapFile->isVolumeMappable()) {
-                fileMatchFlag = true;
-            }
-        }
-        
-        /*
-         * Test structures?
-         */
-        if (matchToStructures.empty() == false) {
-            const StructureEnum::Enum mapFileStructure = mapFile->getStructure();
-            /*
-             * File maps to ALL structures?
-             */
-            if (mapFileStructure == StructureEnum::ALL) {
-                fileMatchFlag = true;
-            }
-            else {
-                /*
-                 * Specific structutures
-                 */
-                if (std::find(matchToStructures.begin(),
-                              matchToStructures.end(),
-                              mapFileStructure) != matchToStructures.end()) {
-                    fileMatchFlag = true;
-                }
-            }
-        }
-        
-        if (fileMatchFlag) {
-            if (mapFile->getNumberOfMaps() > 0) {
-                testMapFiles.push_back(mapFile);
-            }
-        }
-    }
-    
-    /*
-     * No files to test?
-     */
-    const int32_t numTestFiles = static_cast<int32_t>(testMapFiles.size());
-    if (numTestFiles <= 0) {
-        return false;
-    }
-    
-    
-    std::set<StructureEnum::Enum> matchedStructures;
-    
-    /*
-     * If there are names to match
-     */
-    if (matchToNamesRegularExpressionText.isEmpty() == false) {
-        /*
-         * First preference is matching MAP name
-         */
-            for (int32_t iFile = 0; iFile < numTestFiles; iFile++) {
-                CaretMappableDataFile* mapFile = testMapFiles[iFile];
-                const StructureEnum::Enum mapFileStructure = mapFile->getStructure();
-                
-                if (matchOneFilePerStructure) {
-                    if (matchedStructures.find(mapFileStructure) != matchedStructures.end()) {
-                        continue;
-                    }
-                }
-                
-                /*
-                 * If NOT matching, exclude files whose name matches
-                 */
-                if (matchToNamesRegularExpressionResult == false) {
-                    const AString fileName = mapFile->getFileNameNoPath().toLower();
-                    
-                    const bool fileNameMatch = (regularExpression.indexIn(fileName) >= 0);
-                    if (fileNameMatch) {
-                        continue;
-                    }
-                }
-                
-                const int32_t numMaps = mapFile->getNumberOfMaps();
-                for (int32_t iMap = 0; iMap < numMaps; iMap++) {
-                    const AString mapName = mapFile->getMapName(iMap).toLower();
-                    const bool match = (regularExpression.indexIn(mapName) >= 0);
-                    if (match == matchToNamesRegularExpressionResult) {
-                        matchedFiles.push_back(mapFile);
-                        matchedFileIndices.push_back(iMap);
-                        
-                        matchedStructures.insert(mapFileStructure);
-                        break;
-                    }
-                }
-            }
-        
-        /*
-         * Find matching FILE name if NO map matches
-         */
-        if (matchedFiles.empty()) {
-            for (int32_t iFile = 0; iFile < numTestFiles; iFile++) {
-                CaretMappableDataFile* mapFile = testMapFiles[iFile];
-                const StructureEnum::Enum mapFileStructure = mapFile->getStructure();
-                
-                if (matchOneFilePerStructure) {
-                    if (matchedStructures.find(mapFileStructure) != matchedStructures.end()) {
-                        continue;
-                    }
-                }
-                
-                const AString fileName = mapFile->getFileNameNoPath().toLower();
-                const bool match = (regularExpression.indexIn(fileName) >= 0);
-                if (match == matchToNamesRegularExpressionResult) {
-                    matchedFiles.push_back(mapFile);
-                    matchedFileIndices.push_back(0);
-                    
-                    matchedStructures.insert(mapFileStructure);
-                }
-            }
-        }
-    }
-    else {
-        /*
-         * No names to match so just match to first map in each file
-         */
-        for (int32_t iFile = 0; iFile < numTestFiles; iFile++) {
-            CaretMappableDataFile* mapFile = testMapFiles[iFile];
-            const StructureEnum::Enum mapFileStructure = mapFile->getStructure();
-            
-            if (matchOneFilePerStructure) {
-                if (matchedStructures.find(mapFileStructure) != matchedStructures.end()) {
-                    continue;
-                }
-            }
-            
-            matchedFiles.push_back(mapFile);
-            matchedFileIndices.push_back(0);
-            
-            matchedStructures.insert(mapFileStructure);
-        }
-    }
-    
-    CaretAssert(matchedFiles.size() == matchedFileIndices.size());
-    
-    const bool filesFound = (matchedFiles.empty() == false);
-    
-    /*
-     * APPEND to output, do not replace
-     */
-    matchedFilesOut.insert(matchedFilesOut.end(),
-                           matchedFiles.begin(),
-                           matchedFiles.end());
-    matchedFileIndicesOut.insert(matchedFileIndicesOut.end(),
-                                 matchedFileIndices.begin(),
-                                 matchedFileIndices.end());
-    
-    return filesFound;
-}
-
-/**
- * Find underlay files.
- *
- * @param matchToStructures
- *    Structures to include.
- * @param includeVolumeFiles 
- *    Include volume files.
- * @param filesOut
- *    Output containing files that were selected.
- * @param mapIndicesOut
- *    Output containing maps indices in files that were selected.
- */
-void
-OverlaySet::findUnderlayFiles( const std::vector<StructureEnum::Enum>& matchToStructures,
-                              const bool includeVolumeFiles,
-                              std::vector<CaretMappableDataFile*>& filesOut,
-                              std::vector<int32_t>& mapIndicesOut)
-{
-    /*
-     * First, try to find CIFTI shape files
-     */
-    if (findFilesWithMapNamed(filesOut,
-                              mapIndicesOut,
-                              matchToStructures,
-                              DataFileTypeEnum::CONNECTIVITY_DENSE_SCALAR,
-                              false,
-                              s_shapeMatchRegularExpressionText,
-                              true,
-                              true) == false) {
-        
-        /*
-         * Second, try to find METRIC shape files
-         */
-        findFilesWithMapNamed(filesOut,
-                              mapIndicesOut,
-                              matchToStructures,
-                              DataFileTypeEnum::METRIC,
-                              false,
-                              s_shapeMatchRegularExpressionText,
-                              true,
-                              true);
-    }
-    
-    if (includeVolumeFiles) {
-        std::vector<VolumeFile*> volumeFiles = getVolumeFiles();
-        
-        const int32_t numVolumes = static_cast<int32_t>(volumeFiles.size());
-        if (numVolumes > 0) {
-            bool foundAnatomyVolume = false;
-
-            for (int32_t i = 0; i < numVolumes; i++) {
-                VolumeFile* vf = volumeFiles[i];
-                if (vf->getType() == SubvolumeAttributes::ANATOMY) {
-                    if (vf->getNumberOfMaps() > 0) {
-                        filesOut.push_back(vf);
-                        mapIndicesOut.push_back(0);
-                        foundAnatomyVolume = true;
-                        break;
-                    }
-                }
-            }
-            
-            if (foundAnatomyVolume == false) {
-                for (int32_t i = 0; i < numVolumes; i++) {
-                    VolumeFile* vf = volumeFiles[i];
-                    bool testIt = true;
-                    if (vf->getType() == SubvolumeAttributes::LABEL) {
-                        testIt = false;
-                    }
-                    if (testIt) {
-                        if (vf->getNumberOfMaps() > 0) {
-                            PaletteColorMapping* pcm = vf->getMapPaletteColorMapping(0);
-                            if (pcm != NULL) {
-                                const AString paletteName = pcm->getSelectedPaletteName();
-                                if (paletteName.contains("gray")
-                                    || paletteName.contains("grey")) {
-                                    filesOut.push_back(vf);
-                                    mapIndicesOut.push_back(0);
-                                    foundAnatomyVolume = true;
-                                    break;
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-        }
-    }
-    
-    CaretAssert(filesOut.size() == mapIndicesOut.size());
-}
-
-/**
- * @return All volume files.
- */
-std::vector<VolumeFile*>
-OverlaySet::getVolumeFiles() const
-{
-    std::vector<VolumeFile*> volumeFiles;
-    
-    EventCaretMappableDataFilesGet mapFileGetEvent(DataFileTypeEnum::VOLUME);
-    EventManager::get()->sendEvent(mapFileGetEvent.getPointer());
-    std::vector<CaretMappableDataFile*> matchToMapFiles;
-    mapFileGetEvent.getAllFiles(matchToMapFiles);
-    
-    for (std::vector<CaretMappableDataFile*>::iterator iter = matchToMapFiles.begin();
-         iter != matchToMapFiles.end();
-         iter++) {
-        VolumeFile* vf = dynamic_cast<VolumeFile*>(*iter);
-        CaretAssert(vf);
-        volumeFiles.push_back(vf);
-    }
-    
-    return volumeFiles;
-}
-
-
-/**
- * Find middle layer files.
- *
- * @param matchToStructures
- *    Structures to include.
- * @param includeVolumeFiles
- *    Include volume files.
- * @param filesOut
- *    Output containing files that were selected.
- * @param mapIndicesOut
- *    Output containing maps indices in files that were selected.
- */
-void
-OverlaySet::findMiddleLayerFiles(const std::vector<StructureEnum::Enum>& matchToStructures,
-                                 const bool includeVolumeFiles,
-                                 std::vector<CaretMappableDataFile*>& filesOut,
-                                 std::vector<int32_t>& mapIndicesOut)
-{
-    std::vector<AString> matchToNames;
-    
-    /*
-     * First, try to find CIFTI scalar files with myelin
-     */
-    if (findFilesWithMapNamed(filesOut,
-                              mapIndicesOut,
-                              matchToStructures,
-                              DataFileTypeEnum::CONNECTIVITY_DENSE_SCALAR,
-                              includeVolumeFiles,
-                              s_myelinMatchRegularExpressionText,
-                              true,
-                              false) == false) {
-        
-        /*
-         * Second, try to find METRIC files with neither shape nor myelin
-         */
-        findFilesWithMapNamed(filesOut,
-                              mapIndicesOut,
-                              matchToStructures,
-                              DataFileTypeEnum::METRIC,
-                              includeVolumeFiles,
-                              s_myelinMatchRegularExpressionText,
-                              true,
-                              false);
-    }
-    
-    /*
-     * Second, try to find CIFTI scalar files with neither shape nor myelin
-     */
-    if (findFilesWithMapNamed(filesOut,
-                              mapIndicesOut,
-                              matchToStructures,
-                              DataFileTypeEnum::CONNECTIVITY_DENSE_SCALAR,
-                              includeVolumeFiles,
-                              s_shapeMyelinMatchRegularExpressionText,
-                              false,
-                              false) == false) {
-        
-        /*
-         * Second, try to find METRIC files with neither shape nor myelin
-         */
-        findFilesWithMapNamed(filesOut,
-                              mapIndicesOut,
-                              matchToStructures,
-                              DataFileTypeEnum::METRIC,
-                              includeVolumeFiles,
-                              s_shapeMyelinMatchRegularExpressionText,
-                              false,
-                              false);
-    }
-    
-    if (includeVolumeFiles) {
-        std::vector<VolumeFile*> volumeFiles = getVolumeFiles();
-        const int32_t numVolumes = static_cast<int32_t>(volumeFiles.size());
-        for (int32_t i = 0; i < numVolumes; i++) {
-            VolumeFile* vf = volumeFiles[i];
-            if ((vf->getType() == SubvolumeAttributes::FUNCTIONAL)) {
-                if (vf->getNumberOfMaps() > 0) {
-                    filesOut.push_back(vf);
-                    mapIndicesOut.push_back(0);
-                    break;
-                }
-            }
-        }
-    }
-    
-    CaretAssert(filesOut.size() == mapIndicesOut.size());
-}
-
-/**
- * Find overlay files.
- *
- * @param matchToStructures
- *    Structures to include.
- * @param includeVolumeFiles
- *    Include volume files.
- * @param filesOut
- *    Output containing files that were selected.
- * @param mapIndicesOut
- *    Output containing maps indices in files that were selected.
- */
-void
-OverlaySet::findOverlayFiles(const std::vector<StructureEnum::Enum>& matchToStructures,
-                             const bool includeVolumeFiles,
-                             std::vector<CaretMappableDataFile*>& filesOut,
-                             std::vector<int32_t>& mapIndicesOut)
-{
-    /*
-     * First, try to find CIFTI LABEL files
-     */
-    if (findFilesWithMapNamed(filesOut,
-                              mapIndicesOut,
-                              matchToStructures,
-                              DataFileTypeEnum::CONNECTIVITY_DENSE_LABEL,
-                              includeVolumeFiles,
-                              "",
-                              true,
-                              true) == false) {
-        
-        /*
-         * Second, try to find LABEL files
-         */
-        findFilesWithMapNamed(filesOut,
-                              mapIndicesOut,
-                              matchToStructures,
-                              DataFileTypeEnum::LABEL,
-                              includeVolumeFiles,
-                              "",
-                              true,
-                              true);
-    }
-    
-    if (includeVolumeFiles) {
-        std::vector<VolumeFile*> volumeFiles = getVolumeFiles();
-        const int32_t numVolumes = static_cast<int32_t>(volumeFiles.size());
-        for (int32_t i = 0; i < numVolumes; i++) {
-            VolumeFile* vf = volumeFiles[i];
-            if (vf->getType() == SubvolumeAttributes::LABEL) {
-                if (vf->getNumberOfMaps() > 0) {
-                    filesOut.push_back(vf);
-                    mapIndicesOut.push_back(0);
-                    break;
-                }
-            }
-        }
-    }
-    CaretAssert(filesOut.size() == mapIndicesOut.size());
-}
-
-
 /**
  * Initialize the overlays.
  */
@@ -957,139 +531,86 @@ void
 OverlaySet::initializeOverlays()
 {
     bool isMatchToVolumeUnderlay = false;
-    bool isMatchToVolumeOverlays = false;
+    bool isVolumesForHistology   = false;
+    bool atLeastOneOverlayEnabledFlag(false);
     
     switch (m_includeVolumeFiles) {
         case Overlay::INCLUDE_VOLUME_FILES_NO:
             break;
         case Overlay::INCLUDE_VOLUME_FILES_YES:
-            /*
-             * If no surface structures, then it must be volume slice view
-             * so allow volumes to be in the overlays.
-             */
-            if (m_includeSurfaceStructures.empty()) {
-                isMatchToVolumeOverlays = true;
-            }
             isMatchToVolumeUnderlay = true;
             break;
-    }
-    
-    /*
-     * Underlays consist of anatomical type data
-     */
-    std::vector<CaretMappableDataFile*> underlayMapFiles;
-    std::vector<int32_t> underlayMapIndices;
-    findUnderlayFiles(m_includeSurfaceStructures,
-                      isMatchToVolumeUnderlay,
-                      underlayMapFiles,
-                      underlayMapIndices);
-    
-    /*
-     * Middle layers are Cifti labels or Gifti Labels
-     * that do not contain shape data
-     */
-    std::vector<CaretMappableDataFile*> middleLayerMapFiles;
-    std::vector<int32_t> middleLayerMapIndices;
-    findMiddleLayerFiles(m_includeSurfaceStructures,
-                         isMatchToVolumeOverlays,
-                         middleLayerMapFiles,
-                         middleLayerMapIndices);
-    
-    /*
-     * Overlays consist of Cifti scalars or Gifti Metric
-     */
-    std::vector<CaretMappableDataFile*> overlayMapFiles;
-    std::vector<int32_t> overlayMapIndices;
-    findOverlayFiles(m_includeSurfaceStructures,
-                         isMatchToVolumeOverlays,
-                         overlayMapFiles,
-                         overlayMapIndices);
-    
-    const int32_t numberOfUnderlayFiles = static_cast<int32_t>(underlayMapFiles.size());
-    
-    /*
-     * Number of overlay that are displayed.
-     */
-    const int32_t numberOfDisplayedOverlays = getNumberOfDisplayedOverlays();
-    
-    
-    /*
-     * Track overlay that were initialized
-     */
-    std::vector<bool> overlayInitializedFlag(numberOfDisplayedOverlays,
-                                             false);
-    
-    /*
-     * Put in the shape files at the bottom
-     * Note that highest overlay index is bottom
-     */
-    int32_t overlayIndexForUnderlay = (numberOfDisplayedOverlays - 1);
-    for (int32_t underlayFileIndex = 0; underlayFileIndex < numberOfUnderlayFiles; underlayFileIndex++) {
-        if (overlayIndexForUnderlay >= 0) {
-            Overlay* overlay = getOverlay(overlayIndexForUnderlay);
-            overlay->setSelectionData(underlayMapFiles[underlayFileIndex],
-                                      underlayMapIndices[underlayFileIndex]);
-            overlayInitializedFlag[overlayIndexForUnderlay] = true;
-            overlayIndexForUnderlay--;
-        }
-        else {
+        case Overlay::INCLUDE_VOLUME_FILES_FOR_HISTOLOGY_MODEL:
+            isVolumesForHistology = true;
             break;
-        }
     }
     
-    /*
-     * Combine overlay and middle layer files
-     */
-    std::vector<CaretMappableDataFile*> upperLayerFiles;
-    std::vector<int32_t> upperLayerIndices;
-    upperLayerFiles.insert(upperLayerFiles.end(),
-                           overlayMapFiles.begin(),
-                           overlayMapFiles.end());
-    upperLayerIndices.insert(upperLayerIndices.end(),
-                             overlayMapIndices.begin(),
-                             overlayMapIndices.end());
-    upperLayerFiles.insert(upperLayerFiles.end(),
-                           middleLayerMapFiles.begin(),
-                           middleLayerMapFiles.end());
-    upperLayerIndices.insert(upperLayerIndices.end(),
-                             middleLayerMapIndices.begin(),
-                             middleLayerMapIndices.end());
-    CaretAssert(upperLayerFiles.size() == upperLayerIndices.size());
-    
-    const int32_t numberOfUpperFiles = static_cast<int32_t>(upperLayerFiles.size());
-    
-    /*
-     * Put in overlay and middle layer files
-     */
-    for (int32_t upperFileIndex = 0; upperFileIndex < numberOfUpperFiles; upperFileIndex++) {
-        /*
-         * Find available overlay
-         */
-        int32_t upperLayerOverlayIndex = -1;
-        for (int32_t overlayIndex = 0; overlayIndex < numberOfDisplayedOverlays; overlayIndex++) {
-            if (overlayInitializedFlag[overlayIndex] == false) {
-                upperLayerOverlayIndex = overlayIndex;
-                break;
-            }
+    {
+        const bool logFlag(m_tabIndex == 0);
+        OverlaySetInitializer initializer;
+        std::vector<OverlaySetInitializer::FileAndMapIndex> layers(initializer.initializeOverlaySet(m_includeSurfaceStructures,
+                                                                                                    isMatchToVolumeUnderlay,
+                                                                                                    logFlag));
+        if (layers.size() > BrainConstants::MAXIMUM_NUMBER_OF_OVERLAYS) {
+            layers.resize(BrainConstants::MAXIMUM_NUMBER_OF_OVERLAYS);
         }
         
-        if (upperLayerOverlayIndex >= 0) {
-            Overlay* upperLayerOverlay = getOverlay(upperLayerOverlayIndex);
-            upperLayerOverlay->setSelectionData(upperLayerFiles[upperFileIndex],
-                                                upperLayerIndices[upperFileIndex]);
-            overlayInitializedFlag[upperLayerOverlayIndex] = true;
-        }
-        else {
-            break;
+        /*
+         * Note: There are minimum and maximum number of overlays
+         * that are applied when setNumberOfDisplayedOverlays() is
+         * called.  Thus the displayed number of overlays may
+         * be different than the number of layers to load.
+         *
+         * Layers from overlay initializer are ordered underlay to
+         * overlay.  Put underlay at bottom of these overlays.
+         */
+        int32_t layerIndex(0);
+        const int32_t numLayers(layers.size());
+        setNumberOfDisplayedOverlays(numLayers);
+        const int32_t numDisplayedLayers(getNumberOfDisplayedOverlays());
+        for (int32_t i = (numDisplayedLayers - 1); i >= 0; i--) {
+            Overlay* overlay(getOverlay(i));
+            CaretAssert(overlay);
+            if (layerIndex < numLayers) {
+                CaretAssertVectorIndex(layers, layerIndex);
+                overlay->setSelectionData(layers[layerIndex].m_file,
+                                          layers[layerIndex].m_mapIndex);
+                overlay->setEnabled(true);
+                atLeastOneOverlayEnabledFlag = true;
+                ++layerIndex;
+            }
+            else {
+                overlay->setEnabled(false);
+            }
+            overlay->setMapYokingGroup(MapYokingGroupEnum::MAP_YOKING_GROUP_OFF);
         }
     }
     
     /*
-     * Disable overlays that were not initialized
+     * If no overlays were enabled, enable the underlay
+     * if it contains a file.  This problem was observed
+     * if one loaded an RGB volume (no overlay was enabled).
      */
-    for (int32_t i = 0; i < numberOfDisplayedOverlays; i++) {
-        CaretAssertVectorIndex(overlayInitializedFlag, i);
-        getOverlay(i)->setEnabled(overlayInitializedFlag[i]);
+    if ( ! atLeastOneOverlayEnabledFlag) {
+        const int32_t numberOfDisplayedOverlays(getNumberOfDisplayedOverlays());
+        if (numberOfDisplayedOverlays > 0) {
+            const int32_t overlayIndex(numberOfDisplayedOverlays - 1);
+            Overlay* overlay(getOverlay(overlayIndex));
+            CaretMappableDataFile* mapFile(NULL);
+            int32_t mapIndex(0);
+            overlay->getSelectionData(mapFile, mapIndex);
+            if ((mapFile != NULL)
+                && (mapIndex >= 0)) {
+                overlay->setEnabled(true);
+            }
+        }
+    }
+    
+    if (isVolumesForHistology) {
+        for (Overlay* overlay : m_overlays) {
+            CaretAssert(overlay);
+            overlay->setEnabled(false);
+        }
     }
 }
 
@@ -1163,7 +684,7 @@ OverlaySet::getSelectedMapIndicesForFile(const CaretMappableDataFile* caretMappa
         Overlay* overlay = const_cast<Overlay*>(getOverlay(i));
         bool checkIt = true;
         if (isLimitToEnabledOverlays) {
-            if (overlay->isEnabled() == false) {
+            if ( ! overlay->isEnabled()) {
                 checkIt = false;
             }
         }
@@ -1221,7 +742,6 @@ OverlaySet::saveToScene(const SceneAttributes* sceneAttributes,
     m_sceneAssistant->saveMembers(sceneAttributes, 
                                   sceneClass);
     
-//    const int32_t numOverlaysToSave = BrainConstants::MAXIMUM_NUMBER_OF_OVERLAYS;
     const int32_t numOverlaysToSave = getNumberOfDisplayedOverlays();
     
     std::vector<SceneClass*> overlayClassVector;
@@ -1353,9 +873,36 @@ OverlaySet::receiveEvent(Event* event)
             
             selectMapEvent->setEventProcessed();
         }
-        
-        
-//        const MapYokingGroupEnum::Enum mapYokingGroup = selectMapEvent->get
     }
 }
 
+/**
+ * @return True if an enabled overlay contains a volume file that is an oblique only volume
+ */
+bool
+OverlaySet::hasObliqueOnlyVolumeSelected() const
+{
+    const int32_t numberOfOverlays = getNumberOfDisplayedOverlays();
+    for (int32_t i = 0; i < numberOfOverlays; i++) {
+        Overlay* overlay = const_cast<Overlay*>(getOverlay(i));
+        if (overlay->isEnabled()) {
+            CaretMappableDataFile* mapFile;
+            int32_t mapIndex;
+            overlay->getSelectionData(mapFile, mapIndex);
+            if (mapFile != NULL) {
+                if (mapFile->getDataFileType() == DataFileTypeEnum::VOLUME) {
+                    const VolumeFile* vf = dynamic_cast<const VolumeFile*>(mapFile);
+                    CaretAssert(vf);
+                    if ( ! vf->isPlumb()) {
+                        return true;
+                    }
+                }
+            }
+        }
+    }
+    
+    return false;
+}
+
+
+
