@@ -21,6 +21,8 @@
  */
 /*LICENSE_END*/
 
+#include <memory>
+
 #include "CaretObject.h"
 #include "EventListenerInterface.h"
 
@@ -35,17 +37,27 @@ namespace caret {
     class SelectionItemChartFrequencySeries;
     class SelectionItemChartMatrix;
     class SelectionItemChartTimeSeries;
+    class SelectionItemChartTwoHistogram;
+    class SelectionItemChartTwoLabel;
+    class SelectionItemChartTwoLineLayer;
+    class SelectionItemChartTwoLineLayerVerticalNearest;
+    class SelectionItemChartTwoLineSeries;
+    class SelectionItemChartTwoMatrix;
     class SelectionItemCiftiConnectivityMatrixRowColumn;
     class SelectionItemFocusSurface;
-    class SelectionItemFocusVolume;
-    class SelectionItemImage;
+    class SelectionItemFocus;
+    class SelectionItemHistologyCoordinate;
+    class SelectionItemImageControlPoint;
+    class SelectionItemMediaLogicalCoordinate;
+    class SelectionItemMediaPlaneCoordinate;
     class SelectionItemSurfaceNode;
-    class SelectionItemSurfaceNodeIdentificationSymbol;
     class SelectionItemSurfaceTriangle;
+    class SelectionItemUniversalIdentificationSymbol;
+    class SelectionItemVolumeMprCrosshair;
     class SelectionItemVoxel;
     class SelectionItemVoxelEditing;
-    class SelectionItemVoxelIdentificationSymbol;
-    class IdentificationTextGenerator;
+    class IdentificationFormattedTextGenerator;
+    class IdentificationSimpleTextGenerator;
     class Surface;
     
     class SelectionManager : public CaretObject, public EventListenerInterface {
@@ -62,7 +74,7 @@ namespace caret {
         SelectionItemAnnotation* getAnnotationIdentification();
         
         const SelectionItemAnnotation* getAnnotationIdentification() const;
-        
+                
         SelectionItemBorderSurface* getSurfaceBorderIdentification();
         
         const SelectionItemBorderSurface* getSurfaceBorderIdentification() const;
@@ -71,22 +83,34 @@ namespace caret {
         
         const SelectionItemFocusSurface* getSurfaceFocusIdentification() const;
         
-        SelectionItemFocusVolume* getVolumeFocusIdentification();
+        SelectionItemFocus* getFocusIdentification();
         
-        const SelectionItemFocusVolume* getVolumeFocusIdentification() const;
+        const SelectionItemFocus* getFocusIdentification() const;
         
-        SelectionItemImage* getImageIdentification();
+        SelectionItemHistologyCoordinate* getHistologyPlaneCoordinateIdentification();
         
-        const SelectionItemImage* getImageIdentification() const;
+        const SelectionItemHistologyCoordinate* getHistologyPlaneCoordinateIdentification() const;
         
+        SelectionItemImageControlPoint* getImageControlPointIdentification();
+        
+        const SelectionItemImageControlPoint* getImageControlPointIdentification() const;
+        
+        SelectionItemMediaLogicalCoordinate* getMediaLogicalCoordinateIdentification();
+        
+        const SelectionItemMediaLogicalCoordinate* getMediaLogicalCoordinateIdentification() const;
+        
+        SelectionItemMediaPlaneCoordinate* getMediaPlaneCoordinateIdentification();
+        
+        const SelectionItemMediaPlaneCoordinate* getMediaPlaneCoordinateIdentification() const;
+        
+        SelectionItemAnnotation* getSamplesIdentification();
+        
+        const SelectionItemAnnotation* getSamplesIdentification() const;
+
         SelectionItemSurfaceNode* getSurfaceNodeIdentification();
         
         const SelectionItemSurfaceNode* getSurfaceNodeIdentification() const;
         
-        SelectionItemSurfaceNodeIdentificationSymbol* getSurfaceNodeIdentificationSymbol();
-        
-        const SelectionItemSurfaceNodeIdentificationSymbol* getSurfaceNodeIdentificationSymbol() const;
-        
         SelectionItemSurfaceTriangle* getSurfaceTriangleIdentification();
         
         const SelectionItemSurfaceTriangle* getSurfaceTriangleIdentification() const;
@@ -95,9 +119,13 @@ namespace caret {
         
         SelectionItemVoxel* getVoxelIdentification();
         
-        SelectionItemVoxelIdentificationSymbol* getVoxelIdentificationSymbol();
+        SelectionItemUniversalIdentificationSymbol* getUniversalIdentificationSymbol();
+
+        const SelectionItemUniversalIdentificationSymbol* getUniversalIdentificationSymbol() const;
         
-        const SelectionItemVoxelIdentificationSymbol* getVoxelIdentificationSymbol() const;
+        SelectionItemVolumeMprCrosshair* getVolumeMprCrosshairIdentification();
+        
+        const SelectionItemVolumeMprCrosshair* getVolumeMprCrosshairIdentification() const;
         
         SelectionItemVoxelEditing* getVoxelEditingIdentification();
         
@@ -123,7 +151,34 @@ namespace caret {
         
         const SelectionItemCiftiConnectivityMatrixRowColumn* getCiftiConnectivityMatrixRowColumnIdentification() const;
         
-        AString getIdentificationText(const Brain* brain) const;
+        SelectionItemChartTwoHistogram* getChartTwoHistogramIdentification();
+        
+        const SelectionItemChartTwoHistogram* getChartTwoHistogramIdentification() const;
+        
+        SelectionItemChartTwoLineLayer* getChartTwoLineLayerIdentification();
+        
+        const SelectionItemChartTwoLineLayer* getChartTwoLineLayerIdentification() const;
+        
+        SelectionItemChartTwoLineLayerVerticalNearest* getChartTwoLineLayerVerticalNearestIdentification();
+        
+        const SelectionItemChartTwoLineLayerVerticalNearest* getChartTwoLineLayerVerticalNearestIdentification() const;
+        
+        SelectionItemChartTwoLineSeries* getChartTwoLineSeriesIdentification();
+        
+        const SelectionItemChartTwoLineSeries* getChartTwoLineSeriesIdentification() const;
+        
+        SelectionItemChartTwoLabel* getChartTwoLabelIdentification();
+        
+        const SelectionItemChartTwoLabel* getChartTwoLabelIdentification() const;
+        
+        SelectionItemChartTwoMatrix* getChartTwoMatrixIdentification();
+        
+        const SelectionItemChartTwoMatrix* getChartTwoMatrixIdentification() const;
+        
+        AString getSimpleIdentificationText(const Brain* brain) const;
+        
+        AString getFormattedIdentificationText(const Brain* brain,
+                                               const int32_t tabIndex) const;
         
         void filterSelections(const bool applySelectionBackgroundFiltering);
         
@@ -172,28 +227,50 @@ namespace caret {
         
         SelectionItemChartTimeSeries* m_chartTimeSeriesIdentification;
         
+        std::unique_ptr<SelectionItemChartTwoHistogram>  m_chartTwoHistogramIdentification;
+        
+        std::unique_ptr<SelectionItemChartTwoLineSeries> m_chartTwoLineSeriesIdentification;
+        
+        std::unique_ptr<SelectionItemChartTwoLineLayer> m_chartTwoLineLayerIdentification;
+        
+        std::unique_ptr<SelectionItemChartTwoLineLayerVerticalNearest> m_chartTwoLineLayerVerticalNearestIdentification;
+        
+        std::unique_ptr<SelectionItemChartTwoLabel> m_chartTwoLabelIdentification;
+        
+        std::unique_ptr<SelectionItemChartTwoMatrix>     m_chartTwoMatrixIdentification;
+
         SelectionItemCiftiConnectivityMatrixRowColumn* m_ciftiConnectivityMatrixRowColumnIdentfication;
         
         SelectionItemFocusSurface* m_surfaceFocusIdentification;
         
-        SelectionItemFocusVolume* m_volumeFocusIdentification;
+        SelectionItemFocus* m_focusIdentification;
         
-        SelectionItemImage* m_imageIdentification;
+        SelectionItemImageControlPoint* m_imageControlPointIdentification;
+        
+        std::unique_ptr<SelectionItemHistologyCoordinate> m_histologyPlaneCoordinateIdentification;
+        
+        std::unique_ptr<SelectionItemMediaLogicalCoordinate> m_mediaLogicalCoordinateIdentification;
+        
+        std::unique_ptr<SelectionItemMediaPlaneCoordinate> m_mediaPlaneCoordinateIdentification;
+        
+        std::unique_ptr<SelectionItemAnnotation> m_samplesIdentification;
         
         SelectionItemSurfaceNode* m_surfaceNodeIdentification;
         
-        SelectionItemSurfaceNodeIdentificationSymbol* m_surfaceNodeIdentificationSymbol;
-        
         SelectionItemSurfaceTriangle* m_surfaceTriangleIdentification;
         
-        IdentificationTextGenerator* m_idTextGenerator;
+        IdentificationSimpleTextGenerator* m_idTextGenerator;
+        
+        std::unique_ptr<IdentificationFormattedTextGenerator> m_idFormattedTextGenerator;
+        
+        std::unique_ptr<SelectionItemUniversalIdentificationSymbol> m_universalIdentificationSymbol;
         
         SelectionItemVoxel* m_voxelIdentification;
         
-        SelectionItemVoxelIdentificationSymbol* m_voxelIdentificationSymbol;
-        
         SelectionItemVoxelEditing* m_voxelEditingIdentification;
         
+        std::unique_ptr<SelectionItemVolumeMprCrosshair> m_volumeMprCrosshairIdentification;
+        
         /** Last selected item DOES NOT GET PUT IN m_allSelectionItems */
         SelectionItem* m_lastSelectedItem;
     };
