@@ -305,21 +305,25 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
         if (surface != NULL) {
             if (m_anteriorEnabled) {
                 anteriorViewports.push_back(SurfaceMontageViewport(surface,
-                                                                 ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_ANTERIOR));
+                                                                   surface,
+                                                                   ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_ANTERIOR));
                 numFirst++;
             }
             if (m_dorsalEnabled) {
                 dorsalViewports.push_back(SurfaceMontageViewport(surface,
+                                                                 surface,
                                                                  ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_DORSAL));
                 numFirst++;
             }
             if (m_posteriorEnabled) {
                 posteriorViewports.push_back(SurfaceMontageViewport(surface,
-                                                                 ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_POSTERIOR));
+                                                                    surface,
+                                                                    ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_POSTERIOR));
                 numFirst++;
             }
             if (m_ventralEnabled) {
                 ventralViewports.push_back(SurfaceMontageViewport(surface,
+                                                                  surface,
                                                                  ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_VENTRAL));
                 numFirst++;
             }
@@ -332,22 +336,26 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
         if (surface != NULL) {
             if (m_anteriorEnabled) {
                 anteriorViewports.push_back(SurfaceMontageViewport(surface,
-                                                                 ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_ANTERIOR));
+                                                                   surface,
+                                                                   ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_ANTERIOR));
                 numSecond++;
             }
             if (m_dorsalEnabled) {
                 dorsalViewports.push_back(SurfaceMontageViewport(surface,
+                                                                 surface,
                                                                  ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_DORSAL));
                 numSecond++;
             }
             if (m_posteriorEnabled) {
                 posteriorViewports.push_back(SurfaceMontageViewport(surface,
-                                                                 ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_POSTERIOR));
+                                                                    surface,
+                                                                    ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_POSTERIOR));
                 numSecond++;
             }
             if (m_ventralEnabled) {
                 ventralViewports.push_back(SurfaceMontageViewport(surface,
-                                                                 ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_VENTRAL));
+                                                                  surface,
+                                                                  ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_VENTRAL));
                 numSecond++;
             }
         }
@@ -387,6 +395,10 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
         CaretAssert(surfaceMontageViewports.size() == 2);
         
         switch (getLayoutOrientation()) {
+            case SurfaceMontageLayoutOrientationEnum::COLUMN_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[1].setRowAndColumn(1, 0);
+                break;
             case SurfaceMontageLayoutOrientationEnum::LANDSCAPE_LAYOUT_ORIENTATION:
                 surfaceMontageViewports[0].setRowAndColumn(0, 0);
                 surfaceMontageViewports[1].setRowAndColumn(0, 1);
@@ -395,8 +407,11 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
                 surfaceMontageViewports[0].setRowAndColumn(0, 0);
                 surfaceMontageViewports[1].setRowAndColumn(1, 0);
                 break;
+            case SurfaceMontageLayoutOrientationEnum::ROW_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[1].setRowAndColumn(0, 1);
+                break;
         }
-        
     }
     else if (totalNum == 3) {
         surfaceMontageViewports.insert(surfaceMontageViewports.end(),
@@ -405,25 +420,89 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
         
         CaretAssert(surfaceMontageViewports.size() == 3);
         
+        int32_t dorsalIndex(-1);
+        int32_t ventralIndex(-1);
+        int32_t anteriorIndex(-1);
+        int32_t posteriorIndex(-1);
         for (int32_t i = 0; i < 3; i++) {
             SurfaceMontageViewport& svp = surfaceMontageViewports[i];
             switch (svp.getProjectionViewType()) {
                 case ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_ANTERIOR:
                     svp.setRowAndColumn(1, 0);
+                    anteriorIndex = i;
                     break;
                 case ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_DORSAL:
                     svp.setRowAndColumn(0, 0);
+                    dorsalIndex = i;
                     break;
                 case ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_POSTERIOR:
                     svp.setRowAndColumn(1, 1);
+                    posteriorIndex = i;
                     break;
                 case ProjectionViewTypeEnum::PROJECTION_VIEW_CEREBELLUM_VENTRAL:
                     svp.setRowAndColumn(0, 1);
+                    ventralIndex = i;
                     break;
                 default:
                     CaretAssert(0);
             }
         }
+        switch (getLayoutOrientation()) {
+            case SurfaceMontageLayoutOrientationEnum::COLUMN_LAYOUT_ORIENTATION:
+            {
+                int32_t columnIndex = 0;
+                if (dorsalIndex >= 0) {
+                    CaretAssertVectorIndex(surfaceMontageViewports, dorsalIndex);
+                    surfaceMontageViewports[dorsalIndex].setRowAndColumn(columnIndex, 0);
+                    columnIndex++;
+                }
+                if (ventralIndex >= 0) {
+                    CaretAssertVectorIndex(surfaceMontageViewports, ventralIndex);
+                    surfaceMontageViewports[ventralIndex].setRowAndColumn(columnIndex, 0);
+                    columnIndex++;
+                }
+                if (anteriorIndex >= 0) {
+                    CaretAssertVectorIndex(surfaceMontageViewports, anteriorIndex);
+                    surfaceMontageViewports[anteriorIndex].setRowAndColumn(columnIndex, 0);
+                    columnIndex++;
+                }
+                if (posteriorIndex >= 0) {
+                    CaretAssertVectorIndex(surfaceMontageViewports, posteriorIndex);
+                    surfaceMontageViewports[posteriorIndex].setRowAndColumn(columnIndex, 0);
+                    columnIndex++;
+                }
+            }
+                break;
+            case SurfaceMontageLayoutOrientationEnum::LANDSCAPE_LAYOUT_ORIENTATION:
+                break;
+            case SurfaceMontageLayoutOrientationEnum::PORTRAIT_LAYOUT_ORIENTATION:
+                break;
+            case SurfaceMontageLayoutOrientationEnum::ROW_LAYOUT_ORIENTATION:
+            {
+                int32_t rowIndex = 0;
+                if (dorsalIndex >= 0) {
+                    CaretAssertVectorIndex(surfaceMontageViewports, dorsalIndex);
+                    surfaceMontageViewports[dorsalIndex].setRowAndColumn(0, rowIndex);
+                    rowIndex++;
+                }
+                if (ventralIndex >= 0) {
+                    CaretAssertVectorIndex(surfaceMontageViewports, ventralIndex);
+                    surfaceMontageViewports[ventralIndex].setRowAndColumn(0, rowIndex);
+                    rowIndex++;
+                }
+                if (anteriorIndex >= 0) {
+                    CaretAssertVectorIndex(surfaceMontageViewports, anteriorIndex);
+                    surfaceMontageViewports[anteriorIndex].setRowAndColumn(0, rowIndex);
+                    rowIndex++;
+                }
+                if (posteriorIndex >= 0) {
+                    CaretAssertVectorIndex(surfaceMontageViewports, posteriorIndex);
+                    surfaceMontageViewports[posteriorIndex].setRowAndColumn(0, rowIndex);
+                    rowIndex++;
+                }
+            }
+                break;
+        }
     }
     else if (totalNum == 4) {
         if ((numFirst == 4)
@@ -451,6 +530,25 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
         else {
             CaretAssert(0);
         }
+        
+        switch (getLayoutOrientation()) {
+            case SurfaceMontageLayoutOrientationEnum::COLUMN_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[2].setRowAndColumn(1, 0);
+                surfaceMontageViewports[1].setRowAndColumn(2, 0);
+                surfaceMontageViewports[3].setRowAndColumn(3, 0);
+                break;
+            case SurfaceMontageLayoutOrientationEnum::LANDSCAPE_LAYOUT_ORIENTATION:
+                break;
+            case SurfaceMontageLayoutOrientationEnum::PORTRAIT_LAYOUT_ORIENTATION:
+                break;
+            case SurfaceMontageLayoutOrientationEnum::ROW_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[2].setRowAndColumn(0, 1);
+                surfaceMontageViewports[1].setRowAndColumn(0, 2);
+                surfaceMontageViewports[3].setRowAndColumn(0, 3);
+                break;
+        }
     }
     else if (totalNum == 6) {
         surfaceMontageViewports.insert(surfaceMontageViewports.end(),
@@ -459,6 +557,14 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
         CaretAssert(surfaceMontageViewports.size() == 6);
         
         switch (getLayoutOrientation()) {
+            case SurfaceMontageLayoutOrientationEnum::COLUMN_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[2].setRowAndColumn(1, 0);
+                surfaceMontageViewports[4].setRowAndColumn(2, 0);
+                surfaceMontageViewports[1].setRowAndColumn(3, 0);
+                surfaceMontageViewports[3].setRowAndColumn(4, 0);
+                surfaceMontageViewports[5].setRowAndColumn(5, 0);
+                break;
             case SurfaceMontageLayoutOrientationEnum::LANDSCAPE_LAYOUT_ORIENTATION:
                 surfaceMontageViewports[0].setRowAndColumn(0, 0);
                 surfaceMontageViewports[1].setRowAndColumn(1, 0);
@@ -475,6 +581,14 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
                 surfaceMontageViewports[4].setRowAndColumn(2, 0);
                 surfaceMontageViewports[5].setRowAndColumn(2, 1);
                 break;
+            case SurfaceMontageLayoutOrientationEnum::ROW_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[2].setRowAndColumn(0, 1);
+                surfaceMontageViewports[4].setRowAndColumn(0, 2);
+                surfaceMontageViewports[1].setRowAndColumn(0, 3);
+                surfaceMontageViewports[3].setRowAndColumn(0, 4);
+                surfaceMontageViewports[5].setRowAndColumn(0, 5);
+                break;
         }
     }
     else if (totalNum == 8) {
@@ -484,6 +598,16 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
         CaretAssert(surfaceMontageViewports.size() == 8);
         
         switch (getLayoutOrientation()) {
+            case SurfaceMontageLayoutOrientationEnum::COLUMN_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[2].setRowAndColumn(1, 0);
+                surfaceMontageViewports[4].setRowAndColumn(2, 0);
+                surfaceMontageViewports[6].setRowAndColumn(3, 0);
+                surfaceMontageViewports[1].setRowAndColumn(4, 0);
+                surfaceMontageViewports[3].setRowAndColumn(5, 0);
+                surfaceMontageViewports[5].setRowAndColumn(6, 0);
+                surfaceMontageViewports[7].setRowAndColumn(7, 0);
+                break;
             case SurfaceMontageLayoutOrientationEnum::LANDSCAPE_LAYOUT_ORIENTATION:
                 surfaceMontageViewports[0].setRowAndColumn(0, 0);
                 surfaceMontageViewports[1].setRowAndColumn(0, 2);
@@ -504,6 +628,16 @@ SurfaceMontageConfigurationCerebellar::updateSurfaceMontageViewports(std::vector
                 surfaceMontageViewports[6].setRowAndColumn(3, 0);
                 surfaceMontageViewports[7].setRowAndColumn(3, 1);
                 break;
+            case SurfaceMontageLayoutOrientationEnum::ROW_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[2].setRowAndColumn(0, 1);
+                surfaceMontageViewports[4].setRowAndColumn(0, 2);
+                surfaceMontageViewports[6].setRowAndColumn(0, 3);
+                surfaceMontageViewports[1].setRowAndColumn(0, 4);
+                surfaceMontageViewports[3].setRowAndColumn(0, 5);
+                surfaceMontageViewports[5].setRowAndColumn(0, 6);
+                surfaceMontageViewports[7].setRowAndColumn(0, 7);
+                break;
         }
     }
     else if (totalNum > 0) {
