@@ -189,11 +189,13 @@ SurfaceMontageConfigurationCerebral::updateSurfaceMontageViewports(std::vector<S
             if (leftSurface != NULL) {
                 if (m_lateralEnabled) {
                     SurfaceMontageViewport smv(leftSurface,
+                                               leftSurface,
                                                ProjectionViewTypeEnum::PROJECTION_VIEW_LEFT_LATERAL);
                     leftLateralViewports.push_back(smv);
                 }
                 if (m_medialEnabled) {
                     SurfaceMontageViewport smv(leftSurface,
+                                               leftSurface,
                                                ProjectionViewTypeEnum::PROJECTION_VIEW_LEFT_MEDIAL);
                     leftMedialViewports.push_back(smv);
                 }
@@ -204,11 +206,13 @@ SurfaceMontageConfigurationCerebral::updateSurfaceMontageViewports(std::vector<S
             if (leftSurface != NULL) {
                 if (m_lateralEnabled) {
                     SurfaceMontageViewport smv(leftSurface,
+                                               leftSurface,
                                                ProjectionViewTypeEnum::PROJECTION_VIEW_LEFT_LATERAL);
                     leftLateralViewports.push_back(smv);
                 }
                 if (m_medialEnabled) {
                     SurfaceMontageViewport smv(leftSurface,
+                                               leftSurface,
                                                ProjectionViewTypeEnum::PROJECTION_VIEW_LEFT_MEDIAL);
                     leftMedialViewports.push_back(smv);
                 }
@@ -222,11 +226,13 @@ SurfaceMontageConfigurationCerebral::updateSurfaceMontageViewports(std::vector<S
             if (rightSurface != NULL) {
                 if (m_lateralEnabled) {
                     SurfaceMontageViewport smv(rightSurface,
+                                               rightSurface,
                                                ProjectionViewTypeEnum::PROJECTION_VIEW_RIGHT_LATERAL);
                     rightLateralViewports.push_back(smv);
                 }
                 if (m_medialEnabled) {
                     SurfaceMontageViewport smv(rightSurface,
+                                               rightSurface,
                                                ProjectionViewTypeEnum::PROJECTION_VIEW_RIGHT_MEDIAL);
                     rightMedialViewports.push_back(smv);
                 }
@@ -239,11 +245,13 @@ SurfaceMontageConfigurationCerebral::updateSurfaceMontageViewports(std::vector<S
             if (rightSurface != NULL) {
                 if (m_lateralEnabled) {
                     SurfaceMontageViewport smv(rightSurface,
+                                               rightSurface,
                                                ProjectionViewTypeEnum::PROJECTION_VIEW_RIGHT_LATERAL);
                     rightLateralViewports.push_back(smv);
                 }
                 if (m_medialEnabled) {
                     SurfaceMontageViewport smv(rightSurface,
+                                               rightSurface,
                                                ProjectionViewTypeEnum::PROJECTION_VIEW_RIGHT_MEDIAL);
                     rightMedialViewports.push_back(smv);
                 }
@@ -294,6 +302,10 @@ SurfaceMontageConfigurationCerebral::updateSurfaceMontageViewports(std::vector<S
         CaretAssert(surfaceMontageViewports.size() == 2);
         
         switch (getLayoutOrientation()) {
+            case SurfaceMontageLayoutOrientationEnum::COLUMN_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[1].setRowAndColumn(1, 0);
+                break;
             case SurfaceMontageLayoutOrientationEnum::LANDSCAPE_LAYOUT_ORIENTATION:
                 surfaceMontageViewports[0].setRowAndColumn(0, 0);
                 surfaceMontageViewports[1].setRowAndColumn(0, 1);
@@ -302,6 +314,10 @@ SurfaceMontageConfigurationCerebral::updateSurfaceMontageViewports(std::vector<S
                 surfaceMontageViewports[0].setRowAndColumn(0, 0);
                 surfaceMontageViewports[1].setRowAndColumn(1, 0);
                 break;
+            case SurfaceMontageLayoutOrientationEnum::ROW_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[1].setRowAndColumn(0, 1);
+                break;
         }
         
     }
@@ -349,6 +365,26 @@ SurfaceMontageConfigurationCerebral::updateSurfaceMontageViewports(std::vector<S
         else {
             CaretAssert(0);
         }
+        
+        /*
+         * Override layout for row/column layout
+         */
+        switch (getLayoutOrientation()) {
+            case SurfaceMontageLayoutOrientationEnum::COLUMN_LAYOUT_ORIENTATION:
+                for (int32_t i = 0; i < 4; i++) {
+                    surfaceMontageViewports[i].setRowAndColumn(i, 0);
+                }
+                break;
+            case SurfaceMontageLayoutOrientationEnum::LANDSCAPE_LAYOUT_ORIENTATION:
+                break;
+            case SurfaceMontageLayoutOrientationEnum::PORTRAIT_LAYOUT_ORIENTATION:
+                break;
+            case SurfaceMontageLayoutOrientationEnum::ROW_LAYOUT_ORIENTATION:
+                for (int32_t i = 0; i < 4; i++) {
+                    surfaceMontageViewports[i].setRowAndColumn(0, i);
+                }
+                break;
+        }
     }
     else if (totalNum == 8) {
         surfaceMontageViewports.insert(surfaceMontageViewports.end(),
@@ -366,6 +402,16 @@ SurfaceMontageConfigurationCerebral::updateSurfaceMontageViewports(std::vector<S
         CaretAssert(surfaceMontageViewports.size() == 8);
         
         switch (getLayoutOrientation()) {
+            case SurfaceMontageLayoutOrientationEnum::COLUMN_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[2].setRowAndColumn(1, 0);
+                surfaceMontageViewports[1].setRowAndColumn(2, 0);
+                surfaceMontageViewports[3].setRowAndColumn(3, 0);
+                surfaceMontageViewports[4].setRowAndColumn(4, 0);
+                surfaceMontageViewports[6].setRowAndColumn(5, 0);
+                surfaceMontageViewports[5].setRowAndColumn(6, 0);
+                surfaceMontageViewports[7].setRowAndColumn(7, 0);
+                break;
             case SurfaceMontageLayoutOrientationEnum::LANDSCAPE_LAYOUT_ORIENTATION:
                 surfaceMontageViewports[0].setRowAndColumn(0, 0);
                 surfaceMontageViewports[1].setRowAndColumn(1, 0);
@@ -386,6 +432,16 @@ SurfaceMontageConfigurationCerebral::updateSurfaceMontageViewports(std::vector<S
                 surfaceMontageViewports[6].setRowAndColumn(2, 1);
                 surfaceMontageViewports[7].setRowAndColumn(3, 1);
                 break;
+            case SurfaceMontageLayoutOrientationEnum::ROW_LAYOUT_ORIENTATION:
+                surfaceMontageViewports[0].setRowAndColumn(0, 0);
+                surfaceMontageViewports[2].setRowAndColumn(0, 1);
+                surfaceMontageViewports[1].setRowAndColumn(0, 2);
+                surfaceMontageViewports[3].setRowAndColumn(0, 3);
+                surfaceMontageViewports[4].setRowAndColumn(0, 4);
+                surfaceMontageViewports[6].setRowAndColumn(0, 5);
+                surfaceMontageViewports[5].setRowAndColumn(0, 6);
+                surfaceMontageViewports[7].setRowAndColumn(0, 7);
+                break;
         }
     }
     else if (totalNum > 0) {
