@@ -42,6 +42,7 @@
 #include "DisplayPropertiesSurface.h"
 #include "GroupAndNameHierarchyModel.h"
 #include "DisplayPropertiesLabels.h"
+#include "DisplayPropertiesSurface.h"
 #include "EventManager.h"
 #include "EventModelSurfaceGet.h"
 #include "GiftiLabel.h"
@@ -59,7 +60,6 @@
 #include "OverlaySet.h"
 #include "Palette.h"
 #include "PaletteColorMapping.h"
-#include "PaletteFile.h"
 #include "PaletteScalarAndColor.h"
 #include "RgbaFile.h"
 #include "SessionManager.h"
@@ -198,6 +198,12 @@ SurfaceNodeColoring::colorSurfaceNodes(Model* model,
         displayPropertiesLabels = brain->getDisplayPropertiesLabels();
     }
     
+    std::array<uint8_t, 3> defaultColor { 178, 178, 178 };
+    if (brain != NULL) {
+        const DisplayPropertiesSurface* dsp = brain->getDisplayPropertiesSurface();
+        defaultColor = dsp->getDefaultColorRGB();
+    }
+    
     const int numNodes = surface->getNumberOfNodes();
     const int numColorComponents = numNodes * 4;
     float *rgbaColor = new float[numColorComponents];
@@ -208,6 +214,7 @@ SurfaceNodeColoring::colorSurfaceNodes(Model* model,
     this->colorSurfaceNodes(displayPropertiesLabels,
                             browserTabIndex,
                             surface,
+                            defaultColor,
                             overlaySet, 
                             rgbaColor);
     
@@ -249,12 +256,12 @@ SurfaceNodeColoring::showBrainordinateHighlightRegionOfInterest(const Brain* bra
 {
     CaretPreferences* prefs = SessionManager::get()->getCaretPreferences();
     uint8_t foregroundColorByte[4];
-    prefs->getColorForegroundSurfaceView(foregroundColorByte);
+    prefs->getBackgroundAndForegroundColors()->getColorForegroundSurfaceView(foregroundColorByte);
     const float foregroundColor[4] = {
-        foregroundColorByte[0],
-        foregroundColorByte[1],
-        foregroundColorByte[2],
-        1.0
+        static_cast<float>(foregroundColorByte[0]) / 255.0f,
+        static_cast<float>(foregroundColorByte[1]) / 255.0f,
+        static_cast<float>(foregroundColorByte[2]) / 255.0f,
+        1.0f
     };
     
     const BrainordinateRegionOfInterest* roi = brain->getBrainordinateHighlightRegionOfInterest();
@@ -286,8 +293,14 @@ SurfaceNodeColoring::showBrainordinateHighlightRegionOfInterest(const Brain* bra
 /**
  * Assign color components to surface nodes. 
  *
+ * @param displayPropertiesLabels
+ *    Label display properties
+ * @param browserTabIndex
+ *    Index of browser tab
  * @param surface
  *    Surface that has its nodes colored.
+ * @param defaultSurfaceColor
+ *    Default coloring for surface
  * @param overlaySet
  *    Surface overlay assignments for surface.
  * @param rgbaNodeColors
@@ -297,20 +310,27 @@ void
 SurfaceNodeColoring::colorSurfaceNodes(const DisplayPropertiesLabels* displayPropertiesLabels,
                                        const int32_t browserTabIndex,
                                        const Surface* surface,
+                                       const std::array<uint8_t, 3>& defaultSurfaceColor,
                                        OverlaySet* overlaySet,
                                        float* rgbaNodeColors)
 {
     const int32_t numNodes = surface->getNumberOfNodes();
     const int32_t numberOfDisplayedOverlays = overlaySet->getNumberOfDisplayedOverlays();
     
+    const float defaultColorFloat[3] = {
+        defaultSurfaceColor[0] / 255.0f,
+        defaultSurfaceColor[1] / 255.0f,
+        defaultSurfaceColor[2] / 255.0f
+    };
+    
     /*
      * Default color.
      */
     for (int32_t i = 0; i < numNodes; i++) {
         const int32_t i4 = i * 4;
-        rgbaNodeColors[i4] = 0.70;
-        rgbaNodeColors[i4+1] = 0.70;
-        rgbaNodeColors[i4+2] = 0.70;
+        rgbaNodeColors[i4]   = defaultColorFloat[0];
+        rgbaNodeColors[i4+1] = defaultColorFloat[1];
+        rgbaNodeColors[i4+2] = defaultColorFloat[2];
         rgbaNodeColors[i4+3] = 1.0;
     }
     
@@ -340,9 +360,23 @@ SurfaceNodeColoring::colorSurfaceNodes(const DisplayPropertiesLabels* displayPro
             
             bool isColoringValid = false;
             switch (mapDataFileType) {
+                case DataFileTypeEnum::ANNOTATION:
+                    break;
+                case DataFileTypeEnum::ANNOTATION_TEXT_SUBSTITUTION:
+                    break;
                 case DataFileTypeEnum::BORDER:
                     break;
                 case DataFileTypeEnum::CONNECTIVITY_DENSE:
+                {
+                    CiftiMappableConnectivityMatrixDataFile* cmf = dynamic_cast<CiftiMappableConnectivityMatrixDataFile*>(selectedMapFile);
+                    isColoringValid = assignCiftiMappableConnectivityMatrixColoring(brainStructure,
+                                                                                    cmf,
+                                                                                    selectedMapIndex,
+                                                                                    numNodes,
+                                                                                    overlayRGBV);
+                }
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC:
                 {
                     CiftiMappableConnectivityMatrixDataFile* cmf = dynamic_cast<CiftiMappableConnectivityMatrixDataFile*>(selectedMapFile);
                     isColoringValid = assignCiftiMappableConnectivityMatrixColoring(brainStructure,
@@ -390,6 +424,8 @@ SurfaceNodeColoring::colorSurfaceNodes(const DisplayPropertiesLabels* displayPro
                     break;
                 case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_TEMPORARY:
                     break;
+                case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_MAPS:
+                    break;
                 case DataFileTypeEnum::CONNECTIVITY_PARCEL:
                 {
                     CiftiMappableConnectivityMatrixDataFile* cmf = dynamic_cast<CiftiMappableConnectivityMatrixDataFile*>(selectedMapFile);
@@ -410,6 +446,16 @@ SurfaceNodeColoring::colorSurfaceNodes(const DisplayPropertiesLabels* displayPro
                                                                             overlayRGBV);
                 }
                     break;
+                case DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC:
+                {
+                    CiftiMappableConnectivityMatrixDataFile* cmf = dynamic_cast<CiftiMappableConnectivityMatrixDataFile*>(selectedMapFile);
+                    isColoringValid = assignCiftiMappableConnectivityMatrixColoring(brainStructure,
+                                                                                    cmf,
+                                                                                    selectedMapIndex,
+                                                                                    numNodes,
+                                                                                    overlayRGBV);
+                }
+                    break;
                 case DataFileTypeEnum::CONNECTIVITY_PARCEL_LABEL:
                 {
                     CiftiParcelLabelFile* cplf = dynamic_cast<CiftiParcelLabelFile*>(selectedMapFile);
@@ -439,8 +485,12 @@ SurfaceNodeColoring::colorSurfaceNodes(const DisplayPropertiesLabels* displayPro
                     break;
                 case DataFileTypeEnum::CONNECTIVITY_SCALAR_DATA_SERIES:
                     break;
+                case DataFileTypeEnum::CZI_IMAGE_FILE:
+                    break;
                 case DataFileTypeEnum::FOCI:
                     break;
+                case DataFileTypeEnum::HISTOLOGY_SLICES:
+                    break;
                 case DataFileTypeEnum::IMAGE:
                     break;
                 case DataFileTypeEnum::LABEL:
@@ -454,12 +504,15 @@ SurfaceNodeColoring::colorSurfaceNodes(const DisplayPropertiesLabels* displayPro
                                                                 overlayRGBV);
                     break;
                 case DataFileTypeEnum::METRIC:
-                    isColoringValid = this->assignMetricColoring(brainStructure, 
+                case DataFileTypeEnum::METRIC_DYNAMIC: // same as metric
+                    isColoringValid = this->assignMetricColoring(brainStructure,
                                                                  dynamic_cast<MetricFile*>(selectedMapFile),
                                                                  selectedMapIndex,
                                                                  numNodes, 
                                                                  overlayRGBV);
                     break;
+                case DataFileTypeEnum::OME_ZARR_IMAGE_FILE:
+                    break;
                 case DataFileTypeEnum::PALETTE:
                     break;
                 case DataFileTypeEnum::RGBA:
@@ -469,6 +522,8 @@ SurfaceNodeColoring::colorSurfaceNodes(const DisplayPropertiesLabels* displayPro
                                                                numNodes, 
                                                                overlayRGBV);
                     break;
+                case DataFileTypeEnum::SAMPLES:
+                    break;
                 case DataFileTypeEnum::SCENE:
                     break;
                 case DataFileTypeEnum::SPECIFICATION:
@@ -477,10 +532,79 @@ SurfaceNodeColoring::colorSurfaceNodes(const DisplayPropertiesLabels* displayPro
                     break;
                 case DataFileTypeEnum::VOLUME:
                     break;
+                case DataFileTypeEnum::VOLUME_DYNAMIC:
+                    break;
                 case DataFileTypeEnum::UNKNOWN:
                     break;
             }
             
+            if (isColoringValid) {
+                if (selectedMapFile->isMappedWithPalette()) {
+                    const PaletteColorMapping* pcm = selectedMapFile->getMapPaletteColorMapping(selectedMapIndex);
+                    CaretAssert(pcm);
+                    bool hideDataFlag    = false;
+                    bool showOutlineFlag = false;
+                    const PaletteThresholdOutlineDrawingModeEnum::Enum outlineMode = pcm->getThresholdOutlineDrawingMode();
+                    switch (outlineMode) {
+                        case PaletteThresholdOutlineDrawingModeEnum::OFF:
+                            break;
+                        case PaletteThresholdOutlineDrawingModeEnum::OUTLINE:
+                            hideDataFlag    = true;
+                            showOutlineFlag = true;
+                            break;
+                        case PaletteThresholdOutlineDrawingModeEnum::OUTLINE_AND_DATA:
+                            showOutlineFlag = true;
+                            break;
+                    }
+                    
+                    if (showOutlineFlag) {
+                        const CaretColorEnum::Enum outlineColor = pcm->getThresholdOutlineDrawingColor();
+                        float outlineRGBA[4];
+                        CaretColorEnum::toRGBAFloat(outlineColor, outlineRGBA);
+                        
+                        CaretPointer<TopologyHelper> topologyHelper = surface->getTopologyHelper();
+                        std::vector<float> rgbaCopy(numNodes * 4);
+                        for (int32_t i = 0; i < (numNodes*4); i++) {
+                            rgbaCopy[i] = overlayRGBV[i];
+                        }
+                        for (int32_t i = 0; i < numNodes; i++) {
+                            const int32_t i4 = i * 4;
+                            CaretAssertVectorIndex(rgbaCopy, i4 + 3);
+                            const float alpha = rgbaCopy[i4 + 3];
+                            if (alpha > 0.0 ) {
+                                /*
+                                 * If a node is the same color as all of its neighbors,
+                                 * use the fill color.  Otherwise, use the outline color.
+                                 */
+                                bool isLabelBoundaryNode = false;
+                                int32_t numNeighbors = 0;
+                                const int32_t* allNeighbors = topologyHelper->getNodeNeighbors(i, numNeighbors);
+                                for (int32_t n = 0; n < numNeighbors; n++) {
+                                    const int32_t neighborNodeIndex = allNeighbors[n];
+                                    const int32_t n4 = neighborNodeIndex * 4;
+                                    CaretAssertVectorIndex(rgbaCopy, n4 + 3);
+                                    const float neighborAlpha = rgbaCopy[n4 + 3];
+                                    if (neighborAlpha <= 0.0) {
+                                        isLabelBoundaryNode = true;
+                                        break;
+                                    }
+                                }
+                                CaretAssertArrayIndex(overlayRGBV, numNodes * 4, i4 + 3);
+                                if (isLabelBoundaryNode) {
+                                    overlayRGBV[i4]   = outlineRGBA[0];
+                                    overlayRGBV[i4+1] = outlineRGBA[1];
+                                    overlayRGBV[i4+2] = outlineRGBA[2];
+                                    overlayRGBV[i4+3] = 1.0;
+                                }
+                                else if (hideDataFlag) {
+                                    overlayRGBV[i4+3] = 0.0;
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+            
             if (isColoringValid) {
                 const float opacity = overlay->getOpacity();
                 const float oneMinusOpacity = 1.0 - opacity;
@@ -677,7 +801,7 @@ SurfaceNodeColoring::assignLabelTableColors(const GiftiLabelTable* labelTable,
     
     
     float outlineRGBA[4];
-    CaretColorEnum::toRGBFloat(outlineColor, outlineRGBA);
+    CaretColorEnum::toRGBAFloat(outlineColor, outlineRGBA);
     outlineRGBA[3] = 1.0;
     
     /*
@@ -807,14 +931,51 @@ SurfaceNodeColoring::assignMetricColoring(const BrainStructure* brainStructure,
         return false;
     }
     
-    const PaletteColorMapping* paletteColorMapping = metricFile->getPaletteColorMapping(displayColumn);
+    PaletteColorMapping* paletteColorMapping = metricFile->getPaletteColorMapping(displayColumn);
+    
+    bool useThreshMapFileFlag = false;
+    switch (paletteColorMapping->getThresholdType()) {
+        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_FILE:
+            useThreshMapFileFlag = true;
+            break;
+        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_MAPPED:
+            break;
+        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_MAPPED_AVERAGE_AREA:
+            break;
+        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_NORMAL:
+            break;
+        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_OFF:
+            break;
+    }
+    
+    const float* metricDisplayData = metricFile->getValuePointerForColumn(displayColumn);
+    float* metricThresholdData = const_cast<float*>(metricDisplayData);
+    PaletteColorMapping* thresholdPaletteColorMapping = paletteColorMapping;
+    
+    if (useThreshMapFileFlag) {
+        const CaretMappableDataFileAndMapSelectionModel* threshFileModel = metricFile->getMapThresholdFileSelectionModel(displayColumn);
+        CaretAssert(threshFileModel);
+        const CaretMappableDataFile* threshMapFile = threshFileModel->getSelectedFile();
+        if (threshMapFile != NULL) {
+            const MetricFile* threshMetricFile = dynamic_cast<const MetricFile*>(threshMapFile);
+            if (threshMetricFile != NULL) {
+                const int32_t threshMapIndex = threshFileModel->getSelectedMapIndex();
+                if ((threshMapIndex >= 0)
+                    && (threshMapIndex < threshMapFile->getNumberOfMaps())) {
+                    metricThresholdData = const_cast<float*>(threshMetricFile->getValuePointerForColumn(threshMapIndex));
+                    thresholdPaletteColorMapping = const_cast<PaletteColorMapping*>(threshMapFile->getMapPaletteColorMapping(threshMapIndex));
+                    CaretAssert(thresholdPaletteColorMapping);
+                }
+            }
+        }
+    }
     
     /*
      * Invalidate all coloring.
      */
     for (int32_t i = 0; i < numberOfNodes; i++) {
         rgbv[i*4+3] = 0.0;
-    }   
+    }
     
     /*
      * Get min/max ranges.
@@ -823,8 +984,6 @@ SurfaceNodeColoring::assignMetricColoring(const BrainStructure* brainStructure,
     if (thresholdColumn < 0) {
         thresholdColumn = displayColumn;
     }
-    const float* metricDisplayData = metricFile->getValuePointerForColumn(displayColumn);
-    const float* metricThresholdData = metricFile->getValuePointerForColumn(thresholdColumn);
     
     
     FastStatistics* statistics = NULL;
@@ -838,24 +997,16 @@ SurfaceNodeColoring::assignMetricColoring(const BrainStructure* brainStructure,
     }
     CaretAssert(statistics);
     
-    //const FastStatistics* statistics = metricFile->getMapFastStatistics(displayColumn);
-    
-    const Brain* brain = brainStructure->getBrain();
-    const AString paletteName = paletteColorMapping->getSelectedPaletteName();
-    const Palette* palette = brain->getPaletteFile()->getPaletteByName(paletteName);
-    if ((statistics != NULL)
-        && (palette != NULL)) {
+    if (statistics != NULL) {
         NodeAndVoxelColoring::colorScalarsWithPalette(statistics, 
                                                       paletteColorMapping, 
-                                                      palette, 
-                                                      metricDisplayData, 
+                                                      metricDisplayData,
+                                                      thresholdPaletteColorMapping,
                                                       metricThresholdData, 
                                                       numberOfNodes, 
                                                       rgbv);
     }
-    else {
-        CaretLogSevere("Selected palette for metric is invalid: \"" + paletteName + "\"");
-    }
+    
     return true;
 }
 
@@ -900,8 +1051,7 @@ SurfaceNodeColoring::assignCiftiMappableConnectivityMatrixColoring(const BrainSt
     const StructureEnum::Enum structure = brainStructure->getStructure();
     std::vector<float> dataValues(numberOfNodes);
     
-    ciftiConnectivityMatrixFile->getMapSurfaceNodeColoring(brainStructure->getBrain()->getPaletteFile(),
-                                                           mapIndex,
+    ciftiConnectivityMatrixFile->getMapSurfaceNodeColoring(mapIndex,
                                                            structure,
                                                            rgbv,
                                                            &dataValues[0],
@@ -922,7 +1072,7 @@ SurfaceNodeColoring::assignCiftiMappableConnectivityMatrixColoring(const BrainSt
         
         if(selectedParcelValid)	{
             const CaretColorEnum::Enum parcelColor = parcelFile->getSelectedParcelColor();
-            const float* rgb = CaretColorEnum::toRGB(parcelColor);
+            const float* rgb = CaretColorEnum::toRGBA(parcelColor);
             CaretAssert(rgb);
             
             switch (parcelFile->getSelectedParcelColoringMode()) {
@@ -1045,8 +1195,7 @@ SurfaceNodeColoring::assignCiftiDenseLabelColoring(const DisplayPropertiesLabels
      * Update coloring
      */
     if (ciftiLabelFile->isMapColoringValid(mapIndex) == false) {
-        ciftiLabelFile->updateScalarColoringForMap(mapIndex,
-                                                   brain->getPaletteFile());
+        ciftiLabelFile->updateScalarColoringForMap(mapIndex);
     }
     
     std::vector<float> dataValues(numberOfNodes);
@@ -1055,8 +1204,7 @@ SurfaceNodeColoring::assignCiftiDenseLabelColoring(const DisplayPropertiesLabels
      * Assigns colors for all nodes.
      */
     const StructureEnum::Enum structure = brainStructure->getStructure();
-    ciftiLabelFile->getMapSurfaceNodeColoring(brainStructure->getBrain()->getPaletteFile(),
-                                              mapIndex,
+    ciftiLabelFile->getMapSurfaceNodeColoring(mapIndex,
                                               structure,
                                               rgbv,
                                               &dataValues[0],
@@ -1143,8 +1291,7 @@ SurfaceNodeColoring::assignCiftiParcelLabelColoring(const DisplayPropertiesLabel
      * Update coloring
      */
     if (ciftiParcelLabelFile->isMapColoringValid(mapIndex) == false) {
-        ciftiParcelLabelFile->updateScalarColoringForMap(mapIndex,
-                                                   brain->getPaletteFile());
+        ciftiParcelLabelFile->updateScalarColoringForMap(mapIndex);
     }
     
     std::vector<float> dataValues(numberOfNodes);
@@ -1153,8 +1300,7 @@ SurfaceNodeColoring::assignCiftiParcelLabelColoring(const DisplayPropertiesLabel
      * Assigns colors for all nodes.
      */
     const StructureEnum::Enum structure = brainStructure->getStructure();
-    ciftiParcelLabelFile->getMapSurfaceNodeColoring(brainStructure->getBrain()->getPaletteFile(),
-                                                    mapIndex,
+    ciftiParcelLabelFile->getMapSurfaceNodeColoring(mapIndex,
                                               structure,
                                               rgbv,
                                               &dataValues[0],
@@ -1227,14 +1373,12 @@ SurfaceNodeColoring::assignCiftiScalarColoring(const BrainStructure* brainStruct
      * Update coloring
      */
     if (ciftiScalarFile->isMapColoringValid(mapIndex) == false) {
-        ciftiScalarFile->updateScalarColoringForMap(mapIndex,
-                                                    brain->getPaletteFile());
+        ciftiScalarFile->updateScalarColoringForMap(mapIndex);
     }
     
     std::vector<float> dataValues(numberOfNodes);
     const StructureEnum::Enum structure = brainStructure->getStructure();
-    ciftiScalarFile->getMapSurfaceNodeColoring(brainStructure->getBrain()->getPaletteFile(),
-                                               mapIndex,
+    ciftiScalarFile->getMapSurfaceNodeColoring(mapIndex,
                                                structure,
                                                rgbv,
                                                &dataValues[0],
@@ -1286,14 +1430,12 @@ SurfaceNodeColoring::assignCiftiParcelScalarColoring(const BrainStructure* brain
      * Update coloring
      */
     if (ciftiParcelScalarFile->isMapColoringValid(mapIndex) == false) {
-        ciftiParcelScalarFile->updateScalarColoringForMap(mapIndex,
-                                                    brain->getPaletteFile());
+        ciftiParcelScalarFile->updateScalarColoringForMap(mapIndex);
     }
     
     std::vector<float> dataValues(numberOfNodes);
     const StructureEnum::Enum structure = brainStructure->getStructure();
-    ciftiParcelScalarFile->getMapSurfaceNodeColoring(brainStructure->getBrain()->getPaletteFile(),
-                                                     mapIndex,
+    ciftiParcelScalarFile->getMapSurfaceNodeColoring(mapIndex,
                                                structure,
                                                rgbv,
                                                &dataValues[0],
@@ -1345,8 +1487,7 @@ SurfaceNodeColoring::assignCiftiDataSeriesColoring(const BrainStructure* brainSt
      * Update coloring
      */
     if (ciftiDataSeriesFile->isMapColoringValid(mapIndex) == false) {
-        ciftiDataSeriesFile->updateScalarColoringForMap(mapIndex,
-                                                    brain->getPaletteFile());
+        ciftiDataSeriesFile->updateScalarColoringForMap(mapIndex);
     }
 
     /*
@@ -1354,8 +1495,7 @@ SurfaceNodeColoring::assignCiftiDataSeriesColoring(const BrainStructure* brainSt
      */
     std::vector<float> dataValues(numberOfNodes);
     const StructureEnum::Enum structure = brainStructure->getStructure();
-    ciftiDataSeriesFile->getMapSurfaceNodeColoring(brainStructure->getBrain()->getPaletteFile(),
-                                                   mapIndex,
+    ciftiDataSeriesFile->getMapSurfaceNodeColoring(mapIndex,
                                                structure,
                                                rgbv,
                                                &dataValues[0],
@@ -1407,8 +1547,7 @@ SurfaceNodeColoring::assignCiftiParcelSeriesColoring(const BrainStructure* brain
      * Update coloring
      */
     if (ciftiParcelSeriesFile->isMapColoringValid(mapIndex) == false) {
-        ciftiParcelSeriesFile->updateScalarColoringForMap(mapIndex,
-                                                        brain->getPaletteFile());
+        ciftiParcelSeriesFile->updateScalarColoringForMap(mapIndex);
     }
     
     /*
@@ -1416,8 +1555,7 @@ SurfaceNodeColoring::assignCiftiParcelSeriesColoring(const BrainStructure* brain
      */
     std::vector<float> dataValues(numberOfNodes);
     const StructureEnum::Enum structure = brainStructure->getStructure();
-    ciftiParcelSeriesFile->getMapSurfaceNodeColoring(brainStructure->getBrain()->getPaletteFile(),
-                                                     mapIndex,
+    ciftiParcelSeriesFile->getMapSurfaceNodeColoring(mapIndex,
                                                    structure,
                                                    rgbv,
                                                    &dataValues[0],
@@ -1444,19 +1582,24 @@ SurfaceNodeColoring::assignCiftiParcelSeriesColoring(const BrainStructure* brain
  *    True if coloring is valid, else false.
  */
 bool 
-SurfaceNodeColoring::assignRgbaColoring(const BrainStructure* /*brainStructure*/, 
-                                        const RgbaFile* /*rgbaFile*/,
+SurfaceNodeColoring::assignRgbaColoring(const BrainStructure* brainStructure,
+                                        const RgbaFile* rgbaFile,
                                         const int32_t /*mapIndex*/,
                                         const int32_t numberOfNodes,
                                         float* rgbv)
 {
-    CaretAssertMessage(0, "Add implementation.");
+    if ( ! rgbaFile->isMappableToSurfaceStructure(brainStructure->getStructure())) {
+        return false;
+    }
+    
+    if (rgbaFile->isEmpty()) {
+        return false;
+    }
+    
     for (int32_t i = 0; i < numberOfNodes; i++) {
         const int32_t i4 = i * 4;
-        rgbv[i4]   = 0.0;
-        rgbv[i4+1] = 0.0;
-        rgbv[i4+2] = 1.0;
-        rgbv[i4+3] = 1.0;
+        
+        rgbaFile->getVertexRGBA(i, &rgbv[i4]);
     }
     
     return true;
