@@ -64,7 +64,7 @@ VolumeSurfaceOutlineSetModel::VolumeSurfaceOutlineSetModel()
     for (int32_t i = 0; i < BrainConstants::MAXIMUM_NUMBER_OF_VOLUME_SURFACE_OUTLINES; i++) {
         m_outlineModels[i] = new VolumeSurfaceOutlineModel();
     }
-    m_numberOfDisplayedVolumeSurfaceOutlines = 6; //BrainConstants::MINIMUM_NUMBER_OF_VOLUME_SURFACE_OUTLINES;
+    m_numberOfDisplayedVolumeSurfaceOutlines = 6; 
     
     m_sceneAssistant = new SceneClassAssistant();
     m_sceneAssistant->add("m_numberOfDisplayedVolumeSurfaceOutlines",
@@ -167,6 +167,8 @@ VolumeSurfaceOutlineSetModel::selectSurfacesAfterSpecFileLoaded(Brain* brain,
     EventBrowserTabGetAll getAllTabs;
     EventManager::get()->sendEvent(getAllTabs.getPointer());
     
+    m_numberOfDisplayedVolumeSurfaceOutlines = 6;
+
     /*
      * Find tabs with left/right
      */
@@ -231,45 +233,137 @@ VolumeSurfaceOutlineSetModel::selectSurfacesAfterSpecFileLoaded(Brain* brain,
         rightPialSurface     = rightBrainStructure->getSurfaceContainingTextInName("pial");
     }
     
+    std::pair<Surface*,Surface*> leftHippoSurfaces(getHippoInnerOuterSurfaces(brain,
+                                                                              StructureEnum::HIPPOCAMPUS_LEFT));
+    Surface* leftHippoOuterSurface(leftHippoSurfaces.first);
+    Surface* leftHippoInnerSurface(leftHippoSurfaces.second);
+
+    std::pair<Surface*,Surface*> rightHippoSurfaces(getHippoInnerOuterSurfaces(brain,
+                                                                               StructureEnum::HIPPOCAMPUS_RIGHT));
+    Surface* rightHippoOuterSurface(rightHippoSurfaces.first);
+    Surface* rightHippoInnerSurface(rightHippoSurfaces.second);
+    
+    std::pair<Surface*,Surface*> leftDentateHippoSurfaces(getHippoInnerOuterSurfaces(brain,
+                                                                                     StructureEnum::HIPPOCAMPUS_DENTATE_LEFT));
+    Surface* leftDentateHippoOuterSurface(leftDentateHippoSurfaces.first);
+    Surface* leftDentateHippoInnerSurface(leftDentateHippoSurfaces.second);
+    
+    std::pair<Surface*,Surface*> rightDentateHippoSurfaces(getHippoInnerOuterSurfaces(brain,
+                                                                                      StructureEnum::HIPPOCAMPUS_DENTATE_RIGHT));
+    Surface* rightDentateHippoOuterSurface(rightDentateHippoSurfaces.first);
+    Surface* rightDentateHippoInnerSurface(rightDentateHippoSurfaces.second);
+    
     for (int32_t i = 0; i < BrainConstants::MAXIMUM_NUMBER_OF_VOLUME_SURFACE_OUTLINES; i++) {
         m_outlineModels[i]->getColorOrTabModel()->setColor(CaretColorEnum::BLACK);
-        m_outlineModels[i]->setThickness(VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS);
+        m_outlineModels[i]->setThicknessPixelsObsolete(VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS_PIXELS_OBSOLETE);
     }
     
     int nextOutlineIndex = 0;
     
     addSurfaceOutline(leftMidThickSurface,
-                      VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS,
                       leftTabIndex,
                       CaretColorEnum::BLACK,
                       nextOutlineIndex);
     addSurfaceOutline(rightMidThickSurface,
-                      VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS,
                       rightTabIndex,
                       CaretColorEnum::BLACK,
                       nextOutlineIndex);
     
     addSurfaceOutline(leftWhiteSurface,
-                      VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS,
                       -1,
                       CaretColorEnum::LIME,
                       nextOutlineIndex);
     addSurfaceOutline(rightWhiteSurface,
-                      VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS,
                       -1,
                       CaretColorEnum::LIME,
                       nextOutlineIndex);
     
     addSurfaceOutline(leftPialSurface,
-                      VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS,
                       -1,
                       CaretColorEnum::BLUE,
                       nextOutlineIndex);
     addSurfaceOutline(rightPialSurface,
-                      VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS,
                       -1,
                       CaretColorEnum::BLUE,
                       nextOutlineIndex);
+    
+    
+    addSurfaceOutline(leftHippoOuterSurface,
+                      -1,
+                      CaretColorEnum::BLUE,
+                      nextOutlineIndex);
+    addSurfaceOutline(rightHippoOuterSurface,
+                      -1,
+                      CaretColorEnum::BLUE,
+                      nextOutlineIndex);
+    
+    addSurfaceOutline(leftHippoInnerSurface,
+                      -1,
+                      CaretColorEnum::LIME,
+                      nextOutlineIndex);
+    addSurfaceOutline(rightHippoInnerSurface,
+                      -1,
+                      CaretColorEnum::LIME,
+                      nextOutlineIndex);
+    
+    
+    addSurfaceOutline(leftDentateHippoOuterSurface,
+                      -1,
+                      CaretColorEnum::BLUE,
+                      nextOutlineIndex);
+    addSurfaceOutline(rightDentateHippoOuterSurface,
+                      -1,
+                      CaretColorEnum::BLUE,
+                      nextOutlineIndex);
+    
+    addSurfaceOutline(leftDentateHippoInnerSurface,
+                      -1,
+                      CaretColorEnum::LIME,
+                      nextOutlineIndex);
+    addSurfaceOutline(rightDentateHippoInnerSurface,
+                      -1,
+                      CaretColorEnum::LIME,
+                      nextOutlineIndex);
+}
+
+/**
+ * @return Pair containing the inner and outer surfaces
+ * Find hippocampus surfaces for volume surface outline
+ * @param brain
+ *    The brain
+ * @param structure
+ *    The structure
+ */
+std::pair<Surface*,Surface*>
+VolumeSurfaceOutlineSetModel::getHippoInnerOuterSurfaces(Brain* brain,
+                                                         const StructureEnum::Enum structure)
+{
+    Surface* innerSurface(NULL);
+    Surface* outerSurface(NULL);
+    
+    BrainStructure* brainStructure(brain->getBrainStructure(structure,
+                                                            false));
+    if (brainStructure != NULL) {
+        outerSurface = brainStructure->getSurfaceOfTypeAndSecondaryType(SurfaceTypeEnum::ANATOMICAL,
+                                                                        SecondarySurfaceTypeEnum::OUTER);
+        innerSurface = brainStructure->getSurfaceOfTypeAndSecondaryType(SurfaceTypeEnum::ANATOMICAL,
+                                                                        SecondarySurfaceTypeEnum::INNER);
+        if (outerSurface == NULL) {
+            outerSurface = brainStructure->getSurfaceContainingTextInName("outer");
+        }
+        if (innerSurface == NULL) {
+            innerSurface = brainStructure->getSurfaceContainingTextInName("inner");
+        }
+        if (outerSurface == NULL) {
+            outerSurface = brainStructure->getPrimaryAnatomicalSurface();
+        }
+        if (innerSurface == NULL) {
+            innerSurface = brainStructure->getPrimaryAnatomicalSurface();
+        }
+    }
+    
+    return std::make_pair(innerSurface,
+                          outerSurface);
 }
 
 /**
@@ -278,8 +372,6 @@ VolumeSurfaceOutlineSetModel::selectSurfacesAfterSpecFileLoaded(Brain* brain,
  *
  * @param surface
  *    Surface that is added.  If NULL, no action is taken.
- * @param thickness
- *    Thickness for surface outline.
  * @param browserTabIndex
  *    If greater than or equal to zero, the color source
  *    is set to this tab index.
@@ -294,7 +386,6 @@ VolumeSurfaceOutlineSetModel::selectSurfacesAfterSpecFileLoaded(Brain* brain,
  */
 void 
 VolumeSurfaceOutlineSetModel::addSurfaceOutline(Surface* surface,
-                                           const float thickness,
                                            const int32_t browserTabIndex,
                                            const CaretColorEnum::Enum color,
                                            int32_t& outlineIndex)
@@ -304,7 +395,8 @@ VolumeSurfaceOutlineSetModel::addSurfaceOutline(Surface* surface,
             if (outlineIndex < BrainConstants::MAXIMUM_NUMBER_OF_VOLUME_SURFACE_OUTLINES) {
                 VolumeSurfaceOutlineModel* vsos = m_outlineModels[outlineIndex];
                 vsos->getSurfaceSelectionModel()->setSurface(surface);
-                vsos->setThickness(thickness);
+                vsos->setThicknessPixelsObsolete(VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS_PIXELS_OBSOLETE);
+                vsos->setThicknessPercentageViewportHeight(VolumeSurfaceOutlineModel::DEFAULT_LINE_THICKNESS_PERCENTAGE_VIEWPORT_HEIGHT);
                 if (browserTabIndex >= 0) {
                     vsos->getColorOrTabModel()->setBrowserTabIndex(browserTabIndex);
                 }
@@ -313,6 +405,13 @@ VolumeSurfaceOutlineSetModel::addSurfaceOutline(Surface* surface,
                 }
                 
                 outlineIndex++;
+                
+                /*
+                 * Ensure that outline is displayed
+                 */
+                if (m_numberOfDisplayedVolumeSurfaceOutlines < outlineIndex) {
+                    m_numberOfDisplayedVolumeSurfaceOutlines = outlineIndex;
+                }
             }
         }
     }
@@ -336,7 +435,7 @@ VolumeSurfaceOutlineSetModel::saveToScene(const SceneAttributes* sceneAttributes
 {
     SceneClass* sceneClass = new SceneClass(instanceName,
                                             "VolumeSurfaceOutlineSetModel",
-                                            1);
+                                            2);
     m_sceneAssistant->saveMembers(sceneAttributes, 
                                   sceneClass);
     
@@ -371,6 +470,8 @@ VolumeSurfaceOutlineSetModel::restoreFromScene(const SceneAttributes* sceneAttri
         return;
     }
     
+    const int32_t version = sceneClass->getVersionNumber();
+    
     m_sceneAssistant->restoreMembers(sceneAttributes, 
                                      sceneClass);
     
@@ -382,5 +483,19 @@ VolumeSurfaceOutlineSetModel::restoreFromScene(const SceneAttributes* sceneAttri
             m_outlineModels[i]->restoreFromScene(sceneAttributes,
                                                  outlineModelsArrayClass->getClassAtIndex(i));
         }
+        
+        /*
+         * In older scenes, last was drawn on top.
+         * Now, zero is drawn on top so need to reverse
+         * the order of the volume surface outlines
+         */
+        if (version < 2) {
+            if (maxNum > 1) {
+                const int32_t numDisplayed = getNumberOfDislayedVolumeSurfaceOutlines();
+                CaretAssertArrayIndex(m_outlineModels, BrainConstants::MAXIMUM_NUMBER_OF_VOLUME_SURFACE_OUTLINES, numDisplayed - 1);
+                std::reverse(m_outlineModels,
+                             m_outlineModels + numDisplayed);
+            }
+        }
     }
 }
