@@ -23,7 +23,7 @@
 
 #include "BrainConstants.h"
 #include "CaretObject.h"
-#include "ChartDataTypeEnum.h"
+#include "ChartOneDataTypeEnum.h"
 #include "SceneableInterface.h"
 
 
@@ -35,7 +35,7 @@ namespace caret {
     class ChartData : public CaretObject, public SceneableInterface {
         
     public:
-        static ChartData* newChartDataForChartDataType(const ChartDataTypeEnum::Enum chartDataType);
+        static ChartData* newChartDataForChartDataType(const ChartOneDataTypeEnum::Enum chartDataType);
         
         virtual ~ChartData();
         
@@ -50,7 +50,7 @@ namespace caret {
          */
         virtual ChartData* clone() const = 0;
         
-        ChartDataTypeEnum::Enum getChartDataType() const;
+        ChartOneDataTypeEnum::Enum getChartDataType() const;
         
         const ChartDataSource* getChartDataSource() const;
         
@@ -67,6 +67,8 @@ namespace caret {
         
         void setUniqueIdentifier(const AString& uniqueIdentifier);
         
+        void copySelectionStatusForAllTabs(const ChartData* copyFrom);
+        
         virtual SceneClass* saveToScene(const SceneAttributes* sceneAttributes,
                                         const AString& instanceName);
         
@@ -78,7 +80,7 @@ namespace caret {
         virtual AString toString() const;
         
     protected:
-        ChartData(const ChartDataTypeEnum::Enum chartDataType);
+        ChartData(const ChartOneDataTypeEnum::Enum chartDataType);
         
         ChartData(const ChartData& obj);
         
@@ -122,7 +124,7 @@ namespace caret {
 
         SceneClassAssistant* m_sceneAssistant;
 
-        ChartDataTypeEnum::Enum m_chartDataType;
+        ChartOneDataTypeEnum::Enum m_chartDataType;
         
         ChartDataSource* m_chartDataSource;
         
