@@ -21,7 +21,7 @@
 #include "CiftiXML.h"
 
 #include "CaretAssert.h"
-#include "CaretException.h"
+#include "DataFileException.h"
 #include "CaretLogger.h"
 #include "GiftiMetaData.h"
 #include "PaletteColorMapping.h"
@@ -33,6 +33,25 @@
 using namespace std;
 using namespace caret;
 
+int CiftiXML::directionFromString(const QString& input)
+{
+    bool ok = false;
+    int converted = input.toInt(&ok);
+    if (ok)
+    {
+        if (converted < 1) throw DataFileException("invalid integer direction, use 1 or greater");
+        return converted - 1;//use 1-indexed convention for input
+    }
+    if (input == "ROW") return ALONG_ROW;
+    if (input == "COLUMN") return ALONG_COLUMN;//should we also allow STACK?  integers seem cleaner
+    throw DataFileException("unrecognized direction string, please use an integer, 'ROW', or 'COLUMN'");
+}
+
+QString CiftiXML::directionFromStringExplanation()
+{
+    return "The direction can be either an integer starting from 1, or the strings 'ROW' or 'COLUMN'.";
+}
+
 CiftiXML::CiftiXML(const CiftiXML& rhs)
 {
     copyHelper(rhs);
@@ -226,6 +245,16 @@ CiftiMappingType::MappingType CiftiXML::getMappingType(const int& direction) con
 void CiftiXML::setMap(const int& direction, const CiftiMappingType& mapIn)
 {
     CaretAssertVectorIndex(m_indexMaps, direction);
+    if (mapIn.getType() == CiftiMappingType::LABELS)
+    {
+        for (int i = 0; i < getNumberOfDimensions(); ++i)
+        {
+            if (i != direction && m_indexMaps[i] != NULL && m_indexMaps[i]->getType() == CiftiMappingType::LABELS)
+            {
+                throw DataFileException("Cifti XML cannot contain a label mapping on more than one dimension");
+            }
+        }
+    }
     m_indexMaps[direction] = CaretPointer<CiftiMappingType>(mapIn.clone());
 }
 
@@ -294,7 +323,7 @@ int32_t CiftiXML::getIntentInfo(const CiftiVersion& writingVersion, char intentN
                 break;
         }
     } else {
-        throw CaretException("unknown cifti version: " + writingVersion.toString());
+        throw DataFileException("unknown cifti version: " + writingVersion.toString());
     }
     int i;
     for (i = 0; i < 16 && name[i] != '\0'; ++i) intentNameOut[i] = name[i];
@@ -312,19 +341,19 @@ void CiftiXML::readXML(QXmlStreamReader& xml)
         {
             if (xml.isStartElement())
             {
-                QStringRef name = xml.name();
-                if (name == "CIFTI")
+                auto name = xml.name();
+                if (name == QLatin1String("CIFTI"))
                 {
                     if (haveCifti)
                     {
-                        throw CaretException("CIFTI element may only be specified once");
+                        throw DataFileException("CIFTI element may only be specified once");
                     }
                     QXmlStreamAttributes attributes = xml.attributes();
-                    if(attributes.hasAttribute("Version"))
+                    if (attributes.hasAttribute("Version"))
                     {
                         m_parsedVersion = CiftiVersion(attributes.value("Version").toString());
                     } else {
-                        throw CaretException("Cifti XML missing Version attribute.");
+                        throw DataFileException("Cifti XML missing Version attribute.");
                     }
                     if (m_parsedVersion == CiftiVersion(1, 0))//switch/case on major/minor would be much harder to read
                     {
@@ -338,24 +367,41 @@ void CiftiXML::readXML(QXmlStreamReader& xml)
                         parseCIFTI2(xml);
                         if (xml.hasError()) break;
                     } else {
-                        throw CaretException("unknown Cifti Version: '" + m_parsedVersion.toString());
+                        throw DataFileException("unknown Cifti Version: '" + m_parsedVersion.toString());
                     }
                     haveCifti = true;
                 } else {
-                    throw CaretException("unexpected root element in Cifti XML: " + name.toString());
+                    throw DataFileException("unexpected root element in Cifti XML: " + name.toString());
                 }
             }
+#if QT_VERSION >= 0x060000
+            /*
+             * In Qt 6, calling xml.readNext() after reading the CIFTI end element
+             * results in a TokenType of Invalid and hasError() is set to true.
+             * The expection is that atEnd() would get set but that does not
+             * happen.
+             */
+            if (xml.isEndElement())
+            {
+                if ( ! xml.hasError()) {
+                    if (xml.name() == QLatin1String("CIFTI"))
+                    {
+                        break;
+                    }
+                }
+            }
+#endif
         }
         if (!xml.hasError() && !haveCifti)
         {
-            throw CaretException("CIFTI element not found");
+            throw DataFileException("CIFTI element not found");
         }
     } catch (CaretException& e) {
-        throw CaretException("Cifti XML error: " + e.whatString());//so we can throw on error instead of doing a bunch of dancing with xml.raiseError and xml.hasError
+        throw DataFileException("Cifti XML error: " + e.whatString());//so we can throw on error instead of doing a bunch of dancing with xml.raiseError and xml.hasError
     }
     if(xml.hasError())
     {
-        throw CaretException("Cifti XML error: " + xml.errorString());
+        throw DataFileException("Cifti XML error: " + xml.errorString());
     }
 }
 
@@ -364,12 +410,12 @@ void CiftiXML::parseCIFTI1(QXmlStreamReader& xml)
     QXmlStreamAttributes attributes = xml.attributes();
     if (attributes.hasAttribute("NumberOfMatrices"))
     {
-        if (attributes.value("NumberOfMatrices") != "1")
+        if (attributes.value("NumberOfMatrices") != QLatin1String("1"))
         {
-            throw CaretException("attribute NumberOfMatrices in CIFTI is required to be 1 for CIFTI-1");
+            throw DataFileException("attribute NumberOfMatrices in CIFTI is required to be 1 for CIFTI-1");
         }
     } else {
-        throw CaretException("missing attribute NumberOfMatrices in CIFTI");
+        throw DataFileException("missing attribute NumberOfMatrices in CIFTI");
     }
     bool haveMatrix = false;
     while (!xml.atEnd())
@@ -377,18 +423,18 @@ void CiftiXML::parseCIFTI1(QXmlStreamReader& xml)
         xml.readNext();
         if (xml.isStartElement())
         {
-            QStringRef name = xml.name();
-            if (name == "Matrix")
+            auto name = xml.name();
+            if (name == QLatin1String("Matrix"))
             {
                 if (haveMatrix)
                 {
-                    throw CaretException("Matrix element may only be specified once");
+                    throw DataFileException("Matrix element may only be specified once");
                 }
                 parseMatrix1(xml);
                 if (xml.hasError()) return;
                 haveMatrix = true;
             } else {
-                throw CaretException("unexpected element in CIFTI: " + name.toString());
+                throw DataFileException("unexpected element in CIFTI: " + name.toString());
             }
         } else if (xml.isEndElement()) {
             break;
@@ -396,10 +442,10 @@ void CiftiXML::parseCIFTI1(QXmlStreamReader& xml)
     }
     if (!haveMatrix)
     {
-        throw CaretException("Matrix element not found in CIFTI");
+        throw DataFileException("Matrix element not found in CIFTI");
     }
     if (xml.hasError()) return;
-    CaretAssert(xml.isEndElement() && xml.name() == "CIFTI");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("CIFTI"));
 }
 
 void CiftiXML::parseCIFTI2(QXmlStreamReader& xml)//yes, these will often have largely similar code, but it seems cleaner than having only some functions split, or constantly rechecking the version
@@ -411,18 +457,18 @@ void CiftiXML::parseCIFTI2(QXmlStreamReader& xml)//yes, these will often have la
         if (xml.hasError()) return;
         if (xml.isStartElement())
         {
-            QStringRef name = xml.name();
-            if (name == "Matrix")
+            auto name = xml.name();
+            if (name == QLatin1String("Matrix"))
             {
                 if (haveMatrix)
                 {
-                    throw CaretException("Matrix element may only be specified once");
+                    throw DataFileException("Matrix element may only be specified once");
                 }
                 parseMatrix2(xml);
                 if (xml.hasError()) return;
                 haveMatrix = true;
             } else {
-                throw CaretException("unexpected element in CIFTI: " + name.toString());
+                throw DataFileException("unexpected element in CIFTI: " + name.toString());
             }
         } else if (xml.isEndElement()) {
             break;
@@ -430,9 +476,9 @@ void CiftiXML::parseCIFTI2(QXmlStreamReader& xml)//yes, these will often have la
     }
     if (!haveMatrix)
     {
-        throw CaretException("Matrix element not found in CIFTI");
+        throw DataFileException("Matrix element not found in CIFTI");
     }
-    CaretAssert(xml.isEndElement() && xml.name() == "CIFTI");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("CIFTI"));
 }
 
 void CiftiXML::parseMatrix1(QXmlStreamReader& xml)
@@ -445,32 +491,32 @@ void CiftiXML::parseMatrix1(QXmlStreamReader& xml)
         if (xml.hasError()) return;
         if (xml.isStartElement())
         {
-            QStringRef name = xml.name();
-            if (name == "MetaData")
+            auto name = xml.name();
+            if (name == QLatin1String("MetaData"))
             {
                 if (haveMetadata)
                 {
-                    throw CaretException("MetaData may only be specified once in Matrix");
+                    throw DataFileException("MetaData may only be specified once in Matrix");
                 }
                 m_fileMetaData.readCiftiXML1(xml);
                 if (xml.hasError()) return;
                 haveMetadata = true;
-            } else if (name == "MatrixIndicesMap") {
+            } else if (name == QLatin1String("MatrixIndicesMap")) {
                 parseMatrixIndicesMap1(xml);
                 if (xml.hasError()) return;
-            } else if (name == "Volume") {
+            } else if (name == QLatin1String("Volume")) {
                 if (haveVolSpace)
                 {
-                    throw CaretException("Volume may only be specified once in Matrix");
+                    throw DataFileException("Volume may only be specified once in Matrix");
                 }
                 fileVolSpace.readCiftiXML1(xml);
                 if (xml.hasError()) return;
                 haveVolSpace = true;
-            } else if (name == "LabelTable") {
+            } else if (name == QLatin1String("LabelTable")) {
                 CaretLogFiner("skipping unused LabelTable element in Matrix in CIFTI-1");
                 xml.readElementText(QXmlStreamReader::SkipChildElements);
             } else {
-                throw CaretException("unexpected element in Matrix: " + name.toString());
+                throw DataFileException("unexpected element in Matrix: " + name.toString());
             }
         } else if (xml.isEndElement()) {
             break;
@@ -482,7 +528,7 @@ void CiftiXML::parseMatrix1(QXmlStreamReader& xml)
         {
             int displaynum = i;
             if (displaynum < 2) displaynum = 1 - displaynum;//re-invert so that it shows the same number as the XML is missing
-            throw CaretException("missing mapping for dimension '" + QString::number(displaynum) + "'");
+            throw DataFileException("missing mapping for dimension '" + QString::number(displaynum) + "'");
         }
         switch (m_indexMaps[i]->getType())
         {
@@ -495,7 +541,7 @@ void CiftiXML::parseMatrix1(QXmlStreamReader& xml)
                     {
                         myMap.setVolumeSpace(fileVolSpace);//also does the needed checking of voxel indices
                     } else {
-                        throw CaretException("BrainModels map uses voxels, but no Volume element exists");
+                        throw DataFileException("BrainModels map uses voxels, but no Volume element exists");
                     }
                 }
                 break;
@@ -509,7 +555,7 @@ void CiftiXML::parseMatrix1(QXmlStreamReader& xml)
                     {
                         myMap.setVolumeSpace(fileVolSpace);//ditto
                     } else {
-                        throw CaretException("Parcels map uses voxels, but no Volume element exists");
+                        throw DataFileException("Parcels map uses voxels, but no Volume element exists");
                     }
                 }
                 break;
@@ -518,7 +564,7 @@ void CiftiXML::parseMatrix1(QXmlStreamReader& xml)
                 break;
         }
     }
-    CaretAssert(xml.isEndElement() && xml.name() == "Matrix");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("Matrix"));
 }
 
 void CiftiXML::parseMatrix2(QXmlStreamReader& xml)
@@ -530,21 +576,21 @@ void CiftiXML::parseMatrix2(QXmlStreamReader& xml)
         if (xml.hasError()) return;
         if (xml.isStartElement())
         {
-            QStringRef name = xml.name();
-            if (name == "MetaData")
+            auto name = xml.name();
+            if (name == QLatin1String("MetaData"))
             {
                 if (haveMetadata)
                 {
-                    throw CaretException("MetaData may only be specified once in Matrix");
+                    throw DataFileException("MetaData may only be specified once in Matrix");
                 }
                 m_fileMetaData.readCiftiXML2(xml);
                 if (xml.hasError()) return;
                 haveMetadata = true;
-            } else if (name == "MatrixIndicesMap") {
+            } else if (name == QLatin1String("MatrixIndicesMap")) {
                 parseMatrixIndicesMap2(xml);
                 if (xml.hasError()) return;
             } else {
-                throw CaretException("unexpected element in Matrix: " + name.toString());
+                throw DataFileException("unexpected element in Matrix: " + name.toString());
             }
         } else if (xml.isEndElement()) {
             break;
@@ -554,10 +600,10 @@ void CiftiXML::parseMatrix2(QXmlStreamReader& xml)
     {
         if (m_indexMaps[i] == NULL)
         {
-            throw CaretException("missing mapping for dimension '" + QString::number(i) + "'");
+            throw DataFileException("missing mapping for dimension '" + QString::number(i) + "'");
         }
     }
-    CaretAssert(xml.isEndElement() && xml.name() == "Matrix");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("Matrix"));
 }
 
 void CiftiXML::parseMatrixIndicesMap1(QXmlStreamReader& xml)
@@ -565,11 +611,11 @@ void CiftiXML::parseMatrixIndicesMap1(QXmlStreamReader& xml)
     QXmlStreamAttributes attributes = xml.attributes();
     if (!attributes.hasAttribute("AppliesToMatrixDimension"))
     {
-        throw CaretException("missing attribute AppliesToMatrixDimension in MatrixIndicesMap");
+        throw DataFileException("missing attribute AppliesToMatrixDimension in MatrixIndicesMap");
     }
     if (!attributes.hasAttribute("IndicesMapToDataType"))
     {
-        throw CaretException("missing attribute IndicesMapToDataType in MatrixIndicesMap");
+        throw DataFileException("missing attribute IndicesMapToDataType in MatrixIndicesMap");
     }
     QStringList values = attributes.value("AppliesToMatrixDimension").toString().split(',');
     bool ok = false;
@@ -579,30 +625,30 @@ void CiftiXML::parseMatrixIndicesMap1(QXmlStreamReader& xml)
         int parsed = values[i].toInt(&ok);
         if (!ok || parsed < 0)
         {
-            throw CaretException("bad value in AppliesToMatrixDimension list: " + values[i]);
+            throw DataFileException("bad value in AppliesToMatrixDimension list: " + values[i]);
         }
         if (parsed < 2) parsed = 1 - parsed;//in other words, 0 becomes 1 and 1 becomes 0, since cifti-1 had them reversed
         if (used.find(parsed) != used.end())
         {
-            throw CaretException("AppliesToMatrixDimension contains repeated value: " + values[i]);
+            throw DataFileException("AppliesToMatrixDimension contains repeated value: " + values[i]);
         }
         used.insert(parsed);
     }
     CaretPointer<CiftiMappingType> toRead;
-    QStringRef type = attributes.value("IndicesMapToDataType");
-    if (type == "CIFTI_INDEX_TYPE_BRAIN_MODELS")
+    auto type = attributes.value("IndicesMapToDataType");
+    if (type == QLatin1String("CIFTI_INDEX_TYPE_BRAIN_MODELS"))
     {
         toRead = CaretPointer<CiftiBrainModelsMap>(new CiftiBrainModelsMap());
-    } else if (type == "CIFTI_INDEX_TYPE_TIME_POINTS") {
+    } else if (type == QLatin1String("CIFTI_INDEX_TYPE_TIME_POINTS")) {
         toRead = CaretPointer<CiftiSeriesMap>(new CiftiSeriesMap());
-    } else if (type == "CIFTI_INDEX_TYPE_LABELS") {//this and below are nonstandard
+    } else if (type == QLatin1String("CIFTI_INDEX_TYPE_LABELS")) {//this and below are nonstandard
         toRead = CaretPointer<CiftiLabelsMap>(new CiftiLabelsMap());
-    } else if (type == "CIFTI_INDEX_TYPE_PARCELS") {
+    } else if (type == QLatin1String("CIFTI_INDEX_TYPE_PARCELS")) {
         toRead = CaretPointer<CiftiParcelsMap>(new CiftiParcelsMap());
-    } else if (type == "CIFTI_INDEX_TYPE_SCALARS") {
+    } else if (type == QLatin1String("CIFTI_INDEX_TYPE_SCALARS")) {
         toRead = CaretPointer<CiftiScalarsMap>(new CiftiScalarsMap());
     } else {
-        throw CaretException("invalid value for IndicesMapToDataType in CIFTI-1: " + type.toString());
+        throw DataFileException("invalid value for IndicesMapToDataType in CIFTI-1: " + type.toString());
     }
     toRead->readXML1(xml);//this will warn (with 'finer' log level?) if it is nonstandard
     if (xml.hasError()) return;
@@ -618,7 +664,7 @@ void CiftiXML::parseMatrixIndicesMap1(QXmlStreamReader& xml)
             m_indexMaps[*iter] = CaretPointer<CiftiMappingType>(toRead->clone());//make in-memory information independent per-dimension, rather than dealing with deduplication everywhere
         }
     }
-    CaretAssert(xml.isEndElement() && xml.name() == "MatrixIndicesMap");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("MatrixIndicesMap"));
 }
 
 void CiftiXML::parseMatrixIndicesMap2(QXmlStreamReader& xml)
@@ -626,11 +672,11 @@ void CiftiXML::parseMatrixIndicesMap2(QXmlStreamReader& xml)
     QXmlStreamAttributes attributes = xml.attributes();
     if (!attributes.hasAttribute("AppliesToMatrixDimension"))
     {
-        throw CaretException("missing attribute AppliesToMatrixDimension in MatrixIndicesMap");
+        throw DataFileException("missing attribute AppliesToMatrixDimension in MatrixIndicesMap");
     }
     if (!attributes.hasAttribute("IndicesMapToDataType"))
     {
-        throw CaretException("missing attribute IndicesMapToDataType in MatrixIndicesMap");
+        throw DataFileException("missing attribute IndicesMapToDataType in MatrixIndicesMap");
     }
     QStringList values = attributes.value("AppliesToMatrixDimension").toString().split(',');
     bool ok = false;
@@ -640,29 +686,29 @@ void CiftiXML::parseMatrixIndicesMap2(QXmlStreamReader& xml)
         int parsed = values[i].toInt(&ok);
         if (!ok || parsed < 0)
         {
-            throw CaretException("bad value in AppliesToMatrixDimension list: " + values[i]);
+            throw DataFileException("bad value in AppliesToMatrixDimension list: " + values[i]);
         }
         if (used.find(parsed) != used.end())
         {
-            throw CaretException("AppliesToMatrixDimension contains repeated value: " + values[i]);
+            throw DataFileException("AppliesToMatrixDimension contains repeated value: " + values[i]);
         }
         used.insert(parsed);
     }
     CaretPointer<CiftiMappingType> toRead;
-    QStringRef type = attributes.value("IndicesMapToDataType");
-    if (type == "CIFTI_INDEX_TYPE_BRAIN_MODELS")
+    auto type = attributes.value("IndicesMapToDataType");
+    if (type == QLatin1String("CIFTI_INDEX_TYPE_BRAIN_MODELS"))
     {
         toRead = CaretPointer<CiftiBrainModelsMap>(new CiftiBrainModelsMap());
-    } else if (type == "CIFTI_INDEX_TYPE_LABELS") {
+    } else if (type == QLatin1String("CIFTI_INDEX_TYPE_LABELS")) {
         toRead = CaretPointer<CiftiLabelsMap>(new CiftiLabelsMap());
-    } else if (type == "CIFTI_INDEX_TYPE_PARCELS") {
+    } else if (type == QLatin1String("CIFTI_INDEX_TYPE_PARCELS")) {
         toRead = CaretPointer<CiftiParcelsMap>(new CiftiParcelsMap());
-    } else if (type == "CIFTI_INDEX_TYPE_SCALARS") {
+    } else if (type == QLatin1String("CIFTI_INDEX_TYPE_SCALARS")) {
         toRead = CaretPointer<CiftiScalarsMap>(new CiftiScalarsMap());
-    } else if (type == "CIFTI_INDEX_TYPE_SERIES") {
+    } else if (type == QLatin1String("CIFTI_INDEX_TYPE_SERIES")) {
         toRead = CaretPointer<CiftiSeriesMap>(new CiftiSeriesMap());
     } else {
-        throw CaretException("invalid value for IndicesMapToDataType in CIFTI-1: " + type.toString());
+        throw DataFileException("invalid value for IndicesMapToDataType in CIFTI-2: " + type.toString());
     }
     toRead->readXML2(xml);
     if (xml.hasError()) return;
@@ -678,7 +724,7 @@ void CiftiXML::parseMatrixIndicesMap2(QXmlStreamReader& xml)
             m_indexMaps[*iter] = CaretPointer<CiftiMappingType>(toRead->clone());//make in-memory information independent per-dimension, rather than dealing with deduplication everywhere
         }
     }
-    CaretAssert(xml.isEndElement() && xml.name() == "MatrixIndicesMap");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("MatrixIndicesMap"));
 }
 
 QByteArray CiftiXML::writeXMLToQByteArray(const CiftiVersion& writingVersion) const
@@ -717,7 +763,7 @@ void CiftiXML::writeXML(QXmlStreamWriter& xml, const CiftiVersion& writingVersio
     } else if (writingVersion == CiftiVersion(2, 0)) {
         writeMatrix2(xml);
     } else {
-        throw CaretException("unknown Cifti writing version: '" + writingVersion.toString() + "'");
+        throw DataFileException("unknown Cifti writing version: '" + writingVersion.toString() + "'");
     }
     xml.writeEndElement();
 }
@@ -729,7 +775,7 @@ void CiftiXML::writeMatrix1(QXmlStreamWriter& xml) const
     VolumeSpace volSpace;
     for (int i = 0; i < numDims; ++i)
     {
-        if (m_indexMaps[i] == NULL) throw CaretException("dimension " + QString::number(i) + " was not given a mapping");
+        if (m_indexMaps[i] == NULL) throw DataFileException("dimension " + QString::number(i) + " was not given a mapping");
         switch (m_indexMaps[i]->getType())
         {
             case CiftiMappingType::BRAIN_MODELS:
@@ -741,7 +787,7 @@ void CiftiXML::writeMatrix1(QXmlStreamWriter& xml) const
                     {
                         if (myMap.getVolumeSpace() != volSpace)
                         {
-                            throw CaretException("cannot write different volume spaces for different dimensions in CIFTI-1");
+                            throw DataFileException("cannot write different volume spaces for different dimensions in CIFTI-1");
                         }
                     } else {
                         haveVolData = true;
@@ -759,7 +805,7 @@ void CiftiXML::writeMatrix1(QXmlStreamWriter& xml) const
                     {
                         if (myMap.getVolumeSpace() != volSpace)
                         {
-                            throw CaretException("cannot write different volume spaces for different dimensions in CIFTI-1");
+                            throw DataFileException("cannot write different volume spaces for different dimensions in CIFTI-1");
                         }
                     } else {
                         haveVolData = true;
@@ -818,7 +864,7 @@ void CiftiXML::writeMatrix2(QXmlStreamWriter& xml) const
     int numDims = (int)m_indexMaps.size();
     for (int i = 0; i < numDims; ++i)
     {
-        if (m_indexMaps[i] == NULL) throw CaretException("dimension " + QString::number(i) + " was not given a mapping");
+        if (m_indexMaps[i] == NULL) throw DataFileException("dimension " + QString::number(i) + " was not given a mapping");
     }
     xml.writeStartElement("Matrix");
     if (m_filePalette != NULL)
@@ -852,3 +898,24 @@ void CiftiXML::writeMatrix2(QXmlStreamWriter& xml) const
     }
     xml.writeEndElement();
 }
+
+bool CiftiXML::mutablesModified() const
+{
+    if (m_fileMetaData.isModified()) return true;
+    if (m_filePalette != NULL && m_filePalette->isModified()) return true;
+    for (int d = 0; d < (int)m_indexMaps.size(); ++d)
+    {
+        if (m_indexMaps[d] != NULL && m_indexMaps[d]->mutablesModified()) return true;
+    }
+    return false;
+}
+
+void CiftiXML::clearMutablesModified() const
+{
+    m_fileMetaData.clearModified();
+    if (m_filePalette != NULL) m_filePalette->clearModified();
+    for (int d = 0; d < (int)m_indexMaps.size(); ++d)
+    {
+        if (m_indexMaps[d] != NULL) m_indexMaps[d]->clearMutablesModified();
+    }
+}
