@@ -93,6 +93,12 @@ namespace caret
         bool operator==(const CiftiXML& rhs) const;
         bool operator!=(const CiftiXML& rhs) const { return !((*this) == rhs); }
         bool approximateMatch(const CiftiXML& rhs) const;
+        
+        bool mutablesModified() const;
+        void clearMutablesModified() const;//HACK: clear modified status on a const object
+        
+        static int directionFromString(const QString& input);//convenience conversion function, throws on error
+        static QString directionFromStringExplanation();//and explanation text
     private:
         std::vector<CaretPointer<CiftiMappingType> > m_indexMaps;
         CiftiVersion m_parsedVersion;
