@@ -23,6 +23,7 @@
 #include "CaretAssert.h"
 #include "CommandClassAddMember.h"
 #include "CommandClassCreate.h"
+#include "DataFileException.h"
 #include "FileInformation.h"
 #include "ProgramParameters.h"
 #include "TextFile.h"
@@ -102,15 +103,14 @@ CommandClassAddMember::getHelpInformation(const AString& /*programName*/)
  *   If there is an error in the parameters.
  */
 void 
-CommandClassAddMember::executeOperation(ProgramParameters& parameters) throw (CommandException,
-                                                               ProgramParametersException)
+CommandClassAddMember::executeOperation(ProgramParameters& parameters)
 {
     bool isAddToFiles = false;
     AString headerMemberCode = "";
     AString headerMethodCode = "";
     AString implementationCode = "";
     
-    const AString indentText = "        ";
+    const AString indentText = "    ";
     
     /*
      * Process parameters
@@ -162,6 +162,7 @@ CommandClassAddMember::executeOperation(ProgramParameters& parameters) throw (Co
              * is a class and use a pointer for the class.  However,
              * treat AString and QString as primitive types.
              */
+            //bool isEnum   = false;
             bool isString = false;
             bool isClass = false;
             const QChar firstDataTypeChar = dataType[0];
@@ -170,6 +171,9 @@ CommandClassAddMember::executeOperation(ProgramParameters& parameters) throw (Co
                     || (dataType == "QString")) {
                     isString = true;
                 }
+                else if (dataType.endsWith("Enum")) {
+                    //isEnum = true;
+                }
                 else {
                     isClass = true;
                 }
@@ -199,17 +203,25 @@ CommandClassAddMember::executeOperation(ProgramParameters& parameters) throw (Co
                 firstGetSetNameChar = firstGetSetNameChar.toUpper();
                 getSetName[0] = firstGetSetNameChar;
             }
-            const AString getterName = ("get" + getSetName);
+            const AString getterName = (((dataType == "bool")
+                                         ? "is"
+                                         : "get") + getSetName);
             const AString setterName = ("set" + getSetName);
             
             /*
              * Declare the member for the header file
              */
+            if (isClass) {
+                headerMemberCode += ("#include <memory>\n"
+                                     "\n");
+            }
             if (description.isEmpty() == false) {
                 headerMemberCode += (indentText + "/** " + description + "*/\n");
             }
-            headerMemberCode += (indentText 
-                                 + returnType
+            headerMemberCode += (indentText
+                                 + (isClass ? "std::unique_ptr<" : "")
+                                 + dataType //returnType
+                                 + (isClass ? ">" : "")
                                  + " "
                                  + name
                                  + ";"
@@ -220,6 +232,17 @@ CommandClassAddMember::executeOperation(ProgramParameters& parameters) throw (Co
             const AString classColonName = (className + "::");
             
             if (isClass) {
+                implementationCode += ("\n"
+                                       + indentText
+                                       + name
+                                       + " = std::unique_ptr<"
+                                       + dataType
+                                       + ">(new"
+                                       + dataType
+                                       + "());"
+                                       + "\n"
+                                       + "\n");
+                
                 /*
                  * For class members, create only getter methods
                  * that are both const and non-const
@@ -248,7 +271,7 @@ CommandClassAddMember::executeOperation(ProgramParameters& parameters) throw (Co
                                            + (isConstMethod ? " const" : "")
                                            + "\n"
                                            + "{\n"
-                                           + "    return " + name + ";\n"
+                                           + "    return " + name + ".get();\n"
                                            + "}\n"
                                            + "\n");
                 }
@@ -291,6 +314,7 @@ CommandClassAddMember::executeOperation(ProgramParameters& parameters) throw (Co
                 
                 implementationCode += ("/**\n"
                                        " * Set " + description + "\n"
+                                       " *\n"
                                        " * @param " + parameterName + "\n"
                                        " *    New value for " + description + "\n"
                                        " */\n"
