@@ -24,10 +24,10 @@
 #include "CommandOperationManager.h"
 #undef __COMMAND_OPERATION_MANAGER_DEFINE__
 
+#include "AlgorithmAnnotationResample.h"
 #include "AlgorithmBorderResample.h"
 #include "AlgorithmBorderToVertices.h"
 #include "AlgorithmCiftiAllLabelsToROIs.h"
-#include "AlgorithmCiftiAverage.h"
 #include "AlgorithmCiftiAverageDenseROI.h"
 #include "AlgorithmCiftiAverageROICorrelation.h"
 #include "AlgorithmCiftiCorrelation.h"
@@ -43,6 +43,7 @@
 #include "AlgorithmCiftiFindClusters.h"
 #include "AlgorithmCiftiGradient.h"
 #include "AlgorithmCiftiLabelAdjacency.h"
+#include "AlgorithmCiftiLabelModifyKeys.h"
 #include "AlgorithmCiftiLabelProbability.h"
 #include "AlgorithmCiftiLabelToBorder.h"
 #include "AlgorithmCiftiLabelToROI.h"
@@ -55,6 +56,7 @@
 #include "AlgorithmCiftiReorder.h"
 #include "AlgorithmCiftiReplaceStructure.h"
 #include "AlgorithmCiftiResample.h"
+#include "AlgorithmCiftiRestrictDenseMap.h"
 #include "AlgorithmCiftiROIsFromExtrema.h"
 #include "AlgorithmCiftiSeparate.h"
 #include "AlgorithmCiftiSmoothing.h"
@@ -122,6 +124,7 @@
 #include "AlgorithmVolumeFillHoles.h"
 #include "AlgorithmVolumeFindClusters.h"
 #include "AlgorithmVolumeGradient.h"
+#include "AlgorithmVolumeLabelModifyKeys.h"
 #include "AlgorithmVolumeLabelProbability.h"
 #include "AlgorithmVolumeLabelToROI.h"
 #include "AlgorithmVolumeLabelToSurfaceMapping.h"
@@ -130,6 +133,7 @@
 #include "AlgorithmVolumeParcelSmoothing.h"
 #include "AlgorithmVolumeReduce.h"
 #include "AlgorithmVolumeRemoveIslands.h"
+#include "AlgorithmVolumeResample.h"
 #include "AlgorithmVolumeROIsFromExtrema.h"
 #include "AlgorithmVolumeSmoothing.h"
 #include "AlgorithmVolumeTFCE.h"
@@ -145,6 +149,7 @@
 #include "OperationBorderFileExportToCaret5.h"
 #include "OperationBorderLength.h"
 #include "OperationBorderMerge.h"
+#include "OperationCiftiAverage.h"
 #include "OperationCiftiChangeMapping.h"
 #include "OperationCiftiChangeTimestep.h"
 #include "OperationCiftiConvert.h"
@@ -161,7 +166,6 @@
 #include "OperationCiftiMerge.h"
 #include "OperationCiftiPalette.h"
 #include "OperationCiftiResampleDconnMemory.h"
-#include "AlgorithmCiftiRestrictDenseMap.h"
 #include "OperationCiftiROIAverage.h"
 #include "OperationCiftiSeparateAll.h"
 #include "OperationCiftiStats.h"
@@ -174,6 +178,7 @@
 #include "OperationEstimateFiberBinghams.h"
 #include "OperationFileConvert.h"
 #include "OperationFileInformation.h"
+#include "OperationFociCreate.h"
 #include "OperationFociGetProjectionVertex.h"
 #include "OperationFociListCoords.h"
 #include "OperationGiftiConvert.h"
@@ -195,11 +200,14 @@
 #include "OperationProbtrackXDotConvert.h"
 #include "OperationSceneFileMerge.h"
 #include "OperationSceneFileRelocate.h"
+#include "OperationSceneFileUpdate.h"
 #include "OperationSetMapName.h"
 #include "OperationSetMapNames.h"
 #include "OperationSetStructure.h"
 #include "OperationShowScene.h"
+#include "OperationShowSceneTwo.h"
 #include "OperationSpecFileMerge.h"
+#include "OperationSpecFileModify.h"
 #include "OperationSpecFileRelocate.h"
 #include "OperationSurfaceClosestVertex.h"
 #include "OperationSurfaceCoordinatesToMetric.h"
@@ -207,12 +215,15 @@
 #include "OperationSurfaceFlipNormals.h"
 #include "OperationSurfaceGeodesicDistance.h"
 #include "OperationSurfaceGeodesicDistanceAllToAll.h"
+#include "OperationSurfaceGeodesicDistanceSparseText.h"
 #include "OperationSurfaceGeodesicROIs.h"
 #include "OperationSurfaceInformation.h"
 #include "OperationSurfaceNormals.h"
 #include "OperationSurfaceSetCoordinates.h"
+#include "OperationSurfaceSphereTriangularPatches.h"
 #include "OperationSurfaceVertexAreas.h"
 #include "OperationVolumeCapturePlane.h"
+#include "OperationVolumeComponentsToFrames.h"
 #include "OperationVolumeCopyExtensions.h"
 #include "OperationVolumeCreate.h"
 #include "OperationVolumeLabelExportTable.h"
@@ -244,7 +255,7 @@
 
 #include "CaretLogger.h"
 #include "dot_wrapper.h"
-#include "StructureEnum.h"
+#include "CaretCommandGlobalOptions.h"
 
 #include <iostream>
 #include <map>
@@ -285,10 +296,10 @@ CommandOperationManager::deleteCommandOperationManager()
  */
 CommandOperationManager::CommandOperationManager()
 {
+    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmAnnotationResample()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmBorderResample()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmBorderToVertices()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiAllLabelsToROIs()));
-    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiAverage()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiAverageDenseROI()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiAverageROICorrelation()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiCorrelation()));
@@ -304,6 +315,7 @@ CommandOperationManager::CommandOperationManager()
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiFindClusters()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiGradient()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiLabelAdjacency()));
+    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiLabelModifyKeys()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiLabelProbability()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiLabelToBorder()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmCiftiLabelToROI()));
@@ -374,7 +386,6 @@ CommandOperationManager::CommandOperationManager()
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmSurfaceSphereProjectUnproject()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmSurfaceToSurface3dDistance()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmSurfaceWedgeVolume()));
-    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeAffineResample()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeAllLabelsToROIs()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeDilate()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeDistortion()));
@@ -384,6 +395,7 @@ CommandOperationManager::CommandOperationManager()
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeFillHoles()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeFindClusters()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeGradient()));
+    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeLabelModifyKeys()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeLabelProbability()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeLabelToROI()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeLabelToSurfaceMapping()));
@@ -392,21 +404,21 @@ CommandOperationManager::CommandOperationManager()
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeParcelSmoothing()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeReduce()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeRemoveIslands()));
+    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeResample()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeROIsFromExtrema()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeSmoothing()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeTFCE()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeToSurfaceMapping()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeVectorOperation()));
     this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeWarpfieldAffineRegression()));
-    this->commandOperations.push_back(new CommandParser(new AutoAlgorithmVolumeWarpfieldResample()));
     
-    this->commandOperations.push_back(new CommandParser(new AutoOperationAddToSpecFile()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationBackendAverageDenseROI()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationBackendAverageROICorrelation()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationBorderExportColorTable()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationBorderFileExportToCaret5()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationBorderLength()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationBorderMerge()));
+    this->commandOperations.push_back(new CommandParser(new AutoOperationCiftiAverage()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationCiftiChangeMapping()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationCiftiConvert()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationCiftiCreateDenseFromTemplate()));
@@ -431,6 +443,7 @@ CommandOperationManager::CommandOperationManager()
     this->commandOperations.push_back(new CommandParser(new AutoOperationEstimateFiberBinghams()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationFileConvert()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationFileInformation()));
+    this->commandOperations.push_back(new CommandParser(new AutoOperationFociCreate()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationFociGetProjectionVertex()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationFociListCoords()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationGiftiConvert()));
@@ -451,12 +464,12 @@ CommandOperationManager::CommandOperationManager()
     this->commandOperations.push_back(new CommandParser(new AutoOperationProbtrackXDotConvert()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSceneFileMerge()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSceneFileRelocate()));
+    this->commandOperations.push_back(new CommandParser(new AutoOperationSceneFileUpdate()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSetMapNames()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSetStructure()));
-    if (OperationShowScene::isShowSceneCommandAvailable()) {
-        this->commandOperations.push_back(new CommandParser(new AutoOperationShowScene()));
-    }
+    this->commandOperations.push_back(new CommandParser(new AutoOperationShowSceneTwo()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSpecFileMerge()));
+    this->commandOperations.push_back(new CommandParser(new AutoOperationSpecFileModify()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSpecFileRelocate()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceClosestVertex()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceCoordinatesToMetric()));
@@ -464,12 +477,15 @@ CommandOperationManager::CommandOperationManager()
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceFlipNormals()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceGeodesicDistance()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceGeodesicDistanceAllToAll()));
+    this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceGeodesicDistanceSparseText()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceGeodesicROIs()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceInformation()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceNormals()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceSetCoordinates()));
+    this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceSphereTriangularPatches()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationSurfaceVertexAreas()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationVolumeCapturePlane()));
+    this->commandOperations.push_back(new CommandParser(new AutoOperationVolumeComponentsToFrames()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationVolumeCopyExtensions()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationVolumeCreate()));
     this->commandOperations.push_back(new CommandParser(new AutoOperationVolumeLabelExportTable()));
@@ -495,12 +511,17 @@ CommandOperationManager::CommandOperationManager()
 #endif // WORKBENCH_HAVE_C11X
     this->commandOperations.push_back(new CommandUnitTest());
     
+    this->deprecatedOperations.push_back(new CommandParser(new AutoAlgorithmVolumeAffineResample()));
+    this->deprecatedOperations.push_back(new CommandParser(new AutoAlgorithmVolumeWarpfieldResample()));
+    
+    this->deprecatedOperations.push_back(new CommandParser(new AutoOperationAddToSpecFile()));
     this->deprecatedOperations.push_back(new CommandParser(new AutoOperationCiftiChangeTimestep()));
     this->deprecatedOperations.push_back(new CommandParser(new AutoOperationCiftiConvertToScalar()));
     this->deprecatedOperations.push_back(new CommandParser(new AutoOperationCiftiCopyMapping()));
     this->deprecatedOperations.push_back(new CommandParser(new AutoOperationCiftiSeparateAll()));
     this->deprecatedOperations.push_back(new CommandParser(new AutoOperationMetricVertexSum()));
     this->deprecatedOperations.push_back(new CommandParser(new AutoOperationSetMapName()));
+    this->deprecatedOperations.push_back(new CommandParser(new AutoOperationShowScene()));
 }
 
 /**
@@ -525,7 +546,7 @@ CommandOperationManager::~CommandOperationManager()
 namespace
 {
     //quick hack to convert type argument to internal integer
-    int16_t stringToCiftiType(const AString& input)
+    int16_t stringToNiftiType(const AString& input)
     {
         map<AString, int16_t> nameToCode;
         nameToCode["INT8"] = NIFTI_TYPE_INT8;
@@ -542,7 +563,7 @@ namespace
         map<AString, int16_t>::iterator iter = nameToCode.find(input);
         if (iter == nameToCode.end())
         {
-            throw CommandException("Unrecognized cifti datatype: '" + input + "'");
+            throw CommandException("Unrecognized nifti datatype: '" + input + "'");
         }
         return iter->second;
     }
@@ -579,22 +600,40 @@ CommandOperationManager::runCommand(ProgramParameters& parameters)
             CaretLogWarning("SIMD type '" + DotSIMDEnum::toName(impl) + "' not supported (could be cpu, compiler, or build options), using '" + DotSIMDEnum::toName(retval) + "'");
         }
     }
-    int16_t ciftiDType = NIFTI_TYPE_FLOAT32;
-    bool ciftiScale = false;
-    double ciftiMin = -1.0, ciftiMax = -1.0;
+    if (getGlobalOption(parameters, "-nifti-output-datatype", 1, globalOptionArgs))
+    {
+        caret_global_command_options.m_ciftiDType =
+            caret_global_command_options.m_volumeDType = stringToNiftiType(globalOptionArgs[0]);
+    }
+    if (getGlobalOption(parameters, "-nifti-output-range", 2, globalOptionArgs))
+    {
+        caret_global_command_options.m_ciftiScale =
+            caret_global_command_options.m_volumeScale = true;
+        bool valid = false;
+        caret_global_command_options.m_ciftiMin =
+            caret_global_command_options.m_volumeMin = globalOptionArgs[0].toDouble(&valid);
+        if (!valid) throw CommandException("non-numeric option to -nifti-output-range: '" + globalOptionArgs[0] + "'");
+        caret_global_command_options.m_ciftiMax =
+            caret_global_command_options.m_volumeMax = globalOptionArgs[1].toDouble(&valid);
+        if (!valid) throw CommandException("non-numeric option to -nifti-output-range: '" + globalOptionArgs[1] + "'");
+    }
     if (getGlobalOption(parameters, "-cifti-output-datatype", 1, globalOptionArgs))
     {
-        ciftiDType = stringToCiftiType(globalOptionArgs[0]);
+        caret_global_command_options.m_ciftiDType = stringToNiftiType(globalOptionArgs[0]);
     }
     if (getGlobalOption(parameters, "-cifti-output-range", 2, globalOptionArgs))
     {
-        ciftiScale = true;
+        caret_global_command_options.m_ciftiScale = true;
         bool valid = false;
-        ciftiMin = globalOptionArgs[0].toDouble(&valid);
+        caret_global_command_options.m_ciftiMin = globalOptionArgs[0].toDouble(&valid);
         if (!valid) throw CommandException("non-numeric option to -cifti-output-range: '" + globalOptionArgs[0] + "'");
-        ciftiMax = globalOptionArgs[1].toDouble(&valid);
+        caret_global_command_options.m_ciftiMax = globalOptionArgs[1].toDouble(&valid);
         if (!valid) throw CommandException("non-numeric option to -cifti-output-range: '" + globalOptionArgs[1] + "'");
     }
+    if (getGlobalOption(parameters, "-cifti-read-memory", 0, globalOptionArgs))
+    {
+        caret_global_command_options.m_ciftiReadMemory = true;
+    }
 
     const uint64_t numberOfCommands = this->commandOperations.size();
     const uint64_t numberOfDeprecated = this->deprecatedOperations.size();
@@ -613,7 +652,7 @@ CommandOperationManager::runCommand(ProgramParameters& parameters)
     {
         printHelpInfo();
     } else if (commandSwitch == "-arguments-help") {
-        printArgumentsHelp(myProgramName);
+        printArgumentsHelp();
     } else if (commandSwitch == "-global-options") {
         printGlobalOptions();
     } else if (commandSwitch == "-cifti-help") {
@@ -634,7 +673,7 @@ CommandOperationManager::runCommand(ProgramParameters& parameters)
         printAllCommandsHelpInfo(myProgramName);
     } else {
         
-        CommandOperation* operation = NULL;
+        CommandOperation* operation = NULL, *compatOperation = NULL; //separate so we can do both in one pass while giving priority on collision
         
         for (uint64_t i = 0; i < numberOfCommands; i++)
         {
@@ -643,6 +682,15 @@ CommandOperationManager::runCommand(ProgramParameters& parameters)
                 operation = this->commandOperations[i];
                 break;
             }
+            for (const AString compatSwitch : this->commandOperations[i]->getCompatibilitySwitches())
+            {
+                if (compatSwitch == commandSwitch)
+                {
+                    CaretAssert(compatOperation == NULL); //try to catch switch collisions in debug builds
+                    compatOperation = this->commandOperations[i];
+                    break; //do NOT break outer loop, we want to give priority to non-compat switches
+                }
+            }
         }
         if (operation == NULL)
         {
@@ -653,8 +701,23 @@ CommandOperationManager::runCommand(ProgramParameters& parameters)
                     operation = this->deprecatedOperations[i];
                     break;
                 }
+                for (const AString compatSwitch : this->deprecatedOperations[i]->getCompatibilitySwitches())
+                {
+                    if (compatSwitch == commandSwitch)
+                    {
+                        CaretAssert(compatOperation == NULL); //try to catch switch collisions in debug builds
+                        compatOperation = this->deprecatedOperations[i];
+                        break; //do NOT break outer loop, we want to give priority to non-compat switches
+                    }
+                }
             }
         }
+        if (operation == NULL)
+        {
+            operation = compatOperation; //may also be null, but that is fine
+        } else {
+            CaretAssert(compatOperation == NULL); //try to catch switch collisions in debug builds
+        }
         
         if (operation == NULL) {
             if (!parameters.hasNext())
@@ -668,12 +731,6 @@ CommandOperationManager::runCommand(ProgramParameters& parameters)
             {
                 cout << operation->getHelpInformation(myProgramName) << endl;
             } else {
-                if (ciftiScale)
-                {
-                    operation->setCiftiOutputDTypeAndScale(ciftiDType, ciftiMin, ciftiMax);
-                } else {
-                    operation->setCiftiOutputDTypeNoScale(ciftiDType);
-                }
                 operation->execute(parameters, preventProvenance);
             }
         }
@@ -713,14 +770,25 @@ AString CommandOperationManager::doCompletion(ProgramParameters& parameters, con
     OptionInfo ciftiDTypeInfo = parseGlobalOption(parameters, "-cifti-output-datatype", 1, globalOptionArgs, true);
     if (ciftiDTypeInfo.specified && !ciftiDTypeInfo.complete)
     {
-        return "wordlist INT8 UINT8 INT16 UINT16 INT32 UINT32 INT64 UINT64 FLOAT32 FLOAT64 FLOAT128";
+        return "wordlist INT8\\ UINT8\\ INT16\\ UINT16\\ INT32\\ UINT32\\ INT64\\ UINT64\\ FLOAT32\\ FLOAT64\\ FLOAT128";
     }
     OptionInfo ciftiRangeInfo = parseGlobalOption(parameters, "-cifti-output-range", 2, globalOptionArgs, true);
     if (ciftiRangeInfo.specified && !ciftiRangeInfo.complete)
     {//can't tab complete a literal number
         return "";
     }
-    ret = "wordlist -disable-provenance\\ -logging\\ -simd\\ -cifti-output-datatype\\ -cifti-output-range";//we could prevent suggesting an already-provided global option, but that would be a bit surprising
+    OptionInfo niftiDTypeInfo = parseGlobalOption(parameters, "-nifti-output-datatype", 1, globalOptionArgs, true);
+    if (niftiDTypeInfo.specified && !niftiDTypeInfo.complete)
+    {
+        return "wordlist INT8\\ UINT8\\ INT16\\ UINT16\\ INT32\\ UINT32\\ INT64\\ UINT64\\ FLOAT32\\ FLOAT64\\ FLOAT128";
+    }
+    OptionInfo niftiRangeInfo = parseGlobalOption(parameters, "-nifti-output-range", 2, globalOptionArgs, true);
+    if (niftiRangeInfo.specified && !niftiRangeInfo.complete)
+    {
+        return "";
+    }
+    /*OptionInfo ciftiReadMemInfo = */parseGlobalOption(parameters, "-cifti-read-memory", 0, globalOptionArgs, true);
+    ret = "wordlist -disable-provenance\\ -logging\\ -simd\\ -cifti-output-datatype\\ -cifti-output-range\\ -nifti-output-datatype\\ -nifti-output-range\\ -cifti-read-memory";//we could prevent suggesting an already-provided global option, but that would be a bit surprising
     const uint64_t numberOfCommands = this->commandOperations.size();
     const uint64_t numberOfDeprecated = this->deprecatedOperations.size();
     if (!parameters.hasNext())
@@ -925,12 +993,6 @@ CommandOperationManager::printAllCommandsMatching(const AString& partialSwitch)
             
             cmdMap.insert(make_pair(cmdSwitch,
                                         op->getOperationShortDescription()));
-#ifndef NDEBUG
-            const AString helpInfo = op->getHelpInformation("");//TSC: generating help info takes a little processing (populating and walking an OperationParameters tree for each command)
-            if (helpInfo.isEmpty()) {//So, test the same define as for asserts and skip this check in release
-                CaretLogSevere("Command has no help info: " + cmdSwitch);
-            }
-#endif
         }
     }
     if (longestSwitch == -1)//no command found
@@ -991,18 +1053,19 @@ void CommandOperationManager::printHelpInfo()
     cout << endl;
 }
 
-void CommandOperationManager::printArgumentsHelp(const AString& programName)
+void CommandOperationManager::printArgumentsHelp()
 {
     //guide for wrap, assuming 80 columns:                                                  |
     cout << "   To get the help information on a subcommand, run it without any additional" << endl;
-    cout << "   arguments.  Options can occur in any position within the correct scope, and" << endl;
-    cout << "   can have suboptions, which must occur within the scope of the option.  The" << endl;
-    cout << "   easiest way to get this right is to specify options and arguments in the" << endl;
-    cout << "   order they are listed.  As an example, consider this help information:" << endl;
+    cout << "   arguments.  Options can occur in any order, however suboptions and arguments" << endl;
+    cout << "   to options must occur next to their parent option.  The easiest way to get" << endl;
+    cout << "   this right is to specify options and arguments in the order they are listed." << endl;
+    cout << "   As an example, consider this abbreviated version of the -volume-math help" << endl;
+    cout << "   information:" << endl;
     cout << endl;//guide for wrap, assuming 80 columns:                                     |
-    cout << "$ " << programName << " -volume-math" << endl;
+    cout << "$ wb_command -volume-math" << endl;
     cout << "EVALUATE EXPRESSION ON VOLUME FILES" << endl;
-    cout << "   " << programName << " -volume-math" << endl;
+    cout << "   wb_command -volume-math" << endl;
     cout << "      <expression> - the expression to evaluate, in quotes" << endl;
     cout << "      <volume-out> - output - the output volume" << endl;
     cout << endl;//guide for wrap, assuming 80 columns:                                     |
@@ -1018,37 +1081,52 @@ void CommandOperationManager::printArgumentsHelp(const AString& programName)
     cout << endl;//guide for wrap, assuming 80 columns:                                     |
     cout << "         [-repeat] - reuse a single subvolume for each subvolume of calculation" << endl;
     cout << "..." << endl;
+    cout << "      The following functions are supported:" << endl;
+    cout << "..." << endl;
+    cout << "         abs: 1 argument, the absolute value of the argument" << endl;
+    cout << "..." << endl;
+    cout << endl;//guide for wrap, assuming 80 columns:                                     |
+    cout << "   '<expression>' represents a required input parameter (required parameters" << endl;
+    cout << "   are marked with the < and > symbols), and '<volume-out> - output' represents" << endl;
+    cout << "   a required output filename (marked by the presence of the word 'output')." << endl;
+    cout << "   '[-fixnan]' represents an option (marked with the [ and ] symbols), taking" << endl;
+    cout << "   one required parameter '<replace>' (the indentation level indicates what" << endl;
+    cout << "   parameters and suboptions are associated with a given option), and '[-var] -" << endl;
+    cout << "   repeatable' denotes a repeatable option (marked by the presence of the word" << endl;
+    cout << "   'repeatable') with required parameters '<name>' and '<volume>', and two" << endl;
+    cout << "   suboptions: '[-subvolume]', which has a required parameter '<subvol>', and" << endl;
+    cout << "   '[-repeat]', which takes no parameters." << endl;
     cout << endl;//guide for wrap, assuming 80 columns:                                     |
-    cout << "   '<expression>' and '<volume-out>' denote mandatory parameters.  '[-fixnan]'" << endl;
-    cout << "   denotes an option taking one mandatory parameter '<replace>', and" << endl;
-    cout << "   '[-var] - repeatable' denotes a repeatable option with mandatory parameters" << endl;
-    cout << "   '<name>' and '<volume>', and two suboptions: '[-subvolume]', which has a" << endl;
-    cout << "   mandatory parameter '<subvol>', and '[-repeat]', which takes no parameters." << endl;
-    cout << "   Commands also provide additional help info below the section in the example." << endl;
     cout << "   Each option starts a new scope, and all options and arguments end any scope" << endl;
-    cout << "   that they are not valid in.  For example, this command is correct:" << endl;
+    cout << "   that they are not valid in.  This means that for any option, you must" << endl;
+    cout << "   specify all of its arguments and any desired suboptions before specifying" << endl;
+    cout << "   any other option or argument on the same or a previous level.  For example," << endl;
+    cout << "   this annotated command is valid:" << endl;
     cout << endl;//guide for wrap, assuming 80 columns:                                     |
-    cout << "$ " << programName << " -volume-math 'sin(x)' sin_x.nii.gz -fixnan 0 -var x x.nii.gz -subvolume 1" << endl;
+    cout << "$ wb_command -volume-math 'abs(x)'  abs_x.nii.gz -fixnan 0  -var x    x.nii.gz" << endl;
+    cout << "annotation:            <expression> <volume-out>     <replace> <name> <volume>" << endl;
     cout << endl;
-    cout << "   as is this one (though less intuitive):" << endl;
+    cout << "   Here is another annotated command that results in the same output, but" << endl;
+    cout << "   uses a different order of the options:" << endl;
     cout << endl;
-    cout << "$ " << programName << " -volume-math -fixnan 0 'sin(x)' -var x -subvolume 1 x.nii.gz sin_x.nii.gz" << endl;
+    cout << "$ wb_command -volume-math -fixnan 0     'abs(x)' -var x   x.nii.gz abs_x.nii.gz" << endl;
+    cout << "annotation:                 <replace> <expression> <name> <volume> <volume-out>" << endl;
     cout << endl;//guide for wrap, assuming 80 columns:                                     |
-    cout << "   while this one is not, because the -fixnan option ends the scope of the -var" << endl;
-    cout << "   option before all of its mandatory arguments are given:" << endl;
+    cout << "   This next command is invalid, because the -fixnan option ends the scope of" << endl;
+    cout << "   the -var option before all of its required arguments are given:" << endl;
     cout << endl;
-    cout << "$ " << programName << " -volume-math 'sin(x)' sin_x.nii.gz -var x -fixnan 0 x.nii.gz -subvolume 1" << endl;
+    cout << "wrong: wb_command -volume-math 'abs(x)' abs_x.nii.gz -var x -fixnan 0 x.nii.gz" << endl;
     cout << endl;
     //guide for wrap, assuming 80 columns:                                                  |
-    cout << "   and this one is incorrect because the -subvolume option occurs after the" << endl;
-    cout << "   scope of the -var option has ended due to -fixnan:" << endl;
+    cout << "   This command is invalid because the -subvolume option occurs after the" << endl;
+    cout << "   scope of the -var option has ended due to the -fixnan option:" << endl;
     cout << endl;
-    cout << "$ " << programName << " -volume-math 'sin(x)' sin_x.nii.gz -var x x.nii.gz -fixnan 0 -subvolume 1" << endl;
+    cout << "wrong: wb_command -volume-math 'abs(x)' abs_x.nii.gz -var x x.nii.gz -fixnan 0 -subvolume 1" << endl;
     cout << endl;//guide for wrap, assuming 80 columns:                                     |
-    cout << "   and this one is similarly incorrect because the -subvolume option occurs" << endl;
+    cout << "   This command is similarly invalid because the -subvolume option occurs" << endl;
     cout << "   after the scope of the -var option has ended due to the volume-out argument:" << endl;
     cout << endl;
-    cout << "$ " << programName << " -volume-math 'sin(x)' -fixnan 0 -var x x.nii.gz sin_x.nii.gz -subvolume 1" << endl;
+    cout << "wrong: wb_command -volume-math 'abs(x)' -var x x.nii.gz abs_x.nii.gz -subvolume 1 -fixnan 0" << endl;
     cout << endl;
 }
 
@@ -1060,10 +1138,10 @@ void CommandOperationManager::printGlobalOptions()
     cout << "   -disable-provenance               don't generate provenance info in output" << endl;
     cout << "                                        files" << endl;
     cout << endl;
-    cout << "   -cifti-output-datatype <type>     write cifti output with the given" << endl;
-    cout << "                                        datatype (default FLOAT32), note that" << endl;
-    cout << "                                        calculation precision is only float32," << endl;
-    cout << "                                        valid values are:" << endl;
+    cout << "   -nifti-output-datatype <type>     write cifti and volume output with the" << endl;
+    cout << "                                        given datatype (default FLOAT32), note" << endl;
+    cout << "                                        that calculation precision is only" << endl;
+    cout << "                                        float32, valid values are:" << endl;
     cout << "                          INT8" << endl;
     cout << "                          UINT8" << endl;
     cout << "                          INT16" << endl;
@@ -1077,11 +1155,12 @@ void CommandOperationManager::printGlobalOptions()
     cout << "                          FLOAT128" << endl;
     cout << endl;
     //guide for wrap, assuming 80 columns:                                                  |
-    cout << "   -cifti-output-range <min> <max>   write cifti output with scaling and offset" << endl;
-    cout << "                                        header fields such that <min> and <max>" << endl;
-    cout << "                                        are the most extreme values that can be" << endl;
-    cout << "                                        represented, mostly useful with integer" << endl;
-    cout << "                                        output datatypes (see above)" << endl;
+    cout << "   -nifti-output-range <min> <max>   write cifti and volume output with scaling" << endl;
+    cout << "                                        and offset header fields such that" << endl;
+    cout << "                                        <min> and <max> are the most extreme" << endl;
+    cout << "                                        values that can be represented, not" << endl;
+    cout << "                                        recommended with floating point types" << endl;
+    cout << "                                        (see above)" << endl;
     cout << endl;
     cout << "   -logging <level>                  set the logging level, valid values are:" << endl;
     vector<LogLevelEnum::Enum> logLevels;
@@ -1104,6 +1183,13 @@ void CommandOperationManager::printGlobalOptions()
         cout << "         " << DotSIMDEnum::toName(*iter) << endl;
     }
     cout << endl;
+    cout << "   -cifti-read-memory                read cifti input files into memory, to" << endl;
+    cout << "                                        avoid hitting limits on number of open" << endl;
+    cout << "                                        files" << endl;
+    cout << endl;
+    cout << "   -cifti-output-datatype <type>     deprecated, only affects cifti outputs" << endl;
+    cout << "   -cifti-output-range <min> <max>   deprecated, only affects cifti outputs" << endl;
+    cout << endl;
 }
 
 void CommandOperationManager::printCiftiHelp()
@@ -1145,7 +1231,7 @@ void CommandOperationManager::printCiftiHelp()
     cout << "         timeseries of 0 sec, 0.7 sec, 1.4 sec, ...)" << endl;
     cout << "      Labels: each index is assigned a name (i.e., 'Visual Areas'), but also a" << endl;
     cout << "         list of labels that maps integer data values to names and colors (i.e." << endl;
-    cout << "         {(5, 'V1', #ff0000), (7, 'V2', #00ff00), ...}" << endl;
+    cout << "         {(5, 'V1', #ff0000), (7, 'V2', #00ff00), ...})" << endl;
     cout << endl;//guide for wrap, assuming 80 columns:                                     |
     cout << "   The commands that operate on cifti files often require you to specify which" << endl;
     cout << "   dimension they should operate on.  Because cifti files can contain 3" << endl;
