@@ -44,6 +44,8 @@ namespace caret {
         
         void runCommand(ProgramParameters& parameters);
         
+        AString doCompletion(ProgramParameters& parameters, const bool& useExtGlob);
+        
         std::vector<CommandOperation*> getCommandOperations();
         
     private:
@@ -63,17 +65,33 @@ namespace caret {
         
         void printHelpInfo();
         
-        void printArgumentsHelp(const AString& programName);
+        void printArgumentsHelp();
         
-        void printCiftiHelp(const AString& programName);
+        void printGlobalOptions();
         
-        void printGiftiHelp(const AString& programName);
+        void printCiftiHelp();
+        
+        void printGiftiHelp();
+        
+        void printVolumeHelp();
+        
+        void printParallelHelp(const AString& programName);
         
         void printVersionInfo();
         
         bool getGlobalOption(ProgramParameters& parameters, const AString& optionString, const int& numArgs, std::vector<AString>& arguments);
         
-        static AString fixUnicode(const AString& input);
+        struct OptionInfo
+        {
+            bool specified;
+            bool complete;
+            int index;//only valid when complete is false, might not be needed
+            OptionInfo() { specified = false; complete = false; index = -1; }
+        };
+        
+        OptionInfo parseGlobalOption(ProgramParameters& parameters, const AString& optionString, const int& numArgs, std::vector<AString>& arguments, const bool& quiet);
+        
+        static AString fixUnicode(const AString& input, const bool& quiet);
         
     private:
         std::vector<CommandOperation*> commandOperations, deprecatedOperations;
