@@ -18,10 +18,13 @@
  */
 /*LICENSE_END*/
 
+#include <QList>
 #include <QTextStream>
 
 #include "AString.h"
+#include "CaretAssert.h"
 #include "CaretLogger.h"
+#include "Vector3D.h"
 #include <iostream>
 
 using namespace caret;
@@ -170,26 +173,33 @@ AString::fromNumbers(const std::vector<uint64_t>& v, const AString& separator)
     return s;
 }
 
-
 /**
  * Convert a vector of values into a string.
  * @param v
  *   The vector of values.
  * @param separator
  *   Inserted between each pair of values.
+ * @param format
+ *   e format as [-]9.9e[+|-]999
+ *   E format as [-]9.9E[+|-]999
+ *   f format as [-]9.9
+ *   g use e or f format, whichever is the most concise
+ *   G use E or f format, whichever is the most concise
+ * @param precision
+ *   Maximum number of digits following decimal.
  * @return
  *   String containing the vector's values separated
  *   by the separator.
  */
 AString
-AString::fromNumbers(const std::vector<float>& v, const AString& separator)
+AString::fromNumbers(const Vector3D& v, const AString& separator, char format, const int32_t precision)
 {
     AString s;
-    for (uint64_t i = 0; i < v.size(); i++) {
+    for (int32_t i = 0; i < 3; i++) {
         if (i > 0) {
             s += separator;
         }
-        s += AString::number(v[i]);
+        s += AString::number(v[i], format, precision);
     }
     return s;
 }
@@ -200,19 +210,58 @@ AString::fromNumbers(const std::vector<float>& v, const AString& separator)
  *   The vector of values.
  * @param separator
  *   Inserted between each pair of values.
+ * @param format
+ *   e format as [-]9.9e[+|-]999
+ *   E format as [-]9.9E[+|-]999
+ *   f format as [-]9.9
+ *   g use e or f format, whichever is the most concise
+ *   G use E or f format, whichever is the most concise
+ * @param precision
+ *   Maximum number of digits following decimal.
  * @return
  *   String containing the vector's values separated
  *   by the separator.
  */
 AString
-AString::fromNumbers(const std::vector<double>& v, const AString& separator)
+AString::fromNumbers(const std::vector<float>& v, const AString& separator, char format, const int32_t precision)
 {
     AString s;
     for (uint64_t i = 0; i < v.size(); i++) {
         if (i > 0) {
             s += separator;
         }
-        s += AString::number(v[i]);
+        s += AString::number(v[i], format, precision);
+    }
+    return s;
+}
+
+/**
+ * Convert a vector of values into a string.
+ * @param v
+ *   The vector of values.
+ * @param separator
+ *   Inserted between each pair of values.
+ * @param format
+ *   e format as [-]9.9e[+|-]999
+ *   E format as [-]9.9E[+|-]999
+ *   f format as [-]9.9
+ *   g use e or f format, whichever is the most concise
+ *   G use E or f format, whichever is the most concise
+ * @param precision
+ *   Maximum number of digits following decimal.
+ * @return
+ *   String containing the vector's values separated
+ *   by the separator.
+ */
+AString
+AString::fromNumbers(const std::vector<double>& v, const AString& separator, char format, const int32_t precision)
+{
+    AString s;
+    for (uint64_t i = 0; i < v.size(); i++) {
+        if (i > 0) {
+            s += separator;
+        }
+        s += AString::number(v[i], format, precision);
     }
     return s;
 }
@@ -225,19 +274,27 @@ AString::fromNumbers(const std::vector<double>& v, const AString& separator)
  *   Number of elements in the array.
  * @param separator
  *   Inserted between each pair of values.
+ * @param format
+ *   e format as [-]9.9e[+|-]999
+ *   E format as [-]9.9E[+|-]999
+ *   f format as [-]9.9
+ *   g use e or f format, whichever is the most concise
+ *   G use E or f format, whichever is the most concise
+ * @param precision
+ *   Maximum number of digits following decimal.
  * @return
  *   String containing the array values separated
  *   by the separator.
  */
 AString
-AString::fromNumbers(const float* array, const int64_t numberOfElements, const AString& separator)
+AString::fromNumbers(const float* array, const int64_t numberOfElements, const AString& separator, char format, const int32_t precision)
 {
     AString s;
     for (int64_t i = 0; i < numberOfElements; i++) {
         if (i > 0) {
             s += separator;
         }
-        s += AString::number(array[i]);
+        s += AString::number(array[i], format, precision);
     }
     return s;
 }
@@ -359,21 +416,31 @@ AString::fromNumbers(const int64_t* array,
  *   Number of elements in the array.
  * @param separator
  *   Inserted between each pair of values.
+ * @param format
+ *   e format as [-]9.9e[+|-]999
+ *   E format as [-]9.9E[+|-]999
+ *   f format as [-]9.9
+ *   g use e or f format, whichever is the most concise
+ *   G use E or f format, whichever is the most concise
+ * @param precision
+ *   Maximum number of digits following decimal.
  * @return
  *   String containing the array values separated
  *   by the separator.
  */
 AString
 AString::fromNumbers(const double* array,
-                             const int64_t numberOfElements,
-                             const AString& separator)
+                     const int64_t numberOfElements,
+                     const AString& separator,
+                     char format,
+                     const int32_t precision)
 {
     AString s;
     for (int64_t i = 0; i < numberOfElements; i++) {
         if (i > 0) {
             s += separator;
         }
-        s += AString::number(array[i]);
+        s += AString::number(array[i], format, precision);
     }
     return s;
 }
@@ -425,19 +492,6 @@ AString::toNumbers(const AString& s,
             numbersOut.push_back(floatValue);
         }
     }
-
-//    AString copy = s;
-//    QTextStream stream(&copy);
-//    
-//    AString numberString;
-//    bool valid = false;
-//    while (stream.atEnd() == false) {
-//        stream >> numberString;
-//        const float floatValue = numberString.toFloat(&valid);
-//        if (valid) {
-//            numbersOut.push_back(floatValue);
-//        }
-//    }
 }
 
 /**
@@ -460,12 +514,30 @@ AString::toNumbers(const AString& s,
     AString copy = s;
     QTextStream stream(&copy);
     
-    AString numberString;
-    bool valid = false;
+    int intValue;
     while (stream.atEnd() == false) {
-        stream >> numberString;
-        const int32_t intValue = numberString.toInt(&valid);
-        if (valid) {
+        /*
+         * Try to read a int value from the current position
+         */
+        stream >> intValue;
+        
+        /*
+         * If the text stream could not create an int from
+         * the current text position, the corrupt data flag
+         * will be set.
+         */
+        if (stream.status() == QTextStream::ReadCorruptData) {
+            /*
+             * Reset the status of the string (to OK) and
+             * then read one character to remove the character
+             * from the stream since it was not the start
+             * of a number.
+             */
+            stream.resetStatus();
+            QChar oneChar;
+            stream >> oneChar;
+        }
+        else {
             numbersOut.push_back(intValue);
         }
     }
@@ -714,7 +786,7 @@ AString::replaceHtmlSpecialCharactersWithEscapeCharacters() const
     const int64_t length = this->count();
     for (int64_t i = 0; i < length; i++) {
         const QChar ch = this->at(i);
-        switch (ch.toAscii()) {
+        switch (ch.toLatin1()) {
             case '&':
                 htmlString.append("&amp;");
                 break;
@@ -745,7 +817,7 @@ AString::replaceHtmlSpecialCharactersWithEscapeCharacters() const
     return htmlString;
 }
 
-AString AString::fixUnicodeHyphens(bool* hyphenReplaced, bool* hadOtherNonAscii) const
+AString AString::fixUnicodeHyphens(bool* hyphenReplaced, bool* hadOtherNonAscii, const bool& quiet) const
 {
     AString ret = this->normalized(QString::NormalizationForm_C);//first, normalize multi-char forms to their combined equivalents, etc, tons of nasties
     for (int i = 0; i < ret.length(); ++i)
@@ -759,11 +831,11 @@ AString AString::fixUnicodeHyphens(bool* hyphenReplaced, bool* hadOtherNonAscii)
                 charCode == 65123 || //small hyphen-minus
                 charCode == 65293)//
             {
-                CaretLogFine("character code " + AString::number(charCode) + " replaced with ascii dash");
+                if (!quiet) CaretLogFine("character code " + AString::number(charCode) + " replaced with ascii dash");
                 ret[i] = '-';
                 if (hyphenReplaced != NULL) *hyphenReplaced = true;
             } else {//other stuff
-                CaretLogInfo("non-ascii character code " + AString::number(charCode) + " not recognized as dash/hyphen/minus");
+                if (!quiet) CaretLogInfo("non-ascii character code " + AString::number(charCode) + " not recognized as dash/hyphen/minus");
                 if (hadOtherNonAscii != NULL) *hadOtherNonAscii = true;
             }
         }
@@ -913,4 +985,158 @@ AString::countMatchingCharactersFromEnd(const AString& rhs) const
     return matchCount;
 }
 
+/**
+ * Find the longest common prefix in the given vector of strings.
+ *
+ * @param v
+ *     A vector of strings
+ * @return
+ *     Longest common prefix found (may be an empty string).
+ */
+AString
+AString::findLongestCommonPrefix(const std::vector<AString>& stringVector)
+{
+    AString longestPrefix;
+    
+    const int32_t numStrings = static_cast<int32_t>(stringVector.size());
+    if (numStrings == 1) {
+        CaretAssertVectorIndex(stringVector, 0);
+        longestPrefix = stringVector[0];
+    }
+    else if (numStrings > 1) {
+        /*
+         * Find string with shortest name
+         */
+        CaretAssertVectorIndex(stringVector, 0);
+        AString shortestString = stringVector[0];
+        for (const auto& name : stringVector) {
+            if (name.length() < shortestString.length()) {
+                shortestString = name;
+            }
+        }
+        
+        /*
+         * Loop through each character in the shortest string
+         * until a character does not match in any strings
+         */
+        const int32_t numChars = static_cast<int32_t>(shortestString.length());
+        for (int32_t ich = 0; ich < numChars; ich++) {
+            const QChar character = shortestString.at(ich);
+            
+            bool allMatchFlag = true;
+            for (const auto& name : stringVector) {
+                CaretAssert(ich  < name.length());
+                if (name.at(ich) != character) {
+                    allMatchFlag = false;
+                }
+            }
+            
+            if (allMatchFlag) {
+                longestPrefix.append(character);
+            }
+            else {
+                break;
+            }
+        }
+    }
+    
+    /*
+     * If the last character is a '/', remove it
+     */
+    if (longestPrefix.length() > 1) {
+        if (longestPrefix.endsWith('/')) {
+            longestPrefix.resize(longestPrefix.length() - 1);
+        }
+    }
+    
+    const bool debugFlag = false;
+    if (debugFlag) {
+        std::cout << "Longest prefix: " << longestPrefix << std::endl;
+        for (const auto& name : stringVector) {
+            std::cout << "   " << name << std::endl;
+        }
+    }
+    
+    return longestPrefix;
+}
+
+/**
+ * Convert a QStringList to an std::vector of AStrings.
+ *
+ * @param stringList
+ *     The QStringList
+ * @return
+ *     Vector with elements copied from the string list.
+ */
+std::vector<AString>
+AString::stringListToVector(const QStringList& stringList)
+{
+    std::vector<AString> sv(stringList.begin(),
+                            stringList.end());
+    return sv;
+}
+
+/**
+ * Join the elements from the string vector adding 'separator'
+ * between each of the elements.
+ *
+ * @param elements
+ *     Elements joined to form a string.
+ * @param separtor
+ *     Characters added between each pair of elements.
+ */
+AString
+AString::join(const std::vector<AString>& elements,
+              const AString& separator)
+{
+    AString joinedString;
+    
+    const int32_t numElements = static_cast<int32_t>(elements.size());
+    for (int32_t i = 0; i < numElements; i++) {
+        if (i > 0) {
+            joinedString.append(separator);
+        }
+        CaretAssertVectorIndex(elements, i);
+        joinedString.append(elements[i]);
+    }
+    
+    return joinedString;
+}
+
+/**
+ * Count the number of corresponding matching elements from the two vectors.
+ * Compares first element in each, second element in each, etc and stops
+ * when the elements at the same index are different or no more
+ * elements in one of the vectors.
+ *
+ * @param v1
+ *     The first vector.
+ * @param v2
+ *     The second vector.
+ * @return
+ *     Number of matching elements.
+ */
+int32_t
+AString::matchingCount(const std::vector<AString>& v1,
+                       const std::vector<AString>& v2)
+{
+    const int32_t count = std::min(v1.size(),
+                                   v2.size());
+    if (count <= 0) {
+        return 0;
+    }
+    
+    int32_t matchCount = count;
+
+    for (int32_t i = 0; i < count; i++) {
+        CaretAssertVectorIndex(v1, i);
+        CaretAssertVectorIndex(v2, i);
+        if (v1[i] != v2[i]) {
+            matchCount = i;
+            break;
+        }
+    }
+    
+    return matchCount;
+}
 
