@@ -25,8 +25,14 @@
 
 namespace caret {
     
+    class ProgramParameters;
+    
     extern AString caret_global_commandLine;
     
+    void caret_global_commandLine_init(const ProgramParameters& params);
+    
+    void caret_global_commandLine_init(const int& argc, const char *const * argv);
+    
 }
 
 #endif //__CARET_COMMAND_LINE_H__
