@@ -34,6 +34,7 @@
 #include "CaretLogger.h"
 #include "EventAlertUser.h"
 #include "EventListenerInterface.h"
+#include "SystemUtilities.h"
 
 using namespace caret;
 /**
@@ -175,12 +176,6 @@ EventManager::addEventListener(EventListenerInterface* eventListener,
 #else
     INTENTIONAL_COMPILER_ERROR_MISSING_CONTAINER_TYPE
 #endif
-    
-    //std::cout << "Adding listener from class "
-    //<< typeid(*eventListener).name()
-    //<< " for "
-    //<< EventTypeEnum::toName(listenForEventType)
-    //<< std::endl;
 }
 
 /**
@@ -205,12 +200,6 @@ EventManager::addProcessedEventListener(EventListenerInterface* eventListener,
 #else
     INTENTIONAL_COMPILER_ERROR_MISSING_CONTAINER_TYPE
 #endif
-    
-    //std::cout << "Adding listener from class "
-    //<< typeid(*eventListener).name()
-    //<< " for "
-    //<< EventTypeEnum::toName(listenForEventType)
-    //<< std::endl;
 }
 
 /**
@@ -248,59 +237,6 @@ EventManager::removeEventFromListener(EventListenerInterface* eventListener,
     if (processedEventIter != processedListeners.end()) {
         processedListeners.erase(processedEventIter);
     }
-    
-//    EVENT_LISTENER_CONTAINER listeners = m_eventListeners[listenForEventType];
-//    
-//    /*
-//     * Remove the listener by creating a new container
-//     * of non-matching listeners.
-//     */
-//    EVENT_LISTENER_CONTAINER updatedListeners;
-//    for (EVENT_LISTENER_CONTAINER_ITERATOR iter = listeners.begin();
-//         iter != listeners.end();
-//         iter++) {
-//        if (*iter == eventListener) {
-//            //std::cout << "Removing listener from class "
-//            //<< typeid(*eventListener).name()
-//            //<< " for "
-//            //<< EventTypeEnum::toName(listenForEventType)
-//            //<< std::endl;
-//        }
-//        else {
-//            updatedListeners.push_back(*iter);
-//        }
-//    }
-//    
-//    if (updatedListeners.size() != listeners.size()) {
-//        m_eventListeners[listenForEventType] = updatedListeners;
-//    }
-//    
-//    
-//    EVENT_LISTENER_CONTAINER processedListeners = m_eventProcessedListeners[listenForEventType];
-//    
-//    /*
-//     * Remove the listener by creating a new container
-//     * of non-matching listeners.
-//     */
-//    EVENT_LISTENER_CONTAINER updatedProcessedListeners;
-//    for (EVENT_LISTENER_CONTAINER_ITERATOR iter = processedListeners.begin();
-//         iter != processedListeners.end();
-//         iter++) {
-//        if (*iter == eventListener) {
-//            //std::cout << "Removing listener from class "
-//            //<< typeid(*eventListener).name()
-//            //<< " for "
-//            //<< EventTypeEnum::toName(listenForEventType)
-//            //<< std::endl;
-//        }
-//        else {
-//            updatedProcessedListeners.push_back(*iter);
-//        }
-//    }
-//    
-//    if (updatedProcessedListeners.size() != processedListeners.size()) {
-//        m_eventProcessedListeners[listenForEventType] = updatedProcessedListeners;
-//    }
 #elif CONTAINER_HASH_SET
     m_eventListeners[listenForEventType].erase(eventListener);
     m_eventProcessedListeners[listenForEventType].erase(eventListener);
@@ -374,11 +310,6 @@ EventManager::sendEvent(Event* event)
         
         const AString eventNumberString = AString::number(m_eventIssuedCounter);
         
-        // Too many prints (JWH)
-        //AString msg = (eventMessagePrefix + " SENT.");
-        //CaretLogFiner(msg);
-        //std::cout << msg << std::endl;
-        
         /*
          * Send event to each of the listeners.
          */
@@ -386,14 +317,7 @@ EventManager::sendEvent(Event* event)
              iter != listeners.end();
              iter++) {
             EventListenerInterface* listener = *iter;
-            
-            //std::cout << "Sending event from class "
-            //<< typeid(*listener).name()
-            //<< " for "
-            //<< EventTypeEnum::toName(eventType)
-            //<< std::endl;
-            
-            
+
             listener->receiveEvent(event);
             
             if (event->isError()) {
@@ -414,14 +338,6 @@ EventManager::sendEvent(Event* event)
                  iter != processedListeners.end();
                  iter++) {
                 EventListenerInterface* listener = *iter;
-                
-                //std::cout << "Sending event from class "
-                //<< typeid(*listener).name()
-                //<< " for "
-                //<< EventTypeEnum::toName(eventType)
-                //<< std::endl;
-                
-                
                 listener->receiveEvent(event);
                 
                 if (event->isError()) {
@@ -431,7 +347,6 @@ EventManager::sendEvent(Event* event)
             }
         }
         else {
-            // Too many prints (JWH) CaretLogFine("Event " + eventNumberString + " not processed: " + event->toString());
         }
 
         m_eventIssuedCounter++;
@@ -452,8 +367,14 @@ EventManager::sendSimpleEvent(const EventTypeEnum::Enum eventType)
 {
     
     switch (eventType) {
+        /*
+         * Simple Events (no Event subclass)
+         */
         case EventTypeEnum::EVENT_ANNOTATION_TOOLBAR_UPDATE:
         case EventTypeEnum::EVENT_BROWSER_WINDOW_MENUS_UPDATE:
+        case EventTypeEnum::EVENT_MOVIE_RECORDING_DIALOG_UPDATE:
+        case EventTypeEnum::EVENT_TOOLBAR_CHART_ORIENTED_AXES_UPDATE:
+        case EventTypeEnum::EVENT_UPDATE_VOLUME_SLICE_INDICES_COORDS_TOOLBAR:
         {
             sendEvent(Event(eventType).getPointer());
         }
@@ -467,68 +388,132 @@ EventManager::sendSimpleEvent(const EventTypeEnum::Enum eventType)
             CaretLogSevere(msg);
         }
             break;
+        /*
+         * Events that have a subclass of Event
+         */
         case EventTypeEnum::EVENT_ALERT_USER:
         case EventTypeEnum::EVENT_ANNOTATION_ADD_TO_REMOVE_FROM_FILE:
-        case EventTypeEnum::EVENT_ANNOTATION_COLOR_BAR_GET:
+        case EventTypeEnum::EVENT_ANNOTATION_BARS_GET:
+        case EventTypeEnum::EVENT_ANNOTATION_CHART_LABEL_GET:
         case EventTypeEnum::EVENT_ANNOTATION_CREATE_NEW_TYPE:
+        case EventTypeEnum::EVENT_ANNOTATION_DRAWING_FINISH_CANCEL:
+        case EventTypeEnum::EVENT_ANNOTATION_GET_BEING_DRAWN_IN_WINDOW:
         case EventTypeEnum::EVENT_ANNOTATION_GET_DRAWN_IN_WINDOW:
+        case EventTypeEnum::EVENT_ANNOTATION_GET_SELECTED_INSERT_NEW_FILE:
         case EventTypeEnum::EVENT_ANNOTATION_GROUP_GET_WITH_KEY:
         case EventTypeEnum::EVENT_ANNOTATION_GROUPING:
+        case EventTypeEnum::EVENT_ANNOTATION_NEW_DRAWING_POLYHEDRON_SLICE_DEPTH:
+        case EventTypeEnum::EVENT_ANNOTATION_POLYHEDRON_GET_BY_LINKED_IDENTIFIER:
+        case EventTypeEnum::EVENT_ANNOTATION_POLYHEDRON_NAME_COMPONENT_SETTINGS:
+        case EventTypeEnum::EVENT_ANNOTATION_TEXT_GET_BOUNDS:
+        case EventTypeEnum::EVENT_ANNOTATION_TEXT_SUBSTITUTION_GET:
+        case EventTypeEnum::EVENT_ANNOTATION_TEXT_SUBSTITUTION_GET_ALL_GROUP_IDS:
+        case EventTypeEnum::EVENT_ANNOTATION_TEXT_SUBSTITUTION_INVALIDATE:
+        case EventTypeEnum::EVENT_ANNOTATION_VALIDATE:
         case EventTypeEnum::EVENT_BRAIN_RESET:
         case EventTypeEnum::EVENT_BRAIN_STRUCTURE_GET_ALL:
+        case EventTypeEnum::EVENT_BROWSER_TAB_CLOSE:
+        case EventTypeEnum::EVENT_BROWSER_TAB_CLOSE_IN_TOOL_BAR:
         case EventTypeEnum::EVENT_BROWSER_TAB_DELETE:
+        case EventTypeEnum::EVENT_BROWSER_TAB_DELETE_IN_TOOL_BAR:
         case EventTypeEnum::EVENT_BROWSER_TAB_GET:
         case EventTypeEnum::EVENT_BROWSER_TAB_GET_ALL:
         case EventTypeEnum::EVENT_BROWSER_TAB_GET_ALL_VIEWED:
+        case EventTypeEnum::EVENT_BROWSER_TAB_GET_AT_WINDOW_XY:
+        case EventTypeEnum::EVENT_BROWSER_TAB_INDEX_GET_WINDOW_INDEX:
         case EventTypeEnum::EVENT_BROWSER_TAB_INDICES_GET_ALL:
+        case EventTypeEnum::EVENT_BROWSER_TAB_INDICES_GET_ALL_VIEWED:
         case EventTypeEnum::EVENT_BROWSER_TAB_NEW:
-        case EventTypeEnum::EVENT_BROWSER_WINDOW_CONTENT_GET:
+        case EventTypeEnum::EVENT_BROWSER_TAB_NEW_IN_GUI:
+        case EventTypeEnum::EVENT_BROWSER_TAB_NEW_CLONE:
+        case EventTypeEnum::EVENT_BROWSER_TAB_REOPEN_AVAILBLE:
+        case EventTypeEnum::EVENT_BROWSER_TAB_REOPEN_CLOSED:
+        case EventTypeEnum::EVENT_BROWSER_TAB_SELECT_IN_WINDOW:
+        case EventTypeEnum::EVENT_BROWSER_TAB_VALIDATE:
+        case EventTypeEnum::EVENT_BROWSER_WINDOW_CONTENT:
         case EventTypeEnum::EVENT_BROWSER_WINDOW_CREATE_TABS:
+        case EventTypeEnum::EVENT_BROWSER_WINDOW_GET_TABS:
+        case EventTypeEnum::EVENT_BROWSER_WINDOW_DRAWING_CONTENT_GET:
         case EventTypeEnum::EVENT_BROWSER_WINDOW_GRAPHICS_HAVE_BEEN_REDRAWN:
         case EventTypeEnum::EVENT_BROWSER_WINDOW_NEW:
+        case EventTypeEnum::EVENT_BROWSER_WINDOW_PIXEL_SIZE_INFO:
+        case EventTypeEnum::EVENT_BROWSER_WINDOW_TILE_TAB_OPERATION:
+        case EventTypeEnum::EVENT_CARET_DATA_FILES_GET:
         case EventTypeEnum::EVENT_CARET_MAPPABLE_DATA_FILES_GET:
         case EventTypeEnum::EVENT_CARET_MAPPABLE_DATA_FILE_MAPS_VIEWED_IN_OVERLAYS:
+        case EventTypeEnum::EVENT_CARET_PREFERENCES_GET:
+        case EventTypeEnum::EVENT_CARET_MAPPABLE_DATA_FILES_AND_MAPS_IN_DISPLAYED_OVERLAYS:
         case EventTypeEnum::EVENT_CHART_MATRIX_YOKING_VALIDATION:
+        case EventTypeEnum::EVENT_CHART_TWO_CARTEISAN_AXIS_DISPLAY_GROUP:
+        case EventTypeEnum::EVENT_CHART_TWO_CARTESIAN_ORIENTED_AXES_YOKING:
+        case EventTypeEnum::EVENT_CHART_TWO_LOAD_LINE_SERIES_DATA:
+        case EventTypeEnum::EVENT_CHART_TWO_OVERLAY_VALIDATE:
         case EventTypeEnum::EVENT_DATA_FILE_ADD:
         case EventTypeEnum::EVENT_DATA_FILE_DELETE:
         case EventTypeEnum::EVENT_DATA_FILE_READ:
         case EventTypeEnum::EVENT_DATA_FILE_RELOAD:
+        case EventTypeEnum::EVENT_DATA_FILE_RELOAD_ALL:
+        case EventTypeEnum::EVENT_DRAWING_VIEWPORT_CONTENT_ADD:
+        case EventTypeEnum::EVENT_DRAWING_VIEWPORT_CONTENT_CLEAR:
+        case EventTypeEnum::EVENT_DRAWING_VIEWPORT_CONTENT_GET:
+        case EventTypeEnum::EVENT_FOCUS_FILE_GET_COLOR:
         case EventTypeEnum::EVENT_GET_DISPLAYED_DATA_FILES:
         case EventTypeEnum::EVENT_GET_NODE_DATA_FILES:
         case EventTypeEnum::EVENT_GET_OR_SET_USER_INPUT_MODE:
         case EventTypeEnum::EVENT_GET_TEXT_RENDERER_FOR_WINDOW:
+        case EventTypeEnum::EVENT_GET_USER_INPUT_MODE:
         case EventTypeEnum::EVENT_GET_VIEWPORT_SIZE:
-        case EventTypeEnum::EVENT_GRAPHICS_UPDATE_ALL_WINDOWS:
-        case EventTypeEnum::EVENT_GRAPHICS_UPDATE_ONE_WINDOW:
+        case EventTypeEnum::EVENT_GRAPHICS_OPENGL_CREATE_BUFFER_OBJECT:
+        case EventTypeEnum::EVENT_GRAPHICS_OPENGL_CREATE_TEXTURE_NAME:
+        case EventTypeEnum::EVENT_GRAPHICS_OPENGL_DELETE_BUFFER_OBJECT:
+        case EventTypeEnum::EVENT_GRAPHICS_OPENGL_DELETE_TEXTURE_NAME:
+        case EventTypeEnum::EVENT_GRAPHICS_PAINT_NOW_ALL_WINDOWS:
+        case EventTypeEnum::EVENT_GRAPHICS_PAINT_NOW_ONE_WINDOW:
+        case EventTypeEnum::EVENT_GRAPHICS_TIMING_ONE_WINDOW:
+        case EventTypeEnum::EVENT_GRAPHICS_PAINT_SOON_ALL_WINDOWS:
+        case EventTypeEnum::EVENT_GRAPHICS_PAINT_SOON_ONE_WINDOW:
+        case EventTypeEnum::EVENT_GRAPHICS_WINDOW_SHOW_TOOL_TIP:
         case EventTypeEnum::EVENT_HELP_VIEWER_DISPLAY:
+        case EventTypeEnum::EVENT_HISTOLOGY_SLICES_FILES_GET:
         case EventTypeEnum::EVENT_IDENTIFICATION_HIGHLIGHT_LOCATION:
         case EventTypeEnum::EVENT_IDENTIFICATION_SYMBOL_REMOVAL:
         case EventTypeEnum::EVENT_IDENTIFICATION_REQUEST:
         case EventTypeEnum::EVENT_IMAGE_CAPTURE:
-        case EventTypeEnum::EVENT_MAC_DOCK_MENU_UPDATE:
         case EventTypeEnum::EVENT_MAP_YOKING_SELECT_MAP:
         case EventTypeEnum::EVENT_MAP_YOKING_VALIDATION:
+        case EventTypeEnum::EVENT_MEDIA_FILES_GET:
         case EventTypeEnum::EVENT_MODEL_ADD:
         case EventTypeEnum::EVENT_MODEL_DELETE:
         case EventTypeEnum::EVENT_MODEL_GET_ALL:
+        case EventTypeEnum::EVENT_MODEL_GET_ALL_DISPLAYED:
         case EventTypeEnum::EVENT_MODEL_SURFACE_GET:
+        case EventTypeEnum::EVENT_MOVIE_RECORDING_MANUAL_MODE_CAPTURE:
         case EventTypeEnum::EVENT_NODE_IDENTIFICATION_COLORS_GET_FROM_CHARTS:
-        case EventTypeEnum::EVENT_OPENGL_TEXTURE:
+        case EventTypeEnum::EVENT_OPENGL_OBJECT_TO_WINDOW_TRANSFORM:
         case EventTypeEnum::EVENT_OPERATING_SYSTEM_REQUEST_OPEN_DATA_FILE:
         case EventTypeEnum::EVENT_OVERLAY_SETTINGS_EDITOR_SHOW:
         case EventTypeEnum::EVENT_OVERLAY_VALIDATE:
         case EventTypeEnum::EVENT_PALETTE_COLOR_MAPPING_EDITOR_SHOW:
         case EventTypeEnum::EVENT_PALETTE_GET_BY_NAME:
+        case EventTypeEnum::EVENT_PALETTE_GROUPS_GET:
+        case EventTypeEnum::EVENT_RECENT_FILES_SYSTEM_ACCESS_MODE:
+        case EventTypeEnum::EVENT_RESET_VIEW:
+        case EventTypeEnum::EVENT_SCENE_ACTIVE:
+        case EventTypeEnum::EVENT_SHOW_FILE_DATA_READ_WARNING_DIALOG:
+        case EventTypeEnum::EVENT_SPACER_TAB_GET:
         case EventTypeEnum::EVENT_SPEC_FILE_READ_DATA_FILES:
         case EventTypeEnum::EVENT_SURFACE_COLORING_INVALIDATE:
+        case EventTypeEnum::EVENT_VOLUME_COLORING_INVALIDATE:
         case EventTypeEnum::EVENT_SURFACES_GET:
+        case EventTypeEnum::EVENT_SURFACE_FILE_GET:
         case EventTypeEnum::EVENT_SURFACE_STRUCTURES_VALID_GET:
+        case EventTypeEnum::EVENT_TILE_TABS_MODIFICATION:
         case EventTypeEnum::EVENT_TOOLBOX_SELECTION_DISPLAY:
         case EventTypeEnum::EVENT_USER_INTERFACE_UPDATE:
         case EventTypeEnum::EVENT_PROGRESS_UPDATE:
         case EventTypeEnum::EVENT_UPDATE_INFORMATION_WINDOWS:
-        case EventTypeEnum::EVENT_UPDATE_YOKED_WINDOWS:
         case EventTypeEnum::EVENT_UPDATE_VOLUME_EDITING_TOOLBAR:
+        case EventTypeEnum::EVENT_UPDATE_YOKED_WINDOWS:
         {
             const AString msg(EventTypeEnum::toName(eventType)
                               + " has an special subclass of class Event and should never be sent as an event.");
@@ -599,4 +584,35 @@ EventManager::getEventIssuedCounter() const
     return m_eventIssuedCounter;
 }
 
+/**
+ * Verify that all listeners have been removed from the given event listener.
+ *
+ * @param eventListener
+ *     The event listener.
+ */
+void
+EventManager::verifyAllListenersRemoved(EventListenerInterface* eventListener)
+{
+    AString eventNames;
+    
+    for (int32_t i = 0; i < EventTypeEnum::EVENT_COUNT; i++) {
+        const EventTypeEnum::Enum eventType = static_cast<EventTypeEnum::Enum>(i);
+        if ((m_eventListeners[eventType].find(eventListener) != m_eventListeners[eventType].end())
+            || (m_eventProcessedListeners[eventType].find(eventListener) != m_eventProcessedListeners[eventType].end())) {
+            eventNames.appendWithNewLine("    "
+                                  + EventTypeEnum::toName(eventType));
+        }
+    }
+    
+    if ( ! eventNames.isEmpty()) {
+        SystemBacktrace myBacktrace;
+        SystemUtilities::getBackTrace(myBacktrace);
+        
+        CaretLogSevere("Failed to remove events from class instance.  Event names:\n"
+                       + eventNames
+                       + ":\n"
+                       + myBacktrace.toSymbolString());
+    }
+}
+
 
