@@ -99,6 +99,8 @@ namespace caret {
         
         virtual ~EventManager();
         
+        void verifyAllListenersRemoved(EventListenerInterface* eventListener);
+        
         /**
          * Define the container
          */
@@ -137,6 +139,8 @@ namespace caret {
         
         static EventManager* s_singletonEventManager;
         
+        friend EventListenerInterface;
+        
     };
     
 #ifdef __EVENT_MANAGER_MAIN__
