@@ -78,18 +78,30 @@ EventTypeEnum::initialize()
                                      "EVENT_ANNOTATION_CHART_LABEL_GET",
                                      "Event to get annotation chart labels"));
     
-    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_COLOR_BAR_GET,
-                                     "EVENT_ANNOTATION_COLOR_BAR_GET",
-                                     "Event to get annotation color bars from tab(s)"));
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_BARS_GET,
+                                     "EVENT_ANNOTATION_BARS_GET",
+                                     "Event to get annotation color and scale bars from tab(s)"));
     
     enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_CREATE_NEW_TYPE,
                                      "EVENT_ANNOTATION_CREATE_NEW_TYPE",
                                      "Event to create a new annotation of a particular type"));
     
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_GET_BEING_DRAWN_IN_WINDOW,
+                                     "EVENT_ANNOTATION_GET_BEING_DRAWN_IN_WINDOW",
+                                     "Event for getting annotation being drawn in a window"));
+    
     enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_GET_DRAWN_IN_WINDOW,
                                      "EVENT_ANNOTATION_GET_DRAWN_IN_WINDOW",
                                      "Event for getting annotations drawn in a window"));
     
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_DRAWING_FINISH_CANCEL,
+                                     "EVENT_ANNOTATION_DRAWING_FINISH_CANCEL",
+                                     "Event for finishing or cancel annotation being drawn in window"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_GET_SELECTED_INSERT_NEW_FILE,
+                                     "EVENT_ANNOTATION_GET_SELECTED_INSERT_NEW_FILE",
+                                     "Event for getting file selecte in 'insert new' section of toolbar"));
+
     enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_GROUP_GET_WITH_KEY,
                                      "EVENT_ANNOTATION_GROUP_GET_WITH_KEY",
                                      "Event for getting an annotation group using its key"));
@@ -98,10 +110,30 @@ EventTypeEnum::initialize()
                                      "EVENT_ANNOTATION_GROUPING",
                                      "Event for annotation grouping"));
     
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_NEW_DRAWING_POLYHEDRON_SLICE_DEPTH,
+                                     "EVENT_ANNOTATION_NEW_DRAWING_POLYHEDRON_SLICE_DEPTH",
+                                     "Get slice depth while drawing a new polyhedron"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_POLYHEDRON_GET_BY_LINKED_IDENTIFIER,
+                                     "EVENT_ANNOTATION_POLYHEDRON_GET_BY_LINKED_IDENTIFIER",
+                                     "Get an annotation polyhedron using its linked identifer"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_POLYHEDRON_NAME_COMPONENT_SETTINGS,
+                                     "EVENT_ANNOTATION_POLYHEDRON_NAME_COMPONENT_SETTINGS",
+                                     "Event for annotation polyhedron name component settings"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_TEXT_GET_BOUNDS,
+                                     "EVENT_ANNOTATION_TEXT_GET_BOUNDS",
+                                     "Get bounds for annotation text"));
+    
     enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_TEXT_SUBSTITUTION_GET,
                                      "EVENT_ANNOTATION_TEXT_SUBSTITUTION_GET",
                                      "Get an annotation text substitution"));
     
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_TEXT_SUBSTITUTION_GET_ALL_GROUP_IDS,
+                                     "EVENT_ANNOTATION_TEXT_SUBSTITUTION_GET_ALL_GROUP_IDS",
+                                     "Get all annotation text substitution group identifiers"));
+    
     enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_TEXT_SUBSTITUTION_INVALIDATE,
                                      "EVENT_ANNOTATION_TEXT_SUBSTITUTION_INVALIDATE",
                                      "Invalidate text substitutions in text annotations"));
@@ -110,6 +142,10 @@ EventTypeEnum::initialize()
                                      "EVENT_ANNOTATION_TOOLBAR_UPDATE",
                                      "Event to update annotation toolbar"));
     
+    enumData.push_back(EventTypeEnum(EVENT_ANNOTATION_VALIDATE,
+                                     "EVENT_ANNOTATION_VALIDATE",
+                                     "Verify that an annotation is valid (valid pointer to annotation"));
+
     enumData.push_back(EventTypeEnum(EVENT_BRAIN_RESET,
                                      "EVENT_BRAIN_RESET",
                                      "Brain has been reset"));
@@ -118,11 +154,23 @@ EventTypeEnum::initialize()
                                      "EVENT_BRAIN_STRUCTURE_GET_ALL",
                                      "Get all brain structures"));
     
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_CLOSE,
+                                     "EVENT_BROWSER_TAB_CLOSE",
+                                     "Close a browser tab, may reopen later"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_CLOSE_IN_TOOL_BAR,
+                                     "EVENT_BROWSER_TAB_CLOSE_IN_TOOL_BAR",
+                                     "Close (for possible reopening) a browser tab in the tool bar"));
+    
     enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_DELETE,
                                      "EVENT_BROWSER_TAB_DELETE", 
                                      "Delete a browser tab"));
     
-    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_GET, 
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_DELETE_IN_TOOL_BAR,
+                                     "EVENT_BROWSER_TAB_DELETE_IN_TOOL_BAR",
+                                     "Delete a browser tab in the tool bar"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_GET,
                                     "EVENT_BROWSER_TAB_GET", 
                                     "Get a browser tab by number"));
     
@@ -138,10 +186,46 @@ EventTypeEnum::initialize()
                                      "EVENT_BROWSER_TAB_INDICES_GET_ALL",
                                      "Browser Tab Indices Get All"));
     
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_INDICES_GET_ALL_VIEWED,
+                                     "EVENT_BROWSER_TAB_INDICES_GET_ALL_VIEWED",
+                                     "Browser Tab Indices Get All Viewed"));
+    
     enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_NEW,
                                      "EVENT_BROWSER_TAB_NEW", 
                                      "Create a browser tab"));
     
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_GET_AT_WINDOW_XY,
+                                     "EVENT_BROWSER_TAB_GET_AT_WINDOW_XY",
+                                     "Get the browser tab at the window's XY"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_INDEX_GET_WINDOW_INDEX,
+                                     "EVENT_BROWSER_TAB_INDEX_GET_WINDOW_INDEX",
+                                     "Get index of window containing tab with index"));
+                       
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_NEW_IN_GUI,
+                                     "EVENT_BROWSER_TAB_NEW_IN_GUI",
+                                     "Create a browser tab from/in the GUI so toolbar is updated"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_NEW_CLONE,
+                                     "EVENT_BROWSER_TAB_NEW_CLONE",
+                                     "Create a browser tab by cloning an existing browser tab"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_REOPEN_AVAILBLE,
+                                     "EVENT_BROWSER_TAB_REOPEN_AVAILBLE",
+                                     "Event for getting available closed browser tab(s)"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_REOPEN_CLOSED,
+                                     "EVENT_BROWSER_TAB_REOPEN_CLOSED",
+                                     "Event for reopening a closed browser tab"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_SELECT_IN_WINDOW,
+                                     "EVENT_BROWSER_TAB_SELECT_IN_WINDOW",
+                                     "Event for selecting a browser tab in a window"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_TAB_VALIDATE,
+                                     "EVENT_BROWSER_TAB_VALIDATE",
+                                     "Event for validate a browser tab still exists"));
+
     enumData.push_back(EventTypeEnum(EVENT_BROWSER_WINDOW_CONTENT,
                                      "EVENT_BROWSER_WINDOW_CONTENT",
                                      "Event for browser window content"));
@@ -154,6 +238,10 @@ EventTypeEnum::initialize()
                                      "EVENT_BROWSER_WINDOW_CREATE_TABS", 
                                      "Create tabs (if needed) after loading data files"));
     
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_WINDOW_GET_TABS,
+                                     "EVENT_BROWSER_WINDOW_GET_TABS",
+                                     "Get the tabs in a browser window"));
+    
     enumData.push_back(EventTypeEnum(EVENT_BROWSER_WINDOW_GRAPHICS_HAVE_BEEN_REDRAWN,
                                      "EVENT_BROWSER_WINDOW_GRAPHICS_HAVE_BEEN_REDRAWN",
                                      "A Browser Window's graphics have been redrawn"));
@@ -165,6 +253,10 @@ EventTypeEnum::initialize()
     enumData.push_back(EventTypeEnum(EVENT_BROWSER_WINDOW_NEW,
                                      "EVENT_BROWSER_WINDOW_NEW", 
                                      "Create a new browser window"));
+
+    enumData.push_back(EventTypeEnum(EVENT_BROWSER_WINDOW_PIXEL_SIZE_INFO,
+                                     "EVENT_BROWSER_WINDOW_PIXEL_SIZE_INFO",
+                                     "Info about a window's pixel sizes"));
     
     enumData.push_back(EventTypeEnum(EVENT_BROWSER_WINDOW_TILE_TAB_OPERATION,
                                      "EVENT_BROWSER_WINDOW_TILE_TAB_OPERATION",
@@ -182,6 +274,10 @@ EventTypeEnum::initialize()
                                      "EVENT_CARET_MAPPABLE_DATA_FILE_MAPS_VIEWED_IN_OVERLAYS",
                                      "Get Caret Mappable data file maps viewed in overlays"));
 
+    enumData.push_back(EventTypeEnum(EVENT_CARET_MAPPABLE_DATA_FILES_AND_MAPS_IN_DISPLAYED_OVERLAYS,
+                                     "EVENT_CARET_MAPPABLE_DATA_FILES_AND_MAPS_IN_DISPLAYED_OVERLAYS",
+                                     "Get all selected mappable data and map indices in displayed overlays"));
+                       
     enumData.push_back(EventTypeEnum(EVENT_CARET_PREFERENCES_GET,
                                      "EVENT_CARET_PREFERENCES_GET",
                                      "Get the Caret Preferences"));
@@ -202,22 +298,23 @@ EventTypeEnum::initialize()
                                      "EVENT_GRAPHICS_OPENGL_DELETE_TEXTURE_NAME",
                                      "Delete an OpenGL Texture Name for an OpenGL Context"));
 
-    enumData.push_back(EventTypeEnum(EVENT_CHART_OVERLAY_VALIDATE,
-                                     "EVENT_CHART_OVERLAY_VALIDATE",
-                                     "Validate a chart overlay for validity (it exists)"));
     
-    enumData.push_back(EventTypeEnum(EVENT_CHART_TWO_ATTRIBUTES_CHANGED,
-                                     "EVENT_CHART_TWO_ATTRIBUTES_CHANGED",
-                                     "GUI notification of the change in chart two atttributes"));
-    
-    enumData.push_back(EventTypeEnum(EVENT_CHART_TWO_AXIS_GET_DATA_RANGE,
-                                     "EVENT_CHART_TWO_AXIS_GET_DATA_RANGE",
-                                     "Get the range of data for a chart two axis"));
+    enumData.push_back(EventTypeEnum(EVENT_CHART_TWO_CARTEISAN_AXIS_DISPLAY_GROUP,
+                                     "EVENT_CHART_TWO_CARTEISAN_AXIS_DISPLAY_GROUP",
+                                     "Get a cartesian axis for a display group"));
     
+    enumData.push_back(EventTypeEnum(EVENT_CHART_TWO_CARTESIAN_ORIENTED_AXES_YOKING,
+                                     "EVENT_CHART_TWO_CARTESIAN_ORIENTED_AXES_YOKING",
+                                     "Get/set/query chart two cartesian axes yoking"));
+
     enumData.push_back(EventTypeEnum(EVENT_CHART_TWO_LOAD_LINE_SERIES_DATA,
                                      "EVENT_CHART_TWO_LOAD_LINE_SERIES_DATA",
                                      "Load line series data for chart two implementation"));
     
+    enumData.push_back(EventTypeEnum(EVENT_CHART_TWO_OVERLAY_VALIDATE,
+                                     "EVENT_CHART_TWO_OVERLAY_VALIDATE",
+                                     "Validate a chart two overlay for validity (it exists)"));
+    
     enumData.push_back(EventTypeEnum(EVENT_DATA_FILE_ADD,
                                      "EVENT_DATA_FILE_ADD",
                                      "Add a data file to the Brain"));
@@ -234,6 +331,26 @@ EventTypeEnum::initialize()
                                      "EVENT_DATA_FILE_RELOAD",
                                      "Reopen a data file (replace it with saved version) in the Brain"));
     
+    enumData.push_back(EventTypeEnum(EVENT_DATA_FILE_RELOAD_ALL,
+                                     "EVENT_DATA_FILE_RELOAD_ALL",
+                                     "Reopen all data files (replace it with saved version) in the Brain"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_DRAWING_VIEWPORT_CONTENT_ADD,
+                                     "EVENT_DRAWING_VIEWPORT_CONTENT_ADD",
+                                     "Event to add viewport while drawing"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_DRAWING_VIEWPORT_CONTENT_CLEAR,
+                                     "EVENT_DRAWING_VIEWPORT_CONTENT_CLEAR",
+                                     "Event to clear viewports for window before drawing"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_DRAWING_VIEWPORT_CONTENT_GET,
+                                     "EVENT_DRAWING_VIEWPORT_CONTENT_GET",
+                                     "Event get a drawing viewport after drawing"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_FOCUS_FILE_GET_COLOR,
+                                     "EVENT_FOCUS_FILE_GET_COLOR",
+                                     "Get a focus class or name color from a focus file"));
+    
     enumData.push_back(EventTypeEnum(EVENT_GET_DISPLAYED_DATA_FILES,
                                      "EVENT_GET_DISPLAYED_DATA_FILES",
                                      "Get data files displayed in windows/tabs"));
@@ -250,22 +367,46 @@ EventTypeEnum::initialize()
                                      "EVENT_GET_TEXT_RENDERER_FOR_WINDOW",
                                      "Get the text renderer for a window"));
     
+    enumData.push_back(EventTypeEnum(EVENT_GET_USER_INPUT_MODE,
+                                     "EVENT_GET_USER_INPUT_MODE",
+                                     "Get the user input mode for a window"));
+    
     enumData.push_back(EventTypeEnum(EVENT_GET_VIEWPORT_SIZE,
                                      "EVENT_GET_VIEWPORT_SIZE",
                                      "Get the viewport size"));
     
-    enumData.push_back(EventTypeEnum(EVENT_GRAPHICS_UPDATE_ALL_WINDOWS,
-                                     "EVENT_GRAPHICS_UPDATE_ALL_WINDOWS", 
-                                     "Update all graphics windows"));
+    enumData.push_back(EventTypeEnum(EVENT_GRAPHICS_TIMING_ONE_WINDOW,
+                                     "EVENT_GRAPHICS_TIMING_ONE_WINDOW",
+                                     "Graphics timing in one window"));
     
-    enumData.push_back(EventTypeEnum(EVENT_GRAPHICS_UPDATE_ONE_WINDOW, 
-                                     "EVENT_GRAPHICS_UPDATE_ONE_WINDOW", 
-                                     "Update graphics in one window"));
+    enumData.push_back(EventTypeEnum(EVENT_GRAPHICS_PAINT_NOW_ALL_WINDOWS,
+                                     "EVENT_GRAPHICS_PAINT_NOW_ALL_WINDOWS",
+                                     "Paint (draw immediately) graphics in all windows"));
     
+    enumData.push_back(EventTypeEnum(EVENT_GRAPHICS_PAINT_NOW_ONE_WINDOW,
+                                     "EVENT_GRAPHICS_PAINT_NOW_ONE_WINDOW",
+                                     "Paint (draw immediately) graphics in a window"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_GRAPHICS_PAINT_SOON_ALL_WINDOWS,
+                                     "EVENT_GRAPHICS_PAINT_SOON_ALL_WINDOWS", 
+                                     "Update (draw soon but not immediately) all graphics windows"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_GRAPHICS_PAINT_SOON_ONE_WINDOW, 
+                                     "EVENT_GRAPHICS_PAINT_SOON_ONE_WINDOW", 
+                                     "Update (draw soon but not immediately) graphics in one window"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_GRAPHICS_WINDOW_SHOW_TOOL_TIP,
+                                     "EVENT_GRAPHICS_WINDOW_SHOW_TOOL_TIP",
+                                     "Show tooltip in graphics window"));
+
     enumData.push_back(EventTypeEnum(EVENT_HELP_VIEWER_DISPLAY,
                                      "EVENT_HELP_VIEWER_DISPLAY", 
                                      "Display the help viewer"));
     
+    enumData.push_back(EventTypeEnum(EVENT_HISTOLOGY_SLICES_FILES_GET,
+                                     "EVENT_HISTOLOGY_SLICES_FILES_GET",
+                                     "Get histology slices files"));
+    
     enumData.push_back(EventTypeEnum(EVENT_IDENTIFICATION_HIGHLIGHT_LOCATION,
                                      "EVENT_IDENTIFICATION_HIGHLIGHT_LOCATION",
                                      "Highlight the location when identification takes place"));
@@ -282,10 +423,6 @@ EventTypeEnum::initialize()
                                      "EVENT_IMAGE_CAPTURE",
                                      "Capture an Image of Browser Window Graphics Region"));
 
-    enumData.push_back(EventTypeEnum(EVENT_MAC_DOCK_MENU_UPDATE,
-                                     "EVENT_MAC_DOCK_MENU_UPDATE",
-                                     "Update the Mac Dock Menu"));
-    
     enumData.push_back(EventTypeEnum(EVENT_MAP_YOKING_SELECT_MAP,
                                      "EVENT_MAP_YOKING_SELECT_MAP", 
                                      "Map Yoking Select Map"));
@@ -294,6 +431,10 @@ EventTypeEnum::initialize()
                                      "EVENT_MAP_YOKING_VALIDATION",
                                      "Map Yoking Validation"));
     
+    enumData.push_back(EventTypeEnum(EVENT_MEDIA_FILES_GET,
+                                     "EVENT_MEDIA_FILES_GET",
+                                     "Get media files"));
+    
     enumData.push_back(EventTypeEnum(EVENT_MODEL_ADD,
                                      "EVENT_MODEL_ADD",
                                      "Add a model"));
@@ -314,6 +455,14 @@ EventTypeEnum::initialize()
                                      "EVENT_MODEL_SURFACE_GET", 
                                      "Get a specific model surface"));
     
+    enumData.push_back(EventTypeEnum(EVENT_MOVIE_RECORDING_MANUAL_MODE_CAPTURE,
+                                     "EVENT_MOVIE_RECORDING_MANUAL_MODE_CAPTURE",
+                                     "Movie recording manual mode capture"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_MOVIE_RECORDING_DIALOG_UPDATE,
+                                     "EVENT_MOVIE_RECORDING_DIALOG_UPDATE",
+                                     "Update the movie recording dialog"));
+
     enumData.push_back(EventTypeEnum(EVENT_NODE_IDENTIFICATION_COLORS_GET_FROM_CHARTS,
                                      "EVENT_NODE_IDENTIFICATION_COLORS_GET_FROM_CHARTS",
                                      "Get the color for node identification symbols from all charts that contain nodes"));
@@ -342,10 +491,30 @@ EventTypeEnum::initialize()
                                      "EVENT_PALETTE_GET_BY_NAME",
                                      "Read the selected files in a spec file"));
     
+    enumData.push_back(EventTypeEnum(EVENT_PALETTE_GROUPS_GET,
+                                     "EVENT_PALETTE_GROUPS_GET",
+                                     "Get all palette groups"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_RECENT_FILES_SYSTEM_ACCESS_MODE,
+                                     "EVENT_RECENT_FILES_SYSTEM_ACCESS_MODE",
+                                     "Get file system access mode for recent files"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_RESET_VIEW,
+                                     "EVENT_RESET_VIEW",
+                                     "Reset the View"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_SCENE_ACTIVE,
+                                     "EVENT_SCENE_ACTIVE",
+                                     "Get/Set the active scene"));
+
     enumData.push_back(EventTypeEnum(EVENT_SHOW_FILE_DATA_READ_WARNING_DIALOG,
                                      "EVENT_SHOW_FILE_DATA_READ_WARNING_DIALOG",
                                      "Show a dialog with warnings encountered reading data files"));
     
+    enumData.push_back(EventTypeEnum(EVENT_SPACER_TAB_GET,
+                                     "EVENT_SPACER_TAB_GET",
+                                     "Get a spacer tagb"));
+    
     enumData.push_back(EventTypeEnum(EVENT_SPEC_FILE_READ_DATA_FILES,
                                      "EVENT_SPEC_FILE_READ_DATA_FILES",
                                      "Read the selected data files in a spec file"));
@@ -353,20 +522,36 @@ EventTypeEnum::initialize()
     enumData.push_back(EventTypeEnum(EVENT_SURFACE_COLORING_INVALIDATE, 
                                      "EVENT_SURFACE_COLORING_INVALIDATE", 
                                      "Invalidate surface coloring"));
+
+    enumData.push_back(EventTypeEnum(EVENT_VOLUME_COLORING_INVALIDATE,
+                                     "EVENT_VOLUME_COLORING_INVALIDATE",
+                                     "Invalidate volume coloring"));
     
     enumData.push_back(EventTypeEnum(EVENT_SURFACES_GET, 
                                      "EVENT_SURFACES_GET", 
                                      "Get Surfaces"));
     
+    enumData.push_back(EventTypeEnum(EVENT_SURFACE_FILE_GET,
+                                     "EVENT_SURFACE_FILE_GET",
+                                     "Get surface files with structure and number of nodes"));
+    
     enumData.push_back(EventTypeEnum(EVENT_SURFACE_STRUCTURES_VALID_GET,
                                      "EVENT_SURFACE_STRUCTURES_VALID_GET",
                                      "GGet valid surface strucutures and their number of node"));
 
+    enumData.push_back(EventTypeEnum(EVENT_TILE_TABS_MODIFICATION,
+                                     "EVENT_TILE_TABS_MODIFICATION",
+                                     "Tile tabs modification"));
+    
+    enumData.push_back(EventTypeEnum(EVENT_TOOLBAR_CHART_ORIENTED_AXES_UPDATE,
+                                     "EVENT_TOOLBAR_CHART_ORIENTED_AXES_UPDATE",
+                                     "Update the toolbar's chart two oriented axes controls"));
+
     enumData.push_back(EventTypeEnum(EVENT_TOOLBOX_SELECTION_DISPLAY,
                                      "EVENT_TOOLBOX_SELECTION_DISPLAY", 
                                      "Display or hide the selection toolbox"));
-    
-    enumData.push_back(EventTypeEnum(EVENT_USER_INTERFACE_UPDATE, 
+        
+    enumData.push_back(EventTypeEnum(EVENT_USER_INTERFACE_UPDATE,
                                      "EVENT_USER_INTERFACE_UPDATE", 
                                      "Update the user-interface"));
     
@@ -386,6 +571,10 @@ EventTypeEnum::initialize()
                                      "EVENT_UPDATE_VOLUME_EDITING_TOOLBAR",
                                      "Update the volume editing toolbar"));
 
+    enumData.push_back(EventTypeEnum(EVENT_UPDATE_VOLUME_SLICE_INDICES_COORDS_TOOLBAR,
+                                     "EVENT_UPDATE_VOLUME_SLICE_INDICES_COORDS_TOOLBAR",
+                                     "Update the volume slices indices and coords in the toolbar"));
+    
     enumData.push_back(EventTypeEnum(EVENT_COUNT,
                                     "EVENT_COUNT", 
                                     "Count of events"));
