@@ -68,6 +68,8 @@ StructureEnum::initialize()
         return;
     }
     initializedFlag = true;
+    
+    //TSC: WARNING: the order of these determines the standard order of the structures in some -cifti-create-* commands, DO NOT reorder any *existing* entries
 
     enumData.push_back(StructureEnum(CORTEX_LEFT,
                                      "CORTEX_LEFT",
@@ -157,7 +159,7 @@ StructureEnum::initialize()
                                      "DIENCEPHALON_VENTRAL_RIGHT", 
                                      "DiencephalonVentralRight"));
     
-    enumData.push_back(StructureEnum(HIPPOCAMPUS_LEFT, 
+    enumData.push_back(StructureEnum(HIPPOCAMPUS_LEFT,
                                      "HIPPOCAMPUS_LEFT", 
                                      "HippocampusLeft"));
     
@@ -165,6 +167,14 @@ StructureEnum::initialize()
                                      "HIPPOCAMPUS_RIGHT", 
                                      "HippocampusRight"));
     
+    enumData.push_back(StructureEnum(HIPPOCAMPUS_DENTATE_LEFT,
+                                     "HIPPOCAMPUS_DENTATE_LEFT",
+                                     "HippocampusDentateLeft"));
+    
+    enumData.push_back(StructureEnum(HIPPOCAMPUS_DENTATE_RIGHT,
+                                     "HIPPOCAMPUS_DENTATE_RIGHT",
+                                     "HippocampusDentateRight"));
+    
     enumData.push_back(StructureEnum(INVALID,
                                      "INVALID",
                                      "Invalid"));
@@ -285,7 +295,7 @@ StructureEnum::fromName(const AString& name, bool* isValidOut)
         *isValidOut = validFlag;
     }
     else if (validFlag == false) {
-        CaretAssertMessage(0, AString("Name " + name + "failed to match enumerated value for type StructureEnum"));
+        CaretAssertMessage(0, AString("Name " + name + " failed to match enumerated value for type StructureEnum"));
     }
     return enumValue;
 }
@@ -338,7 +348,7 @@ StructureEnum::fromGuiName(const AString& guiName, bool* isValidOut)
         *isValidOut = validFlag;
     }
     else if (validFlag == false) {
-        CaretAssertMessage(0, AString("guiName " + guiName + "failed to match enumerated value for type StructureEnum"));
+        CaretAssertMessage(0, AString("guiName " + guiName + " failed to match enumerated value for type StructureEnum"));
     }
     return enumValue;
 }
@@ -394,7 +404,7 @@ StructureEnum::fromCiftiName(const AString& ciftiName, bool* isValidOut)
         *isValidOut = validFlag;
     }
     else if (validFlag == false) {
-        CaretAssertMessage(0, AString("guiName " + ciftiName + "failed to match enumerated value for type StructureEnum"));
+        CaretAssertMessage(0, AString("ciftiName " + ciftiName + " failed to match enumerated value for type StructureEnum"));
     }
     return enumValue;
 }
@@ -447,7 +457,7 @@ StructureEnum::fromIntegerCode(const int32_t integerCode, bool* isValidOut)
         *isValidOut = validFlag;
     }
     else if (validFlag == false) {
-        CaretAssertMessage(0, AString("Integer code " + AString::number(integerCode) + "failed to match enumerated value for type StructureEnum"));
+        CaretAssertMessage(0, AString("Integer code " + AString::number(integerCode) + " failed to match enumerated value for type StructureEnum"));
     }
     return enumValue;
 }
@@ -500,6 +510,7 @@ StructureEnum::isRight(const Enum enumValue)
         case CEREBRAL_WHITE_MATTER_RIGHT:
         case CORTEX_RIGHT:
         case DIENCEPHALON_VENTRAL_RIGHT:
+        case HIPPOCAMPUS_DENTATE_RIGHT:
         case HIPPOCAMPUS_RIGHT:
         case PALLIDUM_RIGHT:
         case PUTAMEN_RIGHT:
@@ -524,6 +535,7 @@ StructureEnum::isRight(const Enum enumValue)
         case CEREBRAL_WHITE_MATTER_LEFT:
         case CORTEX_LEFT:
         case DIENCEPHALON_VENTRAL_LEFT:
+        case HIPPOCAMPUS_DENTATE_LEFT:
         case HIPPOCAMPUS_LEFT:
         case PALLIDUM_LEFT:
         case PUTAMEN_LEFT:
@@ -554,6 +566,7 @@ StructureEnum::isLeft(const Enum enumValue)
         case CEREBRAL_WHITE_MATTER_LEFT:
         case CORTEX_LEFT:
         case DIENCEPHALON_VENTRAL_LEFT:
+        case HIPPOCAMPUS_DENTATE_LEFT:
         case HIPPOCAMPUS_LEFT:
         case PALLIDUM_LEFT:
         case PUTAMEN_LEFT:
@@ -578,6 +591,7 @@ StructureEnum::isLeft(const Enum enumValue)
         case CEREBRAL_WHITE_MATTER_RIGHT:
         case CORTEX_RIGHT:
         case DIENCEPHALON_VENTRAL_RIGHT:
+        case HIPPOCAMPUS_DENTATE_RIGHT:
         case HIPPOCAMPUS_RIGHT:
         case PALLIDUM_RIGHT:
         case PUTAMEN_RIGHT:
@@ -649,6 +663,10 @@ StructureEnum::isSingleStructure(const Enum enumValue)
             break;
         case DIENCEPHALON_VENTRAL_RIGHT:
             break;
+        case HIPPOCAMPUS_DENTATE_LEFT:
+            break;
+        case HIPPOCAMPUS_DENTATE_RIGHT:
+            break;
         case HIPPOCAMPUS_LEFT:
             break;
         case HIPPOCAMPUS_RIGHT:
@@ -795,6 +813,12 @@ StructureEnum::getContralateralStructure(const Enum enumValue)
         case DIENCEPHALON_VENTRAL_RIGHT:
             contralateralStructure = DIENCEPHALON_VENTRAL_LEFT;
             break;
+        case HIPPOCAMPUS_DENTATE_LEFT:
+            contralateralStructure = HIPPOCAMPUS_DENTATE_RIGHT;
+            break;
+        case HIPPOCAMPUS_DENTATE_RIGHT:
+            contralateralStructure = HIPPOCAMPUS_DENTATE_LEFT;
+            break;
         case HIPPOCAMPUS_LEFT:
             contralateralStructure = HIPPOCAMPUS_RIGHT;
             break;
