@@ -21,7 +21,8 @@
  */
 /*LICENSE_END*/
 
-
+#include <memory>
+#include <vector>
 #include <stdint.h>
 
 #include <AString.h>
@@ -32,7 +33,7 @@ class SystemBacktrace
 {
 #ifdef CARET_OS_WINDOWS
 #else // CARET_OS_WINDOWS
-    void* m_callstack[1024];
+    std::vector<void*> m_callstack;
     int m_numFrames;
 #endif // CARET_OS_WINDOWS
 public:
@@ -41,6 +42,8 @@ public:
     friend class SystemUtilities;
 };
 
+    class InfoItem;
+    
 /**
  * Methods to help out with files and directories.
  */
@@ -100,6 +103,10 @@ public:
     static void sleepSeconds(const float numberOfSeconds);
 
     static AString getWorkbenchHome();
+    
+    static AString getLocalHostName();
+    
+    static std::vector<std::unique_ptr<InfoItem>> getSystemInfo();
 };
 
 } // namespace
