@@ -26,11 +26,11 @@
 #include "CaretOpenGLInclude.h"
 
 #include <QApplication>
-#include <QDesktopWidget>
 #include <QLabel>
 #ifndef WORKBENCH_USE_QT5_QOPENGL_WIDGET
 #include <QGLPixelBuffer>
 #endif
+#include <QScreen>
 #include <QSplashScreen>
 #include <QStyleFactory>
 #include <QThread>
@@ -41,6 +41,7 @@
 
 #include "ApplicationInformation.h"
 #include "BrainBrowserWindow.h"
+#include "BrainBrowserWindowOrientedToolBox.h"
 #include "BrainOpenGL.h"
 #include "BrainOpenGLWidget.h"
 #include "CaretAssert.h"
@@ -49,224 +50,34 @@
 #include "CaretLogger.h"
 #include "CaretPreferences.h"
 #include "CommandOperationManager.h"
+#include "DeveloperFlagsEnum.h"
+#if QT_VERSION < 0x060000
+#include <QDesktopWidget>
+#endif
 #include "EventBrowserWindowNew.h"
 #include "EventManager.h"
 #include "FileInformation.h"
 #include "GuiManager.h"
 #include "MacApplication.h"
 #include "ProgramParameters.h"
+#include "QtPluginsPathSetup.h"
+#include "RecentFilesDialog.h"
+#include "RecentFilesSystemAccessModeEnum.h"
+#include "SceneDialog.h"
 #include "SessionManager.h"
-#include "SplashScreen.h"
 #include "SystemUtilities.h"
+#include "WorkbenchInstallationAssistant.h"
+#include "WorkbenchQtMessageHandler.h"
 #include "WuQMessageBox.h"
 #include "WuQtUtilities.h"
 
-static bool caretLoggerIsValid = false;
+#ifdef Q_OS_MAC
+#include "macos.h"
+#endif
 
 using namespace caret;
 using namespace std;
 
-#if QT_VERSION >= 0x050000
-/**
- * Handles message produced by Qt 5
- */
-static void
-messageHandlerForQt5(QtMsgType type, const QMessageLogContext& context, const QString& msg)
-{
-    const AString backtrace = SystemUtilities::getBackTrace();
-    
-    const AString contextInfo = ("   Context Info File ("
-                                 + QString(context.file)
-                                 + ") Function (" + QString(context.function)
-                                 + ") Line (" + QString::number(context.line)
-                                 + ") Version (" + QString::number(context.version)
-                                 + ") Category (" + QString(context.category)
-                                 + ")");
-    const AString message = (AString(msg) + "\n"
-                             + contextInfo + "\n"
-                             + backtrace);
-    
-    if (caretLoggerIsValid) {
-        bool abortFlag = false;
-        bool displayedFlag = false;
-        switch (type) {
-            case QtDebugMsg:
-                CaretLogInfo(message);
-                displayedFlag = CaretLogger::getLogger()->isInfo();
-                break;
-            case QtWarningMsg:
-                CaretLogWarning(message);
-                displayedFlag = CaretLogger::getLogger()->isWarning();
-                break;
-            case QtCriticalMsg:
-                CaretLogSevere(message);
-                displayedFlag = CaretLogger::getLogger()->isSevere();
-                break;
-            case QtFatalMsg:
-                cerr << "Qt Fatal: " << message << endl;
-                abortFlag = true;//fatal will cause an abort, so always display it, bypassing logger entirely
-                displayedFlag = true;
-                break;
-#if QT_VERSION >= 0x050500
-            case QtInfoMsg:
-                CaretLogInfo(message);
-                displayedFlag = CaretLogger::getLogger()->isInfo();
-                break;
-#endif
-        }
-        
-        /*
-         * Beep to alert user about an error!!!
-         */
-        if (displayedFlag && (type != QtDebugMsg))//don't beep for debug
-        {
-            GuiManager::beep();
-        }
-#ifndef NDEBUG
-        if (!displayedFlag)
-        {
-            cerr << "DEBUG: Qt ";
-            switch (type)
-            {
-                case QtDebugMsg:
-                    cerr << "Debug ";
-                    break;
-                case QtWarningMsg:
-                    cerr << "Warning ";
-                    break;
-                case QtCriticalMsg:
-                    cerr << "Critical ";
-                    break;
-                case QtFatalMsg:
-                    cerr << "FATAL (?!?) ";//should never happen
-                    break;
-#if QT_VERSION >= 0x050500
-                case QtInfoMsg:
-                    std::cerr << "Info ";
-                    break;
-#endif
-            }
-            cerr << "message hidden" << endl;
-        }
-#endif
-        
-        if (abortFlag) {
-            std::abort();
-        }
-    }
-    else {
-        switch (type) {
-            case QtDebugMsg:
-                std::cerr << "Qt Debug: " << message << std::endl;
-                break;
-            case QtWarningMsg:
-                std::cerr << "Qt Warning: " << message << std::endl;
-                break;
-            case QtCriticalMsg:
-                std::cerr << "Qt Critical: " << message << std::endl;
-                break;
-            case QtFatalMsg:
-                std::cerr << "Qt Fatal: " << message << std::endl;
-                std::abort();
-                break;
-#if QT_VERSION >= 0x050500
-            case QtInfoMsg:
-                std::cerr << "Qt Info: " << message << std::endl;
-                break;
-#endif
-        }
-    }
-}
-
-#else // QT_VERSION
-
-/**
- * Handles message produced by Qt 4.
- */
-static void
-messageHandlerForQt4(QtMsgType type, const char* msg)
-{
-    const AString backtrace = SystemUtilities::getBackTrace();
-    
-    const AString message = (AString(msg) + "\n" + backtrace);
-    
-    if (caretLoggerIsValid) {
-        bool abortFlag = false;
-        bool displayedFlag = false;
-        switch (type) {
-            case QtDebugMsg:
-                CaretLogInfo(message);
-                displayedFlag = CaretLogger::getLogger()->isInfo();
-                break;
-            case QtWarningMsg:
-                CaretLogWarning(message);
-                displayedFlag = CaretLogger::getLogger()->isWarning();
-                break;
-            case QtCriticalMsg:
-                CaretLogSevere(message);
-                displayedFlag = CaretLogger::getLogger()->isSevere();
-                break;
-            case QtFatalMsg:
-                cerr << "Qt Fatal: " << message << endl;
-                abortFlag = true;//fatal will cause an abort, so always display it, bypassing logger entirely
-                displayedFlag = true;
-                break;
-        }
-        
-        /*
-         * Beep to alert user about an error!!!
-         */
-        if (displayedFlag && (type != QtDebugMsg))//don't beep for debug
-        {
-            GuiManager::beep();
-        }
-#ifndef NDEBUG
-        if (!displayedFlag)
-        {
-            cerr << "DEBUG: Qt ";
-            switch (type)
-            {
-                case QtDebugMsg:
-                    cerr << "Debug ";
-                    break;
-                case QtWarningMsg:
-                    cerr << "Warning ";
-                    break;
-                case QtCriticalMsg:
-                    cerr << "Critical ";
-                    break;
-                case QtFatalMsg:
-                    cerr << "FATAL (?!?) ";//should never happen
-                    break;
-            }
-            cerr << "message hidden" << endl;
-        }
-#endif
-        
-        if (abortFlag) {
-            std::abort();
-        }
-    }
-    else {
-        switch (type) {
-            case QtDebugMsg:
-                std::cerr << "Qt Debug: " << message << std::endl;
-                break;
-            case QtWarningMsg:
-                std::cerr << "Qt Warning: " << message << std::endl;
-                break;
-            case QtCriticalMsg:
-                std::cerr << "Qt Critical: " << message << std::endl;
-                break;
-            case QtFatalMsg:
-                std::cerr << "Qt Fatal: " << message << std::endl;
-                std::abort();
-                break;
-        }
-    }
-}
-
-#endif // QT_VERSION
 
 //struct for communicating stuff back to main from parseCommandLine
 struct ProgramState
@@ -276,16 +87,35 @@ struct ProgramState
     int windowPosXY[2];
     int graphicsSizeXY[2];
     bool showSplash;
+    bool macMenuFlag = false;
+    
+    AString directoryName;
     
     AString specFileNameLoadWithDialog;
     AString specFileNameLoadAll;
     
     AString sceneFileName;
+    AString sceneFileNameNoDialog;
     AString sceneNameOrNumber;
     
+    bool sceneMostRecentFlag;
+    
     ProgramState();
 };
 
+class GuiBeeper : public WorkbenchQtMessageHandler::Beeper
+{
+public:
+    GuiBeeper() { }
+    
+    ~GuiBeeper() { }
+    
+    /**
+     * Override to make the beep sound
+     */
+    virtual void makeBeep() override { GuiManager::get()->beep(); }
+    
+};
 
 //declare the functions associated with command line
 void printHelp(const AString& progName);
@@ -293,6 +123,25 @@ void parseCommandLine(const AString& progName, ProgramParameters* myParams, Prog
 int 
 main(int argc, char* argv[])
 {
+
+#ifdef Q_OS_MAC 
+    /*
+     * Removes items that Apple adds to menus
+     *   Edit Menu - Start Dictation
+     *   Edit Menu - Emoji & Symbols
+     *   View Menu - Enter Full Screen
+     *
+     * The code uses the equivalent of the terminal's
+     * 'default' command.  The code could be replaced
+     * by having the user enter commands in the terminal
+     * to remove the menu items.
+     *
+     * Shows workbench's: defaults read workbench
+     * See: 'man defaults'.
+     */
+    OpenCOR::removeMacosSpecificMenuItems();
+#endif
+
     srand(time(NULL));
     int result;
     {
@@ -301,11 +150,17 @@ main(int argc, char* argv[])
         */
         SystemUtilities::setUnexpectedHandler();
         
+        /*
+         * Set environment variable QT_PLUGIN_PATH for plugin loading on some operating systems
+         * Neither qt.conf (in app's resource directory) or using QCoreApplication::addLibraryPath()
+         * work for setting plugin paths.
+         */
+        QtPluginsPathSetup::setupPluginsPathEnvironmentVariable(argv[0]);
+        
         /*
         * Create the session manager.
         */
         SessionManager::createSessionManager(ApplicationTypeEnum::APPLICATION_TYPE_GRAPHICAL_USER_INTERFACE);
-        caretLoggerIsValid = true;
 
         /*
         * Parameters for the program.
@@ -324,6 +179,11 @@ main(int argc, char* argv[])
         */
         CaretLogFine("Running: " + caret_global_commandLine);
         
+        /*
+         * Provide the application file path needed by the installation assistant
+         */
+        WorkbenchInstallationAssistant::setApplicationFilePath(argv[0]);
+        
         /*
          * Setup OpenGL if using Qt 5's QOpenGLWidget.
          * QOpenGLWidget's documentation indicates that
@@ -342,7 +202,17 @@ main(int argc, char* argv[])
 #else //CARET_OS_MACOSX        
         QApplication app(argc, argv);
 #endif //CARET_OS_MACOSX
-        
+
+        /*
+         * Create the GUI Manager.
+         * Moved here as part of WB-842.  In OSX Mojave (10.14),
+         * the open file event appears to be delivered very quickly
+         * so the GUI manager need to be created sooner than
+         * before.
+         */
+        GuiManager::createGuiManager();
+        app.processEvents();
+
         ApplicationInformation applicationInformation;
 
         QApplication::addLibraryPath(
@@ -389,16 +259,25 @@ main(int argc, char* argv[])
         BrainOpenGLWidget::initializeDefaultGLFormat();
 #endif
         
-#if QT_VERSION >= 0x050000
-        qInstallMessageHandler(messageHandlerForQt5);//this handler uses CaretLogger and GuiManager, so we must install it after the logger is available and the application is created
-#else // QT_VERSION
-        qInstallMsgHandler(messageHandlerForQt4);//this handler uses CaretLogger and GuiManager, so we must install it after the logger is available and the application is created
-#endif // QT_VERSION
+        /*
+         * Setup handlers (callbacks) for messages produced by Qt
+         */
+        GuiBeeper beeper;
+        WorkbenchQtMessageHandler::setupHandler(&beeper);
+
         /*
          * Log debug status
          */
+        CaretLogConfig("Version " + applicationInformation.getVersion());
         CaretLogConfig(applicationInformation.getCompiledWithDebugStatus());
 
+
+        /*
+         * Mac Option to put menus on window and not use the native tool bar
+         */
+        if (myState.macMenuFlag) {
+            app.setAttribute(Qt::AA_DontUseNativeMenuBar);
+        }
         
         /*
          * Enabled the desired splash screen based upon user preferences
@@ -433,11 +312,6 @@ main(int argc, char* argv[])
             }
         }
         
-        /*
-         * Create the GUI Manager.
-         */
-        GuiManager::createGuiManager();
-        
         /*
          * Letting the App process events will allow the message for a 
          * double-clicked spec file in Mac OSX to get processed.
@@ -463,24 +337,39 @@ main(int argc, char* argv[])
          * Show file selection splash screen if enabled via user's preferences
          */
         if (showSelectionSplashScreen) {
-            /*
-             * Show selection splash screen.
-             * Need to process events since QApplication::exec() has not
-             * been called.
-             */
-            SplashScreen splashScreen(NULL);
-            app.processEvents();
-            if (splashScreen.exec()) {
-                const QString dataFileName = splashScreen.getSelectedDataFileName();
-                if ( ! dataFileName.isEmpty()) {
-                    myState.fileList.clear();
-                    if (dataFileName.endsWith(DataFileTypeEnum::toFileExtension(DataFileTypeEnum::SPECIFICATION))) {
-                        myState.specFileNameLoadWithDialog = dataFileName;
+            AString dataFileName;
+            int32_t sceneIndex(-1);
+            RecentFilesDialog::ResultModeEnum result = RecentFilesDialog::runDialog(RecentFilesDialog::RunMode::SPLASH_SCREEN,
+                                                                                    dataFileName,
+                                                                                    sceneIndex);
+            switch (result) {
+                case RecentFilesDialog::ResultModeEnum::CANCEL:
+                    break;
+                case RecentFilesDialog::ResultModeEnum::LOAD_FILES_IN_SPEC_FILE:
+                    myState.specFileNameLoadAll = dataFileName;
+                    break;
+                case RecentFilesDialog::ResultModeEnum::LOAD_SCENE_FROM_SCENE_FILE:
+                    myState.sceneFileName     = dataFileName;
+                    myState.sceneNameOrNumber = AString::number(sceneIndex);
+                    break;
+                case RecentFilesDialog::ResultModeEnum::OPEN_DIRECTORY:
+                    myState.directoryName = dataFileName;
+                    break;
+                case RecentFilesDialog::ResultModeEnum::OPEN_FILE:
+                    if ( ! dataFileName.isEmpty()) {
+                        bool validFlag(false);
+                        if (DataFileTypeEnum::fromFileExtension(dataFileName,
+                                                                &validFlag) == DataFileTypeEnum::SPECIFICATION) {
+                            myState.specFileNameLoadWithDialog = dataFileName;
+                        }
+                        else {
+                            myState.fileList.push_back(dataFileName);
+                        }
                     }
-                    else {
-                        myState.fileList.push_back(dataFileName);
-                    }
-                }
+                    break;
+                case RecentFilesDialog::ResultModeEnum::OPEN_OTHER:
+                    myState.directoryName = dataFileName;
+                    break;
             }
         }
         
@@ -497,6 +386,13 @@ main(int argc, char* argv[])
 
         
         BrainBrowserWindow* myWindow = GuiManager::get()->getBrowserWindowByWindowIndex(0);
+        if (myWindow == NULL) {
+            std::cerr << "Unable to create a window.  This may result if you are trying to run "
+            << "wb_view from a remote login to a computer.  You must use the display directly "
+            << "attached to the computer or use some sort of 'Remote Desktop' system." << std::endl;
+            return -1;
+        }
+        
         if ( ! myWindow->hasValidOpenGL()) {
             app.processEvents();
             WuQMessageBox::errorOk(NULL,
@@ -509,8 +405,10 @@ main(int argc, char* argv[])
         /**
          * Test for the required OpenGL version is available.
          */
+        const bool guiFlag(true);
         AString requiredOpenGLMessage;
-        if ( ! BrainOpenGL::testForRequiredOpenGLVersion(requiredOpenGLMessage)) {
+        if ( ! BrainOpenGL::testForRequiredOpenGLVersion(guiFlag,
+                                                         requiredOpenGLMessage)) {
             app.processEvents();
             if ( ! WuQMessageBox::warningAcceptReject(NULL,
                                                       requiredOpenGLMessage,
@@ -585,7 +483,31 @@ main(int argc, char* argv[])
         
         if ( ! myState.sceneFileName.isEmpty()) {
             myWindow->loadSceneFromCommandLine(myState.sceneFileName,
-                                               myState.sceneNameOrNumber);
+                                               myState.sceneNameOrNumber,
+                                               BrainBrowserWindow::LoadSceneFromCommandLineDialogMode::SHOW_YES);
+        }
+        
+        if ( ! myState.sceneFileNameNoDialog.isEmpty()) {
+            myWindow->loadSceneFromCommandLine(myState.sceneFileNameNoDialog,
+                                               myState.sceneNameOrNumber,
+                                               BrainBrowserWindow::LoadSceneFromCommandLineDialogMode::SHOW_NO);
+        }
+        
+        if (myState.sceneMostRecentFlag) {
+            std::vector<RecentSceneInfoContainer> recentSceneInfo;
+            CaretPreferences* preferences = SessionManager::get()->getCaretPreferences();
+            preferences->getMostRecentScenes(recentSceneInfo);
+
+            if ( ! recentSceneInfo.empty()) {
+                CaretAssertVectorIndex(recentSceneInfo, 0);
+                const RecentSceneInfoContainer& rsic(recentSceneInfo[0]);
+                myWindow->loadRecentScene(rsic.getSceneFileName(),
+                                          rsic.getSceneName());
+            }
+        }
+        
+        if ( ! myState.directoryName.isEmpty()) {
+            myWindow->loadDirectoryFromCommandLine(myState.directoryName);
         }
         
 #ifndef WORKBENCH_USE_QT5_QOPENGL_WIDGET
@@ -610,10 +532,12 @@ main(int argc, char* argv[])
          * Resolution of screens
          */
         AString screenSizeText = "Screen Sizes: ";
-        QDesktopWidget* dw = QApplication::desktop();
-        const int numScreens = dw->screenCount();
+        QList<QScreen*> allScreens = QGuiApplication::screens();
+        const int32_t numScreens = allScreens.size();
         for (int i = 0; i < numScreens; i++) {
-            const QRect rect = dw->screenGeometry(i);
+            CaretAssertVectorIndex(allScreens, i);
+            CaretAssert(allScreens[i]);
+            const QRect rect = allScreens[i]->availableGeometry();
             const int x = rect.x();
             const int y = rect.y();
             const int w = rect.width();
@@ -629,6 +553,67 @@ main(int argc, char* argv[])
                                              + ", h="
                                              + AString::number(h));
         }
+        
+#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+        if (numScreens > 0) {
+            screenSizeText = "Screen Sizes: ";
+            QScreen* primaryScreen = QGuiApplication::primaryScreen();
+            for (int32_t i = 0; i < numScreens; i++) {
+                CaretAssertVectorIndex(allScreens, i);
+                QScreen* screen(allScreens[i]);
+                CaretAssert(screen);
+                QString primaryScreenText;
+                if (screen == primaryScreen) {
+                    primaryScreenText = " PRIMARY";
+                }
+                
+                /*
+                 * The manufacturer, model, and name may be empty.
+                 * Using trimmed() will make the overall string empty
+                 * so that it is not displayed.
+                 */
+                QString displayInfoText;
+#if QT_VERSION >= QT_VERSION_CHECK(6, 2, 0)
+                displayInfoText = (("   "
+                                    + screen->manufacturer()
+                                    + " " + screen->model()
+                                    + " " + screen->name()).trimmed());
+#endif
+                screenSizeText.appendWithNewLine("Screen="
+                                                 + AString::number(i)
+                                                 + primaryScreenText);
+                if ( ! displayInfoText.isEmpty()) {
+                    screenSizeText.appendWithNewLine(displayInfoText);
+                }
+                QRect screenWidgetRect = screen->availableGeometry();
+                screenSizeText.appendWithNewLine("   Desktop: x="
+                                                 + AString::number(screenWidgetRect.x())
+                                                 + ", y="
+                                                 + AString::number(screenWidgetRect.y())
+                                                 + ", w="
+                                                 + AString::number(screenWidgetRect.width())
+                                                 + ", h="
+                                                 + AString::number(screenWidgetRect.height()));
+                
+                screenSizeText.appendWithNewLine("   Logical DPI: x="
+                                                 + AString::number(screen->logicalDotsPerInchX())
+                                                 + ", y="
+                                                 + AString::number(screen->logicalDotsPerInchY()));
+                
+                screenSizeText.appendWithNewLine("   Physical DPI: x="
+                                                 + AString::number(screen->physicalDotsPerInch())
+                                                 + ", y="
+                                                 + AString::number(screen->physicalDotsPerInchY()));
+                
+                const QSizeF physicalSizeMM(primaryScreen->physicalSize());
+                screenSizeText.appendWithNewLine("   Width/height (mm): x="
+                                                 + AString::number(physicalSizeMM.width())
+                                                 + ", y="
+                                                 + AString::number(physicalSizeMM.height()));
+            }
+        }
+#else
+        QDesktopWidget* dw = QApplication::desktop();
         screenSizeText.appendWithNewLine("Primary Screen="
                                          + AString::number(dw->primaryScreen()));
         if (dw->isVirtualDesktop()) {
@@ -662,6 +647,7 @@ main(int argc, char* argv[])
                                          + AString::number(dw->widthMM())
                                          + ", y="
                                          + AString::number(dw->heightMM()));
+#endif
         
         CaretLogConfig(screenSizeText);
         
@@ -715,6 +701,9 @@ void printHelp(const AString& progName)
     << "    -help" << endl
     << "        display this usage text" << endl
     << endl
+    << "    -enable-perf" << endl
+    << "        Enable graphics performance improvements (surface buffers, volume textures)"
+    << endl
     << "    -graphics-size  <X Y>" << endl
     << "        Set the size of the graphics region." << endl
     << "        If this option is used you WILL NOT be able" << endl
@@ -736,15 +725,48 @@ void printHelp(const AString& progName)
     
     cout
     << endl
+    << "    -mac-menu-in-window" << endl
+    << "        MacOS Only - Menus are at the top of each main window  "
+    << "        similar to Linux and Window Applications.  "
+    << "        May be useful for creating tutorial images and videos." << endl
+    << endl
+    << "    -no-recent-files-dialog" << endl
+    << "         Inhibits display of Open Recent Files Dialog at wb_view startup." << endl
+    << endl
     << "    -no-splash" << endl
-    << "        disable all splash screens" << endl
+    << "        (Obsolete) Replaced by \"-no-recent-files-dialog\"" << endl
+    << "        Splash screen was replaced with Open Recent Fiels Dialog." << endl
+    << endl
+    << "    -recent-files-mode <mode>" << endl
+    << "        Set the recent file's file system access mode" << endl
+    << "        (overrides and replaces value in preferences)." << endl
+    << "        Using an off mode prevents file system access for " << endl
+    << "        obtaining last modified time and file existance." << endl
+    << "        This option may be useful if recent files are on a mounted" << endl
+    << "        file system that is having problems that may cause wb_view" << endl
+    << "        to hang at startup.  Valid modes are:" << endl;
+    
+    std::vector<RecentFilesSystemAccessModeEnum::Enum> recentFilesModes;
+    RecentFilesSystemAccessModeEnum::getAllEnums(recentFilesModes);
+    for (auto rfm : recentFilesModes) {
+        std::cout  << "            " <<RecentFilesSystemAccessModeEnum::toName(rfm) << endl;
+    }
+    
+    cout
     << endl
     << "    -scene-load <scene-file-name> <scene-name-or-number>" << endl
     << "        load the specified scene file and display the scene " << endl
     << "        in the file that matches by name or number.  Name" << endl
     << "        takes precedence over number.  The scene numbers " << endl
-    << "        start at one." << endl
-    << "        " << endl
+    << "        start at one.  The scene dialog remains visible " << endl
+    << "        after loading of the scene." << endl
+    << endl
+    << "    -scene-load-hd <scene-file-name> <scene-name-of-number>" << endl
+    << "        Same as \"-scene-load\" except that the scene dialog " << endl
+    << "        is hidden after the scene has loaded." << endl
+    << endl
+    << "    -scene-recent" << endl
+    << "        Loads the most recently loaded scene from its scene file." << endl
     << endl
     << "    -style <style-name>" << endl
     << "        change the window style to the specified style" << endl
@@ -765,6 +787,9 @@ void printHelp(const AString& progName)
     << "    -spec-load-all" << endl
     << "        load all files in the given spec file, don't show spec file dialog" << endl
     << endl
+    << "    -toolbox" << endl
+    << "        Combine Features and Overlay Vertical Toolboxes" << endl
+    << endl
     << "    -window-size  <X Y>" << endl
     << "        Set the size of the browser window" << endl
     << endl
@@ -791,6 +816,10 @@ void parseCommandLine(const AString& progName, ProgramParameters* myParams, Prog
                 } else if (thisParam == "-help") {
                     printHelp(progName);
                     exit(0);
+                } else if (thisParam == "-toolbox") {
+                    BrainBrowserWindowOrientedToolBox::setCombineFeaturesAndOverlayToolBox(true);
+                } else if (thisParam == "-enable-perf") {
+                    DeveloperFlagsEnum::setFlag(DeveloperFlagsEnum::DEVELOPER_FLAG_SURFACE_BUFFER, true);
                 } else if (thisParam == "-logging") {
                     if (myParams->hasNext()) {
                         const AString logLevelName = myParams->nextString("Logging Level").toUpper();
@@ -812,8 +841,32 @@ void parseCommandLine(const AString& progName, ProgramParameters* myParams, Prog
                             hasFatalError = true;
                         }
                     }
-                } else if (thisParam == "-no-splash") {
+                } else if (thisParam == "-mac-menu-in-window") {
+                    myState.macMenuFlag = true;
+                } else if ((thisParam == "-no-splash")
+                           || (thisParam == "-no-recent-files-dialog")) {
                     myState.showSplash = false;
+                } else if (thisParam == "-recent-files-mode") {
+                    if (myParams->hasNext()) {
+                        const AString recentFilesModeName = myParams->nextString("Recent Files Mode").toUpper();
+                        bool valid = false;
+                        const RecentFilesSystemAccessModeEnum::Enum recentFilesMode = RecentFilesSystemAccessModeEnum::fromName(recentFilesModeName, &valid);
+                        if (valid)
+                        {
+                            CaretPreferences* prefs = SessionManager::get()->getCaretPreferences();
+                            prefs->setRecentFilesSystemAccessMode(recentFilesMode);
+                        }
+                        else {
+                            cerr << "Invalid recent files mode \""
+                            << qPrintable(recentFilesModeName)
+                            << "\" for \"-recent-files-mode\" option" << std::endl;
+                            hasFatalError = true;
+                        }
+                    }
+                    else {
+                        cerr << "Missing recent files mode for \"-recent-files-mode\" option" << std::endl;
+                        hasFatalError = true;
+                    }
                 } else if (thisParam == "-scene-load") {
                     if (myParams->hasNext()) {
                         myState.sceneFileName = myParams->nextString("Scene File Name");
@@ -821,14 +874,31 @@ void parseCommandLine(const AString& progName, ProgramParameters* myParams, Prog
                             myState.sceneNameOrNumber = myParams->nextString("Scene Name or Number");
                         }
                         else {
-                            cerr << "Missing scene name/number for \"-scene\" option" << std::endl;
+                            cerr << "Missing scene name/number for \"-scene-load\" option" << std::endl;
                             hasFatalError = true;
                         }
                     }
                     else {
-                        cerr << "Missing scene file name for \"-scene\" option" << std::endl;
+                        cerr << "Missing scene file name for \"-scene-load\" option" << std::endl;
                         hasFatalError = true;
                     }
+                } else if (thisParam == "-scene-load-hd") {
+                    if (myParams->hasNext()) {
+                        myState.sceneFileNameNoDialog = myParams->nextString("Scene File Name");
+                        if (myParams->hasNext()) {
+                            myState.sceneNameOrNumber = myParams->nextString("Scene Name or Number");
+                        }
+                        else {
+                            cerr << "Missing scene name/number for " << thisParam << " option" << std::endl;
+                            hasFatalError = true;
+                        }
+                    }
+                    else {
+                        cerr << "Missing scene file name for " << thisParam << " option" << std::endl;
+                        hasFatalError = true;
+                    }
+                } else if (thisParam == "-scene-recent") {
+                    myState.sceneMostRecentFlag = true;
                 } else if (thisParam == "-spec-load-all") {
                     if ( ! myState.specFileNameLoadAll.isEmpty()) {
                         cerr << qPrintable(moreThanOneSpecFileErrorMessage) << endl;
@@ -954,9 +1024,15 @@ void parseCommandLine(const AString& progName, ProgramParameters* myParams, Prog
     if ( ! myState.sceneFileName.isEmpty()) {
         myState.showSplash = false;
     }
+    if ( ! myState.sceneFileNameNoDialog.isEmpty()) {
+        myState.showSplash = false;
+    }
     if ( ! myState.specFileNameLoadWithDialog.isEmpty()) {
         myState.showSplash = false;
     }
+    if (myState.sceneMostRecentFlag) {
+        myState.showSplash = false;
+    }
     if ( ! myState.specFileNameLoadAll.isEmpty()) {
         myState.showSplash = false;
     }
@@ -965,7 +1041,9 @@ void parseCommandLine(const AString& progName, ProgramParameters* myParams, Prog
 ProgramState::ProgramState()
 {
     sceneFileName = "";
+    sceneFileNameNoDialog = "";
     sceneNameOrNumber = "";
+    sceneMostRecentFlag = false;
     windowSizeXY[0] = -1;
     windowSizeXY[1] = -1;
     windowPosXY[0] = -1;
@@ -973,4 +1051,5 @@ ProgramState::ProgramState()
     graphicsSizeXY[0] = -1;
     graphicsSizeXY[1] = -1;
     showSplash = true;
+    macMenuFlag = false;
 }
