@@ -27,6 +27,7 @@
 #include <cmath>
 #include <vector>
 
+#include <QRegularExpression>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 #include <QString>
@@ -35,6 +36,7 @@
 #include "CaretAssert.h"
 #include "CaretLogger.h"
 #include "CaretPointer.h"
+#include "DataFileException.h"
 #include "GeodesicHelper.h"
 #include "MathFunctions.h"
 #include "SurfaceFile.h"
@@ -159,6 +161,13 @@ Border::copyHelperBorder(const Border& obj)
     
     clearModified();
     setNameOrClassModified(); // new name/class so modified
+
+    if (obj.isNameRgbaValid()) {
+        setNameRgba(obj.m_nameRgbaColor);
+    }
+    if (obj.isClassRgbaValid()) {
+        setClassRgba(obj.m_classRgbaColor);
+    }
 }
 
 /**
@@ -266,12 +275,13 @@ Border::getClassRgba(float rgba[4]) const
  *     Red, green, blue, alpha ranging zero to one.
  */
 void 
-Border::setClassRgba(const float rgba[3])
+Border::setClassRgba(const float rgba[4])
 {
     m_classRgbaColor[0] = rgba[0];
     m_classRgbaColor[1] = rgba[1];
     m_classRgbaColor[2] = rgba[2];
     m_classRgbaColor[3] = rgba[3];
+    CaretAssert(rgba[3] != 0.0);
     m_classRgbaColorValid = true;
 }
 
@@ -328,6 +338,7 @@ Border::setNameRgba(const float rgba[4])
     m_nameRgbaColor[1] = rgba[1];
     m_nameRgbaColor[2] = rgba[2];
     m_nameRgbaColor[3] = rgba[3];
+    CaretAssert(rgba[3] != 0.0);
     m_nameRgbaColorValid = true;
 }
 
@@ -745,6 +756,95 @@ Border::reverse()
     setModified();
 }
 
+/**
+ * Revise a border by extending from a of point border.
+ *
+ * @param surfaceFile
+ *    Surface on which border extension is performed.
+ * @param pointIndex
+ *    Point nearest the first point in the segment.
+ * @param segment
+ *    A border segment containing the extension.
+ * @throws BorderException
+ *    If there is an error revising the border.
+ */
+void
+Border::reviseExtendFromPointIndex(SurfaceFile* surfaceFile,
+                                   const int32_t pointIndex,
+                                   const Border* segment)
+{
+    const int32_t numPoints = getNumberOfPoints();
+    if (numPoints <= 2) {
+        throw BorderException("Border being update contains less than two points");
+    }
+    
+    const int numberOfSegmentPoints = segment->getNumberOfPoints();
+    if (numberOfSegmentPoints <= 0) {
+        throw BorderException("Border segment for extending contains no points");
+    }
+
+    if ((pointIndex < 0)
+        || (pointIndex >= numPoints)) {
+        throw BorderException("Point index for extending border is invalid.");
+    }
+    
+    /*
+     * Copy the border just in case something goes wrong
+     */
+    
+    /*
+     * Lengths from point index to start and end points
+     */
+    const float distToStart = getSegmentLength(surfaceFile, 0, pointIndex);
+    const float distToEnd   = getSegmentLength(surfaceFile, pointIndex, numPoints - 1);
+    
+    /*
+     * Create a temporary border
+     */
+    Border tempBorder;
+    
+    /*
+     * Add on to start or ending end of border
+     */
+    if (distToStart < distToEnd) {
+        /*
+         * Reverse the new segment and it becomes the first part of the border
+         */
+        Border segmentCopy = *segment;
+        segmentCopy.reverse();
+        tempBorder.addPoints(&segmentCopy);
+        
+        /*
+         * Add points from this border starting AFTER pointIndex
+         * to the last point)
+         */
+        const int32_t startPointIndex = pointIndex + 1;
+        if (startPointIndex < numPoints) {
+            tempBorder.addPoints(this,
+                                 startPointIndex);
+        }
+    }
+    else {
+        /*
+         * Add points from this border from the first point to
+         * the point BEFORE pointIndex
+         */
+        const int32_t pointCount = pointIndex;
+        if (pointCount > 0) {
+            tempBorder.addPoints(this,
+                                 0,
+                                 pointCount);
+        }
+        
+        /*
+         * Add the new segment
+         */
+        tempBorder.addPoints(segment);
+    }
+    
+    replacePointsWithUndoSaving(&tempBorder);
+}
+
 /**
  * Revise a border by extending from the end of a border.
  *
@@ -755,9 +855,9 @@ Border::reverse()
  * @throws BorderException
  *    If there is an error revising the border.
  */
-void 
+void
 Border::reviseExtendFromEnd(SurfaceFile* surfaceFile,
-                            const Border* segment) throw (BorderException)
+                            const Border* segment)
 {
     const int32_t numPoints = getNumberOfPoints();
     if (numPoints <= 0) {
@@ -766,7 +866,7 @@ Border::reviseExtendFromEnd(SurfaceFile* surfaceFile,
     
     const int numberOfSegmentPoints = segment->getNumberOfPoints();
     if (numberOfSegmentPoints <= 0) {
-        throw BorderException("Border segment for erasing contains no points");
+        throw BorderException("Border segment for extending contains no points");
     }
     
     /*
@@ -776,7 +876,7 @@ Border::reviseExtendFromEnd(SurfaceFile* surfaceFile,
     if (segment->getPoint(0)->getProjectedPosition(*surfaceFile, 
                                                    segmentStartXYZ, 
                                                    true) == false) {
-        throw BorderException("First point in erase segment has invalid coordinate.  Redraw.");
+        throw BorderException("First point in extending segment has invalid coordinate.  Redraw.");
     }
     
     /*
@@ -869,9 +969,7 @@ Border::reviseExtendFromEnd(SurfaceFile* surfaceFile,
                              (endPointIndex + 1));
     }
     
-    saveBorderForUndoEditing();
-    
-    replacePoints(&tempBorder);
+    replacePointsWithUndoSaving(&tempBorder);
 }
 
 /**
@@ -886,7 +984,7 @@ Border::reviseExtendFromEnd(SurfaceFile* surfaceFile,
  */
 void 
 Border::reviseEraseFromEnd(SurfaceFile* surfaceFile,
-                           const Border* segment) throw (BorderException)
+                           const Border* segment)
 {
     /*
      * Get coordinate of first and last points in the segment
@@ -979,7 +1077,7 @@ Border::reviseEraseFromEnd(SurfaceFile* surfaceFile,
  */
 void
 Border::reviseReplaceSegment(SurfaceFile* surfaceFile,
-                             const Border* segment) throw (BorderException)
+                             const Border* segment)
 {
     /*
      * Get coordinate of first and last points in the segment
@@ -1159,12 +1257,10 @@ Border::reviseReplaceSegment(SurfaceFile* surfaceFile,
                                         newBorderSecondSegment.getNumberOfPoints());
                 }
                 
-                saveBorderForUndoEditing();
-                
                 /*
                  * Replace this border with the newly created border
                  */
-                replacePoints(&newBorder);
+                replacePointsWithUndoSaving(&newBorder);
             }
             else {
                 throw BorderException("Border replacement failed: First or last point in new segment failed to project.");
@@ -1191,7 +1287,7 @@ Border::reviseReplaceSegment(SurfaceFile* surfaceFile,
 // */
 //void
 //Border::reviseReplaceSegment(SurfaceFile* surfaceFile,
-//                             const Border* segment) throw (BorderException)
+//                             const Border* segment)
 //{
 //    /*
 //     * Get coordinate of first and last points in the segment
@@ -1397,6 +1493,19 @@ Border::getSegmentLength(SurfaceFile* surfaceFile,
     return segmentLength;
 }
 
+/**
+ * Replace the points in this border with points from the given border.
+ * An "undo" copy of the border is also created.
+ *
+ * @param border Border whose points are copied into this border.
+ */
+void
+Border::replacePointsWithUndoSaving(const Border* border)
+{
+    saveBorderForUndoEditing();
+    
+    replacePoints(border);
+}
 
 /**
  * Replace the points in this border with
@@ -1509,7 +1618,7 @@ Border::toString() const
  *   Writer for XML output.
  */
 void 
-Border::writeAsXML(XmlWriter& xmlWriter) throw (XmlException)
+Border::writeAsXML(XmlWriter& xmlWriter)
 {
     xmlWriter.writeStartElement(XML_TAG_BORDER);
     
@@ -1553,30 +1662,30 @@ void Border::writeXML3(QXmlStreamWriter& xml) const
 void Border::readXML1(QXmlStreamReader& xml)
 {
     clear();
-    CaretAssert(xml.isStartElement() && xml.name() == "Border");
+    CaretAssert(xml.isStartElement() && xml.name() == QLatin1String("Border"));
     bool haveName = false, haveClass = false, haveColorType = false;
     for (xml.readNext(); !xml.atEnd() && !xml.isEndElement(); xml.readNext())
     {
         if (xml.isStartElement())
         {
-            QStringRef name = xml.name();
-            if (name == "Name")
+            auto name = xml.name();
+            if (name == QLatin1String("Name"))
             {
                 if (haveName) throw DataFileException("multiple Name elements in one Border element");
                 m_name = xml.readElementText();//sets error on unexpected child element
                 if (xml.hasError()) throw DataFileException("XML parsing error in Name: " + xml.errorString());
                 haveName = true;
-            } else if (name == "ClassName") {
+            } else if (name == QLatin1String("ClassName")) {
                 if (haveClass) throw DataFileException("multiple ClassName elements in one Border element");
                 m_className = xml.readElementText();//sets error on unexpected child element
                 if (xml.hasError()) throw DataFileException("XML parsing error in ClassName: " + xml.errorString());
                 haveClass = true;
-            } else if (name == "ColorName") {//a gui setting that caret5 wrote into the border file, so ignore it
+            } else if (name == QLatin1String("ColorName")) {//a gui setting that caret5 wrote into the border file, so ignore it
                 if (haveColorType) throw DataFileException("multiple ColorName elements in one Border element");
                 xml.readElementText();//errors on unexpected element
                 if (xml.hasError()) throw DataFileException("XML parsing error in ColorName: " + xml.errorString());
                 haveColorType = true;
-            } else if (name == "SurfaceProjectedItem") {
+            } else if (name == QLatin1String("SurfaceProjectedItem")) {
                 CaretPointer<SurfaceProjectedItem> myItem(new SurfaceProjectedItem());//again, because current interface requires ownership passing of pointer
                 myItem->readBorderFileXML1(xml);
                 addPoint(myItem.releasePointer());
@@ -1586,7 +1695,7 @@ void Border::readXML1(QXmlStreamReader& xml)
         }
     }
     if (xml.hasError()) throw DataFileException("XML parsing error in Border: " + xml.errorString());
-    CaretAssert(xml.isEndElement() && xml.name() == "Border");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("Border"));
     if (getNumberOfPoints() > 1 && (*getPoint(0) == *getPoint(getNumberOfPoints() - 1)))
     {
         m_closed = true;
@@ -1597,14 +1706,14 @@ void Border::readXML1(QXmlStreamReader& xml)
 void Border::readXML3(QXmlStreamReader& xml)
 {
     clear();
-    CaretAssert(xml.isStartElement() && xml.name() == "BorderPart");
+    CaretAssert(xml.isStartElement() && xml.name() == QLatin1String("BorderPart"));
     QXmlStreamAttributes myAttrs = xml.attributes();
     if (!myAttrs.hasAttribute("Closed")) throw DataFileException("BorderPart element missing required attribute Closed");
-    QStringRef closedStr = myAttrs.value("Closed");
-    if (closedStr == "True")
+    auto closedStr = myAttrs.value("Closed");
+    if (closedStr == QLatin1String("True"))
     {
         setClosed(true);
-    } else if (closedStr == "False") {
+    } else if (closedStr == QLatin1String("False")) {
         setClosed(false);
     } else {
         throw DataFileException("unrecognized value for Closed attribute in BorderPart: " + closedStr.toString());
@@ -1616,13 +1725,17 @@ void Border::readXML3(QXmlStreamReader& xml)
     {
         if (xml.isStartElement())
         {
-            QStringRef name = xml.name();
-            if (name == "Vertices")
+            auto name = xml.name();
+            if (name == QLatin1String("Vertices"))
             {
                 if (haveVertices) throw DataFileException("multiple Vertices elements in one BorderPart element");
                 QString vertexText = xml.readElementText();//errors on unexpected element
                 if (xml.hasError()) throw DataFileException("XML parsing error in Vertices: " + xml.errorString());
-                QStringList vertexStrings = vertexText.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+#if QT_VERSION >= 0x060000
+                QStringList vertexStrings = vertexText.split(QRegularExpression("\\s+"), Qt::SkipEmptyParts);
+#else
+                QStringList vertexStrings = vertexText.split(QRegularExpression("\\s+"), QString::SkipEmptyParts);
+#endif
                 int numItems = (int)vertexStrings.size();
                 if (numItems % 3 != 0) throw DataFileException("number of items in Vertices element text is not a multiple of 3");
                 for (int i = 0; i < numItems; ++i)
@@ -1634,11 +1747,15 @@ void Border::readXML3(QXmlStreamReader& xml)
                     vertices.push_back(tempVal);
                 }
                 haveVertices = true;
-            } else if (name == "Weights") {
+            } else if (name == QLatin1String("Weights")) {
                 if (haveWeights) throw DataFileException("multiple Weights elements in one BorderPart element");
                 QString vertexText = xml.readElementText();//errors on unexpected element
                 if (xml.hasError()) throw DataFileException("XML parsing error in Weights: " + xml.errorString());
-                QStringList vertexStrings = vertexText.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+#if QT_VERSION >= 0x060000
+                QStringList vertexStrings = vertexText.split(QRegularExpression("\\s+"), Qt::SkipEmptyParts);
+#else
+                QStringList vertexStrings = vertexText.split(QRegularExpression("\\s+"), QString::SkipEmptyParts);
+#endif
                 int numItems = (int)vertexStrings.size();
                 if (numItems % 3 != 0) throw DataFileException("number of items in Weights element text is not a multiple of 3");
                 for (int i = 0; i < numItems; ++i)
@@ -1646,7 +1763,11 @@ void Border::readXML3(QXmlStreamReader& xml)
                     bool ok = false;
                     float tempVal = vertexStrings[i].toFloat(&ok);
                     if (!ok) throw DataFileException("non-numeric item in Weights text: " + vertexStrings[i]);
-                    if (tempVal < 0.0f) throw DataFileException("negative value in Weights");
+                    if (tempVal < 0.0f)
+                    {
+                        CaretLogWarning("negative value in Weights, set to zero");
+                        tempVal = 0.0f;
+                    }
                     weights.push_back(tempVal);
                 }
                 haveWeights = true;
@@ -1656,7 +1777,7 @@ void Border::readXML3(QXmlStreamReader& xml)
         }
     }
     if (xml.hasError()) throw DataFileException("XML parsing error in BorderPart: " + xml.errorString());
-    CaretAssert(xml.isEndElement() && xml.name() == "BorderPart");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("BorderPart"));
     if (!haveVertices || !haveWeights) throw DataFileException("BorderPart missing required Vertices or Weights element");
     if (vertices.size() != weights.size()) throw DataFileException("Vertices and Weights don't contain the same number of elements");
     int numPoints = (int)vertices.size() / 3;
