@@ -21,6 +21,7 @@
  */
 /*LICENSE_END*/
 
+#include <map>
 #include <memory>
 
 #include "CaretDataFile.h"
@@ -28,6 +29,7 @@
 #include "ChartOneDataTypeEnum.h"
 #include "CaretPointer.h"
 #include "CiftiXML.h"
+#include "DisplayGroupEnum.h"
 #include "NiftiEnums.h"
 #include "PaletteModifiedStatusEnum.h"
 #include "PaletteNormalizationModeEnum.h"
@@ -36,11 +38,15 @@ namespace caret {
 
     class ChartDataCartesian;
     class ChartableTwoFileDelegate;
+    class ClusterContainer;
+    class DataFileColorModulateSelector;
+    class CaretMappableDataFileLabelSelectionDelegate;
     class FastStatistics;
     class GiftiMetaData;
     class GiftiLabelTable;
     class Histogram;
     class LabelDrawingProperties;
+    class LabelSelectionItemModel;
     class MapFileDataSelector;
     class PaletteColorMapping;
     
@@ -383,7 +389,40 @@ namespace caret {
          *    not mapped using a label table).
          */         
         virtual const GiftiLabelTable* getMapLabelTable(const int32_t mapIndex) const = 0;
+
+        /**
+         * @return The clusters for the given map's label table (may be NULL)
+         * @param mapIndex
+         *    Index of the map
+         */
+        virtual const ClusterContainer* getMapLabelTableClusters(const int32_t mapIndex) const;
         
+        /*
+         * @return Label selection hierarchy for the map in the tab (may be NULL)
+         * @param mapIndex
+         *    Index of map
+         * @param displayGroup
+         *    The display group
+         * @param tabIndex
+         *    Index of the tab if displayGroup is TAB
+         */
+        LabelSelectionItemModel* getLabelSelectionHierarchyForMapAndTab(const int32_t mapIndex,
+                                                                        const DisplayGroupEnum::Enum displayGroup,
+                                                                        const int32_t tabIndex);
+        
+        /*
+         * @return Label selection delegate for the map in the tab (may be NULL)
+         * @param mapIndex
+         *    Index of map
+         * @param displayGroup
+         *    The display group
+         * @param tabIndex
+         *    Index of the tab if displayGroup is TAB
+         */
+        const LabelSelectionItemModel* getLabelSelectionHierarchyForMapAndTab(const int32_t mapIndex,
+                                                                              const DisplayGroupEnum::Enum displayGroup,
+                                                                              const int32_t tabIndex) const;
+
         bool isMedialWallLabelInMapLabelTable(const int32_t mapIndex) const;
         
         /**
@@ -492,8 +531,10 @@ namespace caret {
         
         CaretMappableDataFileAndMapSelectionModel* getMapThresholdFileSelectionModel(const int32_t mapIndex);
         
-        /**
-         * Are all brainordinates in this file also in the given file?  
+        DataFileColorModulateSelector* getMapColorModulateFileSelector(const int32_t mapIndex);
+        
+        /*
+         * Are all brainordinates in this file also in the given file?
          * That is, the brainordinates are equal to or a subset of the brainordinates
          * in the given file.
          *
@@ -502,8 +543,30 @@ namespace caret {
          * @return 
          *     Match status.
          */
-        virtual BrainordinateMappingMatch getBrainordinateMappingMatch(const CaretMappableDataFile* mapFile) const = 0;
+        virtual BrainordinateMappingMatch getBrainordinateMappingMatchImplementation(const CaretMappableDataFile* mapFile) const = 0;
         
+        BrainordinateMappingMatch getBrainordinateMappingMatch(const CaretMappableDataFile* mapFile) const;
+        
+       virtual bool getSurfaceNodeIdentificationForMaps(const std::vector<int32_t>& mapIndices,
+                                                         const StructureEnum::Enum structure,
+                                                         const int nodeIndex,
+                                                         const int32_t numberOfNodes,
+                                                         const AString& dataValueSeparator,
+                                                         const int32_t digitsRightOfDecimal,
+                                                         AString& textOut) const;
+        
+        virtual bool getVolumeVoxelIdentificationForMaps(const std::vector<int32_t>& mapIndices,
+                                                         const float xyz[3],
+                                                         const AString& dataValueSeparator,
+                                                         const int32_t digitsRightOfDecimal,
+                                                         int64_t ijkOut[3],
+                                                         AString& textOut) const;
+        
+        void updateAfterFileDataChanges();
+        
+        virtual CaretMappableDataFile* castToCaretMappableDataFile() override;
+        virtual const CaretMappableDataFile* castToCaretMappableDataFile() const override;
+
     protected:
         CaretMappableDataFile(const CaretMappableDataFile&);
 
@@ -513,8 +576,6 @@ namespace caret {
         
         void helpGetSupportedLineSeriesChartDataTypes(std::vector<ChartOneDataTypeEnum::Enum>& chartDataTypesOut) const;
         
-        void updateAfterFileDataChanges();
-        
         virtual void saveFileDataToScene(const SceneAttributes* sceneAttributes,
                                          SceneClass* sceneClass);
         
@@ -537,9 +598,16 @@ namespace caret {
 
         mutable std::unique_ptr<ChartableTwoFileDelegate> m_chartingDelegate;
         
+        mutable std::vector<std::unique_ptr<CaretMappableDataFileLabelSelectionDelegate>> m_labelHierarchySelectionDelegate;
+        
         std::vector<std::unique_ptr<CaretMappableDataFileAndMapSelectionModel>> m_mapThresholdFileSelectionModels;
         
-        /** 
+        std::map<int32_t, std::unique_ptr<DataFileColorModulateSelector>> m_mapColorModulateFileSelectors;
+        
+        /* Cache's status of files that have compatible mapping (brainordinates, parcels, etc.) */
+        mutable std::map<const CaretMappableDataFile*, BrainordinateMappingMatch> m_mappingMatchedFilesCache;
+        
+        /**
          * Added by WB-781 Apply to All Maps for ColorBar.
          * This value is saved to scenes but NOT to the data file.
          */
