@@ -45,14 +45,21 @@ using namespace caret;
  *
  * @param caretMappableDataFile
  *    Mappable data file.
+ * @param excludeSelfFlag
+ *    If true exclude self from file selection
  */
-CaretMappableDataFileAndMapSelectionModel::CaretMappableDataFileAndMapSelectionModel(const CaretMappableDataFile* caretMappableDataFile)
+CaretMappableDataFileAndMapSelectionModel::CaretMappableDataFileAndMapSelectionModel(const CaretMappableDataFile* caretMappableDataFile,
+                                                                                     const bool excludeSelfFlag)
 : CaretObject(),
-m_mode(Mode::MAP_TO_SAME_BRAINORDINATES),
+m_mode(excludeSelfFlag
+       ? Mode::MAP_TO_SAME_BRAINORDINATES_EXCLUDE_SELF
+       : Mode::MAP_TO_SAME_BRAINORDINATES),
 m_mappableDataFile(caretMappableDataFile)
 {
     std::vector<DataFileTypeEnum::Enum> dataFileTypesVector;
-    performConstruction(dataFileTypesVector);
+    std::vector<SubvolumeAttributes::VolumeType> volumeTypes;
+    performConstruction(dataFileTypesVector,
+                        volumeTypes);
 }
 
 
@@ -69,7 +76,9 @@ m_mappableDataFile(NULL)
 {
     std::vector<DataFileTypeEnum::Enum> dataFileTypesVector;
     dataFileTypesVector.push_back(dataFileType);
-    performConstruction(dataFileTypesVector);
+    std::vector<SubvolumeAttributes::VolumeType> volumeTypes;
+    performConstruction(dataFileTypesVector,
+                        volumeTypes);
 }
 
 /**
@@ -83,7 +92,28 @@ CaretMappableDataFileAndMapSelectionModel::CaretMappableDataFileAndMapSelectionM
 m_mode(Mode::MATCH_DATA_FILE_TYPES),
 m_mappableDataFile(NULL)
 {
-    performConstruction(dataFileTypes);
+    std::vector<SubvolumeAttributes::VolumeType> volumeTypes;
+    performConstruction(dataFileTypes,
+                        volumeTypes);
+}
+
+/**
+ * Constructor for instance that will contain files of the given data types.
+ * Volume are limited to the given types
+ *
+ * @param dataFileTypes
+ *    Types of data files available for selection.
+ * @param volumeTypes
+ *    If not empty, volumes must be one of these types
+ */
+CaretMappableDataFileAndMapSelectionModel::CaretMappableDataFileAndMapSelectionModel(const std::vector<DataFileTypeEnum::Enum>& dataFileTypes,
+                                                                                     const std::vector<SubvolumeAttributes::VolumeType>& volumeTypes)
+: CaretObject(),
+m_mode(Mode::MATCH_DATA_FILE_TYPES),
+m_mappableDataFile(NULL)
+{
+    performConstruction(dataFileTypes,
+                        volumeTypes);
 }
 
 /**
@@ -138,12 +168,18 @@ CaretMappableDataFileAndMapSelectionModel::validateDataFileTypes()
             case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_TEMPORARY:
                 isMappableFile = true;
                 break;
+            case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_MAPS:
+                isMappableFile = true;
+                break;
             case DataFileTypeEnum::CONNECTIVITY_PARCEL:
                 isMappableFile = true;
                 break;
             case DataFileTypeEnum::CONNECTIVITY_PARCEL_DENSE:
                 isMappableFile = true;
                 break;
+            case DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC:
+                isMappableFile = true;
+                break;
             case DataFileTypeEnum::CONNECTIVITY_PARCEL_LABEL:
                 isMappableFile = true;
                 break;
@@ -156,8 +192,12 @@ CaretMappableDataFileAndMapSelectionModel::validateDataFileTypes()
             case DataFileTypeEnum::CONNECTIVITY_SCALAR_DATA_SERIES:
                 isMappableFile = true;
                 break;
+            case DataFileTypeEnum::CZI_IMAGE_FILE:
+                break;
             case DataFileTypeEnum::FOCI:
                 break;
+            case DataFileTypeEnum::HISTOLOGY_SLICES:
+                break;
             case DataFileTypeEnum::IMAGE:
                 break;
             case DataFileTypeEnum::LABEL:
@@ -166,11 +206,18 @@ CaretMappableDataFileAndMapSelectionModel::validateDataFileTypes()
             case DataFileTypeEnum::METRIC:
                 isMappableFile = true;
                 break;
+            case DataFileTypeEnum::METRIC_DYNAMIC:
+                isMappableFile = true;;
+                break;
+            case DataFileTypeEnum::OME_ZARR_IMAGE_FILE:
+                break;
             case DataFileTypeEnum::PALETTE:
                 break;
             case DataFileTypeEnum::RGBA:
                 isMappableFile = true;
                 break;
+            case DataFileTypeEnum::SAMPLES:
+                break;
             case DataFileTypeEnum::SCENE:
                 break;
             case DataFileTypeEnum::SPECIFICATION:
@@ -182,6 +229,9 @@ CaretMappableDataFileAndMapSelectionModel::validateDataFileTypes()
             case DataFileTypeEnum::VOLUME:
                 isMappableFile = true;
                 break;
+            case DataFileTypeEnum::VOLUME_DYNAMIC:
+                isMappableFile = true;
+                break;
         }
         
         CaretAssert(isMappableFile);
@@ -207,20 +257,29 @@ CaretMappableDataFileAndMapSelectionModel::validateDataFileTypes()
  *
  * @param dataFileTypes
  *    Types of data files available for selection.
+ * @param volumeTypes
+ *    If not empty, volumes must be one of these types
  */
 void
-CaretMappableDataFileAndMapSelectionModel::performConstruction(const std::vector<DataFileTypeEnum::Enum>& dataFileTypes)
+CaretMappableDataFileAndMapSelectionModel::performConstruction(const std::vector<DataFileTypeEnum::Enum>& dataFileTypes,
+                                                               const std::vector<SubvolumeAttributes::VolumeType>& volumeTypes)
 {
     m_dataFileTypes = dataFileTypes;
+    m_volumeTypes   = volumeTypes;
     
     switch (m_mode) {
         case Mode::MAP_TO_SAME_BRAINORDINATES:
             CaretAssert(m_mappableDataFile);
             m_caretDataFileSelectionModel = CaretDataFileSelectionModel::newInstanceMapsToSameBrainordinates(m_mappableDataFile);
             break;
+        case Mode::MAP_TO_SAME_BRAINORDINATES_EXCLUDE_SELF:
+            CaretAssert(m_mappableDataFile);
+            m_caretDataFileSelectionModel = CaretDataFileSelectionModel::newInstanceMapsToSameBrainordinatesExcludeSelf(m_mappableDataFile);
+            break;
         case Mode::MATCH_DATA_FILE_TYPES:
             validateDataFileTypes();
-            m_caretDataFileSelectionModel = CaretDataFileSelectionModel::newInstanceForCaretDataFileTypes(dataFileTypes);
+            m_caretDataFileSelectionModel = CaretDataFileSelectionModel::newInstanceForCaretDataFileTypes(dataFileTypes,
+                                                                                                          volumeTypes);
             break;
     }
     
@@ -343,6 +402,8 @@ CaretMappableDataFileAndMapSelectionModel::getAvailableFiles() const
     switch (m_mode) {
         case Mode::MAP_TO_SAME_BRAINORDINATES:
             break;
+        case Mode::MAP_TO_SAME_BRAINORDINATES_EXCLUDE_SELF:
+            break;
         case Mode::MATCH_DATA_FILE_TYPES:
             break;
     }
@@ -391,6 +452,8 @@ CaretMappableDataFileAndMapSelectionModel::setSelectedFile(CaretMappableDataFile
         switch (m_mode) {
             case Mode::MAP_TO_SAME_BRAINORDINATES:
                 break;
+            case Mode::MAP_TO_SAME_BRAINORDINATES_EXCLUDE_SELF:
+                break;
             case Mode::MATCH_DATA_FILE_TYPES:
             {
                 const DataFileTypeEnum::Enum fileType = selectedFile->getDataFileType();
@@ -422,6 +485,33 @@ CaretMappableDataFileAndMapSelectionModel::setSelectedFile(CaretMappableDataFile
     m_caretDataFileSelectionModel->setSelectedFile(selectedFile);
 }
 
+/**
+ * Set the selected file by name
+ *
+ * @param filename
+ *    Name of file.
+ */
+void
+CaretMappableDataFileAndMapSelectionModel::setSelectedFileName(const AString& filename)
+{
+    std::vector<CaretDataFile*> files(m_caretDataFileSelectionModel->getAvailableFiles());
+    for (CaretDataFile* cdf : files) {
+        CaretAssert(cdf);
+        if (filename == cdf->getFileName()) {
+            m_caretDataFileSelectionModel->setSelectedFile(cdf);
+            return;
+        }
+    }
+    for (CaretDataFile* cdf : files) {
+        CaretAssert(cdf);
+        if (cdf->getFileName().endsWith(filename)) {
+            m_caretDataFileSelectionModel->setSelectedFile(cdf);
+            return;
+        }
+    }
+}
+
+
 /**
  * Set the selected map index
  *
@@ -431,7 +521,13 @@ CaretMappableDataFileAndMapSelectionModel::setSelectedFile(CaretMappableDataFile
 void
 CaretMappableDataFileAndMapSelectionModel::setSelectedMapIndex(const int32_t mapIndex)
 {
-    m_selectedMapIndex = mapIndex;
+    const CaretMappableDataFile* cdf(getSelectedFile());
+    if (cdf != NULL) {
+        if ((mapIndex >= 0)
+            && (mapIndex < cdf->getNumberOfMaps())) {
+            m_selectedMapIndex = mapIndex;
+        }
+    }
 }
 
 /**
