@@ -20,11 +20,17 @@
 
 #include "CaretVolumeExtension.h"
 #include "XmlSaxParser.h"
+#include "GiftiMetaData.h"
+#include "GiftiMetaDataSaxReader.h"
+#include "GiftiMetaDataXmlElements.h"
 #include "GiftiXmlElements.h"
 #include "PaletteColorMapping.h"
 #include "PaletteColorMappingSaxReader.h"
 #include "PaletteColorMappingXmlElements.h"
+#include "PaletteNormalizationModeEnum.h"
+#include "CaretHierarchy.h"
 #include "CaretLogger.h"
+#include "XmlUnexpectedElementSaxParser.h"
 #include <ctime>
 
 using namespace caret;
@@ -46,6 +52,7 @@ CaretVolumeExtension::clear() {
     m_comment.clear();
     m_date.clear();
     m_attributes.clear();
+    m_metadata.clear();
 }
 
 void CaretVolumeExtension::readFromXmlString(const AString& s)
@@ -70,6 +77,29 @@ void CaretVolumeExtension::writeAsXML(XmlWriter& xmlWriter)
     char buf[101];//we actually only need 20 bytes, but hey
     strftime(buf, 100, "%Y-%m-%dT%H:%M:%S", timeinfo);
     xmlWriter.writeElementCData(CARET_VOL_EXT_DATE, AString(buf));
+    if ( ! m_metadata.isEmpty()) {
+        /*
+         * Prior to "WB-664 Data normalization should be saved in file", VolumeFile did not contain
+         * file metadata.  If one attempts to read a volume file containing metadata with older
+         * versions of wb_view/wb_command or Caret5, XML parsing of the CaretVolumeExtension will
+         * immediately cease if an unexpected element is encountered.  So, to minimize isses with
+         * older versions of software, DO NOT write file metadata if it contains only one
+         * element that is the default value (selected map) of palette normalization.
+         */
+        bool writeFileMetaDataFlag = true;
+        if (m_metadata.getNumberOfMetaData() == 1) {
+            const AString normalizationValueString = m_metadata.get(GiftiMetaDataXmlElements::METADATA_PALETTE_NORMALIZATION_MODE);
+            if ( ! normalizationValueString.isEmpty()) {
+                if (normalizationValueString ==
+                    PaletteNormalizationModeEnum::toName(PaletteNormalizationModeEnum::NORMALIZATION_SELECTED_MAP_DATA)) {
+                    writeFileMetaDataFlag = false;
+                }
+            }
+        }
+        if (writeFileMetaDataFlag) {
+            m_metadata.writeAsXML(xmlWriter);
+        }
+    }
     int numVols = (int)m_attributes.size();
     for (int i = 0; i < numVols; ++i)
     {
@@ -86,7 +116,18 @@ void SubvolumeAttributes::writeAsXML(XmlWriter& xmlWriter, int index)
     xmlWriter.writeStartElement(CARET_VOL_EXT_VOL_INFO, myattrs);
     if (!m_comment.isEmpty()) xmlWriter.writeElementCData(CARET_VOL_EXT_VI_COMMENT, m_comment);
     if (!m_guiLabel.isEmpty()) xmlWriter.writeElementCData(CARET_VOL_EXT_VI_GUI_LABEL, m_guiLabel);
-    if (m_labelTable != NULL) m_labelTable->writeAsXML(xmlWriter);//expect the extension to not have stuff it doesn't need, so just write everything it has
+    GiftiMetaData tempMD = m_metadata;
+    if (m_labelTable != NULL)
+    {
+        const CaretHierarchy& myHier = m_labelTable->getHierarchy();
+        if (myHier.isEmpty())
+        {
+            tempMD.remove("CaretHierarchy");
+        } else {
+            tempMD.set("CaretHierarchy", myHier.writeXMLToString());
+        }
+        m_labelTable->writeAsXML(xmlWriter); //expect the extension to not have stuff it doesn't need, so just write everything it has
+    }
     m_studyMetadata.writeAsXML(xmlWriter);
     if (m_palette != NULL) m_palette->writeAsXML(xmlWriter);
     AString typeString;
@@ -104,6 +145,9 @@ void SubvolumeAttributes::writeAsXML(XmlWriter& xmlWriter, int index)
         case RGB:
             typeString = "RGB";
             break;
+        case RGB_WORKBENCH:
+            typeString = "RGB_WORKBENCH";
+            break;
         case SEGMENTATION:
             typeString = "Segmentation";
             break;
@@ -114,9 +158,49 @@ void SubvolumeAttributes::writeAsXML(XmlWriter& xmlWriter, int index)
             typeString = "Unknown";
     }
     xmlWriter.writeElementCData(CARET_VOL_EXT_VI_TYPE, typeString);
+    if ( ! tempMD.isEmpty()) {
+        tempMD.writeAsXML(xmlWriter);
+    }
     xmlWriter.writeEndElement();
 }
 
+/**
+ * @return Name for the given volume type
+ * @param volumeType
+ *    The volume type
+ */
+AString SubvolumeAttributes::getTypeNameFromType(SubvolumeAttributes::VolumeType volumeType)
+{
+    AString name("INVALID");
+    switch (volumeType) {
+        case SubvolumeAttributes::ANATOMY:
+            name = "ANATOMY";
+            break;
+        case SubvolumeAttributes::FUNCTIONAL:
+            name = "FUNCTIONAL";
+            break;
+        case SubvolumeAttributes::LABEL:
+            name = "LABEL";
+            break;
+        case SubvolumeAttributes::RGB:
+            name = "RGB";
+            break;
+        case SubvolumeAttributes::RGB_WORKBENCH:
+            name = "RGB_WORKBENCH";
+            break;
+        case SubvolumeAttributes::SEGMENTATION:
+            name = "SEGMENTATION";
+            break;
+        case SubvolumeAttributes::UNKNOWN:
+            name = "UNKNOWN";
+            break;
+        case SubvolumeAttributes::VECTOR:
+            name = "VECTOR";
+            break;
+    }
+    return name;
+}
+
 void StudyMetadataLinkSet::writeAsXML(XmlWriter& xmlWriter)
 {//TODO: something
     xmlWriter.writeStartElement(CARET_VOL_EXT_VI_STUDY_META_SET);
@@ -128,6 +212,7 @@ CaretVolumeExtensionXMLReader::CaretVolumeExtensionXMLReader(CaretVolumeExtensio
     CaretAssert(toFill != NULL);
     m_toFill = toFill;
     m_viIndex = -1;
+    m_unexpectedXmlElementSaxParser.grabNew(NULL);
 }
 
 void CaretVolumeExtensionXMLReader::characters(const char* ch)
@@ -144,6 +229,15 @@ void CaretVolumeExtensionXMLReader::characters(const char* ch)
             CaretAssert(m_paletteReader != NULL);
             m_paletteReader->characters(ch);
             break;
+        case ROOT_META_DATA:
+        case VI_META_DATA:
+            CaretAssert(m_metadataReader);
+            m_metadataReader->characters(ch);
+            break;
+        case UNEXPECTED_XML:
+            CaretAssert(m_unexpectedXmlElementSaxParser);
+            m_unexpectedXmlElementSaxParser->characters(ch);
+            break;
         default:
             m_charDataStack.back() += ch;
     }
@@ -178,8 +272,23 @@ void CaretVolumeExtensionXMLReader::endElement(const AString& namespaceURI, cons
             m_toFill->m_date = elemCharData;
             break;
         case VOLUME_INFORMATION:
+        {
+            CaretAssertVectorIndex(m_toFill->m_attributes, m_viIndex);
+            auto thisAttr = m_toFill->m_attributes[m_viIndex];
+            try
+            {
+                if (thisAttr->m_type == SubvolumeAttributes::LABEL && thisAttr->m_labelTable != NULL && thisAttr->m_metadata.exists("CaretHierarchy"))
+                {
+                    CaretHierarchy myHier;
+                    myHier.readXML(thisAttr->m_metadata.get("CaretHierarchy"));
+                    thisAttr->m_labelTable->setHierarchy(myHier);
+                }
+            } catch (CaretException& e) {
+                CaretLogWarning("failed to parse hierarchy XML, discarding hierarchy: " + e.whatString());
+            }
             m_viIndex = -1;
             break;
+        }
         case VI_COMMENT:
             CaretAssertVectorIndex(m_toFill->m_attributes, m_viIndex);
             m_toFill->m_attributes[m_viIndex]->m_comment = elemCharData;
@@ -217,6 +326,18 @@ void CaretVolumeExtensionXMLReader::endElement(const AString& namespaceURI, cons
                 popState = false;
             }
             break;
+        case ROOT_META_DATA:
+        case VI_META_DATA:
+            CaretAssert(m_metadataReader);
+            m_metadataReader->endElement(namespaceURI, localName, qualifiedName);
+            if (qualifiedName == GiftiXmlElements::TAG_METADATA) {
+                m_metadataReader->endDocument();
+                m_metadataReader.grabNew(NULL);
+            }
+            else {
+                popState = false;
+            }
+            break;
         case VOLUME_TYPE:
             CaretAssertVectorIndex(m_toFill->m_attributes, m_viIndex);
             if (elemCharData == "Anatomy")
@@ -228,6 +349,8 @@ void CaretVolumeExtensionXMLReader::endElement(const AString& namespaceURI, cons
                 m_toFill->m_attributes[m_viIndex]->m_type = SubvolumeAttributes::LABEL;
             } else if (elemCharData == "RGB") {
                 m_toFill->m_attributes[m_viIndex]->m_type = SubvolumeAttributes::RGB;
+            } else if (elemCharData == "RGB_WORKBENCH") {
+                m_toFill->m_attributes[m_viIndex]->m_type = SubvolumeAttributes::RGB_WORKBENCH;
             } else if (elemCharData == "Segmentation") {
                 m_toFill->m_attributes[m_viIndex]->m_type = SubvolumeAttributes::SEGMENTATION;
             } else if (elemCharData == "Vector") {
@@ -236,6 +359,23 @@ void CaretVolumeExtensionXMLReader::endElement(const AString& namespaceURI, cons
                 m_toFill->m_attributes[m_viIndex]->m_type = SubvolumeAttributes::UNKNOWN;
             }
             break;
+        case UNEXPECTED_XML:
+            CaretAssert(m_unexpectedXmlElementSaxParser);
+            switch (m_unexpectedXmlElementSaxParser->endElement(namespaceURI, localName, qualifiedName)) {
+                case XmlUnexpectedElementSaxParser::ReturnCodeEnum::DONE:
+                    warning(XmlSaxParserException("Unexpected XML ignored:\n"
+                                                  + m_unexpectedXmlElementSaxParser->getUnexpectedContentXML()));
+                    m_unexpectedXmlElementSaxParser.grabNew(NULL);
+                    break;
+                case XmlUnexpectedElementSaxParser::ReturnCodeEnum::ERROR:
+                    throw XmlSaxParserException("Processing of unexpected elements failed: "
+                                                + m_unexpectedXmlElementSaxParser->getUnexpectedContentXML());
+                    break;
+                case XmlUnexpectedElementSaxParser::ReturnCodeEnum::NOT_DONE:
+                    popState = false;
+                    break;
+            }
+            break;
     }
     if (popState)
     {
@@ -300,6 +440,11 @@ void CaretVolumeExtensionXMLReader::startElement(const AString& uri, const AStri
                         m_toFill->m_attributes.resize(m_viIndex + 1);//don't worry, CaretPointer copy is relatively cheap
                     }
                     m_toFill->m_attributes[m_viIndex].grabNew(new SubvolumeAttributes());
+                } else if (qName == GiftiXmlElements::TAG_METADATA) {
+                    nextState = ROOT_META_DATA;
+                    m_metadataReader.grabNew(new GiftiMetaDataSaxReader(&m_toFill->m_metadata));
+                    m_metadataReader->startDocument();
+                    m_metadataReader->startElement(uri, localName, qName, atts);
                 }//anything else gets caught in INVALID below
                 break;
             case VOLUME_INFORMATION:
@@ -324,6 +469,12 @@ void CaretVolumeExtensionXMLReader::startElement(const AString& uri, const AStri
                     m_paletteReader.grabNew(new PaletteColorMappingSaxReader(m_toFill->m_attributes[m_viIndex]->m_palette));
                     m_paletteReader->startDocument();
                     m_paletteReader->startElement(uri, localName, qName, atts);
+                } else if (qName == GiftiXmlElements::TAG_METADATA) {
+                    nextState = VI_META_DATA;
+                    CaretAssertVectorIndex(m_toFill->m_attributes, m_viIndex);
+                    m_metadataReader.grabNew(new GiftiMetaDataSaxReader(&m_toFill->m_attributes[m_viIndex]->m_metadata));
+                    m_metadataReader->startDocument();
+                    m_metadataReader->startElement(uri, localName, qName, atts);
                 } else if (qName == CARET_VOL_EXT_VI_TYPE) {
                     nextState = VOLUME_TYPE;
                 }
@@ -341,13 +492,32 @@ void CaretVolumeExtensionXMLReader::startElement(const AString& uri, const AStri
                 CaretAssert(m_paletteReader != NULL);
                 m_paletteReader->startElement(uri, localName, qName, atts);
                 break;
+            case ROOT_META_DATA:
+            case VI_META_DATA:
+                addState = false;
+                CaretAssert(m_metadataReader);
+                m_metadataReader->startElement(uri, localName, qName, atts);
+                break;
+            case UNEXPECTED_XML:
+                addState = false;
+                CaretAssert(m_unexpectedXmlElementSaxParser);
+                m_unexpectedXmlElementSaxParser->startElement(uri, localName, qName, atts);
+                break;
         }
     }
     if (addState)
     {
         if (nextState == INVALID)
         {
-            throw XmlSaxParserException(AString("CaretVolumeExtension encountered an unexpected element: ") + invalidInfo);
+            /*
+             * If the invalid element name is not empty,
+             * then we are processing child elements of the
+             * invalid element.
+             */
+            CaretAssert(m_unexpectedXmlElementSaxParser == NULL);
+            m_unexpectedXmlElementSaxParser.grabNew(new XmlUnexpectedElementSaxParser());
+            m_unexpectedXmlElementSaxParser->startElement(uri, localName, qName, atts);
+            nextState = UNEXPECTED_XML;
         }
         m_stateStack.push_back(nextState);
         m_charDataStack.push_back(AString());
