@@ -28,6 +28,7 @@
 #include "CaretAssert.h"
 #include "CaretMappableDataFile.h"
 #include "ChartableTwoFileHistogramChart.h"
+#include "ChartableTwoFileLineLayerChart.h"
 #include "ChartableTwoFileLineSeriesChart.h"
 #include "ChartableTwoFileMatrixChart.h"
 #include "CiftiConnectivityMatrixParcelFile.h"
@@ -82,6 +83,7 @@ ChartableTwoFileDelegate::updateAfterFileChanged()
     
     ChartTwoHistogramContentTypeEnum::Enum histogramType   = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_UNSUPPORTED;
     ChartTwoLineSeriesContentTypeEnum::Enum lineSeriesType = ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_UNSUPPORTED;
+    ChartTwoLineLayerContentTypeEnum::Enum lineLayerType   = ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_UNSUPPORTED;
     ChartTwoMatrixContentTypeEnum::Enum matrixType         = ChartTwoMatrixContentTypeEnum::MATRIX_CONTENT_UNSUPPORTED;
     
     std::vector<ChartTwoMatrixLoadingDimensionEnum::Enum> validMatrixRowColumnSelectionDimensions;
@@ -100,10 +102,12 @@ ChartableTwoFileDelegate::updateAfterFileChanged()
             break;
         case DataFileTypeEnum::CONNECTIVITY_DENSE_SCALAR:
             histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            lineLayerType = ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_ROW_DATA;
             lineSeriesType = ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_BRAINORDINATE_DATA;
             break;
         case DataFileTypeEnum::CONNECTIVITY_DENSE_TIME_SERIES:
             histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            lineLayerType = ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_ROW_DATA;
             lineSeriesType = ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_BRAINORDINATE_DATA;
             break;
         case DataFileTypeEnum::CONNECTIVITY_PARCEL:
@@ -114,6 +118,14 @@ ChartableTwoFileDelegate::updateAfterFileChanged()
             break;
         case DataFileTypeEnum::CONNECTIVITY_PARCEL_DENSE:
             histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            matrixType = ChartTwoMatrixContentTypeEnum::MATRIX_CONTENT_BRAINORDINATE_MAPPABLE;
+            validMatrixRowColumnSelectionDimensions.push_back(ChartTwoMatrixLoadingDimensionEnum::CHART_MATRIX_LOADING_BY_ROW);
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC:
+            histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            lineLayerType = ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_ROW_DATA;
+            lineSeriesType = ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_BRAINORDINATE_DATA;
+            validMatrixRowColumnSelectionDimensions.push_back(ChartTwoMatrixLoadingDimensionEnum::CHART_MATRIX_LOADING_BY_COLUMN);
             break;
         case DataFileTypeEnum::CONNECTIVITY_PARCEL_LABEL:
             matrixType = ChartTwoMatrixContentTypeEnum::MATRIX_CONTENT_BRAINORDINATE_MAPPABLE;
@@ -121,43 +133,63 @@ ChartableTwoFileDelegate::updateAfterFileChanged()
             break;
         case DataFileTypeEnum::CONNECTIVITY_PARCEL_SCALAR:
             histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            lineLayerType = ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_ROW_DATA;
             lineSeriesType = ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_BRAINORDINATE_DATA;
             matrixType = ChartTwoMatrixContentTypeEnum::MATRIX_CONTENT_BRAINORDINATE_MAPPABLE;
             validMatrixRowColumnSelectionDimensions.push_back(ChartTwoMatrixLoadingDimensionEnum::CHART_MATRIX_LOADING_BY_COLUMN);
             break;
         case DataFileTypeEnum::CONNECTIVITY_PARCEL_SERIES:
             histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            lineLayerType = ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_ROW_DATA;
             lineSeriesType = ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_BRAINORDINATE_DATA;
             matrixType = ChartTwoMatrixContentTypeEnum::MATRIX_CONTENT_BRAINORDINATE_MAPPABLE;
             validMatrixRowColumnSelectionDimensions.push_back(ChartTwoMatrixLoadingDimensionEnum::CHART_MATRIX_LOADING_BY_COLUMN);
             break;
         case DataFileTypeEnum::CONNECTIVITY_SCALAR_DATA_SERIES:
+            histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            lineLayerType = ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_ROW_DATA;
             lineSeriesType = ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_ROW_SCALAR_DATA;
             matrixType = ChartTwoMatrixContentTypeEnum::MATRIX_CONTENT_SCALARS;
             validMatrixRowColumnSelectionDimensions.push_back(ChartTwoMatrixLoadingDimensionEnum::CHART_MATRIX_LOADING_BY_ROW);
             break;
         case DataFileTypeEnum::ANNOTATION:
             break;
+        case DataFileTypeEnum::ANNOTATION_TEXT_SUBSTITUTION:
+            break;
         case DataFileTypeEnum::BORDER:
             break;
         case DataFileTypeEnum::CONNECTIVITY_FIBER_ORIENTATIONS_TEMPORARY:
             break;
         case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_TEMPORARY:
             break;
+        case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_MAPS:
+            break;
+        case DataFileTypeEnum::CZI_IMAGE_FILE:
+            break;
         case DataFileTypeEnum::FOCI:
             break;
+        case DataFileTypeEnum::HISTOLOGY_SLICES:
+            break;
         case DataFileTypeEnum::IMAGE:
             break;
         case DataFileTypeEnum::LABEL:
             break;
         case DataFileTypeEnum::METRIC:
             histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            lineLayerType = ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_BRAINORDINATE_DATA;
             lineSeriesType = ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_BRAINORDINATE_DATA;
             break;
+        case DataFileTypeEnum::METRIC_DYNAMIC:
+            histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            break;
+        case DataFileTypeEnum::OME_ZARR_IMAGE_FILE:
+            break;
         case DataFileTypeEnum::PALETTE:
             break;
         case DataFileTypeEnum::RGBA:
             break;
+        case DataFileTypeEnum::SAMPLES:
+            break;
         case DataFileTypeEnum::SCENE:
             break;
         case DataFileTypeEnum::SPECIFICATION:
@@ -169,10 +201,13 @@ ChartableTwoFileDelegate::updateAfterFileChanged()
         case DataFileTypeEnum::VOLUME:
             if ( ! m_caretMappableDataFile->isMappedWithLabelTable()) {
                 histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+                lineLayerType = ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_BRAINORDINATE_DATA;
                 lineSeriesType = ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_BRAINORDINATE_DATA;
             }
             break;
-            
+        case DataFileTypeEnum::VOLUME_DYNAMIC:
+            histogramType = ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_MAP_DATA;
+            break;
     }
     
     if (m_histogramCharting) {
@@ -185,6 +220,16 @@ ChartableTwoFileDelegate::updateAfterFileChanged()
                                                                                                                  m_caretMappableDataFile));
     }
     
+    if (m_lineLayerCharting) {
+        if (lineLayerType != m_lineLayerCharting->getLineLayerContentType()) {
+            m_lineLayerCharting.reset();
+        }
+    }
+    if ( ! m_lineLayerCharting) {
+        m_lineLayerCharting = std::unique_ptr<ChartableTwoFileLineLayerChart>(new ChartableTwoFileLineLayerChart(lineLayerType,
+                                                                                                                    m_caretMappableDataFile));
+    }
+    
     if (m_lineSeriesCharting) {
         if (lineSeriesType != m_lineSeriesCharting->getLineSeriesContentType()) {
             m_lineSeriesCharting.reset();
@@ -238,6 +283,9 @@ ChartableTwoFileDelegate::clear()
     m_histogramCharting = std::unique_ptr<ChartableTwoFileHistogramChart>(new ChartableTwoFileHistogramChart(ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_UNSUPPORTED,
                                                                                                              m_caretMappableDataFile));
     
+    m_lineLayerCharting = std::unique_ptr<ChartableTwoFileLineLayerChart>(new ChartableTwoFileLineLayerChart(ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_UNSUPPORTED,
+                                                                                                                m_caretMappableDataFile));
+    
     m_lineSeriesCharting = std::unique_ptr<ChartableTwoFileLineSeriesChart>(new ChartableTwoFileLineSeriesChart(ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_UNSUPPORTED,
                                                                                                                 m_caretMappableDataFile));
     
@@ -254,6 +302,7 @@ void
 ChartableTwoFileDelegate::clearModified()
 {
     m_histogramCharting->clearModified();
+    m_lineLayerCharting->clearModified();
     m_lineSeriesCharting->clearModified();
     m_matrixCharting->clearModified();
 }
@@ -270,6 +319,9 @@ ChartableTwoFileDelegate::isModified() const
     if (m_matrixCharting->isModified()) {
         return true;
     }
+    if (m_lineLayerCharting->isModified()) {
+        return true;
+    }
     if (m_lineSeriesCharting->isModified()) {
         return true;
     }
@@ -315,6 +367,24 @@ ChartableTwoFileDelegate::getHistogramCharting() const
     return m_histogramCharting.get();
 }
 
+/**
+ * @return Line layer charting.
+ */
+ChartableTwoFileLineLayerChart*
+ChartableTwoFileDelegate::getLineLayerCharting()
+{
+    return m_lineLayerCharting.get();
+}
+
+/**
+ * @return Line layer charting.
+ */
+const ChartableTwoFileLineLayerChart*
+ChartableTwoFileDelegate::getLineLayerCharting() const
+{
+    return m_lineLayerCharting.get();
+}
+
 /**
  * @return Line series charting.
  */
@@ -397,11 +467,11 @@ ChartableTwoFileDelegate::isChartingSupportedForChartTwoDataType(const ChartTwoD
 bool
 ChartableTwoFileDelegate::isChartingSupportedForChartTwoCompoundDataType(const ChartTwoCompoundDataType& chartCompoundDataType) const
 {
-    std::vector<ChartTwoCompoundDataType> chartCompoundDataTypes;
+    std::vector<const ChartTwoCompoundDataType*> chartCompoundDataTypes;
     getSupportedChartTwoCompoundDataTypes(chartCompoundDataTypes);
     
     for (auto& ccdt : chartCompoundDataTypes) {
-        if (ccdt == chartCompoundDataType) {
+        if (*ccdt == chartCompoundDataType) {
             return true;
         }
     }
@@ -423,6 +493,9 @@ ChartableTwoFileDelegate::getSupportedChartTwoDataTypes(std::vector<ChartTwoData
     if (m_histogramCharting != NULL) {
         chartDataTypesOut.push_back(m_histogramCharting->getChartTwoDataType());
     }
+    if (m_lineLayerCharting != NULL) {
+        chartDataTypesOut.push_back(m_lineLayerCharting->getChartTwoDataType());
+    }
     if (m_lineSeriesCharting!= NULL) {
         chartDataTypesOut.push_back(m_lineSeriesCharting->getChartTwoDataType());
     }
@@ -438,13 +511,16 @@ ChartableTwoFileDelegate::getSupportedChartTwoDataTypes(std::vector<ChartTwoData
  *     Output containing all chart data types supported by this data file.
  */
 void
-ChartableTwoFileDelegate::getSupportedChartTwoCompoundDataTypes(std::vector<ChartTwoCompoundDataType>& chartCompoundDataTypesOut) const
+ChartableTwoFileDelegate::getSupportedChartTwoCompoundDataTypes(std::vector<const ChartTwoCompoundDataType*>& chartCompoundDataTypesOut) const
 {
     chartCompoundDataTypesOut.clear();
     
     if (m_histogramCharting->getHistogramContentType() != ChartTwoHistogramContentTypeEnum::HISTOGRAM_CONTENT_TYPE_UNSUPPORTED) {
         chartCompoundDataTypesOut.push_back(m_histogramCharting->getChartTwoCompoundDataType());
     }
+    if (m_lineLayerCharting->getLineLayerContentType() != ChartTwoLineLayerContentTypeEnum::LINE_LAYER_CONTENT_UNSUPPORTED) {
+        chartCompoundDataTypesOut.push_back(m_lineLayerCharting->getChartTwoCompoundDataType());
+    }
     if (m_lineSeriesCharting->getLineSeriesContentType() != ChartTwoLineSeriesContentTypeEnum::LINE_SERIES_CONTENT_UNSUPPORTED) {
         chartCompoundDataTypesOut.push_back(m_lineSeriesCharting->getChartTwoCompoundDataType());
     }
@@ -469,11 +545,11 @@ bool
 ChartableTwoFileDelegate::getChartTwoCompoundDataTypeForChartTwoDataType(const ChartTwoDataTypeEnum::Enum chartDataType,
                                                                 ChartTwoCompoundDataType& chartCompoundDataTypeOut) const
 {
-    std::vector<ChartTwoCompoundDataType> chartCompoundDataTypes;
+    std::vector<const ChartTwoCompoundDataType*> chartCompoundDataTypes;
     getSupportedChartTwoCompoundDataTypes(chartCompoundDataTypes);
     for (auto& cdt : chartCompoundDataTypes) {
-        if (cdt.getChartTwoDataType() == chartDataType) {
-            chartCompoundDataTypeOut = cdt;
+        if (cdt->getChartTwoDataType() == chartDataType) {
+            chartCompoundDataTypeOut = *cdt;
             return true;
         }
     }
@@ -512,9 +588,13 @@ ChartableTwoFileDelegate::saveToScene(const SceneAttributes* sceneAttributes,
         sceneClass->addClass(m_histogramCharting->saveToScene(sceneAttributes,
                                                               "m_histogramCharting"));
     }
+    if (m_lineLayerCharting != NULL) {
+        sceneClass->addClass(m_lineLayerCharting->saveToScene(sceneAttributes,
+                                                              "m_lineLayerCharting"));
+    }
     if (m_lineSeriesCharting != NULL) {
         sceneClass->addClass(m_lineSeriesCharting->saveToScene(sceneAttributes,
-                                                              "m_lineSeriesCharting"));
+                                                               "m_lineSeriesCharting"));
     }
     if (m_matrixCharting != NULL) {
         sceneClass->addClass(m_matrixCharting->saveToScene(sceneAttributes,
@@ -553,11 +633,18 @@ ChartableTwoFileDelegate::restoreFromScene(const SceneAttributes* sceneAttribute
                                               histogramClass);
     }
     
+    const SceneClass* lineLayerClass = sceneClass->getClass("m_lineLayerCharting");
+    if (lineLayerClass != NULL) {
+        CaretAssert(m_lineLayerCharting);
+        m_lineLayerCharting->restoreFromScene(sceneAttributes,
+                                              lineLayerClass);
+    }
+    
     const SceneClass* lineSeriesClass = sceneClass->getClass("m_lineSeriesCharting");
     if (lineSeriesClass != NULL) {
         CaretAssert(m_lineSeriesCharting);
         m_lineSeriesCharting->restoreFromScene(sceneAttributes,
-                                           lineSeriesClass);
+                                               lineSeriesClass);
     }
     
     const SceneClass* matrixClass = sceneClass->getClass("m_matrixCharting");
