@@ -387,7 +387,7 @@ CiftiFiberTrajectoryFile::getFileMetaData() const
 bool
 CiftiFiberTrajectoryFile::isSurfaceMappable() const
 {
-    return false;
+    return true;
 }
 
 /**
@@ -906,7 +906,7 @@ CiftiFiberTrajectoryFile::newFiberTrajectoryFileFromLoadedRowData(const AString&
         
         
         const AString tempFileName = (QDir::tempPath()
-                              + QDir::separator()
+                                      + "/"
                                       + newFile->getFileNameNoPath());
         std::cout << "Filename: " << qPrintable(tempFileName) << std::endl;
         
@@ -966,7 +966,6 @@ CiftiFiberTrajectoryFile::writeLoadedDataToFile(const AString& filename) const
     std::vector<int64_t> fiberIndices;
     std::vector<FiberFractions> fiberFractions;
     
-    int ctr = 0;
     for (std::vector<const FiberOrientationTrajectory*>::const_iterator iter = trajectories.begin();
          iter != trajectories.end();
          iter++) {
@@ -990,8 +989,6 @@ CiftiFiberTrajectoryFile::writeLoadedDataToFile(const AString& filename) const
 //                std::cout << "Fiber " << ctr << vec[i] << std::endl;
 //            }
 //        }
-        
-        ctr++;
     }
     
     /*
@@ -1760,10 +1757,10 @@ CiftiFiberTrajectoryFile::finishRestorationOfScene()
 /**
  * @return a REFERENCE to the fiber fractions that were loaded.
  */
-const std::vector<FiberOrientationTrajectory*>&
+const std::vector<FiberOrientationTrajectory*>*
 CiftiFiberTrajectoryFile::getLoadedFiberOrientationTrajectories() const
 {
-    return m_fiberOrientationTrajectories;
+    return &m_fiberOrientationTrajectories;
 }
 
 /**
@@ -1942,7 +1939,7 @@ CiftiFiberTrajectoryFile::getDataForSelector(const MapFileDataSelector& /*mapFil
  *     True if brainordinates in this file are subset or equal to the given file, else false.
  */
 CaretMappableDataFile::BrainordinateMappingMatch
-CiftiFiberTrajectoryFile::getBrainordinateMappingMatch(const CaretMappableDataFile* /*mapFile*/) const
+CiftiFiberTrajectoryFile::getBrainordinateMappingMatchImplementation(const CaretMappableDataFile* /*mapFile*/) const
 {
     return BrainordinateMappingMatch::NO;
 }
