@@ -108,10 +108,10 @@ namespace caret {
         
         virtual const GiftiLabelTable* getMapLabelTable(const int32_t mapIndex) const;
         
-        virtual void getPaletteNormalizationModesSupported(std::vector<PaletteNormalizationModeEnum::Enum>& modesSupportedOut);
+        virtual void getPaletteNormalizationModesSupported(std::vector<PaletteNormalizationModeEnum::Enum>& modesSupportedOut) const;
+        
+        virtual void updateScalarColoringForMap(const int32_t mapIndex) override;
         
-        virtual void updateScalarColoringForMap(const int32_t mapIndex,
-                                                const PaletteFile* paletteFile);
         virtual void readFile(const AString& filename);
         
         virtual void writeFile(const AString& filename);
@@ -140,7 +140,7 @@ namespace caret {
         
         void loadDataForRowIndex(const int64_t rowIndex);
         
-        const std::vector<FiberOrientationTrajectory*>& getLoadedFiberOrientationTrajectories() const;
+        const std::vector<FiberOrientationTrajectory*>* getLoadedFiberOrientationTrajectories() const;
         
         void clearLoadedFiberOrientations();
         
@@ -171,6 +171,15 @@ namespace caret {
         
         void addToDataFileContentInformation(DataFileContentInformation& dataFileInformation);
         
+        bool hasCiftiXML() const;
+        
+        const CiftiXML getCiftiXML() const;
+
+        virtual void getDataForSelector(const MapFileDataSelector& mapFileDataSelector,
+                                        std::vector<float>& dataOut) const override;
+
+        virtual BrainordinateMappingMatch getBrainordinateMappingMatchImplementation(const CaretMappableDataFile* mapFile) const override;
+        
         // ADD_NEW_METHODS_HERE
         
     private:
