@@ -62,7 +62,7 @@ CiftiMappableConnectivityMatrixDataFile::CiftiMappableConnectivityMatrixDataFile
     m_sceneAssistant->add("m_connectivityDataLoaded",
                           "ConnectivityDataLoaded",
                           m_connectivityDataLoaded);
-    m_sceneAssistant->add("m_dataLoadingEnabled",
+    m_sceneAssistant->add("+",
                            &m_dataLoadingEnabled);
 }
 
@@ -103,6 +103,9 @@ CiftiMappableConnectivityMatrixDataFile::clearPrivate()
     if (getDataFileType() == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC) {
         m_chartLoadingDimension = ChartMatrixLoadingDimensionEnum::CHART_MATRIX_LOADING_BY_COLUMN;
     }
+    if (getDataFileType() == DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC) {
+        m_chartLoadingDimension = ChartMatrixLoadingDimensionEnum::CHART_MATRIX_LOADING_BY_COLUMN;
+    }
 }
 
 /**
@@ -193,7 +196,12 @@ void
 CiftiMappableConnectivityMatrixDataFile::getMapData(const int32_t /*mapIndex*/,
                                   std::vector<float>& dataOut) const
 {
-    dataOut = m_loadedRowData;
+    if (!isEnabledAsLayer())
+    {//TSC: HACK to make identification show empty string instead of number when dynconn is used, then set to not load, not layer
+        dataOut.clear();
+    } else {
+        dataOut = m_loadedRowData;
+    }
 }
 
 /**
@@ -261,9 +269,9 @@ CiftiMappableConnectivityMatrixDataFile::getRowColumnIndexForNodeWhenLoading(con
         case CiftiMappingType::PARCELS:
             rowOrColumnIndex = ciftiXML.getParcelsMap(ciftiDirection).getIndexForNode(nodeIndex, structure);
             break;
-        case CIFTI_INDEX_TYPE_SCALARS:
+        case CiftiMappingType::SCALARS:
             break;
-        case CIFTI_INDEX_TYPE_TIME_POINTS:
+        case CiftiMappingType::SERIES:
             break;
         default:
             CaretAssert(0);
@@ -335,34 +343,33 @@ CiftiMappableConnectivityMatrixDataFile::getRowColumnIndicesForNodesWhenLoading(
  *     Indices of the row.
  * @param columnIndices
  *     Indices of the column.
- * @param rowAverageOut
- *     Average values for rows.
- * @param columnAverageOut
- *     Average value for columns.
+ * @param dataAverageOut
+ *     Average values for data.
+ * @return True if data was loaded, else false.
  */
-void
+bool
 CiftiMappableConnectivityMatrixDataFile::getRowColumnAverageForIndices(const std::vector<int64_t>& rowIndices,
                                                                        const std::vector<int64_t>& columnIndices,
-                                                                       std::vector<float>& rowAverageOut,
-                                                                       std::vector<float>& columnAverageOut)
+                                                                       std::vector<float>& dataAverageOut)
 {
-    columnAverageOut.clear();
-    rowAverageOut.clear();
+    std::fill(dataAverageOut.begin(),
+              dataAverageOut.end(),
+              0.0);
     
-    int64_t dataLength = 0;
+    const int64_t dataLength(dataAverageOut.size());
     std::vector<int64_t> indices;
     
     bool doRowsFlag = false;
     if (! rowIndices.empty()) {
-        dataLength = m_ciftiFile->getNumberOfColumns();
         indices    = rowIndices;
         doRowsFlag = true;
     }
     else if ( ! columnIndices.empty()) {
-        dataLength = m_ciftiFile->getNumberOfRows();
         indices    = columnIndices;
     }
     
+    bool dataWasLoadedFlag(false);
+    
     const int64_t numIndices = static_cast<int64_t>(indices.size());
     if (numIndices > 0) {
         std::vector<double> sum(dataLength, 0.0);
@@ -372,10 +379,10 @@ CiftiMappableConnectivityMatrixDataFile::getRowColumnAverageForIndices(const std
              iter != indices.end();
              iter++) {
             if (doRowsFlag) {
-                getDataForRow(&data[0], *iter);
+                getProcessedDataForRow(data, *iter);
             }
             else {
-                getDataForColumn(&data[0], *iter);
+                getProcessedDataForColumn(&data[0], *iter);
             }
             
             for (int64_t i = 0; i < dataLength; i++) {
@@ -384,23 +391,19 @@ CiftiMappableConnectivityMatrixDataFile::getRowColumnAverageForIndices(const std
                 sum[i] += data[i];
             }
         }
-
-        CaretAssert(dataLength > 0);
+        
         std::vector<float> average(dataLength);
-        const float floatNumIndices = numIndices; //dataLength;
+        const float floatNumIndices = numIndices;
         for (int64_t i = 0; i < dataLength; i++) {
-            CaretAssertVectorIndex(average, i);
             CaretAssertVectorIndex(sum, i);
-            average[i] = sum[i] / floatNumIndices;
-        }
-
-        if (doRowsFlag) {
-            rowAverageOut = average;
-        }
-        else {
-            columnAverageOut = average;
+            CaretAssertVectorIndex(dataAverageOut, i);
+            dataAverageOut[i] = sum[i] / floatNumIndices;
         }
+        
+        dataWasLoadedFlag = true;
     }
+    
+    return dataWasLoadedFlag;
 }
 
 /**
@@ -457,7 +460,13 @@ CiftiMappableConnectivityMatrixDataFile::getRowColumnIndexForVoxelAtCoordinateWh
     }
     
     int64_t ijk[3];
-    enclosingVoxel(xyz[0], xyz[1], xyz[2], ijk[0], ijk[1], ijk[2]);
+    if ((getDataFileType() == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC)
+        || (getDataFileType() == DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC)) {
+        enclosingVoxel(xyz[0], xyz[1], xyz[2], ijk[0], ijk[1], ijk[2]);
+    }
+    else {
+        enclosingVoxelForDataLoading(xyz[0], xyz[1], xyz[2], ijk[0], ijk[1], ijk[2]);
+    }
     return getRowColumnIndexForVoxelIndexWhenLoading(ijk,
                                                      rowIndexOut,
                                                      columnIndexOut);
@@ -499,12 +508,16 @@ CiftiMappableConnectivityMatrixDataFile::getRowColumnIndexForVoxelIndexWhenLoadi
     /*
      * Get the mapping type
      */
-    if (indexValid(ijk[0], ijk[1], ijk[2])) {
+    const bool indexValidFlag = (((getDataFileType() == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC)
+                                  || (getDataFileType() == DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC))
+                                 ? indexValid(ijk[0], ijk[1], ijk[2])
+                                 : indexValidForDataLoading(ijk[0], ijk[1], ijk[2]));
+    if (indexValidFlag) {
         switch (rowMappingType) {
-            case CIFTI_INDEX_TYPE_BRAIN_MODELS:
+            case CiftiMappingType::BRAIN_MODELS:
                 rowOrColumnIndex = ciftiXML.getBrainModelsMap(ciftiDirection).getIndexForVoxel(ijk);
                 break;
-            case CIFTI_INDEX_TYPE_PARCELS:
+            case CiftiMappingType::PARCELS:
                 rowOrColumnIndex = ciftiXML.getParcelsMap(ciftiDirection).getIndexForVoxel(ijk);
                 break;
             default:
@@ -617,9 +630,9 @@ CiftiMappableConnectivityMatrixDataFile::getProcessedDataForColumn(float* dataOu
  * @param index of the row.
  */
 void
-CiftiMappableConnectivityMatrixDataFile::getProcessedDataForRow(float* dataOut, const int64_t& index) const
+CiftiMappableConnectivityMatrixDataFile::getProcessedDataForRow(std::vector<float>& dataOut, const int64_t& index) const
 {
-    m_ciftiFile->getRow(dataOut,
+    m_ciftiFile->getRow(&dataOut[0],
                         index);
 }
 
@@ -659,17 +672,17 @@ CiftiMappableConnectivityMatrixDataFile::loadDataForRowIndex(const int64_t rowIn
         if ((rowIndex >= 0)
             && (rowIndex < m_ciftiFile->getNumberOfRows())) {
             m_rowLoadedTextForMapName = ("Row: "
-                                         + AString::number(rowIndex));
+                                         + AString::number(rowIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI));
             
             m_rowLoadedText = ("Row_"
-                               + AString::number(rowIndex));
+                               + AString::number(rowIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI));
             CaretAssert((rowIndex >= 0) && (rowIndex < m_ciftiFile->getNumberOfRows()));
             m_loadedRowData.resize(dataCount);
             
-            getProcessedDataForRow(&m_loadedRowData[0],
+            getProcessedDataForRow(m_loadedRowData,
                                    rowIndex);
             
-            CaretLogFine("Read row " + AString::number(rowIndex));
+            CaretLogFine("Read row " + AString::number(rowIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI));
             m_connectivityDataLoaded->setRowColumnLoading(rowIndex,
                                                           -1);
         }
@@ -700,17 +713,17 @@ CiftiMappableConnectivityMatrixDataFile::loadDataForColumnIndex(const int64_t co
         if ((columnIndex >= 0)
             && (columnIndex < m_ciftiFile->getNumberOfColumns())) {
             m_rowLoadedTextForMapName = ("Column: "
-                                         + AString::number(columnIndex));
+                                         + AString::number(columnIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI));
             
             m_rowLoadedText = ("Column_"
-                               + AString::number(columnIndex));
+                               + AString::number(columnIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI));
             CaretAssert((columnIndex >= 0) && (columnIndex < m_ciftiFile->getNumberOfColumns()));
             m_loadedRowData.resize(dataCount);
             
             getProcessedDataForColumn(&m_loadedRowData[0],
                                       columnIndex);
             
-            CaretLogFine("Read column " + AString::number(columnIndex));
+            CaretLogFine("Read column " + AString::number(columnIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI));
             m_connectivityDataLoaded->setRowColumnLoading(-1,
                                                           columnIndex);
         }
@@ -741,15 +754,12 @@ CiftiMappableConnectivityMatrixDataFile::loadDataForColumnIndex(const int64_t co
  */
 void
 CiftiMappableConnectivityMatrixDataFile::loadMapDataForSurfaceNode(const int32_t /*mapIndex*/,
-                                                           const int32_t surfaceNumberOfNodes,
-                                                           const StructureEnum::Enum structure,
+                                                                   const int32_t surfaceNumberOfNodes,
+                                                                   const StructureEnum::Enum structure,
                                                                    const int32_t nodeIndex,
                                                                    int64_t& rowIndexOut,
                                                                    int64_t& columnIndexOut)
 {
-    ElapsedTimer timer;
-    timer.start();
-    
     rowIndexOut    = -1;
     columnIndexOut = -1;
     
@@ -758,6 +768,14 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForSurfaceNode(const int32_t
         return;
     }
   
+    /*
+     * If not enabled as a layer, clear any previous
+     * loaded data and do not load any new data
+     */
+    if ( ! isEnabledAsLayer()) {
+        setLoadedRowDataToAllZeros();
+        return;
+    }
     
     /*
      * Loading of data disabled?
@@ -766,6 +784,9 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForSurfaceNode(const int32_t
         return;
     }
     
+    ElapsedTimer timer;
+    timer.start();
+    
     /*
      * Zero out here so that data only gets cleared when data
      * is to be loaded.
@@ -786,7 +807,8 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForSurfaceNode(const int32_t
         
         if (rowIndex >= 0) {
             int64_t dataCount = m_ciftiFile->getNumberOfColumns();
-            if (getDataFileType() == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC) {
+            if ((getDataFileType() == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC)
+                || (getDataFileType() == DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC)) {
                 /* 
                  * Dense dynamic is special case where number of rows equals number of brainordinates.
                  * Number of columns is number of time points
@@ -796,24 +818,24 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForSurfaceNode(const int32_t
             
             if (dataCount > 0) {
                 m_rowLoadedTextForMapName = ("Row: "
-                                         + AString::number(rowIndex)
-                                         + ", Node Index: "
+                                         + AString::number(rowIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI)
+                                         + ", Vertex Index: "
                                          + AString::number(nodeIndex)
                                          + ", Structure: "
                                          + StructureEnum::toName(structure));
 
                 m_rowLoadedText = ("Row_"
-                                   + AString::number(rowIndex)
-                                   + "_Node_Index_"
+                                   + AString::number(rowIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI)
+                                   + "_Vertex_Index_"
                                    + AString::number(nodeIndex)
                                    + "_Structure_"
                                    + StructureEnum::toGuiName(structure));
                 CaretAssert((rowIndex >= 0) && (rowIndex < m_ciftiFile->getNumberOfRows()));
                 m_loadedRowData.resize(dataCount);
-                getProcessedDataForRow(&m_loadedRowData[0],
+                getProcessedDataForRow(m_loadedRowData,
                                        rowIndex);
                 
-                CaretLogFine("Read row for node " + AString::number(nodeIndex));
+                CaretLogFine("Read row for vertex " + AString::number(nodeIndex));
                 
                 m_connectivityDataLoaded->setSurfaceNodeLoading(structure,
                                                                 surfaceNumberOfNodes,
@@ -829,15 +851,15 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForSurfaceNode(const int32_t
             const int64_t dataCount = m_ciftiFile->getNumberOfRows();
             if (dataCount > 0) {
                 m_rowLoadedTextForMapName = ("Column: "
-                                             + AString::number(columnIndex)
-                                             + ", Node Index: "
+                                             + AString::number(columnIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI)
+                                             + ", Vertex Index: "
                                              + AString::number(nodeIndex)
                                              + ", Structure: "
                                              + StructureEnum::toName(structure));
                 
                 m_rowLoadedText = ("Column_"
-                                   + AString::number(columnIndex)
-                                   + "_Node_Index_"
+                                   + AString::number(columnIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI)
+                                   + "_Vertex_Index_"
                                    + AString::number(nodeIndex)
                                    + "_Structure_"
                                    + StructureEnum::toGuiName(structure));
@@ -846,7 +868,7 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForSurfaceNode(const int32_t
                 getProcessedDataForColumn(&m_loadedRowData[0],
                                           columnIndex);
                 
-                CaretLogFine("Read column for node " + AString::number(nodeIndex));
+                CaretLogFine("Read column for vertex " + AString::number(nodeIndex));
                 
                 m_connectivityDataLoaded->setSurfaceNodeLoading(structure,
                                                                 surfaceNumberOfNodes,
@@ -859,23 +881,23 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForSurfaceNode(const int32_t
         }
         
         if (dataWasLoaded == false) {
-            CaretLogFine("FAILED to read data for node " + AString::number(nodeIndex));
+            CaretLogFine("FAILED to read data for vertex " + AString::number(nodeIndex));
             m_connectivityDataLoaded->reset();
         }
     }
     catch (DataFileException& e) {
         m_connectivityDataLoaded->reset();
-        throw e;
+        throw DataFileException(getFileNameNoPath() + ": " + e.whatString());
     }
     
     updateForChangeInMapDataWithMapIndex(0);
 
-    AString msg = ("Time load data for surface node in "
+    AString msg = ("Time load data for surface vertex in "
                    + getFileNameNoPath()
                    + " was "
                    + AString::number(timer.getElapsedTimeSeconds())
                    + " seconds.");
-    CaretLogInfo(msg);
+    CaretLogFine(msg);
 }
 
 
@@ -898,15 +920,24 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForSurfaceNode(const int32_t
  */
 void
 CiftiMappableConnectivityMatrixDataFile::loadMapAverageDataForSurfaceNodes(const int32_t /*mapIndex*/,
-                                                                   const int32_t surfaceNumberOfNodes,
-                                                                   const StructureEnum::Enum structure,
-                                                                   const std::vector<int32_t>& nodeIndices)
+                                                                           const int32_t surfaceNumberOfNodes,
+                                                                           const StructureEnum::Enum structure,
+                                                                           const std::vector<int32_t>& nodeIndices)
 {
     if (m_ciftiFile == NULL) {
         setLoadedRowDataToAllZeros();
         return;
     }
     
+    /*
+     * If not enabled as a layer, clear any previous
+     * loaded data and do not load any new data
+     */
+    if ( ! isEnabledAsLayer()) {
+        setLoadedRowDataToAllZeros();
+        return;
+    }
+
     /*
      * Loading of data disabled?
      */
@@ -932,32 +963,33 @@ CiftiMappableConnectivityMatrixDataFile::loadMapAverageDataForSurfaceNodes(const
                                            nodeIndices,
                                            rowIndices,
                                            columnIndices);
-    if (rowIndices.empty()
-        && columnIndices.empty()) {
-        return;
+
+    int64_t dataCount = m_ciftiFile->getNumberOfColumns();
+    if ((getDataFileType() == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC)
+        || (getDataFileType() == DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC)) {
+        /*
+         * Dense dynamic is special case where number of rows equals number of brainordinates.
+         * Number of columns is number of time points
+         */
+        dataCount = m_ciftiFile->getNumberOfRows();
     }
+
+    m_loadedRowData.resize(dataCount);
     
-    std::vector<float> rowAverage, columnAverage;
-    getRowColumnAverageForIndices(rowIndices,
-                                  columnIndices,
-                                  rowAverage,
-                                  columnAverage);
+    std::vector<float> dataAverage(m_loadedRowData.size());
+    const bool dataWasLoadedFlag(getRowColumnAverageForIndices(rowIndices,
+                                                               columnIndices,
+                                                               dataAverage));
 
     /*
      * Update the viewed data
      */
-    bool dataWasLoaded = false;
-    if ( ! rowAverage.empty()) {
-        processRowAverageData(rowAverage);
-        m_loadedRowData = rowAverage;
-        dataWasLoaded = true;
+    if (dataWasLoadedFlag) {
+        processRowAverageData(dataAverage);
+        m_loadedRowData = dataAverage;
     }
-    else if ( ! columnAverage.empty()) {
-        m_loadedRowData = columnAverage;
-        dataWasLoaded = true;
-    }
-    
-    if (dataWasLoaded) {
+        
+    if (dataWasLoadedFlag) {
         m_rowLoadedTextForMapName = ("Structure: "
                                      + StructureEnum::toName(structure)
                                      + ", Averaged Vertex Count: "
@@ -968,13 +1000,13 @@ CiftiMappableConnectivityMatrixDataFile::loadMapAverageDataForSurfaceNodes(const
                             + AString::number(numberOfNodeIndices));
     }
     
-    if (dataWasLoaded == false) {
-        CaretLogFine("FAILED to read data for node average" + AString::fromNumbers(nodeIndices, ","));
+    if ( ! dataWasLoadedFlag) {
+        CaretLogFine("FAILED to read data for vertex average" + AString::fromNumbers(nodeIndices, ","));
     }
 
     updateForChangeInMapDataWithMapIndex(0);
 
-    if (dataWasLoaded) {
+    if (dataWasLoadedFlag) {
         m_connectivityDataLoaded->setSurfaceAverageNodeLoading(structure,
                                                                 surfaceNumberOfNodes,
                                                                 nodeIndices);
@@ -1012,6 +1044,15 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForVoxelAtCoordinate(const i
         return;
     }
     
+    /*
+     * If not enabled as a layer, clear any previous
+     * loaded data and do not load any new data
+     */
+    if ( ! isEnabledAsLayer()) {
+        setLoadedRowDataToAllZeros();
+        return;
+    }
+
     if (m_ciftiFile == NULL) {
         setLoadedRowDataToAllZeros();
         return;
@@ -1040,7 +1081,8 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForVoxelAtCoordinate(const i
     
     if (rowIndex >= 0) {
         int64_t dataCount = m_ciftiFile->getNumberOfColumns();
-        if (getDataFileType() == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC) {
+        if ((getDataFileType() == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC)
+            || (getDataFileType() == DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC)) {
             /*
              * Dense dynamic is special case where number of rows equals number of brainordinates.
              * Number of columns is number of time points
@@ -1050,17 +1092,17 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForVoxelAtCoordinate(const i
         if (dataCount > 0) {
             m_loadedRowData.resize(dataCount);
             CaretAssert((rowIndex >= 0) && (rowIndex < m_ciftiFile->getNumberOfRows()));
-            getProcessedDataForRow(&m_loadedRowData[0],
+            getProcessedDataForRow(m_loadedRowData,
                                    rowIndex);
             
             m_rowLoadedTextForMapName = ("Row: "
-                                        + AString::number(rowIndex)
+                                        + AString::number(rowIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI)
                                         + ", Voxel XYZ: ("
                                         + AString::fromNumbers(xyz, 3, ",")
                                         + ")");
             
             m_rowLoadedText = ("Row_"
-                                + AString::number(rowIndex)
+                                + AString::number(rowIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI)
                                 + "_Voxel_XYZ_"
                                 + AString::fromNumbers(xyz, 3, "_").replace('-', 'm'));
             
@@ -1079,13 +1121,13 @@ CiftiMappableConnectivityMatrixDataFile::loadMapDataForVoxelAtCoordinate(const i
                                       columnIndex);
             
             m_rowLoadedTextForMapName = ("Column: "
-                                         + AString::number(columnIndex)
+                                         + AString::number(columnIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI)
                                          + ", Voxel XYZ: ("
                                          + AString::fromNumbers(xyz, 3, ",")
                                          + ")");
             
             m_rowLoadedText = ("Column_"
-                               + AString::number(columnIndex)
+                               + AString::number(columnIndex + CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI)
                                + "_Voxel_XYZ_"
                                + AString::fromNumbers(xyz, 3, "_").replace('-', 'm'));
             
@@ -1190,6 +1232,15 @@ CiftiMappableConnectivityMatrixDataFile::loadMapAverageDataForVoxelIndices(const
         return false;
     }
     
+    /*
+     * If not enabled as a layer, clear any previous
+     * loaded data and do not load any new data
+     */
+    if ( ! isEnabledAsLayer()) {
+        setLoadedRowDataToAllZeros();
+        return false;
+    }
+
     /*
      * Loading of data disabled?
      */
@@ -1214,31 +1265,17 @@ CiftiMappableConnectivityMatrixDataFile::loadMapAverageDataForVoxelIndices(const
         return false;
     }
 
-    std::vector<float> rowAverage, columnAverage;
-    getRowColumnAverageForIndices(rowIndices,
-                                  columnIndices,
-                                  rowAverage,
-                                  columnAverage);
-    
-    bool dataWasLoadedFlag = false;
-    if ( ! rowAverage.empty()) {
-        processRowAverageData(rowAverage);
-        m_loadedRowData = rowAverage;
-        dataWasLoadedFlag = true;
+    std::vector<float> dataAverage(m_loadedRowData.size());
+    const bool dataWasLoadedFlag(getRowColumnAverageForIndices(rowIndices,
+                                                               columnIndices,
+                                                               dataAverage));
+
+    if (dataWasLoadedFlag) {
+        processRowAverageData(dataAverage);
+        m_loadedRowData = dataAverage;
     }
-    else if ( ! columnAverage.empty()) {
-        m_loadedRowData = columnAverage;
-        dataWasLoadedFlag = true;
-    }
-//    if (userCancelled) {
-//        m_loadedRowData.clear();
-//        m_loadedRowData.resize(dataCount, 0.0);
-//    }
+
     if (dataWasLoadedFlag) {
-//        progressEvent.setProgress(numberOfVoxelIndices - 1,
-//                                  "Averaging voxel data");
-//        EventManager::get()->sendEvent(progressEvent.getPointer());
-        
         const int32_t numberOfVoxelIndices = static_cast<int32_t>(voxelIndices.size());
         m_rowLoadedTextForMapName = ("Averaged Voxel Count: "
                                      + AString::number(numberOfVoxelIndices));
@@ -1281,6 +1318,37 @@ CiftiMappableConnectivityMatrixDataFile::getMapName(const int32_t /*mapIndex*/)
     return m_rowLoadedTextForMapName;
 }
 
+/**
+ * @return True if the number of rows and columns is identical
+ * and the row and column names are identical for the same index 
+ * (row "i" name is same as column "i" name for all rows/columns).
+ */
+bool
+CiftiMappableConnectivityMatrixDataFile::hasSymetricRowColumnNames() const
+{
+    const CiftiXML& xml = m_ciftiFile->getCiftiXML();
+    if ((xml.getMappingType(CiftiXML::ALONG_COLUMN) == CiftiMappingType::PARCELS)
+        && (xml.getMappingType(CiftiXML::ALONG_ROW) == CiftiMappingType::PARCELS)) {
+        const std::vector<CiftiParcelsMap::Parcel>& rowParcels    = xml.getParcelsMap(CiftiXML::ALONG_COLUMN).getParcels();
+        const std::vector<CiftiParcelsMap::Parcel>& columnParcels = xml.getParcelsMap(CiftiXML::ALONG_ROW).getParcels();
+        if (rowParcels.size() == columnParcels.size()) {
+            const int numRowsColumns = static_cast<int32_t>(rowParcels.size());
+            if (numRowsColumns > 0) {
+                for (int32_t i = 0; i < numRowsColumns; i++) {
+                    CaretAssertVectorIndex(rowParcels, i);
+                    CaretAssertVectorIndex(columnParcels, i);
+                    if (rowParcels[i].m_name != columnParcels[i].m_name) {
+                        return false;
+                    }
+                }
+                return true;
+            }
+        }
+    }
+    
+    return false;
+}
+
 AString
 CiftiMappableConnectivityMatrixDataFile::getRowName(const int32_t rowIndex) const
 {
@@ -1333,7 +1401,7 @@ CiftiMappableConnectivityMatrixDataFile::setChartMatrixLoadingDimension(const Ch
             m_dataMappingAccessMethod      = DATA_ACCESS_FILE_COLUMNS_OR_XML_ALONG_ROW;
             break;
     }
-    
+
     initializeAfterReading(getFileName());
     
     resetLoadedRowDataToEmpty();
@@ -1418,87 +1486,92 @@ CiftiMappableConnectivityMatrixDataFile::restoreFileDataFromScene(const SceneAtt
     
     setMapDataLoadingEnabled(mapIndex, true);
     
-    switch (m_connectivityDataLoaded->getMode()) {
-        case ConnectivityDataLoaded::MODE_NONE:
-            setLoadedRowDataToAllZeros();
-            break;
-        case ConnectivityDataLoaded::MODE_ROW:
-        {
-            int64_t rowIndex;
-            int64_t columnIndex;
-            m_connectivityDataLoaded->getRowColumnLoading(rowIndex,
-                                                          columnIndex);
-            loadDataForRowIndex(rowIndex);
+    try {
+        switch (m_connectivityDataLoaded->getMode()) {
+            case ConnectivityDataLoaded::MODE_NONE:
+                setLoadedRowDataToAllZeros();
+                break;
+            case ConnectivityDataLoaded::MODE_ROW:
+            {
+                int64_t rowIndex;
+                int64_t columnIndex;
+                m_connectivityDataLoaded->getRowColumnLoading(rowIndex,
+                                                              columnIndex);
+                loadDataForRowIndex(rowIndex);
+            }
+                break;
+            case ConnectivityDataLoaded::MODE_COLUMN:
+            {
+                int64_t rowIndex;
+                int64_t columnIndex;
+                m_connectivityDataLoaded->getRowColumnLoading(rowIndex,
+                                                              columnIndex);
+                loadDataForColumnIndex(columnIndex);
+            }
+                break;
+            case ConnectivityDataLoaded::MODE_SURFACE_NODE:
+            {
+                StructureEnum::Enum structure;
+                int32_t surfaceNumberOfNodes;
+                int32_t surfaceNodeIndex;
+                int64_t rowIndex;
+                int64_t columnIndex;
+                m_connectivityDataLoaded->getSurfaceNodeLoading(structure,
+                                                                surfaceNumberOfNodes,
+                                                                surfaceNodeIndex,
+                                                                rowIndex,
+                                                                columnIndex);
+                loadMapDataForSurfaceNode(mapIndex,
+                                          surfaceNumberOfNodes,
+                                          structure,
+                                          surfaceNodeIndex,
+                                          rowIndex,
+                                          columnIndex);
+            }
+                break;
+            case ConnectivityDataLoaded::MODE_SURFACE_NODE_AVERAGE:
+            {
+                StructureEnum::Enum structure;
+                int32_t surfaceNumberOfNodes;
+                std::vector<int32_t> surfaceNodeIndices;
+                m_connectivityDataLoaded->getSurfaceAverageNodeLoading(structure,
+                                                                       surfaceNumberOfNodes,
+                                                                       surfaceNodeIndices);
+                loadMapAverageDataForSurfaceNodes(mapIndex,
+                                                  surfaceNumberOfNodes,
+                                                  structure,
+                                                  surfaceNodeIndices);
+            }
+                break;
+            case ConnectivityDataLoaded::MODE_VOXEL_XYZ:
+            {
+                float volumeXYZ[3];
+                int64_t rowIndex;
+                int64_t columnIndex;
+                m_connectivityDataLoaded->getVolumeXYZLoading(volumeXYZ,
+                                                              rowIndex,
+                                                              columnIndex);
+                loadMapDataForVoxelAtCoordinate(mapIndex,
+                                                volumeXYZ,
+                                                rowIndex,
+                                                columnIndex);
+            }
+                break;
+            case ConnectivityDataLoaded::MODE_VOXEL_IJK_AVERAGE:
+            {
+                int64_t volumeDimensionsIJK[3];
+                std::vector<VoxelIJK> voxelIndicesIJK;
+                m_connectivityDataLoaded->getVolumeAverageVoxelLoading(volumeDimensionsIJK,
+                                                                       voxelIndicesIJK);
+                loadMapAverageDataForVoxelIndices(mapIndex,
+                                                  volumeDimensionsIJK,
+                                                  voxelIndicesIJK);
+            }
+                break;
         }
-            break;
-        case ConnectivityDataLoaded::MODE_COLUMN:
-        {
-            int64_t rowIndex;
-            int64_t columnIndex;
-            m_connectivityDataLoaded->getRowColumnLoading(rowIndex,
-                                                          columnIndex);
-            loadDataForColumnIndex(columnIndex);
-        }
-            break;
-        case ConnectivityDataLoaded::MODE_SURFACE_NODE:
-        {
-            StructureEnum::Enum structure;
-            int32_t surfaceNumberOfNodes;
-            int32_t surfaceNodeIndex;
-            int64_t rowIndex;
-            int64_t columnIndex;
-            m_connectivityDataLoaded->getSurfaceNodeLoading(structure,
-                                                            surfaceNumberOfNodes,
-                                                            surfaceNodeIndex,
-                                                            rowIndex,
-                                                            columnIndex);
-            loadMapDataForSurfaceNode(mapIndex,
-                                      surfaceNumberOfNodes,
-                                      structure,
-                                      surfaceNodeIndex,
-                                      rowIndex,
-                                      columnIndex);
-        }
-            break;
-        case ConnectivityDataLoaded::MODE_SURFACE_NODE_AVERAGE:
-        {
-            StructureEnum::Enum structure;
-            int32_t surfaceNumberOfNodes;
-            std::vector<int32_t> surfaceNodeIndices;
-            m_connectivityDataLoaded->getSurfaceAverageNodeLoading(structure,
-                                                                   surfaceNumberOfNodes,
-                                                                   surfaceNodeIndices);
-            loadMapAverageDataForSurfaceNodes(mapIndex,
-                                              surfaceNumberOfNodes,
-                                              structure,
-                                              surfaceNodeIndices);
-        }
-            break;
-        case ConnectivityDataLoaded::MODE_VOXEL_XYZ:
-        {
-            float volumeXYZ[3];
-            int64_t rowIndex;
-            int64_t columnIndex;
-            m_connectivityDataLoaded->getVolumeXYZLoading(volumeXYZ,
-                                                          rowIndex,
-                                                          columnIndex);
-            loadMapDataForVoxelAtCoordinate(mapIndex,
-                                            volumeXYZ,
-                                            rowIndex,
-                                            columnIndex);
-        }
-            break;
-        case ConnectivityDataLoaded::MODE_VOXEL_IJK_AVERAGE:
-        {
-            int64_t volumeDimensionsIJK[3];
-            std::vector<VoxelIJK> voxelIndicesIJK;
-            m_connectivityDataLoaded->getVolumeAverageVoxelLoading(volumeDimensionsIJK,
-                                                                   voxelIndicesIJK);
-            loadMapAverageDataForVoxelIndices(mapIndex,
-                                              volumeDimensionsIJK,
-                                              voxelIndicesIJK);
-        }
-            break;
+    }
+    catch (const DataFileException& dfe) {
+        sceneAttributes->addToErrorMessage(dfe.whatString());
     }
     
     setMapDataLoadingEnabled(mapIndex,
