@@ -24,11 +24,16 @@
 #include "CaretMappableDataFile.h"
 #include "CaretPointer.h"
 #include "CaretObjectTracksModification.h"
+#include "CaretUnitsTypeEnum.h"
+#include "ChartTwoMatrixTriangularViewingModeEnum.h"
 #include "CiftiMappingType.h"
 #include "CiftiXMLElements.h"
 #include "DisplayGroupEnum.h"
+#include "EventListenerInterface.h"
+#include "GroupAndNameHierarchyUserInterface.h"
 #include "VolumeMappableInterface.h"
 
+#include <memory>
 #include <set>
 
 namespace caret {
@@ -37,15 +42,22 @@ namespace caret {
     class ChartDataCartesian;
     class CiftiFile;
     class CiftiParcelsMap;
+    class CiftiScalarsMap;
     class CiftiXML;
     class FastStatistics;
+    class GraphicsPrimitive;
+    class GraphicsPrimitiveV3fC4f;
+    class GraphicsPrimitiveV3fT2f;
     class GroupAndNameHierarchyModel;
     class Histogram;
     class SparseVolumeIndexer;
+    class VolumeGraphicsPrimitiveManager;
 
     
     class CiftiMappableDataFile :
     public CaretMappableDataFile,
+    public EventListenerInterface,
+    public GroupAndNameHierarchyUserInterface,
     public VolumeMappableInterface {
         
     protected:
@@ -148,6 +160,8 @@ namespace caret {
                                                                      const int32_t numberOfNodes,
                                                                      AString& errorMessageOut);
         
+        virtual void receiveEvent(Event* event);
+        
         virtual void addToDataFileContentInformation(DataFileContentInformation& dataFileInformation);
         
         static void addCiftiXmlToDataFileContentInformation(DataFileContentInformation& dataFileInformation,
@@ -156,6 +170,9 @@ namespace caret {
         static void getDataFileContentInformationForGenericCiftiFile(const AString& filename,
                                                                      DataFileContentInformation& dataFileInformation);
         
+        static bool isMatrixTooLargeForOpenGL(const int64_t numberOfRows,
+                                              const int64_t numberOfColumns);
+        
         virtual void clear();
         
         virtual bool isEmpty() const;
@@ -197,6 +214,8 @@ namespace caret {
         
         virtual bool isMappedWithPalette() const;
         
+        virtual bool isOnePaletteUsedForAllMaps() const;
+        
         virtual const FastStatistics* getMapFastStatistics(const int32_t mapIndex);
         
         virtual const Histogram* getMapHistogram(const int32_t mapIndex);
@@ -208,7 +227,7 @@ namespace caret {
                                                  const float mostNegativeValueInclusive,
                                                  const bool includeZeroValues);
         
-        virtual void getPaletteNormalizationModesSupported(std::vector<PaletteNormalizationModeEnum::Enum>& modesSupportedOut);
+        virtual void getPaletteNormalizationModesSupported(std::vector<PaletteNormalizationModeEnum::Enum>& modesSupportedOut) const;
         
         virtual int64_t getDataSizeUncompressedInBytes() const;
         
@@ -232,16 +251,19 @@ namespace caret {
         
         const CiftiParcelsMap* getCiftiParcelsMapForDirection(const int direction) const;
         
+        const CiftiScalarsMap* getCiftiScalarsMapForDirection(const int direction) const;
+        
         virtual bool isMappedWithLabelTable() const;
         
         virtual GiftiLabelTable* getMapLabelTable(const int32_t mapIndex);
         
         virtual const GiftiLabelTable* getMapLabelTable(const int32_t mapIndex) const;
         
-        virtual void updateScalarColoringForAllMaps(const PaletteFile* paletteFile);
+        virtual const ClusterContainer* getMapLabelTableClusters(const int32_t mapIndex) const override;
         
-        virtual void updateScalarColoringForMap(const int32_t mapIndex,
-                                                const PaletteFile* paletteFile);
+        virtual void updateScalarColoringForAllMaps() override;
+        
+        virtual void updateScalarColoringForMap(const int32_t mapIndex) override;
         
         virtual bool isMapColoringValid(const int32_t mapIndex) const;
         
@@ -249,30 +271,58 @@ namespace caret {
                                    int64_t& dimOut2,
                                    int64_t& dimOut3,
                                    int64_t& dimTimeOut,
-                                   int64_t& numComponents) const;
+                                   int64_t& numComponents) const override;
         
-        virtual void getDimensions(std::vector<int64_t>& dimsOut) const;
+        virtual void getDimensions(std::vector<int64_t>& dimsOut) const override;
+        
+        virtual void getDimensionsForDataLoading(int64_t& dimOut1,
+                                                 int64_t& dimOut2,
+                                                 int64_t& dimOut3,
+                                                 int64_t& dimTimeOut,
+                                                 int64_t& numComponents) const;
+        
+        virtual void getDimensionsForDataLoading(std::vector<int64_t>& dimsOut) const;
         
         virtual void getMapDimensions(std::vector<int64_t> &dim) const;
 
-        virtual const int64_t& getNumberOfComponents() const;
+        /**
+         * @return Instance cast to a Volume Mappable CaretMappableDataFile
+         */
+        virtual CaretMappableDataFile* castToVolumeMappableDataFile() override;
+        
+        /**
+         * @return Instance cast to a Volume Mappable CaretMappableDataFile (const method)
+         */
+        virtual const CaretMappableDataFile* castToVolumeMappableDataFile() const override;
+
+        virtual const int64_t& getNumberOfComponents() const override;
         
         virtual void indexToSpace(const float& indexIn1,
                                   const float& indexIn2,
                                   const float& indexIn3,
                                   float& coordOut1,
                                   float& coordOut2,
-                                  float& coordOut3) const;
+                                  float& coordOut3) const override;
         
         virtual void indexToSpace(const float& indexIn1,
                                   const float& indexIn2,
                                   const float& indexIn3,
-                                  float* coordOut) const;
+                                  float* coordOut) const override;
         
         virtual void indexToSpace(const int64_t* indexIn,
-                                  float* coordOut) const;
+                                  float* coordOut) const override;
         
         virtual void enclosingVoxel(const float& coordIn1,
+                                    const float& coordIn2,
+                                    const float& coordIn3,
+                                    int64_t& indexOut1,
+                                    int64_t& indexOut2,
+                                    int64_t& indexOut3) const override;
+        
+        virtual void enclosingVoxel(const float* coordIn,
+                                    int64_t* indexOut) const override;
+        
+        virtual void enclosingVoxelForDataLoading(const float& coordIn1,
                                     const float& coordIn2,
                                     const float& coordIn3,
                                     int64_t& indexOut1,
@@ -280,51 +330,96 @@ namespace caret {
                                     int64_t& indexOut3) const;
         
         virtual bool indexValid(const int64_t& indexIn1,
+                                const int64_t& indexIn2,
+                                const int64_t& indexIn3,
+                                const int64_t brickIndex = 0,
+                                const int64_t component = 0) const override;
+        
+        virtual bool indexValid(const int64_t* indexIn,
+                                const int64_t brickIndex = 0,
+                                const int64_t component = 0) const override;
+        
+        virtual bool indexValidForDataLoading(const int64_t& indexIn1,
                                 const int64_t& indexIn2,
                                 const int64_t& indexIn3,
                                 const int64_t brickIndex = 0,
                                 const int64_t component = 0) const;
         
-        virtual void getVoxelSpaceBoundingBox(BoundingBox& boundingBoxOut) const;
+        virtual const VolumeSpace& getVolumeSpace() const;
         
-        virtual int64_t getVoxelColorsForSliceInMap(const PaletteFile* paletteFile,
-                                                 const int32_t mapIndex,
+        virtual void getVoxelSpaceBoundingBox(BoundingBox& boundingBoxOut) const override;
+        
+        virtual void getNonZeroVoxelCoordinateBoundingBox(const int32_t mapIndex,
+                                                   BoundingBox& boundingBoxOut) const override;
+
+        virtual int64_t getVoxelColorsForSliceInMap(const int32_t mapIndex,
                                                  const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                                  const int64_t sliceIndex,
-                                                 const DisplayGroupEnum::Enum displayGroup,
-                                                 const int32_t tabIndex,
-                                                 uint8_t* rgbaOut) const;
+                                                 const TabDrawingInfo& tabDrawingInfo,
+                                                    uint8_t* rgbaOut) const override;
         
-        virtual int64_t getVoxelColorsForSubSliceInMap(const PaletteFile* paletteFile,
-                                                    const int32_t mapIndex,
+        int64_t getVoxelColorsForSliceInMap(const int32_t mapIndex,
+                                            const int64_t firstVoxelIJK[3],
+                                            const int64_t rowStepIJK[3],
+                                            const int64_t columnStepIJK[3],
+                                            const int64_t numberOfRows,
+                                            const int64_t numberOfColumns,
+                                            const TabDrawingInfo& tabDrawingInfo,
+                                            uint8_t* rgbaOut) const override;
+        
+        virtual int64_t getVoxelColorsForSubSliceInMap(const int32_t mapIndex,
                                                     const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                                     const int64_t sliceIndex,
                                                     const int64_t firstCornerVoxelIndex[3],
                                                     const int64_t lastCornerVoxelIndex[3],
                                                     const int64_t voxelCountIJK[3],
-                                                    const DisplayGroupEnum::Enum displayGroup,
-                                                    const int32_t tabIndex,
-                                                    uint8_t* rgbaOut) const;
+                                                       const TabDrawingInfo& tabDrawingInfo,
+                                                    uint8_t* rgbaOut) const override;
         
-        virtual void getVoxelColorInMap(const PaletteFile* paletteFile,
+        virtual void getVoxelColorInMap(const int64_t indexIn1,
+                                        const int64_t indexIn2,
+                                        const int64_t indexIn3,
+                                        const int64_t mapIndex,
+                                        const TabDrawingInfo& tabDrawingInfo,
+                                        uint8_t rgbaOut[4]) const override;
+        
+        virtual void getVoxelColorInMap(const int64_t indexIn1,
+                                        const int64_t indexIn2,
+                                        const int64_t indexIn3,
+                                        const int64_t mapIndex,
+                                        uint8_t rgbaOut[4]) const override;
+        
+        virtual void getVoxelColorInMapForLabelData(const std::vector<float>& dataForMap,
                                         const int64_t indexIn1,
                                         const int64_t indexIn2,
                                         const int64_t indexIn3,
                                         const int64_t mapIndex,
-                                        const DisplayGroupEnum::Enum displayGroup,
-                                        const int32_t tabIndex,
+                                        const TabDrawingInfo& tabDrawingInfo,
                                         uint8_t rgbaOut[4]) const;
         
-        virtual void getVoxelColorInMapForLabelData(const PaletteFile* paletteFile,
-                                                    const std::vector<float>& dataForMap,
-                                        const int64_t indexIn1,
-                                        const int64_t indexIn2,
-                                        const int64_t indexIn3,
-                                        const int64_t mapIndex,
-                                        const DisplayGroupEnum::Enum displayGroup,
-                                        const int32_t tabIndex,
-                                        uint8_t rgbaOut[4]) const;
+        virtual GraphicsPrimitiveV3fT3f* getVolumeDrawingTriangleStripPrimitive(const int32_t mapIndex,
+                                                                                const TabDrawingInfo& tabDrawingInfo) const override;
         
+        virtual GraphicsPrimitiveV3fT3f* getVolumeDrawingTriangleFanPrimitive(const int32_t mapIndex,
+                                                                              const TabDrawingInfo& tabDrawingInfo) const override;
+        
+        virtual GraphicsPrimitiveV3fT3f* getVolumeDrawingTrianglesPrimitive(const int32_t mapIndex,
+                                                                            const TabDrawingInfo& tabDrawingInfo) const override;
+
+        virtual GraphicsPrimitive* getHistologyImageIntersectionPrimitive(const int32_t mapIndex,
+                                                                          const TabDrawingInfo& tabDrawingInfo,
+                                                                          const MediaFile* mediaFile,
+                                                                          const VolumeToImageMappingModeEnum::Enum volumeMappingMode,
+                                                                          const float volumeSliceThickness,
+                                                                          AString& errorMessageOut) const override;
+        
+        virtual std::vector<GraphicsPrimitive*> getHistologySliceIntersectionPrimitive(const int32_t mapIndex,
+                                                                                       const TabDrawingInfo& tabDrawingInfo,
+                                                                                       const HistologySlice* histologySlice,
+                                                                                       const VolumeToImageMappingModeEnum::Enum volumeMappingMode,
+                                                                                       const float volumeSliceThickness,
+                                                                                       AString& errorMessageOut) const override;
+
         virtual bool getMapVolumeVoxelValue(const int32_t mapIndex,
                                             const float xyz[3],
                                             int64_t ijkOut[3],
@@ -334,6 +429,8 @@ namespace caret {
         
         virtual bool getMapVolumeVoxelValues(const std::vector<int32_t> mapIndices,
                                              const float xyz[3],
+                                             const AString& dataValueSeparator,
+                                             const int32_t digitsRightOfDecimal,
                                              int64_t ijkOut[3],
                                              std::vector<float>& numericalValuesOut,
                                              std::vector<bool>& numericalValuesOutValid,
@@ -345,19 +442,21 @@ namespace caret {
         virtual float getVoxelValue(const float* coordinateIn,
                                     bool* validOut = NULL,
                                     const int64_t mapIndex = 0,
-                                    const int64_t component = 0) const;
+                                    const int64_t component = 0) const override;
         
         virtual float getVoxelValue(const float coordinateX,
                                     const float coordinateY,
                                     const float coordinateZ,
                                     bool* validOut = NULL,
                                     const int64_t mapIndex = 0,
-                                    const int64_t component = 0) const;
+                                    const int64_t component = 0) const override;
         
         virtual bool getVolumeVoxelIdentificationForMaps(const std::vector<int32_t>& mapIndices,
                                                          const float xyz[3],
+                                                         const AString& dataValueSeparator,
+                                                         const int32_t digitsRightOfDecimal,
                                                          int64_t ijkOut[3],
-                                                         AString& textOut) const;
+                                                         AString& textOut) const override;
         
         std::vector<int32_t> getUniqueLabelKeysUsedInMap(const int32_t mapIndex) const;
         
@@ -377,16 +476,20 @@ namespace caret {
                                              const StructureEnum::Enum structure,
                                              const int nodeIndex,
                                              const int32_t numberOfNodes,
+                                             const AString& dataValueSeparator,
+                                             const int32_t digitsRightOfDecimal,
                                              std::vector<float>& numericalValuesOut,
                                              std::vector<bool>& numericalValuesOutValid,
                                              AString& textValueOut) const;
         
         virtual bool getSurfaceNodeIdentificationForMaps(const std::vector<int32_t>& mapIndices,
-                                                            const StructureEnum::Enum structure,
-                                                            const int nodeIndex,
-                                                            const int32_t numberOfNodes,
-                                                            AString& textOut) const;
-        
+                                                         const StructureEnum::Enum structure,
+                                                         const int nodeIndex,
+                                                         const int32_t numberOfNodes,
+                                                         const AString& dataValueSeparator,
+                                                         const int32_t digitsRightOfDecimal,
+                                                         AString& textOut) const override;
+
         int32_t getMappingSurfaceNumberOfNodes(const StructureEnum::Enum structure) const;
         
         bool getSeriesDataForSurfaceNode(const StructureEnum::Enum structure,
@@ -396,8 +499,7 @@ namespace caret {
         bool getSeriesDataForVoxelAtCoordinate(const float xyz[3],
                                                std::vector<float>& seriesDataOut) const;
         
-        virtual bool getMapSurfaceNodeColoring(const PaletteFile* paletteFile,
-                                               const int32_t mapIndex,
+        virtual bool getMapSurfaceNodeColoring(const int32_t mapIndex,
                                                const StructureEnum::Enum structure,
                                                float* surfaceRGBAOut,
                                                float* dataValuesOut,
@@ -409,6 +511,11 @@ namespace caret {
         
         virtual NiftiTimeUnitsEnum::Enum getMapIntervalUnits() const;
         
+        void getDimensionUnits(const int32_t dimensionIndex,
+                               CaretUnitsTypeEnum::Enum& unitsOut,
+                               float& startValueOut,
+                               float& stepValueOut) const;
+        
         virtual void getMapIntervalStartAndStep(float& firstMapUnitsValueOut,
                                                 float& mapIntervalStepValueOut) const;
         
@@ -430,6 +537,16 @@ namespace caret {
         
         void invalidateColoringInAllMaps();
         
+        bool getRowColumnIndexFromVolumeXYZ(const float xyz[3],
+                                            int64_t& rowIndexOut,
+                                            int64_t& columnIndexOut) const;
+        
+        bool getRowColumnIndexFromSurfaceVertex(const StructureEnum::Enum structure,
+                                                const int64_t surfaceNumberOfVertices,
+                                                const int64_t vertexIndex,
+                                                int64_t& rowIndexOut,
+                                                int64_t& columnIndexOut) const;
+        
         void getBrainordinateFromRowIndex(const int64_t rowIndex,
                                           StructureEnum::Enum& surfaceStructureOut,
                                           int32_t& surfaceNodeIndexOut,
@@ -438,6 +555,25 @@ namespace caret {
                                           int64_t voxelIJKOut[3],
                                           float voxelXYZOut[3],
                                           bool& voxelValidOut) const;
+                                          
+        bool hasCiftiXML() const;
+        
+        const CiftiXML getCiftiXML() const;
+        
+        /** 
+         * @return The index base (zero or one) for displaying row and column indices in the Graphical User Interface 
+         */
+        static inline int32_t getCiftiFileRowColumnIndexBaseForGUI() { return CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI; }
+
+        virtual void getDataForSelector(const MapFileDataSelector& mapFileDataSelector,
+                                        std::vector<float>& dataOut) const override;
+        
+        virtual BrainordinateMappingMatch getBrainordinateMappingMatchImplementation(const CaretMappableDataFile* mapFile) const override;
+        
+        virtual void groupAndNameHierarchyItemStatusChanged() override;
+
+        void printMappingsForDebugging(const AString& filename,
+                                       const AString& message);
         
     private:
         
@@ -446,6 +582,11 @@ namespace caret {
         CiftiMappableDataFile& operator=(const CiftiMappableDataFile&);
         
     public:
+        enum class MatrixGridMode {
+            FILLED_TRIANGLES,
+            FILLED_TEXTURE,
+            OUTLINE
+        };
         
         virtual void getMapData(const int32_t mapIndex,
                                 std::vector<float>& dataOut) const;
@@ -453,8 +594,19 @@ namespace caret {
         virtual void setMapData(const int32_t mapIndex,
                                 const std::vector<float>& data);
         
-        virtual void getMatrixRGBA(std::vector<float>& rgba, PaletteFile *paletteFile);
+        virtual void getMatrixRGBA(std::vector<float>& rgba);
     
+        bool getMatrixForChartingRGBA(int32_t& numberOfRowsOut,
+                                      int32_t& numberOfColumnsOut,
+                                      std::vector<float>& rgbaOut) const;
+        
+        GraphicsPrimitive* getMatrixChartingGraphicsPrimitive(const ChartTwoMatrixTriangularViewingModeEnum::Enum matrixViewMode,
+                                                              const MatrixGridMode gridMode,
+                                                              const float opacity) const;
+        
+        /** Identifier for the matrix primitives alternative color used for the grid coloring */
+        int32_t getMatrixChartGraphicsPrimitiveGridColorIdentifier() const { return 1; }
+        
         virtual void getFileData(std::vector<float>& data) const;
         
         const CiftiFile* getCiftiFile() const { return m_ciftiFile; }
@@ -491,21 +643,13 @@ namespace caret {
                                                    const std::vector<int32_t>& rowIndicesIn,
                                                    std::vector<float>& rgbaOut) const;
         
-//        bool helpLoadChartDataMatrixRGBA(int32_t& numberOfRowsOut,
-//                                         int32_t& numberOfColumnsOut,
-//                                         std::vector<float>& rgbaOut) const;
-//
-//        bool helpLoadChartDataMatrixRGBAWithRowIndicese(int32_t& numberOfRowsOut,
-//                                                        int32_t& numberOfColumnsOut,
-//                                                        const std::vector<int32_t>& rowIndices,
-//                                                        std::vector<float>& rgbaOut) const;
-        
     private:
         class MapContent : public CaretObjectTracksModification {
             
         public:
             
-            MapContent(CiftiFile* ciftiFile,
+            MapContent(CiftiMappableDataFile* ciftiMappableDataFile,
+                       CiftiFile* ciftiFile,
                        const FileMapDataType fileMapDataType,
                        const int32_t readingDirectionForCiftiXML,
                        const int32_t mappingDirectionForCiftiXML,
@@ -520,24 +664,26 @@ namespace caret {
             void updateForChangeInMapData();
             
             void updateColoring(const std::vector<float>& data,
-                                const PaletteFile* paletteFile,
                                 const FastStatistics* fastStatistics);
             
             bool isFastStatisticsValid() const;
             
             void updateFastStatistics(const std::vector<float>& data);
             
-            bool isHistogramValid() const;
+            bool isHistogramValid(const int32_t numberOfBuckets) const;
             
-            void updateHistogram(const std::vector<float>& data);
+            void updateHistogram(const int32_t numberOfBuckets,
+                                 const std::vector<float>& data);
             
-            bool isHistogramLimitedValuesValid(const float mostPositiveValueInclusive,
+            bool isHistogramLimitedValuesValid(const int32_t numberOfBuckets,
+                                               const float mostPositiveValueInclusive,
                                                const float leastPositiveValueInclusive,
                                                const float leastNegativeValueInclusive,
                                                const float mostNegativeValueInclusive,
                                                const bool includeZeroValues) const;
             
-            void updateHistogramLimitedValues(const std::vector<float>& data,
+            void updateHistogramLimitedValues(const int32_t numberOfBuckets,
+                                              const std::vector<float>& data,
                                               const float mostPositiveValueInclusive,
                                               const float leastPositiveValueInclusive,
                                               const float leastNegativeValueInclusive,
@@ -548,6 +694,13 @@ namespace caret {
             
             void setName(const AString& name);
             
+            bool getThresholdData(const CaretMappableDataFile* threshMapFile,
+                                  const int32_t threshMapIndex,
+                                  std::vector<float>& thresholdData) const;
+            
+            /** CIFTI file containing the map */
+            CiftiMappableDataFile* m_ciftiMappableDataFile;
+            
             /** The CIFTI file pointer */
             CiftiFile *m_ciftiFile;
             
@@ -593,15 +746,21 @@ namespace caret {
             /** histogram for all of map map */
             CaretPointer<Histogram> m_histogram;
         
+            int32_t m_histogramNumberOfBuckets = 100;
+            
             /** histogram for limited values from map */
             CaretPointer<Histogram> m_histogramLimitedValues;
             
+            int32_t m_histogramLimitedValuesNumberOfBuckets;
             float m_histogramLimitedValuesMostPositiveValueInclusive;
             float m_histogramLimitedValuesLeastPositiveValueInclusive;
             float m_histogramLimitedValuesLeastNegativeValueInclusive;
             float m_histogramLimitedValuesMostNegativeValueInclusive;
             bool m_histogramLimitedValuesIncludeZeroValues;
             
+            /** Have bounding box for  map */
+            mutable std::unique_ptr<BoundingBox> m_nonZeroVoxelCoordinateBoundingBox;
+            
         private:
             /** Name of map */
             AString m_name;
@@ -640,6 +799,18 @@ namespace caret {
         
         static AString mappingTypeToName(const CiftiMappingType::MappingType mappingType);
 
+        const CiftiBrainModelsMap* getBrainordinateMapping() const;
+        
+        const CiftiParcelsMap* getParcelsMapping() const;
+        
+        GraphicsPrimitiveV3fT2f* createMatrixPrimitive(std::vector<uint8_t>& matrixRGBA,
+                                                       const int64_t numberOfColumns,
+                                                       const int64_t numberOfRows,
+                                                       const float xLeft,
+                                                       const float xRight,
+                                                       const float yBottom,
+                                                       const float yTop) const;
+
         /**
          * Point to the CIFTI file object.
          */
@@ -707,9 +878,27 @@ namespace caret {
         /** Histogram used when statistics computed on all data in file */
         CaretPointer<Histogram> m_fileHistogram;
         
+        std::unique_ptr<VolumeGraphicsPrimitiveManager> m_graphicsPrimitiveManager;
+        
+        /** Primitive for matrix cells drawn with triangles*/
+        mutable std::unique_ptr<GraphicsPrimitiveV3fC4f> m_matrixGraphicsTrianglesPrimitive;
+        
+        /** Primitive for matrix cells drawn using a texture */
+        mutable std::unique_ptr<GraphicsPrimitiveV3fT2f> m_matrixGraphicsTexturePrimitive;
+        
+        /** Primitive for grid outline around matrix cells */
+        mutable std::unique_ptr<GraphicsPrimitiveV3fC4f> m_matrixGraphicsOutlinePrimitive;
+        
+        mutable uint8_t m_previousMatrixGridRGBA[4] = { 0, 1, 2, 3 };
+        
+        mutable float m_previousMatrixOpacity = -1.0;
+        
+        int32_t m_fileHistogramNumberOfBuckets = 100;
+        
         /** Histogram with limited values used when statistics computed on all data in file */
         CaretPointer<Histogram> m_fileHistorgramLimitedValues;
         
+        int32_t m_fileHistogramLimitedValuesNumberOfBuckets = 100;
         float m_fileHistogramLimitedValuesMostPositiveValueInclusive;
         float m_fileHistogramLimitedValuesLeastPositiveValueInclusive;
         float m_fileHistogramLimitedValuesLeastNegativeValueInclusive;
@@ -717,7 +906,8 @@ namespace caret {
         bool m_fileHistogramLimitedValuesIncludeZeroValues;
         
         /** Fast conversion of IJK to data offset */
-        CaretPointer<SparseVolumeIndexer> m_voxelIndicesToOffset;
+        CaretPointer<SparseVolumeIndexer> m_voxelIndicesToOffsetForDataMapping;
+        CaretPointer<SparseVolumeIndexer> m_voxelIndicesToOffsetForDataReading;
         
         /** Holds class and name hierarchy used for display selection */
         mutable CaretPointer<GroupAndNameHierarchyModel> m_classNameHierarchy;
@@ -725,10 +915,37 @@ namespace caret {
         /** force an update of the class and name hierarchy */
         mutable bool m_forceUpdateOfGroupAndNameHierarchy;
 
-        
         static const int32_t S_CIFTI_XML_ALONG_INVALID;
         
-//        std::vector<int64_t> m_ciftiDimensions;
+    protected:
+        /**
+         * This value is used for the "base index" of CIFTI rows/columns
+         * in the user-interface.
+         */
+        static const int32_t CIFTI_FILE_ROW_COLUMN_INDEX_BASE_FOR_GUI = 1;
+        
+    private:
+        friend class ChartableTwoFileDelegate;
+        friend class ChartableTwoFileMatrixChart;
+        
+        /** Is lazily initialized and caches CiftiBrainModelsMap for comparison with other CIFTI files */
+        mutable std::unique_ptr<CiftiBrainModelsMap> m_brainordinateMapping;
+        
+        /** Controls lazy initialization of m_brainordinateMapping */
+        mutable bool m_brainordinateMappingCachedFlag = false;
+        
+        /** Is lazily initialized and caches CiftiParcelsMap for comparison with other CIFTI files */
+        mutable std::unique_ptr<CiftiParcelsMap> m_parcelsMapping;
+        
+        /** Controls lazy initialization of m_parcelsMapping */
+        mutable bool m_parcelsMappingCachedFlag = false;
+        
+        /** Prevents logging 'too large' message more than once for a file */
+        mutable bool m_matrixDimensionsTooLargeLoggedFlag = false;
+        
+        bool m_blockInvalidateColorsInAllMapsFlag = false;
+        
+        mutable std::map<int32_t, std::unique_ptr<ClusterContainer>> m_mapLabelClusterContainers;
         
         // ADD_NEW_MEMBERS_HERE
         
