@@ -56,8 +56,10 @@ m_parentCiftiMappableDataFile(parentCiftiMappableDataFile)
     
     switch (parentCiftiMappableDataFile->getDataFileType()) {
         case DataFileTypeEnum::CONNECTIVITY_PARCEL:
-        case DataFileTypeEnum::CONNECTIVITY_PARCEL_SCALAR:
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC:
         case DataFileTypeEnum::CONNECTIVITY_PARCEL_LABEL:
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL_SCALAR:
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL_SERIES:
             break;
         default:
             CaretAssert(0);
@@ -81,6 +83,15 @@ CiftiParcelReorderingModel::~CiftiParcelReorderingModel()
 {
     delete m_sceneAssistant;
     
+    clearCiftiParcelReordering();
+}
+
+/**
+ * Clear all parcel reordering.
+ */
+void
+CiftiParcelReorderingModel::clearCiftiParcelReordering()
+{
     for (std::vector<CiftiParcelReordering*>::iterator iter = m_parcelReordering.begin();
          iter != m_parcelReordering.end();
          iter++) {
@@ -89,6 +100,7 @@ CiftiParcelReorderingModel::~CiftiParcelReorderingModel()
     m_parcelReordering.clear();
 }
 
+
 /**
  * Copy constructor.
  * @param obj
@@ -191,11 +203,15 @@ CiftiParcelReorderingModel::getParcelLabelFiles() const
                         testAlongRow    = true;
                         break;
                     case DataFileTypeEnum::CONNECTIVITY_PARCEL_SCALAR:
+                    case DataFileTypeEnum::CONNECTIVITY_PARCEL_SERIES:
                         testAlongColumn = true;
                         break;
                     case DataFileTypeEnum::CONNECTIVITY_PARCEL_LABEL:
                         testAlongColumn = true;
                         break;
+                    case DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC:
+                        testAlongColumn = true;
+                        break;
                     default:
                         CaretAssert(0);
                 }
@@ -513,7 +529,7 @@ CiftiParcelReorderingModel::restoreFromScene(const SceneAttributes* sceneAttribu
                                      sceneClass);    
     
     m_selectedParcelLabelFile = NULL;
-    m_parcelReordering.clear();
+    clearCiftiParcelReordering();
     
     const AString parcelLabelFileName = sceneClass->getPathNameValue("m_selectedParcelLabelFile");
     if ( ! parcelLabelFileName.isEmpty()) {
