@@ -26,15 +26,18 @@
 #include "CaretLogger.h"
 #include "ChartDataCartesian.h"
 #include "ChartMatrixDisplayProperties.h"
+#include "CiftiConnectivityMatrixParcelDynamicFile.h"
+#include "CiftiConnectivityMatrixParcelFile.h"
 #include "CiftiFile.h"
 #include "CiftiParcelReordering.h"
 #include "CiftiParcelReorderingModel.h"
 #include "CiftiXML.h"
+#include "DataFileException.h"
 #include "FastStatistics.h"
+#include "FileInformation.h"
 #include "NodeAndVoxelColoring.h"
 #include "Palette.h"
 #include "PaletteColorMapping.h"
-#include "PaletteFile.h"
 #include "SceneClass.h"
 #include "SceneClassArray.h"
 #include "SceneClassAssistant.h"
@@ -50,6 +53,143 @@ using namespace caret;
  *
  */
 
+/**
+ * Create a Cifti Parcel Scalar File using the currently loaded row in a Cifti matrix file
+ *
+ * @param sourceCiftiMatrixFile
+ *    The connectivity matrix file.
+ * @param destinationDirectory
+ *    Directory in which file is placed if the input matrix file is not
+ *    in a valid local (user's file system) directory.
+ * @param errorMessageOut
+ *    Will describe problem if there is an error.
+ * @return
+ *    Pointer to the newly created Cifti Parcel Scalar File.  If there is an error,
+ *    NULL will be returned and errorMessageOut will describe the problem.
+ */
+
+CiftiParcelScalarFile*
+CiftiParcelScalarFile::newInstanceFromRowInCiftiConnectivityMatrixFile(const CiftiMappableConnectivityMatrixDataFile* sourceCiftiMatrixFile,
+                                                                       const AString& destinationDirectory,
+                                                                       AString& errorMessageOut)
+{
+    CaretAssert(sourceCiftiMatrixFile);
+    errorMessageOut.clear();
+    
+    const CiftiConnectivityMatrixParcelDynamicFile* parcelDynamicFile =
+        dynamic_cast<const CiftiConnectivityMatrixParcelDynamicFile*>(sourceCiftiMatrixFile);
+    const CiftiConnectivityMatrixParcelFile* parcelMatrixFile =
+       dynamic_cast<const CiftiConnectivityMatrixParcelFile*>(sourceCiftiMatrixFile);
+    if ((parcelDynamicFile != NULL)
+        || (parcelMatrixFile != NULL)) {
+        /* ok, acceptable file type */
+    }
+    else {
+        errorMessageOut = "Only Cifti Parcel Matrix and Parcel Dynamic Files are supported for conversion to Cifti Scalar Files.";
+        return NULL;
+    }
+    const CiftiFile* sourceCiftiFile = sourceCiftiMatrixFile->m_ciftiFile;
+    
+    if (sourceCiftiMatrixFile->getNumberOfMaps() <= 0) {
+        errorMessageOut = "No data appears to be loaded in the Cifti Matrix File (No Maps).";
+        return NULL;
+    }
+    
+    std::vector<float> data;
+    sourceCiftiMatrixFile->getMapData(0, data);
+    if (data.empty()) {
+        errorMessageOut = "No data appears to be loaded in the Cifti Matrix File (mapData empty).";
+        return NULL;
+    }
+    
+    CiftiParcelScalarFile* parcelScalarFile(NULL);
+
+    try {
+        CiftiFile* ciftiFile = new CiftiFile();
+        
+        /*
+         * Copy XML from matrix file
+         * and update to be a scalar file.
+         */
+        const CiftiXML& sourceCiftiMatrixXML = sourceCiftiFile->getCiftiXML();
+        CiftiXML ciftiScalarXML = sourceCiftiMatrixXML;
+        
+        CaretAssert(sourceCiftiMatrixXML.getMappingType(CiftiXML::ALONG_COLUMN) == CiftiMappingType::PARCELS);
+        CiftiParcelsMap parcelsMap = sourceCiftiMatrixXML.getParcelsMap(CiftiXML::ALONG_COLUMN);
+
+        CiftiScalarsMap scalarsMap;
+        scalarsMap.setLength(1);
+        scalarsMap.setMapName(0,
+                              sourceCiftiMatrixFile->getMapName(0));
+        ciftiScalarXML.setMap(CiftiXML::ALONG_ROW,
+                              scalarsMap);
+        ciftiScalarXML.setMap(CiftiXML::ALONG_COLUMN,
+                              parcelsMap);
+        ciftiFile->setCiftiXML(ciftiScalarXML);
+        
+        /*
+         * Add data to the file
+         */
+        ciftiFile->setColumn(&data[0],
+                             0);
+        
+        /*
+         * Create a scalar file
+         */
+        parcelScalarFile = new CiftiParcelScalarFile();
+        parcelScalarFile->m_ciftiFile.grabNew(ciftiFile);
+        
+        
+        
+        /*
+         * May need to convert a remote path to a local path
+         */
+        FileInformation initialFileNameInfo(sourceCiftiMatrixFile->getFileName());
+        const AString scalarFileName = initialFileNameInfo.getAsLocalAbsoluteFilePath(destinationDirectory,
+                                                                                      parcelScalarFile->getDataFileType());
+        
+        /*
+         * Create name of scalar file with row/column information
+         */
+        FileInformation scalarFileInfo(scalarFileName);
+        AString thePath, theName, theExtension;
+        scalarFileInfo.getFileComponents(thePath,
+                                         theName,
+                                         theExtension);
+        theName.append("_" + sourceCiftiMatrixFile->getRowLoadedText());
+        const AString newFileName = FileInformation::assembleFileComponents(thePath,
+                                                                            theName,
+                                                                            theExtension);
+        parcelScalarFile->setFileName(newFileName);
+        
+        parcelScalarFile->initializeAfterReading(newFileName);
+        
+        /*
+         * Need to copy color palette since it may be the default
+         */
+        PaletteColorMapping* scalarPalette = parcelScalarFile->getMapPaletteColorMapping(0);
+        CaretAssert(scalarPalette);
+        const PaletteColorMapping* densePalette = sourceCiftiMatrixFile->getMapPaletteColorMapping(0);
+        CaretAssert(densePalette);
+        scalarPalette->copy(*densePalette,
+                            true);
+        
+        parcelScalarFile->updateAfterFileDataChanges();
+        parcelScalarFile->setModified();
+        
+        return parcelScalarFile;
+    }
+    catch (const DataFileException& de) {
+        if (parcelScalarFile != NULL) {
+            delete parcelScalarFile;
+            parcelScalarFile = NULL;
+        }
+        errorMessageOut = de.whatString();
+    }
+    
+    return NULL;
+}
+
 /**
  * Constructor.
  */
@@ -124,7 +264,7 @@ CiftiParcelScalarFile::isLineSeriesChartingSupported() const
  *    Chart types supported by this file.
  */
 void
-CiftiParcelScalarFile::getSupportedLineSeriesChartDataTypes(std::vector<ChartDataTypeEnum::Enum>& chartDataTypesOut) const
+CiftiParcelScalarFile::getSupportedLineSeriesChartDataTypes(std::vector<ChartOneDataTypeEnum::Enum>& chartDataTypesOut) const
 {
     helpGetSupportedLineSeriesChartDataTypes(chartDataTypesOut);
 }
@@ -164,41 +304,6 @@ CiftiParcelScalarFile::loadLineSeriesChartDataForSurfaceNode(const StructureEnum
     ChartDataCartesian* chartData = helpLoadChartDataForSurfaceNode(structure,
                                                            nodeIndex);
     return chartData;
-    
-//    ChartDataCartesian* chartData = NULL;
-//    
-//    try {
-//        std::vector<float> data;
-//        if (getSeriesDataForSurfaceNode(structure,
-//                                        nodeIndex,
-//                                        data)) {
-//            const int64_t numData = static_cast<int64_t>(data.size());
-//            
-//            chartData = new ChartDataCartesian(ChartDataTypeEnum::CHART_DATA_TYPE_LINE_DATA_SERIES,
-//                                               ChartAxisUnitsEnum::CHART_AXIS_UNITS_NONE,
-//                                               ChartAxisUnitsEnum::CHART_AXIS_UNITS_NONE);
-//            for (int64_t i = 0; i < numData; i++) {
-//                float xValue = i;
-//                chartData->addPoint(xValue,
-//                                    data[i]);
-//            }
-//            
-//            const AString description = (getFileNameNoPath()
-//                                         + " node "
-//                                         + AString::number(nodeIndex));
-//            chartData->setDescription(description);
-//        }
-//    }
-//    catch (const DataFileException& dfe) {
-//        if (chartData != NULL) {
-//            delete chartData;
-//            chartData = NULL;
-//        }
-//        
-//        throw dfe;
-//    }
-//    
-//    return chartData;
 }
 
 /**
@@ -393,10 +498,10 @@ CiftiParcelScalarFile::setMatrixChartingEnabled(const int32_t tabIndex,
  *    Chart types supported by this file.
  */
 void
-CiftiParcelScalarFile::getSupportedMatrixChartDataTypes(std::vector<ChartDataTypeEnum::Enum>& chartDataTypesOut) const
+CiftiParcelScalarFile::getSupportedMatrixChartDataTypes(std::vector<ChartOneDataTypeEnum::Enum>& chartDataTypesOut) const
 {
     chartDataTypesOut.clear();
-    chartDataTypesOut.push_back(ChartDataTypeEnum::CHART_DATA_TYPE_MATRIX_LAYER);
+    chartDataTypesOut.push_back(ChartOneDataTypeEnum::CHART_DATA_TYPE_MATRIX_LAYER);
 }
 
 /**
