@@ -27,10 +27,12 @@
 #include "CaretAssert.h"
 #include "GroupAndNameHierarchyGroup.h"
 #include "GroupAndNameHierarchyName.h"
+#include "GroupAndNameHierarchyUserInterface.h"
 #include "SceneAttributes.h"
 #include "SceneClass.h"
 #include "SceneClassArray.h"
 #include "SceneClassAssistant.h"
+#include "TabDrawingInfo.h"
 
 #include <algorithm>
 
@@ -46,6 +48,8 @@ using namespace caret;
 /**
  * Constructor.
  *
+ * @param groupAndNameHierarchyUserInterface;
+ *    Interface for files that use group and name hierarchy
  * @param itemType
  *    Type of this item.
  * @param name
@@ -53,10 +57,12 @@ using namespace caret;
  * @param idNumber
  *    Id number for the item.
  */
-GroupAndNameHierarchyItem::GroupAndNameHierarchyItem(const ItemType itemType,
-                                                   const AString& name,
-                                                   const int32_t idNumber)
+GroupAndNameHierarchyItem::GroupAndNameHierarchyItem(GroupAndNameHierarchyUserInterface* groupAndNameHierarchyUserInterface,
+                                                     const ItemType itemType,
+                                                     const AString& name,
+                                                     const int32_t idNumber)
 : CaretObject(),
+m_groupAndNameHierarchyUserInterface(groupAndNameHierarchyUserInterface),
 m_itemType(itemType),
 m_name(name),
 m_idNumber(idNumber),
@@ -349,7 +355,6 @@ GroupAndNameHierarchyItem::addChildPrivate(GroupAndNameHierarchyItem* child)
 
 /**
  * Add a child with the given name and id number.
- *
  * @param itemType
  *     Type of the item.
  * @param name
@@ -362,8 +367,8 @@ GroupAndNameHierarchyItem::addChildPrivate(GroupAndNameHierarchyItem* child)
  */
 GroupAndNameHierarchyItem*
 GroupAndNameHierarchyItem::addChild(const ItemType itemType,
-                                   const AString& name,
-                                   const int32_t idNumber)
+                                    const AString& name,
+                                    const int32_t idNumber)
 {
     GroupAndNameHierarchyItem* child = getChildWithNameAndIdNumber(name,
                                                                   idNumber);
@@ -374,11 +379,13 @@ GroupAndNameHierarchyItem::addChild(const ItemType itemType,
     
     switch (itemType) {
         case ITEM_TYPE_GROUP:
-            child = new GroupAndNameHierarchyGroup(name,
-                                                  idNumber);
+            child = new GroupAndNameHierarchyGroup(m_groupAndNameHierarchyUserInterface,
+                                                   name,
+                                                   idNumber);
             break;
         case ITEM_TYPE_NAME:
-            child = new GroupAndNameHierarchyName(name,
+            child = new GroupAndNameHierarchyName(m_groupAndNameHierarchyUserInterface,
+                                                  name,
                                                   idNumber);
             break;
         case ITEM_TYPE_MODEL:
@@ -421,6 +428,18 @@ GroupAndNameHierarchyItem::removeChild(GroupAndNameHierarchyItem* child)
     }
 }
 
+/**
+ * @return True if selected
+ * @param tabDrawingInfo
+ *    Info for drawing tab
+ */
+bool
+GroupAndNameHierarchyItem::isSelected(const TabDrawingInfo& tabDrawingInfo) const
+{
+    return isSelected(tabDrawingInfo.getDisplayGroup(),
+                      tabDrawingInfo.getTabIndex());
+}
+
 /**
  * Is this item selected?
  *
@@ -449,6 +468,23 @@ GroupAndNameHierarchyItem::isSelected(const DisplayGroupEnum::Enum displayGroup,
     return m_selectedInDisplayGroup[displayIndex];
 }
 
+/**
+ * Get the "check state" of an item.
+ * @param tabDrawingInfo
+ *    Info for drawing tab
+ * @return
+ *    CHECKED if this item and ALL of its children are selected.
+ *    PARTIALLY_CHECKED if this item is selected and any of its
+ *       children, but not all of its children, are selected.
+ *    UNCHECKED if this item is not selected.
+ */
+GroupAndNameCheckStateEnum::Enum
+GroupAndNameHierarchyItem::getCheckState(const TabDrawingInfo& tabDrawingInfo) const
+{
+    return getCheckState(tabDrawingInfo.getDisplayGroup(),
+                         tabDrawingInfo.getTabIndex());
+}
+
 /**
  * Get the "check state" of an item.
  *
@@ -501,6 +537,24 @@ GroupAndNameHierarchyItem::getCheckState(const DisplayGroupEnum::Enum displayGro
     return GroupAndNameCheckStateEnum::UNCHECKED;
 }
 
+/**
+ * Set the selected status of this item only.  It does not alter
+ * the status of ancestors and children.
+ *
+ * @param tabDrawingInfo
+ *    Info for drawing tab
+ * @param status
+ *    True if item is selected, else false.
+ */
+void
+GroupAndNameHierarchyItem::setSelected(const TabDrawingInfo& tabDrawingInfo,
+                                       const bool status)
+{
+    setSelected(tabDrawingInfo.getDisplayGroup(),
+                tabDrawingInfo.getTabIndex(),
+                status);
+}
+
 /**
  * Set the selected status of this item only.  It does not alter
  * the status of ancestors and children.
@@ -530,6 +584,27 @@ GroupAndNameHierarchyItem::setSelected(const DisplayGroupEnum::Enum displayGroup
     else {
         m_selectedInDisplayGroup[displayIndex] = status;
     }
+    
+    if (m_groupAndNameHierarchyUserInterface != NULL) {
+        m_groupAndNameHierarchyUserInterface->groupAndNameHierarchyItemStatusChanged();
+    }
+}
+
+/**
+ * Set the selected status for all of this item's descendants.
+ *
+ * @param tabDrawingInfo
+ *    Info for drawing tab
+ * @param status
+ *    True if item is selected, else false.
+ */
+void
+GroupAndNameHierarchyItem::setDescendantsSelected(const TabDrawingInfo& tabDrawingInfo,
+                                                  const bool status)
+{
+    setDescendantsSelected(tabDrawingInfo.getDisplayGroup(),
+                           tabDrawingInfo.getTabIndex(),
+                           status);
 }
 
 /**
@@ -560,6 +635,24 @@ GroupAndNameHierarchyItem::setDescendantsSelected(const DisplayGroupEnum::Enum d
     }
 }
 
+/**
+ * Set the selected status of this item's ancestor's (parent,
+ * its parent, etc).
+ *
+ * @param tabDrawingInfo
+ *    Info for drawing tab
+ * @param status
+ *    True if item is selected, else false.
+ */
+void
+GroupAndNameHierarchyItem::setAncestorsSelected(const TabDrawingInfo& tabDrawingInfo,
+                                                const bool status)
+{
+    setAncestorsSelected(tabDrawingInfo.getDisplayGroup(),
+                         tabDrawingInfo.getTabIndex(),
+                         status);
+}
+
 /**
  * Set the selected status of this item's ancestor's (parent,
  * its parent, etc).
@@ -586,6 +679,24 @@ GroupAndNameHierarchyItem::setAncestorsSelected(const DisplayGroupEnum::Enum dis
     }
 }
 
+/**
+ * Set the selected status of this item, its ancestors, and all
+ * of its children.
+ *
+ * @param tabDrawingInfo
+ *    Info for drawing tab
+ * @param status
+ *    True if item is selected, else false.
+ */
+void
+GroupAndNameHierarchyItem::setSelfAncestorsAndDescendantsSelected(const TabDrawingInfo& tabDrawingInfo,
+                                                                  const bool status)
+{
+    setSelfAncestorsAndDescendantsSelected(tabDrawingInfo.getDisplayGroup(),
+                                           tabDrawingInfo.getTabIndex(),
+                                           status);
+}
+
 /**
  * Set the selected status of this item, its ancestors, and all
  * of its children.
@@ -645,6 +756,10 @@ GroupAndNameHierarchyItem::setIconColorRGBA(const float rgba[4])
     m_iconRGBA[1] = rgba[1];
     m_iconRGBA[2] = rgba[2];
     m_iconRGBA[3] = rgba[3];
+
+    if (m_groupAndNameHierarchyUserInterface != NULL) {
+        m_groupAndNameHierarchyUserInterface->groupAndNameHierarchyItemStatusChanged();
+    }
 }
 
 /**
@@ -722,6 +837,18 @@ GroupAndNameHierarchyItem::copySelections(const int32_t sourceTabIndex,
     m_selectedInTab[targetTabIndex] = m_selectedInTab[sourceTabIndex];
     m_expandedStatusInTab[targetTabIndex] = m_expandedStatusInTab[sourceTabIndex];
 
+    AString indent;
+    switch (m_itemType) {
+        case ITEM_TYPE_GROUP:
+            indent = "   ";
+            break;
+        case ITEM_TYPE_MODEL:
+            break;
+        case ITEM_TYPE_NAME:
+            indent = "      ";
+            break;
+    }
+    
     for (std::vector<GroupAndNameHierarchyItem*>::const_iterator iter = m_children.begin();
          iter != m_children.end();
          iter++) {
@@ -928,6 +1055,7 @@ GroupAndNameHierarchyItem::restoreFromScene(const SceneAttributes* sceneAttribut
         return;
     }
     
+    sceneClass->getStringValue("m_name"); // prevents "failed to restore"
     m_sceneAssistant->restoreMembers(sceneAttributes,
                                      sceneClass);
     
