@@ -26,6 +26,7 @@
 #include <stdint.h>
 
 #include "GiftiTypeFile.h"
+#include "GroupAndNameHierarchyUserInterface.h"
 
 namespace caret {
 
@@ -36,7 +37,7 @@ namespace caret {
     /**
      * \brief A Label data file.
      */
-    class LabelFile : public GiftiTypeFile {
+    class LabelFile : public GiftiTypeFile, public GroupAndNameHierarchyUserInterface {
         
     public:
         LabelFile();
@@ -62,6 +63,8 @@ namespace caret {
         
         const GiftiLabelTable* getLabelTable() const;
         
+        virtual const ClusterContainer* getMapLabelTableClusters(const int32_t mapIndex) const override;
+        
         int32_t getLabelKey(const int32_t nodeIndex,
                             const int32_t columnIndex) const;
         
@@ -86,6 +89,11 @@ namespace caret {
         
         const GroupAndNameHierarchyModel* getGroupAndNameHierarchyModel() const;
         
+        //override writeFile in order to check filename against type of file
+        virtual void writeFile(const AString& filename);
+        
+        virtual void groupAndNameHierarchyItemStatusChanged() override;
+
     protected:
         /**
          * Validate the contents of the file after it
@@ -104,6 +112,9 @@ namespace caret {
         /** Points to actual data in each Gifti Data Array */
         std::vector<int32_t*> columnDataPointers;
 
+        mutable std::map<int32_t, std::unique_ptr<ClusterContainer>> m_mapLabelClusterContainers;
+        
+
         /** Holds class and name hierarchy used for display selection */
         mutable GroupAndNameHierarchyModel* m_classNameHierarchy;
         
