@@ -26,20 +26,21 @@
 #include "CaretColorEnum.h"
 #include "DisplayGroupEnum.h"
 #include "LabelDrawingTypeEnum.h"
+#include "PaletteThresholdOutlineDrawingModeEnum.h"
 
 namespace caret {
     class FastStatistics;
     class GiftiLabelTable;
-    class Palette;
     class PaletteColorMapping;
+    class TabDrawingInfo;
     
     class NodeAndVoxelColoring {
         
     public:
         static void colorScalarsWithPalette(const FastStatistics* statistics,
                                             const PaletteColorMapping* paletteColorMapping,
-                                            const Palette* palette,
                                             const float* scalars,
+                                            const PaletteColorMapping* thresholdPaletteColorMapping,
                                             const float* scalarThresholds,
                                             const int64_t numberOfScalars,
                                             float* rgbaOut,
@@ -47,30 +48,27 @@ namespace caret {
         
         static void colorScalarsWithPalette(const FastStatistics* statistics,
                                             const PaletteColorMapping* paletteColorMapping,
-                                            const Palette* palette,
                                             const float* scalars,
+                                            const PaletteColorMapping* thresholdPaletteColorMapping,
                                             const float* scalarThresholds,
                                             const int64_t numberOfScalars,
                                             uint8_t* rgbaOut,
                                             const bool ignoreThresholding = false);
         
-        // JWH 24 April 2015  static const float SMALL_POSITIVE;
-        // JWH 24 April 2015  static const float SMALL_NEGATIVE;
-        
-        static void colorIndicesWithLabelTableForDisplayGroupTab(const GiftiLabelTable* labelTable,
-                                               const float* labelIndices,
-                                               const int64_t numberOfIndices,
-                                               const DisplayGroupEnum::Enum displayGroup,
-                                               const int32_t tabIndex,
-                                               float* rgbv);
-        
-        static void colorIndicesWithLabelTableForDisplayGroupTab(const GiftiLabelTable* labelTable,
-                                               const float* labelIndices,
-                                               const int64_t numberOfIndices,
-                                               const DisplayGroupEnum::Enum displayGroup,
-                                               const int32_t tabIndex,
-                                               uint8_t* rgbv);
+        static void colorScalarsWithRGBA(const float* redComponents,
+                                         const float* greenComponents,
+                                         const float* blueComponents,
+                                         const float* alphaComponents,
+                                         const int64_t numberOfComponents,
+                                         const uint8_t rgbThreshold[3],
+                                         uint8_t* rgbaOut);
         
+        static void colorIndicesWithLabelTableForObliqueVolume(const GiftiLabelTable* labelTable,
+                                                               const float* labelIndices,
+                                                               const int64_t numberOfIndices,
+                                                               const TabDrawingInfo& tabDrawingInfo,
+                                                               uint8_t* rgbv);
+
         static void colorIndicesWithLabelTable(const GiftiLabelTable* labelTable,
                                                const float* labelIndices,
                                                const int64_t numberOfIndices,
@@ -87,6 +85,12 @@ namespace caret {
                                                       const int64_t xdim,
                                                       const int64_t ydim);
         
+        static void convertPaletteSliceColoringToOutlineMode(uint8_t* rgbaInOut,
+                                                             const PaletteThresholdOutlineDrawingModeEnum::Enum outlineMode,
+                                                             const CaretColorEnum::Enum outlineColor,
+                                                             const int64_t xdim,
+                                                             const int64_t ydim);
+        
     private:
         enum ColorDataType {
             COLOR_TYPE_FLOAT,
@@ -94,14 +98,14 @@ namespace caret {
         };
         
         static void colorScalarsWithPalettePrivate(const FastStatistics* statistics,
-                                            const PaletteColorMapping* paletteColorMapping,
-                                            const Palette* palette,
-                                            const float* scalars,
-                                            const float* scalarThresholds,
-                                            const int64_t numberOfScalars,
-                                            const ColorDataType colorDataType,
-                                            void* rgbaOutPointer,
-                                            const bool ignoreThresholding);
+                                                   const PaletteColorMapping* paletteColorMapping,
+                                                   const float* scalars,
+                                                   const PaletteColorMapping* thresholdPaletteColorMapping,
+                                                   const float* scalarThresholds,
+                                                   const int64_t numberOfScalars,
+                                                   const ColorDataType colorDataType,
+                                                   void* rgbaOutPointer,
+                                                   const bool ignoreThresholding);
         
         static void colorIndicesWithLabelTableForDisplayGroupTabPrivate(const GiftiLabelTable* labelTable,
                                                       const float* labelIndices,
@@ -111,6 +115,16 @@ namespace caret {
                                                       const ColorDataType colorDataType,
                                                       void* rgbaOutPointer);
         
+        static void colorScalarsWithRGBAPrivate(const float* redComponents,
+                                                const float* greenComponents,
+                                                const float* blueComponents,
+                                                const float* alphaComponents,
+                                                const int64_t numberOfComponents,
+                                                const ColorDataType colorDataType,
+                                                const uint8_t* rgbThreshold,
+                                                uint8_t* rgbaOutPointer);
+        
+        
         NodeAndVoxelColoring();
         
         virtual ~NodeAndVoxelColoring();
