@@ -34,6 +34,7 @@
 #include "CaretLogger.h"
 #include "DataFileContentInformation.h"
 #include "DataFileException.h"
+#include "DeveloperFlagsEnum.h"
 #include "FileAdapter.h"
 #include "FileInformation.h"
 #include "GiftiMetaData.h"
@@ -41,15 +42,15 @@
 #include "SceneAttributes.h"
 #include "SceneClass.h"
 #include "SceneClassArray.h"
-#include "SceneFileSaxReader.h"
 #include "SceneInfo.h"
 #include "ScenePathName.h"
-#include "SceneXmlElements.h"
-#include "SceneWriterXml.h"
+#include "ScenePathNameArray.h"
+#include "SceneFileXmlStreamReader.h"
+#include "SceneFileXmlStreamWriter.h"
 #include "SpecFile.h"
 #include "SystemUtilities.h"
-#include "XmlSaxParser.h"
-#include "XmlWriter.h"
+#include "XmlException.h"
+#include "WuQMacroGroup.h"
 
 using namespace caret;
 
@@ -273,6 +274,28 @@ SceneFile::getSceneWithName(const AString& sceneName)
     return NULL;
 }
 
+/**
+ * Get the scene with the given scene ID.
+ * @param sceneID
+ *    ID of scene.
+ * @return
+ *    Scene with given ID or NULL if no scene with
+ *    the given ID.
+ */
+Scene*
+SceneFile::getSceneWithSceneID(const AString& sceneID)
+{
+    const int32_t numScenes = getNumberOfScenes();
+    for (int32_t i = 0; i < numScenes; i++) {
+        Scene* scene = getSceneAtIndex(i);
+        if (scene->getSceneInfo()->getBalsaSceneID() == sceneID) {
+            return scene;
+        }
+    }
+    return NULL;
+}
+
+
 /**
  * Remove the given scene.
  * @param scene
@@ -570,6 +593,64 @@ void SceneFile::setBasePathType(const SceneFileBasePathTypeEnum::Enum basePathTy
     }
 }
 
+/**
+ * Get the type of base directory and the name of the base directory
+ * @param basePathTypeOut
+ *    Output with type of base path
+ * @param basePathNameOut
+ *    Output with name of base path
+ * @param errorMessageOut
+ *    Output with error information
+ * @return
+ *    True if the outputs are valid, else false.
+ */
+bool
+SceneFile::getSelectedBasePathTypeAndName(SceneFileBasePathTypeEnum::Enum& basePathTypeOut,
+                                          AString& basePathNameOut,
+                                          AString& errorMessageOut) const
+{
+    basePathNameOut.clear();
+    bool validFlag(false);
+    
+    switch (getBasePathType()) {
+        case SceneFileBasePathTypeEnum::AUTOMATIC:
+        {
+            std::vector<AString> missingFileNames;
+            validFlag = findBaseDirectoryForDataFiles(basePathNameOut,
+                                                      missingFileNames,
+                                                      errorMessageOut);
+        }
+            break;
+        case SceneFileBasePathTypeEnum::CUSTOM:
+        {
+            basePathNameOut = getBalsaCustomBaseDirectory();
+            if ( ! basePathNameOut.isEmpty()) {
+                 validFlag = true;
+            }
+            else {
+                errorMessageOut = "Custom path is invalid (empty)";
+            }
+        }
+            break;
+    }
+    
+    basePathTypeOut = getBasePathType();
+
+    return validFlag;
+}
+
+
+/**
+ * Set the name of the file.
+ *
+ * @param filename
+ *    New name of file
+ */
+void
+SceneFile::setFileName(const AString& filename)
+{
+    CaretDataFile::setFileName(filename);
+}
 
 /**
  * Read the scene file.
@@ -584,58 +665,122 @@ SceneFile::readFile(const AString& filenameIn)
     clear();
     
     AString filename = filenameIn;
-    if (DataFile::isFileOnNetwork(filename) == false) {
+    if ( ! DataFile::isFileOnNetwork(filename)) {
         FileInformation specInfo(filename);
         filename = specInfo.getAbsoluteFilePath();
     }
     checkFileReadability(filename);
     
     this->setFileName(filename);
-    SceneFileSaxReader saxReader(this,
-                                 filename);
-    std::auto_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
+
     try {
-        parser->parseFile(filename, &saxReader);
+        SceneFileXmlStreamReader streamReader;
+        streamReader.readFile(filename,
+                              this);
     }
-    catch (const XmlSaxParserException& e) {
-        clear();
-        this->setFileName("");
-        
-        int lineNum = e.getLineNumber();
-        int colNum  = e.getColumnNumber();
-        
-        AString msg = "Parse Error while reading:";
-        
-        if ((lineNum >= 0) && (colNum >= 0)) {
-            msg += (" line/col ("
-                    + AString::number(e.getLineNumber())
-                    + "/"
-                    + AString::number(e.getColumnNumber())
-                    + ")");
-        }
-        
-        msg += (": " + e.whatString());
-        
-        DataFileException dfe(filenameIn,
-                              msg);
+    catch (const DataFileException& e) {
+        DataFileException dfe(filename,
+                              e.whatString());
         CaretLogThrowing(dfe);
         throw dfe;
     }
-    
+
     this->setFileName(filename);
 
     this->clearModified();
 }
 
+/**
+ * Read the scene file using the new Stream parser
+ * @param filenameIn
+ *    Name of scene file.
+ * @throws DataFileException
+ *    If there is an error reading the file.
+ */
+void
+SceneFile::readFileStreamReader(const AString& filenameIn)
+{
+    clear();
+    
+    AString filename = filenameIn;
+    if ( ! DataFile::isFileOnNetwork(filename)) {
+        FileInformation specInfo(filename);
+        filename = specInfo.getAbsoluteFilePath();
+    }
+    checkFileReadability(filename);
+    
+    this->setFileName(filename);
+    
+    try {
+        SceneFileXmlStreamReader streamReader;
+        streamReader.readFile(filename,
+                              this);
+    }
+    catch (const DataFileException& e) {
+        DataFileException dfe(filename,
+                              e.whatString());
+        CaretLogThrowing(dfe);
+        throw dfe;
+    }
+
+    this->setFileName(filename);
+    
+    this->clearModified();
+}
+
+
 /**
  * Write the scene file.
- * @param filename
+ * @param filenameIn
  *    Name of scene file.
  * @throws DataFileException
  *    If there is an error writing the file.
  */
 void 
-SceneFile::writeFile(const AString& filename)
+SceneFile::writeFile(const AString& filenameIn)
+{
+    if (!(filenameIn.endsWith(".scene") || filenameIn.endsWith(".wb_scene")))
+    {
+        CaretLogWarning("scene file '" + filenameIn + "' should be saved ending in .scene");
+    }
+    AString filename(filenameIn);
+    if ( ! DataFile::isFileOnNetwork(filename)) {
+        FileInformation specInfo(filename);
+        filename = specInfo.getAbsoluteFilePath();
+    }
+    checkFileWritability(filename);
+    
+    this->setFileName(filename);
+    
+    
+    try {
+        /*
+         * Update file's metadata with info about Workbench
+         */
+        SceneInfo::addWorkbenchVersionInfoToMetaData(m_metadata);
+        
+        SceneFileXmlStreamWriter xmlStreamWriter;
+        xmlStreamWriter.writeFile(this);
+
+        this->clearModified();
+    }
+    catch (const GiftiException& e) {
+        throw DataFileException(e);
+    }
+    catch (const XmlException& e) {
+        throw DataFileException(e);
+    }
+}
+
+/**
+ * Write the scene file stream writer
+ * @param filename
+ *    Name of scene file.
+ * @throws DataFileException
+ *    If there is an error writing the file.
+ */
+void
+SceneFile::writeFileStreamWriter(const AString& filename)
 {
     if (!(filename.endsWith(".scene") || filename.endsWith(".wb_scene")))
     {
@@ -646,125 +791,9 @@ SceneFile::writeFile(const AString& filename)
     this->setFileName(filename);
     
     try {
-        //
-        // Format the version string so that it ends with at most one zero
-        //
-        const AString versionString = AString::number(SceneFile::getFileVersion(),
-                                                      'f',
-                                                      1);
-        
-        //
-        // Open the file
-        //
-        FileAdapter file;
-        AString errorMessage;
-        QTextStream* textStream = file.openQTextStreamForWritingFile(this->getFileName(),
-                                                                     errorMessage);
-        if (textStream == NULL) {
-            throw DataFileException(filename,
-                                    errorMessage);
-        }
-        
-        //
-        // Create the xml writer
-        //
-        XmlWriter xmlWriter(*textStream);
-        
-        //
-        // Write header info
-        //
-        xmlWriter.writeStartDocument("1.0");
-        
-        //
-        // Write root element
-        //
-        XmlAttributes attributes;
-        
-        //attributes.addAttribute("xmlns:xsi",
-        //                        "http://www.w3.org/2001/XMLSchema-instance");
-        //attributes.addAttribute("xsi:noNamespaceSchemaLocation",
-        //                        "http://brainvis.wustl.edu/caret6/xml_schemas/GIFTI_Caret.xsd");
-        attributes.addAttribute(SceneFile::XML_ATTRIBUTE_VERSION,
-                                versionString);
-        xmlWriter.writeStartElement(SceneFile::XML_TAG_SCENE_FILE,
-                                    attributes);
-        
-        //
-        // Write Metadata
-        //
-        if (m_metadata != NULL) {
-            m_metadata->writeAsXML(xmlWriter);
-        }
+        SceneFileXmlStreamWriter xmlStreamWriter;
+        xmlStreamWriter.writeFile(this);
 
-        const int32_t numScenes = this->getNumberOfScenes();
-
-        /*
-         * Write the scene info directory
-         */
-        xmlWriter.writeStartElement(SceneFile::XML_TAG_SCENE_INFO_DIRECTORY_TAG);
-        xmlWriter.writeElementCData(SceneXmlElements::SCENE_INFO_BALSA_STUDY_ID_TAG,
-                                    getBalsaStudyID());
-        xmlWriter.writeElementCData(SceneXmlElements::SCENE_INFO_BALSA_STUDY_TITLE_TAG,
-                                    getBalsaStudyTitle());
-        switch (getBasePathType()) {
-            case SceneFileBasePathTypeEnum::AUTOMATIC:
-                xmlWriter.writeElementCData(SceneXmlElements::SCENE_INFO_BALSA_BASE_DIRECTORY_TAG,
-                                            "");
-                break;
-            case SceneFileBasePathTypeEnum::CUSTOM:
-            {
-                /*
-                 * Write base path as a path RELATIVE to the scene file
-                 * but only when base path type is CUSTOM
-                 * Note: we do not use FileInformation::getCanonicalFilePath()
-                 * because it returns an empty string if the file DOES NOT exist
-                 * and this may occur since the file may be new and has not
-                 * been closed.
-                 */
-                if ( ! getBalsaCustomBaseDirectory().isEmpty()) {
-                    const AString baseDirAbsPath = FileInformation(getBalsaCustomBaseDirectory()).getAbsoluteFilePath();
-                    const AString sceneFileAbsPath = FileInformation(filename).getAbsoluteFilePath();
-                    ScenePathName basePathName("basePathName",
-                                               baseDirAbsPath);
-                    
-                    const AString relativeBasePath = basePathName.getRelativePathToSceneFile(sceneFileAbsPath);
-                    
-                    //std::cout << "baseDirAbsPath: " << baseDirAbsPath << std::endl;
-                    //std::cout << "sceneFileAbsPath: " << sceneFileAbsPath << std::endl;
-                    //std::cout << "relativeTempFileName: " << relativeBasePath << std::endl;
-                    
-                    xmlWriter.writeElementCData(SceneXmlElements::SCENE_INFO_BALSA_BASE_DIRECTORY_TAG,
-                                                relativeBasePath);
-                }
-            }
-                break;
-        }
-        xmlWriter.writeElementCData(SceneXmlElements::SCENE_INFO_BALSA_EXTRACT_TO_DIRECTORY_TAG,
-                                    getBalsaExtractToDirectoryName());
-        xmlWriter.writeElementCData(SceneXmlElements::SCENE_INFO_BASE_PATH_TYPE,
-                                    SceneFileBasePathTypeEnum::toName(getBasePathType()));
-        
-        for (int32_t i = 0; i < numScenes; i++) {
-            m_scenes[i]->getSceneInfo()->writeSceneInfo(xmlWriter,
-                                                        i);
-        }
-        xmlWriter.writeEndElement();
-        
-        //
-        // Write scenes
-        //
-        SceneWriterXml sceneWriter(xmlWriter,
-                                   this->getFileName());
-        for (int32_t i = 0; i < numScenes; i++) {
-            sceneWriter.writeScene(*m_scenes[i], 
-                                   i);
-        }
-        
-        xmlWriter.writeEndElement();
-        xmlWriter.writeEndDocument();
-        
-        file.close();
-        
         this->clearModified();
     }
     catch (const GiftiException& e) {
@@ -786,16 +815,32 @@ SceneFile::addToDataFileContentInformation(DataFileContentInformation& dataFileI
 {
     CaretDataFile::addToDataFileContentInformation(dataFileInformation);
     
+    dataFileInformation.addNameAndValue("Base Path Type", SceneFileBasePathTypeEnum::toGuiName(getBasePathType()));
+    dataFileInformation.addNameAndValue("Custom Base Path", getBalsaCustomBaseDirectory());
+    dataFileInformation.addNameAndValue("Extract Directory", getBalsaExtractToDirectoryName());
+    dataFileInformation.addNameAndValue("BALSA Title", getBalsaStudyTitle());
+    dataFileInformation.addNameAndValue("BALSA Study ID", getBalsaStudyID());
+    
     const int32_t numScenes = getNumberOfScenes();
     if (numScenes > 0) {
         AString sceneNamesText = "Scenes:";
         for (int32_t i = 0; i < numScenes; i++) {
             const Scene* scene = getSceneAtIndex(i);
-            sceneNamesText.appendWithNewLine("#" + AString::number(i + 1) + "  " +
+            sceneNamesText.appendWithNewLine("\n#" + AString::number(i + 1) + "  " +
                                              scene->getName());
             if (dataFileInformation.isOptionFlag(DataFileContentInformation::OPTION_SHOW_MAP_INFORMATION))
             {
-                sceneNamesText += ":";
+                const AString indent("        ");
+                const GiftiMetaData* sceneMetaData(scene->getSceneInfo()->getMetaData());
+                const auto namesAndValues(sceneMetaData->getAsMap());
+                if ( ! namesAndValues.empty()) {
+                    sceneNamesText.appendWithNewLine(indent + "Metadata:");
+                    for (const auto& nv : namesAndValues) {
+                        sceneNamesText.appendWithNewLine(indent + "   " + nv.first + ":  " + nv.second);
+                    }
+                }
+
+                //sceneNamesText += ":";
                 const SceneAttributes* myAttrs = scene->getAttributes();
                 const SceneClass* guiMgrClass = scene->getClassWithName("guiManager");
                 if (guiMgrClass == NULL)
@@ -831,7 +876,11 @@ SceneFile::addToDataFileContentInformation(DataFileContentInformation& dataFileI
                     int numNames = (int)tempNames.size();
                     for (int k = 0; k < numNames; ++k)
                     {
-                        sceneNamesText.appendWithNewLine("        " + tempNames[k]);
+                        AString notFoundText;
+                        if ( ! QFile::exists(tempNames[k])) {
+                            notFoundText = " (NOT FOUND)";
+                        }
+                        sceneNamesText.appendWithNewLine(indent + tempNames[k] + notFoundText);
                     }
                 }
             }
@@ -883,6 +932,37 @@ bool
 SceneFile::findBaseDirectoryForDataFiles(AString& baseDirectoryOut,
                                          std::vector<AString>& missingFileNamesOut,
                                          AString& errorMessageOut) const
+{
+    return SceneFile::findBaseDirectoryForDataFiles(getFileName(),
+                                                    getAllDataFileNamesFromAllScenes(),
+                                                    baseDirectoryOut,
+                                                    missingFileNamesOut,
+                                                    errorMessageOut);
+}
+
+/**
+ * Find the base directory that is a directory that is parent to all loaded data files
+ * and also including the scene file.
+ *
+ * @param sceneFileName
+ *    Name of scene file
+ * @param filesFromScenes
+ *    Files in all scenes in the scene file
+ * @param baseDirectoryOut
+ *    Output containing the base directory
+ * @param missingFileNamesOut
+ *    Will contain data files that are in scenes but do not exist.
+ * @param errorMessageOut
+ *    Error message if finding base directory fails
+ * @return
+ *    True if the base directory is valid, else false.
+ */
+bool
+SceneFile::findBaseDirectoryForDataFiles(const AString& sceneFileName,
+                                         const std::set<FileAndSceneIndicesInfo>& filesFromScenes,
+                                         AString& baseDirectoryOut,
+                                         std::vector<AString>& missingFileNamesOut,
+                                         AString& errorMessageOut)
 {
     baseDirectoryOut.clear();
     missingFileNamesOut.clear();
@@ -890,8 +970,13 @@ SceneFile::findBaseDirectoryForDataFiles(AString& baseDirectoryOut,
     
     const AString directorySeparator("/");
     
-    std::vector<AString> allFileNames = getAllDataFileNamesFromAllScenes();
-    allFileNames.push_back(getFileName());
+    std::vector<AString> allFileNames;
+    for (const auto& nameInfo : filesFromScenes) {
+        allFileNames.push_back(nameInfo.m_dataFileName);
+    }
+    if ( ! sceneFileName.isEmpty()) {
+        allFileNames.push_back(sceneFileName);
+    }
     
     /*
      * Find a unique set of directory names used by the data files and
@@ -925,8 +1010,11 @@ SceneFile::findBaseDirectoryForDataFiles(AString& baseDirectoryOut,
         }
     }
     
-    const FileInformation fileInfo(getFileName());
-    const AString sceneFilePath = fileInfo.getAbsolutePath().trimmed();
+    AString sceneFilePath;
+    if ( ! sceneFileName.isEmpty()) {
+        const FileInformation fileInfo(sceneFileName);
+        sceneFilePath = fileInfo.getAbsolutePath().trimmed();
+    }
     
     missingFileNamesOut.insert(missingFileNamesOut.end(),
                                missingFileNames.begin(),
@@ -935,8 +1023,13 @@ SceneFile::findBaseDirectoryForDataFiles(AString& baseDirectoryOut,
     const int32_t numDirs = static_cast<int32_t>(directoryNamesUniqueSet.size());
     if (numDirs <= 0) {
         /*
-         * if no valid files in scene, user path of scene file.
+         * if no valid files in scene, use path of scene file.
          */
+        if (sceneFilePath.isEmpty()) {
+            errorMessageOut = "No data files available for determining base path";
+            return false;
+        }
+        
         baseDirectoryOut = sceneFilePath;
         return true;
     }
@@ -949,7 +1042,7 @@ SceneFile::findBaseDirectoryForDataFiles(AString& baseDirectoryOut,
      */
     bool firstFlag = true;
     std::vector<AString> longestPathMatch;
-    for (const auto dirName : directoryNamesUniqueSet) {
+    for (const auto& dirName : directoryNamesUniqueSet) {
         if (firstFlag) {
             firstFlag = false;
             longestPathMatch = AString::stringListToVector(dirName.split(directorySeparator));
@@ -962,13 +1055,12 @@ SceneFile::findBaseDirectoryForDataFiles(AString& baseDirectoryOut,
         }
     }
     
-    baseDirectoryOut = sceneFilePath;
     if ( ! longestPathMatch.empty()) {
         /*
          * Assemble the path components into a directory.
          */
         baseDirectoryOut = AString::join(longestPathMatch,
-                                          directorySeparator);
+                                         directorySeparator);
     }
     
     /*
@@ -1035,25 +1127,29 @@ SceneFile::getBaseDirectoryHierarchyForDataFiles(const int32_t maximumAncestorCo
 }
 
 /**
- * @return A vector containing the names of all data files from all scenes.
+ * @return A set containing the names of all data files from all scenes.
  */
-std::vector<AString>
+std::set<SceneFile::FileAndSceneIndicesInfo>
 SceneFile::getAllDataFileNamesFromAllScenes() const
 {
     const bool includeSpecFileFlag = false;
     
-    std::set<AString> filenamesSet;
+    std::set<FileAndSceneIndicesInfo> fileInfoOut;
     
     /**
      * Find all 'path name' elements from ALL scenes
      */
-    for (Scene* scene: m_scenes) {
+    const int32_t numScenes = static_cast<int32_t>(m_scenes.size());
+    for (int32_t sceneIndex = 0; sceneIndex < numScenes; sceneIndex++) {
+        CaretAssertVectorIndex(m_scenes, sceneIndex);
+        const Scene* scene = m_scenes[sceneIndex];
         CaretAssert(scene);
         std::vector<SceneObject*> children = scene->getDescendants();
         for (SceneObject* sceneObject : children) {
             CaretAssert(sceneObject);
             if (sceneObject->getDataType() == SceneObjectDataTypeEnum::SCENE_PATH_NAME) {
                 const ScenePathName* scenePathName = dynamic_cast<ScenePathName*>(sceneObject);
+
                 /*
                  * Will be NULL for 'path name arrays' which we ignore
                  */
@@ -1067,18 +1163,127 @@ SceneFile::getAllDataFileNamesFromAllScenes() const
                     if (sceneObject->getName() == "fileName") {
                         useNameFlag = true;
                     }
+                    else if (sceneObject->getName() == "brainChildDataFile") {
+                        useNameFlag = true;
+                    }
                     else if (sceneObject->getName() == "specFileName") {
                         useNameFlag = includeSpecFileFlag;
                     }
                     if (useNameFlag) {
                         AString pathName = scenePathName->stringValue().trimmed();
+                        
                         if ( ! pathName.isEmpty()) {
-                            QFileInfo fileInfo(pathName);
-                            const QString absPathName = fileInfo.absoluteFilePath();
-                            if ( ! absPathName.isEmpty()) {
-                                pathName = absPathName;
+                            bool validExtensionFlag = false;
+                            const DataFileTypeEnum::Enum dataFileType = DataFileTypeEnum::fromFileExtension(pathName,
+                                                                                                            &validExtensionFlag);
+                            
+                            bool validDiskFileFlag = true;
+                            if (validExtensionFlag) {
+                                switch (dataFileType) {
+                                    case DataFileTypeEnum::ANNOTATION:
+                                        break;
+                                    case DataFileTypeEnum::ANNOTATION_TEXT_SUBSTITUTION:
+                                        break;
+                                    case DataFileTypeEnum::BORDER:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_DENSE:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC:
+                                        validDiskFileFlag = false;
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_DENSE_LABEL:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_DENSE_PARCEL:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_DENSE_SCALAR:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_DENSE_TIME_SERIES:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_FIBER_ORIENTATIONS_TEMPORARY:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_TEMPORARY:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_MAPS:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_PARCEL:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_PARCEL_DENSE:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC:
+                                        validDiskFileFlag = false;
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_PARCEL_LABEL:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_PARCEL_SCALAR:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_PARCEL_SERIES:
+                                        break;
+                                    case DataFileTypeEnum::CONNECTIVITY_SCALAR_DATA_SERIES:
+                                        break;
+                                    case DataFileTypeEnum::CZI_IMAGE_FILE:
+                                        break;
+                                    case DataFileTypeEnum::FOCI:
+                                        break;
+                                    case DataFileTypeEnum::HISTOLOGY_SLICES:
+                                        break;
+                                    case DataFileTypeEnum::IMAGE:
+                                        break;
+                                    case DataFileTypeEnum::LABEL:
+                                        break;
+                                    case DataFileTypeEnum::METRIC:
+                                        break;
+                                    case DataFileTypeEnum::METRIC_DYNAMIC:
+                                        validDiskFileFlag = false;
+                                        break;
+                                    case DataFileTypeEnum::OME_ZARR_IMAGE_FILE:
+                                        break;
+                                    case DataFileTypeEnum::PALETTE:
+                                        break;
+                                    case DataFileTypeEnum::RGBA:
+                                        break;
+                                    case DataFileTypeEnum::SAMPLES:
+                                        break;
+                                    case DataFileTypeEnum::SCENE:
+                                        break;
+                                    case DataFileTypeEnum::SPECIFICATION:
+                                        break;
+                                    case DataFileTypeEnum::SURFACE:
+                                        break;
+                                    case DataFileTypeEnum::VOLUME:
+                                        break;
+                                    case DataFileTypeEnum::VOLUME_DYNAMIC:
+                                        validDiskFileFlag = false;
+                                        break;
+                                    case DataFileTypeEnum::UNKNOWN:
+                                        validDiskFileFlag = false;
+                                        break;
+                                }
+                            }
+                            
+                            if (validDiskFileFlag) {
+                                FileInformation fileInfo(pathName);
+                                const QString absPathName = fileInfo.getAbsoluteFilePath();
+                                if ( ! absPathName.isEmpty()) {
+                                    pathName = absPathName;
+                                }
+                                
+                                /*
+                                 * Test to see if this file is already in the output file info
+                                 */
+                                bool foundFlag = false;
+                                for (auto& dfi : fileInfoOut) {
+                                    if (dfi.m_dataFileName == pathName) {
+                                        dfi.addSceneIndex(sceneIndex);
+                                        foundFlag = true;
+                                        break;
+                                    }
+                                }
+                                
+                                if ( ! foundFlag) {
+                                    fileInfoOut.insert(FileAndSceneIndicesInfo(pathName,
+                                                                               sceneIndex));
+                                }
                             }
-                            filenamesSet.insert(pathName);
                         }
                     }
                 }
@@ -1086,12 +1291,39 @@ SceneFile::getAllDataFileNamesFromAllScenes() const
         }
     }
     
-    std::vector<AString> filenamesOut(filenamesSet.begin(),
-                                      filenamesSet.end());
-    
-    return filenamesOut;
+    return fileInfoOut;
 }
 
+/**
+ * @return File info for all files in the scene file.
+ */
+std::vector<SceneDataFileInfo>
+SceneFile::getAllDataFileInfoFromAllScenes() const
+{
+    const std::set<SceneFile::FileAndSceneIndicesInfo> allNamesAndIndices = getAllDataFileNamesFromAllScenes();
+    
+    AString basePath;
+    AString errorMessage;
+    std::vector<AString> missingFiles;
+    const bool validFlag = findBaseDirectoryForDataFiles(basePath, missingFiles, errorMessage);
+    if ( ! validFlag) {
+        CaretLogSevere("Failed to find the base path for scene file: "
+                       + getFileName());
+    }
+    
+    std::vector<SceneDataFileInfo> fileInfoOut;
+ 
+    for (const auto& nameAndIndices : allNamesAndIndices) {
+        fileInfoOut.emplace_back(nameAndIndices.m_dataFileName,
+                                 basePath,
+                                 getFileName(),
+                                 nameAndIndices.m_sceneIndices);
+    }
+    
+    return fileInfoOut;
+}
+
+
 /**
  * @return Default name for a ZIP file containing the scene file and its data files.
  */
@@ -1156,3 +1388,46 @@ SceneFile::clearModified()
         scene->clearModified();
     }
 }
+
+/**
+ * @return The version number to use when writing
+ * an instance of a scene.  This number returned
+ * may version depending upon the content of the scene
+ * and may allow older versions of wb_view to read
+ * the scene file when it does not contain new stuff.
+ */
+int32_t
+SceneFile::getSceneFileVersionForWriting() const
+{
+    int32_t version = s_sceneFileVersionBeforeMacros;
+    
+    for (const auto s : m_scenes) {
+        if ( ! s->getMacroGroup()->isEmpty()) {
+            version = s_sceneFileVersionContainingMacros;
+            break;
+        }
+    }
+    return version;
+}
+
+/**
+ * @return The maximum scene file version supported
+ * by the scene file.
+ */
+int32_t
+SceneFile::getMaxiumSupportedSceneFileVersion()
+{
+    return s_sceneFileVersionContainingMacros;
+}
+
+/**
+ * @return The scene file version before macros were added.
+ */
+int32_t
+SceneFile::getSceneFileVersionBeforeMacros()
+{
+    return s_sceneFileVersionBeforeMacros;
+}
+
+
+
