@@ -1,6 +1,6 @@
 /*LICENSE_START*/
 /*
- *  Copyright (C) 2014  Washington University School of Medicine
+ *  Copyright (C) Washington University School of Medicine
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,32 @@
  */
 /*LICENSE_END*/
 
+/*This code can be used with the Oxford FMRIB Software Library and/or
+ * Multimodal Surface Matching under the MIT license instead of the
+ * above GPL license.  The MIT license follows:*/
+/*
+ *  Copyright (C) Washington University School of Medicine
+ *
+ *  Permission is hereby granted, free of charge, to any person obtaining
+ *  a copy of this software and associated documentation files (the
+ *  "Software"), to deal in the Software without restriction, including
+ *  without limitation the rights to use, copy, modify, merge, publish,
+ *  distribute, sublicense, and/or sell copies of the Software, and to
+ *  permit persons to whom the Software is furnished to do so, subject to
+ *  the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be included
+ *  in all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+ *  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+ *  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+ *  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 /*
  * For the function pointInTri():
  * Original copyright for PNPOLY, though my version is entirely rewritten and modified
@@ -113,6 +139,69 @@ float SignedDistanceHelper::dist(const float coord[3], WindingLogic myWinding)
     return bestTriDist * computeSign(coord, bestInfo, myWinding);
 }
 
+float SignedDistanceHelper::distLimited(const float coord[3], const float limit, bool& validOut, SignedDistanceHelper::WindingLogic myWinding)
+{
+    CaretMutexLocker locked(&m_mutex);
+    validOut = false;
+    CaretSimpleMinHeap<Oct<SignedDistanceHelperBase::TriVector>*, float> myHeap;
+    myHeap.push(m_base->m_indexRoot, m_base->m_indexRoot->distToPoint(coord));
+    ClosestPointInfo tempInfo, bestInfo;
+    float tempf = -1.0f, bestTriDist = limit;
+    int numChanged = 0;
+    while (!myHeap.isEmpty())
+    {
+        Oct<SignedDistanceHelperBase::TriVector>* curOct = myHeap.pop(&tempf);
+        if (tempf < bestTriDist)
+        {
+            if (curOct->m_leaf)
+            {
+                vector<int32_t>& myVecRef = *(curOct->m_data.m_triList);
+                int numTris = (int)myVecRef.size();
+                for (int i = 0; i < numTris; ++i)
+                {
+                    if (m_triMarked[myVecRef[i]] != 1)
+                    {
+                        m_triMarked[myVecRef[i]] = 1;
+                        m_triMarkChanged[numChanged++] = myVecRef[i];
+                        tempf = unsignedDistToTri(coord, myVecRef[i], tempInfo);
+                        if (tempf < bestTriDist)
+                        {
+                            bestInfo = tempInfo;
+                            bestTriDist = tempf;
+                            validOut = true;
+                        }
+                    }
+                }
+            } else {
+                for (int ci = 0; ci < 2; ++ci)
+                {
+                    for (int cj = 0; cj < 2; ++cj)
+                    {
+                        for (int ck = 0; ck < 2; ++ck)
+                        {
+                            tempf = curOct->m_children[ci][cj][ck]->distToPoint(coord);
+                            if (tempf < bestTriDist)
+                            {
+                                myHeap.push(curOct->m_children[ci][cj][ck], tempf);
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+    while (numChanged)
+    {
+        m_triMarked[m_triMarkChanged[--numChanged]] = 0;//need to do this before computeSign
+    }
+    if (validOut)
+    {
+        return bestTriDist * computeSign(coord, bestInfo, myWinding);
+    } else {
+        return NAN;//should never be used in this case, so give a nan
+    }
+}
+
 void SignedDistanceHelper::barycentricWeights(const float coord[3], BarycentricInfo& baryInfoOut)
 {
     CaretMutexLocker locked(&m_mutex);
