@@ -389,13 +389,16 @@ SpecFile::addCaretDataFile(CaretDataFile* caretDataFile)
  *
  * @param If there is a a spec file entry with the given caret data file
  * remove it.  Note: file has likely already been deleted so use only the
- * the caret data file pointer but to not deference it.
+ * caret data file pointer but do not deference it.
  *
  * @param caretDataFile
  *    Caret data file that is removed from a spec file entry.
+ * @param logSevereIfFailureToRemoveFileFlag
+ *   If true, log a message if failure to remove file
  */
-void
-SpecFile::removeCaretDataFile(const CaretDataFile* caretDataFile)
+bool
+SpecFile::removeCaretDataFile(const CaretDataFile* caretDataFile,
+                              const bool logSevereIfFailureToRemoveFileFlag)
 {
     CaretAssert(caretDataFile);
     
@@ -411,17 +414,166 @@ SpecFile::removeCaretDataFile(const CaretDataFile* caretDataFile)
             SpecFileDataFile* sfdf = dataFileTypeGroup->getFileInformation(i);
             if (sfdf->getCaretDataFile() == caretDataFile) {
                 sfdf->setCaretDataFile(NULL);
-                return;
+                sfdf->setRemoveWhenSavingToScene(true);
+                return true;
             }
         }
     }
     
-    CaretLogSevere("Failed to remove CaretDataFile at address "
-                   + AString::number((qulonglong)caretDataFile)
-                   + " from SpecFile: "
-                   + getFileName());
+    if (logSevereIfFailureToRemoveFileFlag) {
+        CaretLogSevere("Failed to remove CaretDataFile at address "
+                       + AString::number((qulonglong)caretDataFile)
+                       + " from SpecFile: "
+                       + getFileName());
+    }
+    
+    return false;
 }
 
+/*
+ * Remove any files that are not loaded.  Entries are
+ * removed when spec file is saved.
+ * @return Number of files that were removed
+ */
+int32_t
+SpecFile::removeAllNonLoadedFiles()
+{
+    int32_t removedCount(0);
+    
+    /*
+     * Get the entry
+     */
+    for (std::vector<SpecFileDataFileTypeGroup*>::const_iterator iter = dataFileTypeGroups.begin();
+         iter != dataFileTypeGroups.end();
+         iter++) {
+        SpecFileDataFileTypeGroup* dataFileTypeGroup = *iter;
+        const int32_t numFiles = dataFileTypeGroup->getNumberOfFiles();
+        for (int32_t i = 0; i < numFiles; i++) {
+            SpecFileDataFile* sfdf = dataFileTypeGroup->getFileInformation(i);
+            if (sfdf->getCaretDataFile() == NULL) {
+                sfdf->setRemoveWhenSavingToScene(true);
+                ++removedCount;
+            }
+        }
+    }
+    
+    return removedCount;
+}
+
+/**
+ * Remove a Caret Data File by its name, when this spec file is written to a scene.
+ *
+ * If there is a a spec file entry with the given caret data file
+ * set it to be removed when a scene file using this spec file is written.  Note: file has
+ * likely already been deleted so use only the caret data file pointer but
+ * do not deference it.
+ *
+ * @param filename
+ *    Name of file.
+ * @param logSevereIfFailureToRemoveFileFlag
+ *   If true, log a message if failure to remove file
+ */
+bool
+SpecFile::removeCaretDataFileByName(const AString& filename,
+                                    const bool logSevereIfFailureToRemoveFileFlag)
+{
+    /*
+     * Get the entry
+     * Go through loop two times:
+     * First time look for exact match.
+     * Second time look for match at end of name.
+     */
+    std::vector<SpecFileDataFile*> endMatchedFiles;
+    for (int32_t counter = 0; counter < 2; counter++) {
+        for (std::vector<SpecFileDataFileTypeGroup*>::const_iterator iter = dataFileTypeGroups.begin();
+             iter != dataFileTypeGroups.end();
+             iter++) {
+            SpecFileDataFileTypeGroup* dataFileTypeGroup = *iter;
+            const int32_t numFiles = dataFileTypeGroup->getNumberOfFiles();
+            for (int32_t i = 0; i < numFiles; i++) {
+                SpecFileDataFile* sfdf = dataFileTypeGroup->getFileInformation(i);
+                if (counter == 0) {
+                    /*
+                     * Exact match
+                     */
+                    if (sfdf->getFileName() == filename) {
+                        sfdf->setCaretDataFile(NULL);
+                        sfdf->setRemoveWhenSavingToScene(true);
+                        return true;
+                    }
+                }
+                else {
+                    /*
+                     * End match
+                     */
+                    if (sfdf->getFileName().endsWith(filename)) {
+                        endMatchedFiles.push_back(sfdf);
+                    }
+                }
+            }
+        }
+    }
+
+    if (endMatchedFiles.size() == 1) {
+        SpecFileDataFile* sfdf(endMatchedFiles[0]);
+        sfdf->setCaretDataFile(NULL);
+        sfdf->setRemoveWhenSavingToScene(true);
+        return true;
+    }
+    
+    if (logSevereIfFailureToRemoveFileFlag) {
+        if (endMatchedFiles.size() > 1) {
+            CaretLogSevere("Unable to remove file from SpecFile by name matching.  More than one file with name: "
+                           + filename);
+        }
+        else {
+            CaretLogSevere("Failed to remove CaretDataFile by name from SpecFile: "
+                           + filename);
+        }
+    }
+    
+    return false;
+}
+
+/**
+ * Immediately Remove a File by its name.
+ *
+ * If there is a a spec file entry with the given data file name,
+ * mark it as not a member.  Note: file has likely not been loaded or has been deleted already,
+ * so use only the caret data file pointer but do not deference it.
+ *
+ * @param filename
+ *    Name of file (exact).
+ * @param logSevereIfFailureToRemoveFileFlag
+ *   If true, log a message if failure to remove file
+ */
+bool
+SpecFile::removeFileFromSpecByName(const AString& filename,
+                                    const bool logSevereIfFailureToRemoveFileFlag)
+{
+    //exact match only
+    for (std::vector<SpecFileDataFileTypeGroup*>::const_iterator iter = dataFileTypeGroups.begin();
+            iter != dataFileTypeGroups.end();
+            iter++) {
+        SpecFileDataFileTypeGroup* dataFileTypeGroup = *iter;
+        const int32_t numFiles = dataFileTypeGroup->getNumberOfFiles();
+        for (int32_t i = 0; i < numFiles; i++) {
+            SpecFileDataFile* sfdf = dataFileTypeGroup->getFileInformation(i);
+            if (sfdf->getFileName() == filename) {
+                sfdf->setCaretDataFile(NULL);
+                sfdf->setSpecFileMember(false);
+                return true;
+            }
+        }
+    }
+    
+    if (logSevereIfFailureToRemoveFileFlag) {
+        CaretLogSevere("Failed to remove CaretDataFile by exact name from SpecFile: "
+                        + filename);
+    }
+    
+    return false;
+}
 
 /**
  * Add a data file to this spec file.
@@ -534,6 +686,9 @@ SpecFile::addDataFilePrivate(const DataFileTypeEnum::Enum dataFileType,
                     if (fileSavingSelectionStatus) {
                         sfdf->setSavingSelected(fileSavingSelectionStatus);
                     }
+                    if (specFileMemberStatus) {
+                        sfdf->setSpecFileMember(specFileMemberStatus);
+                    }
                     return sfdf;
                 }
             }
@@ -1482,40 +1637,53 @@ SpecFile::saveToScene(const SceneAttributes* sceneAttributes,
         else {
             const int32_t numFiles = group->getNumberOfFiles();
             for (int32_t j = 0; j < numFiles; j++) {
-                SpecFileDataFile* file = group->getFileInformation(j);
+                SpecFileDataFile* specFileDataFile = group->getFileInformation(j);
                 
-                /*
-                 * Only write files that are loaded (indicated by its
-                 * "caretDataFile" not NULL.
-                 */
-                const CaretDataFile* caretDataFile = file->getCaretDataFile();
-                if (caretDataFile != NULL) {
-                    bool addFileToSceneFlag = false;
-                    if (allLoadedFilesFlag) {
-                        addFileToSceneFlag = true;
-                    }
-                    else {
-                        if (displayedDataFiles.find(caretDataFile) != displayedDataFiles.end()) {
+                bool addFileToSceneFlag = false;
+                
+                if (specFileDataFile->isRemoveWhenSavingToScene()) {
+                    addFileToSceneFlag = false;
+                }
+                else if (sceneAttributes->isKeepAllFilesInScene()) {
+                    /*
+                     * The scene file update command adds file to the scene that are not loaded
+                     * so we must force adding of the filename to the scene's spec file
+                     */
+                    addFileToSceneFlag = true;
+                }
+                else {
+                    /*
+                     * Only write files that are loaded (indicated by its
+                     * "caretDataFile" not NULL.
+                     */
+                    const CaretDataFile* caretDataFile = specFileDataFile->getCaretDataFile();
+                    if (caretDataFile != NULL) {
+                        if (allLoadedFilesFlag) {
                             addFileToSceneFlag = true;
                         }
+                        else {
+                            if (displayedDataFiles.find(caretDataFile) != displayedDataFiles.end()) {
+                                addFileToSceneFlag = true;
+                            }
+                        }
                     }
+                }
+
+                if (addFileToSceneFlag) {
+                    SceneClass* fileClass = new SceneClass("specFileDataFile",
+                                                           "SpecFileDataFile",
+                                                           1);
+                    fileClass->addEnumeratedType<DataFileTypeEnum, DataFileTypeEnum::Enum>("dataFileType",
+                                                                                           dataFileType);
+                    fileClass->addEnumeratedType<StructureEnum, StructureEnum::Enum>("structure",
+                                                                                     specFileDataFile->getStructure());
+                    const AString name = updateFileNameAndPathForWriting(specFileDataFile->getFileName());
+                    fileClass->addPathName("fileName",
+                                           specFileDataFile->getFileName());
+                    fileClass->addBoolean("selected",
+                                          specFileDataFile->isLoadingSelected());
                     
-                    if (addFileToSceneFlag) {
-                        SceneClass* fileClass = new SceneClass("specFileDataFile",
-                                                               "SpecFileDataFile",
-                                                               1);
-                        fileClass->addEnumeratedType<DataFileTypeEnum, DataFileTypeEnum::Enum>("dataFileType", 
-                                                                                               dataFileType);
-                        fileClass->addEnumeratedType<StructureEnum, StructureEnum::Enum>("structure", 
-                                                                                         file->getStructure());
-                        const AString name = updateFileNameAndPathForWriting(file->getFileName());
-                        fileClass->addPathName("fileName", 
-                                               file->getFileName());
-                        fileClass->addBoolean("selected", 
-                                              file->isLoadingSelected());
-                        
-                        dataFileClasses.push_back(fileClass);
-                    }
+                    dataFileClasses.push_back(fileClass);
                 }
             }
         }
@@ -1775,11 +1943,88 @@ SpecFile::addToDataFileContentInformation(DataFileContentInformation& dataFileIn
 bool
 SpecFile::isDataFileTypeAllowedInSpecFile(const DataFileTypeEnum::Enum dataFileType)
 {
-    if (dataFileType == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC) {
-        return false;
+    bool allowedFlag(true);
+    
+    switch (dataFileType) {
+        case DataFileTypeEnum::ANNOTATION:
+            break;
+        case DataFileTypeEnum::ANNOTATION_TEXT_SUBSTITUTION:
+            break;
+        case DataFileTypeEnum::BORDER:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_DENSE:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC:
+            allowedFlag = false;
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_DENSE_LABEL:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_DENSE_PARCEL:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL_DENSE:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC:
+            allowedFlag = false;
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL_LABEL:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL_SCALAR:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_PARCEL_SERIES:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_DENSE_SCALAR:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_DENSE_TIME_SERIES:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_FIBER_ORIENTATIONS_TEMPORARY:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_TEMPORARY:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_MAPS:
+            break;
+        case DataFileTypeEnum::CONNECTIVITY_SCALAR_DATA_SERIES:
+            break;
+        case DataFileTypeEnum::CZI_IMAGE_FILE:
+            break;
+        case DataFileTypeEnum::FOCI:
+            break;
+        case DataFileTypeEnum::HISTOLOGY_SLICES:
+            break;
+        case DataFileTypeEnum::IMAGE:
+            break;
+        case DataFileTypeEnum::LABEL:
+            break;
+        case DataFileTypeEnum::METRIC:
+            break;
+        case DataFileTypeEnum::METRIC_DYNAMIC:
+            allowedFlag = false;
+            break;
+        case DataFileTypeEnum::OME_ZARR_IMAGE_FILE:
+            break;
+        case DataFileTypeEnum::PALETTE:
+            break;
+        case DataFileTypeEnum::RGBA:
+            break;
+        case DataFileTypeEnum::SAMPLES:
+            break;
+        case DataFileTypeEnum::SCENE:
+            break;
+        case DataFileTypeEnum::SPECIFICATION:
+            break;
+        case DataFileTypeEnum::SURFACE:
+            break;
+        case DataFileTypeEnum::UNKNOWN:
+            break;
+        case DataFileTypeEnum::VOLUME:
+            break;
+        case DataFileTypeEnum::VOLUME_DYNAMIC:
+            allowedFlag = false;
+            break;
     }
     
-    return true;
+    return allowedFlag;
 }
 
 
