@@ -18,6 +18,8 @@
  */
 /*LICENSE_END*/
 
+#include <algorithm>
+#include <cmath>
 #include <limits>
 #include <set>
 
@@ -28,14 +30,16 @@
 #include "DataFileTypeEnum.h"
 #include "SurfaceFile.h"
 #include "CaretAssert.h"
+#include "CaretLogger.h"
 #include "CaretOMP.h"
 #include "DataFileContentInformation.h"
 #include "DescriptiveStatistics.h"
 #include "FastStatistics.h"
 #include "EventSurfaceColoringInvalidate.h"
-
+#include "EventSurfaceFileGet.h"
 #include "GiftiFile.h"
 #include "GiftiMetaDataXmlElements.h"
+#include "GraphicsPrimitiveV3fN3fC4f.h"
 #include "MathFunctions.h"
 #include "Matrix4x4.h"
 #include "Vector3D.h"
@@ -58,6 +62,7 @@ SurfaceFile::SurfaceFile()
                               //which is used by setNumberOfNodesAndTriangles, which temporarily puts it the triangles into an invalid state, which is why this flag exists
     this->initializeMembersSurfaceFile();
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_SURFACE_COLORING_INVALIDATE);
+    EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_SURFACE_FILE_GET);
 }
 
 /**
@@ -110,6 +115,15 @@ SurfaceFile::~SurfaceFile()
     this->invalidateNodeColoringForBrowserTabs();
 }
 
+void SurfaceFile::writeFile(const AString& filename)
+{
+    if (!filename.endsWith(".surf.gii"))
+    {
+        CaretLogWarning("surface file '" + filename + "' should be saved ending in .surf.gii, see wb_command -gifti-help");
+    }
+    caret::GiftiTypeFile::writeFile(filename);
+}
+
 /**
  * Clear the surface file.
  */
@@ -202,17 +216,17 @@ SurfaceFile::validateDataArraysAfterReading()
         }
         
     }
-    
-    AString errorMessage;
     if (this->coordinateDataArray == NULL) {
-        errorMessage += "Unable to find coordinate data array which "
-            " contains data type FLOAT32, Intent POINTSET, and two "
-            " dimensions with the second dimension set to three.  ";
+        throw DataFileException(getFileName(),
+                                "Unable to find coordinate data array which "
+                                " contains data type FLOAT32, Intent POINTSET, and two "
+                                " dimensions with the second dimension set to three.  ");
     }
     if (this->triangleDataArray == NULL) {
-        errorMessage += "Unable to find topology data array which "
-        " contains data type INT32, Intent TRIANGLE, and two "
-        " dimensions with the second dimension set to three.";
+        throw DataFileException(getFileName(),
+                                "Unable to find topology data array which "
+                                " contains data type INT32, Intent TRIANGLE, and two "
+                                " dimensions with the second dimension set to three.");
     }
     const int32_t numNodes = this->getNumberOfNodes();
     if (!m_skipSanityCheck)
@@ -225,26 +239,22 @@ SurfaceFile::validateDataArraysAfterReading()
             {
                 if (thisTri[j] < 0 || thisTri[j] >= numNodes)
                 {
-                    errorMessage += "Invalid vertex in triangle array: triangle " + AString::number(i) + ", vertex " + AString::number(thisTri[j]);
-                    break;
+                    throw DataFileException(getFileName(),
+                                            "Invalid vertex in triangle array: triangle " + AString::number(i) + ", vertex " + AString::number(thisTri[j]));
                 }
                 for (int k = j + 1; k < 3; ++k)
                 {
                     if (thisTri[j] == thisTri[k])
                     {
-                        errorMessage += "Vertex used twice in one triangle: triangle " + AString::number(i) + ", vertex " + AString::number(thisTri[j]);
-                        break;
+                        throw DataFileException(getFileName(),
+                                                "Vertex used twice in one triangle: triangle " + AString::number(i) + ", vertex " + AString::number(thisTri[j]));
                     }
                 }
             }
         }
-        if (errorMessage.isEmpty() == false) {
-            throw DataFileException(getFileName(),
-                                    errorMessage);
-        }
     }
-    
-    this->computeNormals();
+
+    this->computeNormals(); //this will be garbage in the case that m_skipSanityCheck is used for, but we want it to be allocated to the right size, as multiple functions use it unchecked...
 
     /*
      * Apply the first transformation matrix that transforms to 
@@ -546,6 +556,25 @@ SurfaceFile::getNormalVector(const int32_t nodeIndex) const
     return &(this->normalVectors[offset]);    
 }
 
+/**
+ * Get a normal vector for a coordinate.
+ *
+ * @param nodeIndex
+ *    Index of coordinate.
+ * @param normalVectorOut
+ *    Output containing the normal vector for the node.
+ */
+void
+SurfaceFile::getNormalVector(const int32_t nodeIndex,
+                             float normalVectorOut[3]) const
+{
+    const int32_t offset = nodeIndex * 3;
+    CaretAssertVectorIndex(this->normalVectors, offset+2);
+    normalVectorOut[0] = this->normalVectors[offset];
+    normalVectorOut[1] = this->normalVectors[offset+1];
+    normalVectorOut[2] = this->normalVectors[offset+2];
+}
+
 const float* SurfaceFile::getNormalData() const
 {
     return normalVectors.data();
@@ -570,6 +599,14 @@ SurfaceFile::computeNormals()
     int32_t numCoords = this->getNumberOfNodes();
     if (numCoords > 0) {
         this->normalVectors.resize(numCoords * 3);
+
+        /*
+         * Need to fill with zeros so calculations are correct
+         * especially when normal get recomputed (resize does nothing)
+         */
+        std::fill(this->normalVectors.begin(),
+                  this->normalVectors.end(),
+                  0.0);
     }
     else {
         this->normalVectors.clear();
@@ -899,9 +936,200 @@ SurfaceFile::getBoundingBox() const
     return this->boundingBox;
 }
 
+/**
+ * Match the surface to the given anatomical surface.  If this
+ * surface is an anatomical or raw surface, no action is taken.
+ *
+ * @param anatomicalSurfaceFile
+ *     Match to this anatomical surface file.
+ * @param matchStatus
+ *     The match status
+ */
+void
+SurfaceFile::matchToAnatomicalSurface(const SurfaceFile* anatomicalSurfaceFile,
+                                      const bool matchStatus)
+{
+    CaretAssert(anatomicalSurfaceFile);
+    if (anatomicalSurfaceFile == NULL) {
+        return;
+    }
+    if (this == anatomicalSurfaceFile) {
+        return;
+    }
+    
+    if (matchStatus) {
+        bool sphereMatchFlag(false);
+        bool matchFlag(false);
+        switch (getSurfaceType()) {
+            case SurfaceTypeEnum::ANATOMICAL:
+                break;
+            case SurfaceTypeEnum::ELLIPSOID:
+                break;
+            case SurfaceTypeEnum::FLAT:
+                matchFlag = true;
+                break;
+            case SurfaceTypeEnum::HULL:
+                break;
+            case SurfaceTypeEnum::INFLATED:
+                matchFlag = true;
+                break;
+            case SurfaceTypeEnum::RECONSTRUCTION:
+                break;
+            case SurfaceTypeEnum::SEMI_SPHERICAL:
+                break;
+            case SurfaceTypeEnum::SPHERICAL:
+                sphereMatchFlag = true;
+                break;
+            case SurfaceTypeEnum::UNKNOWN:
+                break;
+            case SurfaceTypeEnum::VERY_INFLATED:
+                matchFlag = true;
+                break;
+        }
+        
+        if (matchFlag
+            || sphereMatchFlag) {
+            /*
+             * Save the unmatched coordinates
+             */
+            const int32_t numXYZ = getNumberOfNodes() * 3;
+            m_unmatchedCoordinates.resize(numXYZ);
+            CaretAssert(this->coordinatePointer);
+            std::copy_n(this->coordinatePointer,
+                        numXYZ,
+                        m_unmatchedCoordinates.begin());
+            
+            const bool modStatus(isModified());
+            if (sphereMatchFlag) {
+                matchSphereToSurface(anatomicalSurfaceFile);
+                if ( ! modStatus) {
+                    clearModified();
+                }
+            }
+            else if (matchFlag) {
+                matchSurfaceBoundingBox(anatomicalSurfaceFile);
+                if ( ! modStatus) {
+                    clearModified();
+                }
+            }
+        }
+    }
+    else {
+        /*
+         * Restore the unmatched coordinates
+         */
+        const int32_t numXYZ = getNumberOfNodes() * 3;
+        if (static_cast<int32_t>(m_unmatchedCoordinates.size()) == numXYZ) {
+            CaretAssert(this->coordinatePointer);
+            std::copy_n(m_unmatchedCoordinates.begin(),
+                        numXYZ,
+                        this->coordinatePointer);
+        }
+        m_unmatchedCoordinates.clear();
+    }
+}
+
+
+/**
+ * Match a sphere to the given surface while retaining the spherical
+ * shape.  The center of gravity of the sphere will be the same as the center of gravity of
+ * the match surface and the radius of the sphere will be the
+ * distance furthest from the origin in the match surface.
+ *
+ * @param matchSurfaceFile
+ *     Match to this surface file.
+ */
+void
+SurfaceFile::matchSphereToSurface(const SurfaceFile* matchSurfaceFile)
+{
+    CaretAssert(matchSurfaceFile);
+    
+    const float oldRadius = getSphericalRadius();
+    if (oldRadius <= 0.0) {
+        CaretLogWarning("Match sphere has invalid radius");
+        return;
+    }
+
+    /*
+     * Find match surface vertex distance furthest from origin (0,0,0)
+     * This value will become the radius of the sphere
+     */
+    CaretPointer<TopologyHelper> matchTH = matchSurfaceFile->getTopologyHelper();
+    const int32_t numMatchVertices = matchSurfaceFile->getNumberOfNodes();
+    float newRadius = 0.0;
+    for (int32_t i = 0; i < numMatchVertices; i++) {
+        if (matchTH->getNodeHasNeighbors(i)) {
+            const float* xyz = matchSurfaceFile->getCoordinate(i);
+            const float dist = ((xyz[0]*xyz[0])
+                                 + (xyz[1]*xyz[1])
+                                + (xyz[2]*xyz[2]));
+            if (dist > newRadius) {
+                newRadius = dist;
+            }
+        }
+    }
+    newRadius = std::sqrt(newRadius);
+    
+    float myCOG[3];
+    getCenterOfGravity(myCOG);
+    
+    float matchCOG[3];
+    matchSurfaceFile->getCenterOfGravity(matchCOG);
+    
+    
+    Matrix4x4 matrix;
+    
+    /*
+     * Scale to new radius
+     */
+    const float scale = newRadius / oldRadius;
+    matrix.scale(scale,
+                 scale,
+                 scale);
+
+    applyMatrix(matrix);
+}
+
+/**
+ * Get the center of gravity (average coordinate) of the surface.
+ *
+ * param cogOut
+ *     Output containing center of gravity.
+ */
+void
+SurfaceFile::getCenterOfGravity(float cogOut[3]) const
+{
+    cogOut[0] = 0.0;
+    cogOut[1] = 0.0;
+    cogOut[2] = 0.0;
+    
+    const int32_t numberOfNodes = getNumberOfNodes();
+    CaretPointer<TopologyHelper> th = this->getTopologyHelper();
+    
+    double numberOfNodesWithNeighbors = 0.0;
+    
+    double cx(0.0), cy(0.0), cz(0.0);
+    for (int32_t i = 0; i < numberOfNodes; i++) {
+        if (th->getNodeHasNeighbors(i)) {
+            const float* xyz = getCoordinate(i);
+            
+            cx += xyz[0];
+            cy += xyz[1];
+            cz += xyz[2];
+            numberOfNodesWithNeighbors += 1.0;
+        }
+    }
+    
+    if (numberOfNodesWithNeighbors > 0.0) {
+        cogOut[0] = cx / numberOfNodesWithNeighbors;
+        cogOut[1] = cy / numberOfNodesWithNeighbors;
+        cogOut[2] = cz / numberOfNodesWithNeighbors;
+    }
+}
+
 /**
  * Match this surface to the given surface.  That is, after this
- * method is called, this surface and the given surface will 
+ * method is called, this surface and the given surface will
  * fit within the same bounding box.
  * @param surfaceFile
  *     Match to this surface file.
@@ -920,18 +1148,18 @@ SurfaceFile::matchSurfaceBoundingBox(const SurfaceFile* surfaceFile)
      * Translate min x/y/z to origin
      */
     matrix.translate(-myBoundingBox->getMinX(),
-                           -myBoundingBox->getMinY(),
-                           -myBoundingBox->getMinZ());
+                     -myBoundingBox->getMinY(),
+                     -myBoundingBox->getMinZ());
     
     /*
      * Scale to match size of match surface
      */
     float scaleX = (targetBoundingBox->getDifferenceX()
-                          / myBoundingBox->getDifferenceX());
+                    / myBoundingBox->getDifferenceX());
     float scaleY = (targetBoundingBox->getDifferenceY()
-                          / myBoundingBox->getDifferenceY());
+                    / myBoundingBox->getDifferenceY());
     float scaleZ = (targetBoundingBox->getDifferenceZ()
-                          / myBoundingBox->getDifferenceZ());
+                    / myBoundingBox->getDifferenceZ());
     
     if (getSurfaceType() == SurfaceTypeEnum::FLAT) {
         /*
@@ -947,16 +1175,16 @@ SurfaceFile::matchSurfaceBoundingBox(const SurfaceFile* surfaceFile)
     }
     
     matrix.scale(scaleX,
-                      scaleY,
-                      scaleZ);
+                 scaleY,
+                 scaleZ);
     
     /*
      * Translate to min x/y/z of match surface so that
      * the two surfaces are now within the same "shoebox".
      */
     matrix.translate(targetBoundingBox->getMinX(),
-                              targetBoundingBox->getMinY(),
-                              targetBoundingBox->getMinZ());
+                     targetBoundingBox->getMinY(),
+                     targetBoundingBox->getMinZ());
     
     applyMatrix(matrix);
 }
@@ -1340,7 +1568,11 @@ SurfaceFile::invalidateNodeColoringForBrowserTabs()
         this->surfaceNodeColoringForBrowserTabs[i].clear();
         this->surfaceMontageNodeColoringForBrowserTabs[i].clear();
         this->wholeBrainNodeColoringForBrowserTabs[i].clear();
-    }    
+    }
+    
+    m_surfaceGraphicsPrimitives.clear();
+    m_surfaceMontageGraphicsPrimitives.clear();
+    m_wholeBrainGraphicsPrimitives.clear();
 }
 
 /**
@@ -1469,6 +1701,11 @@ SurfaceFile::setSurfaceNodeColoringRgbaForBrowserTab(const int32_t browserTabInd
     for (int32_t i = 0; i < numberOfComponentsRGBA; i++) {
         rgba[i] = rgbaNodeColorComponents[i];
     }
+    
+    if ((browserTabIndex >= 0)
+        && (browserTabIndex < static_cast<int32_t>(m_surfaceGraphicsPrimitives.size()))) {
+        m_surfaceGraphicsPrimitives[browserTabIndex].reset();
+    }
 }
 
 /**
@@ -1516,6 +1753,11 @@ SurfaceFile::setSurfaceMontageNodeColoringRgbaForBrowserTab(const int32_t browse
     for (int32_t i = 0; i < numberOfComponentsRGBA; i++) {
         rgba[i] = rgbaNodeColorComponents[i];
     }
+    
+    if ((browserTabIndex >= 0)
+        && (browserTabIndex < static_cast<int32_t>(m_surfaceMontageGraphicsPrimitives.size()))) {
+        m_surfaceMontageGraphicsPrimitives[browserTabIndex].reset();
+    }
 }
 
 
@@ -1563,6 +1805,121 @@ SurfaceFile::setWholeBrainNodeColoringRgbaForBrowserTab(const int32_t browserTab
     for (int32_t i = 0; i < numberOfComponentsRGBA; i++) {
         rgba[i] = rgbaNodeColorComponents[i];
     }
+    
+    if ((browserTabIndex >= 0)
+        && (browserTabIndex < static_cast<int32_t>(m_wholeBrainGraphicsPrimitives.size()))) {
+        m_wholeBrainGraphicsPrimitives[browserTabIndex].reset();
+    }
+}
+
+/**
+ * @return the graphics primitive for drawing this surface for a single surface view
+ * in the given tab index
+ * @param browserTabIndex
+ *    Index of the tab
+ */
+GraphicsPrimitiveV3fN3fC4f*
+SurfaceFile::getSurfaceGraphicsPrimitiveForBrowserTab(const int32_t browserTabIndex)
+{
+    const float* allRGBA(getSurfaceNodeColoringRgbaForBrowserTab(browserTabIndex));
+    GraphicsPrimitiveV3fN3fC4f* primitiveOut(getGraphicsPrimitive(m_surfaceGraphicsPrimitives,
+                                                                  browserTabIndex,
+                                                                  allRGBA));
+    return primitiveOut;
+}
+
+/**
+ * @return the graphics primitive for drawing this surface for a single surface view
+ * in the given tab index
+ * @param primitives
+ *    Primitives for each tab index
+ * @param browserTabIndex
+ *    Index of the tab
+ * @param rgba
+ *    The RGBA coloring for the surface
+ */
+GraphicsPrimitiveV3fN3fC4f*
+SurfaceFile::getGraphicsPrimitive(std::vector<std::unique_ptr<GraphicsPrimitiveV3fN3fC4f>>& primitives,
+                                                                     const int32_t browserTabIndex,
+                                                                     const float* rgba)
+{
+    GraphicsPrimitiveV3fN3fC4f* primitiveOut(NULL);
+    
+    if ((browserTabIndex >= 0)
+        && (browserTabIndex < static_cast<int32_t>(primitives.size()))) {
+        primitiveOut = primitives[browserTabIndex].get();
+    }
+    
+    if (primitiveOut == NULL) {
+        primitiveOut = createSurfaceGraphicsPrimitive(rgba);
+        
+        if (primitiveOut != NULL) {
+            if (browserTabIndex >= static_cast<int32_t>(primitives.size())) {
+                primitives.resize(browserTabIndex + 1);
+            }
+            primitives[browserTabIndex].reset(primitiveOut);
+        }
+    }
+    
+    return primitiveOut;
+
+}
+
+/*
+ * @return Graphics primitive for drawing this surface in the given RGBA colors
+ * @param rgba
+ *     The RGBA coloring
+ * @return
+ *     The graphics primitive
+ */
+GraphicsPrimitiveV3fN3fC4f*
+SurfaceFile::createSurfaceGraphicsPrimitive(const float* rgba)
+{
+    GraphicsPrimitiveV3fN3fC4f* primitiveOut(GraphicsPrimitive::newPrimitiveV3fN3fC4f(GraphicsPrimitive::PrimitiveType::OPENGL_TRIANGLES));
+    
+    const int32_t numberOfTriangles(getNumberOfTriangles());
+    for (int32_t i = 0; i < numberOfTriangles; i++) {
+        const int32_t* triangleIndices(getTriangle(i));
+        for (int32_t j = 0; j < 3; j++) {
+            const int32_t vertexIndex(triangleIndices[j]);
+            const float* xyz(getCoordinate(vertexIndex));
+            const float* normalXYZ(getNormalVector(vertexIndex));
+            const float* vertexRgba(&rgba[vertexIndex * 4]);
+            primitiveOut->addVertex(xyz, normalXYZ, vertexRgba);
+        }
+    }
+    return primitiveOut;
+}
+/**
+ * @return the graphics primitive for drawing this surface for a  surface montage view
+ * in the given tab index
+ * @param browserTabIndex
+ *    Index of the tab
+ */
+GraphicsPrimitiveV3fN3fC4f*
+SurfaceFile::getSurfaceMontageGraphicsPrimitiveForBrowserTab(const int32_t browserTabIndex)
+{
+    const float* allRGBA(getSurfaceMontageNodeColoringRgbaForBrowserTab(browserTabIndex));
+    GraphicsPrimitiveV3fN3fC4f* primitiveOut(getGraphicsPrimitive(m_surfaceMontageGraphicsPrimitives,
+                                                                  browserTabIndex,
+                                                                  allRGBA));
+    return primitiveOut;
+}
+
+/**
+ * @return the graphics primitive for drawing this surface for a whole brain view
+ * in the given tab index
+ * @param browserTabIndex
+ *    Index of the tab
+ */
+GraphicsPrimitiveV3fN3fC4f*
+SurfaceFile::getWholeBrainGraphicsPrimitiveForBrowserTab(const int32_t browserTabIndex)
+{
+    const float* allRGBA(getWholeBrainNodeColoringRgbaForBrowserTab(browserTabIndex));
+    GraphicsPrimitiveV3fN3fC4f* primitiveOut(getGraphicsPrimitive(m_wholeBrainGraphicsPrimitives,
+                                                                  browserTabIndex,
+                                                                  allRGBA));
+    return primitiveOut;
 }
 
 /**
@@ -1582,7 +1939,17 @@ SurfaceFile::receiveEvent(Event* event)
         invalidateEvent->setEventProcessed();
         
         this->invalidateNodeColoringForBrowserTabs();
-    }    
+    }  
+    else if (event->getEventType() == EventTypeEnum::EVENT_SURFACE_FILE_GET) {
+        EventSurfaceFileGet* surfaceFileEvent(dynamic_cast<EventSurfaceFileGet*>(event));
+        CaretAssert(surfaceFileEvent);
+        
+        if ((getNumberOfNodes() == surfaceFileEvent->getNumberOfVertices())
+            && (getStructure()  == surfaceFileEvent->getStructure())) {
+            surfaceFileEvent->addSurfaceFile(this);
+            surfaceFileEvent->setEventProcessed();
+        }
+    }
 }
 
 bool SurfaceFile::matchesTopology(const SurfaceFile& rhs) const
