@@ -194,6 +194,30 @@ SurfaceProjectedItem::unprojectToVolumeXYZ(const SurfaceFile& sf,
     }
 }
 
+/**
+ * Get the projected position of this item.
+ * The first valid of this positions is used: (1) Barycentric,
+ * (2) VanEssen, (3) Stereotaxic.
+ *
+ * @param surfaceFile
+ *     Surface File for positioning.
+ * @param xyzOut
+ *     Output containing the projected position.
+ * @param pasteOntoSurfaceFlag
+ *     Place directly on the surface.
+ * @return  true if the position is valid, else false.
+ *
+ */
+bool
+SurfaceProjectedItem::getProjectedPosition(const SurfaceFile& surfaceFile,
+                                           float xyzOut[3],
+                                           const bool isUnprojectedOntoSurface) const
+{
+    return getProjectedPosition(&surfaceFile,
+                                xyzOut,
+                                isUnprojectedOntoSurface);
+}
+
 /**
  * Get the projected position of this item.
  * The first valid of this positions is used: (1) Barycentric,
@@ -209,31 +233,33 @@ SurfaceProjectedItem::unprojectToVolumeXYZ(const SurfaceFile& sf,
  *
  */
 bool
-SurfaceProjectedItem::getProjectedPosition(const SurfaceFile& surfaceFile,
+SurfaceProjectedItem::getProjectedPosition(const SurfaceFile* surfaceFile,
                                            float xyzOut[3],
                                            const bool isUnprojectedOntoSurface) const
 {
     bool valid = false;
     
-    if (valid == false) {
-        if (this->barycentricProjection->isValid()) {
-            valid = this->barycentricProjection->unprojectToSurface(surfaceFile, 
-                                                                    xyzOut, 
-                                                                    0.0,
-                                                                    isUnprojectedOntoSurface);
+    if (surfaceFile != NULL) {
+        if ( ! valid) {
+            if (this->barycentricProjection->isValid()) {
+                valid = this->barycentricProjection->unprojectToSurface(*surfaceFile,
+                                                                        xyzOut,
+                                                                        0.0,
+                                                                        isUnprojectedOntoSurface);
+            }
+        }
+        
+        if ( ! valid) {
+            if (this->vanEssenProjection->isValid()) {
+                valid = this->vanEssenProjection->unprojectToSurface(*surfaceFile,
+                                                                     xyzOut,
+                                                                     0.0,
+                                                                     isUnprojectedOntoSurface);
+            }
         }
     }
     
-    if (valid == false) {
-        if (this->vanEssenProjection->isValid()) {
-            valid = this->vanEssenProjection->unprojectToSurface(surfaceFile, 
-                                                                 xyzOut, 
-                                                                 0.0,
-                                                                 isUnprojectedOntoSurface);
-        }
-    }
-    
-    if (valid == false) {
+    if ( ! valid) {
         if (this->stereotaxicXYZValid) {
             this->getStereotaxicXYZ(xyzOut);
             valid = true;
@@ -253,6 +279,8 @@ SurfaceProjectedItem::getProjectedPosition(const SurfaceFile& surfaceFile,
  * 
  * @param surfaceFile  
  *     Surface File for positioning.
+ * @param topologyHelper
+ *     The topology helper.  This value
  * @param xyzOut
  *     Output containing the projected position.
  * @param distanceAboveSurface   
@@ -262,6 +290,7 @@ SurfaceProjectedItem::getProjectedPosition(const SurfaceFile& surfaceFile,
  */
 bool 
 SurfaceProjectedItem::getProjectedPositionAboveSurface(const SurfaceFile& surfaceFile,
+                                                       const TopologyHelper* topologyHelper,
                                                        float xyzOut[3],
                                                        const float distanceAboveSurface) const
 {
@@ -269,7 +298,8 @@ SurfaceProjectedItem::getProjectedPositionAboveSurface(const SurfaceFile& surfac
     
     if (valid == false) {
         if (this->barycentricProjection->isValid()) {
-            valid = this->barycentricProjection->unprojectToSurface(surfaceFile, 
+            valid = this->barycentricProjection->unprojectToSurface(surfaceFile,
+                                                                    topologyHelper,
                                                                     xyzOut, 
                                                                     distanceAboveSurface,
                                                                     true);
@@ -278,7 +308,8 @@ SurfaceProjectedItem::getProjectedPositionAboveSurface(const SurfaceFile& surfac
     
     if (valid == false) {
         if (this->vanEssenProjection->isValid()) {
-            valid = this->vanEssenProjection->unprojectToSurface(surfaceFile, 
+            valid = this->vanEssenProjection->unprojectToSurface(surfaceFile,
+                                                                 topologyHelper,
                                                                  xyzOut, 
                                                                  distanceAboveSurface,
                                                                  true);
@@ -367,9 +398,7 @@ SurfaceProjectedItem::getVolumeXYZ() const
     /*
      * If not set, return stereotaxic coordinate
      */
-    if ((volumeXYZ[0] == 0.0)
-        && (volumeXYZ[1] == 0.0)
-        && (volumeXYZ[2] == 0.0)) {
+    if ( ! isVolumeXYZValid()) {
         const float* stereoXYZ = getStereotaxicXYZ();
         return stereoXYZ;
     }
@@ -387,9 +416,7 @@ SurfaceProjectedItem::getVolumeXYZ(float xyzOut[3]) const
     /*
      * If not set, return stereotaxic coordinate
      */
-    if ((volumeXYZ[0] == 0.0)
-        && (volumeXYZ[1] == 0.0)
-        && (volumeXYZ[2] == 0.0)) {
+    if ( ! isVolumeXYZValid()) {
         getStereotaxicXYZ(xyzOut);
         return;
     }
@@ -427,6 +454,21 @@ SurfaceProjectedItem::setVolumeXYZ(const float volumeXYZ[3])
     this->setModified();
 }
 
+/**
+ * Set the stereotaxic coordinate and override any previous projection
+ * @param stereotaxicXYZ
+ *    New coordinate
+ */
+void
+SurfaceProjectedItem::setStereotaxicXYZWhileMovingWithMouse(const float stereotaxicXYZ[3])
+{
+    setStereotaxicXYZ(stereotaxicXYZ);
+    setVolumeXYZ(stereotaxicXYZ);
+    setStructure(StructureEnum::INVALID);
+    getBarycentricProjection()->reset();
+    getVanEssenProjection()->reset();
+}
+
 /**
  * Get the structure of this projected item.
  * @return The structure.
@@ -492,7 +534,7 @@ SurfaceProjectedItem::getVanEssenProjection() const
  *   Writer for XML output.
  */
 void 
-SurfaceProjectedItem::writeAsXML(XmlWriter& xmlWriter) throw (XmlException)
+SurfaceProjectedItem::writeAsXML(XmlWriter& xmlWriter)
 {
     xmlWriter.writeStartElement(XML_TAG_SURFACE_PROJECTED_ITEM);
     xmlWriter.writeElementCharacters(XML_TAG_STRUCTURE, StructureEnum::toName(this->structure));
@@ -510,7 +552,7 @@ SurfaceProjectedItem::writeAsXML(XmlWriter& xmlWriter) throw (XmlException)
 void SurfaceProjectedItem::readBorderFileXML1(QXmlStreamReader& xml)
 {
     reset();
-    CaretAssert(xml.isStartElement() && xml.name() == "SurfaceProjectedItem");
+    CaretAssert(xml.isStartElement() && xml.name() == QLatin1String("SurfaceProjectedItem"));
     bool haveStructure = false, haveVanEssen = false, haveStereo = false, haveVolume = false;//track the barycentric projection for being specified more than once by its valid flag
     for (xml.readNext(); !xml.atEnd() && !xml.isEndElement(); xml.readNext())
     {
@@ -518,8 +560,8 @@ void SurfaceProjectedItem::readBorderFileXML1(QXmlStreamReader& xml)
         {
             case QXmlStreamReader::StartElement:
             {
-                QStringRef name = xml.name();
-                if (name == "Structure")
+                auto name = xml.name();
+                if (name == QLatin1String("Structure"))
                 {
                     if (haveStructure) throw DataFileException("multiple Structure elements in one SurfaceProjectedItem element");
                     QString structString = xml.readElementText();//sets error on unexpected child element
@@ -528,22 +570,22 @@ void SurfaceProjectedItem::readBorderFileXML1(QXmlStreamReader& xml)
                     structure = StructureEnum::fromName(structString, &ok);
                     if (!ok) CaretLogWarning("unrecognized string in Structure: " + structString);//HACK: this is what the SAX reader did, don't look at me
                     haveStructure = true;
-                } else if (name == "ProjectionBarycentric") {
+                } else if (name == QLatin1String("ProjectionBarycentric")) {
                     if (barycentricProjection->isValid()) throw DataFileException("multiple ProjectionBarycentric elements in one SurfaceProjectedItem element");
                     barycentricProjection->readBorderFileXML1(xml);
-                } else if (name == "VanEssenProjection") {
+                } else if (name == QLatin1String("VanEssenProjection")) {
                     if (haveVanEssen) throw DataFileException("multiple VanEssenProjection elements in one SurfaceProjectedItem element");
                     CaretLogFine("found Van Essen projection in border file, ignoring");
                     xml.readElementText(QXmlStreamReader::SkipChildElements);//HACK: border files never use this projection type, so don't try to parse it
                     if (xml.hasError()) throw DataFileException("XML parsing error in VanEssenProjection: " + xml.errorString());
                     haveVanEssen = true;
-                } else if (name == "StereotaxicXYZ") {
+                } else if (name == QLatin1String("StereotaxicXYZ")) {
                     if (haveStereo) throw DataFileException("multiple StereotaxicXYZ elements in one SurfaceProjectedItem element");
                     CaretLogFine("found stereotaxic coordinates in border file, ignoring");
                     xml.readElementText(QXmlStreamReader::SkipChildElements);//HACK: ditto
                     if (xml.hasError()) throw DataFileException("XML parsing error in StereotaxicXYZ: " + xml.errorString());
                     haveStereo = true;
-                } else if (name == "VolumeXYZ") {
+                } else if (name == QLatin1String("VolumeXYZ")) {
                     if (haveVolume) throw DataFileException("multiple VolumeXYZ elements in one SurfaceProjectedItem element");
                     CaretLogFine("found volume coordinates in border file, ignoring");
                     xml.readElementText(QXmlStreamReader::SkipChildElements);//HACK: ditto
@@ -559,7 +601,7 @@ void SurfaceProjectedItem::readBorderFileXML1(QXmlStreamReader& xml)
         }
     }
     if (xml.hasError()) throw DataFileException("XML parsing error in SurfaceProjectedItem: " + xml.errorString());
-    CaretAssert(xml.isEndElement() && xml.name() == "SurfaceProjectedItem");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("SurfaceProjectedItem"));
     if (!haveStructure) throw DataFileException("SurfaceProjectedItem is missing Structure element");
 }
 
