@@ -30,6 +30,7 @@
 #include "TopologyHelper.h"
 #include "XmlWriter.h"
 
+#include <QRegularExpression>
 #include <QXmlStreamReader>
 #include <QStringList>
 
@@ -222,6 +223,219 @@ SurfaceProjectionBarycentric::setTriangleAreas(const float triangleAreas[3])
     this->setModified();
 }
 
+/**
+ * Unproject to the surface using 'this' projection.
+ *
+ * @param surfaceFile
+ *    Surface file used for unprojecting.
+ * @param topologyHelperIn
+ *    Topology helper.  If NULL, topology helper from surfaceFile
+ *    will be used but frequent calls to get the topology helper
+ *    may be slow.
+ * @param xyzOut
+ *    Output containing coordinate created by unprojecting.
+ * @param offsetFromSurface
+ *    If 'unprojectWithOffsetFromSurface' is true, unprojected
+ *    position will be this distance above (negative=below)
+ *    the surface.
+ * @param unprojectWithOffsetFromSurface
+ *    If true, ouput coordinate will be offset 'offsetFromSurface'
+ *    distance from the surface.
+ * @return
+ *    True if unprojection was successful.
+ */
+bool
+SurfaceProjectionBarycentric::unprojectToSurface(const SurfaceFile& surfaceFile,
+                                                 const TopologyHelper* topologyHelperIn,
+                                                 float xyzOut[3],
+                                                 const float offsetFromSurface,
+                                                 const bool unprojectWithOffsetFromSurface) const
+{
+    /*
+     * Make sure projection surface number of nodes matches surface.
+     */
+    if (this->projectionSurfaceNumberOfNodes > 0) {
+        if (surfaceFile.getNumberOfNodes() != this->projectionSurfaceNumberOfNodes) {
+            return false;
+        }
+    }
+    
+    const int32_t n1 = this->triangleNodes[0];
+    const int32_t n2 = this->triangleNodes[1];
+    const int32_t n3 = this->triangleNodes[2];
+    
+    CaretAssert(n1 < surfaceFile.getNumberOfNodes());
+    CaretAssert(n2 < surfaceFile.getNumberOfNodes());
+    CaretAssert(n3 < surfaceFile.getNumberOfNodes());
+    
+    /*
+     * All nodes MUST have neighbors (connected)
+     */
+    const TopologyHelper* topologyHelper = ((topologyHelperIn != NULL)
+                                            ? topologyHelperIn
+                                            : surfaceFile.getTopologyHelper().getPointer());
+    if ((topologyHelper->getNodeHasNeighbors(n1) == false)
+        || (topologyHelper->getNodeHasNeighbors(n2) == false)
+        || (topologyHelper->getNodeHasNeighbors(n3) == false)) {
+        return false;
+    }
+    
+    const float* c1 = surfaceFile.getCoordinate(n1);
+    const float* c2 = surfaceFile.getCoordinate(n2);
+    const float* c3 = surfaceFile.getCoordinate(n3);
+    
+    float barycentricXYZ[3];
+    float barycentricNormal[3];
+    
+    /*
+     * If all the nodes are the same (object projects to a single node, not triangle)
+     */
+    if ((n1 == n2) &&
+        (n2 == n3)) {
+        /*
+         * Use node's normal vector and position
+         */
+        barycentricXYZ[0] = c1[0];
+        barycentricXYZ[1] = c1[1];
+        barycentricXYZ[2] = c1[2];
+        const float* nodeNormal = surfaceFile.getNormalVector(n1);
+        barycentricNormal[0] = nodeNormal[0];
+        barycentricNormal[1] = nodeNormal[1];
+        barycentricNormal[2] = nodeNormal[2];
+    }
+    else {
+        /*
+         * Compute position using barycentric coordinates
+         */
+        float t1[3];
+        float t2[3];
+        float t3[3];
+        for (int i = 0; i < 3; i++) {
+            t1[i] = triangleAreas[0] * c1[i];
+            t2[i] = triangleAreas[1] * c2[i];
+            t3[i] = triangleAreas[2] * c3[i];
+        }
+        float area = (triangleAreas[0]
+                      + triangleAreas[1]
+                      + triangleAreas[2]);
+        if (area != 0) {
+            for (int i = 0; i < 3; i++) {
+                barycentricXYZ[i] = (t1[i] + t2[i] + t3[i]) / area;
+            }
+        }
+        else {
+            return false;
+        }
+        
+        if (MathFunctions::normalVector(c1, c2, c3, barycentricNormal) == false) {
+            return false;
+        }
+    }
+    
+    /*
+     * Set output coordinate, possibly offsetting from surface.
+     */
+    for (int j = 0; j < 3; j++) {
+        if (unprojectWithOffsetFromSurface) {
+            xyzOut[j] = (barycentricXYZ[j]
+                         + (barycentricNormal[j] * offsetFromSurface));
+        }
+        else {
+            xyzOut[j] = (barycentricXYZ[j]
+                         + (barycentricNormal[j] * signedDistanceAboveSurface));
+        }
+    }
+    
+    return true;
+}
+
+/**
+ * Unproject to surface
+ * @param surfaceFile
+ *    Unproject to this surface
+ * @param barycentricAreas
+ *    The barycentric areas
+ * @param barycentricVertices
+ *    The vertices
+ * @param xyzOut
+ *    Output with unprojected to XYZ coordinates
+ * @return True if valid
+ */
+bool
+SurfaceProjectionBarycentric::unprojectToSurface(const SurfaceFile* surfaceFile,
+                                                 const float barycentricAreas[3],
+                                                 const int32_t barycentricVertices[3],
+                                                 float xyzOut[3])
+{
+    CaretAssert(surfaceFile);
+
+    xyzOut[0] = 0.0;
+    xyzOut[1] = 0.0;
+    xyzOut[2] = 0.0;
+    const int32_t n1(barycentricVertices[0]);
+    const int32_t n2(barycentricVertices[1]);
+    const int32_t n3(barycentricVertices[2]);
+    
+    if ((n1>= 0) && (n1 < surfaceFile->getNumberOfNodes())
+        && (n2 >= 0) && (n2 < surfaceFile->getNumberOfNodes())
+        && (n3 >= 0) && (n3 < surfaceFile->getNumberOfNodes())) {
+        /*
+         * All nodes MUST have neighbors (connected)
+         */
+        const TopologyHelper* topologyHelper = surfaceFile->getTopologyHelper().getPointer();
+        if ((topologyHelper->getNodeHasNeighbors(n1) == false)
+            || (topologyHelper->getNodeHasNeighbors(n2) == false)
+            || (topologyHelper->getNodeHasNeighbors(n3) == false)) {
+            return false;
+        }
+        
+        const float* xyz1 = surfaceFile->getCoordinate(n1);
+        const float* xyz2 = surfaceFile->getCoordinate(n2);
+        const float* xyz3 = surfaceFile->getCoordinate(n3);
+
+        /*
+         * If all the nodes are the same (object projects to a single node, not triangle)
+         */
+        if ((n1 == n2) &&
+            (n2 == n3)) {
+            /*
+             * Use node's normal vector and position
+             */
+            xyzOut[0] = xyz1[0];
+            xyzOut[1] = xyz1[1];
+            xyzOut[2] = xyz1[2];
+        }
+        else {
+            /*
+             * Compute position using barycentric coordinates
+             */
+            float t1[3];
+            float t2[3];
+            float t3[3];
+            for (int i = 0; i < 3; i++) {
+                t1[i] = barycentricAreas[0] * xyz1[i];
+                t2[i] = barycentricAreas[1] * xyz2[i];
+                t3[i] = barycentricAreas[2] * xyz3[i];
+            }
+            float area = (barycentricAreas[0]
+                          + barycentricAreas[1]
+                          + barycentricAreas[2]);
+            if (area != 0) {
+                for (int i = 0; i < 3; i++) {
+                    xyzOut[i] = (t1[i] + t2[i] + t3[i]) / area;
+                }
+            }
+            else {
+                return false;
+            }
+        }
+        
+        return true;
+    }
+    
+    return false;
+}
+
 /**
  * Unproject to the surface using 'this' projection.
  * 
@@ -441,7 +655,7 @@ SurfaceProjectionBarycentric::writeAsXML(XmlWriter& xmlWriter)
 void SurfaceProjectionBarycentric::readBorderFileXML1(QXmlStreamReader& xml)
 {
     reset();
-    CaretAssert(xml.isStartElement() && xml.name() == "ProjectionBarycentric");
+    CaretAssert(xml.isStartElement() && xml.name() == QLatin1String("ProjectionBarycentric"));
     bool haveAreas = false, haveNodes = false, haveDist = false;
     for (xml.readNext(); !xml.atEnd() && !xml.isEndElement(); xml.readNext())
     {
@@ -449,13 +663,17 @@ void SurfaceProjectionBarycentric::readBorderFileXML1(QXmlStreamReader& xml)
         {
             case QXmlStreamReader::StartElement:
             {
-                QStringRef name = xml.name();
-                if (name == "TriangleAreas")
+                auto name = xml.name();
+                if (name == QLatin1String("TriangleAreas"))
                 {
                     if (haveAreas) throw DataFileException("multiple TriangleAreas elements in one ProjectionBarycentric element");
                     QString text = xml.readElementText();//errors on unexpected element
                     if (xml.hasError()) throw DataFileException("XML parsing error in TriangleAreas: " + xml.errorString());
-                    QStringList areaStrings = text.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+#if QT_VERSION >= 0x060000
+                    QStringList areaStrings = text.split(QRegularExpression("\\s+"), Qt::SkipEmptyParts);
+#else
+                    QStringList areaStrings = text.split(QRegularExpression("\\s+"), QString::SkipEmptyParts);
+#endif
                     if (areaStrings.size() != 3) throw DataFileException("TriangleAreas element must contain 3 numbers separated by whitespace");
                     bool ok = false;
                     for (int i = 0; i < 3; ++i)
@@ -464,11 +682,15 @@ void SurfaceProjectionBarycentric::readBorderFileXML1(QXmlStreamReader& xml)
                         if (!ok) throw DataFileException("found non-numeric string in TriangleAreas: " + areaStrings[i]);
                     }
                     haveAreas = true;
-                } else if (name == "TriangleNodes") {
+                } else if (name == QLatin1String("TriangleNodes")) {
                     if (haveNodes) throw DataFileException("multiple TriangleNodes elements in one ProjectionBarycentric element");
                     QString text = xml.readElementText();//errors on unexpected element
                     if (xml.hasError()) throw DataFileException("XML parsing error in TriangleNodes: " + xml.errorString());
-                    QStringList nodeStrings = text.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+#if QT_VERSION >= 0x060000
+                    QStringList nodeStrings = text.split(QRegularExpression("\\s+"), Qt::SkipEmptyParts);
+#else
+                    QStringList nodeStrings = text.split(QRegularExpression("\\s+"), QString::SkipEmptyParts);
+#endif
                     if (nodeStrings.size() != 3) throw DataFileException("TriangleNodes element must contain 3 integers separated by whitespace");
                     bool ok = false;
                     for (int i = 0; i < 3; ++i)
@@ -477,7 +699,7 @@ void SurfaceProjectionBarycentric::readBorderFileXML1(QXmlStreamReader& xml)
                         if (!ok) throw DataFileException("found non-integer string in TriangleNodes: " + nodeStrings[i]);
                     }
                     haveNodes = true;
-                } else if (name == "SignedDistanceAboveSurface") {
+                } else if (name == QLatin1String("SignedDistanceAboveSurface")) {
                     if (haveDist) throw DataFileException("multiple SignedDistanceAboveSurface elements in one ProjectionBarycentric element");
                     QString text = xml.readElementText();//errors on unexpected element
                     if (xml.hasError()) throw DataFileException("XML parsing error in SignedDistanceAboveSurface: " + xml.errorString());
@@ -495,7 +717,7 @@ void SurfaceProjectionBarycentric::readBorderFileXML1(QXmlStreamReader& xml)
         }
     }
     if (xml.hasError()) throw DataFileException("XML parsing error in ProjectionBarycentric: " + xml.errorString());
-    CaretAssert(xml.isEndElement() && xml.name() == "ProjectionBarycentric");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("ProjectionBarycentric"));
     if (!haveAreas || !haveNodes)//ignore missing distance? should always be zero for BorderFile anyway
     {
         throw DataFileException("SurfaceProjectionBarycentric element missing TriangleNodes and/or TriangleAreas");
