@@ -159,7 +159,7 @@ SurfaceProjector::setSurfaceOffset(const float surfaceOffset)
  *      If projecting an item failed.
  */
 void
-SurfaceProjector::projectFociFile(FociFile* fociFile) throw (SurfaceProjectorException)
+SurfaceProjector::projectFociFile(FociFile* fociFile)
 {
     CaretAssert(fociFile);
     const int32_t numberOfFoci = fociFile->getNumberOfFoci();
@@ -205,7 +205,7 @@ SurfaceProjector::projectFociFile(FociFile* fociFile) throw (SurfaceProjectorExc
  */
 void
 SurfaceProjector::projectFocus(const int32_t focusIndex,
-                               Focus* focus) throw (SurfaceProjectorException)
+                               Focus* focus)
 {
     const int32_t numberOfProjections = focus->getNumberOfProjections();
     CaretAssert(numberOfProjections > 0);
@@ -259,7 +259,7 @@ SurfaceProjector::projectFocus(const int32_t focusIndex,
  *      If projecting an item failed.
  */
 void
-SurfaceProjector::projectItemToTriangle(SurfaceProjectedItem* spi) throw (SurfaceProjectorException)
+SurfaceProjector::projectItemToTriangle(SurfaceProjectedItem* spi)
 {
     CaretAssert(spi);
     m_allowEdgeProjection = false;
@@ -280,7 +280,7 @@ SurfaceProjector::projectItemToTriangle(SurfaceProjectedItem* spi) throw (Surfac
  *      If projecting an item failed.
  */
 void
-SurfaceProjector::projectItemToTriangleOrEdge(SurfaceProjectedItem* spi) throw (SurfaceProjectorException)
+SurfaceProjector::projectItemToTriangleOrEdge(SurfaceProjectedItem* spi)
 {
     CaretAssert(spi);
     m_allowEdgeProjection = true;
@@ -305,7 +305,7 @@ SurfaceProjector::projectItemToTriangleOrEdge(SurfaceProjectedItem* spi) throw (
  */
 void
 SurfaceProjector::projectItem(SurfaceProjectedItem* spi,
-                              SurfaceProjectedItem* secondSpi) throw (SurfaceProjectorException)
+                              SurfaceProjectedItem* secondSpi)
 {
     m_projectionWarning = "";
     
@@ -433,7 +433,8 @@ SurfaceProjector::projectItem(SurfaceProjectedItem* spi,
             }
             
             if (nearestSurfaceIndex < 0) {
-                throw SurfaceProjectorException("Failed to find surface for projection.");
+                throw SurfaceProjectorException("Failed to find surface for projection.  This may occur "
+                                                "if there is no left or right anatomical surface loaded.");
             }
             const SurfaceFile* projectionSurfaceFile = m_surfaceFiles[nearestSurfaceIndex];
             projectItemToSurfaceFile(projectionSurfaceFile,
@@ -457,7 +458,7 @@ SurfaceProjector::projectItem(SurfaceProjectedItem* spi,
  */
 void
 SurfaceProjector::projectItemToSurfaceFile(const SurfaceFile* surfaceFile,
-                                              SurfaceProjectedItem* spi) throw (SurfaceProjectorException)
+                                              SurfaceProjectedItem* spi)
 {
     float originalXYZ[3];
     spi->getStereotaxicXYZ(originalXYZ);
@@ -655,7 +656,7 @@ void
 SurfaceProjector::projectToSurface(const SurfaceFile* surfaceFile,
                                    const float xyz[3],
                                    SurfaceProjectedItem* spi)
-            throw (SurfaceProjectorException)
+           
 {
     //
     // If needed, create node locator
@@ -766,7 +767,7 @@ SurfaceProjector::projectToSurface(const SurfaceFile* surfaceFile,
 void
 SurfaceProjector::convertToTriangleProjection(const SurfaceFile* surfaceFile,
                                               ProjectionLocation& projectionLocation)
-                                                         throw (SurfaceProjectorException)
+                                                        
 {
     bool doIt = false;
     
@@ -827,7 +828,7 @@ void
 SurfaceProjector::projectToSurfaceTriangle(const SurfaceFile* surfaceFile,
                                            const ProjectionLocation& projectionLocation,
                                            SurfaceProjectionBarycentric* baryProj)
-                                            throw (SurfaceProjectorException)
+                                           
 {
     /*
      * At one time, there was a need to 'perturb' (slightly move) the
@@ -858,7 +859,7 @@ SurfaceProjector::projectToSurfaceTriangle(const SurfaceFile* surfaceFile,
 void
 SurfaceProjector::getProjectionLocation(const SurfaceFile* surfaceFile,
                                         const float xyz[3],
-                                        ProjectionLocation& projectionLocation) const throw (SurfaceProjectorException)
+                                        ProjectionLocation& projectionLocation) const
 {
     /*
      * Find nearest point on the surface
@@ -867,22 +868,38 @@ SurfaceProjector::getProjectionLocation(const SurfaceFile* surfaceFile,
     BarycentricInfo baryInfo;
     sdh->barycentricWeights(xyz, baryInfo);
     
+    /*
+     * When projection is to node, the one node with the positive weight
+     * may not be the first node.  For edge projection, positivie weighted
+     * nodes may not be the first two.
+     */
+    std::vector<int32_t> weightedNodeIndices;
+    std::vector<int32_t> zeroWeightedNodeIndices;
     int32_t nearestNode = -1;
     float maxWeight = -1;
-    std::vector<int32_t> nodes;
-    std::vector<float> weights;
     for (int32_t i = 0; i < 3; i++) {
         if (baryInfo.baryWeights[i] > 0.0) {
-            nodes.push_back(baryInfo.nodes[i]);
-            
             const float w = baryInfo.baryWeights[i];
-            weights.push_back(w);
             if (w > maxWeight) {
                 nearestNode = baryInfo.nodes[i];
                 maxWeight = w;
             }
+            weightedNodeIndices.push_back(baryInfo.nodes[i]);
+        }
+        else {
+            zeroWeightedNodeIndices.push_back(baryInfo.nodes[i]);
         }
     }
+    weightedNodeIndices.insert(weightedNodeIndices.end(),
+                               zeroWeightedNodeIndices.begin(),
+                               zeroWeightedNodeIndices.end());
+    CaretAssert(weightedNodeIndices.size() == 3);
+    const int32_t nodeIndexOne(weightedNodeIndices[0]);
+    const int32_t nodeIndexTwo(weightedNodeIndices[1]);
+    CaretAssert((nodeIndexOne >= 0)
+                && (nodeIndexOne < surfaceFile->getNumberOfNodes()));
+    CaretAssert((nodeIndexTwo >= 0)
+                && (nodeIndexTwo < surfaceFile->getNumberOfNodes()));
     
     if (nearestNode < 0) {
         throw SurfaceProjectorException("ERROR: Nearest node is invalid");
@@ -892,57 +909,39 @@ SurfaceProjector::getProjectionLocation(const SurfaceFile* surfaceFile,
     switch (baryInfo.type) {
         case BarycentricInfo::NODE:
         {
-            if (nodes.size() != 1) {
-                throw SurfaceProjectorException("ERROR: project to node number of weights incorrect="
-                              + AString::number(nodes.size()));
-            }
-            else {
-                const float* nodeNormal = surfaceFile->getNormalVector(nodes[0]);
-                const float* c1 = surfaceFile->getCoordinate(nodes[0]);
-                const float aboveBelowPlane =
-                MathFunctions::signedDistanceFromPlane(nodeNormal, c1, xyz);
-                const float signValue = ((aboveBelowPlane > 0.0) ? 1.0 : -1.0);
-                signedDistance = (MathFunctions::distance3D(xyz, c1) * signValue);
-            }
+            const float* nodeNormal = surfaceFile->getNormalVector(nodeIndexOne);
+            const float* c1 = surfaceFile->getCoordinate(nodeIndexOne);
+            const float aboveBelowPlane =
+            MathFunctions::signedDistanceFromPlane(nodeNormal, c1, xyz);
+            const float signValue = ((aboveBelowPlane > 0.0) ? 1.0 : -1.0);
+            signedDistance = (MathFunctions::distance3D(xyz, c1) * signValue);
         }
             break;
         case BarycentricInfo::EDGE:
         {
-            if (nodes.size() != 2) {
-                throw SurfaceProjectorException("ERROR: project to edge number weights incorrect="
-                              + AString::number(nodes.size()));
-            }
-            else {
-                const float* n1 = surfaceFile->getNormalVector(nodes[0]);
-                const float* n2 = surfaceFile->getNormalVector(nodes[1]);
-                float avgNormal[3];
-                MathFunctions::addVectors(n1, n2, avgNormal);
-                MathFunctions::normalizeVector(avgNormal);
-                
-                const float* c1 = surfaceFile->getCoordinate(nodes[0]);
-                const float* c2 = surfaceFile->getCoordinate(nodes[1]);
-                MathFunctions::distanceToLine3D(c1, c2, xyz);
-                
-                const float aboveBelowPlane =
-                MathFunctions::signedDistanceFromPlane(avgNormal, baryInfo.point, xyz);
-                const float signValue = ((aboveBelowPlane > 0.0) ? 1.0 : -1.0);
-                signedDistance = (MathFunctions::distance3D(xyz, baryInfo.point) * signValue);
-            }
+            const float* n1 = surfaceFile->getNormalVector(nodeIndexOne);
+            const float* n2 = surfaceFile->getNormalVector(nodeIndexTwo);
+            float avgNormal[3];
+            MathFunctions::addVectors(n1, n2, avgNormal);
+            MathFunctions::normalizeVector(avgNormal);
+            
+            const float* c1 = surfaceFile->getCoordinate(nodeIndexOne);
+            const float* c2 = surfaceFile->getCoordinate(nodeIndexTwo);
+            MathFunctions::distanceToLine3D(c1, c2, xyz);
+            
+            const float aboveBelowPlane =
+            MathFunctions::signedDistanceFromPlane(avgNormal, baryInfo.point, xyz);
+            const float signValue = ((aboveBelowPlane > 0.0) ? 1.0 : -1.0);
+            signedDistance = (MathFunctions::distance3D(xyz, baryInfo.point) * signValue);
         }
             break;
         case BarycentricInfo::TRIANGLE:
         {
-            if (nodes.size() != 3) {
-                throw SurfaceProjectorException("ERROR: project to triangle number of weights incorrect="
-                              + AString::number(nodes.size()));
-            }
-            else {
-                float triangleNormal[3];
-                surfaceFile->getTriangleNormalVector(baryInfo.triangle, triangleNormal);
-                const float* c1 = surfaceFile->getCoordinate(nodes[0]);
-                signedDistance =
-                MathFunctions::signedDistanceFromPlane(triangleNormal, c1, xyz);
-            }
+            float triangleNormal[3];
+            surfaceFile->getTriangleNormalVector(baryInfo.triangle, triangleNormal);
+            const float* c1 = surfaceFile->getCoordinate(baryInfo.nodes[0]);
+            signedDistance =
+            MathFunctions::signedDistanceFromPlane(triangleNormal, c1, xyz);
         }
             break;
     }
@@ -966,8 +965,8 @@ SurfaceProjector::getProjectionLocation(const SurfaceFile* surfaceFile,
         {
             projectionLocation.m_type = ProjectionLocation::NODE;
             int32_t numTriangles = 0;
-            const int32_t* nodesTriangles = topologyHelper->getNodeTiles(nodes[0],
-                                                                        numTriangles);
+            const int32_t* nodesTriangles = topologyHelper->getNodeTiles(nodeIndexOne,
+                                                                         numTriangles);
             
             /*
              * Make sure nearest triangle is first and 
@@ -994,8 +993,8 @@ SurfaceProjector::getProjectionLocation(const SurfaceFile* surfaceFile,
         case BarycentricInfo::EDGE:
         {
             projectionLocation.m_type = ProjectionLocation::EDGE;
-            const int32_t oppositeTriangle = surfaceFile->getTriangleThatSharesEdge(nodes[0],
-                                                                                    nodes[1],
+            const int32_t oppositeTriangle = surfaceFile->getTriangleThatSharesEdge(nodeIndexOne,
+                                                                                    nodeIndexTwo,
                                                                                     baryInfo.triangle);
             nearbyTriangles.push_back(baryInfo.triangle);
             nearbyTriangles.push_back(oppositeTriangle);
@@ -1010,14 +1009,8 @@ SurfaceProjector::getProjectionLocation(const SurfaceFile* surfaceFile,
     for (int32_t i = 0; i < 3; i++) {
         projectionLocation.m_pointXYZ[i] = xyz[i];
         projectionLocation.m_surfaceXYZ[i] = baryInfo.point[i];
-        if (i < static_cast<int32_t>(nodes.size())) {
-            projectionLocation.m_nodes[i] = nodes[i];
-            projectionLocation.m_weights[i] = weights[i];
-        }
-        else {
-            projectionLocation.m_nodes[i] = -1;
-            projectionLocation.m_weights[i] = 0.0;
-        }
+        projectionLocation.m_nodes[i] = baryInfo.nodes[i];
+        projectionLocation.m_weights[i] = baryInfo.baryWeights[i];
     }
 
     projectionLocation.m_numberOfTriangles = static_cast<int32_t>(nearbyTriangles.size());
@@ -1030,7 +1023,6 @@ SurfaceProjector::getProjectionLocation(const SurfaceFile* surfaceFile,
     projectionLocation.m_nearestNode = nearestNode;
     
     
-    AString distErrorMessage = "";
     float distError = std::fabs(signedDistance) - baryInfo.absDistance;
     if (distError > 0.01) {
         throw SurfaceProjectorException("ERROR: signed/abs distance mismatch: "
@@ -1055,7 +1047,7 @@ int32_t
 SurfaceProjector::projectToSurfaceTriangleAux(const SurfaceFile* surfaceFile,
                                               const ProjectionLocation& projectionLocation,
                                       SurfaceProjectionBarycentric* baryProj)
-                                             throw (SurfaceProjectorException)
+                                            
 {
     /*
      * Set the projection.
@@ -1326,7 +1318,7 @@ void
 SurfaceProjector::projectWithVanEssenAlgorithm(const SurfaceFile* surfaceFile,
                                                const ProjectionLocation& projectionLocation,
                                                SurfaceProjectionVanEssen* spve)
-            throw (SurfaceProjectorException)
+           
 {
     float xyz[3] = {
         projectionLocation.m_pointXYZ[0],
