@@ -22,26 +22,33 @@
  */
 /*LICENSE_END*/
 
+#include <map>
+#include <memory>
+
 #include "BrainConstants.h"
 #include "VolumeBase.h"
 #include "CaretMappableDataFile.h"
 #include "CaretMutex.h"
 #include "CaretVolumeExtension.h"
 #include "ChartableLineSeriesBrainordinateInterface.h"
+#include "GroupAndNameHierarchyUserInterface.h"
 #include "StructureEnum.h"
 #include "GiftiMetaData.h"
 #include "BoundingBox.h"
 #include "VolumeFileVoxelColorizer.h"
 #include "VoxelIJK.h"
+#include "VoxelInterpolationTypeEnum.h"
 
 namespace caret {
     
     class GroupAndNameHierarchyModel;
+    class VolumeDynamicConnectivityFile;
     class VolumeFileEditorDelegate;
     class VolumeFileVoxelColorizer;
+    class VolumeGraphicsPrimitiveManager;
     class VolumeSpline;
     
-    class VolumeFile : public VolumeBase, public CaretMappableDataFile, public ChartableLineSeriesBrainordinateInterface
+    class VolumeFile : public VolumeBase, public CaretMappableDataFile, public ChartableLineSeriesBrainordinateInterface, public GroupAndNameHierarchyUserInterface
     {
         VolumeFile(const VolumeFile&);
         
@@ -96,6 +103,10 @@ namespace caret {
         /** Performs coloring of voxels.  Will be NULL if coloring is disabled. */
         CaretPointer<VolumeFileVoxelColorizer> m_voxelColorizer;
         
+        std::unique_ptr<VolumeDynamicConnectivityFile> m_lazyInitializedDynamicConnectivityFile;
+        
+        std::unique_ptr<VolumeGraphicsPrimitiveManager> m_graphicsPrimitiveManager;
+        
         /** True if the volume is a single slice, needed by interpolateValue() methods */
         bool m_singleSliceFlag;
         
@@ -115,6 +126,9 @@ namespace caret {
         
         mutable float m_dataRangeMaximum;
         
+        /** Have bounding box for each map */
+        mutable std::vector<std::unique_ptr<BoundingBox>> m_nonZeroVoxelCoordinateBoundingBoxes;
+        
         /** Holds class and name hierarchy used for display selection */
         mutable CaretPointer<GroupAndNameHierarchyModel> m_classNameHierarchy;
         
@@ -123,7 +137,19 @@ namespace caret {
         
         CaretPointer<VolumeFileEditorDelegate> m_volumeFileEditorDelegate;
         
+        static const AString s_paletteColorMappingNameInMetaData;
+
+        int16_t m_writingDType;
+
+        bool m_writingDoScale;
+
+        double m_minScalingVal, m_maxScalingVal;
+        
+        mutable std::map<int32_t, std::unique_ptr<ClusterContainer>> m_mapLabelClusterContainers;
+        
     protected:
+        VolumeFile(const DataFileTypeEnum::Enum dataFileType);
+        
         virtual void saveFileDataToScene(const SceneAttributes* sceneAttributes,
                                          SceneClass* sceneClass);
         
@@ -176,10 +202,12 @@ namespace caret {
 
         void freeSpline(const int64_t brickIndex = 0, const int64_t component = 0) const;
 
-        float interpolateValue(const float* coordIn, InterpType interp = TRILINEAR, bool* validOut = NULL, const int64_t brickIndex = 0, const int64_t component = 0) const;
+        float interpolateValue(const float* coordIn, InterpType interp = TRILINEAR, bool* validOut = NULL, const int64_t brickIndex = 0, const int64_t component = 0, const float backgroundVal = INVALID_INTERP_VALUE) const;
 
-        float interpolateValue(const float coordIn1, const float coordIn2, const float coordIn3, InterpType interp = TRILINEAR, bool* validOut = NULL, const int64_t brickIndex = 0, const int64_t component = 0) const;
+        float interpolateValue(const float coordIn1, const float coordIn2, const float coordIn3, InterpType interp = TRILINEAR, bool* validOut = NULL, const int64_t brickIndex = 0, const int64_t component = 0, const float backgroundVal = INVALID_INTERP_VALUE) const;
 
+        float interpolateValue(const float* coordIn, const VoxelInterpolationTypeEnum::Enum interpType = VoxelInterpolationTypeEnum::TRILINEAR, bool* validOut = NULL, const int64_t brickIndex = 0, const int64_t component = 0, const float backgroundVal = INVALID_INTERP_VALUE) const;
+        
         ///returns true if volume space matches in spatial dimensions and sform
         bool matchesVolumeSpace(const VolumeFile* right) const;
         
@@ -189,9 +217,13 @@ namespace caret {
         ///returns true if volume space matches in spatial dimensions and sform
         bool matchesVolumeSpace(const int64_t dims[3], const std::vector<std::vector<float> >& sform) const;
         
-        void readFile(const AString& filename);
+        virtual void readFile(const AString& filename) override;
 
-        void writeFile(const AString& filename);
+        virtual void writeFile(const AString& filename) override;
+
+        ///data type and scaling options
+        void setWritingDataTypeNoScaling(const int16_t& type = NIFTI_TYPE_FLOAT32);
+        void setWritingDataTypeAndScaling(const int16_t& type, const double& minval, const double& maxval);
 
         bool isEmpty() const { return VolumeBase::isEmpty(); }
         
@@ -205,6 +237,15 @@ namespace caret {
         
         void getVoxelSpaceBoundingBox(BoundingBox& boundingBoxOut) const override;
         
+        static void dimensionOriginSpacingXyzToVoxelEdges(const Vector3D& dimensions,
+                                                          const Vector3D& origin,
+                                                          const Vector3D& spacing,
+                                                          Vector3D& firstVoxelEdgeOut,
+                                                          Vector3D& lastVoxelEdgeOut);
+        
+        virtual void getNonZeroVoxelCoordinateBoundingBox(const int32_t mapIndex,
+                                                   BoundingBox& boundingBoxOut) const override;
+        
         /**
          * @return The structure for this file.
          */
@@ -282,6 +323,8 @@ namespace caret {
         
         const GiftiLabelTable* getMapLabelTable(const int32_t mapIndex) const;
 
+        virtual const ClusterContainer* getMapLabelTableClusters(const int32_t mapIndex) const override;
+
         void getVoxelIndicesWithLabelKey(const int32_t mapIndex,
                                          const int32_t labelKey,
                                          std::vector<VoxelIJK>& voxelIndicesOut) const;
@@ -292,6 +335,10 @@ namespace caret {
         
         AString getMapUniqueID(const int32_t mapIndex) const;
         
+        virtual CaretMappableDataFile* castToVolumeMappableDataFile() override;
+        
+        virtual const CaretMappableDataFile* castToVolumeMappableDataFile() const override;
+
         void updateScalarColoringForMap(const int32_t mapIndex) override;
         
         virtual int64_t getVoxelColorsForSliceInMap(const int32_t mapIndex,
@@ -300,16 +347,14 @@ namespace caret {
                                             const int64_t columnStepIJK[3],
                                             const int64_t numberOfRows,
                                             const int64_t numberOfColumns,
-                                            const DisplayGroupEnum::Enum displayGroup,
-                                            const int32_t tabIndex,
+                                                    const TabDrawingInfo& tabDrawingInfo,
                                             uint8_t* rgbaOut) const override;
         
         virtual int64_t getVoxelColorsForSliceInMap(
                                          const int32_t mapIndex,
                                          const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                          const int64_t sliceIndex,
-                                         const DisplayGroupEnum::Enum displayGroup,
-                                         const int32_t tabIndex,
+                                                    const TabDrawingInfo& tabDrawingInfo,
                                          uint8_t* rgbaOut) const override;
 
         virtual int64_t getVoxelColorsForSubSliceInMap(
@@ -319,21 +364,48 @@ namespace caret {
                                                     const int64_t firstCornerVoxelIndex[3],
                                                     const int64_t lastCornerVoxelIndex[3],
                                                     const int64_t voxelCountIJK[3],
-                                                    const DisplayGroupEnum::Enum displayGroup,
-                                                    const int32_t tabIndex,
+                                                       const TabDrawingInfo& tabDrawingInfo,
                                                     uint8_t* rgbaOut) const override;
         
+        virtual GraphicsPrimitiveV3fT3f* getVolumeDrawingTriangleStripPrimitive(const int32_t mapIndex,
+                                                                                const TabDrawingInfo& tabDrawingInfo) const override;
+        
+        virtual GraphicsPrimitiveV3fT3f* getVolumeDrawingTriangleFanPrimitive(const int32_t mapIndex,
+                                                                              const TabDrawingInfo& tabDrawingInfo) const override;
+        
+        virtual GraphicsPrimitiveV3fT3f* getVolumeDrawingTrianglesPrimitive(const int32_t mapIndex,
+                                                                            const TabDrawingInfo& tabDrawingInfo) const override;
+
+        virtual GraphicsPrimitive* getHistologyImageIntersectionPrimitive(const int32_t mapIndex,
+                                                                          const TabDrawingInfo& tabDrawingInfo,
+                                                                          const MediaFile* mediaFile,
+                                                                          const VolumeToImageMappingModeEnum::Enum volumeMappingMode,
+                                                                          const float volumeSliceThickness,
+                                                                          AString& errorMessageOut) const override;
+        
+        virtual std::vector<GraphicsPrimitive*> getHistologySliceIntersectionPrimitive(const int32_t mapIndex,
+                                                                                       const TabDrawingInfo& tabDrawingInfo,
+                                                                                       const HistologySlice* histologySlice,
+                                                                                       const VolumeToImageMappingModeEnum::Enum volumeMappingMode,
+                                                                                       const float volumeSliceThickness,
+                                                                                       AString& errorMessageOut) const override;
+
         void getVoxelValuesForSliceInMap(const int32_t mapIndex,
                                          const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                          const int64_t sliceIndex,
                                          float* sliceValues) const;
         
-        void getVoxelColorInMap(const int64_t i,
+        virtual void getVoxelColorInMap(const int64_t i,
                                 const int64_t j,
                                 const int64_t k,
                                 const int64_t mapIndex,
-                                const DisplayGroupEnum::Enum displayGroup,
-                                const int32_t tabIndex,
+                                uint8_t rgbaOut[4]) const override;
+        
+        virtual void getVoxelColorInMap(const int64_t i,
+                                const int64_t j,
+                                const int64_t k,
+                                const int64_t mapIndex,
+                                        const TabDrawingInfo& tabDrawingInfo,
                                 uint8_t rgbaOut[4]) const override;
         
         void clearVoxelColoringForMap(const int64_t mapIndex);
@@ -368,7 +440,42 @@ namespace caret {
         virtual void getDataForSelector(const MapFileDataSelector& mapFileDataSelector,
                                         std::vector<float>& dataOut) const override;
         
-        virtual BrainordinateMappingMatch getBrainordinateMappingMatch(const CaretMappableDataFile* mapFile) const override;
+        virtual BrainordinateMappingMatch getBrainordinateMappingMatchImplementation(const CaretMappableDataFile* mapFile) const override;
+       
+        virtual bool getVolumeVoxelIdentificationForMaps(const std::vector<int32_t>& mapIndices,
+                                                         const float xyz[3],
+                                                         const AString& dataValueSeparator,
+                                                         const int32_t digitsRightOfDecimal,
+                                                         int64_t ijkOut[3],
+                                                         AString& textOut) const;
+        
+        virtual NiftiTimeUnitsEnum::Enum getMapIntervalUnits() const override;
+        
+        virtual void getMapIntervalStartAndStep(float& firstMapUnitsValueOut,
+                                                float& mapIntervalStepValueOut) const override;
+        
+        VolumeDynamicConnectivityFile* getVolumeDynamicConnectivityFile();
+        
+        const VolumeDynamicConnectivityFile* getVolumeDynamicConnectivityFile() const;
+        
+        virtual bool isModifiedPaletteColorMapping() const override;
+        
+        virtual PaletteModifiedStatusEnum::Enum getPaletteColorMappingModifiedStatus() const override;
+        
+        virtual void groupAndNameHierarchyItemStatusChanged() override;
+
+        void setValuesForVoxelEditing(const int32_t mapIndex,
+                                      const std::vector<VoxelIJK>& voxelsIJK,
+                                      const float value);
+        
+        void getNeigbors26(const VoxelIJK& voxelIJK,
+                           const float* voxelValues,
+                           const float minimumValue,
+                           const float maximumValue,
+                           std::vector<char>& voxelHasBeenSearchedFlags,
+                           std::vector<VoxelIJK>& neighborIJKs) const;
+        
+
     };
 
 }
