@@ -25,11 +25,16 @@
 
 #include "CaretAssert.h"
 #include "CaretLogger.h"
+#include "DataFileColorModulateSelector.h"
 #include "ElapsedTimer.h"
 #include "GiftiLabel.h"
 #include "GroupAndNameHierarchyItem.h"
+#include "LabelSelectionItemModel.h"
 #include "NodeAndVoxelColoring.h"
+#include "Palette.h"
+#include "TabDrawingInfo.h"
 #include "VolumeFile.h"
+#include "VoxelColorUpdate.h"
 
 #include <cmath>
 
@@ -89,45 +94,55 @@ VolumeFileVoxelColorizer::~VolumeFileVoxelColorizer()
  *     Index of map.
  */
 void
-VolumeFileVoxelColorizer::assignVoxelColorsForMap(const int32_t mapIndex)
+VolumeFileVoxelColorizer::assignVoxelColorsForMap(const int32_t mapIndex) const
 {
     CaretAssertVectorIndex(m_mapRGBA, mapIndex);
     
+    CaretAssertVectorIndex(m_mapColoringValid, mapIndex);
+    if ( ! m_mapColoringValid[mapIndex]) {
+        clearVoxelColoringForMap(mapIndex);
+    }
+    
     ElapsedTimer timer;
     timer.start();
     
     /*
-     * Pointer to map's data 
+     * Pointer to map's data
      */
     const float* mapDataPointer = m_volumeFile->getFrame(mapIndex);
     
     VolumeFile* thresholdVolume = NULL;
     int32_t thresholdVolumeMapIndex   = -1;
-    switch (m_volumeFile->getMapPaletteColorMapping(mapIndex)->getThresholdType()) {
-        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_FILE:
-        {
-            CaretMappableDataFileAndMapSelectionModel* threshSel = m_volumeFile->getMapThresholdFileSelectionModel(mapIndex);
-            CaretMappableDataFile* mapFile = threshSel->getSelectedFile();
-            if (mapFile != NULL) {
-                thresholdVolume = dynamic_cast<VolumeFile*>(mapFile);
-                CaretAssert(thresholdVolume);
-                thresholdVolumeMapIndex = threshSel->getSelectedMapIndex();
+    
+    bool showZerosFlag(true);
+    if (m_volumeFile->isMappedWithPalette()) {
+        showZerosFlag = m_volumeFile->getMapPaletteColorMapping(mapIndex)->isDisplayZeroDataFlag();
+        switch (m_volumeFile->getMapPaletteColorMapping(mapIndex)->getThresholdType()) {
+            case PaletteThresholdTypeEnum::THRESHOLD_TYPE_FILE:
+            {
+                CaretMappableDataFileAndMapSelectionModel* threshSel = m_volumeFile->getMapThresholdFileSelectionModel(mapIndex);
+                CaretMappableDataFile* mapFile = threshSel->getSelectedFile();
+                if (mapFile != NULL) {
+                    thresholdVolume = dynamic_cast<VolumeFile*>(mapFile);
+                    CaretAssert(thresholdVolume);
+                    thresholdVolumeMapIndex = threshSel->getSelectedMapIndex();
+                }
             }
+                break;
+            case PaletteThresholdTypeEnum::THRESHOLD_TYPE_MAPPED:
+                break;
+            case PaletteThresholdTypeEnum::THRESHOLD_TYPE_MAPPED_AVERAGE_AREA:
+                break;
+            case PaletteThresholdTypeEnum::THRESHOLD_TYPE_NORMAL:
+                /*
+                 * Thresholding with 'self'
+                 */
+                thresholdVolume = m_volumeFile;
+                thresholdVolumeMapIndex = mapIndex;
+                break;
+            case PaletteThresholdTypeEnum::THRESHOLD_TYPE_OFF:
+                break;
         }
-            break;
-        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_MAPPED:
-            break;
-        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_MAPPED_AVERAGE_AREA:
-            break;
-        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_NORMAL:
-            /*
-             * Thresholding with 'self'
-             */
-            thresholdVolume = m_volumeFile;
-            thresholdVolumeMapIndex = mapIndex;
-            break;
-        case PaletteThresholdTypeEnum::THRESHOLD_TYPE_OFF:
-            break;
     }
     
     /*
@@ -162,7 +177,9 @@ VolumeFileVoxelColorizer::assignVoxelColorsForMap(const int32_t mapIndex)
         }
     }
     
-    switch (m_volumeFile->getType()) {
+    const SubvolumeAttributes::VolumeType volumeType(m_volumeFile->getType());
+        
+    switch (volumeType) {
         case SubvolumeAttributes::UNKNOWN:
         case SubvolumeAttributes::ANATOMY:
         case SubvolumeAttributes::FUNCTIONAL:
@@ -230,12 +247,38 @@ VolumeFileVoxelColorizer::assignVoxelColorsForMap(const int32_t mapIndex)
             }
         }
             break;
+        case SubvolumeAttributes::RGB_WORKBENCH:
+            if (m_volumeFile->getNumberOfMaps() >= 3) {
+                /*
+                 * Use first 3 maps as RGB components
+                 */
+                const float* alphaComponents(NULL);
+                const uint8_t thresholdRGB[3] = { 0, 0, 0 };
+                NodeAndVoxelColoring::colorScalarsWithRGBA(m_volumeFile->getFrame(0),
+                                                           m_volumeFile->getFrame(1),
+                                                           m_volumeFile->getFrame(2),
+                                                           alphaComponents,
+                                                           m_voxelCountPerMap,
+                                                           thresholdRGB,
+                                                           m_mapRGBA[mapIndex]);
+                m_mapColoringValid[mapIndex] = true;
+            }
+            else {
+                CaretLogSevere("An RGB_WORKBENCH volume must contain at least 3 maps: "
+                               + m_volumeFile->getFileNameNoPath());
+            }
+            break;
         case SubvolumeAttributes::SEGMENTATION:
             break;
         case SubvolumeAttributes::VECTOR:
             break;
     }
     
+    if (m_mapColoringValid[mapIndex]) {
+        applyColorModulation(mapIndex,
+                             showZerosFlag);
+    }
+    
     CaretLogFine("Time to color map named \""
                    + m_volumeFile->getMapName(mapIndex)
                    + " in volume file "
@@ -245,6 +288,65 @@ VolumeFileVoxelColorizer::assignVoxelColorsForMap(const int32_t mapIndex)
                    + " milliseconds");
 }
 
+/**
+ * Apply color modulation with another volume file.
+ * Data in modulation file should range [0, 1] and be a single component map.
+ * @param mapIndex
+ *    Index of map in file being colorized
+ * @param showZerosFlag
+ *    If true, show data with a modulate zero value
+ */
+void
+VolumeFileVoxelColorizer::applyColorModulation(const int32_t mapIndex,
+                                               const bool showZerosFlag) const
+{
+    const DataFileColorModulateSelector* modulateSelector(m_volumeFile->getMapColorModulateFileSelector(mapIndex));
+    CaretAssert(modulateSelector);
+    if (modulateSelector->isEnabled()) {
+        const VolumeFile* modulateVolumeFile(modulateSelector->getSelectedVolumeFile());
+        if (modulateVolumeFile != NULL) {
+            const int32_t modulateMapIndex(modulateSelector->getSelectedMapIndex());
+            if ((modulateMapIndex >= 0)
+                && (modulateMapIndex < modulateVolumeFile->getNumberOfMaps())) {
+                int64_t dimI, dimJ, dimK, mapCount, numComps;
+                modulateVolumeFile->getDimensions(dimI,
+                                                  dimJ,
+                                                  dimK,
+                                                  mapCount,
+                                                  numComps);
+                if ((dimI == m_dimI)
+                    && (dimJ == m_dimJ)
+                    && (dimK == m_dimK)) {
+                    const int64_t componentIndex(0);
+                    const float* modData(modulateVolumeFile->getFrame(modulateMapIndex,
+                                                                      componentIndex));
+                    
+                    CaretAssertVectorIndex(m_mapRGBA, mapIndex);
+                    uint8_t* rgba(m_mapRGBA[mapIndex]);
+                    
+                    for (int64_t i = 0; i < m_voxelCountPerMap; i++) {
+                        float modValue(modData[i]);
+                        if (modValue > 1.0) modValue = 1.0;
+                        if (modValue < 0.0) modValue = 0.0;
+
+                        const int64_t i4(i * 4);
+                        for (int64_t j = 0; j < 3; j++) {
+                            const float v(static_cast<float>(rgba[i4 + j]) * modValue);
+                            rgba[i4 + j] = static_cast<uint8_t>(v);
+                        }
+                        
+                        if ( ! showZerosFlag) {
+                            if (modValue <= 0) {
+                                rgba[i4 + 3] = 0;
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+}
+
 /**
  * Invalidate the RGBA coloring for all maps.
  */
@@ -267,10 +369,8 @@ VolumeFileVoxelColorizer::invalidateColoring()
  *    Plane of the slice.
  * @param sliceIndex
  *    Index of the slice.
- * @param displayGroup
- *    The selected display group.
- * @param tabIndex
- *    Index of selected tab.
+ * @param tabDrawingInfo
+ *    Info for drawing the tab
  * @param rgbaOut
  *    RGBA color components out.
  * @return
@@ -280,14 +380,18 @@ int64_t
 VolumeFileVoxelColorizer::getVoxelColorsForSliceInMap(const int32_t mapIndex,
                                                       const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                                       const int64_t sliceIndex,
-                                                      const DisplayGroupEnum::Enum displayGroup,
-                                                      const int32_t tabIndex,
+                                                      const TabDrawingInfo& tabDrawingInfo,
                                                       uint8_t* rgbaOut) const
 {
     CaretAssertVectorIndex(m_mapRGBA, mapIndex);
     CaretAssert(sliceIndex >= 0);
     CaretAssert(rgbaOut);
     
+    CaretAssertVectorIndex(m_mapColoringValid, mapIndex);
+    if ( ! m_mapColoringValid[mapIndex]) {
+        assignVoxelColorsForMap(mapIndex);
+    }
+    
     int64_t iStart = 0;
     int64_t iEnd   = m_dimI - 1;
     int64_t jStart = 0;
@@ -324,6 +428,9 @@ VolumeFileVoxelColorizer::getVoxelColorsForSliceInMap(const int32_t mapIndex,
         CaretAssert(labelTable);
     }
     
+    const LabelSelectionItemModel* labelSelectionItemModel(m_volumeFile->getLabelSelectionHierarchyForMapAndTab(mapIndex,
+                                                                                                                tabDrawingInfo.getDisplayGroup(),
+                                                                                                                tabDrawingInfo.getTabIndex()));
     int64_t validVoxelCount = 0;
     
     /*
@@ -353,11 +460,24 @@ VolumeFileVoxelColorizer::getVoxelColorsForSliceInMap(const int32_t mapIndex,
                                                                                               mapIndex));
                         const GiftiLabel* label = labelTable->getLabel(dataValue);
                         if (label != NULL) {
-                            const GroupAndNameHierarchyItem* item = label->getGroupNameSelectionItem();
-                            if (item != NULL) {
-                                if (item->isSelected(displayGroup, tabIndex) == false) {
-                                    alpha = 0;
+                            switch (tabDrawingInfo.getLabelViewMode()) {
+                                case LabelViewModeEnum::HIERARCHY:
+                                    if (labelSelectionItemModel != NULL) {
+                                        if ( ! labelSelectionItemModel->isLabelChecked(dataValue)) {
+                                            alpha = 0;
+                                        }
+                                    }
+                                    break;
+                                case LabelViewModeEnum::LIST:
+                                {
+                                    const GroupAndNameHierarchyItem* item = label->getGroupNameSelectionItem();
+                                    if (item != NULL) {
+                                        if ( ! item->isSelected(tabDrawingInfo)) {
+                                            alpha = 0;
+                                        }
+                                    }
                                 }
+                                    break;
                             }
                         }
                     }
@@ -390,10 +510,8 @@ VolumeFileVoxelColorizer::getVoxelColorsForSliceInMap(const int32_t mapIndex,
  *    Number of rows.
  * @param numberOfColumns
  *    Number of columns.
- * @param displayGroup
- *    The selected display group.
- * @param tabIndex
- *    Index of selected tab.
+ * @param tabDrawingInfo
+ *    Info for drawing the tab
  * @param rgbaOut
  *    RGBA color components out.
  * @return
@@ -406,10 +524,14 @@ VolumeFileVoxelColorizer::getVoxelColorsForSliceInMap(const int32_t mapIndex,
                                     const int64_t columnStepIJK[3],
                                     const int64_t numberOfRows,
                                     const int64_t numberOfColumns,
-                                    const DisplayGroupEnum::Enum displayGroup,
-                                    const int32_t tabIndex,
+                                                      const TabDrawingInfo& tabDrawingInfo,
                                     uint8_t* rgbaOut) const
 {
+    CaretAssertVectorIndex(m_mapColoringValid, mapIndex);
+    if ( ! m_mapColoringValid[mapIndex]) {
+        assignVoxelColorsForMap(mapIndex);
+    }
+
     /*
      * Pointer to maps RGBA values
      */
@@ -422,6 +544,10 @@ VolumeFileVoxelColorizer::getVoxelColorsForSliceInMap(const int32_t mapIndex,
         CaretAssert(labelTable);
     }
     
+    const LabelSelectionItemModel* labelSelectionItemModel(m_volumeFile->getLabelSelectionHierarchyForMapAndTab(mapIndex,
+                                                                                                                tabDrawingInfo.getDisplayGroup(),
+                                                                                                                tabDrawingInfo.getTabIndex()));
+
     int64_t validVoxelCount = 0;
     int64_t rgbaOutIndex = 0;
     
@@ -449,10 +575,23 @@ VolumeFileVoxelColorizer::getVoxelColorsForSliceInMap(const int32_t mapIndex,
                                                                                           mapIndex));
                     const GiftiLabel* label = labelTable->getLabel(dataValue);
                     if (label != NULL) {
-                        const GroupAndNameHierarchyItem* item = label->getGroupNameSelectionItem();
-                        if (item != NULL) {
-                            if (item->isSelected(displayGroup, tabIndex) == false) {
-                                alpha = 0;
+                        switch (tabDrawingInfo.getLabelViewMode()) {
+                            case LabelViewModeEnum::HIERARCHY:
+                                if (labelSelectionItemModel != NULL) {
+                                    if ( ! labelSelectionItemModel->isLabelChecked(dataValue)) {
+                                        alpha = 0;
+                                    }
+                                }
+                                break;
+                            case LabelViewModeEnum::LIST:
+                            {
+                                const GroupAndNameHierarchyItem* item = label->getGroupNameSelectionItem();
+                                if (item != NULL) {
+                                    if ( ! item->isSelected(tabDrawingInfo.getDisplayGroup(),
+                                                         tabDrawingInfo.getTabIndex())) {
+                                        alpha = 0;
+                                    }
+                                }
                             }
                         }
                     }
@@ -462,6 +601,11 @@ VolumeFileVoxelColorizer::getVoxelColorsForSliceInMap(const int32_t mapIndex,
             if (alpha > 0.0) {
                 ++validVoxelCount;
             }
+            if (alpha == 0) { /* Fixes labels on/off in MPR */
+                rgbaOut[rgbaOutIndex] = 0;
+                rgbaOut[rgbaOutIndex+1] = 0;
+                rgbaOut[rgbaOutIndex+2] = 0;
+            }
             rgbaOut[rgbaOutIndex+3] = alpha;
             rgbaOutIndex += 4;
             
@@ -495,10 +639,8 @@ VolumeFileVoxelColorizer::getVoxelColorsForSliceInMap(const int32_t mapIndex,
  *    Indices of voxel for last corner of sub-slice (inclusive).
  * @param voxelCountIJK
  *    Voxel counts for each axis.
- * @param displayGroup
- *    The selected display group.
- * @param tabIndex
- *    Index of selected tab.
+ * @param tabDrawingInfo
+ *    Info for drawing the tab
  * @param rgbaOut
  *    RGBA color components out.
  * @return
@@ -511,14 +653,22 @@ VolumeFileVoxelColorizer::getVoxelColorsForSubSliceInMap(const int32_t mapIndex,
                                                          const int64_t firstCornerVoxelIndex[3],
                                                          const int64_t lastCornerVoxelIndex[3],
                                                          const int64_t* CaretParameterUsedInDebugCompileOnly(voxelCountIJK),
-                                                         const DisplayGroupEnum::Enum displayGroup,
-                                                         const int32_t tabIndex,
+                                                         const TabDrawingInfo& tabDrawingInfo,
                                                          uint8_t* rgbaOut) const
 {
     CaretAssertVectorIndex(m_mapRGBA, mapIndex);
     CaretAssert(sliceIndex >= 0);
     CaretAssert(rgbaOut);
     
+    CaretAssertVectorIndex(m_mapColoringValid, mapIndex);
+    if ( ! m_mapColoringValid[mapIndex]) {
+        assignVoxelColorsForMap(mapIndex);
+    }
+
+    const LabelSelectionItemModel* labelSelectionItemModel(m_volumeFile->getLabelSelectionHierarchyForMapAndTab(mapIndex,
+                                                                                                                tabDrawingInfo.getDisplayGroup(),
+                                                                                                                tabDrawingInfo.getTabIndex()));
+
     VolumeSpace::OrientTypes orient[3];
     m_volumeFile->getOrientation(orient);
     int orient2dim[3];
@@ -620,12 +770,25 @@ VolumeFileVoxelColorizer::getVoxelColorsForSubSliceInMap(const int32_t mapIndex,
                     const GiftiLabel* label = labelTable->getLabel(dataValue);
                     if (label != NULL)
                     {
-                        const GroupAndNameHierarchyItem* item = label->getGroupNameSelectionItem();
-                        if (item != NULL)
-                        {
-                            if (item->isSelected(displayGroup, tabIndex) == false)
+                        switch (tabDrawingInfo.getLabelViewMode()) {
+                            case LabelViewModeEnum::HIERARCHY:
+                                if (labelSelectionItemModel != NULL) {
+                                    if ( ! labelSelectionItemModel->isLabelChecked(dataValue)) {
+                                        alpha = 0;
+                                    }
+                                }
+                                break;
+                            case LabelViewModeEnum::LIST:
                             {
-                                alpha = 0;
+                                const GroupAndNameHierarchyItem* item = label->getGroupNameSelectionItem();
+                                if (item != NULL)
+                                {
+                                    if ( ! item->isSelected(tabDrawingInfo.getDisplayGroup(),
+                                                         tabDrawingInfo.getTabIndex()))
+                                    {
+                                        alpha = 0;
+                                    }
+                                }
                             }
                         }
                     }
@@ -645,7 +808,7 @@ VolumeFileVoxelColorizer::getVoxelColorsForSubSliceInMap(const int32_t mapIndex,
 }
 
 /**
- * Get the RGBA color components for voxel.
+ * Get the RGBA color components for voxel in a map
  *
  * @param i
  *    Parasaggital index
@@ -655,10 +818,6 @@ VolumeFileVoxelColorizer::getVoxelColorsForSubSliceInMap(const int32_t mapIndex,
  *    Axial index
  * @param mapIndex
  *    Index of map.
- * @param displayGroup
- *    The selected display group.
- * @param tabIndex
- *    Index of selected tab.
  * @param rgbaOut
  *    Contains voxel coloring on exit.
  */
@@ -667,10 +826,13 @@ VolumeFileVoxelColorizer::getVoxelColorInMap(const int64_t i,
                                              const int64_t j,
                                              const int64_t k,
                                              const int64_t mapIndex,
-                                             const DisplayGroupEnum::Enum displayGroup,
-                                             const int32_t tabIndex,
                                              uint8_t rgbaOut[4]) const
 {
+    CaretAssertVectorIndex(m_mapColoringValid, mapIndex);
+    if ( ! m_mapColoringValid[mapIndex]) {
+        assignVoxelColorsForMap(mapIndex);
+    }
+    
     /*
      * Pointer to maps RGBA values
      */
@@ -681,8 +843,58 @@ VolumeFileVoxelColorizer::getVoxelColorInMap(const int64_t i,
     rgbaOut[0] = mapRGBA[rgbaOffset];
     rgbaOut[1] = mapRGBA[rgbaOffset+1];
     rgbaOut[2] = mapRGBA[rgbaOffset+2];
-    uint8_t alpha = mapRGBA[rgbaOffset+3];
+    rgbaOut[3] = mapRGBA[rgbaOffset+3];
+}
+
+/**
+ * Get the RGBA color components for voxel with display group and tab.
+ *
+ * @param i
+ *    Parasaggital index
+ * @param j
+ *    Coronal index
+ * @param k
+ *    Axial index
+ * @param mapIndex
+ *    Index of map.
+ * @param tabDrawingInfo
+ *    Info for drawing the tab
+ * @param rgbaOut
+ *    Contains voxel coloring on exit.
+ */
+void
+VolumeFileVoxelColorizer::getVoxelColorInMap(const int64_t i,
+                                             const int64_t j,
+                                             const int64_t k,
+                                             const int64_t mapIndex,
+                                             const TabDrawingInfo& tabDrawingInfo,
+                                             uint8_t rgbaOut[4]) const
+{
+//    CaretAssertVectorIndex(m_mapColoringValid, mapIndex);
+//    if ( ! m_mapColoringValid[mapIndex]) {
+//        assignVoxelColorsForMap(mapIndex);
+//    }
+//
+//    /*
+//     * Pointer to maps RGBA values
+//     */
+//    CaretAssertVectorIndex(m_mapRGBA, mapIndex);
+//    const uint8_t* mapRGBA = m_mapRGBA[mapIndex];
+//    const int64_t rgbaOffset = getRgbaOffsetForVoxelIndex(i, j, k);
+//    CaretAssertArrayIndex(mapRGBA, m_mapRGBACount, rgbaOffset);
+//    rgbaOut[0] = mapRGBA[rgbaOffset];
+//    rgbaOut[1] = mapRGBA[rgbaOffset+1];
+//    rgbaOut[2] = mapRGBA[rgbaOffset+2];
+//    uint8_t alpha = mapRGBA[rgbaOffset+3];
+
+    const LabelSelectionItemModel* labelSelectionItemModel(m_volumeFile->getLabelSelectionHierarchyForMapAndTab(mapIndex,
+                                                                                                                tabDrawingInfo.getDisplayGroup(),
+                                                                                                                tabDrawingInfo.getTabIndex()));
+    getVoxelColorInMap(i, j, k,
+                       mapIndex,
+                       rgbaOut);
     
+    uint8_t alpha(rgbaOut[3]);
     if (alpha > 0) {
         if (m_volumeFile->isMappedWithLabelTable()) {
             const GiftiLabelTable* labelTable = m_volumeFile->getMapLabelTable(mapIndex);
@@ -698,10 +910,23 @@ VolumeFileVoxelColorizer::getVoxelColorInMap(const int64_t i,
                                                                                   mapIndex));
             const GiftiLabel* label = labelTable->getLabel(dataValue);
             if (label != NULL) {
-                const GroupAndNameHierarchyItem* item = label->getGroupNameSelectionItem();
-                if (item != NULL) {
-                    if (item->isSelected(displayGroup, tabIndex) == false) {
-                        alpha = 0;
+                switch (tabDrawingInfo.getLabelViewMode()) {
+                    case LabelViewModeEnum::HIERARCHY:
+                        if (labelSelectionItemModel != NULL) {
+                            if ( ! labelSelectionItemModel->isLabelChecked(dataValue)) {
+                                alpha = 0;
+                            }
+                        }
+                        break;
+                    case LabelViewModeEnum::LIST:
+                    {
+                        const GroupAndNameHierarchyItem* item = label->getGroupNameSelectionItem();
+                        if (item != NULL) {
+                            if ( ! item->isSelected(tabDrawingInfo.getDisplayGroup(),
+                                                 tabDrawingInfo.getTabIndex())) {
+                                alpha = 0;
+                            }
+                        }
                     }
                 }
             }
@@ -717,7 +942,7 @@ VolumeFileVoxelColorizer::getVoxelColorInMap(const int64_t i,
  *    Index of map.
  */
 void
-VolumeFileVoxelColorizer::clearVoxelColoringForMap(const int64_t mapIndex)
+VolumeFileVoxelColorizer::clearVoxelColoringForMap(const int64_t mapIndex) const
 {
     CaretAssertVectorIndex(m_mapRGBA, mapIndex);
     uint8_t* mapRGBA = m_mapRGBA[mapIndex];
@@ -730,3 +955,34 @@ VolumeFileVoxelColorizer::clearVoxelColoringForMap(const int64_t mapIndex)
     m_mapColoringValid[mapIndex] = false;
 }
 
+/**
+ * Update the voxel coloring for the given voxels in the given map with the given RGBA coloring.
+ * This method is used by voxel editing to avoid recoloring all voxels in the volume
+ * @param voxelColorUpdate
+ *    Color update information
+ */
+void
+VolumeFileVoxelColorizer::updateVoxelColorsInMap(const VoxelColorUpdate& voxelColorUpdate)
+{
+    const int32_t mapIndex(voxelColorUpdate.getMapIndex());
+    CaretAssertVectorIndex(m_mapColoringValid, mapIndex);
+    
+    if (m_mapColoringValid[mapIndex]) {
+        CaretAssertVectorIndex(m_mapRGBA, mapIndex);
+        uint8_t* mapRGBA = m_mapRGBA[mapIndex];
+        
+        const std::array<uint8_t, 4> rgba(voxelColorUpdate.getRGBA());
+        const int32_t numVoxels(voxelColorUpdate.getNumberOfVoxels());
+        for (int32_t i = 0; i < numVoxels; i++) {
+            const VoxelIJK& ijk(voxelColorUpdate.getVoxel(i));
+            const int64_t rgbaOffset = getRgbaOffsetForVoxelIndex(ijk.m_ijk);
+            CaretAssertArrayIndex(mapRGBA, m_mapRGBACount, rgbaOffset);
+            
+            mapRGBA[rgbaOffset]   = rgba[0];
+            mapRGBA[rgbaOffset+1] = rgba[1];
+            mapRGBA[rgbaOffset+2] = rgba[2];
+            mapRGBA[rgbaOffset+3] = rgba[3];
+        }
+    }
+}
+
