@@ -23,13 +23,14 @@
 
 
 #include "CaretObject.h"
-#include "DisplayGroupEnum.h"
 #include "VolumeSliceViewPlaneEnum.h"
+#include "VoxelIJK.h"
 
 namespace caret {
 
-    class Palette;
+    class TabDrawingInfo;
     class VolumeFile;
+    class VoxelColorUpdate;
     
     class VolumeFileVoxelColorizer : public CaretObject {
         
@@ -38,16 +39,21 @@ namespace caret {
         
         virtual ~VolumeFileVoxelColorizer();
         
-        void assignVoxelColorsForMap(const int32_t mapIndex,
-                                     const Palette* palette,
-                                     const VolumeFile* thresholdVolume,
-                                     const int32_t thresholdVolumeMapIndex);
+        void assignVoxelColorsForMap(const int32_t mapIndex) const;
+        
+        int64_t getVoxelColorsForSliceInMap(const int32_t mapIndex,
+                                            const int64_t firstVoxelIJK[3],
+                                            const int64_t rowStepIJK[3],
+                                            const int64_t columnStepIJK[3],
+                                            const int64_t numberOfRows,
+                                            const int64_t numberOfColumns,
+                                            const TabDrawingInfo& tabDrawingInfo,
+                                            uint8_t* rgbaOut) const;
         
         int64_t getVoxelColorsForSliceInMap(const int32_t mapIndex,
                                          const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                          const int64_t sliceIndex,
-                                         const DisplayGroupEnum::Enum displayGroup,
-                                         const int32_t tabIndex,
+                                            const TabDrawingInfo& tabDrawingInfo,
                                          uint8_t* rgbaOut) const;
         
         int64_t getVoxelColorsForSubSliceInMap(const int32_t mapIndex,
@@ -55,20 +61,26 @@ namespace caret {
                                             const int64_t sliceIndex,
                                             const int64_t firstCornerVoxelIndex[3],
                                             const int64_t lastCornerVoxelIndex[3],
-                                            const int64_t voxelCountIJK[3],
-                                            const DisplayGroupEnum::Enum displayGroup,
-                                            const int32_t tabIndex,
+                                            const int64_t* voxelCountIJK,
+                                               const TabDrawingInfo& tabDrawingInfo,
                                             uint8_t* rgbaOut) const;
         
         void getVoxelColorInMap(const int64_t i,
                                 const int64_t j,
                                 const int64_t k,
                                 const int64_t mapIndex,
-                                const DisplayGroupEnum::Enum displayGroup,
-                                const int32_t tabIndex,
                                 uint8_t rgbaOut[4]) const;
         
-        void clearVoxelColoringForMap(const int64_t mapIndex);
+        void getVoxelColorInMap(const int64_t i,
+                                const int64_t j,
+                                const int64_t k,
+                                const int64_t mapIndex,
+                                const TabDrawingInfo& tabDrawingInfo,
+                                uint8_t rgbaOut[4]) const;
+        
+        void updateVoxelColorsInMap(const VoxelColorUpdate& voxelColorUpdate);
+        
+        void clearVoxelColoringForMap(const int64_t mapIndex) const;
         
         void invalidateColoring();
         
@@ -88,6 +100,18 @@ namespace caret {
                          + ((k * m_dimI * m_dimJ))));
         }
 
+        /**
+         * Get theRGBA offset for a voxel index
+         */
+        inline int64_t getRgbaOffsetForVoxelIndex(const int64_t ijk[3]) const {
+            return (4 * (ijk[0]
+                         + (ijk[1] * m_dimI)
+                         + ((ijk[2] * m_dimI * m_dimJ))));
+        }
+        
+        void applyColorModulation(const int32_t mapIndex,
+                                  const bool showZerosFlag) const;
+        
         // ADD_NEW_MEMBERS_HERE
 
         VolumeFile* m_volumeFile;
@@ -99,8 +123,8 @@ namespace caret {
         int64_t m_mapCount;
         int64_t m_mapRGBACount;
         
-        std::vector<bool> m_mapColoringValid;
-        std::vector<uint8_t*> m_mapRGBA;
+        mutable std::vector<bool> m_mapColoringValid;
+        mutable std::vector<uint8_t*> m_mapRGBA;
     };
     
 #ifdef __VOLUME_FILE_VOXEL_COLORIZER_DECLARE__
