@@ -26,6 +26,7 @@
 #include "AStringNaturalComparison.h"
 #include "CaretAssert.h"
 #include "CaretLogger.h"
+#include "CaretHierarchy.h"
 #include "GiftiLabel.h"
 #include "GiftiLabelTable.h"
 #include "GiftiXmlElements.h"
@@ -101,6 +102,7 @@ GiftiLabelTable::copyHelper(const GiftiLabelTable& glt)
             addLabel(iter->second);
         }
     }
+    m_hierarchy = glt.m_hierarchy;
 }
 
 void
@@ -123,6 +125,7 @@ GiftiLabelTable::initializeMembersGiftiLabelTable()
 void
 GiftiLabelTable::clear()
 {
+    m_hierarchy->clear();
     for (LABELS_MAP_CONST_ITERATOR iter = this->labelsMap.begin();
          iter != labelsMap.end();
          iter++) {
@@ -148,7 +151,7 @@ GiftiLabelTable::clear()
  *
  */
 std::map<int32_t,int32_t>
-GiftiLabelTable::append(const GiftiLabelTable& glt)
+GiftiLabelTable::append(const GiftiLabelTable& glt, const bool errorOnLabelConflict)
 {
     std::map<int32_t,int32_t> keyConverterMap;
     
@@ -156,8 +159,8 @@ GiftiLabelTable::append(const GiftiLabelTable& glt)
          iter != glt.labelsMap.end();
          iter++) {
         int32_t key = iter->first;
-        int32_t newKey = this->addLabel(iter->second);
-        
+        int32_t newKey = this->addLabel(iter->second, errorOnLabelConflict);
+
         keyConverterMap.insert(std::make_pair(key, newKey));
     }
     return keyConverterMap;
@@ -183,31 +186,10 @@ GiftiLabelTable::addLabel(
                    const float blue,
                    const float alpha)
 {
-    const GiftiLabel gl(GiftiLabel::getInvalidLabelKey(), labelName, red, green, blue, alpha);
+    const GiftiLabel gl(generateUnusedKey(), labelName, red, green, blue, alpha);
     return this->addLabel(&gl);
 }
 
-/**
- * Add a label.  If a label with the name exists, its colors
- * are replaced with these color components.
- * @param labelName Name of label.
- * @param red  Red color component ranging 0.0 to 1.0.
- * @param green Green color component ranging 0.0 to 1.0.
- * @param blue Blue color component ranging 0.0 to 1.0.
- * @return  Index of the existing label, or, if no label 
- * exists with name, index of new label.
- *
- */
-int32_t
-GiftiLabelTable::addLabel(
-                   const AString& labelName,
-                   const float red,
-                   const float green,
-                   const float blue)
-{
-    return this->addLabel(labelName, red, green, blue, 1.0f);
-}
-
 /**
  * Add a label.  If a label with the name exists, its colors
  * are replaced with these color components.
@@ -228,31 +210,10 @@ GiftiLabelTable::addLabel(
                    const int32_t blue,
                    const int32_t alpha)
 {
-    const GiftiLabel gl(GiftiLabel::getInvalidLabelKey(), labelName, red, green, blue, alpha);
+    const GiftiLabel gl(generateUnusedKey(), labelName, red, green, blue, alpha);
     return this->addLabel(&gl);
 }
 
-/**
- * Add a label.  If a label with the name exists, its colors
- * are replaced with these color components.
- * @param labelName Name of label.
- * @param red  Red color component ranging 0 to 255.
- * @param green Green color component ranging 0 to 255.
- * @param blue Blue color component ranging 0 to 255.
- * @return  Index of the existing label, or, if no label 
- * exists with name, index of new label.
- *
- */
-int32_t
-GiftiLabelTable::addLabel(
-                   const AString& labelName,
-                   const int32_t red,
-                   const int32_t green,
-                   const int32_t blue)
-{
-    return this->addLabel(labelName, red, green, blue, 255);
-}
-
 /**
  * Add a label to the label table.  If the label's key is already in
  * the label table, a new key is created.  If a label of the same
@@ -263,12 +224,16 @@ GiftiLabelTable::addLabel(
  *
  */
 int32_t
-GiftiLabelTable::addLabel(const GiftiLabel* glIn)
+GiftiLabelTable::addLabel(const GiftiLabel* glIn, const bool errorOnLabelConflict)
 {
     /*
      * First see if a label with the same name already exists
      */
     int32_t key = this->getLabelKeyFromName(glIn->getName());
+    if (key != GiftiLabel::getInvalidLabelKey() && key != glIn->getKey())
+    {//Matt says warn if this finds a key other than the one it already has
+        CaretLogWarning("merging keys for label '" + glIn->getName() + "' from key " + AString::number(glIn->getKey()) + " to key " + AString::number(key));
+    }
     
     /*
      * If no label with the name exists, get the key
@@ -287,9 +252,14 @@ GiftiLabelTable::addLabel(const GiftiLabel* glIn)
     /*
      * Still need a key, find an unused key
      */
-    if (key == GiftiLabel::getInvalidLabelKey()) {
+    if (key == GiftiLabel::getInvalidLabelKey()) {//addLabel without a key has been fixed to use generateUnused, not getInvalid...
         key = this->generateUnusedKey();
-        
+        if (errorOnLabelConflict)
+        {
+            throw CaretException("conflicting key value " + AString::number(glIn->getKey()) + " for label '" + glIn->getName() + "', please fix this and any additional conflicts with a label-modify-keys command, or an option in this command, if available");
+        } else {
+            CaretLogWarning("conflicting key value for label '" + glIn->getName() + "' reassigned to key " + AString::number(key));
+        }
         GiftiLabel* gl = new GiftiLabel(*glIn);
         gl->setKey(key);
         this->labelsMap.insert(std::make_pair(key, gl));
@@ -823,56 +793,6 @@ GiftiLabelTable::setLabel(const int32_t key,
     }
 }
 
-/**
- * Get the selection status of the label at the specified key.  If there
- * is no label at the key, false is returned.
- * @param key - key of label
- * @return  selection status of label.
- *
- */
-bool
-GiftiLabelTable::isLabelSelected(const int32_t key) const
-{
-    LABELS_MAP_CONST_ITERATOR iter = this->labelsMap.find(key);
-    if (iter != this->labelsMap.end()) {
-        return iter->second->isSelected();
-    }
-    return false;
-}
-
-/**
- * Set the selection status of a label.
- * @param key - key of label.
- * @param sel - new selection status.
- *
- */
-void
-GiftiLabelTable::setLabelSelected(
-                   const int32_t key,
-                   const bool sel)
-{
-    LABELS_MAP_ITERATOR iter = this->labelsMap.find(key);
-    if (iter != this->labelsMap.end()) {
-        iter->second->setSelected(sel);
-    }
-}
-
-/**
- * Set the selection status for all labels.
- * @param newStatus  New selection status.
- *
- */
-void
-GiftiLabelTable::setSelectionStatusForAllLabels(const bool newStatus)
-{
-    for (LABELS_MAP_ITERATOR iter = this->labelsMap.begin();
-         iter != this->labelsMap.end();
-         iter++) {
-        GiftiLabel* gl = iter->second;
-        gl->setSelected(newStatus);
-    }
-}
-
 /**
  * Get the alpha color component for a label.  If the key is not a
  * valid label, an alpha of zero is returned.
@@ -914,7 +834,7 @@ GiftiLabelTable::getLabelColor(const int32_t key, float rgbaOut[4]) const
 void
 GiftiLabelTable::setLabelColor(
                    const int32_t key,
-                   const float color[])
+                   const float color[4])
 {
     if (key == 0)
     {
@@ -977,6 +897,26 @@ GiftiLabelTable::resetLabelCounts()
     }
 }
 
+/**
+ * @return True if this label table contains a label with the
+ * name of the medial wall, else false.
+ */
+bool
+GiftiLabelTable::hasMedialWallLabel() const
+{
+    for (LABELS_MAP_CONST_ITERATOR iter = this->labelsMap.begin();
+         iter != this->labelsMap.end();
+         iter++) {
+        const GiftiLabel* label = iter->second;
+        CaretAssert(label);
+        if (label->isMedialWallName()) {
+            return true;
+        }
+    }
+    
+    return false;
+}
+
 ///**
 // * @return Are there any labels that have an invalid group/name
 // * hierarchy settings.  This can be caused by changing the name
@@ -1729,4 +1669,19 @@ GiftiLabelTable::exportToCaret5ColorFile(const AString& filename) const
     file.close();
 }
 
+void GiftiLabelTable::setHierarchy(const CaretHierarchy& hierarchy)
+{
+    m_hierarchy = hierarchy; //for =, it pretends to be a non-pointer member
+    setModified();
+}
 
+void GiftiLabelTable::clearHierarchy()
+{
+    m_hierarchy->clear(); //can't overload . operator in forward-declare helper, need -> for member access
+    setModified();
+}
+
+const CaretHierarchy& GiftiLabelTable::getHierarchy() const
+{
+    return m_hierarchy; //converts to template type
+}
