@@ -25,6 +25,7 @@
 #include "CaretObject.h"
 #include "TracksModificationInterface.h"
 
+#include "CaretPointer.h"
 #include "GiftiException.h"
 
 #include <map>
@@ -37,6 +38,7 @@
 
 namespace caret {
 
+class CaretHierarchy;
 class GiftiLabel;
     
 class XmlWriter;
@@ -70,35 +72,23 @@ private:
 public:
     void clear();
 
-    std::map<int32_t,int32_t> append(const GiftiLabelTable& glt);
+    std::map<int32_t,int32_t> append(const GiftiLabelTable& glt, const bool errorOnLabelConflict = false);
 
     int32_t addLabel(
                     const AString& labelName,
                     const float red,
                     const float green,
                     const float blue,
-                    const float alpha);
-
-    int32_t addLabel(
-                    const AString& labelName,
-                    const float red,
-                    const float green,
-                    const float blue);
+                    const float alpha = 1.0f);
 
     int32_t addLabel(
                     const AString& labelName,
                     const int32_t red,
                     const int32_t green,
                     const int32_t blue,
-                    const int32_t alpha);
+                    const int32_t alpha = 255);
 
-    int32_t addLabel(
-                    const AString& labelName,
-                    const int32_t red,
-                    const int32_t green,
-                    const int32_t blue);
-
-    int32_t addLabel(const GiftiLabel* glt);
+    int32_t addLabel(const GiftiLabel* glt, const bool errorOnLabelConflict = false);
 
     void deleteLabel(const int32_t key);
 
@@ -147,14 +137,6 @@ public:
                   const float y, 
                   const float z);
     
-    bool isLabelSelected(const int32_t key) const;
-
-    void setLabelSelected(
-                    const int32_t key,
-                    const bool sel);
-
-    void setSelectionStatusForAllLabels(const bool newStatus);
-
     float getLabelAlpha(const int32_t key) const;
 
     void getLabelColor(const int32_t key, float rgbaOut[4]) const;
@@ -171,6 +153,8 @@ public:
 
     void createLabelsForKeys(const std::set<int32_t>& newKeys);
 
+    bool hasMedialWallLabel() const;
+    
     void writeAsXML(XmlWriter& xmlWriter);
 
     void writeAsXML(QXmlStreamWriter& xmlWriter) const;
@@ -208,6 +192,10 @@ public:
                         const int32_t newKey);
     
     void exportToCaret5ColorFile(const AString& filename) const;
+    
+    void setHierarchy(const CaretHierarchy& hierarchy);
+    void clearHierarchy();
+    const CaretHierarchy& getHierarchy() const;
 
 private:
     void issueLabelKeyZeroWarning(const AString& name) const;
@@ -232,6 +220,7 @@ private:
     int32_t m_tableModelColumnIndexBlue;
     int32_t m_tableModelColumnCount;
     
+    CaretForwardHelper<CaretHierarchy> m_hierarchy; //helper for implementing forward-declared members with reduced per-class code
 };
 
 } // namespace
