@@ -21,15 +21,21 @@
  */
 /*LICENSE_END*/
 
-#include "DisplayGroupEnum.h"
+#include "Vector3D.h"
 #include "VolumeSliceViewPlaneEnum.h"
 #include "VolumeSpace.h"
+#include "VolumeToImageMappingModeEnum.h"
 
 namespace caret {
     
     class BoundingBox;
-    class PaletteFile;
-    
+    class CaretMappableDataFile;
+    class HistologySlice;
+    class MediaFile;
+    class GraphicsPrimitive;
+    class GraphicsPrimitiveV3fT3f;
+    class TabDrawingInfo;
+
     /**
      * \class caret::VolumeMappableInterface
      * \brief Interface for data that is mapped to volumes
@@ -50,6 +56,15 @@ namespace caret {
         VolumeMappableInterface& operator=(const VolumeMappableInterface&);
         
     public:
+        /**
+         * @return Instance cast to a Volume Mappable CaretMappableDataFile
+         */
+        virtual CaretMappableDataFile* castToVolumeMappableDataFile() = 0;
+        
+        /**
+         * @return Instance cast to a Volume Mappable CaretMappableDataFile (const method)
+         */
+        virtual const CaretMappableDataFile* castToVolumeMappableDataFile() const = 0;
         
         /**
          * Get the dimensions of the volume.
@@ -80,6 +95,10 @@ namespace caret {
          */
         virtual void getDimensions(std::vector<int64_t>& dimsOut) const = 0;
 
+        void getDimensionsPCA(int64_t& dimParasagittalOut,
+                              int64_t& dimCoronalOut,
+                              int64_t& dimAxialOut) const;
+
         bool matchesDimensions(const int64_t dim1,
                                const int64_t dim2,
                                const int64_t dim3) const;
@@ -180,6 +199,22 @@ namespace caret {
                                   float* coordOut) const = 0;
         
         /**
+         * Convert an index to space (coordinates).
+         *
+         * @param indexIn1
+         *     First dimension (i).
+         * @param indexIn2
+         *     Second dimension (j).
+         * @param indexIn3
+         *     Third dimension (k).
+         * @return coordOut
+         *     The XYZ coordinate.
+         */
+        Vector3D indexToSpace(const float& indexIn1,
+                              const float& indexIn2,
+                              const float& indexIn3) const;
+
+       /**
          * Convert an index to space (coordinates).
          *
          * @param indexIn
@@ -192,7 +227,7 @@ namespace caret {
         
         /**
          * Convert a coordinate to indices.  Note that output indices
-         * MAY NOT BE WITHING THE VALID VOXEL DIMENSIONS.
+         * MAY NOT BE WITHIN THE VALID VOXEL DIMENSIONS.
          *
          * @param coordIn1
          *     First (x) input coordinate.
@@ -214,6 +249,18 @@ namespace caret {
                                     int64_t& indexOut2,
                                     int64_t& indexOut3) const = 0;
         
+        /**
+         * Convert a coordinate to indices.  Note that output indices
+         * MAY NOT BE WITHIN THE VALID VOXEL DIMENSIONS.
+         *
+         * @param coordIn1
+         *     First (x) input coordinate.
+         * @param coordIn2
+         *     Second (y) input coordinate.
+         */
+        virtual void enclosingVoxel(const float* coordIn,
+                                    int64_t* indexOut) const = 0;
+
         /**
          * Determine in the given voxel indices are valid (within the volume).
          *
@@ -236,6 +283,20 @@ namespace caret {
                                 const int64_t brickIndex = 0,
                                 const int64_t component = 0) const = 0;
         
+        /**
+         * Determine in the given voxel indices are valid (within the volume).
+         *
+         * @param indexIn
+         *     IJK
+         * @param brickIndex
+         *     Time/map index (default 0).
+         * @param component
+         *     Voxel component (default 0).
+         */
+        virtual bool indexValid(const int64_t* indexIn,
+                                const int64_t brickIndex = 0,
+                                const int64_t component = 0) const = 0;
+        
         /**
          * Get a bounding box for the voxel coordinate ranges.
          *
@@ -244,6 +305,16 @@ namespace caret {
          */
         virtual void getVoxelSpaceBoundingBox(BoundingBox& boundingBoxOut) const = 0;
         
+        /**
+         * Get a bounding box containing the non-zero voxel coordinate ranges
+         * @param mapIndex
+         *    Index of map
+         * @param boundingBoxOut
+         *    Output containing coordinate range of non-zero voxels
+         */
+        virtual void getNonZeroVoxelCoordinateBoundingBox(const int32_t mapIndex,
+                                                   BoundingBox& boundingBoxOut) const = 0;
+        
         /**
          * Get the voxel spacing for each of the spatial dimensions.
          *
@@ -258,40 +329,79 @@ namespace caret {
                              float& spacingOut2,
                              float& spacingOut3) const;
         
+        /**
+         * Get the voxel spacing for parasagittal (X), coronal (Y), and axial (Z)
+         *
+         * @param spacingParasagittalXOut
+         *    Spacing for the first dimension (typically X).
+         * @param spacingCoronalYOut
+         *    Spacing for the first dimension (typically Y).
+         * @param spacingAxialZOut
+         *    Spacing for the first dimension (typically Z).
+         */
+        void getVoxelSpacingPCA(float& spacingParasagittalXOut,
+                                float& spacingCoronalYOut,
+                                float& spacingAxialZOut) const;
+
+        float getMaximumVoxelSpacing() const;
+        
         /**
          * Get the voxel colors for a slice in the map.
          *
-         * @param paletteFile
-         *    The palette file.
          * @param mapIndex
          *    Index of the map.
          * @param slicePlane
          *    The slice plane.
          * @param sliceIndex
          *    Index of the slice.
-         * @param displayGroup
-         *    The selected display group.
-         * @param tabIndex
-         *    Index of selected tab.
+         * @param tabDrawingInfo
+         *    Info for drawing the tab
          * @param rgbaOut
          *    Output containing the rgba values (must have been allocated
          *    by caller to sufficient count of elements in the slice).
          * @return
          *    Number of voxels with alpha greater than zero
          */
-        virtual int64_t getVoxelColorsForSliceInMap(const PaletteFile* paletteFile,
-                                                    const int32_t mapIndex,
+        virtual int64_t getVoxelColorsForSliceInMap(const int32_t mapIndex,
                                                     const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                                     const int64_t sliceIndex,
-                                                    const DisplayGroupEnum::Enum displayGroup,
-                                                    const int32_t tabIndex,
+                                                    const TabDrawingInfo& tabDrawingInfo,
+                                                    uint8_t* rgbaOut) const = 0;
+        
+        /**
+         * Get voxel coloring for a set of voxels.
+         *
+         * @param mapIndex
+         *     Index of map.
+         * @param firstVoxelIJK
+         *    IJK Indices of first voxel
+         * @param rowStepIJK
+         *    IJK Step for moving to next row.
+         * @param columnStepIJK
+         *    IJK Step for moving to next column.
+         * @param numberOfRows
+         *    Number of rows.
+         * @param numberOfColumns
+         *    Number of columns.
+         * @param tabDrawingInfo
+         *    Info for drawing the tab
+         * @param rgbaOut
+         *    RGBA color components out.
+         * @return
+         *    Number of voxels with alpha greater than zero
+         */
+        virtual int64_t getVoxelColorsForSliceInMap(const int32_t mapIndex,
+                                                    const int64_t firstVoxelIJK[3],
+                                                    const int64_t rowStepIJK[3],
+                                                    const int64_t columnStepIJK[3],
+                                                    const int64_t numberOfRows,
+                                                    const int64_t numberOfColumns,
+                                                    const TabDrawingInfo& tabDrawingInfo,
                                                     uint8_t* rgbaOut) const = 0;
         
         /**
          * Get the voxel colors for a sub slice in the map.
          *
-         * @param paletteFile
-         *    The palette file.
          * @param mapIndex
          *    Index of the map.
          * @param slicePlane
@@ -304,32 +414,26 @@ namespace caret {
          *    Indices of voxel for last corner of sub-slice (inclusive).
          * @param voxelCountIJK
          *    Voxel counts for each axis.
-         * @param displayGroup
-         *    The selected display group.
-         * @param tabIndex
-         *    Index of selected tab.
+         * @param tabDrawingInfo
+         *    Info for drawing the tab
          * @param rgbaOut
          *    Output containing the rgba values (must have been allocated
          *    by caller to sufficient count of elements in the slice).
          * @return
          *    Number of voxels with alpha greater than zero
          */
-        virtual int64_t getVoxelColorsForSubSliceInMap(const PaletteFile* paletteFile,
-                                                       const int32_t mapIndex,
+        virtual int64_t getVoxelColorsForSubSliceInMap(const int32_t mapIndex,
                                                        const VolumeSliceViewPlaneEnum::Enum slicePlane,
                                                        const int64_t sliceIndex,
                                                        const int64_t firstCornerVoxelIndex[3],
                                                        const int64_t lastCornerVoxelIndex[3],
                                                        const int64_t voxelCountIJK[3],
-                                                       const DisplayGroupEnum::Enum displayGroup,
-                                                       const int32_t tabIndex,
+                                                       const TabDrawingInfo& tabDrawingInfo,
                                                        uint8_t* rgbaOut) const = 0;
         
         /**
          * Get the voxel coloring for the voxel at the given indices.
          *
-         * @param paletteFile
-         *    The palette file.
          * @param indexIn1
          *     First dimension (i).
          * @param indexIn2
@@ -338,22 +442,61 @@ namespace caret {
          *     Third dimension (k).
          * @param brickIndex
          *     Time/map index.
-         * @param displayGroup
-         *    The selected display group.
-         * @param tabIndex
-         *    Index of selected tab.
+         * @param tabDrawingInfo
+         *    Info for drawing the tab
          * @param rgbaOut
          *     Output containing RGBA values for voxel at the given indices.
          */
-        virtual void getVoxelColorInMap(const PaletteFile* paletteFile,
-                                        const int64_t indexIn1,
+        virtual void getVoxelColorInMap(const int64_t indexIn1,
                                         const int64_t indexIn2,
                                         const int64_t indexIn3,
                                         const int64_t brickIndex,
-                                        const DisplayGroupEnum::Enum displayGroup,
-                                        const int32_t tabIndex,
+                                        const TabDrawingInfo& tabDrawingInfo,
                                         uint8_t rgbaOut[4]) const = 0;
         
+        /**
+         * Get the voxel coloring for the voxel at the given indices.
+         *
+         * @param indexIn1
+         *     First dimension (i).
+         * @param indexIn2
+         *     Second dimension (j).
+         * @param indexIn3
+         *     Third dimension (k).
+         * @param brickIndex
+         *     Time/map index.
+         * @param rgbaOut
+         *     Output containing RGBA values for voxel at the given indices.
+         */
+        virtual void getVoxelColorInMap(const int64_t indexIn1,
+                                        const int64_t indexIn2,
+                                        const int64_t indexIn3,
+                                        const int64_t tabIndex,
+                                        uint8_t rgbaOut[4]) const = 0;
+        
+        virtual GraphicsPrimitiveV3fT3f* getVolumeDrawingTriangleStripPrimitive(const int32_t mapIndex,
+                                                                                const TabDrawingInfo& tabDrawingInfo) const = 0;
+        
+        virtual GraphicsPrimitiveV3fT3f* getVolumeDrawingTriangleFanPrimitive(const int32_t mapIndex,
+                                                                              const TabDrawingInfo& tabDrawingInfo) const = 0;
+        
+        virtual GraphicsPrimitiveV3fT3f* getVolumeDrawingTrianglesPrimitive(const int32_t mapIndex,
+                                                                            const TabDrawingInfo& tabDrawingInfo) const = 0;
+
+        virtual GraphicsPrimitive* getHistologyImageIntersectionPrimitive(const int32_t mapIndex,
+                                                                          const TabDrawingInfo& tabDrawingInfo,
+                                                                          const MediaFile* mediaFile,
+                                                                          const VolumeToImageMappingModeEnum::Enum volumeMappingMode,
+                                                                          const float volumeSliceThickness,
+                                                                          AString& errorMessageOut) const = 0;
+        
+        virtual std::vector<GraphicsPrimitive*> getHistologySliceIntersectionPrimitive(const int32_t mapIndex,
+                                                                                       const TabDrawingInfo& tabDrawingInfo,
+                                                                                       const HistologySlice* histologySlice,
+                                                                                       const VolumeToImageMappingModeEnum::Enum volumeMappingMode,
+                                                                                       const float volumeSliceThickness,
+                                                                                       AString& errorMessageOut) const = 0;
+
         /**
          * Get the volume space object, so we have access to all functions associated with volume spaces
          */
