@@ -25,7 +25,7 @@
 #include "CaretLogger.h"
 #include "FloatMatrix.h"
 
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QStringList>
 
 #include <algorithm>
@@ -48,14 +48,14 @@ VolumeSpace::VolumeSpace(const int64_t dims[3], const vector<vector<float> >& sf
     setSpace(dims, sform);
 }
 
-VolumeSpace::VolumeSpace(const int64_t dims[3], const float sform[16])
+VolumeSpace::VolumeSpace(const int64_t dims[3], const float sform[12])
 {
     setSpace(dims, sform);
 }
 
 void VolumeSpace::setSpace(const int64_t dims[3], const vector<vector<float> >& sform)
 {
-    if (sform.size() < 2 || sform.size() > 4)
+    if (sform.size() < 3 || sform.size() > 4)
     {
         CaretAssert(false);
         throw CaretException("VolumeSpace initialized with wrong size sform");
@@ -170,6 +170,13 @@ void VolumeSpace::getSpacingVectors(Vector3D& iStep, Vector3D& jStep, Vector3D&
     FloatMatrix(m_sform).getAffineVectors(iStep, jStep, kStep, origin);
 }
 
+float VolumeSpace::getVoxelVolume() const
+{
+    Vector3D spacingVecs[4];
+    getSpacingVectors(spacingVecs[0], spacingVecs[1], spacingVecs[2], spacingVecs[3]);
+    return abs(spacingVecs[0].dot(spacingVecs[1].cross(spacingVecs[2])));
+}
+
 void VolumeSpace::getOrientAndSpacingForPlumb(OrientTypes* orientOut, float* spacingOut, float* originOut) const
 {
     CaretAssert(isPlumb());
@@ -185,13 +192,7 @@ void VolumeSpace::getOrientAndSpacingForPlumb(OrientTypes* orientOut, float* spa
             {
                 spacingOut[j] = m_sform[i][j];
                 originOut[j] = m_sform[i][3];
-                bool negative;
-                if (m_sform[i][j] > 0.0f)
-                {
-                    negative = true;
-                } else {
-                    negative = false;
-                }
+                bool negative = (m_sform[i][j] < 0.0f);
                 switch (i)
                 {
                 case 0:
@@ -215,7 +216,7 @@ void VolumeSpace::getOrientAndSpacingForPlumb(OrientTypes* orientOut, float* spa
     }
 }
 
-void VolumeSpace::getOrientation(VolumeSpace::OrientTypes orientOut[3]) const
+void VolumeSpace::getOrientation(OrientTypes orientOut[3]) const
 {
     Vector3D ivec, jvec, kvec, origin;
     getSpacingVectors(ivec, jvec, kvec, origin);
@@ -336,9 +337,9 @@ void VolumeSpace::readCiftiXML1(QXmlStreamReader& xml)
     {
         throw CaretException("failed to find TransformationMatrixVoxelIndicesIJKtoXYZ element in Volume");
     }
-    if (xml.name() != "TransformationMatrixVoxelIndicesIJKtoXYZ")
+    if (xml.name() != QLatin1String("TransformationMatrixVoxelIndicesIJKtoXYZ"))
     {
-        throw CaretException("found unknown element in Volume: " + xml.name().toString());
+        throw CaretException("unexpected element in Volume: " + xml.name().toString());
     }
     QXmlStreamAttributes transattribs = xml.attributes();
     if (!transattribs.hasAttribute("UnitsXYZ"))//ignore the space attributes
@@ -346,18 +347,22 @@ void VolumeSpace::readCiftiXML1(QXmlStreamReader& xml)
         throw CaretException("missing UnitsXYZ attribute in TransformationMatrixVoxelIndicesIJKtoXYZ");
     }
     float mult = 0.0f;
-    QStringRef unitstring = transattribs.value("UnitsXYZ");
-    if (unitstring == "NIFTI_UNITS_MM")
+    auto unitstring = transattribs.value("UnitsXYZ");
+    if (unitstring == QLatin1String("NIFTI_UNITS_MM"))
     {
         mult = 1.0f;
-    } else if (unitstring == "NIFTI_UNITS_MICRON") {
+    } else if (unitstring == QLatin1String("NIFTI_UNITS_MICRON")) {
         mult = 0.001f;
     } else {
         throw CaretException("unrecognized value for UnitsXYZ in TransformationMatrixVoxelIndicesIJKtoXYZ: " + unitstring.toString());
     }
     QString accum = xml.readElementText();
     if (xml.hasError()) return;
-    QStringList matrixStrings = accum.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+#if QT_VERSION >= 0x060000
+    QStringList matrixStrings = accum.split(QRegularExpression("\\s+"), Qt::SkipEmptyParts);
+#else
+    QStringList matrixStrings = accum.split(QRegularExpression("\\s+"), QString::SkipEmptyParts);
+#endif
     if (matrixStrings.size() != 16)
     {
         throw CaretException("text content of TransformationMatrixVoxelIndicesIJKtoXYZ must have exactly 16 numbers separated by whitespace");
@@ -385,7 +390,7 @@ void VolumeSpace::readCiftiXML1(QXmlStreamReader& xml)
     newsform *= mult;//apply units
     newsform[3][3] = 1.0f;//reset [3][3], since it isn't spatial
     setSpace(newDims, newsform.getMatrix());//use public function so it can deal with invariants, and we can move this code elsewhere if needed
-    CaretAssert(xml.isEndElement() && xml.name() == "Volume");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("Volume"));
 }
 
 void VolumeSpace::readCiftiXML2(QXmlStreamReader& xml)
@@ -418,16 +423,16 @@ void VolumeSpace::readCiftiXML2(QXmlStreamReader& xml)
     {
         throw CaretException("failed to find TransformationMatrixVoxelIndicesIJKtoXYZ element in Volume");
     }
-    if (xml.name() != "TransformationMatrixVoxelIndicesIJKtoXYZ")
+    if (xml.name() != QLatin1String("TransformationMatrixVoxelIndicesIJKtoXYZ"))
     {
-        throw CaretException("found unknown element in Volume: " + xml.name().toString());
+        throw CaretException("unexpected element in Volume: " + xml.name().toString());
     }
     QXmlStreamAttributes transattribs = xml.attributes();
     if (!transattribs.hasAttribute("MeterExponent"))//ignore the space attributes
     {
         throw CaretException("missing MeterExponent attribute in TransformationMatrixVoxelIndicesIJKtoXYZ");
     }
-    QStringRef exponentstring = transattribs.value("MeterExponent");
+    auto exponentstring = transattribs.value("MeterExponent");
     int exponent = exponentstring.toString().toInt(&ok);
     if (!ok)
     {
@@ -436,7 +441,11 @@ void VolumeSpace::readCiftiXML2(QXmlStreamReader& xml)
     float mult = pow(10.0f, exponent + 3);//because our internal units are mm
     QString accum = xml.readElementText();
     if (xml.hasError()) return;
-    QStringList matrixStrings = accum.split(QRegExp("\\s+"), QString::SkipEmptyParts);
+#if QT_VERSION >= 0x060000
+    QStringList matrixStrings = accum.split(QRegularExpression("\\s+"), Qt::SkipEmptyParts);
+#else
+    QStringList matrixStrings = accum.split(QRegularExpression("\\s+"), QString::SkipEmptyParts);
+#endif
     if (matrixStrings.size() != 16)
     {
         throw CaretException("text content of TransformationMatrixVoxelIndicesIJKtoXYZ must have exactly 16 numbers separated by whitespace");
@@ -464,7 +473,7 @@ void VolumeSpace::readCiftiXML2(QXmlStreamReader& xml)
     newsform *= mult;//apply units
     newsform[3][3] = 1.0f;//reset [3][3], since it isn't spatial
     setSpace(newDims, newsform.getMatrix());//use public function so it can deal with invariants, and we can move this code elsewhere if needed
-    CaretAssert(xml.isEndElement() && xml.name() == "Volume");
+    CaretAssert(xml.isEndElement() && xml.name() == QLatin1String("Volume"));
 }
 
 void VolumeSpace::writeCiftiXML1(QXmlStreamWriter& xml) const
