@@ -267,7 +267,7 @@ GiftiDataArray::deleteRows(const std::vector<int32_t>& rowsToDeleteIn)
    //
    std::vector<int32_t> rowsToDelete = rowsToDeleteIn;
    std::sort(rowsToDelete.begin(), rowsToDelete.end());
-   std::unique(rowsToDelete.begin(), rowsToDelete.end());
+   (void)std::unique(rowsToDelete.begin(), rowsToDelete.end());
    std::reverse(rowsToDelete.begin(), rowsToDelete.end());
    
    //
@@ -621,6 +621,7 @@ GiftiDataArray::readFromText(const AString text,
                         float* ptr = dataPointerFloat;
                         for (int64_t i = 0; i < numElements; i++) {
                            stream >> *ptr;
+                           if (!stream) throw GiftiException("premature end of ascii-encoded gifti array");
                            ptr++;
                         }
                      }
@@ -630,6 +631,7 @@ GiftiDataArray::readFromText(const AString text,
                         int32_t* ptr = dataPointerInt;
                         for (int64_t i = 0; i < numElements; i++) {
                            stream >> *ptr;
+                           if (!stream) throw GiftiException("premature end of ascii-encoded gifti array");
                            ptr++;
                         }
                      }
@@ -640,6 +642,7 @@ GiftiDataArray::readFromText(const AString text,
                          int32_t c;
                         for (int64_t i = 0; i < numElements; i++) {
                            stream >> c;
+                           if (!stream) throw GiftiException("premature end of ascii-encoded gifti array");
                            *ptr = static_cast<uint8_t>(c);
                            ptr++;
                         }
@@ -658,7 +661,8 @@ GiftiDataArray::readFromText(const AString text,
                const uint64_t numDecoded =
                      Base64::decode((const unsigned char*)(text.toStdString().c_str()),
                                                 data.size(),
-                                                &data[0]);
+                                                &data[0],
+                                                text.size());
                if (numDecoded != data.size()) {
                   std::ostringstream str;
                   str << "Decoding of Base64 Binary data failed.\n"
@@ -680,12 +684,13 @@ GiftiDataArray::readFromText(const AString text,
                //
                // Decode the Base64 data using VTK's algorithm
                //
-               unsigned char* dataBuffer = new unsigned char[data.size()];
+               std::vector<unsigned char> dataBuffer(text.size() - text.size() / 4 + 10);//generous constant to make up for integer rounding
                //const char* textChars = text.toLatin1().constData();
                const uint64_t numDecoded =
                      Base64::decode((unsigned char*)text.toStdString().c_str(),
-                                                data.size(),
-                                                dataBuffer);
+                                                dataBuffer.size(),
+                                                dataBuffer.data(),
+                                                text.size());
                if (numDecoded == 0) {
                    std::ostringstream str;
                    str << "Decoding of GZip Base64 Binary data failed."
@@ -700,7 +705,7 @@ GiftiDataArray::readFromText(const AString text,
                // 
                 DataCompressZLib compressor;
                 const uint64_t uncompressedDataLength = 
-                                   compressor.uncompressData(dataBuffer,
+                                   compressor.uncompressData(dataBuffer.data(),
                                                           numDecoded,
                                                           (unsigned char*)&data[0],
                                                           data.size());
@@ -712,11 +717,6 @@ GiftiDataArray::readFromText(const AString text,
                   throw GiftiException(AString::fromStdString(str.str()));
                }
                
-               //
-               // Free memory
-               //
-               delete[] dataBuffer;
-               
                //
                // Is byte swapping needed ? 
                //
@@ -782,7 +782,7 @@ GiftiDataArray::readFromText(const AString text,
                      throw GiftiException("Tried to read "
                                          + AString::number((int64_t)numberOfBytesToRead)
                                          + " from "
-                                         + externalFileOffsetForReading
+                                         + AString::number(externalFileOffsetForReading)
                                          + " but failed");
                   }
                   
@@ -1205,36 +1205,30 @@ GiftiDataArray::writeAsXML(std::ostream& stream,
             // Compress the data with VTK's ZLIB algorithm
             //
              DataCompressZLib compressor;
-             unsigned long compressedDataBufferLength = 
+             uint64_t compressedDataBufferLength =
                               compressor.getMaximumCompressionSpace(data.size());
-            unsigned char* compressedDataBuffer = new unsigned char[compressedDataBufferLength];
-            unsigned long compressedDataLength =
+            std::vector<unsigned char> compressedDataBuffer(compressedDataBufferLength);
+            uint64_t compressedDataLength =
                           compressor.compressData(&data[0], 
                                                data.size(),
-                                               compressedDataBuffer,
+                                               compressedDataBuffer.data(),
                                                compressedDataBufferLength);
             
             //
             // Encode the data with VTK's Base64 algorithm
             //
-            char* buffer = new char[static_cast<int64_t>(compressedDataLength * 1.5)];
+            std::vector<unsigned char> buffer(compressedDataLength + compressedDataLength / 3 + 10);//generous constant for partial bytes, integer rounding, and possible "=" formatting
             const uint64_t compressedLength =
-               Base64::encode(compressedDataBuffer,
+               Base64::encode(compressedDataBuffer.data(),
                                           compressedDataLength,
-                                          (unsigned char*)buffer);
+                                          buffer.data());
             buffer[compressedLength] = '\0';
             
              //
              // Write the data  MUST BE NO space around data
              //
-             xmlWriter.writeElementNoSpace(GiftiXmlElements::TAG_DATA, buffer);
+             xmlWriter.writeElementNoSpace(GiftiXmlElements::TAG_DATA, (char*)buffer.data());
              
-            
-            //
-            // Free memory
-            //
-            delete[] buffer;
-            delete[] compressedDataBuffer;
          }
          break;
        case GiftiEncodingEnum::EXTERNAL_FILE_BINARY:
@@ -1723,10 +1717,21 @@ const FastStatistics* GiftiDataArray::getFastStatistics() const
     return m_fastStatistics;
 }
 
+/**
+ * Invalidate the histograms
+ */
+void
+GiftiDataArray::invalidateHistograms()
+{
+    m_histogramNeedsUpdateFlag = true;
+    m_histogramLimitedValuesNeedsUpdateFlag = true;
+}
+
 const Histogram* GiftiDataArray::getHistogram(const int32_t numberOfBuckets) const
 {
     if (this->getDataType() == NiftiDataTypeEnum::NIFTI_TYPE_FLOAT32) {
-        bool updateHistogramFlag = false;
+        bool updateHistogramFlag = m_histogramNeedsUpdateFlag;
+        m_histogramNeedsUpdateFlag = false;
         if (m_histogram == NULL) {
             m_histogram.grabNew(new Histogram(numberOfBuckets));
             updateHistogramFlag = true;
@@ -1788,7 +1793,8 @@ const Histogram* GiftiDataArray::getHistogram(const int32_t numberOfBuckets,
                                               const bool includeZeroValues) const
 {
     if (this->getDataType() == NiftiDataTypeEnum::NIFTI_TYPE_FLOAT32) {
-        bool updateHistogramFlag = false;
+        bool updateHistogramFlag = m_histogramLimitedValuesNeedsUpdateFlag;
+        m_histogramLimitedValuesNeedsUpdateFlag = false;
         if (m_histogramLimitedValues == NULL)
         {
             m_histogramLimitedValues.grabNew(new Histogram(numberOfBuckets));
