@@ -18,13 +18,15 @@
  */
 /*LICENSE_END*/
 
+#include <cstdio>
 #include <memory>
 #include <set>
 #include <sstream>
 
 #include "CaretAssert.h"
+#include "CaretHierarchy.h"
 #include "CaretLogger.h"
-
+#include "DataFileException.h"
 #include "FileInformation.h"
 #include "GiftiEncodingEnum.h"
 #define __GIFTI_FILE_MAIN__
@@ -37,6 +39,9 @@
 
 #include "XmlSaxParser.h"
 
+#include <QDir>
+#include <QFile>
+
 using namespace caret;
 
 /**
@@ -557,7 +562,7 @@ GiftiFile::addDataArray(GiftiDataArray* gda)
  * append a gifti array data  file to this one.
  */
 void 
-GiftiFile::append(const GiftiFile& gf) throw (GiftiException)
+GiftiFile::append(const GiftiFile& gf)
 {
     const bool copyDataArrayFlag = true;
    const int32_t numArrays = gf.getNumberOfDataArrays();
@@ -611,7 +616,7 @@ GiftiFile::append(const GiftiFile& gf) throw (GiftiException)
  */
 void 
 GiftiFile::append(const GiftiFile& gf, 
-                     std::vector<int32_t>& indexDestination) throw (GiftiException)
+                     std::vector<int32_t>& indexDestination)
 {
     const bool copyDataArrayFlag = true;
    const int32_t numArrays = gf.getNumberOfDataArrays();
@@ -838,13 +843,13 @@ GiftiFile::removeDataArray(const GiftiDataArray* arrayPointer)
  * read the file.
  */
 void
-GiftiFile::readFile(const AString& filename) throw (DataFileException)
+GiftiFile::readFile(const AString& filename)
 {
     this->clear();
     this->setFileName(filename);
     
     GiftiFileSaxReader saxReader(this);
-    std::auto_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
+    std::unique_ptr<XmlSaxParser> parser(XmlSaxParser::createXmlParser());
     try {
         parser->parseFile(filename, &saxReader);
     }
@@ -856,10 +861,9 @@ GiftiFile::readFile(const AString& filename) throw (DataFileException)
         int colNum  = e.getColumnNumber();
         
         std::ostringstream str;
-        str << "Parse Error while reading "
-        << filename.toStdString();
+        str << "File is not a valid GIFTI file";
         if ((lineNum >= 0) && (colNum >= 0)) {
-            str << " line/col ("
+            str << ", error parsing at line/col ("
             << e.getLineNumber()
             << "/"
             << e.getColumnNumber()
@@ -867,7 +871,8 @@ GiftiFile::readFile(const AString& filename) throw (DataFileException)
         }
         str << ": "
             << e.whatString().toStdString();
-        throw DataFileException(AString::fromStdString(str.str()));
+        throw DataFileException(filename,
+                                AString::fromStdString(str.str()));
     }
     
     /*
@@ -883,24 +888,42 @@ GiftiFile::readFile(const AString& filename) throw (DataFileException)
                              arrayName);
         }
     }
+    
+    AString hierMDtext = metaData.get("CaretHierarchy");
+    if (hierMDtext != "")
+    {
+        try {
+            CaretHierarchy tempHier;
+            tempHier.readXML(hierMDtext);
+            labelTable.setHierarchy(tempHier);
+        } catch (const CaretException& e) {
+            CaretLogWarning("error parsing hierarchy metadata: " + e.whatString());
+        } catch (...) {
+            CaretLogWarning("unknown error parsing hierarchy metadata");
+        }
+    }
 }
 
 /**
  * write the file. 
  */
 void 
-GiftiFile::writeFile(const AString& filename) throw (DataFileException)
+GiftiFile::writeFile(const AString& filename)
 {
     try {
         this->setFileName(filename);
         
-        FileInformation fileInfo(filename);
+        //QFile::remove(filename);
+        remove(QDir::toNativeSeparators(filename).toLocal8Bit());//QFile::remove inappropriately checks file permissions and refuses to try deleting (when folder permissions may allow it)
+
+        
+        /*FileInformation fileInfo(filename);
         if (fileInfo.exists()) {
             //if (GiftiDataArrayFile.isFileOverwriteAllowed() == false) {
             //    throw new GiftiException(
             //                             "Overwriting of existing files is currently prohibited");
             //}
-        }
+        }//*/
         
         //
         // Create a GIFTI Data Array File Writer
@@ -929,7 +952,8 @@ GiftiFile::writeFile(const AString& filename) throw (DataFileException)
         giftiFileWriter.finish();
     }
     catch (const GiftiException& e) {
-        throw DataFileException(e);
+        throw DataFileException(filename,
+                                e.whatString());
     }
     
     this->clearModified();
@@ -993,7 +1017,7 @@ GiftiFile::writeFile(const AString& filename) throw (DataFileException)
    std::ofstream* externalBinaryOutputStream = NULL;
    if (encoding == GiftiDataArray::ENCODING_EXTERNAL_FILE_BINARY) {
        externalBinaryFileName = getFileNameNoPath() + ".data";
-       externalBinaryOutputStream = new std::ofstream(externalBinaryFileName.toAscii().constData(),
+       externalBinaryOutputStream = new std::ofstream(externalBinaryFileName.toLatin1().constData(),
                                                       std::ofstream::binary);
        if (externalBinaryOutputStream == NULL) {
           throw GiftiException("Unable to open " + externalBinaryFileName + " for output.");
@@ -1062,7 +1086,7 @@ GiftiFile::setNumberOfNodesForSparseNodeIndexFiles(const int32_t numNodes)
  * process NIFTI_INTENT_NODE_INDEX arrays.
  */
 void 
-GiftiFile::procesNiftiIntentNodeIndexArrays() throw (GiftiException)
+GiftiFile::procesNiftiIntentNodeIndexArrays()
 {
    //
    // See if there is a node index array
@@ -1241,7 +1265,7 @@ GiftiFile::setEncodingForWriting(const GiftiEncodingEnum::Enum encoding)
  * Validate the data arrays (optional for subclasses).
  */
 void 
-GiftiFile::validateDataArrays() throw (GiftiException)
+GiftiFile::validateDataArrays()
 {
     // nothing 
 }
