@@ -21,14 +21,16 @@
  */
 /*LICENSE_END*/
 
-
+#include <array>
 #include <map>
 #include <memory>
 #include <set>
 
 #include "CaretObject.h"
 #include "EventListenerInterface.h"
-
+#include "GraphicsLineMeanDeviationSettings.h"
+#include "GraphicsTextureSettings.h"
+#include "VoxelColorUpdate.h"
 
 namespace caret {
 
@@ -38,7 +40,11 @@ namespace caret {
     class GraphicsPrimitiveV3fC4f;
     class GraphicsPrimitiveV3fC4ub;
     class GraphicsPrimitiveV3fN3f;
+    class GraphicsPrimitiveV3fN3fC4f;
+    class GraphicsPrimitiveV3fN3fC4ub;
+    class GraphicsPrimitiveV3fT2f;
     class GraphicsPrimitiveV3fT3f;
+    class Matrix4x4Interface;
     
     class GraphicsPrimitive : public CaretObject, public EventListenerInterface {
         
@@ -85,16 +91,6 @@ namespace caret {
             PER_VERTEX_RGBA
         };
         
-        /**
-         * Data type of texture components
-         */
-        enum class TextureDataType {
-            /** No texture coordinates */
-            NONE,
-            /** Three float values per vertex contains S, T, and R texture coordinates */
-            FLOAT_STR
-        };
-        
         /**
          * Type of primitives for drawing.  There are NO primitives equivalent to
          * OpenGL's GL_QUAD_STRIP and GL_POLYGON.  The reason is that these
@@ -103,6 +99,10 @@ namespace caret {
          * Descriptions are copied from the glVertex man page.
          */
         enum class PrimitiveType {
+            /**
+             * Draws as a disk (flat, filled circle)
+             */
+            DISKS,
             /**
              * Draws a connected group of line segments from the first vertex to the last.  
              * Vertices n and n+1 define line n.  n-1 lines are drawn. 
@@ -146,26 +146,60 @@ namespace caret {
             OPENGL_TRIANGLES,
             /**
              * Like OPENGL_LINE_LOOP but there is no limit on line width as it draws the lines using polygons
+             * and polygons that form the line use a BEVEL join at vertices.
+             * Draws in MODEL space so lines are affected by model transformations
+             */
+            MODEL_SPACE_POLYGONAL_LINE_LOOP_BEVEL_JOIN,
+            /**
+             * Like OPENGL_LINE_LOOP but there is no limit on line width as it draws the lines using polygons
+             * and polygons that form the line use a MITER join at vertices
+             * Draws in MODEL space so lines are affected by model transformations
+             */
+            MODEL_SPACE_POLYGONAL_LINE_LOOP_MITER_JOIN,
+            /**
+             * Like OPENGL_LINE_STRIP but there is no limit on line width as it draws the lines using polygons
              * and polygons that form the line use a BEVEL join at vertices
+             * Draws in MODEL space so lines are affected by model transformations
+             */
+            MODEL_SPACE_POLYGONAL_LINE_STRIP_BEVEL_JOIN,
+            /**
+             * Like OPENGL_LINE_STRIP but there is no limit on line width as it draws the lines using polygons
+             * and polygons that form the line use a MITER join at vertices
+             * Draws in MODEL space so lines are affected by model transformations
+             */
+            MODEL_SPACE_POLYGONAL_LINE_STRIP_MITER_JOIN,
+            /**
+             * Like OPENGL_LINES but there is no limit on line width as it draws the lines using polygons.
+             * Draws in MODEL space so lines are affected by model transformations
+             */
+            MODEL_SPACE_POLYGONAL_LINES,
+            /**
+             * Like OPENGL_LINE_LOOP but there is no limit on line width as it draws the lines using polygons
+             * and polygons that form the line use a BEVEL join at vertices.
+             * Lines drawn in window space so that lines always face user
              */
             POLYGONAL_LINE_LOOP_BEVEL_JOIN,
             /**
              * Like OPENGL_LINE_LOOP but there is no limit on line width as it draws the lines using polygons
              * and polygons that form the line use a MITER join at vertices
+             * Lines drawn in window space so that lines always face user
              */
             POLYGONAL_LINE_LOOP_MITER_JOIN,
             /**
              * Like OPENGL_LINE_STRIP but there is no limit on line width as it draws the lines using polygons
              * and polygons that form the line use a BEVEL join at vertices
+             * Lines drawn in window space so that lines always face user
              */
             POLYGONAL_LINE_STRIP_BEVEL_JOIN,
             /**
              * Like OPENGL_LINE_STRIP but there is no limit on line width as it draws the lines using polygons
              * and polygons that form the line use a MITER join at vertices
+             * Lines drawn in window space so that lines always face user
              */
             POLYGONAL_LINE_STRIP_MITER_JOIN,
             /**
              * Like OPENGL_LINES but there is no limit on line width as it draws the lines using polygons.
+             * Lines drawn in window space so that lines always face user
              */
             POLYGONAL_LINES,
             /*
@@ -220,6 +254,26 @@ namespace caret {
             MILLIMETERS
         };
         
+        /**
+         * Modes for releasing instance data after OpenGL buffers have been loaded.
+         * For large primitives, this can save substantial memory.  Do not use if
+         * data will be updated.
+         */
+        enum class ReleaseInstanceDataMode {
+            /**
+             * Release of instance data has been completed
+             */
+            COMPLETED,
+            /**
+             * Release of instance data is disabled (DEFAULT)
+             */
+            DISABLED,
+            /**
+             * Release is enabled but not yet completed (OpenGL buffers not loaded yet)
+             */
+            ENABLED
+        };
+        
         /**
          * Hint to graphics engine on how the primitive is used.
          * The hint must be set prior to loading of buffers.
@@ -243,12 +297,12 @@ namespace caret {
         };
         
     protected:
-        GraphicsPrimitive(const VertexDataType       vertexDataType,
-                          const NormalVectorDataType normalVectorDataType,
-                          const ColorDataType        colorDataType,
-                          const VertexColorType      vertexColorType,
-                          const TextureDataType      textureDataType,
-                          const PrimitiveType        primitiveType);
+        GraphicsPrimitive(const VertexDataType        vertexDataType,
+                          const NormalVectorDataType  normalVectorDataType,
+                          const ColorDataType         colorDataType,
+                          const VertexColorType       vertexColorType,
+                          const GraphicsTextureSettings& textureSettings,
+                          const PrimitiveType         primitiveType);
         
         GraphicsPrimitive(const GraphicsPrimitive& obj);
         
@@ -265,15 +319,21 @@ namespace caret {
         static GraphicsPrimitiveV3fN3f* newPrimitiveV3fN3f(const GraphicsPrimitive::PrimitiveType primitiveType,
                                                            const uint8_t unsignedByteRGBA[4]);
         
+        static GraphicsPrimitiveV3fN3fC4f* newPrimitiveV3fN3fC4f(const GraphicsPrimitive::PrimitiveType primitiveType);
+        
+        static GraphicsPrimitiveV3fN3fC4ub* newPrimitiveV3fN3fC4ub(const GraphicsPrimitive::PrimitiveType primitiveType);
+        
         static GraphicsPrimitiveV3fC4f* newPrimitiveV3fC4f(const GraphicsPrimitive::PrimitiveType primitiveType);
         
         static GraphicsPrimitiveV3fC4ub* newPrimitiveV3fC4ub(const GraphicsPrimitive::PrimitiveType primitiveType);
         
-        static GraphicsPrimitiveV3fT3f* newPrimitiveV3fT3f(const GraphicsPrimitive::PrimitiveType primitiveType,
-                                                           const uint8_t* imageBytesRGBA,
-                                                           const int32_t imageWidth,
-                                                           const int32_t imageHeight);
+        static GraphicsPrimitiveV3fT2f* newPrimitiveV3fT2f(const GraphicsPrimitive::PrimitiveType primitiveType,
+                                                           const GraphicsTextureSettings& textureSettings);
         
+        static GraphicsPrimitiveV3fT3f* newPrimitiveV3fT3f(const GraphicsPrimitive::PrimitiveType primitiveType,
+                                                           const GraphicsTextureSettings& textureSettings);
+        
+
         virtual ~GraphicsPrimitive();
         
         void reserveForNumberOfVertices(const int32_t numberOfVertices);
@@ -326,25 +386,62 @@ namespace caret {
         inline PrimitiveType getPrimitiveType() const { return m_primitiveType; }
         
         /**
-         * @return Data type of texture.
+         * @return Mode for release of instance data (xyz, normals, coloring, etc) after buffers are loaded
          */
-        inline TextureDataType getTextureDataType() const { return m_textureDataType; }
+        ReleaseInstanceDataMode getReleaseInstanceDataMode() const { return m_releaseInstanceDataMode; }
+        
+        /**
+         * Set the mode for release of instance data  (xyz, normals, coloring, etc) after buffers are loaded
+         */
+        void setReleaseInstanceDataMode(const ReleaseInstanceDataMode releaseDataMode) {
+            m_releaseInstanceDataMode = releaseDataMode;
+        }
+
+        /**
+         * @return The texture settings
+         */
+        const GraphicsTextureSettings& getTextureSettings() const { return m_textureSettings; }
+                
+        int32_t getTexturePixelFormatBytesPerPixel() const;
+
+        /**
+         * Set type of magnification filter (pixel smaller than texel)
+         */
+        inline void setTextureMagnificationFilter(const GraphicsTextureMagnificationFilterEnum::Enum magFilter) {
+            m_textureSettings.setMagnificationFilter(magFilter);
+        }
+        
+        /**
+         * Set type of minification filter (pixel bigger than texel)
+         */
+        inline void setTextureMinificationFilter(const GraphicsTextureMinificationFilterEnum::Enum minFilter) {
+            m_textureSettings.setMinificationFilter(minFilter);
+        }
         
         /**
          * @return The float coordinates.
          */
         const std::vector<float>& getFloatXYZ() const { return m_xyz; }
         
+        void getVertexFloatXYZ(const int32_t vertexIndex,
+                               float xyzOut[3]) const;
+        
         void replaceFloatXYZ(const std::vector<float>& xyz);
         
-        /**
-         * @return The number of vertices
-         */
-        inline int32_t getNumberOfVertices() const { return (m_xyz.size() / 3); }
+        void getFloatYComponents(std::vector<float>& yComponentsOut) const;
         
+        void setFloatYComponents(const std::vector<float>& yComponents);
+        
+        int32_t getNumberOfVertices() const;
+
         void replaceVertexFloatXYZ(const int32_t vertexIndex,
                                    const float xyz[3]);
         
+        void replaceAndTransformVertices(const GraphicsPrimitive* primitive,
+                                         const Matrix4x4Interface& matrix);
+        
+        void transformVerticesFloatXYZ(const Matrix4x4Interface& matrix);
+        
         void getVertexFloatRGBA(const int32_t vertexIndex,
                                 float rgbaOut[4]) const;
 
@@ -361,10 +458,19 @@ namespace caret {
         
         void replaceAllVertexSolidFloatRGBA(const float rgba[4]);
         
+        void replaceVertexTextureSTR(const int32_t vertexIndex,
+                                     const float str[3]);
+        
         bool getVertexBounds(BoundingBox& boundingBoxOut) const;
         
         void addPrimitiveRestart();
         
+        bool getDrawArrayIndicesSubset(int32_t& firstVertexIndexOut,
+                                       int32_t& vertexCountOut) const;
+        
+        void setDrawArrayIndicesSubset(const int32_t firstVertexIndex,
+                                       const int32_t vertexCount) const;
+        
         void getPointDiameter(PointSizeType& sizeTypeOut,
                               float& pointDiameterOut) const;
         
@@ -395,14 +501,26 @@ namespace caret {
          * Clone this primitive.
          */
         virtual GraphicsPrimitive* clone() const = 0;
+
+        void simplfyLines(const int32_t skipVertexCount);
+        
+        void getMeanAndStandardDeviationForY(float& yMeanOut,
+                                             float& yStandardDeviationOut) const;
+        
+        void applyNewMeanAndDeviationToYComponents(const GraphicsLineMeanDeviationSettings& settings,
+                                                   bool& haveNanInfFlagOut);
+        
+        const VoxelColorUpdate* getVoxelColorUpdate() const;
+        
+        void setVoxelColorUpdate(const VoxelColorUpdate& voxelColorUpdate);
+        
+        void resetVoxelColorUpdate();
+
+        static AString getNewMeanDeviationOperationDescriptionInHtml();
         
     protected:
         AString toStringPrivate(const bool includeAllDataFlag) const;
         
-        void setTextureImage(const uint8_t* imageBytesRGBA,
-                             const int32_t imageWidth,
-                             const int32_t imageHeight);
-        
         void addVertexProtected(const float xyz[3],
                                 const float normalVector[3],
                                 const float rgbaFloat[4],
@@ -419,6 +537,8 @@ namespace caret {
         
         AString getVertexColorTypeAsText(const VertexColorType vertexColorType) const;
         
+        void invalidateVertexMeasurements();
+        
         const VertexDataType  m_vertexDataType;
         
         const NormalVectorDataType m_normalVectorDataType;
@@ -427,10 +547,12 @@ namespace caret {
         
         const VertexColorType m_vertexColorType;
         
-        const TextureDataType m_textureDataType;
+        GraphicsTextureSettings m_textureSettings;
         
         const PrimitiveType m_primitiveType;
 
+        ReleaseInstanceDataMode m_releaseInstanceDataMode = ReleaseInstanceDataMode::DISABLED;
+        
         mutable bool m_boundingBoxValid = false;
         
         UsageType m_usageTypeCoordinates = UsageType::MODIFIED_ONCE_DRAWN_FEW_TIMES;
@@ -443,10 +565,6 @@ namespace caret {
         
         std::unique_ptr<GraphicsEngineDataOpenGL> m_graphicsEngineDataForOpenGL;
         
-        int32_t m_textureImageWidth = -1;
-        
-        int32_t m_textureImageHeight = -1;
-        
         mutable PointSizeType m_pointSizeType = PointSizeType::PIXELS;
         
         mutable float m_pointDiameterValue = 1.0f;
@@ -461,6 +579,10 @@ namespace caret {
         
         mutable std::unique_ptr<BoundingBox> m_boundingBox;
         
+        mutable int32_t m_arrayIndicesSubsetFirstVertexIndex = -1;
+        
+        mutable int32_t m_arrayIndicesSubsetCount = -1;
+        
         std::set<int32_t> m_polygonalLinePrimitiveRestartIndices;
         
         int32_t m_triangleStripPrimitiveRestartIndex = -1;
@@ -474,6 +596,16 @@ namespace caret {
         
         void fillTriangleStripPrimitiveRestartVertices();
         
+        void setOpenGLBuffersHaveBeenLoadedByGraphicsEngine();
+        
+        void applyNewMeanAndDeviationToYComponentsNoNaNs(std::vector<float>& data,
+                                                         const GraphicsLineMeanDeviationSettings& settings);
+        
+        void applyNewMeanAndDeviationToYComponentsWithNaNs(std::vector<float>& data,
+                                                           const GraphicsLineMeanDeviationSettings& settings);
+
+        mutable VoxelColorUpdate m_voxelColorUpdate;
+        
         std::vector<float> m_xyz;
         
         std::vector<float> m_floatNormalVectorXYZ;
@@ -484,15 +616,14 @@ namespace caret {
         
         std::vector<float> m_floatTextureSTR;
         
-        std::vector<uint8_t> m_textureImageBytesRGBA;
+        mutable float m_yMean = 0.0;
+        
+        mutable float m_yStandardDeviation = -1.0;
+        
         friend class GraphicsEngineDataOpenGL;
         friend class GraphicsOpenGLPolylineTriangles;
         friend class GraphicsPrimitiveSelectionHelper;
         
-        std::vector<float> m_dummyFloatRGBAVector;
-        
-        std::vector<uint8_t> m_dummyUnsignedByteRGBAVector;
-        
         // ADD_NEW_MEMBERS_HERE
 
     };
