@@ -27,11 +27,15 @@
 
 #include "CaretAssert.h"
 #include "CaretLogger.h"
+#include "EventManager.h"
+#include "EventOpenGLObjectToWindowTransform.h"
 #include "GraphicsEngineDataOpenGL.h"
 #include "GraphicsPrimitive.h"
 #include "GraphicsPrimitiveV3f.h"
 #include "GraphicsPrimitiveV3fC4ub.h"
 #include "GraphicsPrimitiveV3fN3f.h"
+#include "GraphicsUtilitiesOpenGL.h"
+#include "GraphicsViewport.h"
 #include "MathFunctions.h"
 #include "Matrix4x4.h"
 
@@ -61,6 +65,46 @@ GraphicsShape::~GraphicsShape()
 {
 }
 
+/**
+ * @return New primitive for drawing a circle
+ *
+ * @param radius
+ *    Radius of circle
+ * @param rgba
+ *    Color of circle
+ * @param lineThicknessType
+ *    Type of line thickness
+ * @param lineThickness
+ *    Thickness of line
+ * @return
+ *    Primitive for drawing circle
+ */
+GraphicsPrimitiveV3f*
+GraphicsShape::newInstanceCircleByteColor(const float radius,
+                                                       const uint8_t rgba[4],
+                                                       const GraphicsPrimitive::LineWidthType lineThicknessType,
+                                                       const double lineThickness)
+{
+    std::vector<float> ellipseXYZ;
+    const float minorAxis(radius * 2.0);
+    const float majorAxis(minorAxis);
+    createEllipseVertices(majorAxis, minorAxis, ellipseXYZ);
+    
+    GraphicsPrimitiveV3f* primitive(GraphicsPrimitive::newPrimitiveV3f(GraphicsPrimitive::PrimitiveType::POLYGONAL_LINE_LOOP_MITER_JOIN,
+                                                                       rgba));
+    const int32_t numVertices = static_cast<int32_t>(ellipseXYZ.size() / 3);
+    primitive->reserveForNumberOfVertices(numVertices);
+    for (int32_t i = 0; i < numVertices; i++) {
+        primitive->addVertex(&ellipseXYZ[i * 3]);
+    }
+    
+    primitive->setLineWidth(lineThicknessType,
+                            lineThickness);
+    primitive->setUsageTypeAll(GraphicsPrimitive::UsageType::MODIFIED_ONCE_DRAWN_FEW_TIMES);
+
+    return primitive;
+}
+
 /**
  * Delete all primitives.
  */
@@ -88,6 +132,7 @@ GraphicsShape::deleteAllPrimitives()
      * is deleted.
      */
     s_byteSquarePrimitive.reset();
+    s_yellowCrossPrimitive.reset();
 }
 
 
@@ -287,6 +332,42 @@ GraphicsShape::drawEllipseOutlineByteColor(const double majorAxis,
     GraphicsEngineDataOpenGL::draw(primitive.get());
 }
 
+/**
+ * Draw an outline ellipse in the XY plane (all Z-coordinates will be zero).
+ *
+ * @param majorAxis
+ *    Diameter of the major axis.
+ * @param minorAxis
+ *    Diameter of the minor axis.
+ * @param rgba
+ *    Color for drawing.
+ * @param lineThickness
+ *    Thickness of the line.
+ */
+void
+GraphicsShape::drawEllipseOutlineModelSpaceByteColor(const double majorAxis,
+                                                     const double minorAxis,
+                                                     const uint8_t rgba[4],
+                                                     const double lineThickness)
+{
+    std::vector<float> ellipseXYZ;
+    createEllipseVertices(majorAxis, minorAxis, ellipseXYZ);
+    
+    std::unique_ptr<GraphicsPrimitiveV3f> primitive(GraphicsPrimitive::newPrimitiveV3f(GraphicsPrimitive::PrimitiveType::MODEL_SPACE_POLYGONAL_LINE_LOOP_MITER_JOIN,
+                                                                                       rgba));
+    const int32_t numVertices = static_cast<int32_t>(ellipseXYZ.size() / 3);
+    primitive->reserveForNumberOfVertices(numVertices);
+    for (int32_t i = 0; i < numVertices; i++) {
+        primitive->addVertex(&ellipseXYZ[i * 3]);
+    }
+    
+    primitive->setLineWidth(GraphicsPrimitive::LineWidthType::PIXELS,
+                            lineThickness);
+    primitive->setUsageTypeAll(GraphicsPrimitive::UsageType::MODIFIED_ONCE_DRAWN_FEW_TIMES);
+    
+    GraphicsEngineDataOpenGL::draw(primitive.get());
+}
+
 /**
  * Draw a filled ellipse.
  *
@@ -445,44 +526,7 @@ GraphicsShape::drawSphereByteColor(const float xyz[3],
     drawSpheresByteColor(xyz,
                          1,
                          rgba,
-                         diameter);
-    
-//    const int32_t numLatLonDivisions = 10;
-//    
-//    GraphicsPrimitive* spherePrimitive = NULL;
-//    for (const auto iter : s_byteSpherePrimitives) {
-//        const auto& key = iter.first;
-//        if ((key  == numLatLonDivisions)) {
-//            spherePrimitive = iter.second;
-//            CaretAssert(spherePrimitive);
-//            break;
-//        }
-//    }
-//    
-//    if (spherePrimitive == NULL) {
-//        const bool useStripsFlag = true;
-//        if (useStripsFlag) {
-//            spherePrimitive = createSpherePrimitiveTriangleStrips(numLatLonDivisions);
-//        }
-//        else {
-//            spherePrimitive = createSpherePrimitiveTriangles(numLatLonDivisions);
-//        }
-//        /* colors may change but not coordinates/normals */
-//        spherePrimitive->setUsageTypeAll(GraphicsPrimitive::UsageType::MODIFIED_ONCE_DRAWN_MANY_TIMES);
-//        spherePrimitive->setUsageTypeColors(GraphicsPrimitive::UsageType::MODIFIED_MANY_DRAWN_MANY_TIMES);
-//        s_byteSpherePrimitives.insert(std::make_pair(numLatLonDivisions,
-//                                                     spherePrimitive));
-//    }
-//
-//    CaretAssert(spherePrimitive);
-//    
-//    spherePrimitive->replaceAllVertexSolidByteRGBA(rgba);
-//    
-//    glPushMatrix();
-//    glTranslatef(xyz[0], xyz[1], xyz[2]);
-//    glScalef(diameter, diameter, diameter);
-//    GraphicsEngineDataOpenGL::draw(spherePrimitive);
-//    glPopMatrix();
+                         diameter);    
 }
 
 /**
@@ -507,7 +551,7 @@ GraphicsShape::drawSpheresByteColor(const float xyz[],
     const int32_t numLatLonDivisions = 10;
     
     GraphicsPrimitive* spherePrimitive = NULL;
-    for (const auto iter : s_byteSpherePrimitives) {
+    for (const auto& iter : s_byteSpherePrimitives) {
         const auto& key = iter.first;
         if ((key  == numLatLonDivisions)) {
             spherePrimitive = iter.second;
@@ -546,6 +590,33 @@ GraphicsShape::drawSpheresByteColor(const float xyz[],
 }
 
 
+/**
+ * Draw filled circles at the given XYZ coordinates
+ *
+ * @param xyz
+ *     XYZ-coordinate of circle
+ * @param numberOfSpheres
+ *     Number of spheres to draw
+ * @param rgba
+ *     Color for drawing.
+ * @param diameter
+ *     Diameter of the circle.
+ */
+void
+GraphicsShape::drawCirclesFilled(const float xyz[3],
+                                 const int32_t numberOfSpheres,
+                                 const uint8_t rgba[4],
+                                 const float diameter)
+{
+    for (int32_t i = 0; i < numberOfSpheres; i++) {
+        const int32_t i3(i * 3);
+        const int32_t i4(i * 4);
+        drawCircleFilled(&xyz[i3],
+                         &rgba[i4],
+                         diameter);
+    }
+}
+
 /**
  * Draw a filled circle at the given XYZ coordinate
  *
@@ -561,10 +632,13 @@ GraphicsShape::drawCircleFilled(const float xyz[3],
                              const uint8_t rgba[4],
                              const float diameter)
 {
+    glPushAttrib(GL_ENABLE_BIT);
+    glDisable(GL_LIGHTING);
+    
     const int32_t numberOfDivisions = 20;
     
     GraphicsPrimitive* circlePrimitive = NULL;
-    for (const auto keyPrim : s_byteCirclePrimitives) {
+    for (const auto& keyPrim : s_byteCirclePrimitives) {
         if (keyPrim.first == numberOfDivisions) {
             circlePrimitive = keyPrim.second;
             break;
@@ -592,8 +666,193 @@ GraphicsShape::drawCircleFilled(const float xyz[3],
     glScalef(diameter, diameter, 1.0f);
     GraphicsEngineDataOpenGL::draw(circlePrimitive);
     glPopMatrix();
+    
+    glPopAttrib();
 }
 
+/**
+ * Draw a filled circle at the given XYZ coordinate using a diameter
+ * that is a percentage of the viewport's height.
+ *
+ * @param xyz
+ *     XYZ-coordinate of circle
+ * @param rgba
+ *    Color for drawing.
+ * @param diameterPercentageOfViewportHeight
+ *    Diameter of the circle.
+ * @param windowXYZOut
+ *    Optional parameter.  If not NULL, contains OpenGL window XYZ of the shape.
+ *    X-component is positive if value coordinate is valid, else negative indicates invalid.
+ */
+void
+GraphicsShape::drawCircleFilledPercentViewportHeight(const float xyz[3],
+                                                     const uint8_t rgba[4],
+                                                     const float diameterPercentageOfViewportHeight,
+                                                     std::array<float, 3>* windowXYZOut)
+{
+    const float unusedSecondDiameter(0.0);
+    drawShapePercentViewportHeight(xyz,
+                                   rgba,
+                                   Shape::CIRCLE_FILLED,
+                                   diameterPercentageOfViewportHeight,
+                                   unusedSecondDiameter,
+                                   windowXYZOut);
+}
+
+/**
+ * Draw a ring at the given XYZ coordinate using sizes
+ * that are a percentage of the viewport's height.
+ *
+ * @param xyz
+ *     XYZ-coordinate of circle
+ * @param rgba
+ *    Color for drawing.
+ * @param innerDiameterPercentageOfViewportHeight
+ *    Inner diameter of the ring.
+ * @param outerDiameterPercentageOfViewportHeight
+ *    Outer diameter of the ring.
+ * @param windowXYZOut
+ *    Optional parameter.  If not NULL, contains OpenGL window XYZ of the shape.
+ *    X-component is positive if value coordinate is valid, else negative indicates invalid.
+ */
+void
+GraphicsShape::drawRingPercentViewportHeight(const float xyz[3],
+                                             const uint8_t rgba[4],
+                                             const float innerDiameterPercentageOfViewportHeight,
+                                             const float outerDiameterPercentageOfViewportHeight,
+                                             std::array<float, 3>* windowXYZOut)
+{
+    drawShapePercentViewportHeight(xyz,
+                                   rgba,
+                                   Shape::RING,
+                                   innerDiameterPercentageOfViewportHeight,
+                                   outerDiameterPercentageOfViewportHeight,
+                                   windowXYZOut);
+}
+
+/**
+ * Draw a filled circle at the given XYZ coordinate using a diameter
+ * that is a percentage of the viewport's height.
+ *
+ * @param xyz
+ *     XYZ-coordinate of circle
+ * @param rgba
+ *    Color for drawing.
+ * @param diameterPercentageOfViewportHeight
+ *    Diameter of the circle.
+ * @param windowXYZOut
+ *    Optional parameter.  If not NULL, contains OpenGL window XYZ of the shape.
+ *    X-component is positive if value coordinate is valid, else negative indicates invalid.
+ */
+void
+GraphicsShape::drawSquarePercentViewportHeight(const float xyz[3],
+                                               const uint8_t rgba[4],
+                                               const float diameterPercentageOfViewportHeight,
+                                               std::array<float, 3>* windowXYZOut)
+{
+    const float unusedSecondDiameter(0.0);
+    drawShapePercentViewportHeight(xyz,
+                                   rgba,
+                                   Shape::SQUARE,
+                                   diameterPercentageOfViewportHeight,
+                                   unusedSecondDiameter,
+                                   windowXYZOut);
+}
+
+/**
+ * Draw a shape at the given XYZ coordinate using a diameter
+ * that is a percentage of the viewport's height.
+ *
+ * @param xyz
+ *     XYZ-coordinate of circle
+ * @param rgba
+ *    Color for drawing.
+ * @param shape
+ *    Shape that is drawn
+ * @param diameterPercentageOfViewportHeight
+ *    Diameter of the shape.
+ * @param diameterTwoPercentageOfViewportHeight
+ *    Diameter of the shape for (outer for ring, unused by others).
+ * @param windowXYZOut
+ *    Optional parameter.  If not NULL, contains OpenGL window XYZ of the shape.
+ *    X-component is positive if value coordinate is valid, else negative indicates invalid.
+ */
+void
+GraphicsShape::drawShapePercentViewportHeight(const float xyz[3],
+                                              const uint8_t rgba[4],
+                                              const Shape shape,
+                                              const float diameterPercentageOfViewportHeight,
+                                              const float diameterTwoPercentageOfViewportHeight,
+                                              std::array<float, 3>* windowXYZOut)
+{
+    /*
+     * Invalidate optional output window coordinate
+     */
+    if (windowXYZOut != NULL) {
+        windowXYZOut->at(0) = -1.0f;
+    }
+    
+    /*
+     * The scale of X and Y may be very different and drawing a shape
+     * in model space requires scaling the circle separately in X and Y.
+     * It is easier to draw the shape in window space.
+     */
+    EventOpenGLObjectToWindowTransform xform(EventOpenGLObjectToWindowTransform::SpaceType::MODEL);
+    EventManager::get()->sendEvent(xform.getPointer());
+    if (xform.isValid()) {
+        float windowXYZ[3];
+        xform.transformPoint(xyz,
+                             windowXYZ);
+        
+        std::array<int32_t, 4> vp = xform.getViewport();
+        
+        glMatrixMode(GL_PROJECTION);
+        glPushMatrix();
+        glLoadIdentity();
+        glOrtho(vp[0], vp[0] + vp[2],
+                vp[1], vp[1] + vp[3],
+                -10.0, 10.0);
+        glMatrixMode(GL_MODELVIEW);
+        glPushMatrix();
+        glLoadIdentity();
+        
+        const float pixelSize = GraphicsUtilitiesOpenGL::convertPercentageOfViewportHeightToPixels(diameterPercentageOfViewportHeight);
+        const float pixelSizeTwo = GraphicsUtilitiesOpenGL::convertPercentageOfViewportHeightToPixels(diameterTwoPercentageOfViewportHeight);
+
+        switch (shape) {
+            case Shape::CIRCLE_FILLED:
+                GraphicsShape::drawCircleFilled(windowXYZ,
+                                                rgba,
+                                                pixelSize);
+                break;
+            case Shape::SQUARE:
+                GraphicsShape::drawSquare(windowXYZ,
+                                          rgba,
+                                          pixelSize);
+                break;
+            case Shape::RING:
+                GraphicsShape::drawRing(windowXYZ,
+                                        rgba,
+                                        pixelSize / 2,
+                                        pixelSizeTwo / 2);
+                break;
+        }
+        
+        if (windowXYZOut != NULL) {
+            windowXYZOut->at(0) = windowXYZ[0];
+            windowXYZOut->at(1) = windowXYZ[1];
+            windowXYZOut->at(2) = windowXYZ[2];
+        }
+
+        
+        glPopMatrix();
+        glMatrixMode(GL_PROJECTION);
+        glPopMatrix();
+        glMatrixMode(GL_MODELVIEW);
+    }
+}
+
+
 /**
  * Draw a squares at the given XYZ coordinates
  *
@@ -748,7 +1007,7 @@ GraphicsShape::drawRing(const float xyz[3],
     const int32_t numberOfDivisions = 20;
     
     GraphicsPrimitive* ringPrimitive = NULL;
-    for (const auto keyPrim : s_byteRingPrimitives) {
+    for (const auto& keyPrim : s_byteRingPrimitives) {
         if (keyPrim.first.matches(numberOfDivisions,
                                   innerRadius,
                                   outerRadius)) {
@@ -793,7 +1052,6 @@ GraphicsShape::createCirclePrimitive(const int32_t numberOfDivisions,
     /*
      * Setup step size based upon number of points around circle
      */
-    //    const int32_t numberOfPoints = 8;
     const float step = (2.0 * M_PI) / numberOfDivisions;
     
     const uint8_t rgba[] { 255, 255, 255, 255 };
@@ -1178,13 +1436,409 @@ GraphicsShape::updateModelMatrixToFaceViewer()
     glScalef(sx, sy, sz);
 }
 
+
 /**
- * Get a description of this object's content.
- * @return String describing this object's content.
+ * Draw an rectangle outline.
+ * The normal vector is computed from three consecutive vertices in the rectangle.
+ *
+ * @param bottomLeft
+ *     Bottom left vertex of the rectangle.
+ * @param bottomRight
+ *     Bottom right vertex of the rectangle.
+ * @param topRight
+ *     Top right vertex of the rectangle.
+ * @param topLeft
+ *     Top left vertex of the rectangle.
+ * @param thickness
+ *     Thickness of the outline
+ * @param rgba
+ *     RGBA color for the outline.
+ * @param verticesInMiddleFlag
+ *     If true, the lines are centered around the vertices.  Otherwise,
+ *     the inner sides of the lines are tangent to the vertices.
  */
-AString
-GraphicsShape::toString() const
+void
+GraphicsShape::drawOutlineRectanglePrivate(const float bottomLeft[3],
+                                           const float bottomRight[3],
+                                           const float topRight[3],
+                                           const float topLeft[3],
+                                           const float thicknessIn,
+                                           const uint8_t rgba[4],
+                                           bool verticesInMiddleFlag)
 {
-    return "GraphicsShape";
+    const double bl[3] { bottomLeft[0], bottomLeft[1], bottomLeft[2] };
+    const double br[3] { bottomRight[0], bottomRight[1], bottomRight[2] };
+    const double tr[3] { topRight[0], topRight[1], topRight[2] };
+    const double tl[3] { topLeft[0], topLeft[1], topLeft[2] };
+    
+    drawOutlineRectanglePrivate(bl, br, tr, tl, thicknessIn, rgba, verticesInMiddleFlag);
 }
 
+/**
+ * Draw an rectangle outline.
+ * The normal vector is computed from three consecutive vertices in the rectangle.
+ *
+ * @param bottomLeft
+ *     Bottom left vertex of the rectangle.
+ * @param bottomRight
+ *     Bottom right vertex of the rectangle.
+ * @param topRight
+ *     Top right vertex of the rectangle.
+ * @param topLeft
+ *     Top left vertex of the rectangle.
+ * @param thickness
+ *     Thickness of the outline
+ * @param rgba
+ *     RGBA color for the outline.
+ * @param verticesInMiddleFlag
+ *     If true, the lines are centered around the vertices.  Otherwise,
+ *     the inner sides of the lines are tangent to the vertices.
+ */
+void
+GraphicsShape::drawOutlineRectanglePrivate(const double bottomLeft[3],
+                                           const double bottomRight[3],
+                                           const double topRight[3],
+                                           const double topLeft[3],
+                                           const double thicknessIn,
+                                           const uint8_t rgba[4],
+                                           bool verticesInMiddleFlag)
+{
+    double bottomLeftInner[3] = { bottomLeft[0], bottomLeft[1], bottomLeft[2] };
+    double bottomLeftOuter[3] = { bottomLeft[0], bottomLeft[1], bottomLeft[2] };
+    double bottomRightInner[3] = { bottomRight[0], bottomRight[1], bottomRight[2] };
+    double bottomRightOuter[3] = { bottomRight[0], bottomRight[1], bottomRight[2] };
+    double topRightInner[3] = { topRight[0], topRight[1], topRight[2] };
+    double topRightOuter[3] = { topRight[0], topRight[1], topRight[2] };
+    double topLeftInner[3] = { topLeft[0], topLeft[1], topLeft[2] };
+    double topLeftOuter[3] = { topLeft[0], topLeft[1], topLeft[2] };
+    
+    
+    /*
+     * Limit thickness of the outline to half of width or height
+     * when points in middle
+     */
+    const double boxHeight = MathFunctions::distance3D(bottomLeft, topLeft);
+    const double boxWidth  = MathFunctions::distance3D(bottomLeft, bottomRight);
+    double thickness = thicknessIn;
+    if (verticesInMiddleFlag) {
+        thickness /= 2.0;
+        thickness = std::min(thickness,
+                             boxWidth / 2.0);
+        thickness = std::min(thickness,
+                             boxHeight / 2.0);
+    }
+    
+    /*
+     * Horizontal contraction/expansion of vertices
+     */
+    double horizVector[3];
+    MathFunctions::subtractVectors(bottomRight, bottomLeft, horizVector);
+    MathFunctions::normalizeVector(horizVector);
+    horizVector[0] *= thickness;
+    horizVector[1] *= thickness;
+    horizVector[2] *= thickness;
+    
+    MathFunctions::subtractOffsetFromVector(bottomLeftOuter, horizVector);
+    MathFunctions::subtractOffsetFromVector(topLeftOuter, horizVector);
+    MathFunctions::addOffsetToVector(bottomRightOuter, horizVector);
+    MathFunctions::addOffsetToVector(topRightOuter, horizVector);
+    if (verticesInMiddleFlag) {
+        MathFunctions::subtractOffsetFromVector(bottomRightInner, horizVector);
+        MathFunctions::subtractOffsetFromVector(topRightInner, horizVector);
+        MathFunctions::addOffsetToVector(bottomLeftInner, horizVector);
+        MathFunctions::addOffsetToVector(topLeftInner, horizVector);
+    }
+    
+    /*
+     * Vertical contraction/expansion of vertices
+     */
+    double vertVector[3];
+    MathFunctions::subtractVectors(topLeft, bottomLeft, vertVector);
+    MathFunctions::normalizeVector(vertVector);
+    vertVector[0] *= thickness;
+    vertVector[1] *= thickness;
+    vertVector[2] *= thickness;
+    
+    MathFunctions::subtractOffsetFromVector(bottomRightOuter, vertVector);
+    MathFunctions::subtractOffsetFromVector(bottomLeftOuter, vertVector);
+    MathFunctions::addOffsetToVector(topLeftOuter, vertVector);
+    MathFunctions::addOffsetToVector(topRightOuter, vertVector);
+    if (verticesInMiddleFlag) {
+        MathFunctions::subtractOffsetFromVector(topLeftInner, vertVector);
+        MathFunctions::subtractOffsetFromVector(topRightInner, vertVector);
+        MathFunctions::addOffsetToVector(bottomRightInner, vertVector);
+        MathFunctions::addOffsetToVector(bottomLeftInner, vertVector);
+    }
+    
+    /*
+     * Use three consecutive vertices to calculate normal vector
+     */
+    double normalVector[3];
+    MathFunctions::normalVector(bottomRight, topRight, topLeft, normalVector);
+    
+    /*
+     * Draw the outline as triangles using a triangle strip
+     */
+    GraphicsPrimitiveV3fN3f primitive(GraphicsPrimitive::PrimitiveType::OPENGL_TRIANGLE_STRIP,
+                                      rgba);
+    primitive.addVertex(bottomRightInner, normalVector);
+    primitive.addVertex(bottomRightOuter, normalVector);
+    primitive.addVertex(topRightInner, normalVector);
+    primitive.addVertex(topRightOuter, normalVector);
+    primitive.addVertex(topLeftInner, normalVector);
+    primitive.addVertex(topLeftOuter, normalVector);
+    primitive.addVertex(bottomLeftInner, normalVector);
+    primitive.addVertex(bottomLeftOuter, normalVector);
+    primitive.addVertex(bottomRightInner, normalVector);
+    primitive.addVertex(bottomRightOuter, normalVector);
+    
+    GraphicsEngineDataOpenGL::draw(&primitive);
+}
+
+/**
+ * Draw an rectangle outline.
+ * The given points are in the middle of outline.
+ * The normal vector is computed from three consecutive vertices in the rectangle.
+ *
+ * @param bottomLeft
+ *     Bottom left vertex of the rectangle.
+ * @param bottomRight
+ *     Bottom right vertex of the rectangle.
+ * @param topRight
+ *     Top right vertex of the rectangle.
+ * @param topLeft
+ *     Top left vertex of the rectangle.
+ * @param thickness
+ *     Thickness of the outline
+ * @param rgba
+ *     RGBA color for the outline.
+ */
+void
+GraphicsShape::drawOutlineRectangleVerticesInMiddle(const double bottomLeft[3],
+                                                    const double bottomRight[3],
+                                                    const double topRight[3],
+                                                    const double topLeft[3],
+                                                    const double thickness,
+                                                    const uint8_t rgba[4])
+{
+    drawOutlineRectanglePrivate(bottomLeft,
+                                bottomRight,
+                                topRight,
+                                topLeft,
+                                thickness,
+                                rgba,
+                                true);
+}
+
+/**
+ * Draw an rectangle outline.
+ * The given points are in the middle of outline.
+ * The normal vector is computed from three consecutive vertices in the rectangle.
+ *
+ * @param bottomLeft
+ *     Bottom left vertex of the rectangle.
+ * @param bottomRight
+ *     Bottom right vertex of the rectangle.
+ * @param topRight
+ *     Top right vertex of the rectangle.
+ * @param topLeft
+ *     Top left vertex of the rectangle.
+ * @param thickness
+ *     Thickness of the outline
+ * @param rgba
+ *     RGBA color for the outline.
+ */
+void
+GraphicsShape::drawOutlineRectangleVerticesInMiddle(const float bottomLeft[3],
+                                                    const float bottomRight[3],
+                                                    const float topRight[3],
+                                                    const float topLeft[3],
+                                                    const float thickness,
+                                                    const uint8_t rgba[4])
+{
+    drawOutlineRectanglePrivate(bottomLeft,
+                                bottomRight,
+                                topRight,
+                                topLeft,
+                                thickness,
+                                rgba,
+                                true);
+}
+
+/**
+ * Draw an rectangle outline.
+ * The given points are at the inside of outline.
+ * The normal vector is computed from three consecutive vertices in the rectangle.
+ *
+ * @param bottomLeft
+ *     Bottom left vertex of the rectangle.
+ * @param bottomRight
+ *     Bottom right vertex of the rectangle.
+ * @param topRight
+ *     Top right vertex of the rectangle.
+ * @param topLeft
+ *     Top left vertex of the rectangle.
+ * @param thickness
+ *     Thickness of the outline
+ * @param rgba
+ *     RGBA color for the outline.
+ */
+void
+GraphicsShape::drawOutlineRectangleVerticesAtInside(const float bottomLeft[3],
+                                                    const float bottomRight[3],
+                                                    const float topRight[3],
+                                                    const float topLeft[3],
+                                                    const float thickness,
+                                                    const uint8_t rgba[4])
+{
+    drawOutlineRectanglePrivate(bottomLeft,
+                                bottomRight,
+                                topRight,
+                                topLeft,
+                                thickness,
+                                rgba,
+                                false);
+}
+
+/**
+ * Draw an rectangle outline.
+ * The given points are at the inside of outline.
+ * The normal vector is computed from three consecutive vertices in the rectangle.
+ *
+ * @param bottomLeft
+ *     Bottom left vertex of the rectangle.
+ * @param bottomRight
+ *     Bottom right vertex of the rectangle.
+ * @param topRight
+ *     Top right vertex of the rectangle.
+ * @param topLeft
+ *     Top left vertex of the rectangle.
+ * @param thickness
+ *     Thickness of the outline
+ * @param rgba
+ *     RGBA color for the outline.
+ */
+void
+GraphicsShape::drawOutlineRectangleVerticesAtInside(const double bottomLeft[3],
+                                                    const double bottomRight[3],
+                                                    const double topRight[3],
+                                                    const double topLeft[3],
+                                                    const double thickness,
+                                                    const uint8_t rgba[4])
+{
+    drawOutlineRectanglePrivate(bottomLeft,
+                                bottomRight,
+                                topRight,
+                                topLeft,
+                                thickness,
+                                rgba,
+                                false);
+}
+
+/**
+ * Draw a yellow cross at the viewport center
+ */
+void
+GraphicsShape::drawYellowCrossAtViewportCenter()
+{
+    if ( ! s_yellowCrossPrimitive) {
+        uint8_t yellow[4] { 255, 255, 0, 255 };
+        s_yellowCrossPrimitive.reset(GraphicsPrimitive::newPrimitiveV3f(GraphicsPrimitive::PrimitiveType::OPENGL_LINES, yellow));
+        s_yellowCrossPrimitive->addVertex(-0.5, 0.0);
+        s_yellowCrossPrimitive->addVertex( 0.5, 0.0);
+        s_yellowCrossPrimitive->addVertex( 0.0,-0.5);
+        s_yellowCrossPrimitive->addVertex( 0.0, 0.5);
+        s_yellowCrossPrimitive->setLineWidth(GraphicsPrimitive::LineWidthType::PIXELS, 2.0);
+    }
+    
+    CaretAssert(s_yellowCrossPrimitive);
+    
+    GLint vp[4];
+    glGetIntegerv(GL_VIEWPORT, vp);
+//    const float vpX(vp[0]);
+//    const float vpY(vp[1]);
+    const float vpW(vp[2]);
+    const float vpH(vp[3]);
+    
+    const float halfWidth(vpW / 2.0);
+    const float halfHeight(vpH / 2.0);
+    const float lineLength(std::min(halfWidth, halfHeight));
+    
+    glMatrixMode(GL_PROJECTION);
+    glPushMatrix();
+    glLoadIdentity();
+    glOrtho(-halfWidth, halfWidth, -halfHeight, halfHeight, -1.0, 1.0);
+    
+    glMatrixMode(GL_MODELVIEW);
+    glPushMatrix();
+    glLoadIdentity();
+    glScalef(lineLength, lineLength, 1.0);
+    
+    glPushAttrib(GL_DEPTH_BUFFER_BIT);
+    glDisable(GL_DEPTH_TEST);
+    GraphicsEngineDataOpenGL::draw(s_yellowCrossPrimitive.get());
+    glPopAttrib();
+    
+    
+    glPopMatrix();
+    
+    glMatrixMode(GL_PROJECTION);
+    glPopMatrix();
+    glMatrixMode(GL_MODELVIEW);
+}
+
+/**
+ * Draw an X in the given rgba color with line thickness as percentage of viewport height
+ * @param rgba
+ *    RGBA color components
+ * @param percentageThickness
+ *    Thickness of line as percentage of viewport height [0, 100]
+ */
+void
+GraphicsShape::drawViewportCrossPercentageLineWidth(const uint8_t rgba[4],
+                                                    const float percentageThickness)
+{
+    GraphicsViewport vp(GraphicsViewport::newInstanceCurrentViewport());
+    glPushAttrib(GL_DEPTH_BUFFER_BIT);
+    glDisable(GL_DEPTH_TEST);
+    glMatrixMode(GL_PROJECTION);
+    glPushMatrix();
+    glLoadIdentity();
+    glOrtho(vp.getLeftF(),
+            vp.getRightF(),
+            vp.getBottomF(),
+            vp.getTopF(),
+            -1.0,
+            1.0);
+    glMatrixMode(GL_MODELVIEW);
+    glPushMatrix();
+    glLoadIdentity();
+    
+    std::vector<float> linesXYZ;
+    linesXYZ.push_back(vp.getLeftF());
+    linesXYZ.push_back(vp.getBottomF());
+    linesXYZ.push_back(0.0);
+    linesXYZ.push_back(vp.getRightF());
+    linesXYZ.push_back(vp.getTopF());
+    linesXYZ.push_back(0.0);
+    linesXYZ.push_back(vp.getLeftF());
+    linesXYZ.push_back(vp.getTopF());
+    linesXYZ.push_back(0.0);
+    linesXYZ.push_back(vp.getRightF());
+    linesXYZ.push_back(vp.getBottomF());
+    linesXYZ.push_back(0.0);
+    GraphicsShape::drawLinesByteColor(linesXYZ,
+                                      rgba,
+                                      GraphicsPrimitive::LineWidthType::PIXELS,
+                                      percentageThickness);
+    
+    
+    glPopMatrix();
+    
+    glMatrixMode(GL_PROJECTION);
+    glPopMatrix();
+    glMatrixMode(GL_MODELVIEW);
+    glPopAttrib();
+}
+
+
