@@ -43,7 +43,7 @@
 #include "CaretColorEnumMenu.h"
 #include "EnumComboBoxTemplate.h"
 #include "EventBrowserWindowGraphicsRedrawn.h"
-#include "EventGraphicsUpdateAllWindows.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
 #include "EventManager.h"
 #include "GuiManager.h"
 #include "ModelSurfaceMontage.h"
@@ -64,16 +64,25 @@ using namespace caret;
 /**
  * Constructor.
  *
+ * @param orientation
+ *    Orientation of the widget
+ * @param parentWidgetType
+ *    Type of parent widget
+ * @param userInputMode
+ *     The user input mode
  * @param browserWindowIndex
  *     Index of window in which this instance is displayed
  * @param parent
  *     Parent for this widget.
  */
-AnnotationFontWidget::AnnotationFontWidget(const AnnotationWidgetParentEnum::Enum parentWidgetType,
+AnnotationFontWidget::AnnotationFontWidget(const Qt::Orientation orientation,
+                                           const AnnotationWidgetParentEnum::Enum parentWidgetType,
+                                           const UserInputModeEnum::Enum userInputMode,
                                            const int32_t browserWindowIndex,
                                            QWidget* parent)
 : QWidget(parent),
 m_parentWidgetType(parentWidgetType),
+m_userInputMode(userInputMode),
 m_browserWindowIndex(browserWindowIndex)
 {
     /*
@@ -147,6 +156,27 @@ m_browserWindowIndex(browserWindowIndex)
     m_textColorToolButton->setIconSize(toolButtonSize);
     WuQtUtilities::setToolButtonStyleForQt5Mac(m_textColorToolButton);
     
+    /*
+     * Text background color menu
+     */
+    m_textBackgroundColorMenu = new CaretColorEnumMenu((CaretColorEnum::OPTION_INCLUDE_CUSTOM_COLOR
+                                                        | CaretColorEnum::OPTION_INCLUDE_NONE_COLOR));
+    QObject::connect(m_textBackgroundColorMenu, SIGNAL(colorSelected(const CaretColorEnum::Enum)),
+                     this, SLOT(textBackgroundColorSelected(const CaretColorEnum::Enum)));
+    
+    /*
+     * Text background color action and toolbutton
+     */
+    QLabel* textBackgroundColorLabel = new QLabel("Back");
+    m_textBackgroundColorAction = new QAction("B",
+                                              this);
+    m_textBackgroundColorAction->setToolTip("Adjust the text background color");
+    m_textBackgroundColorAction->setMenu(m_textBackgroundColorMenu);
+    m_textBackgroundColorToolButton = new QToolButton();
+    m_textBackgroundColorToolButton->setDefaultAction(m_textBackgroundColorAction);
+    m_textBackgroundColorToolButton->setIconSize(toolButtonSize);
+    WuQtUtilities::setToolButtonStyleForQt5Mac(m_textBackgroundColorToolButton);
+    
     QToolButton* boldFontToolButton      = NULL;
     QToolButton* italicFontToolButton    = NULL;
     QToolButton* underlineFontToolButton = NULL;
@@ -224,29 +254,71 @@ m_browserWindowIndex(browserWindowIndex)
             QLabel* sizeLabel  = new QLabel("Size");
             QLabel* styleLabel = new QLabel("Style");
             
-            QHBoxLayout* stylesLayout = new QHBoxLayout();
-            WuQtUtilities::setLayoutSpacingAndMargins(stylesLayout, 0, 0);
-            stylesLayout->addWidget(boldFontToolButton);
-            stylesLayout->addWidget(italicFontToolButton);
-            stylesLayout->addWidget(underlineFontToolButton);
-            stylesLayout->addStretch();
-            
-            QGridLayout* fontNameSizeLayout = new QGridLayout(this);
-            WuQtUtilities::setLayoutSpacingAndMargins(fontNameSizeLayout, 2, 0);
-            fontNameSizeLayout->setColumnStretch(0, 0);
-            fontNameSizeLayout->setColumnStretch(1, 0);
-            fontNameSizeLayout->setColumnStretch(2, 0);
-            fontNameSizeLayout->setColumnStretch(3, 100);
-            fontNameSizeLayout->addWidget(fontLabel, 0, 0);
-            fontNameSizeLayout->addWidget(m_fontNameComboBox->getWidget(),
-                                          0, 1, 1, 3);
-            fontNameSizeLayout->addWidget(sizeLabel, 1, 0);
-            fontNameSizeLayout->addWidget(m_fontSizeSpinBox->getWidget(),
-                                          1, 1);
-            fontNameSizeLayout->addWidget(styleLabel, 2, 0);
-            fontNameSizeLayout->addLayout(stylesLayout, 2, 1);
-            fontNameSizeLayout->addWidget(textColorLabel, 1, 2);
-            fontNameSizeLayout->addWidget(m_textColorToolButton, 2, 2);
+            switch (orientation) {
+                case Qt::Horizontal: /* Used in samples toolbar */
+                {
+                    QHBoxLayout* stylesLayout = new QHBoxLayout();
+                    stylesLayout->setContentsMargins(0, 0, 0, 0);
+                    stylesLayout->setSpacing(2);
+                    stylesLayout->addWidget(boldFontToolButton);
+                    stylesLayout->addWidget(italicFontToolButton);
+                    stylesLayout->addWidget(underlineFontToolButton);
+                    stylesLayout->addStretch();
+
+                    QGridLayout* gridLayout(new QGridLayout(this));
+                    gridLayout->setContentsMargins(0, 0, 0, 0);
+                    gridLayout->setHorizontalSpacing(2);
+                    gridLayout->setVerticalSpacing(2);
+                    const int topRowIndex(0);
+                    const int bottomRowIndex(1);
+                    int columnIndex(0);
+                    gridLayout->addWidget(fontLabel, topRowIndex, columnIndex);
+                    gridLayout->addWidget(m_fontNameComboBox->getWidget(), topRowIndex, (columnIndex + 1));
+                    
+                    gridLayout->addWidget(sizeLabel, bottomRowIndex, columnIndex);
+                    ++columnIndex;
+                    gridLayout->addWidget(m_fontSizeSpinBox->getWidget(), bottomRowIndex, columnIndex);
+                    ++columnIndex;
+                    gridLayout->addWidget(styleLabel, topRowIndex, columnIndex, Qt::AlignHCenter);
+                    gridLayout->addLayout(stylesLayout, bottomRowIndex, columnIndex);
+                    ++columnIndex;
+                    gridLayout->addWidget(textColorLabel, topRowIndex, columnIndex, Qt::AlignHCenter);
+                    gridLayout->addWidget(m_textColorToolButton, bottomRowIndex, columnIndex);
+                    ++columnIndex;
+                    gridLayout->addWidget(textBackgroundColorLabel, topRowIndex, columnIndex, Qt::AlignHCenter);
+                    gridLayout->addWidget(m_textBackgroundColorToolButton, bottomRowIndex, columnIndex);
+                }
+                    break;
+                case Qt::Vertical:  /* Used in annotations toolbar */
+                {
+                    QHBoxLayout* stylesLayout = new QHBoxLayout();
+                    WuQtUtilities::setLayoutSpacingAndMargins(stylesLayout, 0, 0);
+                    stylesLayout->addWidget(boldFontToolButton);
+                    stylesLayout->addWidget(italicFontToolButton);
+                    stylesLayout->addWidget(underlineFontToolButton);
+                    stylesLayout->addStretch();
+                    
+                    QGridLayout* fontNameSizeLayout = new QGridLayout(this);
+                    WuQtUtilities::setLayoutSpacingAndMargins(fontNameSizeLayout, 2, 0);
+                    fontNameSizeLayout->setColumnStretch(0, 0);
+                    fontNameSizeLayout->setColumnStretch(1, 0);
+                    fontNameSizeLayout->setColumnStretch(2, 0);
+                    fontNameSizeLayout->setColumnStretch(3, 100);
+                    fontNameSizeLayout->addWidget(fontLabel, 0, 0);
+                    fontNameSizeLayout->addWidget(m_fontNameComboBox->getWidget(),
+                                                  0, 1, 1, 3);
+                    fontNameSizeLayout->addWidget(sizeLabel, 1, 0);
+                    fontNameSizeLayout->addWidget(m_fontSizeSpinBox->getWidget(),
+                                                  1, 1);
+                    fontNameSizeLayout->addWidget(styleLabel, 2, 0);
+                    fontNameSizeLayout->addLayout(stylesLayout, 2, 1);
+                    fontNameSizeLayout->addWidget(textColorLabel, 1, 2);
+                    fontNameSizeLayout->addWidget(m_textColorToolButton, 2, 2);
+                    fontNameSizeLayout->addWidget(textBackgroundColorLabel, 1, 3);
+                    fontNameSizeLayout->addWidget(m_textBackgroundColorToolButton, 2, 3);
+                }
+                    break;
+            }
         }
             break;
         case AnnotationWidgetParentEnum::PARENT_ENUM_FOR_LATER_USE:
@@ -284,7 +356,21 @@ AnnotationFontWidget::receiveEvent(Event* event)
         
         if (m_browserWindowIndex == redrawEvent->getBrowserWindowIndex()) {
             if (isVisible()) {
-                updateFontSizeControls();
+                AnnotationManager* annotationManager = GuiManager::get()->getBrain()->getAnnotationManager(m_userInputMode);
+                std::vector<Annotation*> selectedAnnotations = annotationManager->getAnnotationsSelectedForEditing(m_browserWindowIndex);
+                
+                if (selectedAnnotations.empty()) {
+                    return;
+                }
+
+                std::vector<AnnotationFontAttributesInterface*> textAnnotations;
+                for (auto ann : selectedAnnotations) {
+                    AnnotationFontAttributesInterface* textAnn = dynamic_cast<AnnotationFontAttributesInterface*>(ann);
+                    if (textAnn != NULL) {
+                        textAnnotations.push_back(textAnn);
+                    }
+                }
+                updateContent(textAnnotations);
             }
         }
     }
@@ -299,8 +385,10 @@ AnnotationFontWidget::receiveEvent(Event* event)
 void
 AnnotationFontWidget::updateContent(std::vector<AnnotationFontAttributesInterface*>& annotations)
 {
-    m_annotationsFontColor.clear();
-    m_annotationsFontColor.reserve(annotations.size());
+    m_annotationsTextFontColor.clear();
+    m_annotationsTextFontColor.reserve(annotations.size());
+    m_annotationsTextBackgroundFontColor.clear();
+    m_annotationsTextBackgroundFontColor.reserve(annotations.size());
     m_annotationsFontName.clear();
     m_annotationsFontName.reserve(annotations.size());
     m_annotationsFontSize.clear();
@@ -320,7 +408,10 @@ AnnotationFontWidget::updateContent(std::vector<AnnotationFontAttributesInterfac
             m_annotationsFontStyle.push_back(a);
         }
         if (ann->testProperty(Annotation::Property::TEXT_COLOR)) {
-            m_annotationsFontColor.push_back(a);
+            m_annotationsTextFontColor.push_back(a);
+        }
+        if (ann->testProperty(Annotation::Property::TEXT_COLOR_BACKGROUND)) {
+            m_annotationsTextBackgroundFontColor.push_back(a);
         }
     }
     
@@ -328,8 +419,10 @@ AnnotationFontWidget::updateContent(std::vector<AnnotationFontAttributesInterfac
     updateFontSizeControls();
     updateFontStyleControls();
     updateTextColorButton();
+    updateTextBackgroundColorButton();
     
-    setEnabled((m_annotationsFontColor.size()
+    setEnabled((m_annotationsTextFontColor.size()
+                + m_annotationsTextBackgroundFontColor.size()
                 + m_annotationsFontName.size()
                 + m_annotationsFontSize.size()
                 + m_annotationsFontStyle.size()) > 0);
@@ -393,6 +486,12 @@ AnnotationFontWidget::updateFontSizeControls()
             switch (ann->getCoordinateSpace()) {
                 case AnnotationCoordinateSpaceEnum::CHART:
                     break;
+                case AnnotationCoordinateSpaceEnum::HISTOLOGY:
+                    break;
+                case AnnotationCoordinateSpaceEnum::MEDIA_FILE_NAME_AND_PIXEL:
+                    break;
+                case AnnotationCoordinateSpaceEnum::SPACER:
+                    break;
                 case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
                     sizeValue /= surfaceMontageRowCount;
                     break;
@@ -421,12 +520,6 @@ AnnotationFontWidget::updateFontSizeControls()
             if (annText->isFontTooSmallWhenLastDrawn()) {
                 tooSmallFlag = true;
             }
-//            const AnnotationText* textAnnotation = dynamic_cast<AnnotationText*>(annText);
-//            if (textAnnotation != NULL) {
-//                if (textAnnotation->isFontTooSmallWhenLastDrawn()) {
-//                    tooSmallFlag = true;
-//                }
-//            }
         }
         
         updateFontSizeSpinBox(fontSizeValue,
@@ -443,6 +536,10 @@ AnnotationFontWidget::updateFontSizeControls()
 void
 AnnotationFontWidget::updateFontStyleControls()
 {
+    m_boldFontAction->setEnabled(false);
+    m_italicFontAction->setEnabled(false);
+    m_underlineFontAction->setEnabled(false);
+
     if ( ! m_annotationsFontStyle.empty()) {
         bool boldOnFlag        = true;
         bool italicOnFlag      = true;
@@ -529,14 +626,6 @@ AnnotationFontWidget::updateFontSizeSpinBox(const float value,
     }
     m_fontSizeSpinBox->setValue(value);
     m_fontSizeSpinBox->setSuffix(fontSizeSuffix);
-    
-//    if (tooSmallFontFlag) {
-//        m_fontSizeSpinBox->getWidget()->setPalette(m_fontSizeSpinBoxRedTextPalette);
-//    }
-//    else {
-//        m_fontSizeSpinBox->getWidget()->setPalette(m_fontSizeSpinBoxDefaultPalette);
-//    }
-//    m_fontSizeSpinBox->setValue(value);
 }
 
 /**
@@ -548,9 +637,9 @@ AnnotationFontWidget::updateFontSizeSpinBox(const float value,
 void
 AnnotationFontWidget::textColorSelected(const CaretColorEnum::Enum caretColor)
 {
-    if ( ! m_annotationsFontColor.empty()) {
+    if ( ! m_annotationsTextFontColor.empty()) {
         float rgba[4];
-        m_annotationsFontColor[0]->getTextColorRGBA(rgba);
+        m_annotationsTextFontColor[0]->getTextColorRGBA(rgba);
         
         if (caretColor == CaretColorEnum::CUSTOM) {
             QColor color;
@@ -583,8 +672,8 @@ AnnotationFontWidget::textColorSelected(const CaretColorEnum::Enum caretColor)
                 AnnotationRedoUndoCommand* undoCommand = new AnnotationRedoUndoCommand();
                 undoCommand->setModeTextColor(caretColor,
                                               rgba,
-                                              convertToAnnotations(m_annotationsFontColor));
-                AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager();
+                                              convertToAnnotations(m_annotationsTextFontColor));
+                AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager(m_userInputMode);
                 AString errorMessage;
                 if ( ! annMan->applyCommand(undoCommand,
                                             errorMessage)) {
@@ -611,7 +700,82 @@ AnnotationFontWidget::textColorSelected(const CaretColorEnum::Enum caretColor)
             break;
     }
     
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
+}
+
+/**
+ * Gets called when the text background color is changed.
+ *
+ * @param caretColor
+ *     Background color that was selected.
+ */
+void
+AnnotationFontWidget::textBackgroundColorSelected(const CaretColorEnum::Enum caretColor)
+{
+    if ( ! m_annotationsTextBackgroundFontColor.empty()) {
+        float rgba[4];
+        m_annotationsTextBackgroundFontColor[0]->getTextBackgroundColorRGBA(rgba);
+        
+        if (caretColor == CaretColorEnum::CUSTOM) {
+            QColor color;
+            color.setRgbF(rgba[0], rgba[1], rgba[2]);
+            
+            QColor newColor = QColorDialog::getColor(color,
+                                                     m_textBackgroundColorToolButton,
+                                                     "Text Background Color");
+            if (newColor.isValid()) {
+                rgba[0] = newColor.redF();
+                rgba[1] = newColor.greenF();
+                rgba[2] = newColor.blueF();
+                
+                
+                switch (m_parentWidgetType) {
+                    case AnnotationWidgetParentEnum::ANNOTATION_TOOL_BAR_WIDGET:
+                        AnnotationText::setUserDefaultCustomTextBackgroundColor(rgba);
+                        break;
+                    case AnnotationWidgetParentEnum::PARENT_ENUM_FOR_LATER_USE:
+                        CaretAssert(0);
+                        break;
+                }
+            }
+        }
+        
+        
+        switch (m_parentWidgetType) {
+            case AnnotationWidgetParentEnum::ANNOTATION_TOOL_BAR_WIDGET:
+            {
+                AnnotationRedoUndoCommand* undoCommand = new AnnotationRedoUndoCommand();
+                undoCommand->setModeTextBackgroundColor(caretColor,
+                                              rgba,
+                                              convertToAnnotations(m_annotationsTextBackgroundFontColor));
+                AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager(m_userInputMode);
+                AString errorMessage;
+                if ( ! annMan->applyCommand(undoCommand,
+                                            errorMessage)) {
+                    WuQMessageBox::errorOk(this,
+                                           errorMessage);
+                }
+                AnnotationText::setUserDefaultTextBackgroundColor(caretColor);
+            }
+                break;
+            case AnnotationWidgetParentEnum::PARENT_ENUM_FOR_LATER_USE:
+                CaretAssert(0);
+                break;
+        }
+    }
+    
+    updateTextBackgroundColorButton();
+    
+    switch (m_parentWidgetType) {
+        case AnnotationWidgetParentEnum::ANNOTATION_TOOL_BAR_WIDGET:
+            EventManager::get()->sendSimpleEvent(EventTypeEnum::EVENT_ANNOTATION_TOOLBAR_UPDATE);
+            break;
+        case AnnotationWidgetParentEnum::PARENT_ENUM_FOR_LATER_USE:
+            CaretAssert(0);
+            break;
+    }
+    
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
 }
 
 /**
@@ -630,20 +794,20 @@ AnnotationFontWidget::updateTextColorButton()
     switch (m_parentWidgetType) {
         case AnnotationWidgetParentEnum::ANNOTATION_TOOL_BAR_WIDGET:
         {
-            const int32_t numAnnotations = static_cast<int32_t>(m_annotationsFontColor.size());
+            const int32_t numAnnotations = static_cast<int32_t>(m_annotationsTextFontColor.size());
             if (numAnnotations > 0) {
                 bool firstColorSupportFlag = true;
                 bool allSameColorFlag = true;
                 
                 for (int32_t i = 0; i < numAnnotations; i++) {
                     if (firstColorSupportFlag) {
-                        m_annotationsFontColor[i]->getTextColorRGBA(rgba);
+                        m_annotationsTextFontColor[i]->getTextColorRGBA(rgba);
                         firstColorSupportFlag = false;
                         colorButtonValidFlag = true;
                     }
                     else {
                         float colorRGBA[4];
-                        m_annotationsFontColor[i]->getTextColorRGBA(colorRGBA);
+                        m_annotationsTextFontColor[i]->getTextColorRGBA(colorRGBA);
                         for (int32_t iColor = 0; iColor < 4; iColor++) {
                             if (rgba[iColor] != colorRGBA[iColor]) {
                                 allSameColorFlag = false;
@@ -658,11 +822,11 @@ AnnotationFontWidget::updateTextColorButton()
                 }
                 
                 if (allSameColorFlag) {
-                    colorEnum = m_annotationsFontColor[0]->getTextColor();
-                    m_annotationsFontColor[0]->getTextColorRGBA(rgba);
+                    colorEnum = m_annotationsTextFontColor[0]->getTextColor();
+                    m_annotationsTextFontColor[0]->getTextColorRGBA(rgba);
                     
                     float customRGBA[4];
-                    m_annotationsFontColor[0]->getCustomTextColor(customRGBA);
+                    m_annotationsTextFontColor[0]->getCustomTextColor(customRGBA);
                     m_textColorMenu->setCustomIconColor(customRGBA);
                     
                     switch (m_parentWidgetType) {
@@ -695,6 +859,87 @@ AnnotationFontWidget::updateTextColorButton()
     }
 }
 
+/**
+ * Update the text background color button.
+ */
+void
+AnnotationFontWidget::updateTextBackgroundColorButton()
+{
+    CaretColorEnum::Enum colorEnum = CaretColorEnum::NONE;
+    float rgba[4];
+    CaretColorEnum::toRGBAFloat(colorEnum, rgba);
+    rgba[3] = 1.0;
+    
+    bool colorButtonValidFlag = false;
+    
+    switch (m_parentWidgetType) {
+        case AnnotationWidgetParentEnum::ANNOTATION_TOOL_BAR_WIDGET:
+        {
+            const int32_t numAnnotations = static_cast<int32_t>(m_annotationsTextBackgroundFontColor.size());
+            if (numAnnotations > 0) {
+                bool firstColorSupportFlag = true;
+                bool allSameColorFlag = true;
+                
+                for (int32_t i = 0; i < numAnnotations; i++) {
+                    if (firstColorSupportFlag) {
+                        m_annotationsTextBackgroundFontColor[i]->getTextBackgroundColorRGBA(rgba);
+                        firstColorSupportFlag = false;
+                        colorButtonValidFlag = true;
+                    }
+                    else {
+                        float colorRGBA[4];
+                        m_annotationsTextBackgroundFontColor[i]->getTextBackgroundColorRGBA(colorRGBA);
+                        for (int32_t iColor = 0; iColor < 4; iColor++) {
+                            if (rgba[iColor] != colorRGBA[iColor]) {
+                                allSameColorFlag = false;
+                                break;
+                            }
+                        }
+                        
+                        if ( ! allSameColorFlag) {
+                            break;
+                        }
+                    }
+                }
+                
+                if (allSameColorFlag) {
+                    colorEnum = m_annotationsTextBackgroundFontColor[0]->getTextBackgroundColor();
+                    m_annotationsTextBackgroundFontColor[0]->getTextBackgroundColorRGBA(rgba);
+                    
+                    float customRGBA[4];
+                    m_annotationsTextBackgroundFontColor[0]->getCustomTextBackgroundColor(customRGBA);
+                    m_textBackgroundColorMenu->setCustomIconColor(customRGBA);
+                    
+                    switch (m_parentWidgetType) {
+                        case AnnotationWidgetParentEnum::ANNOTATION_TOOL_BAR_WIDGET:
+                            AnnotationText::setUserDefaultTextBackgroundColor(colorEnum);
+                            AnnotationText::setUserDefaultCustomTextBackgroundColor(customRGBA);
+                            break;
+                        case AnnotationWidgetParentEnum::PARENT_ENUM_FOR_LATER_USE:
+                            CaretAssert(0);
+                            break;
+                    }
+                }
+            }
+        }
+            break;
+        case AnnotationWidgetParentEnum::PARENT_ENUM_FOR_LATER_USE:
+            CaretAssert(0);
+            break;
+    }
+    
+    
+    QPixmap pm = WuQtUtilities::createCaretColorEnumPixmap(m_textBackgroundColorToolButton, 24, 24, colorEnum, rgba, true);
+    m_textBackgroundColorAction->setIcon(QIcon(pm));
+    m_textBackgroundColorMenu->setSelectedColor(colorEnum);
+    
+    m_textBackgroundColorToolButton->setEnabled(colorButtonValidFlag);
+    if (colorButtonValidFlag) {
+        
+    }
+}
+
+
 /**
  * Gets called when font bold changed.
  */
@@ -704,7 +949,7 @@ AnnotationFontWidget::fontBoldChanged()
     AnnotationRedoUndoCommand* command = new AnnotationRedoUndoCommand();
     command->setModeTextFontBold(m_boldFontAction->isChecked(),
                                  convertToAnnotations(m_annotationsFontStyle));
-    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager();
+    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager(m_userInputMode);
     AString errorMessage;
     if ( ! annMan->applyCommand(command,
                                 errorMessage)) {
@@ -712,7 +957,7 @@ AnnotationFontWidget::fontBoldChanged()
                                errorMessage);
     }
     EventManager::get()->sendSimpleEvent(EventTypeEnum::EVENT_ANNOTATION_TOOLBAR_UPDATE);
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     
     switch (m_parentWidgetType) {
         case AnnotationWidgetParentEnum::ANNOTATION_TOOL_BAR_WIDGET:
@@ -732,7 +977,7 @@ AnnotationFontWidget::fontItalicChanged()
     AnnotationRedoUndoCommand* command = new AnnotationRedoUndoCommand();
     command->setModeTextFontItalic(m_italicFontAction->isChecked(),
                                    convertToAnnotations(m_annotationsFontStyle));
-    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager();
+    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager(m_userInputMode);
     AString errorMessage;
     if ( ! annMan->applyCommand(command,
                                 errorMessage)) {
@@ -740,7 +985,7 @@ AnnotationFontWidget::fontItalicChanged()
                                errorMessage);
     }
     EventManager::get()->sendSimpleEvent(EventTypeEnum::EVENT_ANNOTATION_TOOLBAR_UPDATE);
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     
     switch (m_parentWidgetType) {
         case AnnotationWidgetParentEnum::ANNOTATION_TOOL_BAR_WIDGET:
@@ -762,7 +1007,7 @@ AnnotationFontWidget::fontNameChanged()
     AnnotationRedoUndoCommand* command = new AnnotationRedoUndoCommand();
     command->setModeTextFontName(fontName,
                                  convertToAnnotations(m_annotationsFontName));
-    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager();
+    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager(m_userInputMode);
     AString errorMessage;
     if ( ! annMan->applyCommand(command,
                                 errorMessage)) {
@@ -779,7 +1024,7 @@ AnnotationFontWidget::fontNameChanged()
             break;
     }
     
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     
 }
 
@@ -795,7 +1040,7 @@ AnnotationFontWidget::fontSizeChanged()
     command->setModeTextFontPercentSize(fontPercentSize,
                                         convertToAnnotations(m_annotationsFontSize),
                                         getSurfaceMontageRowCount());
-    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager();
+    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager(m_userInputMode);
     AString errorMessage;
     if ( ! annMan->applyCommand(command,
                                 errorMessage)) {
@@ -813,7 +1058,7 @@ AnnotationFontWidget::fontSizeChanged()
             break;
     }
     
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     
     /*
      * "Font too small" status is set while drawing so need
@@ -855,7 +1100,7 @@ AnnotationFontWidget::fontUnderlineChanged()
     AnnotationRedoUndoCommand* command = new AnnotationRedoUndoCommand();
     command->setModeTextFontUnderline(m_underlineFontAction->isChecked(),
                                       convertToAnnotations(m_annotationsFontStyle));
-    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager();
+    AnnotationManager* annMan = GuiManager::get()->getBrain()->getAnnotationManager(m_userInputMode);
     AString errorMessage;
     if ( ! annMan->applyCommand(command,
                                 errorMessage)) {
@@ -863,7 +1108,7 @@ AnnotationFontWidget::fontUnderlineChanged()
                                errorMessage);
     }
     EventManager::get()->sendSimpleEvent(EventTypeEnum::EVENT_ANNOTATION_TOOLBAR_UPDATE);
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     
     switch (m_parentWidgetType) {
         case AnnotationWidgetParentEnum::ANNOTATION_TOOL_BAR_WIDGET:
