@@ -63,6 +63,7 @@
 #include "SystemUtilities.h"
 #include "WuQDataEntryDialog.h"
 #include "WuQMessageBox.h"
+#include "WuQMessageBoxTwo.h"
 #include "WuQtUtilities.h"
 #include "WuQWidgetDisabler.h"
 
@@ -213,7 +214,7 @@ BalsaDatabaseUploadSceneFileDialog::createLoginWidget()
     /*
      * Show password tool button
      */
-    m_showPasswordAction = new QAction("Show");
+    m_showPasswordAction = new QAction("Show", this);
     m_showPasswordAction->setCheckable(true);
     m_showPasswordAction->setChecked(false);
     QObject::connect(m_showPasswordAction, &QAction::triggered,
@@ -447,9 +448,14 @@ BalsaDatabaseUploadSceneFileDialog::createUploadTab()
 QWidget*
 BalsaDatabaseUploadSceneFileDialog::createBalsaDatabaseSelectionWidget()
 {
-    QString hostName = SystemUtilities::getLocalHostName();
-    const bool isWustlDomainFlag = hostName.endsWith(".wustl.edu");
-    
+    bool isWustlDomainFlag = false;
+    /* DOES NOT WORK ON MACOS 10.14, See note in: SystemUtilities::getLocalHostName()
+       QString hostName = SystemUtilities::getLocalHostName();
+       bool isWustlDomainFlag = hostName.endsWith(".wustl.edu");
+     */
+#ifndef NDEBUG
+    isWustlDomainFlag = true;
+#endif
     /*
      * Database selection
      */
@@ -461,7 +467,11 @@ BalsaDatabaseUploadSceneFileDialog::createBalsaDatabaseSelectionWidget()
         m_databaseComboBox->addItem("http://johnsdev.wustl.edu:8080");
         m_databaseComboBox->addItem("https://johnsdev.wustl.edu:8080");
     }
-    QObject::connect(m_databaseComboBox, static_cast<void(QComboBox::*)(const QString&)>(&QComboBox::activated),
+#if QT_VERSION >= 0x060000
+    QObject::connect(m_databaseComboBox, static_cast<void(QComboBox::*)(const QString&)>(&QComboBox::textActivated),
+#else
+                     QObject::connect(m_databaseComboBox, static_cast<void(QComboBox::*)(const QString&)>(&QComboBox::activated),
+#endif
                      this, [=] { this->loginInformationChanged(); });
     QObject::connect(m_databaseComboBox, &QComboBox::editTextChanged,
                      this, [=] { this->loginInformationChanged(); });
@@ -503,7 +513,11 @@ BalsaDatabaseUploadSceneFileDialog::creatZipFileDirectoryWidget()
     buttGroup->addButton(m_zipFileTemporaryDirectoryRadioButton);
     buttGroup->addButton(m_zipFileCustomDirectoryRadioButton);
     m_zipFileTemporaryDirectoryRadioButton->setChecked(true); // do before signal
-    QObject::connect(buttGroup, static_cast<void(QButtonGroup::*)(int)>(&QButtonGroup::buttonClicked),
+#if QT_VERSION >= 0x060000
+    QObject::connect(buttGroup, static_cast<void(QButtonGroup::*)(int)>(&QButtonGroup::idClicked),
+#else
+                     QObject::connect(buttGroup, static_cast<void(QButtonGroup::*)(int)>(&QButtonGroup::buttonClicked),
+#endif
                      [=](int id){ this->zipFileDirectoryRadioButtonClicked(id); });
     
     m_zipFileAutomaticDirectoryLineEdit = new QLineEdit();
@@ -699,7 +713,7 @@ BalsaDatabaseUploadSceneFileDialog::loginButtonClicked()
                                              errorMessage)) {
         if ( ! m_userRoles->isSubmitter()) {
             cursor.restoreCursor();
-            const AString msg("Login to BALSA was successful.  However, your have not agreed "
+            const AString msg("Login to BALSA was successful.  However, you have not agreed "
                               "to the BALSA Submission Terms and Conditions that are required "
                               "to upload data to BALSA.  You will need to login to BALSA "
                               "using your web browser and complete this agreement.");
@@ -805,7 +819,7 @@ BalsaDatabaseUploadSceneFileDialog::checkBalsaExtractionDirectoryPrefix()
             else {
                 AString msg("BALSA was unable to provide an updated \"Extraction Directory Prefix\" for Study ID \""
                             + studyID
-                            + "\".  You may continue uploading.");
+                            + "\".  This will occur the first time a scene file is uploaded.  You may continue uploading.");
                 WuQMessageBox::warningOk(this, msg);
             }
         }
@@ -993,7 +1007,10 @@ BalsaDatabaseUploadSceneFileDialog::okButtonClicked()
     progressDialog.setValue(progressDialog.maximum());
     
     if (successFlag) {
-        WuQMessageBox::informationOk(this, "Upload was successful");
+        AString msg("Upload was successful");
+        msg.appendWithNewLine(m_balsaDatabaseManager->getInfoMessages());
+        msg.appendWithNewLine(m_balsaDatabaseManager->getUploadSummaryMessage());
+        WuQMessageBoxTwo::information(this, "Complete", msg);
     }
     else {
         WuQMessageBox::errorOk(this,
@@ -1144,6 +1161,7 @@ BalsaDatabaseUploadSceneFileDialog::selectStudyTitleButtonClicked()
         if ( ! bsi.isEmpty()) {
             m_balsaStudyIDLineEdit->setText(bsi.getStudyID());
             m_balsaStudyTitleLineEdit->setText(bsi.getStudyTitle());
+            m_extractDirectoryNameLineEdit->setText(bsi.getStudyExtractionDirectoryName());
         }
         validateUploadData();
     }
