@@ -172,6 +172,7 @@ BorderPropertiesEditorDialog::BorderPropertiesEditorDialog(const QString& title,
                                                          SLOT(newBorderFileButtonClicked()));
     QToolButton* newFileToolButton = new QToolButton();
     newFileToolButton->setDefaultAction(newFileAction);
+    WuQtUtilities::setToolButtonStyleForQt5Mac(newFileToolButton);
     
     /*
      * Completer for name
@@ -192,6 +193,7 @@ BorderPropertiesEditorDialog::BorderPropertiesEditorDialog(const QString& title,
                                                                     SLOT(displayNameEditor()));
     QToolButton* displayNameColorEditorToolButton = new QToolButton();
     displayNameColorEditorToolButton->setDefaultAction(displayNameColorEditorAction);
+    WuQtUtilities::setToolButtonStyleForQt5Mac(displayNameColorEditorToolButton);
     
     /*
      * Class
@@ -209,6 +211,7 @@ BorderPropertiesEditorDialog::BorderPropertiesEditorDialog(const QString& title,
                                                                     SLOT(displayClassEditor()));
     QToolButton* displayClassEditorToolButton = new QToolButton();
     displayClassEditorToolButton->setDefaultAction(displayClassEditorAction);
+    WuQtUtilities::setToolButtonStyleForQt5Mac(displayClassEditorToolButton);
     
     /*
      * Closed
@@ -336,7 +339,7 @@ BorderPropertiesEditorDialog::loadBorderFileComboBox()
             if (borderFile->getStructure() == borderStructure) {
                 const AString name = borderFile->getFileNameNoPath();
                 m_borderFileSelectionComboBox->addItem(name,
-                                                       qVariantFromValue((void*)borderFile));
+                                                       QVariant::fromValue((void*)borderFile));
                 if (s_previousBorderFile == borderFile) {
                     defaultFileComboIndex = m_borderFileSelectionComboBox->count() - 1;
                 }
@@ -371,6 +374,7 @@ BorderPropertiesEditorDialog::newBorderFileButtonClicked()
          * Let user choose a different path/name
          */
         BorderFile* newBorderFile = new BorderFile();
+        GuiManager::get()->getBrain()->convertDataFilePathNameToAbsolutePathName(newBorderFile);
         AString newBorderFileName = CaretFileDialog::getSaveFileNameDialog(DataFileTypeEnum::BORDER,
                                                                            this,
                                                                            "Choose Border File Name",
@@ -523,7 +527,11 @@ BorderPropertiesEditorDialog::okButtonClicked()
          * Close border
          */
         if (m_closedCheckBox->isChecked()) {
-            borderBeingEdited->addPointsToCloseBorderWithGeodesic(m_finishBorderSurfaceFile);    
+            borderBeingEdited->addPointsToCloseBorderWithGeodesic(m_finishBorderSurfaceFile);
+            borderBeingEdited->setClosed(true);
+        }
+        else {
+            borderBeingEdited->setClosed(false);
         }
         
         /*
