@@ -42,13 +42,14 @@
 #include "DisplayGroupEnumComboBox.h"
 #include "DisplayPropertiesBorders.h"
 #include "EnumComboBoxTemplate.h"
-#include "EventGraphicsUpdateAllWindows.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
 #include "EventManager.h"
 #include "EventUserInterfaceUpdate.h"
 #include "GuiManager.h"
 #include "SceneClass.h"
 #include "WuQDataEntryDialog.h"
 #include "WuQFactory.h"
+#include "WuQMacroManager.h"
 #include "WuQTabWidget.h"
 #include "WuQtUtilities.h"
 
@@ -66,13 +67,25 @@ using namespace caret;
 
 /**
  * Constructor.
+ *
+ * @param browserWindowIndex
+ *    Index of browser window
+ * @param parentObjectName
+ *    Name of parent object
+ * @param parent
+ *    The parent object
  */
 BorderSelectionViewController::BorderSelectionViewController(const int32_t browserWindowIndex,
-                                             QWidget* parent)
-: QWidget(parent)
+                                                             const QString& parentObjectName,
+                                                             QWidget* parent)
+: QWidget(parent),
+m_objectNamePrefix(parentObjectName
+                   + ":Borders")
 {
     m_browserWindowIndex = browserWindowIndex;
     
+    WuQMacroManager* macroManager = WuQMacroManager::instance();
+    
     QLabel* groupLabel = new QLabel("Group");
     m_bordersDisplayGroupComboBox = new DisplayGroupEnumComboBox(this);
     QObject::connect(m_bordersDisplayGroupComboBox, SIGNAL(displayGroupSelected(const DisplayGroupEnum::Enum)),
@@ -86,6 +99,11 @@ BorderSelectionViewController::BorderSelectionViewController(const int32_t brows
     m_bordersDisplayCheckBox = new QCheckBox("Display Borders");
     QObject::connect(m_bordersDisplayCheckBox, SIGNAL(clicked(bool)),
                      this, SLOT(processAttributesChanges()));
+    m_bordersDisplayCheckBox->setObjectName(m_objectNamePrefix
+                                            + ":DisplayBorders");
+    m_bordersDisplayCheckBox->setToolTip("Display Borders on Surfaces");
+    macroManager->addMacroSupportToObject(m_bordersDisplayCheckBox,
+                                          "Enable borders display");
     
     QWidget* attributesWidget = this->createAttributesWidget();
     QWidget* selectionWidget = this->createSelectionWidget();
@@ -97,9 +115,13 @@ BorderSelectionViewController::BorderSelectionViewController(const int32_t brows
     m_tabWidget->addTab(selectionWidget, 
                       "Selection");
     m_tabWidget->setCurrentWidget(attributesWidget);
+    m_tabWidget->getTabBar()->setObjectName(m_objectNamePrefix
+                                            + ":Tab");
+    macroManager->addMacroSupportToObjectWithToolTip(m_tabWidget->getTabBar(),
+                                                     "Features ToolBox Borders Tab",
+                                                     "Select tab");
     
     QVBoxLayout* layout = new QVBoxLayout(this);
-    //WuQtUtilities::setLayoutSpacingAndMargins(layout, 2, 2);
     layout->addWidget(m_bordersDisplayCheckBox);
     layout->addWidget(WuQtUtilities::createHorizontalLineWidget());
     layout->addLayout(groupLayout);
@@ -125,7 +147,11 @@ BorderSelectionViewController::~BorderSelectionViewController()
 QWidget* 
 BorderSelectionViewController::createSelectionWidget()
 {
-    m_borderClassNameHierarchyViewController = new GroupAndNameHierarchyViewController(m_browserWindowIndex);
+    m_borderClassNameHierarchyViewController = new GroupAndNameHierarchyViewController(m_browserWindowIndex,
+                                                                                       (m_objectNamePrefix
+                                                                                        + ":Selection"),
+                                                                                       "borders",
+                                                                                       this);
     
     return m_borderClassNameHierarchyViewController;
 }
@@ -136,9 +162,16 @@ BorderSelectionViewController::createSelectionWidget()
 QWidget* 
 BorderSelectionViewController::createAttributesWidget()
 {
+    WuQMacroManager* macroManager = WuQMacroManager::instance();
+    
     m_bordersContralateralCheckBox = new QCheckBox("Contralateral");
     QObject::connect(m_bordersContralateralCheckBox, SIGNAL(clicked(bool)),
                      this, SLOT(processAttributesChanges()));
+    m_bordersContralateralCheckBox->setObjectName(m_objectNamePrefix
+                                                  + ":Contralateral");
+    m_bordersContralateralCheckBox->setToolTip("Show Contralateral Borders");
+    macroManager->addMacroSupportToObject(m_bordersContralateralCheckBox,
+                                          "Enable contralateral borders display");
     
     std::vector<BorderDrawingTypeEnum::Enum> drawingTypeEnums;
     BorderDrawingTypeEnum::getAllEnums(drawingTypeEnums);
@@ -151,9 +184,15 @@ BorderSelectionViewController::createAttributesWidget()
         m_drawTypeComboBox->addItem(BorderDrawingTypeEnum::toGuiName(drawType),
                                     (int)drawType);
     }
-    m_drawTypeComboBox->setToolTip("Select the drawing style of borders");
+    m_drawTypeComboBox->setToolTip("Select the drawing style of borders\n"
+                                   "Lines are faster but have limited width\n"
+                                   "Polylines are slower but have unlimited width");
     QObject::connect(m_drawTypeComboBox, SIGNAL(activated(int)),
                      this, SLOT(processAttributesChanges()));
+    m_drawTypeComboBox->setObjectName(m_objectNamePrefix
+                                      + ":DrawingType");
+    macroManager->addMacroSupportToObject(m_drawTypeComboBox,
+                                          "Select border drawing style");
     
     QLabel* coloringLabel = new QLabel("Coloring");
     m_coloringTypeComboBox = new EnumComboBoxTemplate(this);
@@ -162,24 +201,30 @@ BorderSelectionViewController::createAttributesWidget()
     m_coloringTypeComboBox->getWidget()->setToolTip("Select the coloring assignment for borders");
     QObject::connect(m_coloringTypeComboBox, SIGNAL(itemActivated()),
                      this, SLOT(processAttributesChanges()));
+    m_coloringTypeComboBox->getWidget()->setObjectName(m_objectNamePrefix
+                                      + ":ColoringType");
+    macroManager->addMacroSupportToObject(m_coloringTypeComboBox->getWidget(),
+                                          "Select border coloring type");
     
     QLabel* standardColorLabel = new QLabel("Standard Color");
     m_standardColorComboBox = new CaretColorEnumComboBox(this);
+    m_standardColorComboBox->getWidget()->setObjectName((m_objectNamePrefix
+                                                         + ":Color"));
     m_standardColorComboBox->getWidget()->setToolTip("Select the standard color");
+    WuQMacroManager::instance()->addMacroSupportToObject(m_standardColorComboBox->getComboBox(),
+                                                         "Select border standard color");
     QObject::connect(m_standardColorComboBox, SIGNAL(colorSelected(const CaretColorEnum::Enum)),
                      this, SLOT(processAttributesChanges()));
 
-    float minLineWidth = 0;
+    float minLineWidth = 0.1;
     float maxLineWidth = 1000;
-    //BrainOpenGL::getMinMaxLineWidth(minLineWidth,
-    //                                maxLineWidth);
     
     QLabel* lineWidthLabel = new QLabel("Line Diameter");
     m_lineWidthSpinBox = WuQFactory::newDoubleSpinBox();
     m_lineWidthSpinBox->setFixedWidth(80);
     m_lineWidthSpinBox->setRange(minLineWidth,
                                  maxLineWidth);
-    m_lineWidthSpinBox->setSingleStep(1.0);
+    m_lineWidthSpinBox->setSingleStep(0.1);
     m_lineWidthSpinBox->setDecimals(1);
     m_lineWidthSpinBox->setSuffix("px");
     m_lineWidthSpinBox->setToolTip("Adjust the width of borders drawn as lines.\n"
@@ -191,22 +236,35 @@ BorderSelectionViewController::createAttributesWidget()
                                    "value of this control is changing");
     QObject::connect(m_lineWidthSpinBox, SIGNAL(valueChanged(double)),
                      this, SLOT(processAttributesChanges()));
+    m_lineWidthSpinBox->setObjectName(m_objectNamePrefix
+                                                       + ":LineWidth");
+    macroManager->addMacroSupportToObject(m_lineWidthSpinBox,
+                                          "Set border line width");
     
     QLabel* pointSizeLabel = new QLabel("Symbol Diameter");
     m_pointSizeSpinBox = WuQFactory::newDoubleSpinBox();
     m_pointSizeSpinBox->setFixedWidth(80);
     m_pointSizeSpinBox->setRange(minLineWidth,
                                  maxLineWidth);
-    m_pointSizeSpinBox->setSingleStep(1.0);
+    m_pointSizeSpinBox->setSingleStep(0.1);
     m_pointSizeSpinBox->setDecimals(1);
     m_pointSizeSpinBox->setToolTip("Adjust the size of borders drawn as points");
     m_pointSizeSpinBox->setSuffix("mm");
     QObject::connect(m_pointSizeSpinBox, SIGNAL(valueChanged(double)),
                      this, SLOT(processAttributesChanges()));
+    m_pointSizeSpinBox->setObjectName(m_objectNamePrefix
+                                                       + ":PointSize");
+    macroManager->addMacroSupportToObject(m_pointSizeSpinBox,
+                                          "Set border point size");
     
     m_enableUnstretchedLinesCheckBox = new QCheckBox("Unstretched Lines");
     QObject::connect(m_enableUnstretchedLinesCheckBox, SIGNAL(clicked(bool)),
                      this, SLOT(processAttributesChanges()));
+    m_enableUnstretchedLinesCheckBox->setToolTip("Disable lines that are invalid due to cuts in flat surface");
+    m_enableUnstretchedLinesCheckBox->setObjectName(m_objectNamePrefix
+                                      + ":UnstretchedLines");
+    macroManager->addMacroSupportToObject(m_enableUnstretchedLinesCheckBox,
+                                          "Enable border unstretched lines");
     
     m_unstretchedLinesLengthSpinBox = WuQFactory::newDoubleSpinBox();
     m_unstretchedLinesLengthSpinBox->setFixedWidth(80);
@@ -219,6 +277,10 @@ BorderSelectionViewController::createAttributesWidget()
     m_unstretchedLinesLengthSpinBox->setSuffix("mm");
     QObject::connect(m_unstretchedLinesLengthSpinBox, SIGNAL(valueChanged(double)),
                      this, SLOT(processAttributesChanges()));
+    m_unstretchedLinesLengthSpinBox->setObjectName(m_objectNamePrefix
+                                                    + ":UnstretchedLinesLength");
+    macroManager->addMacroSupportToObject(m_unstretchedLinesLengthSpinBox,
+                                          "Set border unstretched lines lenght");
     
     QLabel* aboveSurfaceLabel = new QLabel("Above Offset");
     m_aboveSurfaceOffsetSpinBox =WuQFactory::newDoubleSpinBox();
@@ -231,6 +293,10 @@ BorderSelectionViewController::createAttributesWidget()
                                                                                         "Use with caution."));
     QObject::connect(m_aboveSurfaceOffsetSpinBox, SIGNAL(valueChanged(double)),
                      this, SLOT(processAttributesChanges()));
+    m_aboveSurfaceOffsetSpinBox->setObjectName(m_objectNamePrefix
+                                                   + ":AboveSurfaceOffset");
+    macroManager->addMacroSupportToObject(m_aboveSurfaceOffsetSpinBox,
+                                          "Set border above surface offset");
     
     QWidget* gridWidget = new QWidget();
     QGridLayout* gridLayout = new QGridLayout(gridWidget);
@@ -324,7 +390,7 @@ BorderSelectionViewController::processAttributesChanges()
                                    m_unstretchedLinesLengthSpinBox->value());
     dpb->setAboveSurfaceOffset(m_aboveSurfaceOffsetSpinBox->value());
     
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     
     updateOtherBorderViewControllers();
 }
@@ -486,7 +552,7 @@ void
 BorderSelectionViewController::processSelectionChanges()
 {
     updateOtherBorderViewControllers();
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
 }
 
 /**
