@@ -32,22 +32,26 @@
 #include "DataFileTypeEnum.h"
 #include "EventListenerInterface.h"
 #include "SceneableInterface.h"
+#include "TileTabsLayoutConfigurationTypeEnum.h"
 
 class QAction;
 class QActionGroup;
-class QDockWidget;
 class QMenu;
 class QToolButton;
+class QDockWidget;
 
 namespace caret {
     class BrainBrowserWindowToolBar;
     class BrainBrowserWindowOrientedToolBox;
     class BrainOpenGLWidget;
+    class BrainOpenGLViewportContent;
     class BrowserWindowContent;
     class BrowserTabContent;
+    class EventBrowserTabReopenClosed;
+    class EventTileTabsGridConfigurationModification;
     class PlainTextStringBuilder;
     class SceneClassAssistant;
-    class TileTabsConfiguration;
+    class TileTabsLayoutBaseConfiguration;
 
     
     /**
@@ -81,6 +85,10 @@ namespace caret {
         
         void removeAndReturnAllTabs(std::vector<BrowserTabContent*>& allTabContent);
         
+        void getAllBrainOpenGLViewportContent(std::vector<const BrainOpenGLViewportContent*>& viewportContentOut) const;
+        
+        const BrainOpenGLViewportContent* getViewportContentForSelectedTab() const;
+        
         int32_t getBrowserWindowIndex() const;
 
         bool isTileTabsSelected() const;
@@ -97,11 +105,27 @@ namespace caret {
             LOAD_SPEC_FILE_WITH_DIALOG_VIA_COMMAND_LINE
         };
         
+        /**
+         * For loading scene from command line
+         */
+        enum class LoadSceneFromCommandLineDialogMode {
+            /** After loading scene, show the scene dialog */
+            SHOW_YES,
+            /** After loading scene, close the scene dialog */
+            SHOW_NO
+        };
+        
         void loadFilesFromCommandLine(const std::vector<AString>& filenames,
                                       const LoadSpecFileMode loadSpecFileMode);
         
+        void loadRecentScene(const AString& sceneFileName,
+                             const AString& sceneName);
+        
         void loadSceneFromCommandLine(const AString& sceneFileName,
-                                      const AString& sceneNameOrNumber);
+                                      const AString& sceneNameOrNumber,
+                                      const LoadSceneFromCommandLineDialogMode sceneDialogMode);
+        
+        void loadDirectoryFromCommandLine(const AString& directoryName);
         
         bool loadFilesFromNetwork(QWidget* parentForDialogs,
                                   const std::vector<AString>& filenames,
@@ -115,10 +139,6 @@ namespace caret {
         virtual void restoreFromScene(const SceneAttributes* sceneAttributes,
                                       const SceneClass* sceneClass);
 
-        TileTabsConfiguration* getSelectedTileTabsConfiguration();
-        
-        void setSelectedTileTabsConfiguration(TileTabsConfiguration* configuration);
-        
         void resetGraphicsWidgetMinimumSize();
         
         void setGraphicsWidgetFixedSize(const int32_t width,
@@ -136,10 +156,6 @@ namespace caret {
         
         virtual void getDescriptionOfContent(PlainTextStringBuilder& descriptionOut) const;
         
-        static int32_t loadRecentSpecFileMenu(QMenu* recentSpecFileMenu);
-        
-        static int32_t loadRecentSceneFileMenu(QMenu* recentSpecFileMenu);
-        
         float getOpenGLWidgetAspectRatio() const;
         
         bool changeInputModeToAnnotationsWarningDialog();
@@ -152,9 +168,19 @@ namespace caret {
         
         bool isOpenGLContextSharingValid() const;
 
+        AString getTileTabsConfigurationLabelText(const TileTabsLayoutConfigurationTypeEnum::Enum configurationMode,
+                                                  const bool includeRowsAndColumns) const;
+
+        void resizeDockWidgets(const QList<QDockWidget *> &docks, const QList<int> &sizes, Qt::Orientation orientation);
+        
+        void reopenLastClosedTab(EventBrowserTabReopenClosed& reopenTabEvent);
+
+        QAction* getViewTileTabsConfigurationDialogAction();
+
     protected:
-        void closeEvent(QCloseEvent* event);
-        void keyPressEvent(QKeyEvent* event);
+        void changeEvent(QEvent *event) override;
+        void closeEvent(QCloseEvent* event) override;
+        void keyPressEvent(QKeyEvent* event) override;
         
     private slots:        
         void processAboutWorkbench();
@@ -164,21 +190,28 @@ namespace caret {
         void processDuplicateTab();
         void processDataFileLocationOpen();
         void processDataFileOpen();
+        void processDataFileOpenQuickly();
+        void processOmeZarrDirectoryOpen();
+        void processOpenRecent();
         void processManageSaveLoadedFiles();
         void processCaptureImage();
-        void processRecordMovie();
+        void processMovieRecording();
         void processEditPreferences();
         void processCloseAllFiles();
         void processCloseWindow();
         void processExitProgram();
         void processMoveAllTabsToOneWindow();
         void processViewFullScreenSelected();
+        void processViewMaximizedSelected();
         void processViewTileTabs();
         void processViewTileTabsConfigurationDialog();
         void processShowHelpInformation();
         void processShowIdentifyBrainordinateDialog();
         void processGapsAndMargins();
         
+        void processViewTileTabsAutomaticCustomTriggered(QAction* action);
+        
+
         void processMoveOverlayToolBoxToLeft();
         void processMoveOverlayToolBoxToBottom();
         void processMoveOverlayToolBoxToFloat();
@@ -191,12 +224,6 @@ namespace caret {
         void processMoveSelectedTabToWindowMenuAboutToBeDisplayed();
         void processMoveSelectedTabToWindowMenuSelection(QAction*);
         
-        void processRecentSceneFileMenuAboutToBeDisplayed();
-        void processRecentSceneFileMenuSelection(QAction*);
-        
-        void processRecentSpecFileMenuAboutToBeDisplayed();
-        void processRecentSpecFileMenuSelection(QAction*);
-        
         void processShowOverlayToolBox(bool);
         void processShowFeaturesToolBox(bool);
         void processOverlayHorizontalToolBoxVisibilityChanged(bool);
@@ -205,9 +232,7 @@ namespace caret {
         void processFileMenuAboutToShow();
         void processDataMenuAboutToShow();
         void processViewMenuAboutToShow();
-        
-        void processTileTabsMenuAboutToBeDisplayed();
-        void processTileTabsMenuSelection(QAction*);
+        void processWindowMenuAboutToShow();
         
         void processSurfaceMenuInformation();
         void processSurfaceMenuPrimaryAnatomical();
@@ -216,23 +241,34 @@ namespace caret {
         void processConnectToConnectomeDataBase();
         
         void processHcpWebsiteInBrowser();
+        void processHcpUsersGroup();
         void processHcpFeatureRequestWebsiteInBrowser();
         void processReportWorkbenchBug();
+        void processHelpWorkbenchInstallationAssistant();
         
         void processShowSurfacePropertiesDialog();
+        void processShowVolumePropertiesDialog();
         
         void processDevelopGraphicsTiming();
-        
+        void processDevelopGraphicsTimingDuration();
+        void processDevelopOpenMPTesting();
+
         void processDevelopExportVtkFile();
+        void processDevelopCziFileTransformTesting();
+        void processDevelopOmeZarrOpenTesting();
         void developerMenuAboutToShow();
-        void developerMenuFlagTriggered(QAction*);
         
+        void processEditAnnotations();
+        void processEditBorders();
+        void processEditFoci();
+        void processEditSamples();
         void processProjectFoci();
         void processSplitBorderFiles();
         
         void processWindowMenuLockWindowAspectRatioTriggered(bool checked);
         void processWindowMenuLockAllTabAspectRatioTriggered(bool checked);
         void processToolBarLockWindowAndAllTabAspectTriggered(bool checked);
+        void processToolBarUndoUnlockWindowAndAllTabAspectTriggered();
         void processToolBarLockWindowAndAllTabAspectMenu(const QPoint& pos);
 
         void processEditMenuItemTriggered(QAction* action);
@@ -241,14 +277,16 @@ namespace caret {
         void aspectRatioDialogUpdateForTab(const double aspectRatio);
         void aspectRatioDialogUpdateForWindow(const double aspectRatio);
         
+        void processParallelTest();
+        
     private:
         /** Contains status of components such as enter/exit full screen */
         struct WindowComponentStatus {
-            bool isFeaturesToolBoxDisplayed;
-            bool isOverlayToolBoxDisplayed;
-            bool isToolBarDisplayed;
+            QString name;
+            bool isFeaturesToolBoxDisplayed = false;
+            bool isOverlayToolBoxDisplayed = true;
+            bool isToolBarDisplayed = true;
             QByteArray windowState;
-            QByteArray windowGeometry;
             QByteArray featuresGeometry;
         };
         
@@ -266,7 +304,7 @@ namespace caret {
                            BrowserTabContent* browserTabContent,
                            const CreateDefaultTabsMode createDefaultTabsMode,
                            QWidget* parent = 0,
-                           Qt::WindowFlags flags = 0);
+                           Qt::WindowFlags flags = Qt::WindowFlags());
         
         BrainBrowserWindow(const BrainBrowserWindow&);
         BrainBrowserWindow& operator=(const BrainBrowserWindow&);
@@ -298,11 +336,15 @@ namespace caret {
         void moveOverlayToolBox(Qt::DockWidgetArea area);
         void moveFeaturesToolBox(Qt::DockWidgetArea area);
         
+        void printWindowComponentStatus(const QString& modeText,
+                                        const WindowComponentStatus& wcs);
         void restoreWindowComponentStatus(const WindowComponentStatus& wcs);
         void saveWindowComponentStatus(WindowComponentStatus& wcs);
         
         void openSpecFile(const AString& specFileName);
         
+        void openDataFiles(const QStringList& selectedFiles);
+        
         void processViewFullScreen(bool showFullScreenDisplay,
                                    const bool saveRestoreWindowStatus);
         
@@ -316,6 +358,8 @@ namespace caret {
         void lockAllTabAspectRatios(const bool checked);
         void updateActionsForLockingAspectRatios();
         
+        void processToolBarLockWindowAndAllTabAspectsRatios(bool checked);
+        
         float getAspectRatioFromDialog(const AspectRatioMode aspectRatioMode,
                                        const QString& title,
                                        const float aspectRatio,
@@ -323,6 +367,8 @@ namespace caret {
         
         void saveBrowserWindowContentForScene();
         
+        void modifyTileTabsConfiguration(EventTileTabsGridConfigurationModification* modEvent);
+
         /** Index of this window */
         const int32_t m_browserWindowIndex;
         
@@ -340,10 +386,18 @@ namespace caret {
         
         QAction* m_duplicateTabAction;
         
+        QAction* m_reopenLastClosedTabAction;
+        
+        QAction* m_openOmeZarrDirectoryAction;
+        
         QAction* m_openFileAction;
         
+        QAction* m_openFileQuicklyAction;
+        
         QAction* m_openLocationAction;
         
+        QAction* m_openRecentAction;
+        
         QAction* m_manageFilesAction;
         
         QAction* m_closeSpecFileAction;
@@ -357,7 +411,7 @@ namespace caret {
         
         QAction* m_captureImageAction;
 
-        QAction* m_recordMovieAction;
+        QAction* m_movieRecordingAction;
         
         QAction* m_preferencesAction;
         
@@ -369,10 +423,14 @@ namespace caret {
         QMenu* m_viewMoveOverlayToolBoxMenu;
         
         QAction* m_viewFullScreenAction;
+        QAction* m_viewMaximizedAction;
         QAction* m_viewTileTabsAction;
         
-        QMenu* m_tileTabsMenu;
-        QAction* m_createAndEditTileTabsAction;
+        QAction* m_viewTileTabsConfigurationDialogAction;
+        QAction* m_viewAutomaticTileTabsConfigurationAction;
+        QAction* m_viewCustomTileTabsConfigurationAction;
+        QAction* m_viewManualTileTabsConfigurationAction;
+        std::vector<std::pair<QAction*, AString>> m_viewCustomTileTabsConfigurationActions;
         
         QAction* m_gapsAndMarginsAction;
         
@@ -395,13 +453,18 @@ namespace caret {
         QAction* m_connectToConnectomeDatabaseAction;
 
         QAction* m_helpHcpWebsiteAction;
+        QAction* m_helpHcpUsersAction;
         QAction* m_helpHcpFeatureRequestAction;
         QAction* m_helpWorkbenchBugReportAction;
+        QAction* m_helpWorkbenchInstallationAssistantAction;
         
         QAction* m_developMenuAction;
-        QActionGroup* m_developerFlagsActionGroup;
         QAction* m_developerGraphicsTimingAction;
+        QAction* m_developerGraphicsTimingDurationAction;
         QAction* m_developerExportVtkFileAction;
+        QAction* m_developerCziFileTransformTestingAction;
+        QAction* m_developerOpenMPTestingAction;
+        QAction* m_developerOmeZarrOpenAction;
         
         QAction* m_overlayToolBoxAction;
         
@@ -409,20 +472,21 @@ namespace caret {
         QAction* m_windowMenuLockAllTabAspectRatioAction;
         QAction* m_toolBarLockWindowAndAllTabAspectRatioAction;
         QToolButton* m_toolBarLockWindowAndAllTabAspectRatioButton;
-        
+        QAction* m_toolBarUndoUnlockWindowAndAllTabAspectRatioAction;
+        QToolButton* m_toolBarUndoUnlockWindowAndAllTabAspectRatioButton;
+
         QAction* m_featuresToolBoxAction;
-        
+
+        QAction* m_dataAnnotationsEditAction;
+        QAction* m_dataBordersEditAction;
+        QAction* m_dataFociEditAction;
         QAction* m_dataFociProjectAction;
         QAction* m_dataBorderFilesSplitAction;
+        QAction* m_dataPaletteEditorDialogAction;
+        QAction* m_dataSamplesEditAction;
         
         QMenu* m_moveSelectedTabToWindowMenu;
         
-        QMenu* m_recentSpecFileMenu;
-        AString m_recentSpecFileMenuOpenConfirmTitle;
-        AString m_recentSpecFileMenuLoadNoConfirmTitle;
-        
-        QMenu* m_recentSceneFileMenu;
-        
         QMenu* m_editMenu;
         QAction* m_editMenuRedoAction;
         QAction* m_editMenuUndoAction;
@@ -432,9 +496,6 @@ namespace caret {
         BrainBrowserWindowOrientedToolBox* m_overlayActiveToolBox;
         BrainBrowserWindowOrientedToolBox* m_featuresToolBox;
         
-        AString m_selectedTileTabsConfigurationUniqueIdentifier;
-        TileTabsConfiguration* m_defaultTileTabsConfiguration;
-        
         static AString s_previousOpenFileNameFilter;
         static AString s_previousOpenFileDirectory;
         static QByteArray s_previousOpenFileGeometry;
@@ -456,6 +517,16 @@ namespace caret {
         static int32_t s_sceneFileFirstWindowY;
         
         static std::set<BrainBrowserWindow*> s_brainBrowserWindows;
+        
+        QString m_objectNamePrefix;
+        
+        bool m_keyEventProcessingFlag = false;
+        
+        bool m_restoringSceneNoSaveWindowCompontentStatusFlag = false;
+        
+        const float m_developerTimingDuration = 10.0;
+        
+        const int32_t m_developerTimingIterations = 10;
     };
 #ifdef __BRAIN_BROWSER_WINDOW_DECLARE__
     std::set<BrainBrowserWindow*> BrainBrowserWindow::s_brainBrowserWindows;
