@@ -28,20 +28,27 @@
 #include "EventListenerInterface.h"
 #include "SceneableInterface.h"
 
+class QSplitter;
 class QTabWidget;
 
 namespace caret {
     class AnnotationSelectionViewController;
+    class AnnotationTextSubstitutionLayerSetViewController;
     class BorderSelectionViewController;
     class ChartTwoOverlaySetViewController;
     class ChartToolBoxViewController;
     class CiftiConnectivityMatrixViewController;
     class FiberOrientationSelectionViewController;
     class FociSelectionViewController;
+    class HistologyOverlaySetViewController;
+    class IdentificationDisplayWidget;
     class ImageSelectionViewController;
-    class LabelSelectionViewController;
+    class LabelSelectionViewWidget;
+    class MediaOverlaySetViewController;
     class OverlaySetViewController;
+    class SamplesSelectionViewController;
     class VolumeSurfaceOutlineSetViewController;
+    class WuQTabWidgetWithSizeHint;
     
     class BrainBrowserWindowOrientedToolBox :  public QDockWidget, public EventListenerInterface, public SceneableInterface {
         Q_OBJECT
@@ -53,15 +60,24 @@ namespace caret {
             TOOL_BOX_OVERLAYS_VERTICAL,
         };
         
+        static void setCombineFeaturesAndOverlayToolBox(const bool status);
+        
         BrainBrowserWindowOrientedToolBox(const int32_t browserWindowIndex,
-                                  const QString& title,
-                                  const ToolBoxType toolBoxType,
-                                  QWidget* parent = 0);
+                                          const QString& title,
+                                          const ToolBoxType toolBoxType,
+                                          const QString& parentObjectName,
+                                          QWidget* parent = 0);
         
         ~BrainBrowserWindowOrientedToolBox();
 
         void receiveEvent(Event* event);
         
+        void setSizeHintWidth(const int width);
+        
+        void setSizeHintHeight(const int height);
+        
+        virtual QSize sizeHint() const override;
+        
         virtual SceneClass* saveToScene(const SceneAttributes* sceneAttributes,
                                         const AString& instanceName);
         
@@ -77,13 +93,20 @@ namespace caret {
         BrainBrowserWindowOrientedToolBox(const BrainBrowserWindowOrientedToolBox&);
         BrainBrowserWindowOrientedToolBox& operator=(const BrainBrowserWindowOrientedToolBox&);
         
+        int addToScrolledAreaInTabWidget(QWidget* page,
+                                         const QString& label);
+        
         int addToTabWidget(QWidget* page,
                            const QString& label);
         
+        QWidget* createSplitterAndIdentificationWidget(const Qt::Orientation orientation);
+        
         OverlaySetViewController* m_overlaySetViewController;
         
         AnnotationSelectionViewController* m_annotationViewController;
         
+        AnnotationTextSubstitutionLayerSetViewController* m_annotationTextSubstitutionLayerSetViewController;
+        
         BorderSelectionViewController* m_borderSelectionViewController;
 
         ChartTwoOverlaySetViewController* m_chartOverlaySetViewController;
@@ -96,13 +119,21 @@ namespace caret {
         
         FociSelectionViewController* m_fociSelectionViewController;
         
+        HistologyOverlaySetViewController* m_histologyOverlaySetViewController;
+        
         ImageSelectionViewController* m_imageSelectionViewController;
         
-        LabelSelectionViewController* m_labelSelectionViewController;
+        LabelSelectionViewWidget* m_labelSelectionWidget;
+        
+        MediaOverlaySetViewController* m_mediaSelectionViewController;
+        
+        SamplesSelectionViewController* m_samplesSelectionViewController;
         
         VolumeSurfaceOutlineSetViewController* m_volumeSurfaceOutlineSetViewController;
         
-        QTabWidget* m_tabWidget;
+        WuQTabWidgetWithSizeHint* m_tabWidget;
+        
+        QTabWidget* m_annotationTabWidget;
         
         QString m_toolBoxTitle;
         
@@ -124,17 +155,35 @@ namespace caret {
         
         int32_t m_fociTabIndex;
         
+        int32_t m_histologyTabIndex;
+        
         int32_t m_imageTabIndex;
         
         int32_t m_labelTabIndex;
         
+        int32_t m_mediaTabIndex;
+        
+        int32_t m_samplesTabIndex;
+        
         int32_t m_volumeSurfaceOutlineTabIndex;
         
+        QSplitter* m_splitterWidget = NULL;
+        
+        IdentificationDisplayWidget* m_identificationWidget = NULL;
+        
+        int m_sizeHintWidth = -1;
+        
+        int m_sizeHintHeight = -1;
+        
         QSize m_minimumSizeAfterSceneRestored;
         QSize m_maximumSizeAfterSceneRestored;
         
+        static bool s_combineFeaturesAndOverlayToolBoxFlag;
     };
+
+#ifdef __BRAIN_BROWSER_WINDOW_ORIENTED_TOOLBOX_DEFINE__
+    bool BrainBrowserWindowOrientedToolBox::s_combineFeaturesAndOverlayToolBoxFlag = false;
+#endif  // __BRAIN_BROWSER_WINDOW_ORIENTED_TOOLBOX_DEFINE__
+#endif // __BRAIN_BROWSER_WINDOW_ORIENTED_TOOLBOX_H__
 }
 
-#endif // __BRAIN_BROWSER_WINDOW_ORIENTED_TOOLBOX_H__
-
