@@ -26,6 +26,7 @@
 #include <QStack>
 #include <QToolBar>
 
+#include "BrainBrowserWindowToolBarOrientation.h"
 #include "EnumComboBoxTemplate.h"
 #include "EventListenerInterface.h"
 #include "ModelTypeEnum.h"
@@ -42,25 +43,37 @@ class QDoubleSpinBox;
 class QHBoxLayout;
 class QIcon;
 class QLabel;
+class QMainWindow;
 class QMenu;
 class QRadioButton;
 class QSpinBox;
-class QTabBar;
 class QToolButton;
+class QVBoxLayout;
 
 namespace caret {
     
+    class BrainBrowserWindowToolBarAllSurface;
     class BrainBrowserWindowToolBarChartAxes;
     class BrainBrowserWindowToolBarChartAttributes;
+    class BrainBrowserWindowToolBarChartTwoAttributes;
+    class BrainBrowserWindowToolBarChartTwoOrientation;
+    class BrainBrowserWindowToolBarChartTwoOrientedAxes;
+    class BrainBrowserWindowToolBarChartTwoType;
     class BrainBrowserWindowToolBarChartType;
-    class BrainBrowserWindowToolBarClipping;
+    class BrainBrowserWindowToolBarHistology;
+    class BrainBrowserWindowToolBarImage;
     class BrainBrowserWindowToolBarSlicePlane;
     class BrainBrowserWindowToolBarSliceSelection;
+    class BrainBrowserWindowToolBarSurface;
     class BrainBrowserWindowToolBarSurfaceMontage;
     class BrainBrowserWindowToolBarTab;
+    class BrainBrowserWindowToolBarView;
     class BrainBrowserWindowToolBarVolumeMontage;
+    class BrainBrowserWindowToolBarVolumeMPR;
     class BrainBrowserWindow;
     class BrowserTabContent;
+    class EventBrowserTabReopenClosed;
+    class EventBrowserWindowTileTabOperation;
     class Model;
     class ModelSurface;
     class ModelVolumeInterface;
@@ -69,6 +82,16 @@ namespace caret {
     class Surface;
     class SurfaceSelectionViewController;
     class StructureSurfaceSelectionControl;
+    class UserInputModeAnnotations;
+    class UserInputModeBorders;
+    class UserInputModeFoci;
+    class UserInputModeImage;
+    class UserInputModeSamplesEdit;
+    class UserInputModeTileTabsLayout;
+    class UserInputModeView;
+    class UserInputModeVolumeEdit;
+    class UserInputModeAbstract;
+    class WuQTabBar;
     class WuQWidgetObjectGroup;
     
     class BrainBrowserWindowToolBar : public QToolBar, public EventListenerInterface, public SceneableInterface   {
@@ -80,13 +103,18 @@ namespace caret {
                                   BrowserTabContent* initialBrowserTabContent,
                                   QAction* overlayToolBoxAction,
                                   QAction* layersToolBoxAction,
+                                  QToolButton* toolBarLockWindowAndAllTabAspectRatioButton,
+                                  QToolButton* toolBarUndoUnlockWindowAndAllTabAspectRatioButton,
+                                  const QString& objectNamePrefix,
                                   BrainBrowserWindow* parentBrainBrowserWindow);
         
         ~BrainBrowserWindowToolBar();
         
-        void addNewTab();
+        BrowserTabContent* addNewTab();
         
-        void addNewTabCloneContent(BrowserTabContent* browserTabContentToBeCloned);
+        void reopenLastClosedTab(EventBrowserTabReopenClosed& reopenTabEvent);
+        
+        void addNewDuplicatedTab(BrowserTabContent* browserTabContentToBeCloned);
         
         void addNewTabWithContent(BrowserTabContent* browserTabContent);
         
@@ -94,16 +122,14 @@ namespace caret {
         
         void receiveEvent(Event* event);
         
+        int32_t getNumberOfTabs() const;
         
         virtual SceneClass* saveToScene(const SceneAttributes* sceneAttributes,
                                         const AString& instanceName);
         
         virtual void restoreFromScene(const SceneAttributes* sceneAttributes,
                                       const SceneClass* sceneClass);
-    signals:
-        void viewedModelChanged();
-        
-    private:        
+
         enum WidgetPlacement {
             WIDGET_PLACEMENT_NONE,
             WIDGET_PLACEMENT_BOTTOM,
@@ -112,36 +138,58 @@ namespace caret {
             WIDGET_PLACEMENT_TOP
         };
         
+        static QWidget* createToolWidget(const QString& name,
+                                         QWidget* childWidget,
+                                         const WidgetPlacement verticalBarPlacement,
+                                         const WidgetPlacement contentPlacement,
+                                         const int horizontalStretching);
+        
+    signals:
+        void viewedModelChanged();
+        
+    private:        
+        
         BrainBrowserWindowToolBar(const BrainBrowserWindowToolBar&);
         BrainBrowserWindowToolBar& operator=(const BrainBrowserWindowToolBar&);
         
         BrowserTabContent* getTabContentFromSelectedTab();
         BrowserTabContent* getTabContentFromTab(const int tabIndex);
         
+        int32_t getTabBarIndexWithBrowserTabIndex(const int32_t browserTabIndex);
+        
         Model* getDisplayedModel();
         
         int32_t loadIntoTab(const int32_t tabIndex,
                             Model* controller);
 
         void updateGraphicsWindow();
+        void updateGraphicsWindowAndYokedWindows();
         void updateUserInterface();
         void updateToolBox();
+        void updateAllTabNames();
         void updateTabName(const int32_t tabIndex);
-        void updateOtherYokedWindows();
+        void emitViewModelChangedSignal(); /* for use by friend classes */
         
         QWidget* createViewWidget();
         QWidget* createOrientationWidget();
         QWidget* createWholeBrainSurfaceOptionsWidget();
         QWidget* createVolumeIndicesWidget();
         QWidget* createModeWidget();
-        QWidget* createTabOptionsWidget();
+        QWidget* createTabOptionsWidget(QToolButton* toolBarLockWindowAndAllTabAspectRatioButton,
+                                        QToolButton* toolBarUndoUnlockWindowAndAllTabAspectRatioButton);
         QWidget* createChartAxesWidget();
         QWidget* createChartAttributesWidget();
+        QWidget* createChartTwoOrientationWidget();
+        QWidget* createChartTwoAttributesWidget();
+        QWidget* createChartTwoOrientedAxisWidget();
         QWidget* createChartTypeWidget();
+        QWidget* createChartTypeTwoWidget();
+        QWidget* createHistologyWidget();
+        QWidget* createImageWidget();
         QWidget* createSingleSurfaceOptionsWidget();
         QWidget* createSurfaceMontageOptionsWidget();
-        QWidget* createClippingOptionsWidget();
         QWidget* createVolumeMontageWidget();
+        QWidget* createVolumeMprWidget();
         QWidget* createVolumePlaneWidget();
         
         ModelTypeEnum::Enum updateViewWidget(BrowserTabContent* browserTabContent);
@@ -154,66 +202,67 @@ namespace caret {
         void updateSurfaceMontageOptionsWidget(BrowserTabContent* browserTabContent);
         void updateChartAxesWidget(BrowserTabContent* browserTabContent);
         void updateChartAttributesWidget(BrowserTabContent* browserTabContent);
+        void updateChartTwoAttributesWidget(BrowserTabContent* browserTabContent);
+        void updateChartTwoOrientedAxesWidget(BrowserTabContent* browserTabContent);
+        void updateChartTwoOrientationWidget(BrowserTabContent* browserTabContent);
         void updateChartTypeWidget(BrowserTabContent* browserTabContent);
+        void updateChartTypeTwoWidget(BrowserTabContent* browserTabContent);
+        void updateHistologyWidget(BrowserTabContent* browserTabContent);
+        void updateImageWidget(BrowserTabContent* browserTabContent);
         void updateVolumeMontageWidget(BrowserTabContent* browserTabContent);
+        void updateVolumeMprWidget(BrowserTabContent* browserTabConent);
         void updateVolumePlaneWidget(BrowserTabContent* browserTabContent);
-        void updateClippingOptionsWidget(BrowserTabContent* browserTabContent);
-        
-        QWidget* createToolWidget(const QString& name,
-                                  QWidget* childWidget,
-                                  const WidgetPlacement verticalBarPlacement,
-                                  const WidgetPlacement contentPlacement,
-                                  const int horizontalStretching);
         
         QWidget* viewWidget;
         QWidget* orientationWidget;
         QWidget* wholeBrainSurfaceOptionsWidget;
         QWidget* volumeIndicesWidget;
         QWidget* modeWidget;
-        QWidget* windowWidget;
+        QWidget* tabMiscWidget;
         QWidget* singleSurfaceSelectionWidget;
         QWidget* surfaceMontageSelectionWidget;
-        QWidget* m_clippingOptionsWidget;
         QWidget* volumeMontageWidget;
+        QWidget* volumeMprWidget;
         QWidget* volumePlaneWidget;
         QWidget* chartTypeWidget;
+        QWidget* chartTypeTwoWidget;
         QWidget* chartAxesWidget;
         QWidget* chartAttributesWidget;
+        QWidget* chartTwoOrientationWidget;
+        QWidget* chartTwoAttributesWidget;
+        QWidget* chartTwoOrientedAxesWidget;
+        QWidget* histologyWidget;
+        QWidget* imageWidget;
+
+        UserInputModeAbstract* selectedUserInputProcessor = NULL;
+        UserInputModeAnnotations* userInputAnnotationsModeProcessor;
+        UserInputModeView* userInputViewModeProcessor;
+        UserInputModeBorders* userInputBordersModeProcessor;
+        UserInputModeFoci* userInputFociModeProcessor;
+        UserInputModeImage* userInputImageModeProcessor;
+        UserInputModeSamplesEdit* userInputSamplesEditProcessor;
+        UserInputModeTileTabsLayout* userInputTileTabsManualLayoutProcessor;
+        UserInputModeVolumeEdit* userInputVolumeEditModeProcessor;
+        
+        /* DO NOT delete these as the corresponding processor is the parent */
+        QWidget* annotateModeWidget;
+        QWidget* bordersModeWidget;
+        QWidget* fociModeWidget;
+        QWidget* imageModeWidget;
+        QWidget* samplesModeWidget;
+        QWidget* tileModeWidget;
+        QWidget* volumeModeWidget;
         
-        WuQWidgetObjectGroup* viewWidgetGroup;
-        WuQWidgetObjectGroup* orientationWidgetGroup;
-        WuQWidgetObjectGroup* wholeBrainSurfaceOptionsWidgetGroup;
         WuQWidgetObjectGroup* modeWidgetGroup;
-        WuQWidgetObjectGroup* singleSurfaceSelectionWidgetGroup;
+        
+        QVBoxLayout* m_toolBarMainLayout;
         
         QWidget* fullToolBarWidget;
         QWidget* m_toolbarWidget;
         QHBoxLayout* toolbarWidgetLayout;
         
         QWidget* tabBarWidget;
-        QTabBar* tabBar;
-        
-        /** Widget displayed at bottom of toolbar for mouse input controls */
-        QWidget* userInputControlsWidget;
-
-        /** Layout for widget displayed at bottom of toolbar for mouse input controls */
-        QHBoxLayout* userInputControlsWidgetLayout;
-
-        /** Is set to the user input widget provided by the user input processor */
-        QWidget* userInputControlsWidgetActiveInputWidget;
-        
-        /**
-         * When updating, no signals should be emitted.  This variable
-         * is incremented at the beginning of an update method and
-         * decremented at the end of the update method.  If it is 
-         * non-zero in a slot method, then a signal was emitted during
-         * the update and the widget that emitted the signal should
-         * have its signal blocked.
-         */
-        void incrementUpdateCounter(const char* methodName);
-        void decrementUpdateCounter(const char* methodName);
-        void checkUpdateCounter();
-        int updateCounter;
+        WuQTabBar* tabBar;
         
         void removeAndReturnAllTabs(std::vector<BrowserTabContent*>& allTabContent);
         
@@ -221,8 +270,10 @@ namespace caret {
         
         void removeTabWithContent(BrowserTabContent* browserTabContent);
         
+        void processTileTabOperationEvent(EventBrowserWindowTileTabOperation* tileTabsEvent);
+        
     public slots:
-        void closeSelectedTab();
+        void closeSelectedTabFromFileMenu();
 
         void moveTabsToNewWindows();
         
@@ -238,138 +289,112 @@ namespace caret {
         
         void showHideToolBar(bool showIt);
         
+        void showMacroDialog();
+        
     private slots:
         void selectedTabChanged(int indx);
-        void tabClosed(int index);
         void tabMoved(int, int);
+        void tabCloseSelected(int);
+        void showTabMenu(const QPoint& pos);
+        void tabBarMousePressedSlot(QMouseEvent* event);
+        void tabBarMouseReleasedSlot(QMouseEvent* event);
         
     private:
-        void insertTabAtIndex(BrowserTabContent* browserTabContent,
-                              const int32_t insertAtIndex);
+        enum class InsertTabMode {
+            APPEND,
+            AT_TAB_BAR_INDEX
+        };
         
-        void addOrInsertNewTab(BrowserTabContent* browserTabContent,
-                               const int32_t insertAtIndex);
+        /**
+         * Remove tab mode
+         */
+        enum class RemoveTabMode {
+            /** Close tab but allow it to be reopened later */
+            CLOSE_TAB_CONTENT_FOR_REOPENING,
+            /** Delete the tab, CANNOT be reopened */
+            DELETE_TAB_CONTENT,
+            /** Ignore tab content, caller may delete it or transfer it (content attached to QTab may be NULL too) */
+            INGORE_TAB_CONTENT
+        };
         
-        void removeTab(int index);
+        bool allowAddingNewTab();
+        
+        void insertTabContentPrivate(const InsertTabMode insertTabMode,
+                                     BrowserTabContent* browserTabContent,
+                                     const int32_t tabBarIndex);
+
+        void removeTab(int index,
+                       const RemoveTabMode removeTabMode);
+        
+        void tabClosed(int index,
+                       const RemoveTabMode removeTabMode);
+        
+        BrowserTabContent* insertNewTabAtTabBarIndex(int32_t tabBarIndex);
+        void insertAndCloneTabContentAtTabBarIndex(const BrowserTabContent* tabContentToBeCloned,
+                                                 const int32_t tabBarIndex);
+        void replaceBrowserTabs(const std::vector<BrowserTabContent*>& browserTabs);
         
         BrowserTabContent* createNewTab(AString& errorMessage);
         
-        QRadioButton* viewModeSurfaceRadioButton;
-        QRadioButton* viewModeSurfaceMontageRadioButton;
-        QRadioButton* viewModeVolumeRadioButton;
-        QRadioButton* viewModeWholeBrainRadioButton;
-        QRadioButton* viewModeChartRadioButton;
-        
         QAction* customViewAction;
+        
+        QMenu* customViewMenu;
 
     private slots:
-        void viewModeRadioButtonClicked(QAbstractButton*);
-        
         void customViewActionTriggered();
         
-    private:
-        QAction* orientationLateralMedialToolButtonAction;
-        QAction* orientationDorsalVentralToolButtonAction;
-        QAction* orientationAnteriorPosteriorToolButtonAction;
+        void customViewMenuAboutToShow();
         
-        QToolButton* orientationLateralMedialToolButton;
-        QToolButton* orientationDorsalVentralToolButton;
-        QToolButton* orientationAnteriorPosteriorToolButton;
+        void customViewMenuTriggered(QAction* action);
         
-        QAction* orientationLeftOrLateralToolButtonAction;
-        QAction* orientationRightOrMedialToolButtonAction;
-        QAction* orientationAnteriorToolButtonAction;
-        QAction* orientationPosteriorToolButtonAction;
-        QAction* orientationDorsalToolButtonAction;
-        QAction* orientationVentralToolButtonAction;
-        
-        QToolButton* orientationLeftOrLateralToolButton;
-        QToolButton* orientationRightOrMedialToolButton;
-        QToolButton* orientationAnteriorToolButton;
-        QToolButton* orientationPosteriorToolButton;
-        QToolButton* orientationDorsalToolButton;
-        QToolButton* orientationVentralToolButton;
-
-        QAction* orientationResetToolButtonAction;
-        QToolButton* orientationCustomViewSelectToolButton;
-        
-        QIcon* viewOrientationLeftIcon;
-        QIcon* viewOrientationRightIcon;
-        QIcon* viewOrientationAnteriorIcon;
-        QIcon* viewOrientationPosteriorIcon;
-        QIcon* viewOrientationDorsalIcon;
-        QIcon* viewOrientationVentralIcon;
-        QIcon* viewOrientationLeftLateralIcon;
-        QIcon* viewOrientationLeftMedialIcon;
-        QIcon* viewOrientationRightLateralIcon;
-        QIcon* viewOrientationRightMedialIcon;
-    private slots:
-        void orientationLeftOrLateralToolButtonTriggered(bool checked);
-        void orientationRightOrMedialToolButtonTriggered(bool checked);
-        void orientationAnteriorToolButtonTriggered(bool checked);
-        void orientationPosteriorToolButtonTriggered(bool checked);
-        void orientationDorsalToolButtonTriggered(bool checked);
-        void orientationVentralToolButtonTriggered(bool checked);
-        void orientationResetToolButtonTriggered(bool checked);
-        
-        void orientationLateralMedialToolButtonTriggered(bool checked);
-        void orientationDorsalVentralToolButtonTriggered(bool checked);
-        void orientationAnteriorPosteriorToolButtonTriggered(bool checked);
-
-    
-    private:
-        QComboBox*      wholeBrainSurfaceTypeComboBox;
-        QCheckBox*      wholeBrainSurfaceLeftCheckBox;
-        QCheckBox*      wholeBrainSurfaceRightCheckBox;
-        QCheckBox*      wholeBrainSurfaceCerebellumCheckBox;
-        QDoubleSpinBox* wholeBrainSurfaceSeparationLeftRightSpinBox;
-        QDoubleSpinBox* wholeBrainSurfaceSeparationCerebellumSpinBox;
-
-    private slots:
-        void wholeBrainSurfaceTypeComboBoxIndexChanged(int indx);
-        void wholeBrainSurfaceLeftCheckBoxStateChanged(int state);
-        void wholeBrainSurfaceRightCheckBoxStateChanged(int state);
-        void wholeBrainSurfaceCerebellumCheckBoxStateChanged(int state);
-        void wholeBrainSurfaceSeparationLeftRightSpinBoxValueChanged(double d);
-        void wholeBrainSurfaceSeparationCerebellumSpinBoxSelected(double d);
-        void wholeBrainSurfaceLeftToolButtonTriggered(bool checked);
-        void wholeBrainSurfaceRightToolButtonTriggered(bool checked);
-        void wholeBrainSurfaceCerebellumToolButtonTriggered(bool checked);
-    
-    private:
-        StructureSurfaceSelectionControl* surfaceSurfaceSelectionControl;
-        
-    private slots:
-        void surfaceSelectionControlChanged(const StructureEnum::Enum,
-                                            ModelSurface*);
+        void sceneToolButtonClicked();
         
     private:
+        
+        QToolButton* m_movieToolButton = NULL;
+        
+    private:
+        BrainBrowserWindowToolBarAllSurface* m_allSurfaceToolBarComponent;
         BrainBrowserWindowToolBarChartAxes* m_chartAxisToolBarComponent;
+        BrainBrowserWindowToolBarChartTwoType* m_chartTwoTypeToolBarComponent;
         BrainBrowserWindowToolBarChartType* m_chartTypeToolBarComponent;
         BrainBrowserWindowToolBarChartAttributes* m_chartAttributesToolBarComponent;
+        BrainBrowserWindowToolBarChartTwoOrientedAxes* m_chartTwoOrientedAxesToolBarComponent;
+        BrainBrowserWindowToolBarChartTwoOrientation* m_chartTwoOrientationToolBarComponent;
+        BrainBrowserWindowToolBarChartTwoAttributes* m_chartTwoAttributesToolBarComponent;
+        BrainBrowserWindowToolBarHistology* m_histologyToolBarComponent;
+        BrainBrowserWindowToolBarImage* m_imageToolBarComponent;
+        BrainBrowserWindowToolBarOrientation* m_orientationToolBarComponent;
+        BrainBrowserWindowToolBarSurface* m_surfaceToolBarComponent;
         BrainBrowserWindowToolBarSurfaceMontage* m_surfaceMontageToolBarComponent;
-        
-        BrainBrowserWindowToolBarClipping* m_clippingToolBarComponent;
+        BrainBrowserWindowToolBarView* m_viewToolBarComponent;
         BrainBrowserWindowToolBarSlicePlane* m_slicePlaneComponent;
         BrainBrowserWindowToolBarSliceSelection* m_sliceSelectionComponent;
         BrainBrowserWindowToolBarVolumeMontage* m_volumeMontageComponent;
+        BrainBrowserWindowToolBarVolumeMPR* m_volumeMprComponent;
         BrainBrowserWindowToolBarTab* m_tabOptionsComponent;
         
     private slots:
-        void modeInputModeActionTriggered(QAction*);
+        void modeInputModeRadioButtonClicked(QAbstractButton*);
         
     private:
         void updateDisplayedModeUserInputWidget();
-        QActionGroup* modeInputModeActionGroup;
-        QAction* modeInputModeBordersAction;
-        QAction* modeInputModeFociAction;
-        QAction* modeInputModeViewAction;
-        QAction* modeInputVolumeEditAction;
+        QButtonGroup* modeInputModeRadioButtonGroup;
+        QRadioButton* modeInputModeAnnotationsRadioButton;
+        QRadioButton* modeInputModeBordersRadioButton;
+        QRadioButton* modeInputModeFociRadioButton;
+        QRadioButton* modeInputModeImageRadioButton;
+        QRadioButton* modeInputModeViewRadioButton;
+        QRadioButton* modeInputModeSamplesEditRadioButton;
+        QRadioButton* modeInputModeTileTabsManualLayoutRadioButton;
+        QRadioButton* modeInputVolumeEditRadioButton;
         
     private:
         QAction* toolBarToolButtonAction;
         QAction* toolBoxToolButtonAction;
     
+        BrainBrowserWindow* m_parentBrainBrowserWindow = NULL;
+        
         int32_t browserWindowIndex;
         
     private slots:
@@ -377,14 +402,23 @@ namespace caret {
         
     private:
         friend class BrainBrowserWindow;
+        friend class BrainBrowserWindowToolBarAllSurface;
         friend class BrainBrowserWindowToolBarChartAxes;
+        friend class BrainBrowserWindowToolBarChartTwoOrientation;
+        friend class BrainBrowserWindowToolBarChartTwoAttributes;
+        friend class BrainBrowserWindowToolBarChartTwoType;
         friend class BrainBrowserWindowToolBarChartType;
-        friend class BrainBrowserWindowToolBarClipping;
         friend class BrainBrowserWindowToolBarComponent;
+        friend class BrainBrowserWindowToolBarHistology;
+        friend class BrainBrowserWindowToolBarImage;
+        friend class BrainBrowserWindowToolBarMode;
+        friend class BrainBrowserWindowToolBarOrientation;
         friend class BrainBrowserWindowToolBarSurfaceMontage;
         friend class BrainBrowserWindowToolBarSlicePlane;
         friend class BrainBrowserWindowToolBarSliceSelection;
+        friend class BrainBrowserWindowToolBarSurface;
         friend class BrainBrowserWindowToolBarTab;
+        friend class BrainBrowserWindowToolBarView;
         friend class BrainBrowserWindowToolBarVolumeMontage;
         
         /**
@@ -394,10 +428,26 @@ namespace caret {
          * box around it.
          */
         int32_t m_tabIndexForTileTabsHighlighting;
+        QTimer* m_tileTabsHighlightingTimer = NULL;
+        bool m_tileTabsHighlightingTimerEnabledFlag = true;
+        
+        QString m_objectNamePrefix;
         
         bool isContructorFinished;
         bool isDestructionInProgress;
+        
+        /**
+         * Tracks when update is performed to catch incorrectly emitted signals.
+         */
+        bool m_performingUpdateFlag = false;
+        
+        const int32_t TAB_INDEX_APPEND_TO_TOOLBAR = 10000;
+        
+        friend class BrainBrowserWindowToolBarTabPopUpMenu;
     };
+    
+#ifdef __BRAIN_BROWSER_WINDOW_TOOLBAR_DECLARE__
+#endif // __BRAIN_BROWSER_WINDOW_TOOLBAR_DECLARE__
 }
 
 #endif // __BRAIN_BROWSER_WINDOW_TOOLBAR_H__
