@@ -26,7 +26,7 @@ class QButtonGroup;
 class QRadioButton;
 
 #include "BrainBrowserWindowToolBarComponent.h"
-#include "ChartDataTypeEnum.h"
+#include "ChartOneDataTypeEnum.h"
 
 namespace caret {
 
@@ -36,7 +36,8 @@ namespace caret {
         Q_OBJECT
         
     public:
-        BrainBrowserWindowToolBarChartType(BrainBrowserWindowToolBar* parentToolBar);
+        BrainBrowserWindowToolBarChartType(BrainBrowserWindowToolBar* parentToolBar,
+                                           const QString& parentObjectName);
         
         virtual ~BrainBrowserWindowToolBarChartType();
         
@@ -59,7 +60,7 @@ namespace caret {
 
         BrainBrowserWindowToolBar* m_parentToolBar;
         
-        std::vector<std::pair<ChartDataTypeEnum::Enum, QRadioButton*> > m_chartTypeRadioButtons;
+        std::vector<std::pair<ChartOneDataTypeEnum::Enum, QRadioButton*> > m_chartTypeRadioButtons;
         
         QButtonGroup* m_chartTypeButtonGroup;
         
