@@ -28,7 +28,13 @@
 #undef __CARET_FILE_DIALOG_DECLARE__
 
 #include "Brain.h"
+#include "CaretAssert.h"
+#include "CaretPreferences.h"
 #include "GuiManager.h"
+#include "RecentFileItem.h"
+#include "RecentFileItemsFilter.h"
+#include "RecentFileItemsContainer.h"
+#include "SessionManager.h"
 
 using namespace caret;
 
@@ -97,6 +103,19 @@ FilterFilesProxyModel::setDataFileTypeForFiltering(const DataFileTypeEnum::Enum
     m_dataFileType = dataFileType;
 }
 
+/**
+ * Must override sort() and then call the model's sort().  Without this, columns sort as strings
+ * and date/size sorting does not work properly.
+ *
+ * From: https://stackoverflow.com/questions/53789779/qfiledialog-with-proxy-model-sort-by-date-actually-uses-alphabetical-order-of-da
+ */
+void
+FilterFilesProxyModel::sort(int column, Qt::SortOrder order)
+{
+    sourceModel()->sort(column,
+                        order);
+}
+
 /**
  * \class caret::CaretFileDialog 
  * \brief Adds additional functionality over Qt's QFileDialog.
@@ -104,27 +123,47 @@ FilterFilesProxyModel::setDataFileTypeForFiltering(const DataFileTypeEnum::Enum
 
 /**
  * Constructor.
+ * @param mode
+ *    The mode for open/save
+ * @param parent
+ *    Parent on which this dialog is displayed.
+ * @param f
+ *    Qt's window flags
  */
-CaretFileDialog::CaretFileDialog(QWidget* parent,
+CaretFileDialog::CaretFileDialog(const Mode mode,
+                                 QWidget* parent,
                                  Qt::WindowFlags f)
 : QFileDialog(parent,
-                f)
+                f),
+m_mode(mode)
 {
     this->initializeCaretFileDialog();
         
     this->setDirectory(GuiManager::get()->getBrain()->getCurrentDirectory());
 }
+
 /**
  * Constructor.
+ * @param mode
+ *    The mode for open/save
+ * @param parent
+ *    Parent on which this dialog is displayed.
+ * @param caption
+ *    Caption for dialog (if not provided a default caption is shown)
+ * @param directory
+ *    Directory show by dialog (Brain's current directory if empty string)
+ * @param filter
  */
-CaretFileDialog::CaretFileDialog(QWidget* parent,
+CaretFileDialog::CaretFileDialog(const Mode mode,
+                                 QWidget* parent,
                                  const QString& caption,
                                  const QString& directory,
                                  const QString& filter)
 : QFileDialog(parent,
               caption,
               directory,
-              filter)
+              filter),
+m_mode(mode)
 {
     this->initializeCaretFileDialog();
     
@@ -147,15 +186,24 @@ void
 CaretFileDialog::initializeCaretFileDialog()
 {
     /*
-     * Create a proxy model that hides files that do not match the file filter.
-     * On Macs, Qt shows files that do not match the file filter as disabled
-     * but we don't want them displayed.  The dialog will take ownership of 
-     * the proxy model so it does not need to be deleted by this instance.
+     * We MUST use a non-native dialog.
+     * Otherwise, the file filter proxy model 
+     * will not work.
+     */
+    setOption(QFileDialog::DontUseNativeDialog);
+    
+    /*
+     * Create a proxy model for filtering the data files.
+     * The proxy is used to limit the displayed files to 
+     * only those with the proper file extension.  In addition,
+     * since CIFTI and NIFTI files end in ".nii", filtering is
+     * performed so that CIFTI files are not displayed when
+     * the user selectes Volume files.
      */
     m_filterFilesProxyModel = new FilterFilesProxyModel();
     this->setProxyModel(m_filterFilesProxyModel);
     
-#ifdef Q_OS_MACX
+#ifdef Q_OS_MACOS
     /*
      * On Macs, add /Volumes to the sidebar URLs 
      * so that mounted disks can be accessed.
@@ -163,10 +211,29 @@ CaretFileDialog::initializeCaretFileDialog()
     QList<QUrl> urls = this->sidebarUrls();
     urls.append(QUrl::fromLocalFile("/Volumes"));
     this->setSidebarUrls(urls);
-#endif // Q_OS_MACX
+#endif // Q_OS_MACOS
     
     QObject::connect(this, SIGNAL(filterSelected(const QString&)),
                      this, SLOT(fileFilterWasChanged(const QString&)));
+    
+    /*
+     * Add recent directories to history
+     */
+    CaretPreferences* prefs = SessionManager::get()->getCaretPreferences();
+    std::vector<AString> recentDirectories;
+    const bool favoritesFirstFlag(false);
+    prefs->getRecentDirectoriesForOpenFileDialogHistory(favoritesFirstFlag,
+                                                        recentDirectories);
+    const int32_t displayCount = std::min(static_cast<int32_t>(recentDirectories.size()),
+                                          10);
+    
+    QStringList historyList;
+    for (int32_t i = 0; i < displayCount; i++) {
+        CaretAssertVectorIndex(recentDirectories, i);
+        historyList.push_front(recentDirectories[i]);
+    }
+    
+    setHistory(historyList);
 }
 
 /**
@@ -179,7 +246,7 @@ void
 CaretFileDialog::setVisible(bool visible)
 {
     if (visible) {
-        fileFilterWasChanged(selectedFilter());
+        fileFilterWasChanged(selectedNameFilter());
     }
     
     QFileDialog::setVisible(visible);
@@ -234,12 +301,13 @@ CaretFileDialog::getOpenFileNameDialog(QWidget *parent,
                                      QString *selectedFilter,
                                      Options options )
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_OPEN,
+                        parent,
                         caption,
                         dir,
                         filter);
     if (selectedFilter != 0) {
-        cfd.selectFilter(*selectedFilter);
+        cfd.selectNameFilter(*selectedFilter);
     }
     cfd.setOptions(options);
     cfd.setAcceptMode(CaretFileDialog::AcceptOpen);
@@ -255,6 +323,53 @@ CaretFileDialog::getOpenFileNameDialog(QWidget *parent,
     return QString();
 }
 
+
+/**
+ * Like QFileDialog::getOpenFileName() except that this
+ * NEVER uses the native file dialog thus providing
+ * a consistent user-interface across platforms.
+ *
+ * @param dataFileType
+ *    Type of Workbench data file
+ * @param parent
+ *    Parent on which this dialog is displayed.
+ * @param caption
+ *    Caption for dialog (if not provided a default caption is shown)
+ * @param dir
+ *    Directory show by dialog (Brain's current directory if empty string)
+ * @param options
+ *    Options (see QFileDialog).
+ * @return
+ *    Name of file selected or empty string if user cancelled.
+ */
+QString
+CaretFileDialog::getOpenFileNameDialog(const DataFileTypeEnum::Enum dataFileType,
+                                       QWidget* parent,
+                                       const QString& caption,
+                                       const QString& dir,
+                                       Options options)
+{
+    
+    CaretFileDialog cfd(Mode::MODE_OPEN,
+                        parent,
+                        caption,
+                        dir,
+                        DataFileTypeEnum::toQFileDialogFilterForReading(dataFileType));
+    cfd.selectNameFilter(DataFileTypeEnum::toQFileDialogFilterForReading(dataFileType));
+    cfd.setOptions(options);
+    cfd.setAcceptMode(CaretFileDialog::AcceptOpen);
+    cfd.setFileMode(CaretFileDialog::AnyFile);
+    
+    if (cfd.exec() == CaretFileDialog::Accepted) {
+        QStringList selectedFiles = cfd.selectedFiles();
+        if (selectedFiles.size() > 0) {
+            return selectedFiles[0];
+        }
+    }
+    
+    return QString();
+}
+
 /**
  * Like QFileDialog::getSaveFileName() except that this
  * NEVER uses the native file dialog thus providing 
@@ -283,12 +398,13 @@ CaretFileDialog::getSaveFileNameDialog(QWidget *parent,
                                      QString *selectedFilter,
                                      Options options)
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_SAVE,
+                        parent,
                         caption,
                         dir,
                         filter);
     if (selectedFilter != 0) {
-        cfd.selectFilter(*selectedFilter);
+        cfd.selectNameFilter(*selectedFilter);
     }
     cfd.setOptions(options);
     cfd.setAcceptMode(QFileDialog::AcceptSave);
@@ -334,11 +450,12 @@ CaretFileDialog::getSaveFileNameDialog(const DataFileTypeEnum::Enum dataFileType
                                        const QString &dir,
                                        Options options)
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_SAVE,
+                        parent,
                         caption,
                         dir,
-                        DataFileTypeEnum::toQFileDialogFilter(dataFileType));
-    cfd.selectFilter(DataFileTypeEnum::toQFileDialogFilter(dataFileType));
+                        DataFileTypeEnum::toQFileDialogFilterForWriting(dataFileType));
+    cfd.selectNameFilter(DataFileTypeEnum::toQFileDialogFilterForWriting(dataFileType));
     cfd.setOptions(options);
     cfd.setAcceptMode(QFileDialog::AcceptSave);
     cfd.setFileMode(CaretFileDialog::AnyFile);
@@ -348,7 +465,7 @@ CaretFileDialog::getSaveFileNameDialog(const DataFileTypeEnum::Enum dataFileType
         if (selectedFiles.size() > 0) {
             AString filename = DataFileTypeEnum::addFileExtensionIfMissing(selectedFiles[0], 
                                                                            dataFileType);
-            return filename;
+            return std::move(filename);
         }
     }
     
@@ -386,9 +503,12 @@ CaretFileDialog::getChooseFileNameDialog(const DataFileTypeEnum::Enum dataFileTy
                                          const QString& directoryOrFileName,
                                          QWidget *parent)
 {
-    CaretFileDialog fd(parent);
-    fd.setFilter(DataFileTypeEnum::toQFileDialogFilter(dataFileType));
-    fd.selectFilter(DataFileTypeEnum::toQFileDialogFilter(dataFileType));
+    CaretFileDialog fd(Mode::MODE_SAVE,
+                       parent);
+    if (dataFileType != DataFileTypeEnum::UNKNOWN) {
+        fd.setNameFilter(DataFileTypeEnum::toQFileDialogFilterForWriting(dataFileType));
+    }
+    fd.selectNameFilter(DataFileTypeEnum::toQFileDialogFilterForWriting(dataFileType));
     fd.setAcceptMode(CaretFileDialog::AcceptSave);
     fd.setFileMode(CaretFileDialog::AnyFile);
     fd.setViewMode(CaretFileDialog::List);
@@ -413,7 +533,7 @@ CaretFileDialog::getChooseFileNameDialog(const DataFileTypeEnum::Enum dataFileTy
         if (selectedFiles.size() > 0) {
             AString filename = DataFileTypeEnum::addFileExtensionIfMissing(selectedFiles[0],
                                                                            dataFileType);
-            return filename;
+            return std::move(filename);
         }
     }
     return QString();
@@ -445,7 +565,8 @@ CaretFileDialog::getExistingDirectoryDialog(QWidget *parent,
                                           const QString &dir,
                                           Options options)
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_SAVE,
+                        parent,
                         caption,
                         dir,
                         "");
@@ -492,12 +613,13 @@ CaretFileDialog::getOpenFileNamesDialog(QWidget *parent,
                                           QString *selectedFilter,
                                           Options options)
 {
-    CaretFileDialog cfd(parent,
+    CaretFileDialog cfd(Mode::MODE_OPEN,
+                        parent,
                         caption,
                         dir,
                         filter);
     if (selectedFilter != 0) {
-        cfd.selectFilter(*selectedFilter);
+        cfd.selectNameFilter(*selectedFilter);
     }
     cfd.setOptions(options);
     cfd.setAcceptMode(CaretFileDialog::AcceptOpen);
@@ -514,4 +636,61 @@ CaretFileDialog::getOpenFileNamesDialog(QWidget *parent,
     return sl;
 }
 
+/**
+ * Save the dialog settings (directory, filename filter, and geometry)
+ * for next time dialog displayed.
+ *
+ * @param settingsName
+ *     Name for settings.
+ */
+void
+CaretFileDialog::saveDialogSettings(const AString& settingsName)
+{
+    PreviousDialogSettings previousSettings(selectedNameFilter(),
+                                            directory().absolutePath(),
+                                            saveGeometry());
+    
+    std::map<AString, PreviousDialogSettings>::iterator iter = s_previousDialogSettingsMap.find(settingsName);
+    if (iter != s_previousDialogSettingsMap.end()) {
+        iter->second = previousSettings;
+    }
+    else {
+        s_previousDialogSettingsMap.insert(std::make_pair(settingsName,
+                                                          previousSettings));
+    }
+}
+
+/**
+ * Restore the dialog settings (directory, filename filter, and geometry)
+ * for next time dialog displayed.
+ *
+ * @param settingsName
+ *     Name for settings used when settings were saved.
+ */
+void
+CaretFileDialog::restoreDialogSettings(const AString& settingsName)
+{
+    std::map<AString, PreviousDialogSettings>::iterator iter = s_previousDialogSettingsMap.find(settingsName);
+    if (iter != s_previousDialogSettingsMap.end()) {
+        const PreviousDialogSettings previousSettings = iter->second;
+        
+        FileInformation fileInfo(previousSettings.m_directoryName);
+        if (fileInfo.exists()) {
+            setDirectory(previousSettings.m_directoryName);
+        }
+        
+        QStringList dialogFilters = nameFilters();
+        QStringListIterator filterIter(dialogFilters);
+        while (filterIter.hasNext()) {
+            const AString filterName = filterIter.next();
+            if (filterName == previousSettings.m_fileFilterName) {
+                selectNameFilter(previousSettings.m_fileFilterName);
+                break;
+            }
+        }
+        
+        restoreGeometry(previousSettings.m_dialogGeometry);
+    }
+}
+
 
