@@ -45,9 +45,10 @@
 #include "ChartModelTimeSeries.h"
 #include "EventManager.h"
 #include "EventUserInterfaceUpdate.h"
-#include "EventGraphicsUpdateOneWindow.h"
+#include "EventGraphicsPaintSoonOneWindow.h"
 #include "GuiManager.h"
 #include "ModelChart.h"
+#include "WuQMacroManager.h"
 #include "WuQtUtilities.h"
 
 using namespace caret;
@@ -65,11 +66,15 @@ using namespace caret;
  */
 ChartHistoryViewController::ChartHistoryViewController(const Qt::Orientation orientation,
                                                        const int32_t browserWindowIndex,
+                                                       const QString& parentObjectName,
                                                        QWidget* parent)
 : QWidget(parent),
 m_orientation(orientation),
-m_browserWindowIndex(browserWindowIndex)
+m_browserWindowIndex(browserWindowIndex),
+m_objectNamePrefix(parentObjectName
+                   + ":History")
 {
+    WuQMacroManager* macroManager = WuQMacroManager::instance();
     m_averageCheckBox = new QCheckBox("Show Average");
     WuQtUtilities::setWordWrappedToolTip(m_averageCheckBox,
                                          "Display an average of the displayed chart data.   "
@@ -79,6 +84,10 @@ m_browserWindowIndex(browserWindowIndex)
                                          "chart.");
     QObject::connect(m_averageCheckBox, SIGNAL(clicked(bool)),
                      this, SLOT(averageCheckBoxClicked(bool)));
+    m_averageCheckBox->setObjectName(m_objectNamePrefix
+                                     + ":ShowAverage");
+    macroManager->addMacroSupportToObject(m_averageCheckBox,
+                                          "Enable linew chart history average");
     
     QPushButton* clearPushButton = new QPushButton("Clear");
     clearPushButton->setFixedWidth(clearPushButton->sizeHint().width() + 20);
@@ -86,6 +95,11 @@ m_browserWindowIndex(browserWindowIndex)
                      this, SLOT(clearPushButtonClicked()));
     WuQtUtilities::setWordWrappedToolTip(clearPushButton,
                                          "Remove all charts of the selected type in this tab");
+    clearPushButton->setObjectName(m_objectNamePrefix
+                                   + ":ClearButton");
+    macroManager->addMacroSupportToObject(clearPushButton,
+                                          "Remove line charts in tab");
+    
     
     QLabel* maximumDisplayedLabel = new QLabel("Show last ");
     m_maximumDisplayedSpinBox = new QSpinBox(); 
@@ -93,6 +107,12 @@ m_browserWindowIndex(browserWindowIndex)
     m_maximumDisplayedSpinBox->setMaximum(1000);
     QObject::connect(m_maximumDisplayedSpinBox, SIGNAL(valueChanged(int)),
                      this, SLOT(maximumDisplayedSpinBoxValueChanged(int)));
+    m_maximumDisplayedSpinBox->setToolTip("Show Last Lines");
+    m_maximumDisplayedSpinBox->setObjectName(m_objectNamePrefix
+                                             + ":ShowLastCount");
+    macroManager->addMacroSupportToObject(m_maximumDisplayedSpinBox,
+                                          "Set maximum line charts");
+    
     QHBoxLayout* maxDisplayedLayout = new QHBoxLayout();
     maxDisplayedLayout->addWidget(maximumDisplayedLabel);
     maxDisplayedLayout->addWidget(m_maximumDisplayedSpinBox);
@@ -102,16 +122,31 @@ m_browserWindowIndex(browserWindowIndex)
                                          "displayed in this tab");
     
     m_chartDataCheckBoxesSignalMapper = new QSignalMapper(this);
+#if QT_VERSION >= 0x060000
+    QObject::connect(m_chartDataCheckBoxesSignalMapper, &QSignalMapper::mappedInt,
+                     this, &ChartHistoryViewController::chartDataCheckBoxSignalMapped);
+#else
     QObject::connect(m_chartDataCheckBoxesSignalMapper, SIGNAL(mapped(int)),
                      this, SLOT(chartDataCheckBoxSignalMapped(int)));
+#endif
     
     m_chartDataColorComboBoxesSignalMapper = new QSignalMapper(this);
+#if QT_VERSION >= 0x060000
+    QObject::connect(m_chartDataColorComboBoxesSignalMapper, &QSignalMapper::mappedInt,
+                     this, &ChartHistoryViewController::chartDataColorComboBoxSignalMapped);
+#else
     QObject::connect(m_chartDataColorComboBoxesSignalMapper, SIGNAL(mapped(int)),
                      this, SLOT(chartDataColorComboBoxSignalMapped(int)));
+#endif
     
     m_chartDataColorConstructionButtonSignalMapper = new QSignalMapper(this);
+#if QT_VERSION >= 0x060000
+    QObject::connect(m_chartDataColorConstructionButtonSignalMapper, &QSignalMapper::mappedInt,
+                     this, &ChartHistoryViewController::chartDataConstructionToolButtonSignalMapped);
+#else
     QObject::connect(m_chartDataColorConstructionButtonSignalMapper, SIGNAL(mapped(int)),
                      this, SLOT(chartDataConstructionToolButtonSignalMapped(int)));
+#endif
     
     QWidget* chartDataWidget = new QWidget();
     m_chartDataGridLayout = new QGridLayout(chartDataWidget);
@@ -254,7 +289,7 @@ ChartHistoryViewController::maximumDisplayedSpinBoxValueChanged(int value)
 void
 ChartHistoryViewController::updateAfterSelectionsChanged()
 {
-    EventManager::get()->sendEvent(EventGraphicsUpdateOneWindow(m_browserWindowIndex).getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonOneWindow(m_browserWindowIndex).getPointer());
     updateHistoryViewController();
 }
 
@@ -283,6 +318,7 @@ ChartHistoryViewController::updateHistoryViewController()
             break;
     }
     
+    WuQMacroManager* macroManager = WuQMacroManager::instance();
     const std::vector<ChartData*> chartDataVector = chartModel->getAllChartDatas();
     const int32_t numData = static_cast<int32_t>(chartDataVector.size());
 
@@ -291,6 +327,8 @@ ChartHistoryViewController::updateHistoryViewController()
                                       numWidgetRows);
     for (int32_t i = 0; i < maxItems; i++) {
         if (i >= static_cast<int32_t>(m_chartDataCheckBoxes.size())) {
+            const QString numberString(QString("%1").arg((int)i+1, 2, 10, QLatin1Char('0')));
+                                       
             /*
              * Checkbox
              */
@@ -299,6 +337,13 @@ ChartHistoryViewController::updateHistoryViewController()
                              m_chartDataCheckBoxesSignalMapper, SLOT(map()));
             m_chartDataCheckBoxesSignalMapper->setMapping(checkBox, i);
             m_chartDataCheckBoxes.push_back(checkBox);
+            checkBox->setToolTip("Show Chart Line");
+            checkBox->setObjectName(m_objectNamePrefix
+                                    + ":ShowChartLine"
+                                    + numberString);
+            macroManager->addMacroSupportToObject(checkBox,
+                                                  "Show line chart " + numberString);
+            
 
             /*
              * Construction Tool Button
@@ -326,6 +371,12 @@ ChartHistoryViewController::updateHistoryViewController()
                              m_chartDataColorComboBoxesSignalMapper, SLOT(map()));
             m_chartDataColorComboBoxesSignalMapper->setMapping(colorComboBox, i);
             m_chartDataColorComboBoxes.push_back(colorComboBox);
+            colorComboBox->getWidget()->setToolTip("Select Color");
+            colorComboBox->getWidget()->setObjectName(m_objectNamePrefix
+                                                        + ":Color"
+                                                        + numberString);
+            macroManager->addMacroSupportToObject(colorComboBox->getWidget(),
+                                                  "Set line chart " + numberString + " color");
             
             /*
              * Label
@@ -438,20 +489,20 @@ ChartHistoryViewController::getSelectedChartModelAndTab(ChartModel* &chartModelO
     
     ModelChart* modelChart = brain->getChartModel();
     if (modelChart != NULL) {
-        switch (modelChart->getSelectedChartDataType(tabIndexOut)) {
-            case ChartDataTypeEnum::CHART_DATA_TYPE_INVALID:
+        switch (modelChart->getSelectedChartOneDataType(tabIndexOut)) {
+            case ChartOneDataTypeEnum::CHART_DATA_TYPE_INVALID:
                 break;
-            case ChartDataTypeEnum::CHART_DATA_TYPE_MATRIX_LAYER:
+            case ChartOneDataTypeEnum::CHART_DATA_TYPE_MATRIX_LAYER:
                 break;
-            case ChartDataTypeEnum::CHART_DATA_TYPE_MATRIX_SERIES:
+            case ChartOneDataTypeEnum::CHART_DATA_TYPE_MATRIX_SERIES:
                 break;
-            case ChartDataTypeEnum::CHART_DATA_TYPE_LINE_DATA_SERIES:
+            case ChartOneDataTypeEnum::CHART_DATA_TYPE_LINE_DATA_SERIES:
                 chartModelOut = modelChart->getSelectedDataSeriesChartModel(tabIndexOut);
                 break;
-            case ChartDataTypeEnum::CHART_DATA_TYPE_LINE_FREQUENCY_SERIES:
+            case ChartOneDataTypeEnum::CHART_DATA_TYPE_LINE_FREQUENCY_SERIES:
                 chartModelOut = modelChart->getSelectedFrequencySeriesChartModel(tabIndexOut);
                 break;
-            case ChartDataTypeEnum::CHART_DATA_TYPE_LINE_TIME_SERIES:
+            case ChartOneDataTypeEnum::CHART_DATA_TYPE_LINE_TIME_SERIES:
                 chartModelOut = modelChart->getSelectedTimeSeriesChartModel(tabIndexOut);
                 break;
         }
@@ -559,7 +610,7 @@ ChartHistoryViewController::chartDataConstructionToolButtonSignalMapped(int indx
         else if (selectedAction == removeAction) {
             chartModel->removeChartAtIndex(indx);
         }
-        else {
+        else if (selectedAction != NULL) {
             CaretAssertMessage(0, "Has a new action been added but not processed?");
         }
         
