@@ -43,10 +43,9 @@
 #include "CiftiMappableDataFile.h"
 #include "CiftiParcelLabelFile.h"
 #include "EventManager.h"
-#include "EventGraphicsUpdateAllWindows.h"
-#include "EventGraphicsUpdateOneWindow.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
+#include "EventGraphicsPaintSoonOneWindow.h"
 #include "EventPaletteColorMappingEditorDialogRequest.h"
-#include "EventSurfaceColoringInvalidate.h"
 #include "EventUserInterfaceUpdate.h"
 #include "GuiManager.h"
 #include "MapYokingGroupComboBox.h"
@@ -98,13 +97,22 @@ m_browserWindowIndex(browserWindowIndex)
                                          Qt::AlignHCenter);
     
     m_signalMapperBrainordinateFileEnableCheckBox = new QSignalMapper(this);
+#if QT_VERSION >= 0x060000
+    QObject::connect(m_signalMapperBrainordinateFileEnableCheckBox, &QSignalMapper::mappedInt,
+                     this, &ChartLinesSelectionViewController::brainordinateSelectionCheckBoxClicked);
+#else
     QObject::connect(m_signalMapperBrainordinateFileEnableCheckBox, SIGNAL(mapped(int)),
                      this, SLOT(brainordinateSelectionCheckBoxClicked(int)));
+#endif
     
     m_signalMapperBrainordinateYokingComboBox = new QSignalMapper(this);
+#if QT_VERSION >= 0x060000
+    QObject::connect(m_signalMapperBrainordinateYokingComboBox, &QSignalMapper::mappedInt,
+                     this, &ChartLinesSelectionViewController::brainordinateYokingComboBoxActivated);
+#else
     QObject::connect(m_signalMapperBrainordinateYokingComboBox, SIGNAL(mapped(int)),
                      this, SLOT(brainordinateYokingComboBoxActivated(int)));
-    
+#endif
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_USER_INTERFACE_UPDATE);
     
     QVBoxLayout* layout = new QVBoxLayout(this);
@@ -189,9 +197,6 @@ ChartLinesSelectionViewController::updateSelectionViewController()
                 yokeComboBox = new MapYokingGroupComboBox(this);
                 yokeComboBox->getWidget()->setStatusTip("Synchronize enabled status and map indices)");
                 yokeComboBox->getWidget()->setToolTip("Yoke to Overlay Mapped Files");
-#ifdef CARET_OS_MACOSX
-                yokeComboBox->getWidget()->setFixedWidth(yokeComboBox->getWidget()->sizeHint().width() - 20);
-#endif // CARET_OS_MACOSX
                 QObject::connect(yokeComboBox, SIGNAL(itemActivated()),
                                  m_signalMapperBrainordinateYokingComboBox, SLOT(map()));
                 m_signalMapperBrainordinateYokingComboBox->setMapping(yokeComboBox, i);
@@ -225,7 +230,7 @@ ChartLinesSelectionViewController::updateSelectionViewController()
             
             const bool checkBoxStatus = chartBrainFile->isLineSeriesChartingEnabled(browserTabIndex);
             
-            QVariant brainordinateFilePointerVariant = qVariantFromValue((void*)chartBrainFile);
+            QVariant brainordinateFilePointerVariant = QVariant::fromValue((void*)chartBrainFile);
             
             CaretMappableDataFile* caretMappableDataFile = chartBrainFile->getLineSeriesChartCaretMappableDataFile();
             
