@@ -57,10 +57,9 @@
 #include "EnumComboBoxTemplate.h"
 #include "EventChartMatrixParcelYokingValidation.h"
 #include "EventManager.h"
-#include "EventGraphicsUpdateAllWindows.h"
-#include "EventGraphicsUpdateOneWindow.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
+#include "EventGraphicsPaintSoonOneWindow.h"
 #include "EventPaletteColorMappingEditorDialogRequest.h"
-#include "EventSurfaceColoringInvalidate.h"
 #include "EventUserInterfaceUpdate.h"
 #include "GuiManager.h"
 #include "MapYokingGroupComboBox.h"
@@ -136,9 +135,6 @@ m_browserWindowIndex(browserWindowIndex)
     m_matrixSeriesYokingComboBox = new MapYokingGroupComboBox(this);
     m_matrixSeriesYokingComboBox->getWidget()->setStatusTip("Synchronize enabled status and map indices)");
     m_matrixSeriesYokingComboBox->getWidget()->setToolTip("Yoke to Overlay Mapped Files");
-#ifdef CARET_OS_MACOSX
-    m_matrixSeriesYokingComboBox->getWidget()->setFixedWidth(m_matrixSeriesYokingComboBox->getWidget()->sizeHint().width() - 20);
-#endif // CARET_OS_MACOSX
     QObject::connect(m_matrixSeriesYokingComboBox, SIGNAL(itemActivated()),
                      this, SLOT(matrixSeriesYokingGroupActivated()));
 
@@ -411,7 +407,7 @@ void
 ChartMatrixSeriesSelectionViewController::matrixSeriesFileSelected(CaretDataFile* /*caretDataFile*/)
 {
     updateSelectionViewController();
-    EventManager::get()->sendEvent(EventGraphicsUpdateOneWindow(m_browserWindowIndex).getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonOneWindow(m_browserWindowIndex).getPointer());
 }
 
 /**
@@ -437,7 +433,7 @@ ChartMatrixSeriesSelectionViewController::matrixSeriesColorBarActionTriggered(bo
     
     if (chartableMatrixSeriesInterface != NULL) {
         chartMatrixDisplayProperties->getColorBar()->setDisplayed(status);
-        EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+        EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     }
 }
 
@@ -495,6 +491,6 @@ ChartMatrixSeriesSelectionViewController::matrixSeriesYokingGroupActivated()
     
     m_matrixSeriesYokingComboBox->validateYokingChange(chartableMatrixSeriesInterface,
                                                        browserTabIndex);
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
 }
 
