@@ -37,7 +37,7 @@
 #include "DisplayGroupEnumComboBox.h"
 #include "DisplayPropertiesFiberOrientation.h"
 #include "EventManager.h"
-#include "EventGraphicsUpdateAllWindows.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
 #include "EventUserInterfaceUpdate.h"
 #include "CiftiFiberOrientationFile.h"
 #include "FiberOrientationColoringTypeEnum.h"
@@ -88,6 +88,11 @@ FiberOrientationSelectionViewController::FiberOrientationSelectionViewController
     QObject::connect(m_displayFibersCheckBox, SIGNAL(clicked(bool)),
                      this, SLOT(processSelectionChanges()));
     
+    m_drawFiberTrajectoriesInFrontCheckBox = new QCheckBox("Draw Fiber Trajectories in Front");
+    m_drawFiberTrajectoriesInFrontCheckBox->setToolTip("Draw last so trajectories are in front of everything else");
+    QObject::connect(m_drawFiberTrajectoriesInFrontCheckBox, &QCheckBox::clicked,
+                     this, &FiberOrientationSelectionViewController::processSelectionChanges);
+
     m_tabWidget = new WuQTabWidget(WuQTabWidget::TAB_ALIGN_LEFT,
                                                this);
     m_tabWidget->addTab(attributesWidget,
@@ -103,6 +108,7 @@ FiberOrientationSelectionViewController::FiberOrientationSelectionViewController
     layout->addLayout(groupLayout);
     layout->addSpacing(10);
     layout->addWidget(m_displayFibersCheckBox);
+    layout->addWidget(m_drawFiberTrajectoriesInFrontCheckBox);
     layout->addWidget(m_tabWidget->getWidget(), 0, Qt::AlignLeft);
     layout->addStretch();
     
@@ -172,6 +178,15 @@ FiberOrientationSelectionViewController::createAttributesWidget()
     QObject::connect(m_minimumMagnitudeSpinBox, SIGNAL(valueChanged(double)),
                      this, SLOT(processAttributesChanges()));
     
+    QLabel* maximumUncertaintyLabel = new QLabel("Maximum Uncertainty");
+    m_maximumUncertaintySpinBox = WuQFactory::newDoubleSpinBox();
+    m_maximumUncertaintySpinBox->setRange(0.0, std::numeric_limits<float>::max());
+    m_maximumUncertaintySpinBox->setDecimals(3);
+    m_maximumUncertaintySpinBox->setSingleStep(0.001);
+    m_maximumUncertaintySpinBox->setToolTip("Maximum uncertainty for displaying fibers");
+    QObject::connect(m_maximumUncertaintySpinBox, SIGNAL(valueChanged(double)),
+                     this, SLOT(processAttributesChanges()));
+
     QLabel* lengthMultiplierLabel = new QLabel("Length Multiplier");
     m_lengthMultiplierSpinBox = WuQFactory::newDoubleSpinBox();
     m_lengthMultiplierSpinBox->setRange(0.0, std::numeric_limits<float>::max());
@@ -229,6 +244,9 @@ FiberOrientationSelectionViewController::createAttributesWidget()
     gridLayout->addWidget(minimumMagnitudeLabel, row, 0);
     gridLayout->addWidget(m_minimumMagnitudeSpinBox , row, 1);
     row++;
+    gridLayout->addWidget(maximumUncertaintyLabel, row, 0);
+    gridLayout->addWidget(m_maximumUncertaintySpinBox, row, 1);
+    row++;
     gridLayout->addWidget(lengthMultiplierLabel, row, 0);
     gridLayout->addWidget(m_lengthMultiplierSpinBox , row, 1);
     row++;
@@ -291,6 +309,10 @@ FiberOrientationSelectionViewController::processAttributesChanges()
                         browserTabIndex,
                         m_minimumMagnitudeSpinBox->value());
     
+    dpfo->setMaximumUncertainty(displayGroup,
+                                browserTabIndex,
+                                m_maximumUncertaintySpinBox->value());
+    
     dpfo->setLengthMultiplier(displayGroup,
                         browserTabIndex,
                         m_lengthMultiplierSpinBox->value());
@@ -317,7 +339,7 @@ FiberOrientationSelectionViewController::processAttributesChanges()
                                          browserTabIndex,
                                          m_displaySphereOrientationsCheckBox->isChecked());
     
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     
     updateOtherViewControllers();
 }
@@ -418,6 +440,8 @@ FiberOrientationSelectionViewController::updateViewController()
      */
     m_displayFibersCheckBox->setChecked(dpfo->isDisplayed(displayGroup,
                                                         browserTabIndex));
+    m_drawFiberTrajectoriesInFrontCheckBox->setChecked(dpfo->isDrawFiberTrajectoriesInFront(displayGroup,
+                                                                                            browserTabIndex));
     m_aboveLimitSpinBox->setValue(dpfo->getAboveLimit(displayGroup,
                                                       browserTabIndex));
     m_belowLimitSpinBox->setValue(dpfo->getBelowLimit(displayGroup,
@@ -428,6 +452,8 @@ FiberOrientationSelectionViewController::updateViewController()
                                                             browserTabIndex));
     m_minimumMagnitudeSpinBox->setValue(dpfo->getMinimumMagnitude(displayGroup,
                                                       browserTabIndex));
+    m_maximumUncertaintySpinBox->setValue(dpfo->getMaximumUncertainty(displayGroup,
+                                                                      browserTabIndex));
     m_drawWithMagnitudeCheckBox->setChecked(dpfo->isDrawWithMagnitude(displayGroup,
                                                                      browserTabIndex));
     m_coloringTypeComboBox->setSelectedItem<FiberOrientationColoringTypeEnum, FiberOrientationColoringTypeEnum::Enum>(dpfo->getColoringType(displayGroup,
@@ -470,6 +496,9 @@ FiberOrientationSelectionViewController::processSelectionChanges()
     dpfo->setDisplayed(displayGroup,
                        browserTabIndex,
                        m_displayFibersCheckBox->isChecked());
+    dpfo->setDrawFiberTrajectoriesInFront(displayGroup,
+                                          browserTabIndex,
+                                          m_drawFiberTrajectoriesInFrontCheckBox->isChecked());
     
     const int32_t numberOfFiberOrientFiles = brain->getNumberOfConnectivityFiberOrientationFiles();
     CaretAssert(numberOfFiberOrientFiles <= static_cast<int32_t>(m_fileSelectionCheckBoxes.size()));
@@ -481,7 +510,7 @@ FiberOrientationSelectionViewController::processSelectionChanges()
     }
     
     updateOtherViewControllers();
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
 }
 
 /**
