@@ -43,7 +43,7 @@
 #include "CaretAssert.h"
 #include "CaretMappableDataFile.h"
 #include "ColorEditorWidget.h"
-#include "EventGraphicsUpdateAllWindows.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
 #include "EventManager.h"
 #include "EventSurfaceColoringInvalidate.h"
 #include "EventUserInterfaceUpdate.h"
@@ -642,7 +642,7 @@ GiftiLabelTableEditor::loadLabels(const AString& selectedNameIn,
         setWidgetItemIconColor(colorItem, rgba);
         
         colorItem->setData(Qt::UserRole, 
-                           qVariantFromValue((void*)gl));
+                           QVariant::fromValue((void*)gl));
         m_labelSelectionListWidget->addItem(colorItem);
         
         if (selectedName == gl->getName()) {
@@ -882,12 +882,10 @@ void
 GiftiLabelTableEditor::processApplyButton()
 {
     if (m_caretMappableDataFile != NULL) {
-        const PaletteFile* paletteFile = GuiManager::get()->getBrain()->getPaletteFile();
-        m_caretMappableDataFile->updateScalarColoringForMap(m_caretMappableDataFileMapIndex,
-                                                            paletteFile);
+        m_caretMappableDataFile->updateScalarColoringForMap(m_caretMappableDataFileMapIndex);
     }
     EventManager::get()->sendEvent(EventSurfaceColoringInvalidate().getPointer());
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     EventManager::get()->sendEvent(EventUserInterfaceUpdate().getPointer());
 }
 
