@@ -22,6 +22,7 @@
 /*LICENSE_END*/
 
 #include <map>
+#include <memory>
 
 #include "DataFileTypeEnum.h"
 #include "EventListenerInterface.h"
@@ -30,6 +31,8 @@
 class QLabel;
 class QRadioButton;
 class QSpinBox;
+class QStackedWidget;
+class QTextEdit;
 
 namespace caret {
 
@@ -41,8 +44,8 @@ namespace caret {
     class CiftiParcelSelectionComboBox;
     class GiftiLabelTableSelectionComboBox;
     class StructureEnumComboBox;
-    class WuQGroupBoxExclusiveWidget;
-    
+    class WuQDoubleSpinBox;
+
     class IdentifyBrainordinateDialog : public WuQDialogNonModal, public EventListenerInterface {
         
         Q_OBJECT
@@ -73,7 +76,7 @@ namespace caret {
         
         void slotLabelFileOrMapSelectionChanged();
         
-        void selectedWidgetChanged();
+        void idTypeRadioButtonClicked(QAbstractButton* button);
         
     private:
         enum Mode {
@@ -97,19 +100,27 @@ namespace caret {
         
         QWidget* createLabelFilesWidget(const std::vector<DataFileTypeEnum::Enum>& supportedFileTypes);
         
-        QWidget* createSurfaceVertexlWidget();
+        QWidget* createSurfaceVertexWidget();
+        
+        QWidget* createImagePixelWidget(const std::vector<DataFileTypeEnum::Enum>& supportedFileTypes);
+        
+        QWidget* createStereotaxicWidget();
         
         void processCiftiParcelWidget(AString& errorMessageOut);
         
         void processCiftiRowWidget(AString& errorMessageOut);
         
+        void processImagePixelSelection(AString& errorMessage);
+        
         void processLabelFileWidget(AString& errorMessageOut);
         
+        void processStereotaxicWidget(AString& errorMessageOut);
+        
         void processSurfaceVertexWidget(AString& errorMessageOut);
         
         void flashBrainordinateHighlightingRegionOfInterest(BrainordinateRegionOfInterest* brainROI);
         
-        void updateColoringAndDrawAllWindows();
+        void updateColoringAndDrawAllWindows(const bool doRepaintFlag = false);
 
         ParcelSourceDimension getParcelSourceDimensionFromFile(const CaretMappableDataFile* mapFile);
         
@@ -117,6 +128,8 @@ namespace caret {
         
         QWidget* m_surfaceVertexWidget;
         
+        QWidget* m_stereotaxicWidget;
+        
         QLabel* m_vertexStructureLabel;
         
         QSpinBox* m_vertexIndexSpinBox;
@@ -153,7 +166,7 @@ namespace caret {
         
         CaretDataFileSelectionComboBox* m_ciftiRowFileComboBox;
 
-        CaretDataFileSelectionModel* m_ciftiRowFileSelectionModel;
+        std::unique_ptr<CaretDataFileSelectionModel> m_ciftiRowFileSelectionModel;
         
         QLabel* m_ciftiRowFileIndexLabel;
         
@@ -177,9 +190,36 @@ namespace caret {
         
         CaretMappableDataFileAndMapSelectorObject* m_ciftiParcelFileSelector;
         
-        WuQGroupBoxExclusiveWidget* m_widgetBox;
+        QWidget* m_imagePixelWidget;
+        
+        std::unique_ptr<CaretDataFileSelectionModel> m_imageFileSelectionModel;
+        
+        CaretDataFileSelectionComboBox* m_imageFileSelectionComboBox;
+        
+        QSpinBox* m_imagePixelISpinBox;
+        
+        QSpinBox* m_imagePixelJSpinBox;
+        
+        QTextEdit* m_stereotaxicTextEdit;
+        
+        WuQDoubleSpinBox* m_stereotaxicXWidget;
+        
+        WuQDoubleSpinBox* m_stereotaxicYWidget;
+        
+        WuQDoubleSpinBox* m_stereotaxicZWidget;
+        
+        QStackedWidget* m_stackedWidget;
+        
+        QRadioButton* m_ciftiFileRowRadioButton;
+        QRadioButton* m_ciftiFileParcelRadioButton;
+        QRadioButton* m_imagePixelRadioButton;
+        QRadioButton* m_labelRadioButton;
+        QRadioButton* m_stereotaxicRadioButton;
+        QRadioButton* m_surfaceVertexRadioButton;
         
         std::map<DataFileTypeEnum::Enum, ParcelSourceDimension> m_parcelSourceDimensionMap;
+        std::vector<DataFileTypeEnum::Enum> m_supportedCiftiRowFileTypes;
+        std::vector<DataFileTypeEnum::Enum> m_supportedLabelFileTypes;
     };
     
 #ifdef __IDENTIFY_BRAINORDINATE_DIALOG_DECLARE__
