@@ -48,13 +48,13 @@
 #include "DataFileException.h"
 #include "EnumComboBoxTemplate.h"
 #include "EventBrowserWindowGraphicsRedrawn.h"
-#include "EventGraphicsUpdateOneWindow.h"
+#include "EventGraphicsPaintSoonOneWindow.h"
 #include "EventImageCapture.h"
 #include "EventManager.h"
 #include "FileInformation.h"
 #include "GuiManager.h"
 #include "ImageFile.h"
-#include "ImageCaptureSettings.h"
+#include "ImageCaptureDialogSettings.h"
 #include "SessionManager.h"
 #include "CaretFileDialog.h"
 #include "WuQFactory.h"
@@ -64,8 +64,6 @@
 
 using namespace caret;
 
-
-    
 /**
  * \class caret::ImageCaptureDialog 
  * \brief Dialog for capturing images.
@@ -189,28 +187,57 @@ ImageCaptureDialog::createImageSourceSection()
 QWidget*
 ImageCaptureDialog::createImageOptionsSection()
 {
+    const AString cropToolTip("Using the Window's background color, crop the image to "
+                              "the content of the graphics region");
     m_imageAutoCropCheckBox = new QCheckBox("Automatically Crop Image");
     QObject::connect(m_imageAutoCropCheckBox, SIGNAL(clicked(bool)),
                      this, SLOT(imageCroppingCheckBoxClicked(bool)));
-    QLabel* imageAutoCropMarginLabel = new QLabel("   Margin");
-    m_imageAutoCropMarginSpinBox = new QSpinBox();
-    QObject::connect(m_imageAutoCropMarginSpinBox, SIGNAL(valueChanged(int)),
-                     this, SLOT(imageCroppingMarginValueChanged(int)));
-    m_imageAutoCropMarginSpinBox->setMinimum(0);
-    m_imageAutoCropMarginSpinBox->setMaximum(100000);
-    m_imageAutoCropMarginSpinBox->setSingleStep(1);
-    m_imageAutoCropMarginSpinBox->setMaximumWidth(100);
+    m_imageAutoCropCheckBox->setToolTip(cropToolTip);
+
+    const AString marginToolTip("Add a margin in the window's background color "
+                                "around the captured region.");
+    QLabel* imageMarginLabel = new QLabel("Margin");
+    m_imageMarginSpinBox = new QSpinBox();
+    QObject::connect(m_imageMarginSpinBox, SIGNAL(valueChanged(int)),
+                     this, SLOT(imageMarginValueChanged(int)));
+    m_imageMarginSpinBox->setMinimum(0);
+    m_imageMarginSpinBox->setMaximum(100);
+    m_imageMarginSpinBox->setSingleStep(1);
+    m_imageMarginSpinBox->setMaximumWidth(100);
     
     QHBoxLayout* cropMarginLayout = new QHBoxLayout();
-    cropMarginLayout->addWidget(imageAutoCropMarginLabel);
-    cropMarginLayout->addWidget(m_imageAutoCropMarginSpinBox);
+    cropMarginLayout->addWidget(imageMarginLabel);
+    cropMarginLayout->addWidget(m_imageMarginSpinBox);
     cropMarginLayout->addStretch();
     
-    QGroupBox* groupBox = new QGroupBox("Image Options");
-    QVBoxLayout* layout = new QVBoxLayout(groupBox);
-    layout->addWidget(m_imageAutoCropCheckBox);
-    layout->addLayout(cropMarginLayout);
+    m_backgroundAlphaCheckBox = new QCheckBox("Background Alpha");
+    QObject::connect(m_backgroundAlphaCheckBox, &QCheckBox::clicked,
+                     this, &ImageCaptureDialog::backgroundAlphaCheckBoxClicked);
     
+    m_backgroundAlphaSpinBox  = new QSpinBox();
+    m_backgroundAlphaSpinBox->setRange(0, 255);
+    m_backgroundAlphaSpinBox->setSingleStep(1);
+    QObject::connect(m_backgroundAlphaSpinBox, static_cast<void(QSpinBox::*)(int)>(&QSpinBox::valueChanged),
+                     this, &ImageCaptureDialog::backgroundAlphaSpinBoxValueChanged);
+    
+    /*
+     * Disable until implemented
+     */
+    m_backgroundAlphaCheckBox->setEnabled(false);
+    m_backgroundAlphaSpinBox->setEnabled(false);
+    
+    QGroupBox* groupBox = new QGroupBox("Image Options");
+    QGridLayout* groupLayout = new QGridLayout(groupBox);
+    groupLayout->setColumnStretch(0, 0);
+    groupLayout->setColumnStretch(1, 100);
+    int groupRow(0);
+    groupLayout->addWidget(m_imageAutoCropCheckBox, groupRow, 0);
+    groupLayout->addWidget(m_imageMarginSpinBox, groupRow, 1, Qt::AlignLeft);
+    ++groupRow;
+    groupLayout->addWidget(m_backgroundAlphaCheckBox, groupRow, 0);
+    groupLayout->addWidget(m_backgroundAlphaSpinBox, groupRow, 1, Qt::AlignLeft);
+    ++groupRow;
+
     return groupBox;
 }
 
@@ -277,8 +304,10 @@ ImageCaptureDialog::createImageDimensionsSection()
     QObject::connect(m_imageResolutionSpinBox, SIGNAL(valueChanged(double)),
                      this, SLOT(imageResolutionValueChanged(double)));
 
+    std::vector<ImageSpatialUnitsEnum::Enum> spatialEnums;
+    ImageSpatialUnitsEnum::getAllEnumsExcludingPixels(spatialEnums);
     m_imageSpatialUnitsEnumComboBox = new EnumComboBoxTemplate(this);
-    m_imageSpatialUnitsEnumComboBox->setup<ImageSpatialUnitsEnum,ImageSpatialUnitsEnum::Enum>();
+    m_imageSpatialUnitsEnumComboBox->setupWithItems<ImageSpatialUnitsEnum,ImageSpatialUnitsEnum::Enum>(spatialEnums);
     QObject::connect(m_imageSpatialUnitsEnumComboBox, SIGNAL(itemActivated()),
                      this, SLOT(imageSizeUnitsEnumComboBoxItemActivated()));
     
@@ -392,7 +421,7 @@ ImageCaptureDialog::createImageDimensionsSection()
 void
 ImageCaptureDialog::sizeRadioButtonClicked(QAbstractButton* /*button*/)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     if (m_imageSizeCustomRadioButton->isChecked()) {
         imageCaptureSettings->setImageCaptureDimensionsMode(ImageCaptureDimensionsModeEnum::IMAGE_CAPTURE_DIMENSIONS_MODE_CUSTOM);
     }
@@ -409,7 +438,7 @@ ImageCaptureDialog::sizeRadioButtonClicked(QAbstractButton* /*button*/)
 void
 ImageCaptureDialog::imageResolutionUnitsEnumComboBoxItemActivated()
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setImageResolutionUnits(m_imagePixelsPerSpatialUnitsEnumComboBox->getSelectedItem<ImageResolutionUnitsEnum, ImageResolutionUnitsEnum::Enum>());
     updateDimensionsSection();
 }
@@ -420,7 +449,7 @@ ImageCaptureDialog::imageResolutionUnitsEnumComboBoxItemActivated()
 void
 ImageCaptureDialog::imageSizeUnitsEnumComboBoxItemActivated()
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setSpatialUnits(m_imageSpatialUnitsEnumComboBox->getSelectedItem<ImageSpatialUnitsEnum, ImageSpatialUnitsEnum::Enum>());
     updateDimensionsSection();
 }
@@ -434,7 +463,7 @@ ImageCaptureDialog::imageSizeUnitsEnumComboBoxItemActivated()
 void
 ImageCaptureDialog::cropToTabWindowLockAspectRegionClicked(bool checked)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setCropToTabWindowLockAspectRegionEnabled(checked);
     updateBrowserWindowWidthAndHeightLabel();
 }
@@ -446,7 +475,7 @@ ImageCaptureDialog::cropToTabWindowLockAspectRegionClicked(bool checked)
 void
 ImageCaptureDialog::updateDialog()
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     m_windowCropToLockAspectRegionCheckBox->setChecked(imageCaptureSettings->isCropToTabWindowLockAspectRegionEnabled());
     
     updateBrowserWindowWidthAndHeightLabel();
@@ -462,7 +491,7 @@ ImageCaptureDialog::updateDialog()
 void
 ImageCaptureDialog::updateDimensionsSection()
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     
     CaretAssert(imageCaptureSettings);
     
@@ -543,13 +572,16 @@ ImageCaptureDialog::updateDimensionsSection()
 void
 ImageCaptureDialog::updateImageOptionsSection()
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     CaretAssert(imageCaptureSettings);
     
     m_imageAutoCropCheckBox->setChecked(imageCaptureSettings->isCroppingEnabled());
-    m_imageAutoCropMarginSpinBox->blockSignals(true);
-    m_imageAutoCropMarginSpinBox->setValue(imageCaptureSettings->getCroppingMargin());
-    m_imageAutoCropMarginSpinBox->blockSignals(false);
+    QSignalBlocker marginSpinBoxLocker(m_imageMarginSpinBox);
+    m_imageMarginSpinBox->setValue(imageCaptureSettings->getMargin());
+    
+    m_backgroundAlphaCheckBox->setChecked(imageCaptureSettings->isBackgroundAlphaEnabled());
+    QSignalBlocker backgroundAlphaSpinBoxBlocker(m_backgroundAlphaSpinBox);
+    m_backgroundAlphaSpinBox->setValue(imageCaptureSettings->getBackgroundAlpha());
 }
 
 /**
@@ -558,7 +590,7 @@ ImageCaptureDialog::updateImageOptionsSection()
 void
 ImageCaptureDialog::updateDestinationSection()
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     CaretAssert(imageCaptureSettings);
     
     m_copyImageToClipboardCheckBox->setChecked(imageCaptureSettings->isCopyToClipboardEnabled());
@@ -608,7 +640,7 @@ ImageCaptureDialog::updateImageNumberOfBytesLabel()
 void
 ImageCaptureDialog::updateBrowserWindowWidthAndHeightLabel()
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     AString windowSizeText = "Size of Window";
     
     int32_t width;
@@ -653,7 +685,7 @@ ImageCaptureDialog::updateBrowserWindowWidthAndHeightLabel()
 void
 ImageCaptureDialog::pixelWidthValueChanged(int value)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setPixelWidth(value);
     updateDimensionsSection();
 }
@@ -667,7 +699,7 @@ ImageCaptureDialog::pixelWidthValueChanged(int value)
 void
 ImageCaptureDialog::pixelHeightValueChanged(int value)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setPixelHeight(value);
     updateDimensionsSection();
 }
@@ -681,7 +713,7 @@ ImageCaptureDialog::pixelHeightValueChanged(int value)
 void
 ImageCaptureDialog::imageWidthValueChanged(double value)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setSpatialWidth(value);
 
     updateDimensionsSection();
@@ -696,7 +728,7 @@ ImageCaptureDialog::imageWidthValueChanged(double value)
 void
 ImageCaptureDialog::imageHeightValueChanged(double value)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setSpatialHeight(value);
     
     updateDimensionsSection();
@@ -711,7 +743,7 @@ ImageCaptureDialog::imageHeightValueChanged(double value)
 void
 ImageCaptureDialog::imageResolutionValueChanged(double value)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setImageResolutionInSelectedUnits(value);
     
     updateDimensionsSection();
@@ -727,7 +759,7 @@ ImageCaptureDialog::imageResolutionValueChanged(double value)
 void
 ImageCaptureDialog::scaleProportionallyCheckBoxClicked(bool checked)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setScaleProportionately(checked);
     
     if (checked) {
@@ -754,10 +786,10 @@ ImageCaptureDialog::scaleProportionallyCheckBoxClicked(bool checked)
  *     New value for cropping margin.
  */
 void
-ImageCaptureDialog::imageCroppingMarginValueChanged(int value)
+ImageCaptureDialog::imageMarginValueChanged(int value)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
-    imageCaptureSettings->setCroppingMargin(value);
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    imageCaptureSettings->setMargin(value);
 }
 
 /**
@@ -769,10 +801,37 @@ ImageCaptureDialog::imageCroppingMarginValueChanged(int value)
 void
 ImageCaptureDialog::imageCroppingCheckBoxClicked(bool checked)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setCroppingEnabled(checked);
 }
 
+
+/**
+ * Called when background alpha check box clicked.
+ *
+ * @parm checked
+ *     New checked status.
+ */
+void
+ImageCaptureDialog::backgroundAlphaCheckBoxClicked(bool checked)
+{
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    imageCaptureSettings->setBackgroundAlphaEnabled(checked);
+}
+
+/**
+ * Called when background alpha spin box value changed.
+ *
+ * @parm value
+ *     New alpha.
+ */
+void
+ImageCaptureDialog::backgroundAlphaSpinBoxValueChanged(int value)
+{
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    imageCaptureSettings->setBackgroundAlpha(value);
+}
+
 /**
  * Called when scale proportionately check box clicked.
  *
@@ -782,7 +841,7 @@ ImageCaptureDialog::imageCroppingCheckBoxClicked(bool checked)
 void
 ImageCaptureDialog::copyImageToClipboardCheckBoxClicked(bool checked)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setCopyToClipboardEnabled(checked);
 }
 
@@ -795,7 +854,7 @@ ImageCaptureDialog::copyImageToClipboardCheckBoxClicked(bool checked)
 void
 ImageCaptureDialog::saveImageToFileCheckBoxClicked(bool checked)
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setSaveToFileEnabled(checked);
 }
 
@@ -973,8 +1032,8 @@ ImageCaptureDialog::selectImagePushButtonPressed()
     
     std::vector<AString> imageFileFilters;
     AString defaultFileFilter;
-    ImageFile::getImageFileFilters(imageFileFilters, 
-                                   defaultFileFilter);
+    ImageFile::getSaveQFileDialogImageFilters(imageFileFilters,
+                                              defaultFileFilter);
     QString filters;
     for (std::vector<AString>::iterator filterIterator = imageFileFilters.begin();
          filterIterator != imageFileFilters.end();
@@ -988,11 +1047,13 @@ ImageCaptureDialog::selectImagePushButtonPressed()
     
     AString name = CaretFileDialog::getSaveFileNameDialog(this,
                                                   "Choose File Name",
-                                                  defaultFileName, //GuiManager::get()->getBrain()->getCurrentDirectory(),
+                                                  defaultFileName,
                                                   filters,
                                                   &defaultFileFilter);
     if (name.isEmpty() == false) {
-        ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+        name = DataFileTypeEnum::addFileExtensionIfMissing(name,
+                                                           DataFileTypeEnum::IMAGE);
+        ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
         imageCaptureSettings->setImageFileName(name);
         updateDestinationSection();
     }
@@ -1004,7 +1065,7 @@ ImageCaptureDialog::selectImagePushButtonPressed()
 void
 ImageCaptureDialog::imageFileNameValueChanged()
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     imageCaptureSettings->setImageFileName(m_imageFileNameLineEdit->text().trimmed());
 }
 
@@ -1015,7 +1076,7 @@ ImageCaptureDialog::imageFileNameValueChanged()
 void
 ImageCaptureDialog::applyButtonClicked()
 {
-    ImageCaptureSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
+    ImageCaptureDialogSettings* imageCaptureSettings = SessionManager::get()->getImageCaptureDialogSettings();
     const int browserWindowIndex = m_windowSelectionSpinBox->value() - 1;
     
     int32_t windowX;
@@ -1037,6 +1098,14 @@ ImageCaptureDialog::applyButtonClicked()
         return;
     }
     
+    AString imageFileName = m_imageFileNameLineEdit->text().trimmed();
+    if (m_saveImageToFileCheckBox->isChecked()) {
+        if (imageFileName.isEmpty()) {
+            WuQMessageBox::errorOk(this, "Save to File name is empty.  Choose an Image File.");
+            return;
+        }
+    }
+    
     /*
      * Default to width of window that may exclude empty regions
      * caused by locking of aspect ratio.
@@ -1059,6 +1128,9 @@ ImageCaptureDialog::applyButtonClicked()
         }
     }
     
+    const float pixelsPerCentimeter = imageCaptureSettings->getImageResolutionInCentimeters();
+    const float pixelsPerMeter = pixelsPerCentimeter * 100;
+    
     EventImageCapture imageCaptureEvent(browserWindowIndex,
                                         windowX,
                                         windowY,
@@ -1066,6 +1138,13 @@ ImageCaptureDialog::applyButtonClicked()
                                         windowHeight,
                                         imageWidth,
                                         imageHeight);
+    imageCaptureEvent.setOutputImageWidthAndHeight(imageWidth,
+                                                   imageHeight,
+                                                   ImageSpatialUnitsEnum::PIXELS,
+                                                   ImageResolutionUnitsEnum::PIXELS_PER_CENTIMETER,
+                                                   imageCaptureSettings->getImageResolutionInCentimeters());
+    imageCaptureEvent.setMargin(imageCaptureSettings->getMargin());
+
     EventManager::get()->sendEvent(imageCaptureEvent.getPointer());
     bool errorFlag = false;
     if (imageCaptureEvent.getEventProcessCount() <= 0) {
@@ -1083,17 +1162,14 @@ ImageCaptureDialog::applyButtonClicked()
         uint8_t backgroundColor[3];
         imageCaptureEvent.getBackgroundColor(backgroundColor);
         ImageFile imageFile;
-        imageFile.setFromQImage(imageCaptureEvent.getImage());
-        
-        const float pixelsPerCentimeter = imageCaptureSettings->getImageResolutionInCentimeters();
-        const float pixelsPerMeter = pixelsPerCentimeter * 100;
-        
+        imageFile.setFromQImage(imageCaptureEvent.getCapturedImage());
+                
         imageFile.setDotsPerMeter(pixelsPerMeter,
                                   pixelsPerMeter);
         
         
         if (imageCaptureSettings->isCroppingEnabled()) {
-            const int32_t marginSize = imageCaptureSettings->getCroppingMargin();
+            const int32_t marginSize = imageCaptureSettings->getMargin();
             imageFile.cropImageRemoveBackground(marginSize,
                                                 backgroundColor);
         }
@@ -1102,35 +1178,37 @@ ImageCaptureDialog::applyButtonClicked()
             QApplication::clipboard()->setImage(*imageFile.getAsQImage(),
                                                 QClipboard::Clipboard);
         }
-        
+
         if (m_saveImageToFileCheckBox->isChecked()) {
-            std::vector<AString> imageFileExtensions;
-            AString defaultFileExtension;
-            ImageFile::getImageFileExtensions(imageFileExtensions,
-                                              defaultFileExtension);
+            std::vector<AString> readImageFileExtensions, writeImageFileExtensions;
+            AString defaultImageExtension;
+            ImageFile::getWorkbenchSupportedImageFileExtensions(readImageFileExtensions,
+                                                                writeImageFileExtensions,
+                                                                defaultImageExtension);
             
-            AString filename = m_imageFileNameLineEdit->text().trimmed();
+            CaretAssert( ! imageFileName.isEmpty());
             
             bool validExtension = false;
-            for (std::vector<AString>::iterator extensionIterator = imageFileExtensions.begin();
-                 extensionIterator != imageFileExtensions.end();
+            for (std::vector<AString>::iterator extensionIterator = writeImageFileExtensions.begin();
+                 extensionIterator != writeImageFileExtensions.end();
                  extensionIterator++) {
-                if (filename.endsWith(*extensionIterator)) {
+                if (imageFileName.endsWith(*extensionIterator)) {
                     validExtension = true;
                 }
             }
             
-            if (validExtension == false) {
-                if (defaultFileExtension.isEmpty() == false) {
-                    filename += ("." + defaultFileExtension);
+            if ( ! validExtension) {
+                if (defaultImageExtension.isEmpty() == false) {
+                    imageFileName += ("." + defaultImageExtension);
                 }
             }
             
             try {
-                imageFile.writeFile(filename);
+                imageFile.writeFile(imageFileName);
+                SessionManager::get()->getCaretPreferences()->addToRecentFilesAndOrDirectories(imageFileName);
             }
             catch (const DataFileException& /*e*/) {
-                QString msg("Unable to save: " + filename);
+                QString msg("Unable to save: " + imageFileName);
                 WuQMessageBox::errorOk(this, msg);
                 errorFlag = true;
             }
@@ -1139,7 +1217,7 @@ ImageCaptureDialog::applyButtonClicked()
     
     ChartMatrixDisplayProperties::setManualScaleModeWindowWidthHeightScaling(1.0, 1.0);
     
-    if (errorFlag == false) {
+    if ( ! errorFlag) {
         /*
          * Display over "Capture" (the renamed Apply) button.
          */
@@ -1151,7 +1229,7 @@ ImageCaptureDialog::applyButtonClicked()
                                              "Image captured");
     }
     
-    EventManager::get()->sendEvent(EventGraphicsUpdateOneWindow(browserWindowIndex).getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonOneWindow(browserWindowIndex).getPointer());
 }
 
 
