@@ -36,7 +36,7 @@ namespace caret {
     
     class BrainBrowserWindow;
     class EnumComboBoxTemplate;
-    class ImageDimensionsModel;
+    class ImageCaptureDialogSettings;
     class WuQWidgetObjectGroup;
     
     class ImageCaptureDialog : public WuQDialogNonModal, public EventListenerInterface {
@@ -59,6 +59,8 @@ namespace caret {
     private slots:
         void selectImagePushButtonPressed();
         
+        void cropToTabWindowLockAspectRegionClicked(bool clicked);
+        
         void updateBrowserWindowWidthAndHeightLabel();
         
         void imageResolutionUnitsEnumComboBoxItemActivated();
@@ -69,6 +71,8 @@ namespace caret {
         
         void pixelHeightValueChanged(int);
         
+        void imageFileNameValueChanged();
+        
         void imageWidthValueChanged(double);
         
         void imageHeightValueChanged(double);
@@ -77,6 +81,18 @@ namespace caret {
         
         void scaleProportionallyCheckBoxClicked(bool);
         
+        void imageCroppingCheckBoxClicked(bool);
+        
+        void imageMarginValueChanged(int);
+        
+        void backgroundAlphaCheckBoxClicked(bool);
+        
+        void backgroundAlphaSpinBoxValueChanged(int);
+
+        void copyImageToClipboardCheckBoxClicked(bool);
+        
+        void saveImageToFileCheckBoxClicked(bool);
+        
         void sizeRadioButtonClicked(QAbstractButton* button);
         
     private:
@@ -89,12 +105,22 @@ namespace caret {
         QWidget* createImageDimensionsSection();
         QWidget* createImageDestinationSection();
         
+        void updateDimensionsSection();
+        void updateImageOptionsSection();
+        void updateDestinationSection();
+        
+        bool getSelectedWindowCoordsWidthAndHeight(int32_t& xOut,
+                                                   int32_t& yOut,
+                                                   int32_t& widthOut,
+                                                   int32_t& heightOut,
+                                                   int32_t& graphicsWidthOut,
+                                                   int32_t& graphicsHeightOut,
+                                                   float& aspectRatioOut) const;
+        
         bool getSelectedWindowWidthAndHeight(int32_t& widthOut,
                                              int32_t& heightOut,
                                              float& aspectRatioOut) const;
         
-        void updateDialogWithImageDimensionsModel();
-        
         void updateImageNumberOfBytesLabel();
         
         QCheckBox* m_saveImageToFileCheckBox;
@@ -117,15 +143,17 @@ namespace caret {
         QLabel* m_imageNumberOfBytesLabel;
         
         QCheckBox* m_imageAutoCropCheckBox;
-        QSpinBox*  m_imageAutoCropMarginSpinBox;
+        QSpinBox*  m_imageMarginSpinBox;
+        
+        QCheckBox* m_backgroundAlphaCheckBox;
+        QSpinBox*  m_backgroundAlphaSpinBox;
         
         QSpinBox* m_windowSelectionSpinBox;
+        QCheckBox* m_windowCropToLockAspectRegionCheckBox;
         
         QWidget* m_customDimensionsWidget;
         
         QWidget* m_imageDimensionsWidget;
-        
-        ImageDimensionsModel* m_imageDimensionsModel;
     };
     
 #ifdef __IMAGE_CAPTURE_DIALOG__H__DECLARE__
