@@ -30,7 +30,7 @@
 
 #include "Brain.h"
 #include "BrainBrowserWindow.h"
-#include "EventGraphicsUpdateAllWindows.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
 #include "EventUserInterfaceUpdate.h"
 #include "EventUpdateInformationWindows.h"
 #include "EventManager.h"
@@ -177,6 +177,10 @@ InformationDisplayDialog::updateDialog()
     const AString text = idManager->getIdentificationText();
     m_informationTextBrowser->setContentToHtml(text);
     
+    if (m_propertiesDialog != NULL) {
+        m_propertiesDialog->updateDialog();
+    }
+    
     m_contralateralIdentificationAction->blockSignals(true);
     m_contralateralIdentificationAction->setChecked(idManager->isContralateralIdentificationEnabled());
     m_contralateralIdentificationAction->blockSignals(false);
@@ -259,7 +263,7 @@ InformationDisplayDialog::clearInformationText()
     IdentificationManager* idManager = brain->getIdentificationManager();
     idManager->removeIdentificationText();
     updateDialog();
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
 }
 
 
@@ -273,7 +277,7 @@ InformationDisplayDialog::removeIdSymbols()
     IdentificationManager* idManager = brain->getIdentificationManager();
     idManager->removeAllIdentifiedSymbols();
     updateDialog();
-    EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+    EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
 }
 
 /**
@@ -318,6 +322,6 @@ InformationDisplayDialog::showPropertiesDialog()
     if (m_propertiesDialog == NULL) {
         m_propertiesDialog = new InformationDisplayPropertiesDialog(this);
     }
-    m_propertiesDialog->show();
+    m_propertiesDialog->showDialog();
 }
 
