@@ -21,13 +21,11 @@
  */
 /*LICENSE_END*/
 
-#include <stdint.h>
+#include <cstdint>
 #include <set>
 
 #include <QWidget>
 
-#include "DisplayGroupEnum.h"
-#include "EventListenerInterface.h"
 #include "SceneableInterface.h"
 
 class QCheckBox;
@@ -35,44 +33,42 @@ class QCheckBox;
 namespace caret {
 
     class GroupAndNameHierarchyViewController;
-    class DisplayGroupEnumComboBox;
     
-    class LabelSelectionViewController : public QWidget, public EventListenerInterface, public SceneableInterface {
+    class LabelSelectionViewController : public QWidget, public SceneableInterface {
         
         Q_OBJECT
 
     public:
         LabelSelectionViewController(const int32_t browserWindowIndex,
-                              QWidget* parent = 0);
+                                     const QString& parentObjectName,
+                                     QWidget* parent = 0);
         
         virtual ~LabelSelectionViewController();
         
-        void receiveEvent(Event* event);
-        
         virtual SceneClass* saveToScene(const SceneAttributes* sceneAttributes,
                                         const AString& instanceName);
         
         virtual void restoreFromScene(const SceneAttributes* sceneAttributes,
                                       const SceneClass* sceneClass);
         
+        void updateLabelViewController();
+        
     private slots:
         void processLabelSelectionChanges();
         
         void processSelectionChanges();
         
-        void labelDisplayGroupSelected(const DisplayGroupEnum::Enum);
-        
     private:
         LabelSelectionViewController(const LabelSelectionViewController&);
 
         LabelSelectionViewController& operator=(const LabelSelectionViewController&);
 
-        void updateLabelViewController();
-        
         void updateOtherLabelViewControllers();
         
         QWidget* createSelectionWidget();
         
+        const QString m_objectNamePrefix;
+        
         int32_t m_browserWindowIndex;
         
         GroupAndNameHierarchyViewController* m_labelClassNameHierarchyViewController;
@@ -81,8 +77,6 @@ namespace caret {
         
         QCheckBox* m_labelsContralateralCheckBox;
         
-        DisplayGroupEnumComboBox* m_labelsDisplayGroupComboBox;
-        
         static std::set<LabelSelectionViewController*> allLabelSelectionViewControllers;
     };
     
