@@ -27,10 +27,15 @@
 namespace caret {
 
     class AnnotationTextSubstitutionFile;
+    class AnnotationTextSubstitutionLayer;
     class CaretMappableDataFile;
     class ChartTwoOverlay;
     class ChartableMatrixSeriesInterface;
     class EnumComboBoxTemplate;
+    class HistologyOverlay;
+    class HistologySlicesFile;
+    class MediaFile;
+    class MediaOverlay;
     class Overlay;
     
     class MapYokingGroupComboBox : public WuQWidget {
@@ -39,6 +44,10 @@ namespace caret {
     public:
         MapYokingGroupComboBox(QObject* parent);
         
+        MapYokingGroupComboBox(QObject* parent,
+                               const QString& objectName,
+                               const QString& descriptiveName);
+        
         virtual ~MapYokingGroupComboBox();
         
         virtual QWidget* getWidget();
@@ -54,6 +63,12 @@ namespace caret {
         
         void validateYokingChange(ChartTwoOverlay* chartOverlay);
         
+        void validateYokingChange(HistologyOverlay* histologyOverlay);
+        
+        void validateYokingChange(MediaOverlay* mediaOverlay);
+        
+        void validateYokingChange(AnnotationTextSubstitutionLayer* annTextSubsLayer);
+        
         void validateYokingChange(AnnotationTextSubstitutionFile* annTextSubFile);
         
         // ADD_NEW_METHODS_HERE
@@ -74,8 +89,11 @@ namespace caret {
 
         MapYokingGroupComboBox& operator=(const MapYokingGroupComboBox&);
         
-        YokeValidationResult validateYoking(AnnotationTextSubstitutionFile* annTextSubFile,
-                                            CaretMappableDataFile* selectedFile,
+        YokeValidationResult validateYoking(AnnotationTextSubstitutionLayer* annTextSubsLayer,
+                                            AnnotationTextSubstitutionFile* selectedAnnTextSubFile,
+                                            CaretMappableDataFile* selectedMapFile,
+                                            HistologySlicesFile* selectedHistologySlicesFile,
+                                            MediaFile* selectedMediaFile,
                                             int32_t& selectedMapIndexInOut,
                                             bool& selectionStatusInOut);
         
