@@ -34,7 +34,7 @@
 #include "BrainConstants.h"
 #include "BrowserTabContent.h"
 #include "CaretAssert.h"
-#include "EventGraphicsUpdateOneWindow.h"
+#include "EventGraphicsPaintSoonOneWindow.h"
 #include "EventManager.h"
 #include "EventSurfaceColoringInvalidate.h"
 #include "EventUserInterfaceUpdate.h"
@@ -60,11 +60,14 @@ using namespace caret;
  *    Orientation for layout
  * @param browserWindowIndex
  *    Index of browser window that contains this view controller.
+ * @param parentObjectName
+ *    Name of parent
  * @param parent
  *    Parent widget.
  */
 OverlaySetViewController::OverlaySetViewController(const Qt::Orientation orientation,
                                                    const int32_t browserWindowIndex,
+                                                   const QString& parentObjectName,
                                                    QWidget* parent)
 : QWidget(parent)
 {
@@ -81,6 +84,7 @@ OverlaySetViewController::OverlaySetViewController(const Qt::Orientation orienta
         gridLayout->setColumnStretch(4, 0);
         gridLayout->setColumnStretch(5, 100);
         gridLayout->setColumnStretch(6, 0);
+        gridLayout->setColumnStretch(7, 0);
         gridLayout->setColumnStretch(8, 100);
         
         QLabel* onLabel       = new QLabel("On");
@@ -113,6 +117,7 @@ OverlaySetViewController::OverlaySetViewController(const Qt::Orientation orienta
                                                                gridLayout,
                                                                browserWindowIndex,
                                                                i,
+                                                               parentObjectName,
                                                                this);
         this->overlayViewControllers.push_back(ovc);
         
@@ -135,15 +140,24 @@ OverlaySetViewController::OverlaySetViewController(const Qt::Orientation orienta
         verticalLayout->addStretch();
     }
     else {
-        QVBoxLayout* verticalLayout = new QVBoxLayout();
+        /*
+         * Resolve WB-649
+         */
+        QVBoxLayout* verticalLayout = new QVBoxLayout(this);
         WuQtUtilities::setLayoutSpacingAndMargins(verticalLayout, 1, 1);
         verticalLayout->addWidget(gridWidget);
         verticalLayout->addStretch();
         
-        QHBoxLayout* horizontalLayout = new QHBoxLayout(this);
-        WuQtUtilities::setLayoutSpacingAndMargins(horizontalLayout, 1, 1);
-        horizontalLayout->addLayout(verticalLayout);
-        horizontalLayout->addStretch();
+        
+//        QVBoxLayout* verticalLayout = new QVBoxLayout();
+//        WuQtUtilities::setLayoutSpacingAndMargins(verticalLayout, 1, 1);
+//        verticalLayout->addWidget(gridWidget);
+//        verticalLayout->addStretch();
+//        
+//        QHBoxLayout* horizontalLayout = new QHBoxLayout(this);
+//        WuQtUtilities::setLayoutSpacingAndMargins(horizontalLayout, 1, 1);
+//        horizontalLayout->addLayout(verticalLayout);
+//        horizontalLayout->addStretch();
     }
     
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_USER_INTERFACE_UPDATE);
@@ -309,7 +323,7 @@ OverlaySetViewController::updateColoringAndGraphics()
     this->updateViewController();
     
     EventManager::get()->sendEvent(EventSurfaceColoringInvalidate().getPointer());
-    EventGraphicsUpdateOneWindow graphicsUpdate(this->browserWindowIndex);
+    EventGraphicsPaintSoonOneWindow graphicsUpdate(this->browserWindowIndex);
     EventManager::get()->sendEvent(graphicsUpdate.getPointer());
 }
 
