@@ -34,13 +34,13 @@
 #include "CaretMappableDataFile.h"
 #include "ChartTwoOverlay.h"
 #include "CiftiFiberTrajectoryFile.h"
+#include "CiftiFiberTrajectoryMapFile.h"
 #include "CiftiConnectivityMatrixParcelFile.h"
-#include "EventChartOverlayValidate.h"
+#include "EventChartTwoOverlayValidate.h"
 #include "EventDataFileDelete.h"
-#include "EventGraphicsUpdateAllWindows.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
 #include "EventManager.h"
 #include "EventOverlayValidate.h"
-#include "EventSurfaceColoringInvalidate.h"
 #include "EventUserInterfaceUpdate.h"
 #include "GiftiLabelTableEditor.h"
 #include "MapSettingsChartTwoLineHistoryWidget.h"
@@ -48,6 +48,7 @@
 #include "MapSettingsFiberTrajectoryWidget.h"
 #include "MapSettingsLabelsWidget.h"
 #include "MapSettingsLayerWidget.h"
+#include "MapSettingsModulateWidget.h"
 #include "MapSettingsPaletteColorMappingWidget.h"
 #include "MapSettingsParcelsWidget.h"
 #include "Overlay.h"
@@ -108,6 +109,8 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
     
     m_lineHistoryWidget = new MapSettingsChartTwoLineHistoryWidget();
     
+    m_modulateWidget = new MapSettingsModulateWidget();
+    
     m_tabWidget = new QTabWidget();
     
     m_colorBarWidgetTabIndex = m_tabWidget->addTab(m_colorBarWidget,
@@ -119,9 +122,10 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
 
     m_layersWidgetTabIndex = m_tabWidget->addTab(m_layerWidget,
                       "Layer");
-    
+    m_modulateWidgetTabIndex = m_tabWidget->addTab(m_modulateWidget,
+                                                   "Modulate");
     m_lineHistoryWidgetTabIndex = m_tabWidget->addTab(m_lineHistoryWidget,
-                                                      "Lines");
+                                                      "Dyn-Lines");
     m_metadataWidgetTabIndex = m_tabWidget->addTab(new QWidget(),
                       "Metadata");
     m_tabWidget->setTabEnabled(m_tabWidget->count() - 1, false);
@@ -142,7 +146,6 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
     this->setLayoutSpacingAndMargins(layout);
     layout->addWidget(mapNameWidget);
     layout->addWidget(m_tabWidget);
-    //layout->addWidget(windowOptionsWidget);
 
     this->setCentralWidget(w,
                            WuQDialog::SCROLL_AREA_NEVER);
@@ -151,6 +154,7 @@ OverlaySettingsEditorDialog::OverlaySettingsEditorDialog(QWidget* parent)
     disableAutoDefaultForAllPushButtons();
 
     EventManager::get()->addProcessedEventListener(this, EventTypeEnum::EVENT_DATA_FILE_DELETE);
+    EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_BROWSER_WINDOW_GRAPHICS_HAVE_BEEN_REDRAWN);
 }
 
 /**
@@ -170,7 +174,8 @@ OverlaySettingsEditorDialog::~OverlaySettingsEditorDialog()
 void
 OverlaySettingsEditorDialog::receiveEvent(Event* event)
 {
-    if (event->getEventType() == EventTypeEnum::EVENT_DATA_FILE_DELETE) {
+    if ((event->getEventType() == EventTypeEnum::EVENT_DATA_FILE_DELETE)
+        || (event->getEventType() == EventTypeEnum::EVENT_BROWSER_WINDOW_GRAPHICS_HAVE_BEEN_REDRAWN)) {
         updateDialog();
     }
 }
@@ -308,7 +313,7 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
     bool isParcelsValid = false;
     bool isFiberTrajectoryValid = false;
     bool isVolumeLayer = false;
-    bool isLinesValid = false;
+    bool isLineHistoryValid = false;
     
     QString mapFileName = "";
     QString mapName = "";
@@ -371,13 +376,19 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
                     }
                 }
                 
+                m_modulateWidget->updateEditor(m_caretMappableDataFile, 
+                                               m_selectedMapFileIndex);
+                
                 CiftiFiberTrajectoryFile* trajFile = dynamic_cast<CiftiFiberTrajectoryFile*>(m_caretMappableDataFile);
-                if (trajFile != NULL) {
+                CiftiFiberTrajectoryMapFile* trajMapFile(dynamic_cast<CiftiFiberTrajectoryMapFile*>(m_caretMappableDataFile));
+                if ((trajFile != NULL)
+                    || (trajMapFile != NULL)) {
                     /*
                      * Update trajectory
                      */
                     isFiberTrajectoryValid = true;
-                    m_fiberTrajectoryWidget->updateEditor(trajFile);
+                    m_fiberTrajectoryWidget->updateEditor(trajFile,
+                                                          trajMapFile);
                 }
                 
                 CiftiConnectivityMatrixParcelFile* parcelsFile = dynamic_cast<CiftiConnectivityMatrixParcelFile*>(m_caretMappableDataFile);
@@ -403,14 +414,20 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
             }
         }
         else if (m_chartOverlay != NULL) {
-            bool hasMapsFlag = false;
-            bool hasHistoryFlag = false;
+            mapFileName = m_caretMappableDataFile->getFileNameNoPath();
+
+            bool hasMapsFlag      = false;
+            bool hasHistoryFlag   = false;
+            bool hasLineLayerFlag = false;
             switch (m_chartOverlay->getChartTwoDataType()) {
                 case ChartTwoDataTypeEnum::CHART_DATA_TYPE_INVALID:
                     break;
                 case ChartTwoDataTypeEnum::CHART_DATA_TYPE_HISTOGRAM:
                     hasMapsFlag = true;
                     break;
+                case ChartTwoDataTypeEnum::CHART_DATA_TYPE_LINE_LAYER:
+                    hasLineLayerFlag = true;
+                    break;
                 case ChartTwoDataTypeEnum::CHART_DATA_TYPE_LINE_SERIES:
                     hasHistoryFlag = true;
                     break;
@@ -445,10 +462,6 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
                 }
                 if ((selectedMapIndex >= 0)
                     && (selectedMapIndex < m_caretMappableDataFile->getNumberOfMaps())) {
-                    /*
-                     * Get name of file and map
-                     */
-                    mapFileName = m_caretMappableDataFile->getFileNameNoPath();
                     if (m_selectedMapFileIndex >= 0) {
                         mapName = m_caretMappableDataFile->getMapName(selectedMapIndex);
                     }
@@ -497,8 +510,19 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
                 }
             }
             else if (hasHistoryFlag) {
-                isLinesValid = true;
+                isLineHistoryValid = true;
                 m_lineHistoryWidget->updateContent(m_chartOverlay);
+                if (mapName.isEmpty()) {
+                    mapName = "Line Chart History";
+                }
+            }
+            else if (hasLineLayerFlag) {
+                if (mapName.isEmpty()) {
+                    if ((m_selectedMapFileIndex >= 0)
+                        && (m_selectedMapFileIndex < m_caretMappableDataFile->getNumberOfMaps())) {
+                        mapName = m_caretMappableDataFile->getMapName(m_selectedMapFileIndex);
+                    }
+                }
             }
         }
         else {
@@ -525,7 +549,9 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
     m_tabWidget->setTabEnabled(m_trajectoryWidgetTabIndex,
                                isFiberTrajectoryValid);
     m_tabWidget->setTabEnabled(m_lineHistoryWidgetTabIndex,
-                               isLinesValid);
+                               isLineHistoryValid);
+    m_tabWidget->setTabEnabled(m_modulateWidgetTabIndex,
+                               isVolumeLayer);
 
     /*
      * When the selected tab is invalid, we want to select the
@@ -537,6 +563,7 @@ OverlaySettingsEditorDialog::updateDialogContentPrivate(Overlay* brainordinateOv
     priorityTabIndices.push_back(m_lineHistoryWidgetTabIndex);
     priorityTabIndices.push_back(m_colorBarWidgetTabIndex);
     priorityTabIndices.push_back(m_labelsWidgetTabIndex);
+    priorityTabIndices.push_back(m_modulateWidgetTabIndex);
     priorityTabIndices.push_back(m_parcelsWidgetTabIndex);
     priorityTabIndices.push_back(m_trajectoryWidgetTabIndex);
     priorityTabIndices.push_back(m_layersWidgetTabIndex);
@@ -585,7 +612,7 @@ OverlaySettingsEditorDialog::updateDialog()
     }
     
     if (m_chartOverlay != NULL) {
-        EventChartOverlayValidate validateChartOverlayEvent(m_chartOverlay);
+        EventChartTwoOverlayValidate validateChartOverlayEvent(m_chartOverlay);
         EventManager::get()->sendEvent(validateChartOverlayEvent.getPointer());
         if ( ! validateChartOverlayEvent.isValidChartOverlay()) {
             m_chartOverlay = NULL;
@@ -608,7 +635,7 @@ void
 OverlaySettingsEditorDialog::updateChartLinesInDialog()
 {
     if (m_chartOverlay != NULL) {
-        EventChartOverlayValidate validateChartOverlayEvent(m_chartOverlay);
+        EventChartTwoOverlayValidate validateChartOverlayEvent(m_chartOverlay);
         EventManager::get()->sendEvent(validateChartOverlayEvent.getPointer());
         if ( ! validateChartOverlayEvent.isValidChartOverlay()) {
             m_chartOverlay = NULL;
