@@ -40,8 +40,8 @@
 #include "AnnotationColorBar.h"
 #include "CaretMappableDataFile.h"
 #include "EventDataFileReload.h"
-#include "EventGraphicsUpdateAllWindows.h"
-#include "EventGraphicsUpdateOneWindow.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
+#include "EventGraphicsPaintSoonOneWindow.h"
 #include "EventManager.h"
 #include "EventMapYokingSelectMap.h"
 #include "EventOverlaySettingsEditorDialogRequest.h"
@@ -54,6 +54,7 @@
 #include "Overlay.h"
 #include "UsernamePasswordWidget.h"
 #include "WuQFactory.h"
+#include "WuQMacroManager.h"
 #include "WuQMessageBox.h"
 #include "WuQtUtilities.h"
 #include "WuQGridLayoutGroup.h"
@@ -70,10 +71,16 @@ using namespace caret;
 /**
  * Constructor.
  *
+ * @param orientation
+ *    Orientation of overlay (horizontal/vertical)
+ * @param gridLayout
+ *    Layout for widegets
  * @param browserWindowIndex
  *    Index of browser window in which this view controller resides.
- * @param showTopHorizontalLine
- *    If true, display a horizontal line above the controls.
+ * @param overlayIndex
+ *    Index of the overlay
+ * @param parentObjectName
+ *    Name of parent object for macros
  * @param parent
  *    The parent widget.
  */
@@ -81,6 +88,7 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
                                              QGridLayout* gridLayout,
                                              const int32_t browserWindowIndex,
                                              const int32_t overlayIndex,
+                                             const QString& parentObjectName,
                                              QObject* parent)
 : QObject(parent),
   browserWindowIndex(browserWindowIndex),
@@ -90,83 +98,105 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
     m_constructionReloadFileAction = NULL;
     
     int minComboBoxWidth = 200;
-    int maxComboBoxWidth = 100000; //400;
+    int maxComboBoxWidth = 100000;
     if (orientation == Qt::Horizontal) {
         minComboBoxWidth = 50;
         maxComboBoxWidth = 100000;
     }
-    const QComboBox::SizeAdjustPolicy comboSizePolicy = QComboBox::AdjustToContentsOnFirstShow; //QComboBox::AdjustToContents;
+    const QComboBox::SizeAdjustPolicy comboSizePolicy = QComboBox::AdjustToContentsOnFirstShow;
 
+    WuQMacroManager* macroManager = WuQMacroManager::instance();
+    CaretAssert(macroManager);
+    QString objectNamePrefix = QString(parentObjectName
+                                       + ":Overlay%1"
+                                       + ":").arg((int)(overlayIndex + 1), 2, 10, QLatin1Char('0'));
+    const QString descriptivePrefix = QString("overlay %1").arg(overlayIndex + 1);
     /*
      * Enabled Check Box
      */
     const QString checkboxText = ((orientation == Qt::Horizontal) ? " " : " ");
     this->enabledCheckBox = new QCheckBox(checkboxText);
+    this->enabledCheckBox->setObjectName(objectNamePrefix
+                                         + "OnOff");
     QObject::connect(this->enabledCheckBox, SIGNAL(clicked(bool)),
                      this, SLOT(enabledCheckBoxClicked(bool)));
     this->enabledCheckBox->setToolTip("Enables display of this overlay");
+    macroManager->addMacroSupportToObject(this->enabledCheckBox,
+                                          "Enable " + descriptivePrefix);
     
     /*
-     * File Selection Check Box
+     * File Selection Combo Box
      */
     this->fileComboBox = WuQFactory::newComboBox();
+    this->fileComboBox->setObjectName(objectNamePrefix
+                                      + "FileSelection");
     this->fileComboBox->setMinimumWidth(minComboBoxWidth);
     this->fileComboBox->setMaximumWidth(maxComboBoxWidth);
     QObject::connect(this->fileComboBox, SIGNAL(activated(int)),
                      this, SLOT(fileComboBoxSelected(int)));
     this->fileComboBox->setToolTip("Selects file for this overlay");
     this->fileComboBox->setSizeAdjustPolicy(comboSizePolicy);
+    macroManager->addMacroSupportToObject(this->fileComboBox,
+                                          ("Select file in " + descriptivePrefix));
     
     /*
      * Map Index Spin Box
      */
     m_mapIndexSpinBox = WuQFactory::newSpinBox();
+    this->m_mapIndexSpinBox->setObjectName(objectNamePrefix
+                                         + "MapIndex");
     QObject::connect(m_mapIndexSpinBox, SIGNAL(valueChanged(int)),
                      this, SLOT(mapIndexSpinBoxValueChanged(int)));
     m_mapIndexSpinBox->setToolTip("Select map by its index");
+    macroManager->addMacroSupportToObject(m_mapIndexSpinBox,
+                                          ("Select " + descriptivePrefix + " map index"));
     
     /*
      * Map Name Combo Box
      */
     this->mapNameComboBox = WuQFactory::newComboBox();
+    this->mapNameComboBox->setObjectName(objectNamePrefix
+                                      + "MapSelection");
     this->mapNameComboBox->setMinimumWidth(minComboBoxWidth);
     this->mapNameComboBox->setMaximumWidth(maxComboBoxWidth);
     QObject::connect(this->mapNameComboBox, SIGNAL(activated(int)),
                      this, SLOT(mapNameComboBoxSelected(int)));
     this->mapNameComboBox->setToolTip("Select map by its name");
     this->mapNameComboBox->setSizeAdjustPolicy(comboSizePolicy);
+    macroManager->addMacroSupportToObject(this->mapNameComboBox,
+                                          ("Select " + descriptivePrefix + " map name"));
     
     /*
      * Opacity double spin box
      */
     this->opacityDoubleSpinBox = WuQFactory::newDoubleSpinBox();
+    this->opacityDoubleSpinBox->setObjectName(objectNamePrefix
+                                              + "Opacity");
     this->opacityDoubleSpinBox->setMinimum(0.0);
     this->opacityDoubleSpinBox->setMaximum(1.0);
-    this->opacityDoubleSpinBox->setSingleStep(0.10);
-    this->opacityDoubleSpinBox->setDecimals(1);
+    this->opacityDoubleSpinBox->setSingleStep(0.05);
+    this->opacityDoubleSpinBox->setDecimals(2);
     this->opacityDoubleSpinBox->setFixedWidth(50);
     QObject::connect(this->opacityDoubleSpinBox, SIGNAL(valueChanged(double)),
                      this, SLOT(opacityDoubleSpinBoxValueChanged(double)));
     this->opacityDoubleSpinBox->setToolTip("Opacity (0.0=transparent, 1.0=opaque)");
+    macroManager->addMacroSupportToObject(this->opacityDoubleSpinBox,
+                                          ("Set " + descriptivePrefix + " opacity"));
     
     /*
      * ColorBar Tool Button
      */
-    QIcon colorBarIcon;
-    const bool colorBarIconValid = WuQtUtilities::loadIcon(":/LayersPanel/colorbar.png",
-                                                           colorBarIcon);
-    this->colorBarAction = WuQtUtilities::createAction("CB", 
-                                                       "Display color bar for this overlay", 
-                                                       this, 
-                                                       this, 
-                                                       SLOT(colorBarActionTriggered(bool)));
-    this->colorBarAction->setCheckable(true);
-    if (colorBarIconValid) {
-        this->colorBarAction->setIcon(colorBarIcon);
-    }
-    QToolButton* colorBarToolButton = new QToolButton();
-    colorBarToolButton->setDefaultAction(this->colorBarAction);
-    
+    m_colorBarToolButton = WuQtUtilities::createToolButtonWithIcon("CB",
+                                                                   ":/LayersPanel/colorbar.png",
+                                                                   "Display color bar for this overlay",
+                                                                   this,
+                                                                   SLOT(colorBarActionTriggered(bool)));
+    m_colorBarToolButton->setCheckable(true);
+    m_colorBarToolButton->setObjectName(objectNamePrefix
+                                        + "ShowColorBar");
+    macroManager->addMacroSupportToObject(m_colorBarToolButton,
+                                          ("Enable " + descriptivePrefix + " colorbar"));
+        
     /*
      * Settings Tool Button
      */
@@ -179,14 +209,19 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
                                                           this, 
                                                           this, 
                                                           SLOT(settingsActionTriggered()));
+    this->settingsAction->setObjectName(objectNamePrefix
+                                        + "ShowSettingsDialog");
     if (settingsIconValid) {
         this->settingsAction->setIcon(settingsIcon);
     }
     QToolButton* settingsToolButton = new QToolButton();
     settingsToolButton->setDefaultAction(this->settingsAction);
+    macroManager->addMacroSupportToObject(this->settingsAction,
+                                          ("Display " + descriptivePrefix + " palette settings"));
     
     /*
      * Construction Tool Button
+     * Note: macro support is on each action in menu in 'createConstructionMenu'
      */
     QIcon constructionIcon;
     const bool constructionIconValid = WuQtUtilities::loadIcon(":/LayersPanel/construction.png",
@@ -198,7 +233,10 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
         this->constructionAction->setIcon(constructionIcon);
     }
     m_constructionToolButton = new QToolButton();
-    QMenu* constructionMenu = createConstructionMenu(m_constructionToolButton);
+    QMenu* constructionMenu = createConstructionMenu(m_constructionToolButton,
+                                                     descriptivePrefix,
+                                                     (objectNamePrefix
+                                                      + "ConstructionMenu:"));
     this->constructionAction->setMenu(constructionMenu);
     m_constructionToolButton->setDefaultAction(this->constructionAction);
     m_constructionToolButton->setPopupMode(QToolButton::InstantPopup);
@@ -214,13 +252,14 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
     
     /*
      * Yoking Group
+     * Note: macro support is in the class MapYokingGroupComboBox
      */
-    m_mapYokingGroupComboBox = new MapYokingGroupComboBox(this);
+    m_mapYokingGroupComboBox = new MapYokingGroupComboBox(this,
+                                                          (objectNamePrefix
+                                                           + "MapYokingSelection"),
+                                                          descriptivePrefix);
     m_mapYokingGroupComboBox->getWidget()->setStatusTip("Synchronize enabled status and map indices)");
     m_mapYokingGroupComboBox->getWidget()->setToolTip("Yoke to Overlay Mapped Files");
-#ifdef CARET_OS_MACOSX
-    m_mapYokingGroupComboBox->getWidget()->setFixedWidth(m_mapYokingGroupComboBox->getWidget()->sizeHint().width() - 20);
-#endif // CARET_OS_MACOSX
     QObject::connect(m_mapYokingGroupComboBox, SIGNAL(itemActivated()),
                      this, SLOT(yokingGroupActivated()));
 
@@ -237,7 +276,7 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
         this->gridLayoutGroup->addWidget(settingsToolButton,
                                          row, 1,
                                          Qt::AlignHCenter);
-        this->gridLayoutGroup->addWidget(colorBarToolButton,
+        this->gridLayoutGroup->addWidget(m_colorBarToolButton,
                                          row, 2);
         this->gridLayoutGroup->addWidget(m_constructionToolButton,
                                          row, 3);
@@ -268,7 +307,7 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
                                          row, 0);
         this->gridLayoutGroup->addWidget(settingsToolButton,
                                          row, 1);
-        this->gridLayoutGroup->addWidget(colorBarToolButton,
+        this->gridLayoutGroup->addWidget(m_colorBarToolButton,
                                          row, 2);
         this->gridLayoutGroup->addWidget(m_constructionToolButton,
                                          row, 3);
@@ -359,11 +398,6 @@ OverlayViewController::fileComboBoxSelected(int indx)
     
     validateYokingSelection();
     
-    //validateYokingSelection(overlay->getYokingGroup());
-    // not needed with call to validateYokingSelection: this->updateViewController(this->overlay);
-    
-    // called inside validateYokingSelection();  this->updateUserInterfaceAndGraphicsWindow();
-
     updateOverlaySettingsEditor();
     updateViewController(this->overlay);
     
@@ -408,10 +442,16 @@ OverlayViewController::mapIndexSpinBoxValueChanged(int indx)
     
     const MapYokingGroupEnum::Enum mapYoking = overlay->getMapYokingGroup();
     if (mapYoking != MapYokingGroupEnum::MAP_YOKING_GROUP_OFF) {
+        AnnotationTextSubstitutionFile* nullAnnTextSubsFile(NULL);
+        HistologySlicesFile* nullHistologySlicesFile(NULL);
+        MediaFile* nullMediaFile(NULL);
         EventMapYokingSelectMap selectMapEvent(mapYoking,
                                                file,
-                                               NULL,
+                                               nullAnnTextSubsFile,
+                                               nullHistologySlicesFile,
+                                               nullMediaFile,
                                                overlayIndex,
+                                               MapYokingGroupEnum::MediaAllFramesStatus::ALL_FRAMES_OFF,
                                                overlay->isEnabled());
         EventManager::get()->sendEvent(selectMapEvent.getPointer());
     }
@@ -455,10 +495,16 @@ OverlayViewController::mapNameComboBoxSelected(int indx)
     
     const MapYokingGroupEnum::Enum mapYoking = overlay->getMapYokingGroup();
     if (mapYoking != MapYokingGroupEnum::MAP_YOKING_GROUP_OFF) {
+        AnnotationTextSubstitutionFile* nullAnnTextSubsFile(NULL);
+        HistologySlicesFile* nullHistologySlicesFile(NULL);
+        MediaFile* nullMediaFile(NULL);
         EventMapYokingSelectMap selectMapEvent(mapYoking,
                                                file,
-                                               NULL,
+                                               nullAnnTextSubsFile,
+                                               nullHistologySlicesFile,
+                                               nullMediaFile,
                                                indx,
+                                               MapYokingGroupEnum::MediaAllFramesStatus::ALL_FRAMES_OFF,
                                                overlay->isEnabled());
         EventManager::get()->sendEvent(selectMapEvent.getPointer());
     }
@@ -497,10 +543,16 @@ OverlayViewController::enabledCheckBoxClicked(bool checked)
         this->overlay->getSelectionData(myFile,
                                         myIndex);
         
+        AnnotationTextSubstitutionFile* nullAnnTextSubsFile(NULL);
+        HistologySlicesFile* nullHistologySlicesFile(NULL);
+        MediaFile* nullMediaFile(NULL);
         EventMapYokingSelectMap selectMapEvent(mapYoking,
                                                myFile,
-                                               NULL,
+                                               nullAnnTextSubsFile,
+                                               nullHistologySlicesFile,
+                                               nullMediaFile,
                                                myIndex,
+                                               MapYokingGroupEnum::MediaAllFramesStatus::ALL_FRAMES_OFF,
                                                overlay->isEnabled());
         EventManager::get()->sendEvent(selectMapEvent.getPointer());
     }
@@ -553,7 +605,6 @@ OverlayViewController::validateYokingSelection()
     m_mapYokingGroupComboBox->validateYokingChange(this->overlay);
     updateViewController(this->overlay);
     updateGraphicsWindow();
-    //EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
 }
 
 /**
@@ -615,12 +666,10 @@ OverlayViewController::updateViewController(Overlay* overlay)
      */
     std::vector<CaretMappableDataFile*> dataFiles;
     CaretMappableDataFile* selectedFile = NULL;
-    //AString selectedMapUniqueID = "";
     int32_t selectedMapIndex = -1;
     if (this->overlay != NULL) {
         this->overlay->getSelectionData(dataFiles, 
                                   selectedFile, 
-                                  //selectedMapUniqueID,
                                   selectedMapIndex);
     }
     
@@ -640,7 +689,7 @@ OverlayViewController::updateViewController(Overlay* overlay)
         AString dataTypeName = DataFileTypeEnum::toOverlayTypeName(dataFile->getDataFileType());
         CaretAssertVectorIndex(displayNames, i);
         this->fileComboBox->addItem(displayNames[i],
-                                    qVariantFromValue((void*)dataFile));
+                                    QVariant::fromValue((void*)dataFile));
         if (dataFile == selectedFile) {
             selectedFileIndex = i;
         }
@@ -687,9 +736,7 @@ OverlayViewController::updateViewController(Overlay* overlay)
     
     m_mapYokingGroupComboBox->setMapYokingGroup(overlay->getMapYokingGroup());
     
-    this->colorBarAction->blockSignals(true);
-    this->colorBarAction->setChecked(overlay->getColorBar()->isDisplayed());
-    this->colorBarAction->blockSignals(false);
+    m_colorBarToolButton->setChecked(overlay->getColorBar()->isDisplayed());
     
     this->opacityDoubleSpinBox->blockSignals(true);
     this->opacityDoubleSpinBox->setValue(overlay->getOpacity());
@@ -758,7 +805,7 @@ OverlayViewController::updateViewController(Overlay* overlay)
     this->constructionAction->setEnabled(true);
     this->opacityDoubleSpinBox->setEnabled(haveOpacity);
     this->m_mapYokingGroupComboBox->getWidget()->setEnabled(haveYoking);
-    this->colorBarAction->setEnabled(dataIsMappedWithPalette);
+    this->m_colorBarToolButton->setEnabled(dataIsMappedWithPalette);
     this->settingsAction->setEnabled(true);
 }
 
@@ -794,10 +841,10 @@ OverlayViewController::updateGraphicsWindow()
 {
     EventManager::get()->sendEvent(EventSurfaceColoringInvalidate().getPointer());
     if (this->overlay->getMapYokingGroup() != MapYokingGroupEnum::MAP_YOKING_GROUP_OFF) {
-        EventManager::get()->sendEvent(EventGraphicsUpdateAllWindows().getPointer());
+        EventManager::get()->sendEvent(EventGraphicsPaintSoonAllWindows().getPointer());
     }
     else {
-        EventManager::get()->sendEvent(EventGraphicsUpdateOneWindow(this->browserWindowIndex).getPointer());
+        EventManager::get()->sendEvent(EventGraphicsPaintSoonOneWindow(this->browserWindowIndex).getPointer());
     }
 }
 
@@ -805,53 +852,102 @@ OverlayViewController::updateGraphicsWindow()
  * Create the construction menu.
  * @param parent
  *    Parent widget.
+ * @param descriptivePrefix
+ *    Descriptive prefix
+ * @param menuActionNamePrefix
+ *    Prefix for macros
  */
 QMenu* 
-OverlayViewController::createConstructionMenu(QWidget* parent)
+OverlayViewController::createConstructionMenu(QWidget* parent,
+                                              const AString& descriptivePrefix,
+                                              const AString& menuActionNamePrefix)
 {
+    WuQMacroManager* macroManager = WuQMacroManager::instance();
+    CaretAssert(macroManager);
+    
     QMenu* menu = new QMenu(parent);
     QObject::connect(menu, SIGNAL(aboutToShow()),
                      this, SLOT(menuConstructionAboutToShow()));
     
-    menu->addAction("Add Overlay Above", 
-                    this, 
-                    SLOT(menuAddOverlayAboveTriggered()));
+    QAction* addAboveAction = menu->addAction("Add Overlay Above",
+                                              this,
+                                              SLOT(menuAddOverlayAboveTriggered()));
+    addAboveAction->setObjectName(menuActionNamePrefix
+                                  + "AddOverlayAbove");
+    addAboveAction->setToolTip("Add an overlay above this overlay");
+    macroManager->addMacroSupportToObject(addAboveAction,
+                                          ("Add overlay above " + descriptivePrefix));
     
-    menu->addAction("Add Overlay Below", 
-                    this, 
-                    SLOT(menuAddOverlayBelowTriggered()));
+    QAction* addBelowAction = menu->addAction("Add Overlay Below",
+                                              this,
+                                              SLOT(menuAddOverlayBelowTriggered()));
+    addBelowAction->setObjectName(menuActionNamePrefix
+                                  + "AddOverlayBelow");
+    addBelowAction->setToolTip("Add an overlay below this overlay");
+    macroManager->addMacroSupportToObject(addBelowAction,
+                                          ("Add overlay below " + descriptivePrefix));
     
     menu->addSeparator();
     
-    menu->addAction("Move Overlay Up", 
-                    this, 
-                    SLOT(menuMoveOverlayUpTriggered()));
+    QAction* moveUpAction = menu->addAction("Move Overlay Up",
+                                            this,
+                                            SLOT(menuMoveOverlayUpTriggered()));
+    moveUpAction->setObjectName(menuActionNamePrefix
+                                + "MoveOverlayUp");
+    moveUpAction->setToolTip("Move this overlay up");
+    macroManager->addMacroSupportToObject(moveUpAction,
+                                          ("Move " + descriptivePrefix + " up"));
     
-    menu->addAction("Move Overlay Down", 
-                    this, 
-                    SLOT(menuMoveOverlayDownTriggered()));
+    QAction* moveDownAction = menu->addAction("Move Overlay Down",
+                                              this,
+                                              SLOT(menuMoveOverlayDownTriggered()));
+    moveDownAction->setObjectName(menuActionNamePrefix
+                                  + "MoveOverlayDown");
+    moveDownAction->setToolTip("Move this overlay down");
+    macroManager->addMacroSupportToObject(moveDownAction,
+                                          ("Move " + descriptivePrefix + " down"));
     
     menu->addSeparator();
     
-    menu->addAction("Remove This Overlay", 
-                    this, 
-                    SLOT(menuRemoveOverlayTriggered()));
+    QAction* removeAction = menu->addAction("Remove This Overlay",
+                                            this,
+                                            SLOT(menuRemoveOverlayTriggered()));
+    removeAction->setObjectName(menuActionNamePrefix
+                                + "RemoveOverlay");
+    removeAction->setToolTip("Remove this overlay");
+    macroManager->addMacroSupportToObject(removeAction,
+                                          ("Remove " + descriptivePrefix));
     
     menu->addSeparator();
     
     m_constructionReloadFileAction = menu->addAction("Reload Selected File",
                                                      this,
                                                      SLOT(menuReloadFileTriggered()));
+    m_constructionReloadFileAction->setObjectName(menuActionNamePrefix
+                                                  + "ReloadSelectedFile");
+    m_constructionReloadFileAction->setToolTip("Reload file in this overlay");
+    macroManager->addMacroSupportToObject(m_constructionReloadFileAction,
+                                          ("Reload file in " + descriptivePrefix));
     
     menu->addSeparator();
     
     m_copyPathAndFileNameToClipboardAction = menu->addAction("Copy Path and File Name to Clipboard",
                                                              this,
                                                              SLOT(menuCopyFileNameToClipBoard()));
+    m_copyPathAndFileNameToClipboardAction->setObjectName(menuActionNamePrefix
+                                                          + "CopyPathAndFileNameToClipboard");
+    m_copyPathAndFileNameToClipboardAction->setToolTip("Copy path and file name of file in this overlay to clipboard");
+    macroManager->addMacroSupportToObject(m_copyPathAndFileNameToClipboardAction,
+                                          ("Copy path and filename from " + descriptivePrefix + " to clipboard"));
     
-    menu->addAction("Copy Map Name to Clipboard",
-                    this,
-                    SLOT(menuCopyMapNameToClipBoard()));
+    QAction* copyMapNameAction = menu->addAction("Copy Map Name to Clipboard",
+                                                 this,
+                                                 SLOT(menuCopyMapNameToClipBoard()));
+    copyMapNameAction->setObjectName(menuActionNamePrefix
+                                     + "CopyMapNameToClipboard");
+    copyMapNameAction->setToolTip("Copy name of selected map to the clipboard");
+    macroManager->addMacroSupportToObject(copyMapNameAction,
+                                          ("Copy map namne in " + descriptivePrefix + " to clipboard"));
     
     return menu;
     
@@ -881,9 +977,87 @@ OverlayViewController::menuConstructionAboutToShow()
                 menuText += suffix;
             }
             
-            const bool notDynConnFileFlag = (caretDataFile->getDataFileType() != DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC);
-            m_constructionReloadFileAction->setEnabled(notDynConnFileFlag);
-            m_copyPathAndFileNameToClipboardAction->setEnabled(notDynConnFileFlag);
+            bool dynConnFlag(false);
+            switch (caretDataFile->getDataFileType()) {
+                case DataFileTypeEnum::ANNOTATION:
+                    break;
+                case DataFileTypeEnum::ANNOTATION_TEXT_SUBSTITUTION:
+                    break;
+                case DataFileTypeEnum::BORDER:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_DENSE:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC:
+                    dynConnFlag = true;
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_DENSE_LABEL:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_DENSE_PARCEL:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_PARCEL:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_PARCEL_DENSE:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_PARCEL_DYNAMIC:
+                    dynConnFlag = true;
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_PARCEL_LABEL:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_PARCEL_SCALAR:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_PARCEL_SERIES:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_DENSE_SCALAR:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_DENSE_TIME_SERIES:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_FIBER_ORIENTATIONS_TEMPORARY:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_TEMPORARY:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_FIBER_TRAJECTORY_MAPS:
+                    break;
+                case DataFileTypeEnum::CONNECTIVITY_SCALAR_DATA_SERIES:
+                    break;
+                case DataFileTypeEnum::CZI_IMAGE_FILE:
+                    break;
+                case DataFileTypeEnum::FOCI:
+                    break;
+                case DataFileTypeEnum::HISTOLOGY_SLICES:
+                    break;
+                case DataFileTypeEnum::IMAGE:
+                    break;
+                case DataFileTypeEnum::LABEL:
+                    break;
+                case DataFileTypeEnum::METRIC:
+                    break;
+                case DataFileTypeEnum::METRIC_DYNAMIC:
+                    dynConnFlag = true;
+                    break;
+                case DataFileTypeEnum::OME_ZARR_IMAGE_FILE:
+                    break;
+                case DataFileTypeEnum::PALETTE:
+                    break;
+                case DataFileTypeEnum::RGBA:
+                    break;
+                case DataFileTypeEnum::SAMPLES:
+                    break;
+                case DataFileTypeEnum::SCENE:
+                    break;
+                case DataFileTypeEnum::SPECIFICATION:
+                    break;
+                case DataFileTypeEnum::SURFACE:
+                    break;
+                case DataFileTypeEnum::UNKNOWN:
+                    break;
+                case DataFileTypeEnum::VOLUME:
+                    break;
+                case DataFileTypeEnum::VOLUME_DYNAMIC:
+                    dynConnFlag = true;
+                    break;
+            }
+            m_constructionReloadFileAction->setEnabled( ! dynConnFlag);
+            m_copyPathAndFileNameToClipboardAction->setEnabled( ! dynConnFlag);
         }
         
         m_constructionReloadFileAction->setText(menuText);
