@@ -21,14 +21,18 @@
  */
 /*LICENSE_END*/
 
+#include <memory>
+
 #include <QProgressDialog>
 
+#include "AString.h"
 #include "EventListenerInterface.h"
-#include "ProgressReportingWithSlots.h"
+
+class QMessageBox;
 
 namespace caret {
 
-    class ProgressReportingFromEvent;
+    class ProgressReportingInterface;
     
     class ProgressReportingDialog : public QProgressDialog {
         
@@ -38,12 +42,18 @@ namespace caret {
         ProgressReportingDialog(const AString& title,
                                 const AString& initialMessage,
                                 QWidget* parent,
-                                Qt::WindowFlags f = 0);
-        
+                                Qt::WindowFlags f = Qt::WindowFlags());
+
         static void runEvent(Event* event,
                              QWidget* parent,
                              const AString& title);
         
+        static std::unique_ptr<ProgressReportingDialog> showProgressMessage(const AString& title,
+                                                                            const AString& message,
+                                                                            QWidget* parent);
+
+        void setEventReceivingEnabled(bool status);
+        
     public:
         virtual ~ProgressReportingDialog();
 
@@ -62,19 +72,7 @@ namespace caret {
 
         ProgressReportingInterface* m_progressReporter;
     };
-    
-    class ProgressReportingFromEvent : public ProgressReportingWithSlots, public EventListenerInterface {
-        Q_OBJECT
-        
-    public:
-        ProgressReportingFromEvent(QObject* parent);
-        
-        virtual ~ProgressReportingFromEvent();
-        
-        void receiveEvent(Event* event);
-        
-    };
-    
+
 #ifdef __PROGRESS_REPORTING_DIALOG_DECLARE__
     // <PLACE DECLARATIONS OF STATIC MEMBERS HERE>
 #endif // __PROGRESS_REPORTING_DIALOG_DECLARE__
