@@ -27,24 +27,27 @@
 
 #include <QAction>
 #include <QCheckBox>
-#include <QDateTime>
 #include <QGridLayout>
 #include <QGroupBox>
 #include <QLabel>
 #include <QLineEdit>
 #include <QPlainTextEdit>
 #include <QPushButton>
+#include <QToolButton>
 #include <QVBoxLayout>
 
 #include "ApplicationInformation.h"
 #include "Brain.h"
 #include "BrainBrowserWindow.h"
 #include "CaretAssert.h"
+#include "CaretLogger.h"
 #include "CaretPreferences.h"
 #include "DataFileException.h"
+#include "ElapsedTimer.h"
 #include "EventBrowserTabGetAll.h"
 #include "EventBrowserTabGetAllViewed.h"
 #include "EventImageCapture.h"
+#include "EventSceneActive.h"
 #include "EventManager.h"
 #include "GuiManager.h"
 #include "ImageFile.h"
@@ -95,6 +98,9 @@ SceneCreateReplaceDialog::SceneCreateReplaceDialog(const AString& dialogTitle,
     switch (m_mode) {
         case MODE_ADD_NEW_SCENE:
             break;
+        case MODE_EDIT_SCENE_INFO:
+            CaretAssert(sceneToInsertOrReplace);
+            break;
         case MODE_INSERT_NEW_SCENE:
             CaretAssert(sceneToInsertOrReplace);
             break;
@@ -110,6 +116,7 @@ SceneCreateReplaceDialog::SceneCreateReplaceDialog(const AString& dialogTitle,
         s_previousSelections.m_addAllTabs                 = true;
         s_previousSelections.m_addModifiedPaletteSettings = true;
         s_previousSelections.m_addSpecFileNameToScene     = true;
+        s_previousSelections.m_cropImage                  = true;
     }
     
     m_sceneFile = sceneFile;
@@ -129,9 +136,28 @@ SceneCreateReplaceDialog::SceneCreateReplaceDialog(const AString& dialogTitle,
     QLabel* nameLabel = new QLabel("Name");
     m_nameLineEdit = new QLineEdit();
     
+    const QString sceneIdWarningToolTip("The BALSA Scene ID is an identifier created by BALSA.  ALTERING "
+                                        "THE BALSA SCENE ID MAY CORRUPT THE SCENE FILE.");
     QLabel* sceneIDLabel = new QLabel("BALSA Scene ID");
+    const QString sceneIdLineEditToolTip(sceneIdWarningToolTip
+                                         + "  Click the "
+                                         "\"Unlock\" button if you must edit the ID.");
     m_balsaSceneIDLineEdit = new QLineEdit();
-    m_balsaSceneIDLineEdit->setToolTip("Scene ID is for use with BALSA Database");
+    m_balsaSceneIDLineEdit->setReadOnly(true);
+    m_balsaSceneIDLineEdit->setMaxLength(10);
+    WuQtUtilities::setWordWrappedToolTip(m_balsaSceneIDLineEdit,
+                                         sceneIdLineEditToolTip);
+    
+    const QString balsaIdToolTipText("Click this button to unlock (lock) and enable (disable) editing of the "
+                                     "BALSA Scene ID.  "
+                                     + sceneIdWarningToolTip);
+    m_lockUnlockBalseSceneIdToolButton = new QToolButton();
+    m_lockUnlockBalseSceneIdToolButton->setText("Unlock");
+    WuQtUtilities::setWordWrappedToolTip(m_lockUnlockBalseSceneIdToolButton,
+                                         balsaIdToolTipText);
+    QObject::connect(m_lockUnlockBalseSceneIdToolButton, &QToolButton::clicked,
+                     this, &SceneCreateReplaceDialog::lockUnlockBalseSceneIdToolButtonClicked);
+    
     
     QPushButton* addWindowDescriptionPushButton = new QPushButton("Add Window Info");
     QObject::connect(addWindowDescriptionPushButton, &QPushButton::clicked,
@@ -140,6 +166,12 @@ SceneCreateReplaceDialog::SceneCreateReplaceDialog(const AString& dialogTitle,
     QLabel* descriptionLabel = new QLabel("Description");
     m_descriptionTextEdit = new QPlainTextEdit();
     
+    QHBoxLayout* idLayout = new QHBoxLayout();
+    idLayout->setContentsMargins(0, 0, 0, 0);
+    idLayout->addWidget(m_balsaSceneIDLineEdit);
+    idLayout->addWidget(m_lockUnlockBalseSceneIdToolButton);
+    idLayout->addStretch();
+    
     const Qt::Alignment labelAlignment = (Qt::AlignLeft | Qt::AlignTop);
     
     int32_t columnCounter = 0;
@@ -159,7 +191,9 @@ SceneCreateReplaceDialog::SceneCreateReplaceDialog(const AString& dialogTitle,
     infoGridLayout->addWidget(sceneIDLabel,
                               rowCounter, labelColumn,
                               labelAlignment);
-    infoGridLayout->addWidget(m_balsaSceneIDLineEdit,
+//    infoGridLayout->addWidget(m_balsaSceneIDLineEdit,
+//                              rowCounter, widgetColumn);
+    infoGridLayout->addLayout(idLayout,
                               rowCounter, widgetColumn);
     rowCounter++;
     infoGridLayout->addWidget(descriptionLabel,
@@ -221,6 +255,14 @@ SceneCreateReplaceDialog::SceneCreateReplaceDialog(const AString& dialogTitle,
             m_sceneWindowDescription = ("Created on " + dataTimeCommitText + "\n");
             m_sceneWindowDescription.appendWithNewLine(windowDescription);
             break;
+        case MODE_EDIT_SCENE_INFO:
+            m_nameLineEdit->setText(sceneToInsertOrReplace->getName());
+            m_descriptionTextEdit->setPlainText(sceneToInsertOrReplace->getDescription());
+            m_balsaSceneIDLineEdit->setText(sceneToInsertOrReplace->getBalsaSceneID());
+            addWindowDescriptionPushButton->setHidden(true);
+            optionsLabel->setHidden(true);
+            optionsWidget->setHidden(true);
+            break;
         case MODE_REPLACE_SCENE:
             m_nameLineEdit->setText(sceneToInsertOrReplace->getName());
             m_balsaSceneIDLineEdit->setText(sceneToInsertOrReplace->getBalsaSceneID());
@@ -232,6 +274,7 @@ SceneCreateReplaceDialog::SceneCreateReplaceDialog(const AString& dialogTitle,
             break;
     }
     
+    
     m_nameLineEdit->setFocus();
     m_nameLineEdit->selectAll();
     
@@ -274,6 +317,56 @@ SceneCreateReplaceDialog::createNewScene(QWidget* parent,
     return scene;
 }
 
+/**
+ * Called when the scene ID lock unlock button is clicked
+ */
+void
+SceneCreateReplaceDialog::lockUnlockBalseSceneIdToolButtonClicked()
+{
+    if (m_balsaSceneIDLineEdit->isReadOnly()) {
+        m_balsaSceneIDLineEdit->setReadOnly(false);
+        const AString text("WARNING: Are you sure you want to edit the BALSA Scene ID?");
+        const AString infoText("Changing the BALSA Scene ID may corrupt the Scene File.  Editing of the "
+                               "Scene ID should only be performed by those with expert knowledge of the BALSA "
+                               "Database.");
+        if (WuQMessageBox::warningYesNo(m_lockUnlockBalseSceneIdToolButton,
+                                        text,
+                                        infoText,
+                                        WuQMessageBox::DefaultButtonYesNo::NO)) {
+            m_lockUnlockBalseSceneIdToolButton->setText("Lock");
+        }
+    }
+    else {
+        m_balsaSceneIDLineEdit->setReadOnly(true);
+        m_lockUnlockBalseSceneIdToolButton->setText("Unlock");
+    }
+}
+
+/**
+ * Static method that creates a dialog for editing a scene's info
+ *
+ * @param parent
+ *     Parent widget on which dialog is displayed.
+ * @param sceneFile
+ *     Scene file to which new scene is added.
+ * @param scene
+ *     Scene that is edited
+ * @return
+ *     Scene that was created or NULL if user cancelled or there was an error.
+ */
+void
+SceneCreateReplaceDialog::editSceneInfo(QWidget* parent,
+                                        SceneFile* sceneFile,
+                                        Scene* scene)
+{
+    SceneCreateReplaceDialog dialog("Edit Scene",
+                                    parent,
+                                    sceneFile,
+                                    MODE_EDIT_SCENE_INFO,
+                                    scene);
+    dialog.exec();
+}
+
 /**
  * Static method that creates a dialog for creating a new scene and
  * returns the scene that was created or NULL if scene was not created.
@@ -344,14 +437,19 @@ SceneCreateReplaceDialog::replaceExistingScene(QWidget* parent,
 }
 
 /**
- * Add an image to the scene.
+ * Add an image to the scene and also info about Workbench.
  *
  * @param scene
  *    Scene to which image is added.
+ * @param cropImageFlag
+ *   If true, crop the image
+ * @param errorMessageOut
+ *   Output with any error messages
  */
 void
-SceneCreateReplaceDialog::addImageToScene(Scene* scene,
-                                          AString& errorMessageOut)
+SceneCreateReplaceDialog::addImageAndWorkbenchInfoToScene(Scene* scene,
+                                                          const bool cropImageFlag,
+                                                          AString& errorMessageOut)
 {
     errorMessageOut.clear();
     
@@ -379,7 +477,7 @@ SceneCreateReplaceDialog::addImageToScene(Scene* scene,
                 errorMessageOut.appendWithNewLine(imageCaptureEvent.getErrorMessage());
             }
             else {
-                imageFiles.push_back(new ImageFile(imageCaptureEvent.getImage()));
+                imageFiles.push_back(new ImageFile(imageCaptureEvent.getCapturedImage()));
                 if ( ! backgroundColorValid) {
                     imageCaptureEvent.getBackgroundColor(backgroundColor);
                     backgroundColorValid = true;
@@ -403,9 +501,11 @@ SceneCreateReplaceDialog::addImageToScene(Scene* scene,
                                                                       backgroundColor);
             
             if (backgroundColorValid) {
-                const int marginSize = 5;
-                compositeImageFile.cropImageRemoveBackground(marginSize,
-                                                             backgroundColor);
+                if (cropImageFlag) {
+                    const int marginSize = 5;
+                    compositeImageFile.cropImageRemoveBackground(marginSize,
+                                                                 backgroundColor);
+                }
             }
             
             const int MAXIMUM_IMAGE_WIDTH = 1024;
@@ -434,6 +534,8 @@ SceneCreateReplaceDialog::addImageToScene(Scene* scene,
          iter++) {
         delete *iter;
     }
+    
+    scene->getSceneInfo()->addWorkbenchVersionInfoToSceneMetaData();
 }
 
 /**
@@ -441,6 +543,8 @@ SceneCreateReplaceDialog::addImageToScene(Scene* scene,
  *
  * @param imageOut
  *     Output image of the scene.
+ * @param cropImageFlag
+ *     If true, crop the image
  * @param errorMessageOut
  *     Contains error information if image was not created.
  * @return
@@ -448,6 +552,7 @@ SceneCreateReplaceDialog::addImageToScene(Scene* scene,
  */
 bool
 SceneCreateReplaceDialog::createSceneImage(QImage& imageOut,
+                                           const bool cropImageFlag,
                                            AString& errorMessageOut)
 {
     bool validImageFlag = false;
@@ -476,7 +581,7 @@ SceneCreateReplaceDialog::createSceneImage(QImage& imageOut,
                 errorMessageOut.appendWithNewLine(imageCaptureEvent.getErrorMessage());
             }
             else {
-                imageFiles.push_back(new ImageFile(imageCaptureEvent.getImage()));
+                imageFiles.push_back(new ImageFile(imageCaptureEvent.getCapturedImage()));
                 if ( ! backgroundColorValid) {
                     imageCaptureEvent.getBackgroundColor(backgroundColor);
                     backgroundColorValid = true;
@@ -500,9 +605,11 @@ SceneCreateReplaceDialog::createSceneImage(QImage& imageOut,
                                                                       backgroundColor);
             
             if (backgroundColorValid) {
-                const int marginSize = 5;
-                compositeImageFile.cropImageRemoveBackground(marginSize,
-                                                             backgroundColor);
+                if (cropImageFlag) {
+                    const int marginSize = 5;
+                    compositeImageFile.cropImageRemoveBackground(marginSize,
+                                                                 backgroundColor);
+                }
             }
             
             const int MAXIMUM_IMAGE_WIDTH = 1024;
@@ -575,6 +682,16 @@ SceneCreateReplaceDialog::createSceneOptionsWidget()
                                          "and the data files with modified palette color mapping do not need "
                                          "to be saved.");
     
+    m_cropImageCheckBox = new QCheckBox("Crop Image");
+    m_cropImageCheckBox->setChecked(s_previousSelections.m_cropImage);
+    WuQtUtilities::setWordWrappedToolTip(m_cropImageCheckBox,
+                                         "Crop the image by removing background pixels on the sides of the "
+                                         "brain models.  "
+                                         "Disabling of image cropping may be useful when "
+                                         "debugging scenes where the content occupies a "
+                                         "portion of the viewing region (data may be "
+                                         "panned/zoomed");
+    
     /*
      * Layout for scene options widgets
      */
@@ -583,6 +700,7 @@ SceneCreateReplaceDialog::createSceneOptionsWidget()
     optionsLayout->addWidget(m_addAllTabsCheckBox);
     optionsLayout->addWidget(m_addAllLoadedFilesCheckBox);
     optionsLayout->addWidget(m_addModifiedPaletteSettingsCheckBox);
+    optionsLayout->addWidget(m_cropImageCheckBox);
     
     /*
      * Add the layout to a widget and return the widget.
@@ -606,6 +724,7 @@ SceneCreateReplaceDialog::okButtonClicked()
     s_previousSelections.m_addAllTabs                 = m_addAllTabsCheckBox->isChecked();
     s_previousSelections.m_addModifiedPaletteSettings = m_addModifiedPaletteSettingsCheckBox->isChecked();
     s_previousSelections.m_addSpecFileNameToScene     = m_addSpecFileNameToSceneCheckBox->isChecked();
+    s_previousSelections.m_cropImage                  = m_cropImageCheckBox->isChecked();
     
     const AString newSceneName = m_nameLineEdit->text();
     
@@ -620,6 +739,11 @@ SceneCreateReplaceDialog::okButtonClicked()
             switch (m_mode) {
                 case MODE_ADD_NEW_SCENE:
                     break;
+                case MODE_EDIT_SCENE_INFO:
+                    if (m_sceneToInsertOrReplace == sceneWithName) {
+                        nameErrorFlag = false;
+                    }
+                    break;
                 case MODE_INSERT_NEW_SCENE:
                     break;
                 case MODE_REPLACE_SCENE:
@@ -642,7 +766,25 @@ SceneCreateReplaceDialog::okButtonClicked()
                                errorMessage);
         return;
     }
-    
+
+    switch (m_mode) {
+        case MODE_ADD_NEW_SCENE:
+            break;
+        case MODE_EDIT_SCENE_INFO:
+        {
+            m_sceneToInsertOrReplace->setName(newSceneName);
+            m_sceneToInsertOrReplace->setDescription(m_descriptionTextEdit->toPlainText());
+            m_sceneToInsertOrReplace->setBalsaSceneID(m_balsaSceneIDLineEdit->text().trimmed());
+            WuQDialogModal::okButtonClicked();
+            return;
+        }
+            break;
+        case MODE_INSERT_NEW_SCENE:
+            break;
+        case MODE_REPLACE_SCENE:
+            break;
+    }
+
     if ( ! s_previousSelections.m_addModifiedPaletteSettings) {
         if ( ! SceneDialog::checkForModifiedFiles(GuiManager::TEST_FOR_MODIFIED_FILES_PALETTE_ONLY_MODE_FOR_SCENE_ADD,
                                                   this)) {
@@ -655,6 +797,9 @@ SceneCreateReplaceDialog::okButtonClicked()
         }
     }
     
+    ElapsedTimer timer;
+    timer.start();
+    
     Scene* newScene = new Scene(SceneTypeEnum::SCENE_TYPE_FULL);
     Scene::setSceneBeingCreated(newScene);
     newScene->setName(newSceneName);
@@ -694,17 +839,31 @@ SceneCreateReplaceDialog::okButtonClicked()
                                                       "guiManager"));
     
     AString imageErrorMessage;
-    addImageToScene(newScene,
-                    imageErrorMessage);
+    addImageAndWorkbenchInfoToScene(newScene,
+                                    s_previousSelections.m_cropImage,
+                                    imageErrorMessage);
     if ( ! imageErrorMessage.isEmpty()) {
         WuQMessageBox::errorOk(this,
                                imageErrorMessage);
     }
     
+    /*
+     * Copy macros from active scene to the new scene
+     */
+    EventSceneActive activeSceneEvent(EventSceneActive::MODE_GET);
+    EventManager::get()->sendEvent(activeSceneEvent.getPointer());
+    const Scene* activeScene = activeSceneEvent.getScene();
+    if (activeScene != NULL) {
+        newScene->copyMacrosFromScene(activeScene);
+    }
+    
     switch (m_mode) {
         case MODE_ADD_NEW_SCENE:
             m_sceneFile->addScene(newScene);
             break;
+        case MODE_EDIT_SCENE_INFO:
+            CaretAssert(0);
+            break;
         case MODE_INSERT_NEW_SCENE:
             m_sceneFile->insertScene(newScene,
                                      m_sceneToInsertOrReplace);
@@ -719,6 +878,12 @@ SceneCreateReplaceDialog::okButtonClicked()
     
     Scene::setSceneBeingCreated(NULL);
     
+    CaretLogInfo("Time to create scene "
+                 + newSceneName
+                 + ": "
+                 + AString::number(timer.getElapsedTimeSeconds(), 'f', 3)
+                 + " seconds.");
+    
     WuQDialogModal::okButtonClicked();
 }
 
