@@ -50,8 +50,7 @@ namespace caret {
             EDIT_OPERATION_PROPERTIES
         };
         
-        UserInputModeBorders(Border* borderBeingDrawnByOpenGL,
-                             const int32_t windowIndex);
+        UserInputModeBorders(const int32_t browserIndexIndex);
         
         virtual ~UserInputModeBorders();
         
@@ -71,12 +70,18 @@ namespace caret {
         
         virtual void mouseLeftDragWithCtrlShift(const MouseEvent& mouseEvent);
         
+        virtual void showContextMenu(const MouseEvent& mouseEvent,
+                                     const QPoint& menuPosition,
+                                     BrainOpenGLWidget* openGLWidget);
+        
         virtual CursorEnum::Enum getCursor() const;
 
         virtual AString toString() const;
         
         bool isHighlightBorderEndPoints() const;
         
+        Border* getBorderBeingDrawn() const;
+        
     private:
         /*
          * Some private methods are accessed by this friend class
@@ -117,13 +122,8 @@ namespace caret {
         
         EditOperation editOperation;
         
-        /** 
-         * Pointer to border drawn by OpenGL.  Since owned
-         * by OpenGL, DO NOT delete this!!!
-         */
-        Border* borderBeingDrawnByOpenGL;
+        Border* borderBeingDrawn;
         
-        int32_t windowIndex;
     };
 #ifdef __USER_INPUT_MODE_BORDERS_DECLARE__
     // <PLACE DECLARATIONS OF STATIC MEMBERS HERE>
