@@ -29,6 +29,7 @@ namespace caret {
 
     class BrainOpenGLViewportContent;
     class BrainOpenGLWidget;
+    class Focus;
     class MouseEvent;
     class Surface;
     class UserInputModeFociWidget;
@@ -37,29 +38,37 @@ namespace caret {
         
     public:
         enum Mode {
-            MODE_CREATE,
+            MODE_CREATE_AT_ID,
+            MODE_DELETE,
             MODE_EDIT,
-            MODE_OPERATIONS
+            MODE_MOVE
         };
         
-        enum EditOperation {
-            EDIT_OPERATION_DELETE,
-            EDIT_OPERATION_PROPERTIES
-        };
-        
-        UserInputModeFoci(const int32_t windowIndex);
+        UserInputModeFoci(const int32_t browserIndexIndex);
         
         virtual ~UserInputModeFoci();
         
-        virtual void initialize();
+        virtual void initialize() override;
         
-        virtual void finish();
+        virtual void finish() override;
         
-        virtual void update();
+        virtual void update() override;
         
         virtual CursorEnum::Enum getCursor() const;
         
-        virtual void mouseLeftClick(const MouseEvent& mouseEvent);
+        virtual void mouseLeftClick(const MouseEvent& mouseEvent) override;
+        
+        virtual void mouseMove(const MouseEvent& mouseEvent) override;
+        
+        virtual void mouseLeftDrag(const MouseEvent& mouseEvent) override;
+                
+        virtual void mouseLeftPress(const MouseEvent& mouseEvent) override;
+        
+        virtual void mouseLeftRelease(const MouseEvent& mouseEvent) override;
+
+        virtual void showContextMenu(const MouseEvent& mouseEvent,
+                                     const QPoint& menuPosition,
+                                     BrainOpenGLWidget* openGLWidget) override;
         
     private:
         /*
@@ -77,23 +86,21 @@ namespace caret {
         
         void setMode(const Mode mode);
         
-        EditOperation getEditOperation() const;
-        
-        void setEditOperation(const EditOperation editOperation);
-        
         void updateAfterFociChanged();
         
         Surface* getAnatomicalSurfaceForSurface(Surface* surface);
         
+        void noAnatomicalSurfaceWarningDialog(const Surface* surface) const;
+        
         // ADD_NEW_MEMBERS_HERE
         
-        const int32_t m_windowIndex;
-
         UserInputModeFociWidget* m_inputModeFociWidget;
         
         Mode m_mode;
         
-        EditOperation m_editOperation;
+        Focus* m_focusBeingMovedWithMouse = NULL;
+        
+        Focus* m_focusInMoveModeUnderMouse = NULL;
     };
     
 #ifdef __USER_INPUT_MODE_FOCI_DECLARE__
