@@ -20,11 +20,11 @@
 /*LICENSE_END*/
 
 #include <QAction>
-#include <QActionGroup>
-#include <QComboBox>
+#include <QButtonGroup>
+#include <QGridLayout>
 #include <QHBoxLayout>
 #include <QLabel>
-#include <QStackedWidget>
+#include <QRadioButton>
 #include <QToolButton>
 
 #define __USER_INPUT_MODE_FOCI_WIDGET_DECLARE__
@@ -36,7 +36,7 @@
 #include "BrowserTabContent.h"
 #include "CaretAssert.h"
 #include "DisplayPropertiesFoci.h"
-#include "EventGraphicsUpdateAllWindows.h"
+#include "EventGraphicsPaintSoonAllWindows.h"
 #include "EventManager.h"
 #include "EventUserInterfaceUpdate.h"
 #include "FociFile.h"
@@ -88,27 +88,11 @@ UserInputModeFociWidget::UserInputModeFociWidget(UserInputModeFoci* inputModeFoc
                               );
     
     m_inputModeFoci = inputModeFoci;
-    QLabel* nameLabel = new QLabel("Foci ");
-    
     QWidget* modeWidget = createModeWidget();
     
-    m_createOperationWidget = createCreateOperationWidget();
-    
-    m_editOperationWidget = createEditOperationWidget();
-    
-    m_taskOperationWidget = createTaskOperationWidget();
-    
-    m_operationStackedWidget = new QStackedWidget();
-    m_operationStackedWidget->addWidget(m_createOperationWidget);
-    m_operationStackedWidget->addWidget(m_editOperationWidget);
-    //m_operationStackedWidget->addWidget(m_taskOperationWidget);
-    
-    QHBoxLayout* layout = new QHBoxLayout(this);
+    QVBoxLayout* layout = new QVBoxLayout(this);
     WuQtUtilities::setLayoutSpacingAndMargins(layout, 0, 0);
-    layout->addWidget(nameLabel);
     layout->addWidget(modeWidget);
-    layout->addSpacing(10);
-    layout->addWidget(m_operationStackedWidget);
     layout->addStretch();
 }
 
@@ -130,53 +114,20 @@ UserInputModeFociWidget::updateWidget()
      * Show the proper widget
      */
     switch (m_inputModeFoci->getMode()) {
-        case UserInputModeFoci::MODE_CREATE:
-            m_operationStackedWidget->setCurrentWidget(m_createOperationWidget);
-//            setActionGroupByActionData(m_createOperationActionGroup,
-//                                             m_inputModeFoci->getCreateOperation());
+        case UserInputModeFoci::MODE_CREATE_AT_ID:
+            m_modeCreateLastIdRadioButton->setChecked(true);
+            break;
+        case UserInputModeFoci::MODE_DELETE:
+            m_modeDeleteRadioButton->setChecked(true);
             break;
         case UserInputModeFoci::MODE_EDIT:
-            m_operationStackedWidget->setCurrentWidget(m_editOperationWidget);
-            setActionGroupByActionData(m_editOperationActionGroup,
-                                             m_inputModeFoci->getEditOperation());
+            m_modePropertiesRadioButton->setChecked(true);
             break;
-        case UserInputModeFoci::MODE_OPERATIONS:
-            m_operationStackedWidget->setCurrentWidget(m_taskOperationWidget);
+        case UserInputModeFoci::MODE_MOVE:
+            m_modeMoveRadioButton->setChecked(true);
             break;
+            
     }
-    const int selectedModeInteger = (int)m_inputModeFoci->getMode();
-    
-    const int modeComboBoxIndex = m_modeComboBox->findData(selectedModeInteger);
-    CaretAssert(modeComboBoxIndex >= 0);
-    m_modeComboBox->blockSignals(true);
-    m_modeComboBox->setCurrentIndex(modeComboBoxIndex);
-    m_modeComboBox->blockSignals(false);
-}
-
-/**
- * Set the action with its data value of the given integer
- * as the active action.
- * @param actionGroup
- *   Action group for which action is selected.
- * @param dataInteger
- *   Integer value for data attribute.
- */
-void
-UserInputModeFociWidget::setActionGroupByActionData(QActionGroup* actionGroup,
-                                                       const int dataInteger)
-{
-    actionGroup->blockSignals(true);
-    const QList<QAction*> actionList = actionGroup->actions();
-    QListIterator<QAction*> iter(actionList);
-    while (iter.hasNext()) {
-        QAction* action = iter.next();
-        const int actionDataInteger = action->data().toInt();
-        if (dataInteger == actionDataInteger) {
-            action->setChecked(true);
-            break;
-        }
-    }
-    actionGroup->blockSignals(false);
 }
 
 /**
@@ -185,77 +136,95 @@ UserInputModeFociWidget::setActionGroupByActionData(QActionGroup* actionGroup,
 QWidget*
 UserInputModeFociWidget::createModeWidget()
 {
-    m_modeComboBox = new QComboBox();
-    m_modeComboBox->addItem("Create", (int)UserInputModeFoci::MODE_CREATE);
-    m_modeComboBox->addItem("Edit", (int)UserInputModeFoci::MODE_EDIT);
-//    m_modeComboBox->addItem("Tasks", (int)UserInputModeFoci::MODE_OPERATIONS);
-    QObject::connect(m_modeComboBox, SIGNAL(currentIndexChanged(int)),
-                     this, SLOT(modeComboBoxSelection(int)));
+    const AString newToolTipText = ("Click this button to display a dialog for creating a new focus. "
+                                    + m_transformToolTipText);
+
+    QToolButton* newFocusToolButton = new QToolButton();
+    newFocusToolButton->setText("New Focus...");
+    WuQtUtilities::setToolButtonStyleForQt5Mac(newFocusToolButton);
+    newFocusToolButton->setToolTip(newToolTipText);
+    QObject::connect(newFocusToolButton, &QToolButton::clicked,
+                     this, &UserInputModeFociWidget::createNewFocusActionTriggered);
     
+    const AString lastIDToolTipText = ("While this button is on, clicking the mouse on a surface "
+                                       "will launch the create focus dialog initialized with coordinates "
+                                       "of the selected surface vertex.  "
+                                       + m_transformToolTipText);
+    m_modeCreateLastIdRadioButton = new QRadioButton("Create Last ID");
+    m_modeCreateLastIdRadioButton->setToolTip(lastIDToolTipText);
+    
+    const AString deleteToolTipText = ("While this button is on, clicking the mouse over a focus "
+                                       "will delete the focus.  "
+                                       + m_transformToolTipText);
+    m_modeDeleteRadioButton = new QRadioButton("Delete");
+    m_modeDeleteRadioButton->setToolTip(deleteToolTipText);
+    
+    const AString propertiesToolTipText = ("While this button is on, clicking the mouse over a focus "
+                                           "displays a dialog for editing the focus' properties. "
+                                           + m_transformToolTipText);
+    m_modePropertiesRadioButton   = new QRadioButton("Edit Properties");
+    m_modePropertiesRadioButton->setToolTip(propertiesToolTipText);
+    
+    const AString moveToolTip("<html>Move focus by:"
+                              "<ol>"
+                              "<li> holding down the mouse button over the focus"
+                              "<li> drag the mouse to move the focus"
+                              "<li> release the mouse button when done"
+                              "</ol>"
+                              "<html>");
+    m_modeMoveRadioButton = new QRadioButton("Move");
+    m_modeMoveRadioButton->setToolTip(moveToolTip);
+    
+    QButtonGroup* buttonGroup = new QButtonGroup(this);
+    buttonGroup->addButton(m_modeCreateLastIdRadioButton);
+    buttonGroup->addButton(m_modeDeleteRadioButton);
+    buttonGroup->addButton(m_modePropertiesRadioButton);
+    buttonGroup->addButton(m_modeMoveRadioButton);
+    QObject::connect(buttonGroup, QOverload<QAbstractButton*>::of(&QButtonGroup::buttonClicked),
+                     this, &UserInputModeFociWidget::modeRadioButtonClicked);
+    
+    QLabel* mouseLabel = new QLabel("Mouse Mode:");
     QWidget* widget = new QWidget();
-    QHBoxLayout* layout = new QHBoxLayout(widget);
-    WuQtUtilities::setLayoutSpacingAndMargins(layout, 2, 0);
-    layout->addWidget(m_modeComboBox);
-    
+    QVBoxLayout* layout = new QVBoxLayout(widget);
+    WuQtUtilities::setLayoutSpacingAndMargins(layout, 4, 2);
+    layout->addWidget(mouseLabel);
+    layout->addWidget(m_modeCreateLastIdRadioButton);
+    layout->addWidget(m_modeDeleteRadioButton);
+    layout->addWidget(m_modePropertiesRadioButton);
+    layout->addWidget(m_modeMoveRadioButton);
+    layout->addSpacing(4);
+    layout->addWidget(WuQtUtilities::createHorizontalLineWidget());
+    layout->addSpacing(4);
+    layout->addWidget(newFocusToolButton);
+
     widget->setFixedWidth(widget->sizeHint().width());
     
     return widget;
 }
 
 /**
- * Called when a mode is selected from the mode combo box.
- * @param indx
- *   Index of item selected.
+ * Called when a mode radio button is selected..
+ * @param button
+ *   button selected.
  */
 void
-UserInputModeFociWidget::modeComboBoxSelection(int indx)
+UserInputModeFociWidget::modeRadioButtonClicked(QAbstractButton* button)
 {
-    const int modeInteger = m_modeComboBox->itemData(indx).toInt();
-    const UserInputModeFoci::Mode mode = (UserInputModeFoci::Mode)modeInteger;
-    m_inputModeFoci->setMode(mode);
-}
-
-/**
- * @return The draw operation widget.
- */
-QWidget*
-UserInputModeFociWidget::createCreateOperationWidget()
-{
-    const AString newToolTipText = ("Press this button to display a dialog for creating a new focus. "
-                            "If the mouse is clicked over a model, the dialog for creating a focus is "
-                            "displayed with the focus' coordinates set to the stereotaxic coordinates at "
-                            "the location of the mouse click."
-                            + m_transformToolTipText);
-    QAction* newFocusAction = WuQtUtilities::createAction("New...",
-                                                          WuQtUtilities::createWordWrappedToolTipText(newToolTipText),
-                                                          this,
-                                                          this,
-                                                          SLOT(createNewFocusActionTriggered()));
-    QToolButton* newFocusToolButton = new QToolButton();
-    newFocusToolButton->setDefaultAction(newFocusAction);
-    
-    const AString lastIDToolTipText = ("Press this button to display a dialog for creating a new focus "
-                                       "with the focus' coordinates set to the stereotaxic location of the "
-                                       "last identification operation.  While in focus mode, an identification "
-                                       "is performed by holding down the Shift key and clicking the mouse."
-                                       + m_transformToolTipText);
-    QAction* lastIdFocusAction = WuQtUtilities::createAction("Last ID",
-                                                          WuQtUtilities::createWordWrappedToolTipText(lastIDToolTipText),
-                                                          this,
-                                                          this,
-                                                          SLOT(createLastIdentificationFocusActionTriggered()));
-    QToolButton* lastIdFocusToolButton = new QToolButton();
-    lastIdFocusToolButton->setDefaultAction(lastIdFocusAction);
-    
-    QWidget* widget = new QWidget();
-    QHBoxLayout* layout = new QHBoxLayout(widget);
-    WuQtUtilities::setLayoutSpacingAndMargins(layout, 2, 0);
-    layout->addWidget(newFocusToolButton);
-    layout->addSpacing(5);
-    layout->addWidget(lastIdFocusToolButton);
-    
-    widget->setFixedWidth(widget->sizeHint().width());
-    return widget;
+    if (button == m_modeCreateLastIdRadioButton) {
+        m_inputModeFoci->setMode(UserInputModeFoci::MODE_CREATE_AT_ID);
+    }
+    else if (button == m_modeDeleteRadioButton) {
+        m_inputModeFoci->setMode(UserInputModeFoci::MODE_DELETE);
+    }
+    else if (button == m_modePropertiesRadioButton) {
+        m_inputModeFoci->setMode(UserInputModeFoci::MODE_EDIT);
+    }
+    else if (button == m_modeMoveRadioButton) {
+        m_inputModeFoci->setMode(UserInputModeFoci::MODE_MOVE);
+    }
+    else {
+        CaretAssert(0);
+    }
 }
 
 /**
@@ -349,15 +318,11 @@ UserInputModeFociWidget::createLastIdentificationFocusActionTriggered()
                 const CaretMappableDataFile* cmdf = dynamic_cast<const CaretMappableDataFile*>(volumeFile);
                 if (brain->isFileValid(cmdf)) {
                     CaretAssert(volumeFile);
-                    int64_t ijk[3];
-                    voxelID->getVoxelIJK(ijk);
-                    float xyz[3];
-                    volumeFile->indexToSpace(ijk, xyz);
-                    
+                    const Vector3D xyz(voxelID->getVoxelXYZ());
                     const AString focusName = ("Last ID "
                                                + cmdf->getFileNameNoPath()
-                                               + " IJK ("
-                                               + AString::fromNumbers(ijk, 3, ",")
+                                               + " XYZ ("
+                                               + AString::fromNumbers(xyz, 3, ",")
                                                + ")");
                     
                     const AString comment = ("Created from "
@@ -376,75 +341,4 @@ UserInputModeFociWidget::createLastIdentificationFocusActionTriggered()
     }
 }
 
-/**
- * @return The edit widget.
- */
-QWidget*
-UserInputModeFociWidget::createEditOperationWidget()
-{
-    const AString deleteToolTipText = ("Delete a focus by clicking the mouse over the focus."
-                               + m_transformToolTipText);
-    QAction* deleteAction = WuQtUtilities::createAction("Delete",
-                                                        WuQtUtilities::createWordWrappedToolTipText(deleteToolTipText),
-                                                        this);
-    deleteAction->setCheckable(true);
-    deleteAction->setData(static_cast<int>(UserInputModeFoci::EDIT_OPERATION_DELETE));
-    QToolButton* deleteToolButton = new QToolButton();
-    deleteToolButton->setDefaultAction(deleteAction);
-    
-    const AString propertiesToolTipText = ("Click the mouse over a focus to display a dialog "
-                                           "for editing the focus' properties."
-                                           + m_transformToolTipText);
-    QAction* propertiesAction = WuQtUtilities::createAction("Properties",
-                                                            WuQtUtilities::createWordWrappedToolTipText(propertiesToolTipText),
-                                                            this);
-    propertiesAction->setCheckable(true);
-    propertiesAction->setData(static_cast<int>(UserInputModeFoci::EDIT_OPERATION_PROPERTIES));
-    QToolButton* propertiesToolButton = new QToolButton();
-    propertiesToolButton->setDefaultAction(propertiesAction);
-    
-    m_editOperationActionGroup = new QActionGroup(this);
-    m_editOperationActionGroup->addAction(deleteAction);
-    m_editOperationActionGroup->addAction(propertiesAction);
-    m_editOperationActionGroup->setExclusive(true);
-    QObject::connect(m_editOperationActionGroup, SIGNAL(triggered(QAction*)),
-                     this, SLOT(editOperationActionTriggered(QAction*)));
-    
-    QWidget* widget = new QWidget();
-    QHBoxLayout* layout = new QHBoxLayout(widget);
-    WuQtUtilities::setLayoutSpacingAndMargins(layout, 2, 0);
-    layout->addWidget(deleteToolButton);
-    layout->addWidget(propertiesToolButton);
-    
-    widget->setFixedWidth(widget->sizeHint().width());
-    return widget;
-}
-
-/**
- * @return The task operation widget.
- */
-QWidget*
-UserInputModeFociWidget::createTaskOperationWidget()
-{
-    QWidget* widget = new QWidget();
-    QHBoxLayout* layout = new QHBoxLayout(widget);
-    WuQtUtilities::setLayoutSpacingAndMargins(layout, 2, 0);
-    
-    widget->setFixedWidth(widget->sizeHint().width());
-    return widget;
-}
-
-/**
- * Called when an edit operation button is selected.
- * @param action
- *     Action that was selected.
- */
-void
-UserInputModeFociWidget::editOperationActionTriggered(QAction* action)
-{
-    const int editModeInteger = action->data().toInt();
-    const UserInputModeFoci::EditOperation editOperation =
-    static_cast<UserInputModeFoci::EditOperation>(editModeInteger);
-    m_inputModeFoci->setEditOperation(editOperation);
-}
 
