@@ -24,37 +24,45 @@
 #undef __USER_INPUT_MODE_VOLUME_EDIT_WIDGET_DECLARE__
 
 #include <QAction>
+#include <QButtonGroup>
 #include <QDoubleSpinBox>
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QLineEdit>
+#include <QRadioButton>
 #include <QSpinBox>
 #include <QToolButton>
 #include <QVBoxLayout>
 
 #include "Brain.h"
+#include "BrainBrowserWindowToolBar.h"
+#include "BrowserTabContent.h"
 #include "CaretAssert.h"
 #include "CaretLogger.h"
 #include "EventDataFileAdd.h"
 #include "EventManager.h"
 #include "EventUpdateVolumeEditingToolBar.h"
 #include "EventUserInterfaceUpdate.h"
+#include "EventBrowserWindowDrawingContent.h"
 #include "GiftiLabel.h"
 #include "GiftiLabelTableEditor.h"
 #include "GuiManager.h"
 #include "Overlay.h"
+#include "OverlaySet.h"
 #include "VolumeFile.h"
 #include "VolumeFileEditorDelegate.h"
 #include "VolumeFileCreateDialog.h"
 #include "WuQDataEntryDialog.h"
 #include "WuQFactory.h"
+#include "WuQHyperlinkToolTip.h"
 #include "WuQMessageBox.h"
+#include "WuQMessageBoxTwo.h"
 #include "WuQSpinBoxOddValue.h"
 #include "WuQtUtilities.h"
 
 using namespace caret;
 
-
+static const QString& helpHyperlink("help://VoxelEditing");
     
 /**
  * \class caret::UserInputModeVolumeEditWidget 
@@ -81,12 +89,20 @@ m_windowIndex(windowIndex)
 {
     CaretAssert(inputModeVolumeEdit);
     
-    QVBoxLayout* layout = new QVBoxLayout(this);
+    QHBoxLayout* layout = new QHBoxLayout(this);
     WuQtUtilities::setLayoutSpacingAndMargins(layout, 2, 0);
-    layout->addWidget(createSelectionToolBar());
-    layout->addWidget(createModeToolBar());
-    setSizePolicy(sizePolicy().horizontalPolicy(), QSizePolicy::Fixed);
-    
+    layout->addWidget(BrainBrowserWindowToolBar::createToolWidget("Voxel Operation",
+                                                                  createOperationWidget(),
+                                                                  BrainBrowserWindowToolBar::WIDGET_PLACEMENT_LEFT,
+                                                                  BrainBrowserWindowToolBar::WIDGET_PLACEMENT_TOP,
+                                                                  0));
+    layout->addWidget(BrainBrowserWindowToolBar::createToolWidget("Options",
+                                                                  createSelectionToolBar(),
+                                                                  BrainBrowserWindowToolBar::WIDGET_PLACEMENT_LEFT,
+                                                                  BrainBrowserWindowToolBar::WIDGET_PLACEMENT_TOP,
+                                                                  0));
+    layout->addStretch();
+  
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_UPDATE_VOLUME_EDITING_TOOLBAR);
 }
 
@@ -126,13 +142,13 @@ UserInputModeVolumeEditWidget::updateWidget()
     bool isValid = false;
     
     UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
-    if (m_inputModeVolumeEdit->getVolumeEditInfo(volumeEditInfo)) {
+    if (m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo)) {
         m_lockAction->setChecked(volumeEditInfo.m_volumeFileEditorDelegate->isLocked(volumeEditInfo.m_mapIndex));
         
         if (volumeEditInfo.m_volumeFile != NULL) {
             if (volumeEditInfo.m_volumeFile->isMappedWithLabelTable()) {
-                m_voxelLabelValueToolButton->setVisible(true);
-                m_voxelFloatValueSpinBox->setVisible(false);
+                m_voxelLabelValueToolButton->setEnabled(true);
+                m_voxelFloatValueSpinBox->setEnabled(false);
 
                 AString buttonText = m_voxelLabelValueAction->text();
                 GiftiLabelTable* labelTable = volumeEditInfo.m_volumeFile->getMapLabelTable(volumeEditInfo.m_mapIndex);
@@ -178,61 +194,82 @@ UserInputModeVolumeEditWidget::updateWidget()
                 isValid = true;
             }
             else if (volumeEditInfo.m_volumeFile->isMappedWithPalette()) {
-                m_voxelLabelValueToolButton->setVisible(false);
-                m_voxelFloatValueSpinBox->setVisible(true);
+                m_voxelLabelValueToolButton->setEnabled(false);
+                m_voxelFloatValueSpinBox->setEnabled(true);
                 isValid = true;
             }
+            else if (volumeEditInfo.m_volumeFile->isMappedWithRGBA()) {
+                /* nothing */
+            }
             else {
                 CaretAssert(0);
             }
             
             m_addMapsToolButton->defaultAction()->setEnabled(isValid);
             
-            const bool orthogonalFlag = (volumeEditInfo.m_sliceProjectionType
-                                         == VolumeSliceProjectionTypeEnum::VOLUME_SLICE_PROJECTION_ORTHOGONAL);
-            
-            QList<QAction*> modeActions = m_volumeEditModeActionGroup->actions();
-            const int32_t numModeActions = modeActions.size();
-            for (int32_t i = 0; i < numModeActions; i++) {
-                QAction* action = modeActions.at(i);
-                
-                const int modeInt = action->data().toInt();
-                
-                bool validFlag = false;
-                VolumeEditingModeEnum::Enum mode = VolumeEditingModeEnum::fromIntegerCode(modeInt,
-                                                                                          &validFlag);
-                CaretAssert(validFlag);
-
-                bool modeEnabledFlag = false;
-                
-                if (orthogonalFlag) {
-                    modeEnabledFlag = true;
-                }
-                else {
-                    modeEnabledFlag = VolumeEditingModeEnum::isObliqueEditingAllowed(mode);
-                }
-                
-                action->setEnabled(modeEnabledFlag);
-            }
+            m_modeOnRadioButton->setEnabled(isModeButtonEnabled(VolumeEditingModeEnum::VOLUME_EDITING_MODE_ON));
+            m_modeOffRadioButton->setEnabled(isModeButtonEnabled(VolumeEditingModeEnum::VOLUME_EDITING_MODE_OFF));
+            m_modeDilateRadioButton->setEnabled(isModeButtonEnabled(VolumeEditingModeEnum::VOLUME_EDITING_MODE_DILATE));
+            m_modeErodeRadioButton->setEnabled(isModeButtonEnabled(VolumeEditingModeEnum::VOLUME_EDITING_MODE_ERODE));
+            m_modeFillTwoDimRadioButton->setEnabled(isModeButtonEnabled(VolumeEditingModeEnum::VOLUME_EDITING_MODE_FLOOD_FILL_2D));
+            m_modeFillThreeDimRadioButton->setEnabled(isModeButtonEnabled(VolumeEditingModeEnum::VOLUME_EDITING_MODE_FLOOD_FILL_3D));
+            m_modeRemoveTwoDimRadioButton->setEnabled(isModeButtonEnabled(VolumeEditingModeEnum::VOLUME_EDITING_MODE_REMOVE_CONNECTED_2D));
+            m_modeRemoveThreeDimRadioButton->setEnabled(isModeButtonEnabled(VolumeEditingModeEnum::VOLUME_EDITING_MODE_REMOVE_CONNECTED_3D));
+            m_modeRetainThreeDimRadioButton->setEnabled(isModeButtonEnabled(VolumeEditingModeEnum::VOLUME_EDITING_MODE_RETAIN_CONNECTED_3D));
         }
     }
     
     this->setEnabled(isValid);
 }
 
+/**
+ * @return True if the button for the given mode is valid
+ */
+bool
+UserInputModeVolumeEditWidget::isModeButtonEnabled(const VolumeEditingModeEnum::Enum mode) const
+{
+    bool modeEnabledFlag(false);
+    UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
+    if (m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo)) {
+        m_lockAction->setChecked(volumeEditInfo.m_volumeFileEditorDelegate->isLocked(volumeEditInfo.m_mapIndex));
+        
+        if (volumeEditInfo.m_volumeFile != NULL) {
+            switch (volumeEditInfo.m_sliceProjectionType) {
+                case VolumeSliceProjectionTypeEnum::VOLUME_SLICE_PROJECTION_MPR:
+                    break;
+                case VolumeSliceProjectionTypeEnum::VOLUME_SLICE_PROJECTION_MPR_THREE:
+                    break;
+                case VolumeSliceProjectionTypeEnum::VOLUME_SLICE_PROJECTION_OBLIQUE:
+                    break;
+                case VolumeSliceProjectionTypeEnum::VOLUME_SLICE_PROJECTION_ORTHOGONAL:
+                    modeEnabledFlag = true;
+                    break;
+            }
+            if ( ! modeEnabledFlag) {
+                modeEnabledFlag = VolumeEditingModeEnum::isObliqueEditingAllowed(mode);
+            }
+        }
+    }
+    return modeEnabledFlag;
+}
+                                                
+
 /**
  * @return Create and return the selection toolbar.
  */
 QWidget*
 UserInputModeVolumeEditWidget::createSelectionToolBar()
 {
-    QLabel* volumeLabel = new QLabel("Volume:");
+    QLabel* volumeLabel = new QLabel("File");
     
     m_newFileToolButton = new QToolButton();
     m_newFileToolButton->setDefaultAction(WuQtUtilities::createAction("New",
                                                                     "Create a new volume file that will become the top-most overlay",
                                                                     this,
                                                                     this, SLOT(newFileActionTriggered())));
+    WuQtUtilities::setToolButtonStyleForQt5Mac(m_newFileToolButton);
+    WuQHyperlinkToolTip::addWithHyperlink(m_newFileToolButton,
+                                          helpHyperlink);
     
     m_addMapsToolButton = new QToolButton();
     m_addMapsToolButton->setDefaultAction(WuQtUtilities::createAction("Add",
@@ -240,6 +277,9 @@ UserInputModeVolumeEditWidget::createSelectionToolBar()
                                                                        "First new map will become the top-most overlay."),
                                                                       this,
                                                                       this, SLOT(addMapsActionTriggered())));
+    WuQtUtilities::setToolButtonStyleForQt5Mac(m_addMapsToolButton);
+    WuQHyperlinkToolTip::addWithHyperlink(m_addMapsToolButton,
+                                          helpHyperlink);
     
     m_lockAction = WuQtUtilities::createAction("Lock",
                                                "Lock/unlock volume file to disallow/allow editing",
@@ -248,147 +288,261 @@ UserInputModeVolumeEditWidget::createSelectionToolBar()
     m_lockAction->setCheckable(true);
     QToolButton* lockFileToolButton = new QToolButton();
     lockFileToolButton->setDefaultAction(m_lockAction);
-    
-    
-    QLabel* editLabel = new QLabel("Edit:");
-    
-    QToolButton* undoToolButton = new QToolButton();
-    undoToolButton->setDefaultAction(WuQtUtilities::createAction("Undo",
-                                                                    "Undo the last volume edit",
-                                                                    this,
-                                                                    this, SLOT(undoActionTriggered())));
-    
-    
-    QToolButton* redoToolButton = new QToolButton();
-    redoToolButton->setDefaultAction(WuQtUtilities::createAction("Redo",
-                                                                    "Redo (or is it undo) the last undo",
-                                                                    this,
-                                                                    this, SLOT(redoActionTriggered())));
-    
-    
-    QToolButton* resetToolButton = new QToolButton();
-    resetToolButton->setDefaultAction(WuQtUtilities::createAction("Reset",
-                                                                    "Reset all edting of the volume",
-                                                                    this,
-                                                                    this, SLOT(resetActionTriggered())));
+    WuQtUtilities::setToolButtonStyleForQt5Mac(lockFileToolButton);
+    WuQHyperlinkToolTip::addWithHyperlink(lockFileToolButton,
+                                          m_lockAction,
+                                          helpHyperlink);
 
-    QLabel* brushSizeLabel = new QLabel("Brush Size:");
+    
+
+    QLabel* brushSizeLabel = new QLabel("Brush");
     const int MIN_BRUSH_SIZE = 1;
-    const int MAX_BRUSH_SIZE = 999;
+    const int MAX_BRUSH_SIZE = 9999;
+    
+    QLabel* xLabel = new QLabel("P:");
     m_xBrushSizeSpinBox = new WuQSpinBoxOddValue(this);
     m_xBrushSizeSpinBox->setRange(MIN_BRUSH_SIZE, MAX_BRUSH_SIZE);
+    m_xBrushSizeSpinBox->setSingleStep(2);
     QObject::connect(m_xBrushSizeSpinBox, SIGNAL(valueChanged(int)),
                      this, SLOT(xBrushSizeValueChanged(int)));
     m_xBrushSizeSpinBox->getWidget()->setToolTip("Parasagittal brush size (voxels).\n"
                                                  "Must be an odd value.");
-    
+    WuQHyperlinkToolTip::addWithHyperlink(m_xBrushSizeSpinBox->getWidget(),
+                                          helpHyperlink);
+
+    QLabel* yLabel = new QLabel("C:");
     m_yBrushSizeSpinBox = new WuQSpinBoxOddValue(this);
     m_yBrushSizeSpinBox->setRange(MIN_BRUSH_SIZE, MAX_BRUSH_SIZE);
+    m_yBrushSizeSpinBox->setSingleStep(2);
     QObject::connect(m_yBrushSizeSpinBox, SIGNAL(valueChanged(int)),
                      this, SLOT(yBrushSizeValueChanged(int)));
     m_yBrushSizeSpinBox->getWidget()->setToolTip("Coronal brush size (voxels).\n"
                                                  "Must be an odd value.");
-    
+    WuQHyperlinkToolTip::addWithHyperlink(m_yBrushSizeSpinBox->getWidget(),
+                                          helpHyperlink);
+
+    QLabel* zLabel = new QLabel("A:");
     m_zBrushSizeSpinBox = new WuQSpinBoxOddValue(this);
     m_zBrushSizeSpinBox->setRange(MIN_BRUSH_SIZE, MAX_BRUSH_SIZE);
+    m_zBrushSizeSpinBox->setSingleStep(2);
     QObject::connect(m_zBrushSizeSpinBox, SIGNAL(valueChanged(int)),
                      this, SLOT(zBrushSizeValueChanged(int)));
     m_zBrushSizeSpinBox->getWidget()->setToolTip("Axial brush size (voxels).\n"
                                                  "Must be an odd value.");
-    
-    m_voxelValueLabel = new QLabel("Value:");
+    WuQHyperlinkToolTip::addWithHyperlink(m_zBrushSizeSpinBox->getWidget(),
+                                          helpHyperlink);
+
+    m_voxelValueLabel = new QLabel("Value");
     m_voxelFloatValueSpinBox = WuQFactory::newDoubleSpinBoxWithMinMaxStepDecimalsSignalDouble(-1000.0,
                                                                                          1000.0,
                                                                                          1.0, 1,
                                                                                          this,
                                                                                          SLOT(voxelValueChanged(double)));
     m_voxelFloatValueSpinBox->setValue(1.0);
+    m_voxelFloatValueSpinBox->setToolTip("Set value for functional volume editing");
+    WuQHyperlinkToolTip::addWithHyperlink(m_voxelFloatValueSpinBox,
+                                          helpHyperlink);
 
-    m_voxelLabelValueAction = WuQtUtilities::createAction("XXX",
+    m_voxelLabelValueAction = WuQtUtilities::createAction("Label",
                                                           "Choose Label for Voxels",
                                                           this,
                                                           this, SLOT(labelValueActionTriggered()));
     m_voxelLabelValueToolButton = new QToolButton();
     m_voxelLabelValueToolButton->setDefaultAction(m_voxelLabelValueAction);
+    WuQtUtilities::setToolButtonStyleForQt5Mac(m_voxelLabelValueToolButton);
+    WuQHyperlinkToolTip::addWithHyperlink(m_voxelLabelValueToolButton,
+                                          helpHyperlink);
+
+    m_voxelLabelValueToolButton->setFixedWidth(m_voxelLabelValueToolButton->sizeHint().width());
+    m_voxelFloatValueSpinBox->setFixedWidth(m_voxelLabelValueToolButton->sizeHint().width());
     
-    const int SPACE = 10;
+    WuQtUtilities::matchWidgetWidths(m_newFileToolButton, m_addMapsToolButton, lockFileToolButton);
+    const int32_t spinBoxWidth(20 + m_newFileToolButton->sizeHint().width());
+    m_xBrushSizeSpinBox->setFixedWidth(spinBoxWidth);
+    m_yBrushSizeSpinBox->setFixedWidth(spinBoxWidth);
+    m_zBrushSizeSpinBox->setFixedWidth(spinBoxWidth);
+    m_voxelFloatValueSpinBox->setFixedWidth(spinBoxWidth);
+    
+    QGridLayout* gridLayout = new QGridLayout();
+    WuQtUtilities::setLayoutSpacingAndMargins(gridLayout, 4, 2);
+    int32_t row(0);
+    int32_t col(0);
+    gridLayout->addWidget(volumeLabel, row++, col, Qt::AlignHCenter);
+    gridLayout->addWidget(m_newFileToolButton, row++, col);
+    gridLayout->addWidget(m_addMapsToolButton, row++, col);
+    gridLayout->addWidget(lockFileToolButton, row++, col);
+    row = 0;
+    col++;
+    gridLayout->addWidget(brushSizeLabel, row++, col, 1, 2, Qt::AlignHCenter);
+    gridLayout->addWidget(xLabel, row++, col);
+    gridLayout->addWidget(yLabel, row++, col);
+    gridLayout->addWidget(zLabel, row++, col);
+    row = 0;
+    col++;
+    gridLayout->addWidget(brushSizeLabel, row++, col, Qt::AlignHCenter);
+    gridLayout->addWidget(m_xBrushSizeSpinBox->getWidget(), row++, col);
+    gridLayout->addWidget(m_yBrushSizeSpinBox->getWidget(), row++, col);
+    gridLayout->addWidget(m_zBrushSizeSpinBox->getWidget(), row++, col);
+    row = 0;
+    col++;
+    gridLayout->addWidget(m_voxelValueLabel, row++, col, Qt::AlignHCenter);
+    gridLayout->addWidget(m_voxelFloatValueSpinBox, row++, col);
+    gridLayout->addWidget(m_voxelLabelValueToolButton, row++, col);
+
     QWidget* widget = new QWidget();
-    QHBoxLayout* layout = new QHBoxLayout(widget);
+    QVBoxLayout* layout = new QVBoxLayout(widget);
     WuQtUtilities::setLayoutSpacingAndMargins(layout, 2, 0);
-    layout->addWidget(volumeLabel);
-    layout->addWidget(m_newFileToolButton);
-    layout->addWidget(m_addMapsToolButton);
-    layout->addWidget(lockFileToolButton);
-    layout->addSpacing(SPACE);
-    layout->addWidget(editLabel);
-    layout->addWidget(undoToolButton);
-    layout->addWidget(redoToolButton);
-    layout->addWidget(resetToolButton);
-    layout->addSpacing(SPACE);
-    layout->addWidget(brushSizeLabel);
-    layout->addWidget(m_xBrushSizeSpinBox->getWidget());
-    layout->addWidget(m_yBrushSizeSpinBox->getWidget());
-    layout->addWidget(m_zBrushSizeSpinBox->getWidget());
-    layout->addSpacing(SPACE);
-    layout->addWidget(m_voxelValueLabel);
-    layout->addWidget(m_voxelFloatValueSpinBox);
-    layout->addWidget(m_voxelLabelValueToolButton);
+    layout->addLayout(gridLayout);
     layout->addStretch();
-    
+
     return widget;
 }
 
+/**
+ * @return Create and return the edit widget.
+ */
+QWidget*
+UserInputModeVolumeEditWidget::createEditWidget()
+{
+    QToolButton* undoToolButton = new QToolButton();
+    undoToolButton->setDefaultAction(WuQtUtilities::createAction("Undo",
+                                                                 "Undo the last volume edit",
+                                                                 this,
+                                                                 this, SLOT(undoActionTriggered())));
+    WuQtUtilities::setToolButtonStyleForQt5Mac(undoToolButton);
+    WuQHyperlinkToolTip::addWithHyperlink(undoToolButton,
+                                          helpHyperlink);
+
+    
+    QToolButton* redoToolButton = new QToolButton();
+    redoToolButton->setDefaultAction(WuQtUtilities::createAction("Redo",
+                                                                 "Redo (or is it undo) the last undo",
+                                                                 this,
+                                                                 this, SLOT(redoActionTriggered())));
+    WuQtUtilities::setToolButtonStyleForQt5Mac(redoToolButton);
+    WuQHyperlinkToolTip::addWithHyperlink(redoToolButton,
+                                          helpHyperlink);
+    
+    QToolButton* resetToolButton = new QToolButton();
+    resetToolButton->setDefaultAction(WuQtUtilities::createAction("Reset",
+                                                                  "Reset all edting of the volume",
+                                                                  this,
+                                                                  this, SLOT(resetActionTriggered())));
+    WuQtUtilities::setToolButtonStyleForQt5Mac(resetToolButton);
+    WuQHyperlinkToolTip::addWithHyperlink(resetToolButton,
+                                          helpHyperlink);
+
+    QWidget* widget = new QWidget();
+    QHBoxLayout* editLayout = new QHBoxLayout(widget);
+    WuQtUtilities::setLayoutSpacingAndMargins(editLayout, 4, 2);
+    editLayout->addStretch();
+    editLayout->addWidget(undoToolButton);
+    editLayout->addWidget(redoToolButton);
+    editLayout->addWidget(resetToolButton);
+    editLayout->addStretch();
+
+    return widget;
+}
+
+/**
+ * @return Radio button for selection of the given mode
+ *  @mode The editing mode
+ */
+QRadioButton*
+UserInputModeVolumeEditWidget::createModeRadioButton(const VolumeEditingModeEnum::Enum mode)
+{
+    QRadioButton* rb = new QRadioButton(VolumeEditingModeEnum::toGuiName(mode));
+    WuQtUtilities::setWordWrappedToolTip(rb, VolumeEditingModeEnum::toToolTip(mode));
+    WuQHyperlinkToolTip::addWithHyperlink(rb,
+                                          helpHyperlink);
+    
+
+    return rb;
+}
 /**
  * @return Create and return the mode toolbar.
  */
 QWidget*
-UserInputModeVolumeEditWidget::createModeToolBar()
+UserInputModeVolumeEditWidget::createOperationWidget()
 {
-    QWidget* widget = new QWidget();
-    QHBoxLayout* layout = new QHBoxLayout(widget);
-    WuQtUtilities::setLayoutSpacingAndMargins(layout, 2, 0);
+    m_modeOnRadioButton = createModeRadioButton(VolumeEditingModeEnum::VOLUME_EDITING_MODE_ON);
+    m_modeOnRadioButton->setChecked(true);
     
-    QLabel* modeLabel = new QLabel("Mode:");
-    layout->addWidget(modeLabel);
+    m_modeOffRadioButton = createModeRadioButton(VolumeEditingModeEnum::VOLUME_EDITING_MODE_OFF);
     
-    m_volumeEditModeActionGroup = new QActionGroup(this);
-    m_volumeEditModeActionGroup->setExclusive(true);
-    QObject::connect(m_volumeEditModeActionGroup, SIGNAL(triggered(QAction*)),
-                     this, SLOT(editingModeActionTriggered(QAction*)));
+    m_modeDilateRadioButton = createModeRadioButton(VolumeEditingModeEnum::VOLUME_EDITING_MODE_DILATE);
     
-    std::vector<VolumeEditingModeEnum::Enum> editModes;
-    VolumeEditingModeEnum::getAllEnums(editModes);
+    m_modeErodeRadioButton = createModeRadioButton(VolumeEditingModeEnum::VOLUME_EDITING_MODE_ERODE);
     
-    bool firstActionFlag = true;
-    for (std::vector<VolumeEditingModeEnum::Enum>::iterator iter = editModes.begin();
-         iter != editModes.end();
-         iter++) {
-        const VolumeEditingModeEnum::Enum mode = *iter;
-        
-        const int     modeInt  = VolumeEditingModeEnum::toIntegerCode(mode);
-        const AString modeName = VolumeEditingModeEnum::toGuiName(mode);
-        const AString toolTip  = WuQtUtilities::createWordWrappedToolTipText(VolumeEditingModeEnum::toToolTip(mode));
-        
-        QAction* action = new QAction(modeName,
-                                      this);
-        action->setData(modeInt);
-        action->setToolTip(toolTip);
-        action->setCheckable(true);
-        if (firstActionFlag) {
-            firstActionFlag = false;
-            action->setChecked(true);
-        }
-        
-        m_volumeEditModeActionGroup->addAction(action);
-        
-        QToolButton* toolButton = new QToolButton();
-        toolButton->setDefaultAction(action);
+    m_modeFillTwoDimRadioButton = createModeRadioButton(VolumeEditingModeEnum::VOLUME_EDITING_MODE_FLOOD_FILL_2D);
+    
+    m_modeFillThreeDimRadioButton = createModeRadioButton(VolumeEditingModeEnum::VOLUME_EDITING_MODE_FLOOD_FILL_3D);
+    
+    m_modeRemoveTwoDimRadioButton = createModeRadioButton(VolumeEditingModeEnum::VOLUME_EDITING_MODE_REMOVE_CONNECTED_2D);
+    
+    m_modeRemoveThreeDimRadioButton = createModeRadioButton(VolumeEditingModeEnum::VOLUME_EDITING_MODE_REMOVE_CONNECTED_3D);
+    
+    m_modeRetainThreeDimRadioButton = createModeRadioButton(VolumeEditingModeEnum::VOLUME_EDITING_MODE_RETAIN_CONNECTED_3D);
+    
+    QButtonGroup* buttonGroup = new QButtonGroup(this);
+    buttonGroup->addButton(m_modeOnRadioButton);
+    buttonGroup->addButton(m_modeOffRadioButton);
+    buttonGroup->addButton(m_modeDilateRadioButton);
+    buttonGroup->addButton(m_modeErodeRadioButton);
+    buttonGroup->addButton(m_modeFillTwoDimRadioButton);
+    buttonGroup->addButton(m_modeFillThreeDimRadioButton);
+    buttonGroup->addButton(m_modeRemoveTwoDimRadioButton);
+    buttonGroup->addButton(m_modeRemoveThreeDimRadioButton);
+    buttonGroup->addButton(m_modeRetainThreeDimRadioButton);
 
-        layout->addWidget(toolButton);
-    }
+    QGridLayout* gridLayout = new QGridLayout();
+#ifdef CARET_OS_LINUX
+    WuQtUtilities::setLayoutSpacingAndMargins(gridLayout, 0, 2);
+#else
+    WuQtUtilities::setLayoutSpacingAndMargins(gridLayout, 4, 2);
+#endif
+    int32_t row(0);
+    gridLayout->addWidget(m_modeOnRadioButton,
+                          row, 0);
+    row++;
+    gridLayout->addWidget(m_modeOffRadioButton,
+                          row, 0);
+    row++;
+    gridLayout->addWidget(m_modeDilateRadioButton,
+                          row, 0);
+    row++;
+    gridLayout->addWidget(m_modeErodeRadioButton,
+                          row, 0);
+    row++;
     
+    row = 0;
+    gridLayout->addWidget(m_modeFillTwoDimRadioButton,
+                          row, 1);
+    row++;
+    gridLayout->addWidget(m_modeFillThreeDimRadioButton,
+                          row, 1);
+    row++;
+    gridLayout->addWidget(m_modeRemoveTwoDimRadioButton,
+                          row, 1);
+    row++;
+    gridLayout->addWidget(m_modeRemoveThreeDimRadioButton,
+                          row, 1);
+    row++;
+    gridLayout->addWidget(m_modeRetainThreeDimRadioButton,
+                          row, 1);
+    row++;
+    
+    QWidget* widget = new QWidget();
+    QVBoxLayout* layout = new QVBoxLayout(widget);
+#ifdef CARET_OS_LINUX
+    WuQtUtilities::setLayoutSpacingAndMargins(layout, 0, 0);
+#else
+    WuQtUtilities::setLayoutSpacingAndMargins(layout, 4, 0);
+#endif
+    layout->addLayout(gridLayout);
+    layout->addWidget(createEditWidget());
     layout->addStretch();
-    
+
     return widget;
 }
 
@@ -425,15 +579,38 @@ UserInputModeVolumeEditWidget::getEditingParameters(VolumeEditingModeEnum::Enum&
 VolumeEditingModeEnum::Enum
 UserInputModeVolumeEditWidget::getEditingMode() const
 {
-    QAction* action = m_volumeEditModeActionGroup->checkedAction();
-    CaretAssert(action);
+    VolumeEditingModeEnum::Enum editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_ON;
     
-    const int modeInt = action->data().toInt();
-    
-    bool validFlag = false;
-    const VolumeEditingModeEnum::Enum editMode = VolumeEditingModeEnum::fromIntegerCode(modeInt,
-                                                                                        &validFlag);
-    CaretAssert(validFlag);
+    if (m_modeOnRadioButton->isChecked()) {
+        editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_ON;
+    }
+    else if (m_modeOffRadioButton->isChecked()) {
+        editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_OFF;
+    }
+    else if (m_modeDilateRadioButton->isChecked()) {
+        editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_DILATE;
+    }
+    else if (m_modeErodeRadioButton->isChecked()) {
+        editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_ERODE;
+    }
+    else if (m_modeFillTwoDimRadioButton->isChecked()) {
+        editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_FLOOD_FILL_2D;
+    }
+    else if (m_modeFillThreeDimRadioButton->isChecked()) {
+        editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_FLOOD_FILL_3D;
+    }
+    else if (m_modeRemoveTwoDimRadioButton->isChecked()) {
+        editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_REMOVE_CONNECTED_2D;
+    }
+    else if (m_modeRemoveThreeDimRadioButton->isChecked()) {
+        editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_REMOVE_CONNECTED_3D;
+    }
+    else if (m_modeRetainThreeDimRadioButton->isChecked()) {
+        editMode = VolumeEditingModeEnum::VOLUME_EDITING_MODE_RETAIN_CONNECTED_3D;
+    }
+    else {
+        CaretAssert(0);
+    }
     
     return editMode;
 }
@@ -446,7 +623,14 @@ UserInputModeVolumeEditWidget::getEditingMode() const
 void
 UserInputModeVolumeEditWidget::newFileActionTriggered()
 {
-    VolumeFileCreateDialog newVolumeDialog(m_newFileToolButton);
+    VolumeMappableInterface* underlayVolume(NULL);
+    UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
+    if (m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo)) {
+        underlayVolume = volumeEditInfo.m_underlayVolume;
+    }
+    
+    VolumeFileCreateDialog newVolumeDialog(underlayVolume,
+                                           m_newFileToolButton);
     if (newVolumeDialog.exec() == VolumeFileCreateDialog::Accepted) {
         VolumeFile* vf = newVolumeDialog.getVolumeFile();
         if (vf != NULL) {
@@ -458,24 +642,46 @@ UserInputModeVolumeEditWidget::newFileActionTriggered()
             EventDataFileAdd addFileEvent(vf);
             EventManager::get()->sendEvent(addFileEvent.getPointer());
             
-            const int32_t mapIndex = 0;
-            
-            UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
-            m_inputModeVolumeEdit->getVolumeEditInfo(volumeEditInfo);
-            if (volumeEditInfo.m_topOverlay != NULL) {
-                volumeEditInfo.m_topOverlay->setSelectionData(vf,
-                                                              mapIndex);
-                volumeEditInfo.m_topOverlay->setEnabled(true);
-                volumeEditInfo.m_topOverlay->setMapYokingGroup(MapYokingGroupEnum::MAP_YOKING_GROUP_OFF);
-                m_inputModeVolumeEdit->updateGraphicsAfterEditing(vf,
-                                                                  mapIndex);
-            }
-            
-            EventManager::get()->sendEvent(EventUserInterfaceUpdate().getPointer());
+            viewVolumeInNewOverlay(vf,
+                                   0);
         }
     }
 }
 
+/**
+ * View the volume in a new overlay at the top of the overlays
+ *
+ * @param vf
+ *     The volume file.
+ * @param mapIndex
+ *     Select this map index in overlay.
+ */
+void
+UserInputModeVolumeEditWidget::viewVolumeInNewOverlay(VolumeFile* vf,
+                                                      const int32_t mapIndex)
+{
+    UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
+    m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo);
+    if (volumeEditInfo.m_topOverlay != NULL) {
+        /*
+         * Add new overlay and place new volume file in the top most overlay
+         */
+        volumeEditInfo.m_overlaySet->insertOverlayAbove(0);
+        volumeEditInfo.m_topOverlay = volumeEditInfo.m_overlaySet->getPrimaryOverlay();
+        
+        CaretAssert((mapIndex >= 0)
+                    && (mapIndex < vf->getNumberOfMaps()));
+        volumeEditInfo.m_topOverlay->setSelectionData(vf,
+                                                      mapIndex);
+        volumeEditInfo.m_topOverlay->setEnabled(true);
+        volumeEditInfo.m_topOverlay->setMapYokingGroup(MapYokingGroupEnum::MAP_YOKING_GROUP_OFF);
+        m_inputModeVolumeEdit->updateGraphicsAfterEditing();
+    }
+    
+    EventManager::get()->sendEvent(EventUserInterfaceUpdate().getPointer());
+    updateWidget();
+}
+
 /**
  * Called when the add maps action is triggered.
  */
@@ -483,14 +689,16 @@ void
 UserInputModeVolumeEditWidget::addMapsActionTriggered()
 {
     UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
-    if (m_inputModeVolumeEdit->getVolumeEditInfo(volumeEditInfo)) {
+    if (m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo)) {
         VolumeFile* vf = volumeEditInfo.m_volumeFile;
         
-        WuQDataEntryDialog ded("Add Map to Volume File",
+        WuQDataEntryDialog ded("Add Map",
                                m_addMapsToolButton);
+        ded.setTextAtTop(("Add Map to Volume File: "
+                          + vf->getFileNameNoPath()), true);
         const int32_t newMapIndex = vf->getNumberOfMaps();
         QLineEdit* nameLineEdit = ded.addLineEditWidget("Map Name");
-        nameLineEdit->setText("Editing (" + AString::number(newMapIndex + 1) + ")");
+        nameLineEdit->setText("Map " + AString::number(newMapIndex + 1));
         
         if (ded.exec() == WuQDataEntryDialog::Accepted) {
             /*
@@ -503,13 +711,8 @@ UserInputModeVolumeEditWidget::addMapsActionTriggered()
                            nameLineEdit->text().trimmed());
             vf->getVolumeFileEditorDelegate()->setLocked(newMapIndex,
                                                          false);
-            
-            volumeEditInfo.m_volumeOverlay->setSelectionData(vf,
-                                                             newMapIndex);
-            m_inputModeVolumeEdit->updateGraphicsAfterEditing(vf,
-                                                              newMapIndex);
-            EventManager::get()->sendEvent(EventUserInterfaceUpdate().getPointer());
-            updateWidget();
+            viewVolumeInNewOverlay(vf,
+                                   newMapIndex);
         }
     }
 }
@@ -522,7 +725,21 @@ void
 UserInputModeVolumeEditWidget::lockFileActionTriggered()
 {
     UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
-    if (m_inputModeVolumeEdit->getVolumeEditInfo(volumeEditInfo)) {
+    if (m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo)) {
+        /*
+         * Warn user if unlocking and volume is not plumb
+         */
+        if ( ! m_lockAction->isChecked()) {
+            if ( ! volumeEditInfo.m_volumeFile->isPlumb()) {
+                const AString msg("Volume is not aligned to P/C/A Axes.\n"
+                                  "Editing may not function correctly.");
+                const bool result(WuQMessageBox::warningOkCancel(this, msg));
+                if ( ! result) {
+                    m_lockAction->setChecked(true);
+                    return;
+                }
+            }
+        }
         volumeEditInfo.m_volumeFileEditorDelegate->setLocked(volumeEditInfo.m_mapIndex,
                                                              m_lockAction->isChecked());
     }
@@ -535,15 +752,14 @@ void
 UserInputModeVolumeEditWidget::undoActionTriggered()
 {
     UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
-    if (m_inputModeVolumeEdit->getVolumeEditInfo(volumeEditInfo)) {
+    if (m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo)) {
         AString errorMessage;
         if ( ! volumeEditInfo.m_volumeFileEditorDelegate->undo(volumeEditInfo.m_mapIndex,
                                                                errorMessage)) {
             WuQMessageBox::errorOk(this,
                                    errorMessage);
         }
-        m_inputModeVolumeEdit->updateGraphicsAfterEditing(volumeEditInfo.m_volumeFile,
-                                                          volumeEditInfo.m_mapIndex);
+        m_inputModeVolumeEdit->updateGraphicsAfterEditing();
     }
 }
 
@@ -554,15 +770,14 @@ void
 UserInputModeVolumeEditWidget::redoActionTriggered()
 {
     UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
-    if (m_inputModeVolumeEdit->getVolumeEditInfo(volumeEditInfo)) {
+    if (m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo)) {
         AString errorMessage;
         if ( ! volumeEditInfo.m_volumeFileEditorDelegate->redo(volumeEditInfo.m_mapIndex,
                                                                errorMessage)) {
             WuQMessageBox::errorOk(this,
                                    errorMessage);
         }
-        m_inputModeVolumeEdit->updateGraphicsAfterEditing(volumeEditInfo.m_volumeFile,
-                                                          volumeEditInfo.m_mapIndex);
+        m_inputModeVolumeEdit->updateGraphicsAfterEditing();
     }
 }
 
@@ -573,15 +788,14 @@ void
 UserInputModeVolumeEditWidget::resetActionTriggered()
 {
     UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
-    if (m_inputModeVolumeEdit->getVolumeEditInfo(volumeEditInfo)) {
+    if (m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo)) {
         AString errorMessage;
         if ( ! volumeEditInfo.m_volumeFileEditorDelegate->reset(volumeEditInfo.m_mapIndex,
                                                                errorMessage)) {
             WuQMessageBox::errorOk(this,
                                    errorMessage);
         }
-        m_inputModeVolumeEdit->updateGraphicsAfterEditing(volumeEditInfo.m_volumeFile,
-                                                          volumeEditInfo.m_mapIndex);
+        m_inputModeVolumeEdit->updateGraphicsAfterEditing();
     }
 }
 
@@ -628,7 +842,7 @@ void
 UserInputModeVolumeEditWidget::labelValueActionTriggered()
 {
     UserInputModeVolumeEdit::VolumeEditInfo volumeEditInfo;
-    if (m_inputModeVolumeEdit->getVolumeEditInfo(volumeEditInfo)) {
+    if (m_inputModeVolumeEdit->getVolumeEditInfoForStatus(volumeEditInfo)) {
         if (volumeEditInfo.m_volumeFile != NULL) {
             GiftiLabelTableEditor lte(volumeEditInfo.m_volumeFile,
                                       volumeEditInfo.m_mapIndex,
@@ -642,6 +856,13 @@ UserInputModeVolumeEditWidget::labelValueActionTriggered()
             if (lte.exec() == GiftiLabelTableEditor::Accepted) {
                 const AString selectedName = lte.getLastSelectedLabelName();
                 m_voxelLabelValueAction->setText(selectedName);
+                
+                const AString msg("If a new label was added, the Hierarchy view in "
+                                  "Label's tab of the Features Toolbox will NOT update "
+                                  "(The List view will update).  After editing, saving and reloading "
+                                  "the volume label file will update the Hierarchy view.  "
+                                  "If you are NOT using the Hierarchy view, no action is needed.");
+                WuQMessageBoxTwo::information(this, "Information", msg);
             }
         }
     }
