@@ -72,6 +72,8 @@
 #include <QMessageBox>
 #include <QPixmap>
 #include <QPushButton>
+#include <QRegularExpression>
+#include <QScreen>
 #include <QScrollArea>
 #include <QScrollBar>
 #include <QTimer>
@@ -231,21 +233,52 @@ WuQDialog::setStandardButtonText(QDialogButtonBox::StandardButton button,
     }    
 }
 
+/**
+ * Set the enabled status of a standard button.
+ * If the button is invalid, a sever message is logged.
+ *
+ * @param button
+ *    Standard button enum identifying button.
+ * @param enabled
+ *    New enabled status for button.
+ */
+void
+WuQDialog::setStandardButtonEnabled(QDialogButtonBox::StandardButton button,
+                                    const bool enabled)
+{
+    QPushButton* pushButton = this->buttonBox->button(button);
+    if (pushButton != NULL) {
+        pushButton->setEnabled(enabled);
+    }
+    else {
+        CaretLogSevere("PROGRAM ERROR: Attempted to set enabled status for an invalid Dialog Standard Button, id="
+                       + AString::number((int)button));
+    }
+}
+
 /**
  * called to capture image after timeout so nothing obscures window.
  */
 void 
 WuQDialog::slotCaptureImageAfterTimeOut()
 {
-   QImage image = QPixmap::grabWindow(this->winId()).toImage();
-   if (image.isNull() == false) {
-      QClipboard* clipboard = QApplication::clipboard();
-      clipboard->setImage(image);
-      
-      QMessageBox::information(this,
-          "Information",
-          "An image of this dialog has been placed onto the computer's clipboard.");
-   }
+    QPixmap pixmap = this->grab();
+    if ( ! pixmap.isNull()) {
+        QImage image = pixmap.toImage();
+        if ( ! image.isNull()) {
+            QClipboard* clipboard = QApplication::clipboard();
+            clipboard->setImage(image);
+            
+            QMessageBox::information(this,
+                                     "Information",
+                                     "An image of this dialog has been placed onto the computer's clipboard.");
+            return;
+        }
+    }
+    
+    QMessageBox::warning(this,
+                         "Failed",
+                         "Failed to create image of the dialog.");
 }
 
 /**
@@ -322,7 +355,7 @@ WuQDialog::keyPressEvent(QKeyEvent* e)
     //   Calls reject() if Escape is pressed. Simulates a button
     //   click for the default button if Enter is pressed. Move focus
     //   for the arrow keys. Ignore the rest.
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     if(e->modifiers() == Qt::ControlModifier && e->key() == Qt::Key_Period) {
     } else 
 #endif
@@ -519,7 +552,7 @@ WuQDialog::disableAutoDefaultForAllPushButtons()
     /*
      * Disable auto default for all push buttons
      */
-    QList<QPushButton*> allChildPushButtons = findChildren<QPushButton*>(QRegExp(".*"));
+    QList<QPushButton*> allChildPushButtons = findChildren<QPushButton*>(QRegularExpression(".*"));
     QListIterator<QPushButton*> allChildPushButtonsIterator(allChildPushButtons);
     while (allChildPushButtonsIterator.hasNext()) {
         QPushButton* pushButton = allChildPushButtonsIterator.next();
@@ -549,8 +582,16 @@ WuQDialog::helpButtonClicked()
  *     The focus event.
  */
 void
-WuQDialog::focusInEvent(QFocusEvent* /*event*/)
+WuQDialog::focusInEvent(QFocusEvent* event)
 {
+    if (event->reason() == Qt::PopupFocusReason) {
+        /*
+         * This occurs when a combo box is popped up Mac,
+         * causes a problem, and can be ignored.
+         */
+        return;
+    }
+    
     focusGained();
 }
 
@@ -849,7 +890,9 @@ WuQDialog::clicked(QAbstractButton* button)
 {
     QDialogButtonBox::StandardButton standardButton = this->getDialogButtonBox()->standardButton(button);
     if (standardButton == QDialogButtonBox::Ok) {
+        button->setEnabled(false);
         okButtonClicked();
+        button->setEnabled(true);
     }
     else if (standardButton == QDialogButtonBox::Cancel) {
         cancelButtonClicked();
@@ -864,7 +907,7 @@ WuQDialog::clicked(QAbstractButton* button)
         this->helpButtonClicked();
     }
     else {
-        QPushButton* pushButton = dynamic_cast<QPushButton*>(button);
+        QPushButton* pushButton = qobject_cast<QPushButton*>(button);
         CaretAssert(pushButton);
         const DialogUserButtonResult result = this->userButtonPressed(pushButton);
         switch (result) {
