@@ -22,6 +22,7 @@
 #include "OperationException.h"
 
 #include "CaretAssert.h"
+#include "CaretLogger.h"
 #include "CaretPointer.h"
 #include "CiftiFile.h"
 #include "CiftiXML.h"
@@ -30,11 +31,13 @@
 #include "VolumeFile.h"
 
 #include <fstream>
+#include <cmath>
 #include <string>
 #include <limits>
 #include <vector>
 
 #include <QFile>
+#include <QRegularExpression>
 #include <QStringList>
 
 using namespace caret;
@@ -67,6 +70,7 @@ OperationParameters* OperationCiftiConvert::getParameters()
     OptionalParameter* fgresetTimeunitsOpt = fgresetTimeOpt->createOptionalParameter(3, "-unit", "use a unit other than time");
     fgresetTimeunitsOpt->addStringParameter(1, "unit", "unit identifier (default SECOND)");
     fromGiftiExt->createOptionalParameter(4, "-reset-scalars", "reset mapping along rows to scalars, taking length from the gifti file");
+    fromGiftiExt->createOptionalParameter(6, "-column-reset-scalars", "reset mapping along columns to scalar (useful for changing number of series in a sdseries file)");
     OptionalParameter* fromGiftiReplace = fromGiftiExt->createOptionalParameter(5, "-replace-binary", "replace data with a binary file");
     fromGiftiReplace->addStringParameter(1, "binary-in", "the binary file that contains replacement data");
     fromGiftiReplace->createOptionalParameter(2, "-flip-endian", "byteswap the binary file");
@@ -76,6 +80,7 @@ OperationParameters* OperationCiftiConvert::getParameters()
     toNifti->addCiftiParameter(1, "cifti-in", "the input cifti file");
     toNifti->addVolumeOutputParameter(2, "nifti-out", "the output nifti file");
     toNifti->createOptionalParameter(3, "-smaller-file", "use better-fitting dimension lengths");
+    toNifti->createOptionalParameter(4, "-smaller-dims", "minimize the largest dimension, for tools that don't like large indices");
     
     OptionalParameter* fromNifti = ret->createOptionalParameter(4, "-from-nifti", "convert a NIFTI (1 or 2) file made with this command back into CIFTI");
     fromNifti->addVolumeParameter(1, "nifti-in", "the input nifti file");
@@ -109,11 +114,12 @@ OperationParameters* OperationCiftiConvert::getParameters()
     
     AString myText = AString("This command is used to convert a full CIFTI matrix to/from formats that can be used by programs that don't understand CIFTI.  ") +
         "You must specify exactly one of -to-gifti-ext, -from-gifti-ext, -to-nifti, -from-nifti, -to-text, or -from-text.\n\n" +
+        "This command cannot map surface-based parts of the cifti file to a spatially-correct volume file, or map volume-based data to the surface, see -volume-to-surface-mapping and -metric-to-volume-mapping instead (other commands such as -cifti-separate are also required).\n\n" +
         "If you want to write an existing CIFTI file with a different CIFTI version, see -file-convert, and its -cifti-version-convert option.\n\n" +
         "If you want part of the CIFTI file as a metric, label, or volume file, see -cifti-separate.  " +
         "If you want to create a CIFTI file from metric and/or volume files, see the -cifti-create-* commands.\n\n" +
-        "If you want to import a matrix that is restricted to an ROI, first create a template CIFTI file matching that ROI using a -cifti-create-* command.  " +
-        "After importing to CIFTI, you can then expand the file into a standard brainordinates space with -cifti-create-dense-from-template.  " +
+        "If you want to import a matrix in non-CIFTI format that is restricted to an ROI, first obtain a template CIFTI file matching that ROI (you can use -cifti-create-dense-scalar if you don't have such a file).  " +
+        "Use -cifti-convert to import it to CIFTI format, and you can then expand the file into a standard brainordinates space with -cifti-create-dense-from-template.  " +
         "If you want to export only part of a CIFTI file, first create an roi-restricted CIFTI file with -cifti-restrict-dense-mapping.\n\n" +
         "The -transpose option to -from-gifti-ext is needed if the replacement binary file is in column-major order.\n\n" +
         "The -unit options accept these values:\n";
@@ -126,6 +132,38 @@ OperationParameters* OperationCiftiConvert::getParameters()
     return ret;
 }
 
+namespace
+{
+    
+    bool haveWarned = false;
+    
+    float toFloat(const AString& input)
+    {
+        bool ok = false;
+        double converted = input.toDouble(&ok);
+        if (!ok) throw OperationException("failed to convert text to number: '" + input + "'");
+        float ret = float(converted);//this will turn some non-inf values into +/- inf, so let's fix that
+        if (converted != 0.0 && !std::isinf(converted) && (abs(converted) > numeric_limits<float>::max() || abs(converted) < numeric_limits<float>::denorm_min()))
+        {
+            if (!haveWarned)
+            {
+                CaretLogWarning("input number(s) changed to fit range of float32, first instance: '" + input + "'");
+                haveWarned = true;
+            }
+            if (std::isinf(ret))
+            {
+                if (ret > 0.0f)
+                {
+                    ret = numeric_limits<float>::max();
+                } else {
+                    ret = -numeric_limits<float>::max();
+                }
+            }
+        }
+        return ret;
+    }
+}
+
 void OperationCiftiConvert::useParameters(OperationParameters* myParams, ProgressObject* myProgObj)
 {
     LevelProgress myProgress(myProgObj);
@@ -191,7 +229,7 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
         int64_t numRows = dataArrayRef->getNumberOfRows();
         OptionalParameter* fgresetTimeOpt = fromGiftiExt->getOptionalParameter(3);
         if (fgresetTimeOpt->m_present)
-        {
+        {//-reset-timepoints
             CiftiSeriesMap::Unit myUnit = CiftiSeriesMap::SECOND;
             OptionalParameter* fgresetTimeunitsOpt = fgresetTimeOpt->getOptionalParameter(3);
             if (fgresetTimeunitsOpt->m_present)
@@ -212,12 +250,18 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
             myXML.getSeriesMap(CiftiXML::ALONG_COLUMN).setLength(numRows);
         }
         if (fromGiftiExt->getOptionalParameter(4)->m_present)
-        {
+        {//-reset-scalars
             if (fgresetTimeOpt->m_present) throw OperationException("only one of -reset-timepoints and -reset-scalars may be specified");
             CiftiScalarsMap newMap;
             newMap.setLength(numCols);
             myXML.setMap(CiftiXML::ALONG_ROW, newMap);
         }
+        if (fromGiftiExt->getOptionalParameter(6)->m_present)
+        {//-column-reset-scalars
+            CiftiScalarsMap newMap;
+            newMap.setLength(numRows);
+            myXML.setMap(CiftiXML::ALONG_COLUMN, newMap);
+        }
         if (myXML.getDimensionLength(CiftiXML::ALONG_ROW) != numCols || myXML.getDimensionLength(CiftiXML::ALONG_COLUMN) != numRows)
         {
             throw OperationException("dimensions of input gifti array (" + AString::number(numRows) + " rows, " + AString::number(numCols) + " columns)" +
@@ -295,29 +339,50 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
         if (myCiftiIn->getCiftiXML().getNumberOfDimensions() != 2) throw OperationException("conversion only supported for 2D cifti");
         VolumeFile* myNiftiOut = toNifti->getOutputVolume(2);
         bool betterDims = toNifti->getOptionalParameter(3)->m_present;
+        bool smallerDims = toNifti->getOptionalParameter(4)->m_present;
+        if (betterDims && smallerDims) throw OperationException("-smaller-file and -smaller-dims may not be specified together");
         vector<int64_t> outDims(4, 1);
         outDims[3] = myCiftiIn->getNumberOfColumns();
-        if (outDims[3] > numeric_limits<short>::max()) throw OperationException("cifti rows are too long for nifti1, failing");
-        int64_t numRows = myCiftiIn->getNumberOfRows();
         const int64_t SHORTMAX = numeric_limits<short>::max();//less ugly than writing it out every time
-        if (betterDims)
-        {//find the minimum needed third dimension, then compute from that how many elements the first two dimensions must encompass, etc
-            outDims[2] = (numRows - 1) / (SHORTMAX * SHORTMAX) + 1;//round up
-            if (outDims[2] > SHORTMAX) throw OperationException("too many cifti rows for nifti1 spatial dimensions, failing");
-            int64_t temp = (numRows - 1) / outDims[2] + 1;//and again...
-            outDims[1] = (temp - 1) / SHORTMAX + 1;
-            outDims[0] = (temp - 1) / outDims[1] + 1;
-        } else {
-            int64_t temp = numRows;
-            int index = 0;
-            while (temp > SHORTMAX)
+        if (outDims[3] > SHORTMAX) throw OperationException("cifti rows are too long for nifti-1, failing");
+        int64_t numRows = myCiftiIn->getNumberOfRows();
+        if (numRows > SHORTMAX * SHORTMAX && !(betterDims || smallerDims))
+        {//don't let the default math hit the pathological case of an extra billion elements, behavior change okay because it is likely nobody has hit it yet
+            CaretLogWarning("input has a very large number of rows, and neither -smaller-file or -smaller-dims were specified, using -smaller-file instead");
+            betterDims = true;
+        }
+        if (smallerDims)
+        {
+            outDims[0] = int64_t(ceil(pow(numRows, 1.0f / 3.0f))) - 1;//deliberately start 1 below what floating point says for a cube
+            while (outDims[0] * outDims[0] * outDims[0] < numRows) ++outDims[0];//use integer math to get the exact answer
+            if (outDims[0] > SHORTMAX) throw OperationException("too many cifti rows for nifti-1 spatial dimensions, failing");
+            outDims[1] = outDims[0];
+            outDims[2] = outDims[0];
+            if (outDims[0] * outDims[0] * (outDims[0] - 1) >= numRows)//see whether we can subtract one from a different dimension
             {
-                if (index > 1) throw OperationException("too many cifti rows for nifti1 spatial dimensions, failing");
-                outDims[index] = SHORTMAX;
-                temp = (temp - 1) / SHORTMAX + 1;//round up
-                ++index;
+                outDims[2] = outDims[0] - 1;
+                if (outDims[0] * (outDims[0] - 1) * outDims[2] >= numRows) outDims[1] = outDims[0] - 1;
+            }
+        } else {
+            if (betterDims)
+            {//find the minimum needed third dimension, then compute from that how many elements the first two dimensions must encompass, etc
+                outDims[2] = (numRows - 1) / (SHORTMAX * SHORTMAX) + 1;//round up
+                if (outDims[2] > SHORTMAX) throw OperationException("too many cifti rows for nifti-1 spatial dimensions, failing");
+                int64_t temp = (numRows - 1) / outDims[2] + 1;//and again...
+                outDims[1] = (temp - 1) / SHORTMAX + 1;
+                outDims[0] = (temp - 1) / outDims[1] + 1;
+            } else {
+                int64_t temp = numRows;
+                int index = 0;
+                while (temp > SHORTMAX)
+                {
+                    if (index > 1) throw OperationException("too many cifti rows for nifti-1 spatial dimensions, failing");
+                    outDims[index] = SHORTMAX;
+                    temp = (temp - 1) / SHORTMAX + 1;//round up
+                    ++index;
+                }
+                outDims[index] = temp;
             }
-            outDims[index] = temp;
         }
         CaretAssert(outDims[0] * outDims[1] * outDims[2] >= numRows);//make sure we didn't screw up the math
         myNiftiOut->reinitialize(outDims, FloatMatrix::identity(4).getMatrix());
@@ -357,7 +422,7 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
         if (outXML.getNumberOfDimensions() != 2) throw OperationException("conversion only supported for 2D cifti");
         OptionalParameter* fnresetTimeOpt = fromNifti->getOptionalParameter(4);
         if (fnresetTimeOpt->m_present)
-        {
+        {//-reset-timepoints
             CiftiSeriesMap::Unit myUnit = CiftiSeriesMap::SECOND;
             OptionalParameter* fnresetTimeunitsOpt = fnresetTimeOpt->getOptionalParameter(3);
             if (fnresetTimeunitsOpt->m_present)
@@ -369,7 +434,7 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
             outXML.setMap(CiftiXML::ALONG_ROW, CiftiSeriesMap(myDims[3], fnresetTimeOpt->getDouble(2), fnresetTimeOpt->getDouble(1), myUnit));
         }
         if (fromNifti->getOptionalParameter(5)->m_present)
-        {
+        {//-reset-scalars
             if (fnresetTimeOpt->m_present) throw OperationException("only one of -reset-timepoints and -reset-scalars may be specified");
             CiftiScalarsMap newMap;
             newMap.setLength(myDims[3]);
@@ -386,6 +451,13 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
             throw OperationException("input nifti is too small for column length (need at least " + AString::number(numRows) +
                                      ", product of first three nifti dimensions is " + AString::number(myDims[0] * myDims[1] * myDims[2]) + ")");
         }
+        if (2 * numRows < myDims[0] * myDims[1] * myDims[2])
+        {
+            CaretLogWarning(AString("Input nifti has more than twice as many 'spatial' voxels as the\n") +
+                            "template cifti column length, this is probably not a valid 'fake-nifti' file.\n\nnumber of cifti column elements: " + AString::number(numRows) + "\nproduct of first three nifti dimensions: " + AString::number(myDims[0] * myDims[1] * myDims[2]) + "\n\n" +
+                            "'-cifti-convert -from-nifti' is only intended to be used after '-cifti-convert\n" +
+                            "-to-nifti' (and spatially-naive processing of the produced 'fake-nifti' file).");
+        }
         myCiftiOut->setCiftiXML(outXML);
         vector<float> rowscratch(numCols);
         for (int64_t i = 0; i < numRows; ++i)
@@ -446,9 +518,17 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
         QStringList entries;
         if (delim.isEmpty())
         {
-            entries = QString(templine.c_str()).split(QRegExp("\\s+"), QString::SkipEmptyParts);
+#if QT_VERSION >= 0x060000
+            entries = QString(templine.c_str()).split(QRegularExpression("\\s+"), Qt::SkipEmptyParts);
+#else
+            entries = QString(templine.c_str()).split(QRegularExpression("\\s+"), QString::SkipEmptyParts);
+#endif
         } else {
+#if QT_VERSION >= 0x060000
+            entries = QString(templine.c_str()).split(delim, Qt::SkipEmptyParts);
+#else
             entries = QString(templine.c_str()).split(delim, QString::SkipEmptyParts);
+#endif
         }
         if (numRows < 1) throw OperationException("template cifti file has no data");//this probably throws an exception in CiftiFile, but double check
         int textRowLength = entries.size();
@@ -479,11 +559,9 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
         }
         ciftiOut->setCiftiXML(outXML);
         vector<float> temprow(textRowLength);
-        bool ok = false;
         for (int i = 0; i < textRowLength; ++i)
         {
-            temprow[i] = entries[i].toFloat(&ok);
-            if (!ok) throw OperationException("failed to convert text to number: '" + entries[i] + "'");
+            temprow[i] = toFloat(entries[i]);
         }
         ciftiOut->setRow(temprow.data(), 0);
         for (int64_t j = 1; j < numRows; ++j)
@@ -491,15 +569,22 @@ void OperationCiftiConvert::useParameters(OperationParameters* myParams, Progres
             if (!getline(textIn, templine)) throw OperationException("failed to read from input text file (not enough rows)");
             if (delim.isEmpty())
             {
-                entries = QString(templine.c_str()).split(QRegExp("\\s+"), QString::SkipEmptyParts);
+#if QT_VERSION >= 0x060000
+                entries = QString(templine.c_str()).split(QRegularExpression("\\s+"), Qt::SkipEmptyParts);
+#else
+                entries = QString(templine.c_str()).split(QRegularExpression("\\s+"), QString::SkipEmptyParts);
+#endif
             } else {
+#if QT_VERSION >= 0x060000
+                entries = QString(templine.c_str()).split(delim, Qt::SkipEmptyParts);
+#else
                 entries = QString(templine.c_str()).split(delim, QString::SkipEmptyParts);
+#endif
             }
             if (entries.size() != textRowLength) throw OperationException("text file has inconsistent line length");
             for (int i = 0; i < textRowLength; ++i)
             {
-                temprow[i] = entries[i].toFloat(&ok);
-                if (!ok) throw OperationException("failed to convert text to number: '" + entries[i] + "'");
+                temprow[i] = toFloat(entries[i]);
             }
             ciftiOut->setRow(temprow.data(), j);
         }
