@@ -21,11 +21,18 @@
 #include "OperationVolumeLabelImport.h"
 #include "OperationException.h"
 
+#include "CaretHierarchy.h"
 #include "CaretLogger.h"
 #include "FileInformation.h"
 #include "GiftiLabel.h"
 #include "VolumeFile.h"
 
+#include <QFile>
+#include <QIODevice>
+#include <QJsonArray>
+#include <QJsonDocument>
+#include <QJsonObject>
+
 #include <cmath>
 #include <cstdlib>
 #include <cctype>
@@ -66,19 +73,23 @@ OperationParameters* OperationVolumeLabelImport::getParameters()
     
     ret->createOptionalParameter(7, "-drop-unused-labels", "remove any unused label values from the label table");
     
+    OptionalParameter* hierOpt = ret->createOptionalParameter(8, "-hierarchy", "read label name hierarchy from a json file");
+    hierOpt->addStringParameter(1, "file", "the input json file");
+    
     ret->setHelpText(
-        AString("Creates a new label volume from an integer-valued volume file.  ") +
+        AString("Creates a label volume from an integer-valued volume file.  ") +
         "The label name and color information is stored in the volume header in a nifti extension, with a similar format as in caret5, see -volume-help.  " +
-        "You may specify the empty string ('' will work on linux/mac) for <label-list-file>, which will be treated as if it is an empty file.  " +
-        "The label list file must have pairs of lines of the following format:\n\n" +
-        "<labelname>\n<value> <red> <green> <blue> <alpha>\n\n" +
-        "Do not specify the \"unlabeled\" key in the file, it is assumed that 0 means not labeled unless -unlabeled-value is specified.  " +
-        "Label names must be on a separate line, but may contain spaces or other unusual characters (but not newline).  " +
+        "You may specify the empty string (use \"\") for <label-list-file>, which will be treated as if it is an empty file.  " +
+        "The label list file must have the following format (2 lines per label):\n\n" +
+        "<labelname>\n<key> <red> <green> <blue> <alpha>\n...\n\n" +
+        "Label names are specified on a separate line from their value and color, in order to let label names contain spaces.  " +
         "Whitespace is trimmed from both ends of the label name, but is kept if it is in the middle of a label.  " +
-        "The values of red, green, blue and alpha must be integers from 0 to 255, and will specify the color the label is drawn as " +
-        "(alpha of 255 means opaque, which is probably what you want).  " +
-        "By default, it will set new label names with names of LABEL_# for any values encountered that are not mentioned in the " +
-        "list file, specify -discard-others to instead set these voxels to the \"unlabeled\" key."
+        "Do not specify the \"unlabeled\" key in the file, it is assumed that 0 means not labeled unless -unlabeled-value is specified.  " +
+        "The value of <key> specifies what value in the imported file should be used as this label (these same key values are also used in the output file).  " +
+        "The values of <red>, <green>, <blue> and <alpha> must be integers from 0 to 255, and will specify the color the label is drawn as " +
+        "(alpha of 255 means fully opaque, which is probably what you want).\n\n" +
+        "By default, it will create new label names with names like LABEL_5 for any values encountered that are not mentioned in the " +
+        "list file, specify -discard-others to instead set these values to the \"unlabeled\" key."
     );
     return ret;
 }
@@ -162,7 +173,7 @@ void OperationVolumeLabelImport::useParameters(OperationParameters* myParams, Pr
             }
             if (value == GiftiLabel::getInvalidLabelKey())
             {
-                throw OperationException("entry #" + AString::number(labelCount) + ", " + AString(labelName.c_str()) + " specifies unusable key value: " + value);
+                throw OperationException("entry #" + AString::number(labelCount) + ", " + AString(labelName.c_str()) + " specifies unusable key value: " + AString::number(value));
             }
             while (isspace(labelListFile.peek()))
             {
@@ -211,12 +222,31 @@ void OperationVolumeLabelImport::useParameters(OperationParameters* myParams, Pr
             translate[value] = newValue;
         }
     }
+    int32_t tableUnlabeledKey = myTable.getUnassignedLabelKey();
+    set<AString> hierNames; //will want this for deciding whether to warn
+    OptionalParameter* hierOpt = myParams->getOptionalParameter(8);
+    if (hierOpt->m_present)
+    {
+        AString hierfileName = hierOpt->getString(1);
+        CaretHierarchy myHier;
+        myHier.readJsonFile(hierfileName);
+        hierNames = myHier.getAllNames();
+        map<int32_t, AString> tableMap; //keys aren't needed, but API only exposes names as a map
+        myTable.getKeysAndNames(tableMap);
+        for (auto iter : tableMap)
+        {
+            if (iter.first != tableUnlabeledKey && hierNames.find(iter.second) == hierNames.end())
+            {
+                CaretLogWarning("label name '" + iter.second + "' not found in specified hierarchy");
+            }
+        }
+        myTable.setHierarchy(myHier);
+    }
     vector<int64_t> myDims;
     myVol->getDimensions(myDims);
     const int64_t FRAMESIZE = myDims[0] * myDims[1] * myDims[2];
     CaretArray<float> frameOut(FRAMESIZE);
-    int32_t unusedLabel = myTable.getUnassignedLabelKey();
-    translate[unlabeledValue] = unusedLabel;
+    translate[unlabeledValue] = tableUnlabeledKey;
     if (subvol == -1)
     {
         outVol->reinitialize(myVol->getOriginalDimensions(), myVol->getSform(), myDims[4], SubvolumeAttributes::LABEL);
@@ -238,7 +268,7 @@ void OperationVolumeLabelImport::useParameters(OperationParameters* myParams, Pr
                     {
                         if (discardOthers)
                         {
-                            frameOut[i] = unusedLabel;
+                            frameOut[i] = tableUnlabeledKey;
                         } else {//use a random color, but fully opaque for the label
                             GiftiLabel myLabel(labelval, AString("LABEL_") + AString::number(labelval), rand() & 255, rand() & 255, rand() & 255, 255);
                             if (myTable.getLabelKeyFromName(myLabel.getName()) != GiftiLabel::getInvalidLabelKey())
@@ -262,6 +292,10 @@ void OperationVolumeLabelImport::useParameters(OperationParameters* myParams, Pr
                                 }
                                 myLabel.setName(newName);
                             }
+                            if (hierOpt->m_present && hierNames.find(myLabel.getName()) == hierNames.end())
+                            {
+                                CaretLogWarning("creating label " + myLabel.getName() + ", which does not exist in the hierarchy (note, using -discard-others would de-label voxels with that value instead)");
+                            }
                             int32_t newValue = myTable.addLabel(&myLabel);//don't overwrite any values in the table
                             translate[labelval] = newValue;
                             frameOut[i] = newValue;
@@ -301,7 +335,7 @@ void OperationVolumeLabelImport::useParameters(OperationParameters* myParams, Pr
                 {
                     if (discardOthers)
                     {
-                        frameOut[i] = unusedLabel;
+                        frameOut[i] = tableUnlabeledKey;
                     } else {//use a random color, but fully opaque for the label
                         GiftiLabel myLabel(labelval, AString("LABEL_") + AString::number(labelval), rand() & 255, rand() & 255, rand() & 255, 255);
                         if (myTable.getLabelKeyFromName(myLabel.getName()) != GiftiLabel::getInvalidLabelKey())
