@@ -21,6 +21,7 @@
 #include "CaretLogger.h"
 #include "DataFile.h"
 #include "FileInformation.h"
+#include "HistologySlicesFile.h"
 #include "OperationZipSpecFile.h"
 #include "OperationException.h"
 #include "SpecFile.h"
@@ -34,6 +35,7 @@
 //to print file sizes as it makes the zip
 #include <iostream>
 #include <vector>
+#include <set>
 
 using namespace caret;
 using namespace std;
@@ -60,6 +62,8 @@ OperationParameters* OperationZipSpecFile::getParameters()
     OptionalParameter* baseOpt = ret->createOptionalParameter(4, "-base-dir", "specify a directory that all data files are somewhere within, this will become the root of the zipfile's directory structure");
     baseOpt->addStringParameter(1, "directory", "the directory");
     
+    ret->createOptionalParameter(5, "-skip-missing", "any missing files will generate only warnings, and the zip file will be created anyway");
+    
     ret->setHelpText(AString("If zip-file already exists, it will be overwritten.  ") +
         "If -base-dir is not specified, the directory containing the spec file is used for the base directory.  " +
         "The spec file must contain only relative paths, and no data files may be outside the base directory.  " +
@@ -86,6 +90,7 @@ void OperationZipSpecFile::useParameters(OperationParameters* myParams, Progress
     {//this is actually because the path function strips the final "/" from the path, but not when it is just "/"
         myBaseDir += "/";//so, add the trailing slash to the path
     }
+    bool skipMissing = myParams->getOptionalParameter(5)->m_present;
 
     if (outputSubDirectory.isEmpty()) {
         throw OperationException("extract-dir must contain characters");
@@ -96,7 +101,7 @@ void OperationZipSpecFile::useParameters(OperationParameters* myParams, Progress
         CaretLogWarning("You have specified that the zip file should extract to an absolute path, this is generally frowned on.  "
                         "The <extract-folder> parameter should generally be a string without '/' or '\\' in it.");
     } else {
-        if (outputSubDirectory.indexOfAnyChar("/\\"))//assume backslashes work too
+        if (outputSubDirectory.indexOfAnyChar("/\\") != -1)//assume backslashes work too
         {
             CaretLogWarning("You have specified that the zipfile should create multiple levels of otherwise empty directories "
                             "before the file paths starting from the base directory, this is probably going to be inconvenient.  "
@@ -119,12 +124,19 @@ void OperationZipSpecFile::useParameters(OperationParameters* myParams, Progress
     std::vector<AString> allDataFileNames = specFile.getAllDataFileNames();
     allDataFileNames.push_back(specFileName);
     
+    CaretLogFine("Spec Info:"
+                 "\n   myParams->getString(1): " + myParams->getString(1)
+                 + "\n   specFile.getFileName(): " + specFile.getFileName()
+                 + "\n   specFileName: " + specFileName
+                 + "\n   specPath: " + specPath);
+    
     /*
      * Verify that all data files exist
      */
+    std::set<AString> allChildDataFileNames;
     AString missingDataFileNames;
     AString outsideBaseDirFiles;
-    const int32_t numberOfDataFiles = static_cast<int32_t>(allDataFileNames.size());
+    int32_t numberOfDataFiles = static_cast<int32_t>(allDataFileNames.size());
     for (int32_t i = 0; i < numberOfDataFiles; i++) {
         AString dataFileName = allDataFileNames[i];
         if (DataFile::isFileOnNetwork(dataFileName))
@@ -143,26 +155,54 @@ void OperationZipSpecFile::useParameters(OperationParameters* myParams, Progress
         AString absName = QDir::cleanPath(dataFileInfo.getAbsoluteFilePath());
         if (!absName.startsWith(myBaseDir))
         {
+            CaretLogFine("Outside Path: "
+                         "\n   myBaseDir: " + myBaseDir
+                         + "\n   allDataFileNames[i]: " + allDataFileNames[i]
+                         + "\n   dataFileName: " + dataFileName
+                         + "\n   absFilePath: " + dataFileInfo.getAbsoluteFilePath()
+                         + "\n   absName: " + absName);
             outsideBaseDirFiles += absName + "\n";
         }
         if (dataFileInfo.exists() == false) {
             missingDataFileNames += absName + "\n";
         }
         allDataFileNames[i] = absName;//so we don't have to do this again
+        
+        std::vector<AString> childDataFileNames;
+        addChildDataFiles(allDataFileNames[i],
+                          childDataFileNames);
+        allChildDataFileNames.insert(childDataFileNames.begin(),
+                                     childDataFileNames.end());
     }
     if (!missingDataFileNames.isEmpty())
     {
-        throw OperationException("These data files do not exist:\n" + missingDataFileNames);
+        if (skipMissing)
+        {
+            CaretLogWarning("These data files do not exist and will be skipped:\n" + missingDataFileNames);
+        } else {
+            throw OperationException("These data files do not exist:\n" + missingDataFileNames);
+        }
     }
     if (!outsideBaseDirFiles.isEmpty())
     {
         throw OperationException("These data files lie outside the base directiory:\n" + outsideBaseDirFiles + "Try using -base-dir.");
     }
     
+    /*
+     * Include names of any child data files so that they get zipped
+     */
+    if ( ! allChildDataFileNames.empty()) {
+        allDataFileNames.insert(allDataFileNames.end(),
+                                allChildDataFileNames.begin(),
+                                allChildDataFileNames.end());
+        numberOfDataFiles = static_cast<int32_t>(allDataFileNames.size());
+    }
+    
     /*
      * Create the ZIP file
      */
     QFile zipFileObject(zipFileName);
+    zipFileObject.remove();//delete it if it exists, to play better with file symlinks
     QuaZip zipFile(&zipFileObject);
     if (zipFile.open(QuaZip::mdCreate) == false) {
         throw OperationException("Unable to open ZIP File \""
@@ -180,11 +220,16 @@ void OperationZipSpecFile::useParameters(OperationParameters* myParams, Progress
         AString unzippedDataFileName = outputSubDirectory + "/" + dataFileName.mid(myBaseDir.size());//we know the string matches to the length of myBaseDir, and is cleaned, so we can just chop the right number of characters off
         QFile dataFileIn(dataFileName);
         if (dataFileIn.open(QFile::ReadOnly) == false) {
-            errorMessage = "Unable to open \""
-                                   + dataFileName
-                                   + "\" for reading: "
-                                   + dataFileIn.errorString();
-            break;
+            if (skipMissing)
+            {
+                continue;
+            } else {
+                errorMessage = "Unable to open \""
+                                    + dataFileName
+                                    + "\" for reading: "
+                                    + dataFileIn.errorString();
+                break;
+            }
         }
         float fileSize = (float)dataFileIn.size();
         int unit = 0;
@@ -256,3 +301,32 @@ void OperationZipSpecFile::useParameters(OperationParameters* myParams, Progress
         throw OperationException(errorMessage);
     }
 }
+
+/**
+ * Get names of any child data files (files not in spec file)
+ * @param dataFileName
+ *    Name of data file.
+ * @param childDataFileNamesOut
+ *    Output containing names of child data files
+ */
+void
+OperationZipSpecFile::addChildDataFiles(const AString& dataFileName,
+                                        std::vector<AString>& childDataFileNamesOut)
+{
+    bool validFlag(false);
+    const DataFileTypeEnum::Enum dataFileType = DataFileTypeEnum::fromFileExtension(dataFileName,
+                                                                                    &validFlag);
+    if (validFlag) {
+        if (dataFileType == DataFileTypeEnum::HISTOLOGY_SLICES) {
+            try {
+                HistologySlicesFile hsf;
+                hsf.readFile(dataFileName);
+                childDataFileNamesOut = hsf.getChildDataFilePathNames();
+            }
+            catch (const DataFileException& dfe) {
+                CaretLogWarning("Error reading Histology Slices File to get child data files: "
+                                + dfe.whatString());
+            }
+        }
+    }
+}
