@@ -34,11 +34,14 @@
 #include "SceneFloatArray.h"
 #include "SceneInteger.h"
 #include "SceneIntegerArray.h"
+#include "SceneLongInteger.h"
+#include "SceneLongIntegerArray.h"
 #include "SceneObjectMapIntegerKey.h"
 #include "ScenePathName.h"
 #include "ScenePathNameArray.h"
 #include "SceneString.h"
 #include "SceneStringArray.h"
+#include "SceneUnsignedByteArray.h"
 
 using namespace caret;
 
@@ -69,11 +72,30 @@ SceneClass::SceneClass(const AString& name,
                        const AString& className,
                        const int32_t versionNumber)
 : SceneObject(name,
+              SceneObjectContainerTypeEnum::SINGLE,
               SceneObjectDataTypeEnum::SCENE_CLASS),
   m_className(className),
   m_versionNumber(versionNumber)
 {
-    
+      
+}
+
+SceneClass::SceneClass(const SceneClass& rhs)
+: SceneObject(rhs.getName(),
+              SceneObjectContainerTypeEnum::SINGLE,
+              SceneObjectDataTypeEnum::SCENE_CLASS),
+  m_className(rhs.m_className),
+  m_versionNumber(rhs.m_versionNumber)
+{
+    for (std::vector<SceneObject*>::const_iterator iter = rhs.m_childObjects.begin(); iter != rhs.m_childObjects.end(); ++iter)
+    {
+        m_childObjects.push_back((*iter)->clone());
+    }
+}
+
+SceneObject* SceneClass::clone() const
+{
+    return new SceneClass(*this);
 }
 
 /**
@@ -89,6 +111,30 @@ SceneClass::~SceneClass()
     m_childObjects.clear();
 }
 
+/**
+ * Cast an instance of SceneObject to a SceneClass.
+ * Is used to avoid dynamic casting and overridden by the class.
+ *
+ * @return Valid pointer (non-NULL) this is SceneClass
+ */
+const SceneClass*
+SceneClass::castToSceneClass() const
+{
+    return this;
+}
+
+/**
+ * Cast an instance of SceneObject to a SceneClass.
+ * Is used to avoid dynamic casting and overridden by the class.
+ *
+ * @return Valid pointer (non-NULL) this is SceneClass
+ */
+SceneClass*
+SceneClass::castToSceneClass()
+{
+    return this;
+}
+
 /**
  * @return Name of the class (NOT the instance).
  */
@@ -108,6 +154,30 @@ SceneClass::getVersionNumber() const
     return m_versionNumber;
 }
 
+/**
+ * @return All descendant SceneClasses (children, grandchildren, etc.) of this instance.
+ */
+std::vector<SceneObject*>
+SceneClass::getDescendants() const
+{
+    std::vector<SceneObject*> descendants;
+    
+    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
+         iter != m_childObjects.end();
+         iter++) {
+        SceneObject* so = *iter;
+        descendants.push_back(so);
+        
+        std::vector<SceneObject*> childDescendants = so->getDescendants();
+        descendants.insert(descendants.end(),
+                           childDescendants.begin(),
+                           childDescendants.end());
+    }
+    
+    return descendants;
+}
+
+
 /**
  * Add a child to this class.
  * @param sceneObject
@@ -120,6 +190,28 @@ SceneClass::addChild(SceneObject* sceneObject)
     m_childObjects.push_back(sceneObject);
 }
 
+/**
+ * Replace the given child object with the object for insertion
+ * @param sceneObjectToReplace
+ *    Object that is to be replace (IT IS NOT DELETED !!! )
+ * @param newObjectToInsert
+ *    Object that is inserted
+ * @return True if object replacement is sucessful, else false.
+ */
+bool
+SceneClass::replaceChild(const SceneObject* sceneObjectToReplace,
+                         SceneObject* newObjectToInsert)
+{
+    const int32_t numChildren(static_cast<int32_t>(m_childObjects.size()));
+    for (int32_t i = 0; i < numChildren; i++) {
+        if (m_childObjects[i] == sceneObjectToReplace) {
+            m_childObjects[i] = newObjectToInsert;
+            return true;
+        }
+    }
+    return false;
+}
+
 /**
  * Add a child boolean value to the class.
  * 
@@ -302,7 +394,22 @@ void SceneClass::addInteger(const AString& name,
                             const int32_t value)
 {
     addChild(new SceneInteger(name,
-                                                 value));
+                              value));
+}
+
+/**
+ * Add a child long integer value to the class.
+ *
+ * @param name
+ *    Name associated with value.
+ * @param value
+ *    The value.
+ */
+void SceneClass::addLongInteger(const AString& name,
+                            const int64_t value)
+{
+    addChild(new SceneLongInteger(name,
+                                  value));
 }
 
 /**
@@ -325,6 +432,46 @@ SceneClass::addIntegerArray(const AString& name,
                                  arrayNumberOfElements));
 }
 
+/**
+ * Add a child long integer array values to the class.
+ *
+ * @param name
+ *    Name associated with value.
+ * @param values
+ *    The array containing the values.
+ * @param arrayNumberOfElements
+ *    Number of elements in the array.
+ */
+void
+SceneClass::addLongIntegerArray(const AString& name,
+                                const int64_t values[],
+                                const int32_t arrayNumberOfElements)
+{
+    addChild(new SceneLongIntegerArray(name,
+                                       values,
+                                       arrayNumberOfElements));
+}
+
+/**
+ * Add a child unsigned byte array values to the class.
+ *
+ * @param name
+ *    Name associated with value.
+ * @param values
+ *    The array containing the values.
+ * @param arrayNumberOfElements
+ *    Number of elements in the array.
+ */
+void
+SceneClass::addUnsignedByteArray(const AString& name,
+                            const uint8_t values[],
+                            const int32_t arrayNumberOfElements)
+{
+    addChild(new SceneUnsignedByteArray(name,
+                                   values,
+                                   arrayNumberOfElements));
+}
+
 /**
  * Add a child string value to the class.
  * 
@@ -450,10 +597,20 @@ AString
 SceneClass::getEnumeratedTypeValueAsString(const AString& name,
                                            const AString& defaultValue) const
 {
-    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
-         iter != m_childObjects.end();
-         iter++) {
-        const SceneObject* so = *iter;
+//    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
+//         iter != m_childObjects.end();
+//         iter++) {
+//        const SceneObject* so = *iter;
+//        const SceneEnumeratedType* st = dynamic_cast<const SceneEnumeratedType*>(so);
+//        if (st != NULL) {
+//            if (st->getName() == name) {
+//                return st->stringValue();
+//            }
+//        }
+//    }
+    
+    const SceneObject* so = getObjectWithName(name);
+    if (so != NULL) {
         const SceneEnumeratedType* st = dynamic_cast<const SceneEnumeratedType*>(so);
         if (st != NULL) {
             if (st->getName() == name) {
@@ -486,18 +643,29 @@ SceneClass::getEnumeratedTypeArrayValue(const AString& name,
                                         const int32_t arrayNumberOfElements,
                                         const AString& defaultValue) const
 {
-    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
-         iter != m_childObjects.end();
-         iter++) {
-        const SceneObject* so = *iter;
+//    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
+//         iter != m_childObjects.end();
+//         iter++) {
+//        const SceneObject* so = *iter;
+//        const SceneEnumeratedTypeArray* enumArray = dynamic_cast<const SceneEnumeratedTypeArray*>(so);
+//        if (enumArray != NULL) {
+//            if (enumArray->getName() == name) {
+//                enumArray->stringValues(values,
+//                                        arrayNumberOfElements,
+//                                        defaultValue);
+//                return enumArray->getNumberOfArrayElements();
+//            }
+//        }
+//    }
+    
+    const SceneObject* so = getObjectWithName(name);
+    if (so != NULL) {
         const SceneEnumeratedTypeArray* enumArray = dynamic_cast<const SceneEnumeratedTypeArray*>(so);
         if (enumArray != NULL) {
-            if (enumArray->getName() == name) {
-                enumArray->stringValues(values,
-                                        arrayNumberOfElements,
-                                        defaultValue);
-                return enumArray->getNumberOfArrayElements();
-            }
+            enumArray->stringValues(values,
+                                    arrayNumberOfElements,
+                                    defaultValue);
+            return enumArray->getNumberOfArrayElements();
         }
     }
     
@@ -590,6 +758,29 @@ SceneClass::getIntegerValue(const AString& name,
     return defaultValue;
 }
 
+/**
+ * Find and return the child long integer value with the given name.
+ * If no primitive matches the name, the given default
+ * value is returned.
+ * @param name
+ *    Name of the value.
+ * @param defaultValue
+ *    Value returned if the primitive with the given
+ *    name is not found.
+ * @return
+ *    The integer value.
+ */
+int64_t
+SceneClass::getLongIntegerValue(const AString& name,
+                                const int64_t defaultValue) const
+{
+    const ScenePrimitive* primitive = getPrimitive(name);
+    if (primitive != NULL) {
+        return primitive->longIntegerValue();
+    }
+    return defaultValue;
+}
+
 /**
  * Get the values for the integer array.  If no array is
  * found with the given name, all values are set to the
@@ -625,6 +816,76 @@ SceneClass::getIntegerArrayValue(const AString& name,
     return 0;
 }
 
+/**
+ * Get the values for the long integer array.  If no array is
+ * found with the given name, all values are set to the
+ * default value.
+ * @param name
+ *    Name of the value.
+ * @param values
+ *    Output array into which values are loaded.
+ * @param arrayNumberOfElements
+ *    Number of elements in the array.
+ * @param defaultValue
+ *    Value used for missing elements.
+ * @return Number of elements actually read form scene class.
+ */
+int32_t
+SceneClass::getLongIntegerArrayValue(const AString& name,
+                                     int64_t values[],
+                                     const int32_t arrayNumberOfElements,
+                                     const int64_t defaultValue) const
+{
+    const ScenePrimitiveArray* primitiveArray = getPrimitiveArray(name);
+    
+    if (primitiveArray != NULL) {
+        primitiveArray->longIntegerValues(values,
+                                          arrayNumberOfElements,
+                                          defaultValue);
+        return primitiveArray->getNumberOfArrayElements();
+    }
+    
+    for (int32_t i = 0; i < arrayNumberOfElements; i++) {
+        values[i] = defaultValue;
+    }
+    return 0;
+}
+
+/**
+ * Get the values for the unsigned byte array.  If no array is
+ * found with the given name, all values are set to the
+ * default value.
+ * @param name
+ *    Name of the value.
+ * @param values
+ *    Output array into which values are loaded.
+ * @param arrayNumberOfElements
+ *    Number of elements in the array.
+ * @param defaultValue
+ *    Value used for missing elements.
+ * @return Number of elements actually read form scene class.
+ */
+int32_t
+SceneClass::getUnsignedByteArrayValue(const AString& name,
+                                 uint8_t values[],
+                                 const int32_t arrayNumberOfElements,
+                                 const uint8_t defaultValue) const
+{
+    const ScenePrimitiveArray* primitiveArray = getPrimitiveArray(name);
+    
+    if (primitiveArray != NULL) {
+        primitiveArray->unsignedByteValues(values,
+                                      arrayNumberOfElements,
+                                      defaultValue);
+        return primitiveArray->getNumberOfArrayElements();
+    }
+    
+    for (int32_t i = 0; i < arrayNumberOfElements; i++) {
+        values[i] = defaultValue;
+    }
+    return 0;
+}
+
 /**
  * Find and return the child string value with the given name.
  * If no primitive matches the name, the given default
@@ -718,15 +979,23 @@ SceneClass::getStringArrayValue(const AString& name,
 const ScenePrimitive* 
 SceneClass::getPrimitive(const AString& name) const
 {
-    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
-         iter != m_childObjects.end();
-         iter++) {
-        const SceneObject* so = *iter;
+//    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
+//         iter != m_childObjects.end();
+//         iter++) {
+//        const SceneObject* so = *iter;
+//        const ScenePrimitive* sp = dynamic_cast<const ScenePrimitive*>(so);
+//        if (sp != NULL) {
+//            if (sp->getName() == name) {
+//                return sp;
+//            }
+//        }
+//    }
+
+    const SceneObject* so = getObjectWithName(name);
+    if (so != NULL) {
         const ScenePrimitive* sp = dynamic_cast<const ScenePrimitive*>(so);
         if (sp != NULL) {
-            if (sp->getName() == name) {
-                return sp;
-            }
+            return sp;
         }
     }
     
@@ -751,9 +1020,7 @@ SceneClass::getPathName(const AString& name) const
     if (so != NULL) {
         const ScenePathName* sp = dynamic_cast<const ScenePathName*>(so);
         if (sp != NULL) {
-            if (sp->getName() == name) {
-                return sp;
-            }
+            return sp;
         }
     }
     
@@ -774,15 +1041,23 @@ SceneClass::getPathName(const AString& name) const
 const ScenePrimitiveArray* 
 SceneClass::getPrimitiveArray(const AString& name) const
 {
-    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
-         iter != m_childObjects.end();
-         iter++) {
-        const SceneObject* so = *iter;
+//    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
+//         iter != m_childObjects.end();
+//         iter++) {
+//        const SceneObject* so = *iter;
+//        const ScenePrimitiveArray* spa = dynamic_cast<const ScenePrimitiveArray*>(so);
+//        if (spa != NULL) {
+//            if (spa->getName() == name) {
+//                return spa;
+//            }
+//        }
+//    }
+    
+    const SceneObject* so = getObjectWithName(name);
+    if (so != NULL) {
         const ScenePrimitiveArray* spa = dynamic_cast<const ScenePrimitiveArray*>(so);
         if (spa != NULL) {
-            if (spa->getName() == name) {
-                return spa;
-            }
+            return spa;
         }
     }
     
@@ -802,15 +1077,23 @@ SceneClass::getPrimitiveArray(const AString& name) const
 const ScenePathNameArray*
 SceneClass::getPathNameArray(const AString& name) const
 {
-    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
-         iter != m_childObjects.end();
-         iter++) {
-        const SceneObject* so = *iter;
-        if (so->getName() == name) {
-            const ScenePathNameArray* spa = dynamic_cast<const ScenePathNameArray*>(so);
-            if (spa != NULL) {
-                return spa;
-            }
+//    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
+//         iter != m_childObjects.end();
+//         iter++) {
+//        const SceneObject* so = *iter;
+//        if (so->getName() == name) {
+//            const ScenePathNameArray* spa = dynamic_cast<const ScenePathNameArray*>(so);
+//            if (spa != NULL) {
+//                return spa;
+//            }
+//        }
+//    }
+    
+    const SceneObject* so = getObjectWithName(name);
+    if (so != NULL) {
+        const ScenePathNameArray* spa = dynamic_cast<const ScenePathNameArray*>(so);
+        if (spa != NULL) {
+            return spa;
         }
     }
     
@@ -831,15 +1114,23 @@ SceneClass::getPathNameArray(const AString& name) const
 const SceneClass* 
 SceneClass::getClass(const AString& name) const
 {
-    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
-         iter != m_childObjects.end();
-         iter++) {
-        const SceneObject* so = *iter;
+//    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
+//         iter != m_childObjects.end();
+//         iter++) {
+//        const SceneObject* so = *iter;
+//        const SceneClass* sc = dynamic_cast<const SceneClass*>(so);
+//        if (sc != NULL) {
+//            if (sc->getName() == name) {
+//                return sc;
+//            }
+//        }
+//    }
+    
+    const SceneObject* so = getObjectWithName(name);
+    if (so != NULL) {
         const SceneClass* sc = dynamic_cast<const SceneClass*>(so);
         if (sc != NULL) {
-            if (sc->getName() == name) {
-                return sc;
-            }
+            return sc;
         }
     }
     
@@ -860,15 +1151,24 @@ SceneClass::getClass(const AString& name) const
 SceneClass* 
 SceneClass::getClass(const AString& name)
 {
-    for (std::vector<SceneObject*>::iterator iter = m_childObjects.begin();
-         iter != m_childObjects.end();
-         iter++) {
-        SceneObject* so = *iter;
+//    for (std::vector<SceneObject*>::iterator iter = m_childObjects.begin();
+//         iter != m_childObjects.end();
+//         iter++) {
+//        SceneObject* so = *iter;
+//        SceneClass* sc = dynamic_cast<SceneClass*>(so);
+//        if (sc != NULL) {
+//            if (sc->getName() == name) {
+//                return sc;
+//            }
+//        }
+//    }
+    
+    const SceneObject* soConst = getObjectWithName(name);
+    if (soConst != NULL) {
+        SceneObject* so = const_cast<SceneObject*>(soConst);
         SceneClass* sc = dynamic_cast<SceneClass*>(so);
         if (sc != NULL) {
-            if (sc->getName() == name) {
-                return sc;
-            }
+            return sc;
         }
     }
     
@@ -916,15 +1216,24 @@ SceneClass::getMapIntegerKey(const AString& name) const
 SceneClassArray* 
 SceneClass::getClassArray(const AString& name)
 {
-    for (std::vector<SceneObject*>::iterator iter = m_childObjects.begin();
-         iter != m_childObjects.end();
-         iter++) {
-        SceneObject* so = *iter;
+//    for (std::vector<SceneObject*>::iterator iter = m_childObjects.begin();
+//         iter != m_childObjects.end();
+//         iter++) {
+//        SceneObject* so = *iter;
+//        SceneClassArray* sca = dynamic_cast<SceneClassArray*>(so);
+//        if (sca != NULL) {
+//            if (sca->getName() == name) {
+//                return sca;
+//            }
+//        }
+//    }
+
+    const SceneObject* soConst = getObjectWithName(name);
+    if (soConst != NULL) {
+        SceneObject* so = const_cast<SceneObject*>(soConst);
         SceneClassArray* sca = dynamic_cast<SceneClassArray*>(so);
         if (sca != NULL) {
-            if (sca->getName() == name) {
-                return sca;
-            }
+            return sca;
         }
     }
     
@@ -945,15 +1254,24 @@ SceneClass::getClassArray(const AString& name)
 const SceneClassArray* 
 SceneClass::getClassArray(const AString& name) const
 {
-    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
-         iter != m_childObjects.end();
-         iter++) {
-        const SceneObject* so = *iter;
+//    for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
+//         iter != m_childObjects.end();
+//         iter++) {
+//        const SceneObject* so = *iter;
+//        const SceneClassArray* sca = dynamic_cast<const SceneClassArray*>(so);
+//        if (sca != NULL) {
+//            if (sca->getName() == name) {
+//                return sca;
+//            }
+//        }
+//    }
+    
+    const SceneObject* soConst = getObjectWithName(name);
+    if (soConst != NULL) {
+        SceneObject* so = const_cast<SceneObject*>(soConst);
         const SceneClassArray* sca = dynamic_cast<const SceneClassArray*>(so);
         if (sca != NULL) {
-            if (sca->getName() == name) {
-                return sca;
-            }
+            return sca;
         }
     }
     
@@ -980,6 +1298,7 @@ const SceneObject*
 SceneClass::getObjectAtIndex(const int32_t indx) const
 {
     CaretAssertVectorIndex(m_childObjects, indx);
+    m_childObjects[indx]->m_restoredFlag = true;
     return m_childObjects[indx];
 }
 
@@ -992,18 +1311,53 @@ SceneClass::getObjectAtIndex(const int32_t indx) const
 const SceneObject* 
 SceneClass::getObjectWithName(const AString& name) const
 {
+    SceneObject* sceneObject = NULL;
+    bool wasLoggedFlag = false;
+    
     for (std::vector<SceneObject*>::const_iterator iter = m_childObjects.begin();
          iter != m_childObjects.end();
          iter++) {
-        const SceneObject* so = *iter;
+        SceneObject* so = *iter;
         if (so->getName() == name) {
-            return so;
+            so->m_restoredFlag = true;
+            
+            if (s_debugLoggingEnabledFlag) {
+                if (sceneObject != NULL) {
+                    if ( ! wasLoggedFlag) {
+                        CaretLogWarning("Scene class named \""
+                                        + getName()
+                                        + "\" contains more than one child named \""
+                                        + name
+                                        + "\".  If replacing the scene does not eliminate this message "
+                                        "there is an error in the source code");
+                        wasLoggedFlag = true;
+                    }
+                }
+            }
+            else {
+                return so;
+            }
+            
+            sceneObject = so;
         }
     }
     
-    return NULL;
+    return sceneObject;
 }
 
+/**
+ * Set debug logging enabled.
+ * If enabled, message are logged at the warning level
+ * for duplicated objects and missing objects.
+ * 
+ * @param status
+ *     New status.
+ */
+void
+SceneClass::setDebugLoggingEnabled(const bool status)
+{
+    s_debugLoggingEnabledFlag = status;
+}
 
 /**
  * Log a missing object message to the Caret Logger.
@@ -1015,7 +1369,45 @@ SceneClass::getObjectWithName(const AString& name) const
 void 
 SceneClass::logMissing(const AString& missingInfo) const
 {
-    CaretLogFine(missingInfo);
+    if (missingInfo.isEmpty()) {
+        return;
+    }
+    
+    if (s_debugLoggingEnabledFlag) {
+        CaretLogFiner(missingInfo);
+    }
 }
 
 
+/**
+ * Find and return the scene's child class with the given name that uses a file name,
+ * possibly just the filename or a relative path and filename.
+ *
+ * @param filenameFullPath
+ *     Full path of file
+ * @param filenameNoPath
+ *     Filename no path
+ * @param sceneFileName
+ *     Name of scene file
+ * @return
+ *     Pointer to the class with the given name or NULL if
+ *     no child class exists with the given name.
+ */
+const SceneClass*
+SceneClass::getClassNamedWithFileName(const AString& filenameFullPath,
+                                      const AString& filenameNoPath,
+                                      const AString& sceneFileName) const
+{
+    ScenePathName spn(filenameFullPath,
+                      filenameFullPath);
+    const AString relativePathName(spn.getRelativePathToSceneFile(sceneFileName));
+    const SceneClass* sc(NULL);
+    if ( ! relativePathName.isEmpty()) {
+        sc = getClass(relativePathName);
+    }
+    if (sc == NULL) {
+        sc = getClass(filenameNoPath);
+    }
+    return sc;
+}
+
