@@ -42,12 +42,20 @@ namespace caret {
                    const AString& className,
                    const int32_t versionNumber);
         
+        SceneClass(const SceneClass& rhs);
+        
         virtual ~SceneClass();
         
+        virtual const SceneClass* castToSceneClass() const;
+
+        virtual  SceneClass* castToSceneClass();
+        
         AString getClassName() const;
         
         int32_t getVersionNumber() const;
         
+        virtual std::vector<SceneObject*> getDescendants() const;
+        
         void addBoolean(const AString& name,
                         const bool value);
         
@@ -211,10 +219,21 @@ namespace caret {
         void addInteger(const AString& name,
                         const int32_t value);
         
+        void addLongInteger(const AString& name,
+                            const int64_t value);
+        
         void addIntegerArray(const AString& name,
                              const int32_t values[],
                              const int32_t arrayNumberOfElements);
         
+        void addLongIntegerArray(const AString& name,
+                                 const int64_t values[],
+                                 const int32_t arrayNumberOfElements);
+        
+        void addUnsignedByteArray(const AString& name,
+                             const uint8_t values[],
+                             const int32_t arrayNumberOfElements);
+        
         void addString(const AString& name,
                        const AString& value);
         
@@ -227,6 +246,9 @@ namespace caret {
         
         void addChild(SceneObject* sceneObject);
         
+        bool replaceChild(const SceneObject* sceneObjectToReplace,
+                          SceneObject* newObjectToInsert);
+        
         bool getBooleanValue(const AString& name,
                              const bool defaultValue = false) const;
         
@@ -239,6 +261,10 @@ namespace caret {
         
         SceneClass* getClass(const AString& name);
         
+        const SceneClass* getClassNamedWithFileName(const AString& filenameFullPath,
+                                                    const AString& filenameNoPath,
+                                                    const AString& sceneFileName) const;
+        
         const SceneClassArray* getClassArray(const AString& name) const;
         
         SceneClassArray* getClassArray(const AString& name);
@@ -257,7 +283,20 @@ namespace caret {
         int32_t getIntegerArrayValue(const AString& name,
                                    int32_t values[],
                                    const int32_t arrayNumberOfElements,
-                                   const int32_t defaultValue = 0.0) const;
+                                   const int32_t defaultValue = 0) const;
+        
+        int64_t getLongIntegerValue(const AString& name,
+                                    const int64_t defaultValue = 0) const;
+        
+        int32_t getLongIntegerArrayValue(const AString& name,
+                                         int64_t values[],
+                                         const int32_t arrayNumberOfElements,
+                                         const int64_t defaultValue = 0) const;
+        
+        int32_t getUnsignedByteArrayValue(const AString& name,
+                                     uint8_t values[],
+                                     const int32_t arrayNumberOfElements,
+                                     const uint8_t defaultValue = 0) const;
         
         AString getPathNameValue(const AString& name,
                                  const AString& defaultValue = "") const;
@@ -286,6 +325,10 @@ namespace caret {
         
         const SceneObject* getObjectWithName(const AString& name) const;
         
+        virtual SceneObject* clone() const;
+        
+        static void setDebugLoggingEnabled(const bool status);
+        
         // ADD_NEW_METHODS_HERE
 
 //    private: // and are not used
@@ -302,8 +345,6 @@ namespace caret {
         
         
     private:
-        SceneClass(const SceneClass&);
-        
         SceneClass& operator=(const SceneClass&);
 
         void addEnumeratedType(const AString& name,
@@ -325,12 +366,14 @@ namespace caret {
         
         std::vector<SceneObject*> m_childObjects;
         
+        static bool s_debugLoggingEnabledFlag;
+        
         // ADD_NEW_MEMBERS_HERE
         
     };
     
 #ifdef __SCENE_CLASS_DECLARE__
-    // <PLACE DECLARATIONS OF STATIC MEMBERS HERE>
+    bool SceneClass::s_debugLoggingEnabledFlag = false;
 #endif // __SCENE_CLASS_DECLARE__
 
 } // namespace
