@@ -116,6 +116,30 @@ SceneClassArray::~SceneClassArray()
     m_values.clear();
 }
 
+/**
+ * Cast an instance of SceneObjectArray to a SceneClassArray.
+ * Is used to avoid dynamic casting and overridden by the class.
+ *
+ * @return Valid pointer (non-NULL) this is SceneClass
+ */
+SceneClassArray*
+SceneClassArray::castToSceneClassArray()
+{
+    return this;
+}
+
+/**
+ * Cast an instance of SceneObjectArray to a SceneClassArray.
+ * Is used to avoid dynamic casting and overridden by the class.
+ *
+ * @return Valid pointer (non-NULL) this is SceneClass
+ */
+const SceneClassArray*
+SceneClassArray::castToSceneClassArray() const
+{
+    return this;
+}
+
 /**
  * @return All descendant SceneClasses (children, grandchildren, etc.) of this instance.
  */
@@ -168,6 +192,24 @@ SceneClassArray::setClassAtIndex(const int32_t arrayIndex,
 //    return m_values[arrayIndex];
 //}
 
+/**
+ * Get the element for a given array index.
+ * @param arrayIndex
+ *     Index of array element.
+ * @return
+ *     Class at the given array index.
+ */
+SceneClass*
+SceneClassArray::getClassAtIndex(const int32_t arrayIndex)
+{
+    CaretAssertVectorIndex(m_values, arrayIndex);
+    SceneClass* sc = m_values[arrayIndex];
+    if (sc != NULL) {
+        sc->m_restoredFlag = true;
+    }
+    return sc;
+}
+
 /**
  * Get the element for a given array index.
  * @param arrayIndex
